#line 1 "ad_x86.cpp"
//
// Copyright (c) 1997, 2022, Oracle and/or its affiliates. All rights reserved.
// DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
//
// This code is free software; you can redistribute it and/or modify it
// under the terms of the GNU General Public License version 2 only, as
// published by the Free Software Foundation.
//
// This code is distributed in the hope that it will be useful, but WITHOUT
// ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
// FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
// version 2 for more details (a copy is included in the LICENSE file that
// accompanied this code).
//
// You should have received a copy of the GNU General Public License version
// 2 along with this work; if not, write to the Free Software Foundation,
// Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
//
// Please contact Oracle, 500 Oracle Parkway, Redwood Shores, CA 94065 USA
// or visit www.oracle.com if you need additional information or have any
// questions.
//
//

// Machine Generated File.  Do Not Edit!

#include "precompiled.hpp"
#include "adfiles/adGlobals_x86.hpp"
#include "adfiles/ad_x86.hpp"
#include "memory/allocation.inline.hpp"
#include "code/codeCache.hpp"
#include "code/compiledIC.hpp"
#include "code/nativeInst.hpp"
#include "code/vmreg.inline.hpp"
#include "gc/shared/collectedHeap.inline.hpp"
#include "oops/compiledICHolder.hpp"
#include "oops/compressedOops.hpp"
#include "oops/markWord.hpp"
#include "oops/method.hpp"
#include "oops/oop.inline.hpp"
#include "opto/c2_MacroAssembler.hpp"
#include "opto/cfgnode.hpp"
#include "opto/intrinsicnode.hpp"
#include "opto/locknode.hpp"
#include "opto/opcodes.hpp"
#include "opto/regalloc.hpp"
#include "opto/regmask.hpp"
#include "opto/runtime.hpp"
#include "runtime/safepointMechanism.hpp"
#include "runtime/sharedRuntime.hpp"
#include "runtime/stubRoutines.hpp"
#include "utilities/growableArray.hpp"
#include "utilities/powerOfTwo.hpp"

//SourceForm

#line 251 "src/hotspot/cpu/x86/x86_32.ad"

#define   RELOC_IMM32    Assembler::imm_operand
#define   RELOC_DISP32   Assembler::disp32_operand

#define __ _masm.

// How to find the high register of a Long pair, given the low register
#define   HIGH_FROM_LOW(x) (as_Register((x)->encoding()+2))
#define   HIGH_FROM_LOW_ENC(x) ((x)+2)

// These masks are used to provide 128-bit aligned bitmasks to the XMM
// instructions, to allow sign-masking or sign-bit flipping.  They allow
// fast versions of NegF/NegD and AbsF/AbsD.

void reg_mask_init() {}

// Note: 'double' and 'long long' have 32-bits alignment on x86.
static jlong* double_quadword(jlong *adr, jlong lo, jlong hi) {
  // Use the expression (adr)&(~0xF) to provide 128-bits aligned address
  // of 128-bits operands for SSE instructions.
  jlong *operand = (jlong*)(((uintptr_t)adr)&((uintptr_t)(~0xF)));
  // Store the value to a 128-bits operand.
  operand[0] = lo;
  operand[1] = hi;
  return operand;
}

// Buffer for 128-bits masks used by SSE instructions.
static jlong fp_signmask_pool[(4+1)*2]; // 4*128bits(data) + 128bits(alignment)

// Static initialization during VM startup.
static jlong *float_signmask_pool  = double_quadword(&fp_signmask_pool[1*2], CONST64(0x7FFFFFFF7FFFFFFF), CONST64(0x7FFFFFFF7FFFFFFF));
static jlong *double_signmask_pool = double_quadword(&fp_signmask_pool[2*2], CONST64(0x7FFFFFFFFFFFFFFF), CONST64(0x7FFFFFFFFFFFFFFF));
static jlong *float_signflip_pool  = double_quadword(&fp_signmask_pool[3*2], CONST64(0x8000000080000000), CONST64(0x8000000080000000));
static jlong *double_signflip_pool = double_quadword(&fp_signmask_pool[4*2], CONST64(0x8000000000000000), CONST64(0x8000000000000000));

// Offset hacking within calls.
static int pre_call_resets_size() {
  int size = 0;
  Compile* C = Compile::current();
  if (C->in_24_bit_fp_mode()) {
    size += 6; // fldcw
  }
  if (VM_Version::supports_vzeroupper()) {
    size += 3; // vzeroupper
  }
  return size;
}

// !!!!! Special hack to get all type of calls to specify the byte offset
//       from the start of the call to the point where the return address
//       will point.
int MachCallStaticJavaNode::ret_addr_offset() {
  return 5 + pre_call_resets_size();  // 5 bytes from start of call to where return address points
}

int MachCallDynamicJavaNode::ret_addr_offset() {
  return 10 + pre_call_resets_size();  // 10 bytes from start of call to where return address points
}

static int sizeof_FFree_Float_Stack_All = -1;

int MachCallRuntimeNode::ret_addr_offset() {
  assert(sizeof_FFree_Float_Stack_All != -1, "must have been emitted already");
  return 5 + pre_call_resets_size() + (_leaf_no_fp ? 0 : sizeof_FFree_Float_Stack_All);
}

//
// Compute padding required for nodes which need alignment
//

// The address of the call instruction needs to be 4-byte aligned to
// ensure that it does not span a cache line so that it can be patched.
int CallStaticJavaDirectNode::compute_padding(int current_offset) const {
  current_offset += pre_call_resets_size();  // skip fldcw, if any
  current_offset += 1;      // skip call opcode byte
  return align_up(current_offset, alignment_required()) - current_offset;
}

// The address of the call instruction needs to be 4-byte aligned to
// ensure that it does not span a cache line so that it can be patched.
int CallDynamicJavaDirectNode::compute_padding(int current_offset) const {
  current_offset += pre_call_resets_size();  // skip fldcw, if any
  current_offset += 5;      // skip MOV instruction
  current_offset += 1;      // skip call opcode byte
  return align_up(current_offset, alignment_required()) - current_offset;
}

// EMIT_RM()
void emit_rm(CodeBuffer &cbuf, int f1, int f2, int f3) {
  unsigned char c = (unsigned char)((f1 << 6) | (f2 << 3) | f3);
  cbuf.insts()->emit_int8(c);
}

// EMIT_CC()
void emit_cc(CodeBuffer &cbuf, int f1, int f2) {
  unsigned char c = (unsigned char)( f1 | f2 );
  cbuf.insts()->emit_int8(c);
}

// EMIT_OPCODE()
void emit_opcode(CodeBuffer &cbuf, int code) {
  cbuf.insts()->emit_int8((unsigned char) code);
}

// EMIT_OPCODE() w/ relocation information
void emit_opcode(CodeBuffer &cbuf, int code, relocInfo::relocType reloc, int offset = 0) {
  cbuf.relocate(cbuf.insts_mark() + offset, reloc);
  emit_opcode(cbuf, code);
}

// EMIT_D8()
void emit_d8(CodeBuffer &cbuf, int d8) {
  cbuf.insts()->emit_int8((unsigned char) d8);
}

// EMIT_D16()
void emit_d16(CodeBuffer &cbuf, int d16) {
  cbuf.insts()->emit_int16(d16);
}

// EMIT_D32()
void emit_d32(CodeBuffer &cbuf, int d32) {
  cbuf.insts()->emit_int32(d32);
}

// emit 32 bit value and construct relocation entry from relocInfo::relocType
void emit_d32_reloc(CodeBuffer &cbuf, int d32, relocInfo::relocType reloc,
        int format) {
  cbuf.relocate(cbuf.insts_mark(), reloc, format);
  cbuf.insts()->emit_int32(d32);
}

// emit 32 bit value and construct relocation entry from RelocationHolder
void emit_d32_reloc(CodeBuffer &cbuf, int d32, RelocationHolder const& rspec,
        int format) {
#ifdef ASSERT
  if (rspec.reloc()->type() == relocInfo::oop_type && d32 != 0 && d32 != (int)Universe::non_oop_word()) {
    assert(oopDesc::is_oop(cast_to_oop(d32)), "cannot embed broken oops in code");
  }
#endif
  cbuf.relocate(cbuf.insts_mark(), rspec, format);
  cbuf.insts()->emit_int32(d32);
}

// Access stack slot for load or store
void store_to_stackslot(CodeBuffer &cbuf, int opcode, int rm_field, int disp) {
  emit_opcode( cbuf, opcode );               // (e.g., FILD   [ESP+src])
  if( -128 <= disp && disp <= 127 ) {
    emit_rm( cbuf, 0x01, rm_field, ESP_enc );  // R/M byte
    emit_rm( cbuf, 0x00, ESP_enc, ESP_enc);    // SIB byte
    emit_d8 (cbuf, disp);     // Displacement  // R/M byte
  } else {
    emit_rm( cbuf, 0x02, rm_field, ESP_enc );  // R/M byte
    emit_rm( cbuf, 0x00, ESP_enc, ESP_enc);    // SIB byte
    emit_d32(cbuf, disp);     // Displacement  // R/M byte
  }
}

   // rRegI ereg, memory mem) %{    // emit_reg_mem
void encode_RegMem( CodeBuffer &cbuf, int reg_encoding, int base, int index, int scale, int displace, relocInfo::relocType disp_reloc ) {
  // There is no index & no scale, use form without SIB byte
  if ((index == 0x4) &&
      (scale == 0) && (base != ESP_enc)) {
    // If no displacement, mode is 0x0; unless base is [EBP]
    if ( (displace == 0) && (base != EBP_enc) ) {
      emit_rm(cbuf, 0x0, reg_encoding, base);
    }
    else {                    // If 8-bit displacement, mode 0x1
      if ((displace >= -128) && (displace <= 127)
          && (disp_reloc == relocInfo::none) ) {
        emit_rm(cbuf, 0x1, reg_encoding, base);
        emit_d8(cbuf, displace);
      }
      else {                  // If 32-bit displacement
        if (base == -1) { // Special flag for absolute address
          emit_rm(cbuf, 0x0, reg_encoding, 0x5);
          // (manual lies; no SIB needed here)
          if ( disp_reloc != relocInfo::none ) {
            emit_d32_reloc(cbuf, displace, disp_reloc, 1);
          } else {
            emit_d32      (cbuf, displace);
          }
        }
        else {                // Normal base + offset
          emit_rm(cbuf, 0x2, reg_encoding, base);
          if ( disp_reloc != relocInfo::none ) {
            emit_d32_reloc(cbuf, displace, disp_reloc, 1);
          } else {
            emit_d32      (cbuf, displace);
          }
        }
      }
    }
  }
  else {                      // Else, encode with the SIB byte
    // If no displacement, mode is 0x0; unless base is [EBP]
    if (displace == 0 && (base != EBP_enc)) {  // If no displacement
      emit_rm(cbuf, 0x0, reg_encoding, 0x4);
      emit_rm(cbuf, scale, index, base);
    }
    else {                    // If 8-bit displacement, mode 0x1
      if ((displace >= -128) && (displace <= 127)
          && (disp_reloc == relocInfo::none) ) {
        emit_rm(cbuf, 0x1, reg_encoding, 0x4);
        emit_rm(cbuf, scale, index, base);
        emit_d8(cbuf, displace);
      }
      else {                  // If 32-bit displacement
        if (base == 0x04 ) {
          emit_rm(cbuf, 0x2, reg_encoding, 0x4);
          emit_rm(cbuf, scale, index, 0x04);
        } else {
          emit_rm(cbuf, 0x2, reg_encoding, 0x4);
          emit_rm(cbuf, scale, index, base);
        }
        if ( disp_reloc != relocInfo::none ) {
          emit_d32_reloc(cbuf, displace, disp_reloc, 1);
        } else {
          emit_d32      (cbuf, displace);
        }
      }
    }
  }
}


void encode_Copy( CodeBuffer &cbuf, int dst_encoding, int src_encoding ) {
  if( dst_encoding == src_encoding ) {
    // reg-reg copy, use an empty encoding
  } else {
    emit_opcode( cbuf, 0x8B );
    emit_rm(cbuf, 0x3, dst_encoding, src_encoding );
  }
}

void emit_cmpfp_fixup(MacroAssembler& _masm) {
  Label exit;
  __ jccb(Assembler::noParity, exit);
  __ pushf();
  //
  // comiss/ucomiss instructions set ZF,PF,CF flags and
  // zero OF,AF,SF for NaN values.
  // Fixup flags by zeroing ZF,PF so that compare of NaN
  // values returns 'less than' result (CF is set).
  // Leave the rest of flags unchanged.
  //
  //    7 6 5 4 3 2 1 0
  //   |S|Z|r|A|r|P|r|C|  (r - reserved bit)
  //    0 0 1 0 1 0 1 1   (0x2B)
  //
  __ andl(Address(rsp, 0), 0xffffff2b);
  __ popf();
  __ bind(exit);
}

void emit_cmpfp3(MacroAssembler& _masm, Register dst) {
  Label done;
  __ movl(dst, -1);
  __ jcc(Assembler::parity, done);
  __ jcc(Assembler::below, done);
  __ setb(Assembler::notEqual, dst);
  __ movzbl(dst, dst);
  __ bind(done);
}


//=============================================================================
const RegMask& MachConstantBaseNode::_out_RegMask = RegMask::Empty;

int ConstantTable::calculate_table_base_offset() const {
  return 0;  // absolute addressing, no offset
}

bool MachConstantBaseNode::requires_postalloc_expand() const { return false; }
void MachConstantBaseNode::postalloc_expand(GrowableArray <Node *> *nodes, PhaseRegAlloc *ra_) {
  ShouldNotReachHere();
}

void MachConstantBaseNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  // Empty encoding
}

uint MachConstantBaseNode::size(PhaseRegAlloc* ra_) const {
  return 0;
}

#ifndef PRODUCT
void MachConstantBaseNode::format(PhaseRegAlloc* ra_, outputStream* st) const {
  st->print("# MachConstantBaseNode (empty encoding)");
}
#endif


//=============================================================================
#ifndef PRODUCT
void MachPrologNode::format(PhaseRegAlloc* ra_, outputStream* st) const {
  Compile* C = ra_->C;

  int framesize = C->output()->frame_size_in_bytes();
  int bangsize = C->output()->bang_size_in_bytes();
  assert((framesize & (StackAlignmentInBytes-1)) == 0, "frame size not aligned");
  // Remove wordSize for return addr which is already pushed.
  framesize -= wordSize;

  if (C->output()->need_stack_bang(bangsize)) {
    framesize -= wordSize;
    st->print("# stack bang (%d bytes)", bangsize);
    st->print("\n\t");
    st->print("PUSH   EBP\t# Save EBP");
    if (PreserveFramePointer) {
      st->print("\n\t");
      st->print("MOV    EBP, ESP\t# Save the caller's SP into EBP");
    }
    if (framesize) {
      st->print("\n\t");
      st->print("SUB    ESP, #%d\t# Create frame",framesize);
    }
  } else {
    st->print("SUB    ESP, #%d\t# Create frame",framesize);
    st->print("\n\t");
    framesize -= wordSize;
    st->print("MOV    [ESP + #%d], EBP\t# Save EBP",framesize);
    if (PreserveFramePointer) {
      st->print("\n\t");
      st->print("MOV    EBP, ESP\t# Save the caller's SP into EBP");
      if (framesize > 0) {
        st->print("\n\t");
        st->print("ADD    EBP, #%d", framesize);
      }
    }
  }

  if (VerifyStackAtCalls) {
    st->print("\n\t");
    framesize -= wordSize;
    st->print("MOV    [ESP + #%d], 0xBADB100D\t# Majik cookie for stack depth check",framesize);
  }

  if( C->in_24_bit_fp_mode() ) {
    st->print("\n\t");
    st->print("FLDCW  \t# load 24 bit fpu control word");
  }
  if (UseSSE >= 2 && VerifyFPU) {
    st->print("\n\t");
    st->print("# verify FPU stack (must be clean on entry)");
  }

#ifdef ASSERT
  if (VerifyStackAtCalls) {
    st->print("\n\t");
    st->print("# stack alignment check");
  }
#endif
  st->cr();
}
#endif


void MachPrologNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
  Compile* C = ra_->C;
  C2_MacroAssembler _masm(&cbuf);

  int framesize = C->output()->frame_size_in_bytes();
  int bangsize = C->output()->bang_size_in_bytes();

  __ verified_entry(framesize, C->output()->need_stack_bang(bangsize)?bangsize:0, C->in_24_bit_fp_mode(), C->stub_function() != NULL);

  C->output()->set_frame_complete(cbuf.insts_size());

  if (C->has_mach_constant_base_node()) {
    // NOTE: We set the table base offset here because users might be
    // emitted before MachConstantBaseNode.
    ConstantTable& constant_table = C->output()->constant_table();
    constant_table.set_table_base_offset(constant_table.calculate_table_base_offset());
  }
}

uint MachPrologNode::size(PhaseRegAlloc *ra_) const {
  return MachNode::size(ra_); // too many variables; just compute it the hard way
}

int MachPrologNode::reloc() const {
  return 0; // a large enough number
}

//=============================================================================
#ifndef PRODUCT
void MachEpilogNode::format( PhaseRegAlloc *ra_, outputStream* st ) const {
  Compile *C = ra_->C;
  int framesize = C->output()->frame_size_in_bytes();
  assert((framesize & (StackAlignmentInBytes-1)) == 0, "frame size not aligned");
  // Remove two words for return addr and rbp,
  framesize -= 2*wordSize;

  if (C->max_vector_size() > 16) {
    st->print("VZEROUPPER");
    st->cr(); st->print("\t");
  }
  if (C->in_24_bit_fp_mode()) {
    st->print("FLDCW  standard control word");
    st->cr(); st->print("\t");
  }
  if (framesize) {
    st->print("ADD    ESP,%d\t# Destroy frame",framesize);
    st->cr(); st->print("\t");
  }
  st->print_cr("POPL   EBP"); st->print("\t");
  if (do_polling() && C->is_method_compilation()) {
    st->print("CMPL    rsp, poll_offset[thread]  \n\t"
              "JA      #safepoint_stub\t"
              "# Safepoint: poll for GC");
  }
}
#endif

void MachEpilogNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
  Compile *C = ra_->C;
  MacroAssembler _masm(&cbuf);

  if (C->max_vector_size() > 16) {
    // Clear upper bits of YMM registers when current compiled code uses
    // wide vectors to avoid AVX <-> SSE transition penalty during call.
    _masm.vzeroupper();
  }
  // If method set FPU control word, restore to standard control word
  if (C->in_24_bit_fp_mode()) {
    _masm.fldcw(ExternalAddress(StubRoutines::x86::addr_fpu_cntrl_wrd_std()));
  }

  int framesize = C->output()->frame_size_in_bytes();
  assert((framesize & (StackAlignmentInBytes-1)) == 0, "frame size not aligned");
  // Remove two words for return addr and rbp,
  framesize -= 2*wordSize;

  // Note that VerifyStackAtCalls' Majik cookie does not change the frame size popped here

  if (framesize >= 128) {
    emit_opcode(cbuf, 0x81); // add  SP, #framesize
    emit_rm(cbuf, 0x3, 0x00, ESP_enc);
    emit_d32(cbuf, framesize);
  } else if (framesize) {
    emit_opcode(cbuf, 0x83); // add  SP, #framesize
    emit_rm(cbuf, 0x3, 0x00, ESP_enc);
    emit_d8(cbuf, framesize);
  }

  emit_opcode(cbuf, 0x58 | EBP_enc);

  if (StackReservedPages > 0 && C->has_reserved_stack_access()) {
    __ reserved_stack_check();
  }

  if (do_polling() && C->is_method_compilation()) {
    Register thread = as_Register(EBX_enc);
    MacroAssembler masm(&cbuf);
    __ get_thread(thread);
    Label dummy_label;
    Label* code_stub = &dummy_label;
    if (!C->output()->in_scratch_emit_size()) {
      C2SafepointPollStub* stub = new (C->comp_arena()) C2SafepointPollStub(__ offset());
      C->output()->add_stub(stub);
      code_stub = &stub->entry();
    }
    __ relocate(relocInfo::poll_return_type);
    __ safepoint_poll(*code_stub, thread, true /* at_return */, true /* in_nmethod */);
  }
}

uint MachEpilogNode::size(PhaseRegAlloc *ra_) const {
  return MachNode::size(ra_); // too many variables; just compute it
                              // the hard way
}

int MachEpilogNode::reloc() const {
  return 0; // a large enough number
}

const Pipeline * MachEpilogNode::pipeline() const {
  return MachNode::pipeline_class();
}

//=============================================================================

enum RC { rc_bad, rc_int, rc_kreg, rc_float, rc_xmm, rc_stack };
static enum RC rc_class( OptoReg::Name reg ) {

  if( !OptoReg::is_valid(reg)  ) return rc_bad;
  if (OptoReg::is_stack(reg)) return rc_stack;

  VMReg r = OptoReg::as_VMReg(reg);
  if (r->is_Register()) return rc_int;
  if (r->is_FloatRegister()) {
    assert(UseSSE < 2, "shouldn't be used in SSE2+ mode");
    return rc_float;
  }
  if (r->is_KRegister()) return rc_kreg;
  assert(r->is_XMMRegister(), "must be");
  return rc_xmm;
}

static int impl_helper( CodeBuffer *cbuf, bool do_size, bool is_load, int offset, int reg,
                        int opcode, const char *op_str, int size, outputStream* st ) {
  if( cbuf ) {
    emit_opcode  (*cbuf, opcode );
    encode_RegMem(*cbuf, Matcher::_regEncode[reg], ESP_enc, 0x4, 0, offset, relocInfo::none);
#ifndef PRODUCT
  } else if( !do_size ) {
    if( size != 0 ) st->print("\n\t");
    if( opcode == 0x8B || opcode == 0x89 ) { // MOV
      if( is_load ) st->print("%s   %s,[ESP + #%d]",op_str,Matcher::regName[reg],offset);
      else          st->print("%s   [ESP + #%d],%s",op_str,offset,Matcher::regName[reg]);
    } else { // FLD, FST, PUSH, POP
      st->print("%s [ESP + #%d]",op_str,offset);
    }
#endif
  }
  int offset_size = (offset == 0) ? 0 : ((offset <= 127) ? 1 : 4);
  return size+3+offset_size;
}

// Helper for XMM registers.  Extra opcode bits, limited syntax.
static int impl_x_helper( CodeBuffer *cbuf, bool do_size, bool is_load,
                         int offset, int reg_lo, int reg_hi, int size, outputStream* st ) {
  int in_size_in_bits = Assembler::EVEX_32bit;
  int evex_encoding = 0;
  if (reg_lo+1 == reg_hi) {
    in_size_in_bits = Assembler::EVEX_64bit;
    evex_encoding = Assembler::VEX_W;
  }
  if (cbuf) {
    MacroAssembler _masm(cbuf);
    // EVEX spills remain EVEX: Compressed displacemement is better than AVX on spill mem operations,
    //                          it maps more cases to single byte displacement
    _masm.set_managed();
    if (reg_lo+1 == reg_hi) { // double move?
      if (is_load) {
        __ movdbl(as_XMMRegister(Matcher::_regEncode[reg_lo]), Address(rsp, offset));
      } else {
        __ movdbl(Address(rsp, offset), as_XMMRegister(Matcher::_regEncode[reg_lo]));
      }
    } else {
      if (is_load) {
        __ movflt(as_XMMRegister(Matcher::_regEncode[reg_lo]), Address(rsp, offset));
      } else {
        __ movflt(Address(rsp, offset), as_XMMRegister(Matcher::_regEncode[reg_lo]));
      }
    }
#ifndef PRODUCT
  } else if (!do_size) {
    if (size != 0) st->print("\n\t");
    if (reg_lo+1 == reg_hi) { // double move?
      if (is_load) st->print("%s %s,[ESP + #%d]",
                              UseXmmLoadAndClearUpper ? "MOVSD " : "MOVLPD",
                              Matcher::regName[reg_lo], offset);
      else         st->print("MOVSD  [ESP + #%d],%s",
                              offset, Matcher::regName[reg_lo]);
    } else {
      if (is_load) st->print("MOVSS  %s,[ESP + #%d]",
                              Matcher::regName[reg_lo], offset);
      else         st->print("MOVSS  [ESP + #%d],%s",
                              offset, Matcher::regName[reg_lo]);
    }
#endif
  }
  bool is_single_byte = false;
  if ((UseAVX > 2) && (offset != 0)) {
    is_single_byte = Assembler::query_compressed_disp_byte(offset, true, 0, Assembler::EVEX_T1S, in_size_in_bits, evex_encoding);
  }
  int offset_size = 0;
  if (UseAVX > 2 ) {
    offset_size = (offset == 0) ? 0 : ((is_single_byte) ? 1 : 4);
  } else {
    offset_size = (offset == 0) ? 0 : ((offset <= 127) ? 1 : 4);
  }
  size += (UseAVX > 2) ? 2 : 0; // Need an additional two bytes for EVEX
  // VEX_2bytes prefix is used if UseAVX > 0, so it takes the same 2 bytes as SIMD prefix.
  return size+5+offset_size;
}


static int impl_movx_helper( CodeBuffer *cbuf, bool do_size, int src_lo, int dst_lo,
                            int src_hi, int dst_hi, int size, outputStream* st ) {
  if (cbuf) {
    MacroAssembler _masm(cbuf);
    // EVEX spills remain EVEX: logic complex between full EVEX, partial and AVX, manage EVEX spill code one way.
    _masm.set_managed();
    if (src_lo+1 == src_hi && dst_lo+1 == dst_hi) { // double move?
      __ movdbl(as_XMMRegister(Matcher::_regEncode[dst_lo]),
                as_XMMRegister(Matcher::_regEncode[src_lo]));
    } else {
      __ movflt(as_XMMRegister(Matcher::_regEncode[dst_lo]),
                as_XMMRegister(Matcher::_regEncode[src_lo]));
    }
#ifndef PRODUCT
  } else if (!do_size) {
    if (size != 0) st->print("\n\t");
    if (UseXmmRegToRegMoveAll) {//Use movaps,movapd to move between xmm registers
      if (src_lo+1 == src_hi && dst_lo+1 == dst_hi) { // double move?
        st->print("MOVAPD %s,%s",Matcher::regName[dst_lo],Matcher::regName[src_lo]);
      } else {
        st->print("MOVAPS %s,%s",Matcher::regName[dst_lo],Matcher::regName[src_lo]);
      }
    } else {
      if( src_lo+1 == src_hi && dst_lo+1 == dst_hi ) { // double move?
        st->print("MOVSD  %s,%s",Matcher::regName[dst_lo],Matcher::regName[src_lo]);
      } else {
        st->print("MOVSS  %s,%s",Matcher::regName[dst_lo],Matcher::regName[src_lo]);
      }
    }
#endif
  }
  // VEX_2bytes prefix is used if UseAVX > 0, and it takes the same 2 bytes as SIMD prefix.
  // Only MOVAPS SSE prefix uses 1 byte.  EVEX uses an additional 2 bytes.
  int sz = (UseAVX > 2) ? 6 : 4;
  if (!(src_lo+1 == src_hi && dst_lo+1 == dst_hi) &&
      UseXmmRegToRegMoveAll && (UseAVX == 0)) sz = 3;
  return size + sz;
}

static int impl_movgpr2x_helper( CodeBuffer *cbuf, bool do_size, int src_lo, int dst_lo,
                            int src_hi, int dst_hi, int size, outputStream* st ) {
  // 32-bit
  if (cbuf) {
    MacroAssembler _masm(cbuf);
    // EVEX spills remain EVEX: logic complex between full EVEX, partial and AVX, manage EVEX spill code one way.
    _masm.set_managed();
    __ movdl(as_XMMRegister(Matcher::_regEncode[dst_lo]),
             as_Register(Matcher::_regEncode[src_lo]));
#ifndef PRODUCT
  } else if (!do_size) {
    st->print("movdl   %s, %s\t# spill", Matcher::regName[dst_lo], Matcher::regName[src_lo]);
#endif
  }
  return (UseAVX> 2) ? 6 : 4;
}


static int impl_movx2gpr_helper( CodeBuffer *cbuf, bool do_size, int src_lo, int dst_lo,
                                 int src_hi, int dst_hi, int size, outputStream* st ) {
  // 32-bit
  if (cbuf) {
    MacroAssembler _masm(cbuf);
    // EVEX spills remain EVEX: logic complex between full EVEX, partial and AVX, manage EVEX spill code one way.
    _masm.set_managed();
    __ movdl(as_Register(Matcher::_regEncode[dst_lo]),
             as_XMMRegister(Matcher::_regEncode[src_lo]));
#ifndef PRODUCT
  } else if (!do_size) {
    st->print("movdl   %s, %s\t# spill", Matcher::regName[dst_lo], Matcher::regName[src_lo]);
#endif
  }
  return (UseAVX> 2) ? 6 : 4;
}

static int impl_mov_helper( CodeBuffer *cbuf, bool do_size, int src, int dst, int size, outputStream* st ) {
  if( cbuf ) {
    emit_opcode(*cbuf, 0x8B );
    emit_rm    (*cbuf, 0x3, Matcher::_regEncode[dst], Matcher::_regEncode[src] );
#ifndef PRODUCT
  } else if( !do_size ) {
    if( size != 0 ) st->print("\n\t");
    st->print("MOV    %s,%s",Matcher::regName[dst],Matcher::regName[src]);
#endif
  }
  return size+2;
}

static int impl_fp_store_helper( CodeBuffer *cbuf, bool do_size, int src_lo, int src_hi, int dst_lo, int dst_hi,
                                 int offset, int size, outputStream* st ) {
  if( src_lo != FPR1L_num ) {      // Move value to top of FP stack, if not already there
    if( cbuf ) {
      emit_opcode( *cbuf, 0xD9 );  // FLD (i.e., push it)
      emit_d8( *cbuf, 0xC0-1+Matcher::_regEncode[src_lo] );
#ifndef PRODUCT
    } else if( !do_size ) {
      if( size != 0 ) st->print("\n\t");
      st->print("FLD    %s",Matcher::regName[src_lo]);
#endif
    }
    size += 2;
  }

  int st_op = (src_lo != FPR1L_num) ? EBX_num /*store & pop*/ : EDX_num /*store no pop*/;
  const char *op_str;
  int op;
  if( src_lo+1 == src_hi && dst_lo+1 == dst_hi ) { // double store?
    op_str = (src_lo != FPR1L_num) ? "FSTP_D" : "FST_D ";
    op = 0xDD;
  } else {                   // 32-bit store
    op_str = (src_lo != FPR1L_num) ? "FSTP_S" : "FST_S ";
    op = 0xD9;
    assert( !OptoReg::is_valid(src_hi) && !OptoReg::is_valid(dst_hi), "no non-adjacent float-stores" );
  }

  return impl_helper(cbuf,do_size,false,offset,st_op,op,op_str,size, st);
}

// Next two methods are shared by 32- and 64-bit VM. They are defined in x86.ad.
static void vec_mov_helper(CodeBuffer *cbuf, int src_lo, int dst_lo,
                          int src_hi, int dst_hi, uint ireg, outputStream* st);

void vec_spill_helper(CodeBuffer *cbuf, bool is_load,
                            int stack_offset, int reg, uint ireg, outputStream* st);

static void vec_stack_to_stack_helper(CodeBuffer *cbuf, int src_offset,
                                     int dst_offset, uint ireg, outputStream* st) {
  if (cbuf) {
    MacroAssembler _masm(cbuf);
    switch (ireg) {
    case Op_VecS:
      __ pushl(Address(rsp, src_offset));
      __ popl (Address(rsp, dst_offset));
      break;
    case Op_VecD:
      __ pushl(Address(rsp, src_offset));
      __ popl (Address(rsp, dst_offset));
      __ pushl(Address(rsp, src_offset+4));
      __ popl (Address(rsp, dst_offset+4));
      break;
    case Op_VecX:
      __ movdqu(Address(rsp, -16), xmm0);
      __ movdqu(xmm0, Address(rsp, src_offset));
      __ movdqu(Address(rsp, dst_offset), xmm0);
      __ movdqu(xmm0, Address(rsp, -16));
      break;
    case Op_VecY:
      __ vmovdqu(Address(rsp, -32), xmm0);
      __ vmovdqu(xmm0, Address(rsp, src_offset));
      __ vmovdqu(Address(rsp, dst_offset), xmm0);
      __ vmovdqu(xmm0, Address(rsp, -32));
      break;
    case Op_VecZ:
      __ evmovdquq(Address(rsp, -64), xmm0, 2);
      __ evmovdquq(xmm0, Address(rsp, src_offset), 2);
      __ evmovdquq(Address(rsp, dst_offset), xmm0, 2);
      __ evmovdquq(xmm0, Address(rsp, -64), 2);
      break;
    default:
      ShouldNotReachHere();
    }
#ifndef PRODUCT
  } else {
    switch (ireg) {
    case Op_VecS:
      st->print("pushl   [rsp + #%d]\t# 32-bit mem-mem spill\n\t"
                "popl    [rsp + #%d]",
                src_offset, dst_offset);
      break;
    case Op_VecD:
      st->print("pushl   [rsp + #%d]\t# 64-bit mem-mem spill\n\t"
                "popq    [rsp + #%d]\n\t"
                "pushl   [rsp + #%d]\n\t"
                "popq    [rsp + #%d]",
                src_offset, dst_offset, src_offset+4, dst_offset+4);
      break;
     case Op_VecX:
      st->print("movdqu  [rsp - #16], xmm0\t# 128-bit mem-mem spill\n\t"
                "movdqu  xmm0, [rsp + #%d]\n\t"
                "movdqu  [rsp + #%d], xmm0\n\t"
                "movdqu  xmm0, [rsp - #16]",
                src_offset, dst_offset);
      break;
    case Op_VecY:
      st->print("vmovdqu [rsp - #32], xmm0\t# 256-bit mem-mem spill\n\t"
                "vmovdqu xmm0, [rsp + #%d]\n\t"
                "vmovdqu [rsp + #%d], xmm0\n\t"
                "vmovdqu xmm0, [rsp - #32]",
                src_offset, dst_offset);
      break;
    case Op_VecZ:
      st->print("vmovdqu [rsp - #64], xmm0\t# 512-bit mem-mem spill\n\t"
                "vmovdqu xmm0, [rsp + #%d]\n\t"
                "vmovdqu [rsp + #%d], xmm0\n\t"
                "vmovdqu xmm0, [rsp - #64]",
                src_offset, dst_offset);
      break;
    default:
      ShouldNotReachHere();
    }
#endif
  }
}

uint MachSpillCopyNode::implementation( CodeBuffer *cbuf, PhaseRegAlloc *ra_, bool do_size, outputStream* st ) const {
  // Get registers to move
  OptoReg::Name src_second = ra_->get_reg_second(in(1));
  OptoReg::Name src_first = ra_->get_reg_first(in(1));
  OptoReg::Name dst_second = ra_->get_reg_second(this );
  OptoReg::Name dst_first = ra_->get_reg_first(this );

  enum RC src_second_rc = rc_class(src_second);
  enum RC src_first_rc = rc_class(src_first);
  enum RC dst_second_rc = rc_class(dst_second);
  enum RC dst_first_rc = rc_class(dst_first);

  assert( OptoReg::is_valid(src_first) && OptoReg::is_valid(dst_first), "must move at least 1 register" );

  // Generate spill code!
  int size = 0;

  if( src_first == dst_first && src_second == dst_second )
    return size;            // Self copy, no move

  if (bottom_type()->isa_vect() != NULL && bottom_type()->isa_vectmask() == NULL) {
    uint ireg = ideal_reg();
    assert((src_first_rc != rc_int && dst_first_rc != rc_int), "sanity");
    assert((src_first_rc != rc_float && dst_first_rc != rc_float), "sanity");
    assert((ireg == Op_VecS || ireg == Op_VecD || ireg == Op_VecX || ireg == Op_VecY || ireg == Op_VecZ ), "sanity");
    if( src_first_rc == rc_stack && dst_first_rc == rc_stack ) {
      // mem -> mem
      int src_offset = ra_->reg2offset(src_first);
      int dst_offset = ra_->reg2offset(dst_first);
      vec_stack_to_stack_helper(cbuf, src_offset, dst_offset, ireg, st);
    } else if (src_first_rc == rc_xmm && dst_first_rc == rc_xmm ) {
      vec_mov_helper(cbuf, src_first, dst_first, src_second, dst_second, ireg, st);
    } else if (src_first_rc == rc_xmm && dst_first_rc == rc_stack ) {
      int stack_offset = ra_->reg2offset(dst_first);
      vec_spill_helper(cbuf, false, stack_offset, src_first, ireg, st);
    } else if (src_first_rc == rc_stack && dst_first_rc == rc_xmm ) {
      int stack_offset = ra_->reg2offset(src_first);
      vec_spill_helper(cbuf, true,  stack_offset, dst_first, ireg, st);
    } else {
      ShouldNotReachHere();
    }
    return 0;
  }

  // --------------------------------------
  // Check for mem-mem move.  push/pop to move.
  if( src_first_rc == rc_stack && dst_first_rc == rc_stack ) {
    if( src_second == dst_first ) { // overlapping stack copy ranges
      assert( src_second_rc == rc_stack && dst_second_rc == rc_stack, "we only expect a stk-stk copy here" );
      size = impl_helper(cbuf,do_size,true ,ra_->reg2offset(src_second),ESI_num,0xFF,"PUSH  ",size, st);
      size = impl_helper(cbuf,do_size,false,ra_->reg2offset(dst_second),EAX_num,0x8F,"POP   ",size, st);
      src_second_rc = dst_second_rc = rc_bad;  // flag as already moved the second bits
    }
    // move low bits
    size = impl_helper(cbuf,do_size,true ,ra_->reg2offset(src_first),ESI_num,0xFF,"PUSH  ",size, st);
    size = impl_helper(cbuf,do_size,false,ra_->reg2offset(dst_first),EAX_num,0x8F,"POP   ",size, st);
    if( src_second_rc == rc_stack && dst_second_rc == rc_stack ) { // mov second bits
      size = impl_helper(cbuf,do_size,true ,ra_->reg2offset(src_second),ESI_num,0xFF,"PUSH  ",size, st);
      size = impl_helper(cbuf,do_size,false,ra_->reg2offset(dst_second),EAX_num,0x8F,"POP   ",size, st);
    }
    return size;
  }

  // --------------------------------------
  // Check for integer reg-reg copy
  if( src_first_rc == rc_int && dst_first_rc == rc_int )
    size = impl_mov_helper(cbuf,do_size,src_first,dst_first,size, st);

  // Check for integer store
  if( src_first_rc == rc_int && dst_first_rc == rc_stack )
    size = impl_helper(cbuf,do_size,false,ra_->reg2offset(dst_first),src_first,0x89,"MOV ",size, st);

  // Check for integer load
  if( src_first_rc == rc_stack && dst_first_rc == rc_int )
    size = impl_helper(cbuf,do_size,true ,ra_->reg2offset(src_first),dst_first,0x8B,"MOV ",size, st);

  // Check for integer reg-xmm reg copy
  if( src_first_rc == rc_int && dst_first_rc == rc_xmm ) {
    assert( (src_second_rc == rc_bad && dst_second_rc == rc_bad),
            "no 64 bit integer-float reg moves" );
    return impl_movgpr2x_helper(cbuf,do_size,src_first,dst_first,src_second, dst_second, size, st);
  }
  // --------------------------------------
  // Check for float reg-reg copy
  if( src_first_rc == rc_float && dst_first_rc == rc_float ) {
    assert( (src_second_rc == rc_bad && dst_second_rc == rc_bad) ||
            (src_first+1 == src_second && dst_first+1 == dst_second), "no non-adjacent float-moves" );
    if( cbuf ) {

      // Note the mucking with the register encode to compensate for the 0/1
      // indexing issue mentioned in a comment in the reg_def sections
      // for FPR registers many lines above here.

      if( src_first != FPR1L_num ) {
        emit_opcode  (*cbuf, 0xD9 );           // FLD    ST(i)
        emit_d8      (*cbuf, 0xC0+Matcher::_regEncode[src_first]-1 );
        emit_opcode  (*cbuf, 0xDD );           // FSTP   ST(i)
        emit_d8      (*cbuf, 0xD8+Matcher::_regEncode[dst_first] );
     } else {
        emit_opcode  (*cbuf, 0xDD );           // FST    ST(i)
        emit_d8      (*cbuf, 0xD0+Matcher::_regEncode[dst_first]-1 );
     }
#ifndef PRODUCT
    } else if( !do_size ) {
      if( size != 0 ) st->print("\n\t");
      if( src_first != FPR1L_num ) st->print("FLD    %s\n\tFSTP   %s",Matcher::regName[src_first],Matcher::regName[dst_first]);
      else                      st->print(             "FST    %s",                            Matcher::regName[dst_first]);
#endif
    }
    return size + ((src_first != FPR1L_num) ? 2+2 : 2);
  }

  // Check for float store
  if( src_first_rc == rc_float && dst_first_rc == rc_stack ) {
    return impl_fp_store_helper(cbuf,do_size,src_first,src_second,dst_first,dst_second,ra_->reg2offset(dst_first),size, st);
  }

  // Check for float load
  if( dst_first_rc == rc_float && src_first_rc == rc_stack ) {
    int offset = ra_->reg2offset(src_first);
    const char *op_str;
    int op;
    if( src_first+1 == src_second && dst_first+1 == dst_second ) { // double load?
      op_str = "FLD_D";
      op = 0xDD;
    } else {                   // 32-bit load
      op_str = "FLD_S";
      op = 0xD9;
      assert( src_second_rc == rc_bad && dst_second_rc == rc_bad, "no non-adjacent float-loads" );
    }
    if( cbuf ) {
      emit_opcode  (*cbuf, op );
      encode_RegMem(*cbuf, 0x0, ESP_enc, 0x4, 0, offset, relocInfo::none);
      emit_opcode  (*cbuf, 0xDD );           // FSTP   ST(i)
      emit_d8      (*cbuf, 0xD8+Matcher::_regEncode[dst_first] );
#ifndef PRODUCT
    } else if( !do_size ) {
      if( size != 0 ) st->print("\n\t");
      st->print("%s  ST,[ESP + #%d]\n\tFSTP   %s",op_str, offset,Matcher::regName[dst_first]);
#endif
    }
    int offset_size = (offset == 0) ? 0 : ((offset <= 127) ? 1 : 4);
    return size + 3+offset_size+2;
  }

  // Check for xmm reg-reg copy
  if( src_first_rc == rc_xmm && dst_first_rc == rc_xmm ) {
    assert( (src_second_rc == rc_bad && dst_second_rc == rc_bad) ||
            (src_first+1 == src_second && dst_first+1 == dst_second),
            "no non-adjacent float-moves" );
    return impl_movx_helper(cbuf,do_size,src_first,dst_first,src_second, dst_second, size, st);
  }

  // Check for xmm reg-integer reg copy
  if( src_first_rc == rc_xmm && dst_first_rc == rc_int ) {
    assert( (src_second_rc == rc_bad && dst_second_rc == rc_bad),
            "no 64 bit float-integer reg moves" );
    return impl_movx2gpr_helper(cbuf,do_size,src_first,dst_first,src_second, dst_second, size, st);
  }

  // Check for xmm store
  if( src_first_rc == rc_xmm && dst_first_rc == rc_stack ) {
    return impl_x_helper(cbuf,do_size,false,ra_->reg2offset(dst_first), src_first, src_second, size, st);
  }

  // Check for float xmm load
  if( src_first_rc == rc_stack && dst_first_rc == rc_xmm ) {
    return impl_x_helper(cbuf,do_size,true ,ra_->reg2offset(src_first),dst_first, dst_second, size, st);
  }

  // Copy from float reg to xmm reg
  if( src_first_rc == rc_float && dst_first_rc == rc_xmm ) {
    // copy to the top of stack from floating point reg
    // and use LEA to preserve flags
    if( cbuf ) {
      emit_opcode(*cbuf,0x8D);  // LEA  ESP,[ESP-8]
      emit_rm(*cbuf, 0x1, ESP_enc, 0x04);
      emit_rm(*cbuf, 0x0, 0x04, ESP_enc);
      emit_d8(*cbuf,0xF8);
#ifndef PRODUCT
    } else if( !do_size ) {
      if( size != 0 ) st->print("\n\t");
      st->print("LEA    ESP,[ESP-8]");
#endif
    }
    size += 4;

    size = impl_fp_store_helper(cbuf,do_size,src_first,src_second,dst_first,dst_second,0,size, st);

    // Copy from the temp memory to the xmm reg.
    size = impl_x_helper(cbuf,do_size,true ,0,dst_first, dst_second, size, st);

    if( cbuf ) {
      emit_opcode(*cbuf,0x8D);  // LEA  ESP,[ESP+8]
      emit_rm(*cbuf, 0x1, ESP_enc, 0x04);
      emit_rm(*cbuf, 0x0, 0x04, ESP_enc);
      emit_d8(*cbuf,0x08);
#ifndef PRODUCT
    } else if( !do_size ) {
      if( size != 0 ) st->print("\n\t");
      st->print("LEA    ESP,[ESP+8]");
#endif
    }
    size += 4;
    return size;
  }

  // AVX-512 opmask specific spilling.
  if (src_first_rc == rc_stack && dst_first_rc == rc_kreg) {
    assert((src_first & 1) == 0 && src_first + 1 == src_second, "invalid register pair");
    assert((dst_first & 1) == 0 && dst_first + 1 == dst_second, "invalid register pair");
    int offset = ra_->reg2offset(src_first);
    if (cbuf != nullptr) {
      MacroAssembler _masm(cbuf);
      __ kmov(as_KRegister(Matcher::_regEncode[dst_first]), Address(rsp, offset));
#ifndef PRODUCT
    } else {
      st->print("KMOV    %s, [ESP + %d]", Matcher::regName[dst_first], offset);
#endif
    }
    return 0;
  }

  if (src_first_rc == rc_kreg && dst_first_rc == rc_stack) {
    assert((src_first & 1) == 0 && src_first + 1 == src_second, "invalid register pair");
    assert((dst_first & 1) == 0 && dst_first + 1 == dst_second, "invalid register pair");
    int offset = ra_->reg2offset(dst_first);
    if (cbuf != nullptr) {
      MacroAssembler _masm(cbuf);
      __ kmov(Address(rsp, offset), as_KRegister(Matcher::_regEncode[src_first]));
#ifndef PRODUCT
    } else {
      st->print("KMOV    [ESP + %d], %s", offset, Matcher::regName[src_first]);
#endif
    }
    return 0;
  }

  if (src_first_rc == rc_kreg && dst_first_rc == rc_int) {
    Unimplemented();
    return 0;
  }

  if (src_first_rc == rc_int && dst_first_rc == rc_kreg) {
    Unimplemented();
    return 0;
  }

  if (src_first_rc == rc_kreg && dst_first_rc == rc_kreg) {
    assert((src_first & 1) == 0 && src_first + 1 == src_second, "invalid register pair");
    assert((dst_first & 1) == 0 && dst_first + 1 == dst_second, "invalid register pair");
    if (cbuf != nullptr) {
      MacroAssembler _masm(cbuf);
      __ kmov(as_KRegister(Matcher::_regEncode[dst_first]), as_KRegister(Matcher::_regEncode[src_first]));
#ifndef PRODUCT
    } else {
      st->print("KMOV    %s, %s", Matcher::regName[dst_first], Matcher::regName[src_first]);
#endif
    }
    return 0;
  }

  assert( size > 0, "missed a case" );

  // --------------------------------------------------------------------
  // Check for second bits still needing moving.
  if( src_second == dst_second )
    return size;               // Self copy; no move
  assert( src_second_rc != rc_bad && dst_second_rc != rc_bad, "src_second & dst_second cannot be Bad" );

  // Check for second word int-int move
  if( src_second_rc == rc_int && dst_second_rc == rc_int )
    return impl_mov_helper(cbuf,do_size,src_second,dst_second,size, st);

  // Check for second word integer store
  if( src_second_rc == rc_int && dst_second_rc == rc_stack )
    return impl_helper(cbuf,do_size,false,ra_->reg2offset(dst_second),src_second,0x89,"MOV ",size, st);

  // Check for second word integer load
  if( dst_second_rc == rc_int && src_second_rc == rc_stack )
    return impl_helper(cbuf,do_size,true ,ra_->reg2offset(src_second),dst_second,0x8B,"MOV ",size, st);

  Unimplemented();
  return 0; // Mute compiler
}

#ifndef PRODUCT
void MachSpillCopyNode::format(PhaseRegAlloc *ra_, outputStream* st) const {
  implementation( NULL, ra_, false, st );
}
#endif

void MachSpillCopyNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
  implementation( &cbuf, ra_, false, NULL );
}

uint MachSpillCopyNode::size(PhaseRegAlloc *ra_) const {
  return MachNode::size(ra_);
}


//=============================================================================
#ifndef PRODUCT
void BoxLockNode::format( PhaseRegAlloc *ra_, outputStream* st ) const {
  int offset = ra_->reg2offset(in_RegMask(0).find_first_elem());
  int reg = ra_->get_reg_first(this);
  st->print("LEA    %s,[ESP + #%d]",Matcher::regName[reg],offset);
}
#endif

void BoxLockNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
  int offset = ra_->reg2offset(in_RegMask(0).find_first_elem());
  int reg = ra_->get_encode(this);
  if( offset >= 128 ) {
    emit_opcode(cbuf, 0x8D);      // LEA  reg,[SP+offset]
    emit_rm(cbuf, 0x2, reg, 0x04);
    emit_rm(cbuf, 0x0, 0x04, ESP_enc);
    emit_d32(cbuf, offset);
  }
  else {
    emit_opcode(cbuf, 0x8D);      // LEA  reg,[SP+offset]
    emit_rm(cbuf, 0x1, reg, 0x04);
    emit_rm(cbuf, 0x0, 0x04, ESP_enc);
    emit_d8(cbuf, offset);
  }
}

uint BoxLockNode::size(PhaseRegAlloc *ra_) const {
  int offset = ra_->reg2offset(in_RegMask(0).find_first_elem());
  if( offset >= 128 ) {
    return 7;
  }
  else {
    return 4;
  }
}

//=============================================================================
#ifndef PRODUCT
void MachUEPNode::format( PhaseRegAlloc *ra_, outputStream* st ) const {
  st->print_cr(  "CMP    EAX,[ECX+4]\t# Inline cache check");
  st->print_cr("\tJNE    SharedRuntime::handle_ic_miss_stub");
  st->print_cr("\tNOP");
  st->print_cr("\tNOP");
  if( !OptoBreakpoint )
    st->print_cr("\tNOP");
}
#endif

void MachUEPNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
  MacroAssembler masm(&cbuf);
#ifdef ASSERT
  uint insts_size = cbuf.insts_size();
#endif
  masm.cmpptr(rax, Address(rcx, oopDesc::klass_offset_in_bytes()));
  masm.jump_cc(Assembler::notEqual,
               RuntimeAddress(SharedRuntime::get_ic_miss_stub()));
  /* WARNING these NOPs are critical so that verified entry point is properly
     aligned for patching by NativeJump::patch_verified_entry() */
  int nops_cnt = 2;
  if( !OptoBreakpoint ) // Leave space for int3
     nops_cnt += 1;
  masm.nop(nops_cnt);

  assert(cbuf.insts_size() - insts_size == size(ra_), "checking code size of inline cache node");
}

uint MachUEPNode::size(PhaseRegAlloc *ra_) const {
  return OptoBreakpoint ? 11 : 12;
}


//=============================================================================

// Vector calling convention not supported.
const bool Matcher::supports_vector_calling_convention() {
  return false;
}

OptoRegPair Matcher::vector_return_value(uint ideal_reg) {
  Unimplemented();
  return OptoRegPair(0, 0);
}

// Is this branch offset short enough that a short branch can be used?
//
// NOTE: If the platform does not provide any short branch variants, then
//       this method should return false for offset 0.
bool Matcher::is_short_branch_offset(int rule, int br_size, int offset) {
  // The passed offset is relative to address of the branch.
  // On 86 a branch displacement is calculated relative to address
  // of a next instruction.
  offset -= br_size;

  // the short version of jmpConUCF2 contains multiple branches,
  // making the reach slightly less
  if (rule == jmpConUCF2_rule)
    return (-126 <= offset && offset <= 125);
  return (-128 <= offset && offset <= 127);
}

// Return whether or not this register is ever used as an argument.  This
// function is used on startup to build the trampoline stubs in generateOptoStub.
// Registers not mentioned will be killed by the VM call in the trampoline, and
// arguments in those registers not be available to the callee.
bool Matcher::can_be_java_arg( int reg ) {
  if(  reg == ECX_num   || reg == EDX_num   ) return true;
  if( (reg == XMM0_num  || reg == XMM1_num ) && UseSSE>=1 ) return true;
  if( (reg == XMM0b_num || reg == XMM1b_num) && UseSSE>=2 ) return true;
  return false;
}

bool Matcher::is_spillable_arg( int reg ) {
  return can_be_java_arg(reg);
}

uint Matcher::int_pressure_limit()
{
  return (INTPRESSURE == -1) ? 6 : INTPRESSURE;
}

uint Matcher::float_pressure_limit()
{
  return (FLOATPRESSURE == -1) ? 6 : FLOATPRESSURE;
}

bool Matcher::use_asm_for_ldiv_by_con( jlong divisor ) {
  // Use hardware integer DIV instruction when
  // it is faster than a code which use multiply.
  // Only when constant divisor fits into 32 bit
  // (min_jint is excluded to get only correct
  // positive 32 bit values from negative).
  return VM_Version::has_fast_idiv() &&
         (divisor == (int)divisor && divisor != min_jint);
}

// Register for DIVI projection of divmodI
RegMask Matcher::divI_proj_mask() {
  return EAX_REG_mask();
}

// Register for MODI projection of divmodI
RegMask Matcher::modI_proj_mask() {
  return EDX_REG_mask();
}

// Register for DIVL projection of divmodL
RegMask Matcher::divL_proj_mask() {
  ShouldNotReachHere();
  return RegMask();
}

// Register for MODL projection of divmodL
RegMask Matcher::modL_proj_mask() {
  ShouldNotReachHere();
  return RegMask();
}

const RegMask Matcher::method_handle_invoke_SP_save_mask() {
  return NO_REG_mask();
}

// Returns true if the high 32 bits of the value is known to be zero.
bool is_operand_hi32_zero(Node* n) {
  int opc = n->Opcode();
  if (opc == Op_AndL) {
    Node* o2 = n->in(2);
    if (o2->is_Con() && (o2->get_long() & 0xFFFFFFFF00000000LL) == 0LL) {
      return true;
    }
  }
  if (opc == Op_ConL && (n->get_long() & 0xFFFFFFFF00000000LL) == 0LL) {
    return true;
  }
  return false;
}


#line 1321 "ad_x86.cpp"


//SourceForm

#line 1264 "src/hotspot/cpu/x86/x86.ad"


#include "opto/addnode.hpp"
#include "c2_intelJccErratum_x86.hpp"

void PhaseOutput::pd_perform_mach_node_analysis() {
  if (VM_Version::has_intel_jcc_erratum()) {
    int extra_padding = IntelJccErratum::tag_affected_machnodes(C, C->cfg(), C->regalloc());
    _buf_sizes._code += extra_padding;
  }
}

int MachNode::pd_alignment_required() const {
  if (VM_Version::has_intel_jcc_erratum() && IntelJccErratum::is_jcc_erratum_branch(this)) {
    // Conservatively add worst case padding. We assume that relocInfo::addr_unit() is 1 on x86.
    return IntelJccErratum::largest_jcc_size() + 1;
  } else {
    return 1;
  }
}

int MachNode::compute_padding(int current_offset) const {
  if (flags() & Node::PD::Flag_intel_jcc_erratum) {
    Compile* C = Compile::current();
    PhaseOutput* output = C->output();
    Block* block = output->block();
    int index = output->index();
    return IntelJccErratum::compute_padding(current_offset, this, block, index, C->regalloc());
  } else {
    return 0;
  }
}

// Emit exception handler code.
// Stuff framesize into a register and call a VM stub routine.
int HandlerImpl::emit_exception_handler(CodeBuffer& cbuf) {

  // Note that the code buffer's insts_mark is always relative to insts.
  // That's why we must use the macroassembler to generate a handler.
  C2_MacroAssembler _masm(&cbuf);
  address base = __ start_a_stub(size_exception_handler());
  if (base == NULL) {
    ciEnv::current()->record_failure("CodeCache is full");
    return 0;  // CodeBuffer::expand failed
  }
  int offset = __ offset();
  __ jump(RuntimeAddress(OptoRuntime::exception_blob()->entry_point()));
  assert(__ offset() - offset <= (int) size_exception_handler(), "overflow");
  __ end_a_stub();
  return offset;
}

// Emit deopt handler code.
int HandlerImpl::emit_deopt_handler(CodeBuffer& cbuf) {

  // Note that the code buffer's insts_mark is always relative to insts.
  // That's why we must use the macroassembler to generate a handler.
  C2_MacroAssembler _masm(&cbuf);
  address base = __ start_a_stub(size_deopt_handler());
  if (base == NULL) {
    ciEnv::current()->record_failure("CodeCache is full");
    return 0;  // CodeBuffer::expand failed
  }
  int offset = __ offset();

#ifdef _LP64
  address the_pc = (address) __ pc();
  Label next;
  // push a "the_pc" on the stack without destroying any registers
  // as they all may be live.

  // push address of "next"
  __ call(next, relocInfo::none); // reloc none is fine since it is a disp32
  __ bind(next);
  // adjust it so it matches "the_pc"
  __ subptr(Address(rsp, 0), __ offset() - offset);
#else
  InternalAddress here(__ pc());
  __ pushptr(here.addr(), noreg);
#endif

  __ jump(RuntimeAddress(SharedRuntime::deopt_blob()->unpack()));
  assert(__ offset() - offset <= (int) size_deopt_handler(), "overflow %d", (__ offset() - offset));
  __ end_a_stub();
  return offset;
}

Assembler::Width widthForType(BasicType bt) {
  if (bt == T_BYTE) {
    return Assembler::B;
  } else if (bt == T_SHORT) {
    return Assembler::W;
  } else if (bt == T_INT) {
    return Assembler::D;
  } else {
    assert(bt == T_LONG, "not a long: %s", type2name(bt));
    return Assembler::Q;
  }
}

//=============================================================================

  // Float masks come from different places depending on platform.
#ifdef _LP64
  static address float_signmask()  { return StubRoutines::x86::float_sign_mask(); }
  static address float_signflip()  { return StubRoutines::x86::float_sign_flip(); }
  static address double_signmask() { return StubRoutines::x86::double_sign_mask(); }
  static address double_signflip() { return StubRoutines::x86::double_sign_flip(); }
#else
  static address float_signmask()  { return (address)float_signmask_pool; }
  static address float_signflip()  { return (address)float_signflip_pool; }
  static address double_signmask() { return (address)double_signmask_pool; }
  static address double_signflip() { return (address)double_signflip_pool; }
#endif
  static address vector_short_to_byte_mask() { return StubRoutines::x86::vector_short_to_byte_mask(); }
  static address vector_int_to_byte_mask() { return StubRoutines::x86::vector_int_to_byte_mask(); }
  static address vector_byte_perm_mask() { return StubRoutines::x86::vector_byte_perm_mask(); }
  static address vector_long_sign_mask() { return StubRoutines::x86::vector_long_sign_mask(); }
  static address vector_all_bits_set() { return StubRoutines::x86::vector_all_bits_set(); }
  static address vector_int_mask_cmp_bits() { return StubRoutines::x86::vector_int_mask_cmp_bits(); }
  static address vector_int_to_short_mask() { return StubRoutines::x86::vector_int_to_short_mask(); }
  static address vector_byte_shufflemask() { return StubRoutines::x86::vector_byte_shuffle_mask(); }
  static address vector_short_shufflemask() { return StubRoutines::x86::vector_short_shuffle_mask(); }
  static address vector_int_shufflemask() { return StubRoutines::x86::vector_int_shuffle_mask(); }
  static address vector_long_shufflemask() { return StubRoutines::x86::vector_long_shuffle_mask(); }
  static address vector_32_bit_mask() { return StubRoutines::x86::vector_32_bit_mask(); }
  static address vector_64_bit_mask() { return StubRoutines::x86::vector_64_bit_mask(); }
  static address vector_float_signflip() { return StubRoutines::x86::vector_float_sign_flip();}
  static address vector_double_signflip() { return StubRoutines::x86::vector_double_sign_flip();}

//=============================================================================
const bool Matcher::match_rule_supported(int opcode) {
  if (!has_match_rule(opcode)) {
    return false; // no match rule present
  }
  const bool is_LP64 = LP64_ONLY(true) NOT_LP64(false);
  switch (opcode) {
    case Op_AbsVL:
    case Op_StoreVectorScatter:
      if (UseAVX < 3) {
        return false;
      }
      break;
    case Op_PopCountI:
    case Op_PopCountL:
      if (!UsePopCountInstruction) {
        return false;
      }
      break;
    case Op_PopCountVI:
      if (UseAVX < 2) {
        return false;
      }
      break;
    case Op_PopCountVL:
      if (UseAVX < 2) {
        return false;
      }
      break;
    case Op_MulVI:
      if ((UseSSE < 4) && (UseAVX < 1)) { // only with SSE4_1 or AVX
        return false;
      }
      break;
    case Op_MulVL:
      if (UseSSE < 4) { // only with SSE4_1 or AVX
        return false;
      }
      break;
    case Op_MulReductionVL:
      if (VM_Version::supports_avx512dq() == false) {
        return false;
      }
      break;
    case Op_AddReductionVL:
      if (UseSSE < 2) { // requires at least SSE2
        return false;
      }
      break;
    case Op_AbsVB:
    case Op_AbsVS:
    case Op_AbsVI:
    case Op_AddReductionVI:
    case Op_AndReductionV:
    case Op_OrReductionV:
    case Op_XorReductionV:
      if (UseSSE < 3) { // requires at least SSSE3
        return false;
      }
      break;
    case Op_VectorLoadShuffle:
    case Op_VectorRearrange:
    case Op_MulReductionVI:
      if (UseSSE < 4) { // requires at least SSE4
        return false;
      }
      break;
    case Op_IsInfiniteF:
    case Op_IsInfiniteD:
      if (!VM_Version::supports_avx512dq()) {
        return false;
      }
      break;
    case Op_SqrtVD:
    case Op_SqrtVF:
    case Op_VectorMaskCmp:
    case Op_VectorCastB2X:
    case Op_VectorCastS2X:
    case Op_VectorCastI2X:
    case Op_VectorCastL2X:
    case Op_VectorCastF2X:
    case Op_VectorCastD2X:
    case Op_VectorUCastB2X:
    case Op_VectorUCastS2X:
    case Op_VectorUCastI2X:
    case Op_VectorMaskCast:
      if (UseAVX < 1) { // enabled for AVX only
        return false;
      }
      break;
    case Op_PopulateIndex:
      if (!is_LP64 || (UseAVX < 2)) {
        return false;
      }
      break;
    case Op_RoundVF:
      if (UseAVX < 2) { // enabled for AVX2 only
        return false;
      }
      break;
    case Op_RoundVD:
      if (UseAVX < 3) {
        return false;  // enabled for AVX3 only
      }
      break;
    case Op_CompareAndSwapL:
#ifdef _LP64
    case Op_CompareAndSwapP:
#endif
      if (!VM_Version::supports_cx8()) {
        return false;
      }
      break;
    case Op_StrIndexOf:
      if (!UseSSE42Intrinsics) {
        return false;
      }
      break;
    case Op_StrIndexOfChar:
      if (!UseSSE42Intrinsics) {
        return false;
      }
      break;
    case Op_OnSpinWait:
      if (VM_Version::supports_on_spin_wait() == false) {
        return false;
      }
      break;
    case Op_MulVB:
    case Op_LShiftVB:
    case Op_RShiftVB:
    case Op_URShiftVB:
    case Op_VectorInsert:
    case Op_VectorLoadMask:
    case Op_VectorStoreMask:
    case Op_VectorBlend:
      if (UseSSE < 4) {
        return false;
      }
      break;
#ifdef _LP64
    case Op_MaxD:
    case Op_MaxF:
    case Op_MinD:
    case Op_MinF:
      if (UseAVX < 1) { // enabled for AVX only
        return false;
      }
      break;
#endif
    case Op_CacheWB:
    case Op_CacheWBPreSync:
    case Op_CacheWBPostSync:
      if (!VM_Version::supports_data_cache_line_flush()) {
        return false;
      }
      break;
    case Op_ExtractB:
    case Op_ExtractL:
    case Op_ExtractI:
    case Op_RoundDoubleMode:
      if (UseSSE < 4) {
        return false;
      }
      break;
    case Op_RoundDoubleModeV:
      if (VM_Version::supports_avx() == false) {
        return false; // 128bit vroundpd is not available
      }
      break;
    case Op_LoadVectorGather:
      if (UseAVX < 2) {
        return false;
      }
      break;
    case Op_FmaVD:
    case Op_FmaVF:
      if (!UseFMA) {
        return false;
      }
      break;
    case Op_MacroLogicV:
      if (UseAVX < 3 || !UseVectorMacroLogic) {
        return false;
      }
      break;

    case Op_VectorCmpMasked:
    case Op_VectorMaskGen:
      if (!is_LP64  || UseAVX < 3 || !VM_Version::supports_bmi2()) {
        return false;
      }
      break;
    case Op_VectorMaskFirstTrue:
    case Op_VectorMaskLastTrue:
    case Op_VectorMaskTrueCount:
    case Op_VectorMaskToLong:
      if (!is_LP64 || UseAVX < 1) {
         return false;
      }
      break;
    case Op_RoundF:
    case Op_RoundD:
      if (!is_LP64) {
        return false;
      }
      break;
    case Op_CopySignD:
    case Op_CopySignF:
      if (UseAVX < 3 || !is_LP64)  {
        return false;
      }
      if (!VM_Version::supports_avx512vl()) {
        return false;
      }
      break;
#ifndef _LP64
    case Op_AddReductionVF:
    case Op_AddReductionVD:
    case Op_MulReductionVF:
    case Op_MulReductionVD:
      if (UseSSE < 1) { // requires at least SSE
        return false;
      }
      break;
    case Op_MulAddVS2VI:
    case Op_RShiftVL:
    case Op_AbsVD:
    case Op_NegVD:
      if (UseSSE < 2) {
        return false;
      }
      break;
#endif // !LP64
    case Op_CompressBits:
      if (!VM_Version::supports_bmi2() || (!is_LP64 && UseSSE < 2)) {
        return false;
      }
      break;
    case Op_ExpandBits:
      if (!VM_Version::supports_bmi2() || (!is_LP64 && (UseSSE < 2 || !VM_Version::supports_bmi1()))) {
        return false;
      }
      break;
    case Op_SignumF:
      if (UseSSE < 1) {
        return false;
      }
      break;
    case Op_SignumD:
      if (UseSSE < 2) {
        return false;
      }
      break;
    case Op_CompressM:
      if (!VM_Version::supports_avx512vl() || !VM_Version::supports_bmi2()) {
        return false;
      }
      break;
    case Op_CompressV:
    case Op_ExpandV:
      if (!VM_Version::supports_avx512vl()) {
        return false;
      }
      break;
    case Op_SqrtF:
      if (UseSSE < 1) {
        return false;
      }
      break;
    case Op_SqrtD:
#ifdef _LP64
      if (UseSSE < 2) {
        return false;
      }
#else
      // x86_32.ad has a special match rule for SqrtD.
      // Together with common x86 rules, this handles all UseSSE cases.
#endif
      break;
    case Op_ConvF2HF:
    case Op_ConvHF2F:
      if (!VM_Version::supports_float16()) {
        return false;
      }
      break;
    case Op_VectorCastF2HF:
    case Op_VectorCastHF2F:
      if (!VM_Version::supports_f16c() && !VM_Version::supports_evex()) {
        return false;
      }
      break;
  }
  return true;  // Match rules are supported by default.
}

//------------------------------------------------------------------------

static inline bool is_pop_count_instr_target(BasicType bt) {
  return (is_subword_type(bt) && VM_Version::supports_avx512_bitalg()) ||
         (is_non_subword_integral_type(bt) && VM_Version::supports_avx512_vpopcntdq());
}

const bool Matcher::match_rule_supported_superword(int opcode, int vlen, BasicType bt) {
  return match_rule_supported_vector(opcode, vlen, bt);
}

// Identify extra cases that we might want to provide match rules for vector nodes and
// other intrinsics guarded with vector length (vlen) and element type (bt).
const bool Matcher::match_rule_supported_vector(int opcode, int vlen, BasicType bt) {
  const bool is_LP64 = LP64_ONLY(true) NOT_LP64(false);
  if (!match_rule_supported(opcode)) {
    return false;
  }
  // Matcher::vector_size_supported() restricts vector sizes in the following way (see Matcher::vector_width_in_bytes):
  //   * SSE2 supports 128bit vectors for all types;
  //   * AVX1 supports 256bit vectors only for FLOAT and DOUBLE types;
  //   * AVX2 supports 256bit vectors for all types;
  //   * AVX512F supports 512bit vectors only for INT, FLOAT, and DOUBLE types;
  //   * AVX512BW supports 512bit vectors for BYTE, SHORT, and CHAR types.
  // There's also a limit on minimum vector size supported: 2 elements (or 4 bytes for BYTE).
  // And MaxVectorSize is taken into account as well.
  if (!vector_size_supported(bt, vlen)) {
    return false;
  }
  // Special cases which require vector length follow:
  //   * implementation limitations
  //   * some 512bit vector operations on FLOAT and DOUBLE types require AVX512DQ
  //   * 128bit vroundpd instruction is present only in AVX1
  int size_in_bits = vlen * type2aelembytes(bt) * BitsPerByte;
  switch (opcode) {
    case Op_AbsVF:
    case Op_NegVF:
      if ((vlen == 16) && (VM_Version::supports_avx512dq() == false)) {
        return false; // 512bit vandps and vxorps are not available
      }
      break;
    case Op_AbsVD:
    case Op_NegVD:
      if ((vlen == 8) && (VM_Version::supports_avx512dq() == false)) {
        return false; // 512bit vpmullq, vandpd and vxorpd are not available
      }
      break;
    case Op_RotateRightV:
    case Op_RotateLeftV:
      if (bt != T_INT && bt != T_LONG) {
        return false;
      } // fallthrough
    case Op_MacroLogicV:
      if (!VM_Version::supports_evex() ||
          ((size_in_bits != 512) && !VM_Version::supports_avx512vl())) {
        return false;
      }
      break;
    case Op_ClearArray:
    case Op_VectorMaskGen:
    case Op_VectorCmpMasked:
      if (!is_LP64 || !VM_Version::supports_avx512bw()) {
        return false;
      }
      if ((size_in_bits != 512) && !VM_Version::supports_avx512vl()) {
        return false;
      }
      break;
    case Op_LoadVectorMasked:
    case Op_StoreVectorMasked:
      if (!VM_Version::supports_avx512bw() && (is_subword_type(bt) || UseAVX < 1)) {
        return false;
      }
      break;
    case Op_MaxV:
    case Op_MinV:
      if (UseSSE < 4 && is_integral_type(bt)) {
        return false;
      }
      if ((bt == T_FLOAT || bt == T_DOUBLE)) {
          // Float/Double intrinsics are enabled for AVX family currently.
          if (UseAVX == 0) {
            return false;
          }
          if (UseAVX > 2 && (!VM_Version::supports_avx512dq() && size_in_bits == 512)) { // 512 bit Float/Double intrinsics need AVX512DQ
            return false;
          }
      }
      break;
    case Op_CallLeafVector:
      if (size_in_bits == 512 && !VM_Version::supports_avx512vlbwdq()) {
        return false;
      }
      break;
    case Op_AddReductionVI:
      if (bt == T_INT && (UseSSE < 3 || !VM_Version::supports_ssse3())) {
        return false;
      }
      // fallthrough
    case Op_AndReductionV:
    case Op_OrReductionV:
    case Op_XorReductionV:
      if (is_subword_type(bt) && (UseSSE < 4)) {
        return false;
      }
#ifndef _LP64
      if (bt == T_BYTE || bt == T_LONG) {
        return false;
      }
#endif
      break;
#ifndef _LP64
    case Op_VectorInsert:
      if (bt == T_LONG || bt == T_DOUBLE) {
        return false;
      }
      break;
#endif
    case Op_MinReductionV:
    case Op_MaxReductionV:
      if ((bt == T_INT || is_subword_type(bt)) && UseSSE < 4) {
        return false;
      } else if (bt == T_LONG && (UseAVX < 3 || !VM_Version::supports_avx512vlbwdq())) {
        return false;
      }
      // Float/Double intrinsics enabled for AVX family.
      if (UseAVX == 0 && (bt == T_FLOAT || bt == T_DOUBLE)) {
        return false;
      }
      if (UseAVX > 2 && (!VM_Version::supports_avx512dq() && size_in_bits == 512)) {
        return false;
      }
#ifndef _LP64
      if (bt == T_BYTE || bt == T_LONG) {
        return false;
      }
#endif
      break;
    case Op_VectorTest:
      if (UseSSE < 4) {
        return false; // Implementation limitation
      } else if (size_in_bits < 32) {
        return false; // Implementation limitation
      }
      break;
    case Op_VectorLoadShuffle:
    case Op_VectorRearrange:
      if(vlen == 2) {
        return false; // Implementation limitation due to how shuffle is loaded
      } else if (size_in_bits == 256 && UseAVX < 2) {
        return false; // Implementation limitation
      }
      break;
    case Op_VectorLoadMask:
    case Op_VectorMaskCast:
      if (size_in_bits == 256 && UseAVX < 2) {
        return false; // Implementation limitation
      }
      // fallthrough
    case Op_VectorStoreMask:
      if (vlen == 2) {
        return false; // Implementation limitation
      }
      break;
    case Op_PopulateIndex:
      if (size_in_bits > 256 && !VM_Version::supports_avx512bw()) {
        return false;
      }
      break;
    case Op_VectorCastB2X:
    case Op_VectorCastS2X:
    case Op_VectorCastI2X:
      if (bt != T_DOUBLE && size_in_bits == 256 && UseAVX < 2) {
        return false;
      }
      break;
    case Op_VectorCastL2X:
      if (is_integral_type(bt) && size_in_bits == 256 && UseAVX < 2) {
        return false;
      } else if (!is_integral_type(bt) && !VM_Version::supports_avx512dq()) {
        return false;
      }
      break;
    case Op_VectorCastF2X: {
        // As per JLS section 5.1.3 narrowing conversion to sub-word types
        // happen after intermediate conversion to integer and special handling
        // code needs AVX2 vpcmpeqd instruction for 256 bit vectors.
        int src_size_in_bits = type2aelembytes(T_FLOAT) * vlen * BitsPerByte;
        if (is_integral_type(bt) && src_size_in_bits == 256 && UseAVX < 2) {
          return false;
        }
      }
      // fallthrough
    case Op_VectorCastD2X:
      if (bt == T_LONG && !VM_Version::supports_avx512dq()) {
        return false;
      }
      break;
    case Op_VectorCastF2HF:
    case Op_VectorCastHF2F:
      if (!VM_Version::supports_f16c() &&
         ((!VM_Version::supports_evex() ||
         ((size_in_bits != 512) && !VM_Version::supports_avx512vl())))) {
        return false;
      }
      break;
    case Op_RoundVD:
      if (!VM_Version::supports_avx512dq()) {
        return false;
      }
      break;
    case Op_MulReductionVI:
      if (bt == T_BYTE && size_in_bits == 512 && !VM_Version::supports_avx512bw()) {
        return false;
      }
      break;
    case Op_LoadVectorGatherMasked:
    case Op_StoreVectorScatterMasked:
    case Op_StoreVectorScatter:
      if (is_subword_type(bt)) {
        return false;
      } else if (size_in_bits < 512 && !VM_Version::supports_avx512vl()) {
        return false;
      }
      // fallthrough
    case Op_LoadVectorGather:
      if (size_in_bits == 64 ) {
        return false;
      }
      break;
    case Op_MaskAll:
      if (!VM_Version::supports_evex()) {
        return false;
      }
      if ((vlen > 16 || is_subword_type(bt)) && !VM_Version::supports_avx512bw()) {
        return false;
      }
      if (size_in_bits < 512 && !VM_Version::supports_avx512vl()) {
        return false;
      }
      break;
    case Op_VectorMaskCmp:
      if (vlen < 2 || size_in_bits < 32) {
        return false;
      }
      break;
    case Op_CompressM:
      if (UseAVX < 3 || !VM_Version::supports_bmi2()) {
        return false;
      }
      break;
    case Op_CompressV:
    case Op_ExpandV:
      if (is_subword_type(bt) && !VM_Version::supports_avx512_vbmi2()) {
        return false;
      }
      if (size_in_bits < 128 ) {
        return false;
      }
      if (size_in_bits < 512 && !VM_Version::supports_avx512vl()) {
        return false;
      }
      break;
    case Op_VectorLongToMask:
      if (UseAVX < 1 || !is_LP64) {
        return false;
      }
      if (UseAVX < 3 && !VM_Version::supports_bmi2()) {
        return false;
      }
      break;
    case Op_SignumVD:
    case Op_SignumVF:
      if (UseAVX < 1) {
        return false;
      }
      break;
    case Op_PopCountVI:
    case Op_PopCountVL: {
        if (!is_pop_count_instr_target(bt) &&
            (size_in_bits == 512) && !VM_Version::supports_avx512bw()) {
          return false;
        }
      }
      break;
    case Op_ReverseV:
    case Op_ReverseBytesV:
      if (UseAVX < 2) {
        return false;
      }
      break;
    case Op_CountTrailingZerosV:
    case Op_CountLeadingZerosV:
      if (UseAVX < 2) {
        return false;
      }
      break;
  }
  return true;  // Per default match rules are supported.
}

const bool Matcher::match_rule_supported_vector_masked(int opcode, int vlen, BasicType bt) {
  // ADLC based match_rule_supported routine checks for the existence of pattern based
  // on IR opcode. Most of the unary/binary/ternary masked operation share the IR nodes
  // of their non-masked counterpart with mask edge being the differentiator.
  // This routine does a strict check on the existence of masked operation patterns
  // by returning a default false value for all the other opcodes apart from the
  // ones whose masked instruction patterns are defined in this file.
  if (!match_rule_supported_vector(opcode, vlen, bt)) {
    return false;
  }

  const bool is_LP64 = LP64_ONLY(true) NOT_LP64(false);
  int size_in_bits = vlen * type2aelembytes(bt) * BitsPerByte;
  if (size_in_bits != 512 && !VM_Version::supports_avx512vl()) {
    return false;
  }
  switch(opcode) {
    // Unary masked operations
    case Op_AbsVB:
    case Op_AbsVS:
      if(!VM_Version::supports_avx512bw()) {
        return false;  // Implementation limitation
      }
    case Op_AbsVI:
    case Op_AbsVL:
      return true;

    // Ternary masked operations
    case Op_FmaVF:
    case Op_FmaVD:
      return true;

    case Op_MacroLogicV:
      if(bt != T_INT && bt != T_LONG) {
        return false;
      }
      return true;

    // Binary masked operations
    case Op_AddVB:
    case Op_AddVS:
    case Op_SubVB:
    case Op_SubVS:
    case Op_MulVS:
    case Op_LShiftVS:
    case Op_RShiftVS:
    case Op_URShiftVS:
      assert(size_in_bits == 512 || VM_Version::supports_avx512vl(), "");
      if (!VM_Version::supports_avx512bw()) {
        return false;  // Implementation limitation
      }
      return true;

    case Op_MulVL:
      assert(size_in_bits == 512 || VM_Version::supports_avx512vl(), "");
      if (!VM_Version::supports_avx512dq()) {
        return false;  // Implementation limitation
      }
      return true;

    case Op_AndV:
    case Op_OrV:
    case Op_XorV:
    case Op_RotateRightV:
    case Op_RotateLeftV:
      if (bt != T_INT && bt != T_LONG) {
        return false; // Implementation limitation
      }
      return true;

    case Op_VectorLoadMask:
      assert(size_in_bits == 512 || VM_Version::supports_avx512vl(), "");
      if (is_subword_type(bt) && !VM_Version::supports_avx512bw()) {
        return false;
      }
      return true;

    case Op_AddVI:
    case Op_AddVL:
    case Op_AddVF:
    case Op_AddVD:
    case Op_SubVI:
    case Op_SubVL:
    case Op_SubVF:
    case Op_SubVD:
    case Op_MulVI:
    case Op_MulVF:
    case Op_MulVD:
    case Op_DivVF:
    case Op_DivVD:
    case Op_SqrtVF:
    case Op_SqrtVD:
    case Op_LShiftVI:
    case Op_LShiftVL:
    case Op_RShiftVI:
    case Op_RShiftVL:
    case Op_URShiftVI:
    case Op_URShiftVL:
    case Op_LoadVectorMasked:
    case Op_StoreVectorMasked:
    case Op_LoadVectorGatherMasked:
    case Op_StoreVectorScatterMasked:
      return true;

    case Op_MaxV:
    case Op_MinV:
      if (is_subword_type(bt) && !VM_Version::supports_avx512bw()) {
        return false; // Implementation limitation
      }
      if (is_floating_point_type(bt)) {
        return false; // Implementation limitation
      }
      return true;

    case Op_VectorMaskCmp:
      if (is_subword_type(bt) && !VM_Version::supports_avx512bw()) {
        return false; // Implementation limitation
      }
      return true;

    case Op_VectorRearrange:
      if (bt == T_SHORT && !VM_Version::supports_avx512bw()) {
        return false; // Implementation limitation
      }
      if (bt == T_BYTE && !VM_Version::supports_avx512_vbmi()) {
        return false; // Implementation limitation
      } else if ((bt == T_INT || bt == T_FLOAT) && size_in_bits < 256) {
        return false; // Implementation limitation
      }
      return true;

    // Binary Logical operations
    case Op_AndVMask:
    case Op_OrVMask:
    case Op_XorVMask:
      if (vlen > 16 && !VM_Version::supports_avx512bw()) {
        return false; // Implementation limitation
      }
      return true;

    case Op_PopCountVI:
    case Op_PopCountVL:
      if (!is_pop_count_instr_target(bt)) {
        return false;
      }
      return true;

    case Op_MaskAll:
      return true;

    case Op_CountLeadingZerosV:
      if (is_non_subword_integral_type(bt) && VM_Version::supports_avx512cd()) {
        return true;
      }
    default:
      return false;
  }
}

const bool Matcher::vector_needs_partial_operations(Node* node, const TypeVect* vt) {
  return false;
}

MachOper* Matcher::pd_specialize_generic_vector_operand(MachOper* generic_opnd, uint ideal_reg, bool is_temp) {
  assert(Matcher::is_generic_vector(generic_opnd), "not generic");
  bool legacy = (generic_opnd->opcode() == LEGVEC);
  if (!VM_Version::supports_avx512vlbwdq() && // KNL
      is_temp && !legacy && (ideal_reg == Op_VecZ)) {
    // Conservatively specialize 512bit vec TEMP operands to legVecZ (zmm0-15) on KNL.
    return new legVecZOper();
  }
  if (legacy) {
    switch (ideal_reg) {
      case Op_VecS: return new legVecSOper();
      case Op_VecD: return new legVecDOper();
      case Op_VecX: return new legVecXOper();
      case Op_VecY: return new legVecYOper();
      case Op_VecZ: return new legVecZOper();
    }
  } else {
    switch (ideal_reg) {
      case Op_VecS: return new vecSOper();
      case Op_VecD: return new vecDOper();
      case Op_VecX: return new vecXOper();
      case Op_VecY: return new vecYOper();
      case Op_VecZ: return new vecZOper();
    }
  }
  ShouldNotReachHere();
  return NULL;
}

bool Matcher::is_reg2reg_move(MachNode* m) {
  switch (m->rule()) {
    case MoveVec2Leg_rule:
    case MoveLeg2Vec_rule:
    case MoveF2VL_rule:
    case MoveF2LEG_rule:
    case MoveVL2F_rule:
    case MoveLEG2F_rule:
    case MoveD2VL_rule:
    case MoveD2LEG_rule:
    case MoveVL2D_rule:
    case MoveLEG2D_rule:
      return true;
    default:
      return false;
  }
}

bool Matcher::is_generic_vector(MachOper* opnd) {
  switch (opnd->opcode()) {
    case VEC:
    case LEGVEC:
      return true;
    default:
      return false;
  }
}

//------------------------------------------------------------------------

const RegMask* Matcher::predicate_reg_mask(void) {
  return &_VECTMASK_REG_mask;
}

const TypeVectMask* Matcher::predicate_reg_type(const Type* elemTy, int length) {
  return new TypeVectMask(elemTy, length);
}

// Max vector size in bytes. 0 if not supported.
const int Matcher::vector_width_in_bytes(BasicType bt) {
  assert(is_java_primitive(bt), "only primitive type vectors");
  if (UseSSE < 2) return 0;
  // SSE2 supports 128bit vectors for all types.
  // AVX2 supports 256bit vectors for all types.
  // AVX2/EVEX supports 512bit vectors for all types.
  int size = (UseAVX > 1) ? (1 << UseAVX) * 8 : 16;
  // AVX1 supports 256bit vectors only for FLOAT and DOUBLE.
  if (UseAVX > 0 && (bt == T_FLOAT || bt == T_DOUBLE))
    size = (UseAVX > 2) ? 64 : 32;
  if (UseAVX > 2 && (bt == T_BYTE || bt == T_SHORT || bt == T_CHAR))
    size = (VM_Version::supports_avx512bw()) ? 64 : 32;
  // Use flag to limit vector size.
  size = MIN2(size,(int)MaxVectorSize);
  // Minimum 2 values in vector (or 4 for bytes).
  switch (bt) {
  case T_DOUBLE:
  case T_LONG:
    if (size < 16) return 0;
    break;
  case T_FLOAT:
  case T_INT:
    if (size < 8) return 0;
    break;
  case T_BOOLEAN:
    if (size < 4) return 0;
    break;
  case T_CHAR:
    if (size < 4) return 0;
    break;
  case T_BYTE:
    if (size < 4) return 0;
    break;
  case T_SHORT:
    if (size < 4) return 0;
    break;
  default:
    ShouldNotReachHere();
  }
  return size;
}

// Limits on vector size (number of elements) loaded into vector.
const int Matcher::max_vector_size(const BasicType bt) {
  return vector_width_in_bytes(bt)/type2aelembytes(bt);
}
const int Matcher::min_vector_size(const BasicType bt) {
  int max_size = max_vector_size(bt);
  // Min size which can be loaded into vector is 4 bytes.
  int size = (type2aelembytes(bt) == 1) ? 4 : 2;
  // Support for calling svml double64 vectors
  if (bt == T_DOUBLE) {
    size = 1;
  }
  return MIN2(size,max_size);
}

const int Matcher::superword_max_vector_size(const BasicType bt) {
  // Limit the max vector size for auto vectorization to 256 bits (32 bytes)
  // by default on Cascade Lake
  if (VM_Version::is_default_intel_cascade_lake()) {
    return MIN2(Matcher::max_vector_size(bt), 32 / type2aelembytes(bt));
  }
  return Matcher::max_vector_size(bt);
}

const int Matcher::scalable_vector_reg_size(const BasicType bt) {
  return -1;
}

// Vector ideal reg corresponding to specified size in bytes
const uint Matcher::vector_ideal_reg(int size) {
  assert(MaxVectorSize >= size, "");
  switch(size) {
    case  4: return Op_VecS;
    case  8: return Op_VecD;
    case 16: return Op_VecX;
    case 32: return Op_VecY;
    case 64: return Op_VecZ;
  }
  ShouldNotReachHere();
  return 0;
}

// Check for shift by small constant as well
static bool clone_shift(Node* shift, Matcher* matcher, Matcher::MStack& mstack, VectorSet& address_visited) {
  if (shift->Opcode() == Op_LShiftX && shift->in(2)->is_Con() &&
      shift->in(2)->get_int() <= 3 &&
      // Are there other uses besides address expressions?
      !matcher->is_visited(shift)) {
    address_visited.set(shift->_idx); // Flag as address_visited
    mstack.push(shift->in(2), Matcher::Visit);
    Node *conv = shift->in(1);
#ifdef _LP64
    // Allow Matcher to match the rule which bypass
    // ConvI2L operation for an array index on LP64
    // if the index value is positive.
    if (conv->Opcode() == Op_ConvI2L &&
        conv->as_Type()->type()->is_long()->_lo >= 0 &&
        // Are there other uses besides address expressions?
        !matcher->is_visited(conv)) {
      address_visited.set(conv->_idx); // Flag as address_visited
      mstack.push(conv->in(1), Matcher::Pre_Visit);
    } else
#endif
      mstack.push(conv, Matcher::Pre_Visit);
    return true;
  }
  return false;
}

// This function identifies sub-graphs in which a 'load' node is
// input to two different nodes, and such that it can be matched
// with BMI instructions like blsi, blsr, etc.
// Example : for b = -a[i] & a[i] can be matched to blsi r32, m32.
// The graph is (AndL (SubL Con0 LoadL*) LoadL*), where LoadL*
// refers to the same node.
//
// Match the generic fused operations pattern (op1 (op2 Con{ConType} mop) mop)
// This is a temporary solution until we make DAGs expressible in ADL.
template<typename ConType>
class FusedPatternMatcher {
  Node* _op1_node;
  Node* _mop_node;
  int _con_op;

  static int match_next(Node* n, int next_op, int next_op_idx) {
    if (n->in(1) == NULL || n->in(2) == NULL) {
      return -1;
    }

    if (next_op_idx == -1) { // n is commutative, try rotations
      if (n->in(1)->Opcode() == next_op) {
        return 1;
      } else if (n->in(2)->Opcode() == next_op) {
        return 2;
      }
    } else {
      assert(next_op_idx > 0 && next_op_idx <= 2, "Bad argument index");
      if (n->in(next_op_idx)->Opcode() == next_op) {
        return next_op_idx;
      }
    }
    return -1;
  }

 public:
  FusedPatternMatcher(Node* op1_node, Node* mop_node, int con_op) :
    _op1_node(op1_node), _mop_node(mop_node), _con_op(con_op) { }

  bool match(int op1, int op1_op2_idx,  // op1 and the index of the op1->op2 edge, -1 if op1 is commutative
             int op2, int op2_con_idx,  // op2 and the index of the op2->con edge, -1 if op2 is commutative
             typename ConType::NativeType con_value) {
    if (_op1_node->Opcode() != op1) {
      return false;
    }
    if (_mop_node->outcnt() > 2) {
      return false;
    }
    op1_op2_idx = match_next(_op1_node, op2, op1_op2_idx);
    if (op1_op2_idx == -1) {
      return false;
    }
    // Memory operation must be the other edge
    int op1_mop_idx = (op1_op2_idx & 1) + 1;

    // Check that the mop node is really what we want
    if (_op1_node->in(op1_mop_idx) == _mop_node) {
      Node* op2_node = _op1_node->in(op1_op2_idx);
      if (op2_node->outcnt() > 1) {
        return false;
      }
      assert(op2_node->Opcode() == op2, "Should be");
      op2_con_idx = match_next(op2_node, _con_op, op2_con_idx);
      if (op2_con_idx == -1) {
        return false;
      }
      // Memory operation must be the other edge
      int op2_mop_idx = (op2_con_idx & 1) + 1;
      // Check that the memory operation is the same node
      if (op2_node->in(op2_mop_idx) == _mop_node) {
        // Now check the constant
        const Type* con_type = op2_node->in(op2_con_idx)->bottom_type();
        if (con_type != Type::TOP && ConType::as_self(con_type)->get_con() == con_value) {
          return true;
        }
      }
    }
    return false;
  }
};

static bool is_bmi_pattern(Node* n, Node* m) {
  assert(UseBMI1Instructions, "sanity");
  if (n != NULL && m != NULL) {
    if (m->Opcode() == Op_LoadI) {
      FusedPatternMatcher<TypeInt> bmii(n, m, Op_ConI);
      return bmii.match(Op_AndI, -1, Op_SubI,  1,  0)  ||
             bmii.match(Op_AndI, -1, Op_AddI, -1, -1)  ||
             bmii.match(Op_XorI, -1, Op_AddI, -1, -1);
    } else if (m->Opcode() == Op_LoadL) {
      FusedPatternMatcher<TypeLong> bmil(n, m, Op_ConL);
      return bmil.match(Op_AndL, -1, Op_SubL,  1,  0) ||
             bmil.match(Op_AndL, -1, Op_AddL, -1, -1) ||
             bmil.match(Op_XorL, -1, Op_AddL, -1, -1);
    }
  }
  return false;
}

// Should the matcher clone input 'm' of node 'n'?
bool Matcher::pd_clone_node(Node* n, Node* m, Matcher::MStack& mstack) {
  // If 'n' and 'm' are part of a graph for BMI instruction, clone the input 'm'.
  if (UseBMI1Instructions && is_bmi_pattern(n, m)) {
    mstack.push(m, Visit);
    return true;
  }
  if (is_vshift_con_pattern(n, m)) { // ShiftV src (ShiftCntV con)
    mstack.push(m, Visit);           // m = ShiftCntV
    return true;
  }
  return false;
}

// Should the Matcher clone shifts on addressing modes, expecting them
// to be subsumed into complex addressing expressions or compute them
// into registers?
bool Matcher::pd_clone_address_expressions(AddPNode* m, Matcher::MStack& mstack, VectorSet& address_visited) {
  Node *off = m->in(AddPNode::Offset);
  if (off->is_Con()) {
    address_visited.test_set(m->_idx); // Flag as address_visited
    Node *adr = m->in(AddPNode::Address);

    // Intel can handle 2 adds in addressing mode
    // AtomicAdd is not an addressing expression.
    // Cheap to find it by looking for screwy base.
    if (adr->is_AddP() &&
        !adr->in(AddPNode::Base)->is_top() &&
        LP64_ONLY( off->get_long() == (int) (off->get_long()) && ) // immL32
        // Are there other uses besides address expressions?
        !is_visited(adr)) {
      address_visited.set(adr->_idx); // Flag as address_visited
      Node *shift = adr->in(AddPNode::Offset);
      if (!clone_shift(shift, this, mstack, address_visited)) {
        mstack.push(shift, Pre_Visit);
      }
      mstack.push(adr->in(AddPNode::Address), Pre_Visit);
      mstack.push(adr->in(AddPNode::Base), Pre_Visit);
    } else {
      mstack.push(adr, Pre_Visit);
    }

    // Clone X+offset as it also folds into most addressing expressions
    mstack.push(off, Visit);
    mstack.push(m->in(AddPNode::Base), Pre_Visit);
    return true;
  } else if (clone_shift(off, this, mstack, address_visited)) {
    address_visited.test_set(m->_idx); // Flag as address_visited
    mstack.push(m->in(AddPNode::Address), Pre_Visit);
    mstack.push(m->in(AddPNode::Base), Pre_Visit);
    return true;
  }
  return false;
}

static inline Assembler::ComparisonPredicate booltest_pred_to_comparison_pred(int bt) {
  switch (bt) {
    case BoolTest::eq:
      return Assembler::eq;
    case BoolTest::ne:
      return Assembler::neq;
    case BoolTest::le:
    case BoolTest::ule:
      return Assembler::le;
    case BoolTest::ge:
    case BoolTest::uge:
      return Assembler::nlt;
    case BoolTest::lt:
    case BoolTest::ult:
      return Assembler::lt;
    case BoolTest::gt:
    case BoolTest::ugt:
      return Assembler::nle;
    default : ShouldNotReachHere(); return Assembler::_false;
  }
}

static inline Assembler::ComparisonPredicateFP booltest_pred_to_comparison_pred_fp(int bt) {
  switch (bt) {
  case BoolTest::eq: return Assembler::EQ_OQ;  // ordered non-signaling
  // As per JLS 15.21.1, != of NaNs is true. Thus use unordered compare.
  case BoolTest::ne: return Assembler::NEQ_UQ; // unordered non-signaling
  case BoolTest::le: return Assembler::LE_OQ;  // ordered non-signaling
  case BoolTest::ge: return Assembler::GE_OQ;  // ordered non-signaling
  case BoolTest::lt: return Assembler::LT_OQ;  // ordered non-signaling
  case BoolTest::gt: return Assembler::GT_OQ;  // ordered non-signaling
  default: ShouldNotReachHere(); return Assembler::FALSE_OS;
  }
}

// Helper methods for MachSpillCopyNode::implementation().
static void vec_mov_helper(CodeBuffer *cbuf, int src_lo, int dst_lo,
                          int src_hi, int dst_hi, uint ireg, outputStream* st) {
  assert(ireg == Op_VecS || // 32bit vector
         (src_lo & 1) == 0 && (src_lo + 1) == src_hi &&
         (dst_lo & 1) == 0 && (dst_lo + 1) == dst_hi,
         "no non-adjacent vector moves" );
  if (cbuf) {
    C2_MacroAssembler _masm(cbuf);
    switch (ireg) {
    case Op_VecS: // copy whole register
    case Op_VecD:
    case Op_VecX:
#ifndef _LP64
      __ movdqu(as_XMMRegister(Matcher::_regEncode[dst_lo]), as_XMMRegister(Matcher::_regEncode[src_lo]));
#else
      if ((UseAVX < 3) || VM_Version::supports_avx512vl()) {
        __ movdqu(as_XMMRegister(Matcher::_regEncode[dst_lo]), as_XMMRegister(Matcher::_regEncode[src_lo]));
      } else {
        __ vextractf32x4(as_XMMRegister(Matcher::_regEncode[dst_lo]), as_XMMRegister(Matcher::_regEncode[src_lo]), 0x0);
     }
#endif
      break;
    case Op_VecY:
#ifndef _LP64
      __ vmovdqu(as_XMMRegister(Matcher::_regEncode[dst_lo]), as_XMMRegister(Matcher::_regEncode[src_lo]));
#else
      if ((UseAVX < 3) || VM_Version::supports_avx512vl()) {
        __ vmovdqu(as_XMMRegister(Matcher::_regEncode[dst_lo]), as_XMMRegister(Matcher::_regEncode[src_lo]));
      } else {
        __ vextractf64x4(as_XMMRegister(Matcher::_regEncode[dst_lo]), as_XMMRegister(Matcher::_regEncode[src_lo]), 0x0);
     }
#endif
      break;
    case Op_VecZ:
      __ evmovdquq(as_XMMRegister(Matcher::_regEncode[dst_lo]), as_XMMRegister(Matcher::_regEncode[src_lo]), 2);
      break;
    default:
      ShouldNotReachHere();
    }
#ifndef PRODUCT
  } else {
    switch (ireg) {
    case Op_VecS:
    case Op_VecD:
    case Op_VecX:
      st->print("movdqu  %s,%s\t# spill",Matcher::regName[dst_lo],Matcher::regName[src_lo]);
      break;
    case Op_VecY:
    case Op_VecZ:
      st->print("vmovdqu %s,%s\t# spill",Matcher::regName[dst_lo],Matcher::regName[src_lo]);
      break;
    default:
      ShouldNotReachHere();
    }
#endif
  }
}

void vec_spill_helper(CodeBuffer *cbuf, bool is_load,
                     int stack_offset, int reg, uint ireg, outputStream* st) {
  if (cbuf) {
    C2_MacroAssembler _masm(cbuf);
    if (is_load) {
      switch (ireg) {
      case Op_VecS:
        __ movdl(as_XMMRegister(Matcher::_regEncode[reg]), Address(rsp, stack_offset));
        break;
      case Op_VecD:
        __ movq(as_XMMRegister(Matcher::_regEncode[reg]), Address(rsp, stack_offset));
        break;
      case Op_VecX:
#ifndef _LP64
        __ movdqu(as_XMMRegister(Matcher::_regEncode[reg]), Address(rsp, stack_offset));
#else
        if ((UseAVX < 3) || VM_Version::supports_avx512vl()) {
          __ movdqu(as_XMMRegister(Matcher::_regEncode[reg]), Address(rsp, stack_offset));
        } else {
          __ vpxor(as_XMMRegister(Matcher::_regEncode[reg]), as_XMMRegister(Matcher::_regEncode[reg]), as_XMMRegister(Matcher::_regEncode[reg]), 2);
          __ vinsertf32x4(as_XMMRegister(Matcher::_regEncode[reg]), as_XMMRegister(Matcher::_regEncode[reg]), Address(rsp, stack_offset),0x0);
        }
#endif
        break;
      case Op_VecY:
#ifndef _LP64
        __ vmovdqu(as_XMMRegister(Matcher::_regEncode[reg]), Address(rsp, stack_offset));
#else
        if ((UseAVX < 3) || VM_Version::supports_avx512vl()) {
          __ vmovdqu(as_XMMRegister(Matcher::_regEncode[reg]), Address(rsp, stack_offset));
        } else {
          __ vpxor(as_XMMRegister(Matcher::_regEncode[reg]), as_XMMRegister(Matcher::_regEncode[reg]), as_XMMRegister(Matcher::_regEncode[reg]), 2);
          __ vinsertf64x4(as_XMMRegister(Matcher::_regEncode[reg]), as_XMMRegister(Matcher::_regEncode[reg]), Address(rsp, stack_offset),0x0);
        }
#endif
        break;
      case Op_VecZ:
        __ evmovdquq(as_XMMRegister(Matcher::_regEncode[reg]), Address(rsp, stack_offset), 2);
        break;
      default:
        ShouldNotReachHere();
      }
    } else { // store
      switch (ireg) {
      case Op_VecS:
        __ movdl(Address(rsp, stack_offset), as_XMMRegister(Matcher::_regEncode[reg]));
        break;
      case Op_VecD:
        __ movq(Address(rsp, stack_offset), as_XMMRegister(Matcher::_regEncode[reg]));
        break;
      case Op_VecX:
#ifndef _LP64
        __ movdqu(Address(rsp, stack_offset), as_XMMRegister(Matcher::_regEncode[reg]));
#else
        if ((UseAVX < 3) || VM_Version::supports_avx512vl()) {
          __ movdqu(Address(rsp, stack_offset), as_XMMRegister(Matcher::_regEncode[reg]));
        }
        else {
          __ vextractf32x4(Address(rsp, stack_offset), as_XMMRegister(Matcher::_regEncode[reg]), 0x0);
        }
#endif
        break;
      case Op_VecY:
#ifndef _LP64
        __ vmovdqu(Address(rsp, stack_offset), as_XMMRegister(Matcher::_regEncode[reg]));
#else
        if ((UseAVX < 3) || VM_Version::supports_avx512vl()) {
          __ vmovdqu(Address(rsp, stack_offset), as_XMMRegister(Matcher::_regEncode[reg]));
        }
        else {
          __ vextractf64x4(Address(rsp, stack_offset), as_XMMRegister(Matcher::_regEncode[reg]), 0x0);
        }
#endif
        break;
      case Op_VecZ:
        __ evmovdquq(Address(rsp, stack_offset), as_XMMRegister(Matcher::_regEncode[reg]), 2);
        break;
      default:
        ShouldNotReachHere();
      }
    }
#ifndef PRODUCT
  } else {
    if (is_load) {
      switch (ireg) {
      case Op_VecS:
        st->print("movd    %s,[rsp + %d]\t# spill", Matcher::regName[reg], stack_offset);
        break;
      case Op_VecD:
        st->print("movq    %s,[rsp + %d]\t# spill", Matcher::regName[reg], stack_offset);
        break;
       case Op_VecX:
        st->print("movdqu  %s,[rsp + %d]\t# spill", Matcher::regName[reg], stack_offset);
        break;
      case Op_VecY:
      case Op_VecZ:
        st->print("vmovdqu %s,[rsp + %d]\t# spill", Matcher::regName[reg], stack_offset);
        break;
      default:
        ShouldNotReachHere();
      }
    } else { // store
      switch (ireg) {
      case Op_VecS:
        st->print("movd    [rsp + %d],%s\t# spill", stack_offset, Matcher::regName[reg]);
        break;
      case Op_VecD:
        st->print("movq    [rsp + %d],%s\t# spill", stack_offset, Matcher::regName[reg]);
        break;
       case Op_VecX:
        st->print("movdqu  [rsp + %d],%s\t# spill", stack_offset, Matcher::regName[reg]);
        break;
      case Op_VecY:
      case Op_VecZ:
        st->print("vmovdqu [rsp + %d],%s\t# spill", stack_offset, Matcher::regName[reg]);
        break;
      default:
        ShouldNotReachHere();
      }
    }
#endif
  }
}

template <class T>
static inline GrowableArray<jvalue>* vreplicate_imm(BasicType bt, T con, int len) {
  GrowableArray<jvalue>* val = new GrowableArray<jvalue>(len);
  jvalue ele;
  switch (bt) {
    case T_BYTE:   ele.b = con; break;
    case T_SHORT:  ele.s = con; break;
    case T_INT:    ele.i = con; break;
    case T_LONG:   ele.j = con; break;
    case T_FLOAT:  ele.f = con; break;
    case T_DOUBLE: ele.d = con; break;
    default: ShouldNotReachHere();
  }
  for (int i = 0; i < len; i++) {
    val->append(ele);
  }
  return val;
}

static inline jlong high_bit_set(BasicType bt) {
  switch (bt) {
    case T_BYTE:  return 0x8080808080808080;
    case T_SHORT: return 0x8000800080008000;
    case T_INT:   return 0x8000000080000000;
    case T_LONG:  return 0x8000000000000000;
    default:
      ShouldNotReachHere();
      return 0;
  }
}

#ifndef PRODUCT
  void MachNopNode::format(PhaseRegAlloc*, outputStream* st) const {
    st->print("nop \t# %d bytes pad for loops and calls", _count);
  }
#endif

  void MachNopNode::emit(CodeBuffer &cbuf, PhaseRegAlloc*) const {
    C2_MacroAssembler _masm(&cbuf);
    __ nop(_count);
  }

  uint MachNopNode::size(PhaseRegAlloc*) const {
    return _count;
  }

#ifndef PRODUCT
  void MachBreakpointNode::format(PhaseRegAlloc*, outputStream* st) const {
    st->print("# breakpoint");
  }
#endif

  void MachBreakpointNode::emit(CodeBuffer &cbuf, PhaseRegAlloc* ra_) const {
    C2_MacroAssembler _masm(&cbuf);
    __ int3();
  }

  uint MachBreakpointNode::size(PhaseRegAlloc* ra_) const {
    return MachNode::size(ra_);
  }


#line 2830 "ad_x86.cpp"


#ifndef PRODUCT
void Compile::adlc_verification() {

  // Following assertions generated from definition section
}
#endif

// Map from machine-independent register number to register_save_policy
const        char register_save_policy[] = {
  'C', // ECX
  'C', // EBX
  'N', // EBP
  'C', // EDI
  'C', // EAX
  'C', // EDX
  'C', // ESI
  'N', // ESP
  'C', // FPR0L
  'C', // FPR0H
  'C', // FPR1L
  'C', // FPR1H
  'C', // FPR2L
  'C', // FPR2H
  'C', // FPR3L
  'C', // FPR3H
  'C', // FPR4L
  'C', // FPR4H
  'C', // FPR5L
  'C', // FPR5H
  'C', // FPR6L
  'C', // FPR6H
  'C', // FPR7L
  'C', // FPR7H
  'C', // FILL0
  'C', // FILL1
  'C', // FILL2
  'C', // FILL3
  'C', // FILL4
  'C', // FILL5
  'C', // FILL6
  'C', // FILL7
  'C', // XMM0
  'C', // XMM0b
  'C', // XMM0c
  'C', // XMM0d
  'C', // XMM0e
  'C', // XMM0f
  'C', // XMM0g
  'C', // XMM0h
  'C', // XMM0i
  'C', // XMM0j
  'C', // XMM0k
  'C', // XMM0l
  'C', // XMM0m
  'C', // XMM0n
  'C', // XMM0o
  'C', // XMM0p
  'C', // XMM1
  'C', // XMM1b
  'C', // XMM1c
  'C', // XMM1d
  'C', // XMM1e
  'C', // XMM1f
  'C', // XMM1g
  'C', // XMM1h
  'C', // XMM1i
  'C', // XMM1j
  'C', // XMM1k
  'C', // XMM1l
  'C', // XMM1m
  'C', // XMM1n
  'C', // XMM1o
  'C', // XMM1p
  'C', // XMM2
  'C', // XMM2b
  'C', // XMM2c
  'C', // XMM2d
  'C', // XMM2e
  'C', // XMM2f
  'C', // XMM2g
  'C', // XMM2h
  'C', // XMM2i
  'C', // XMM2j
  'C', // XMM2k
  'C', // XMM2l
  'C', // XMM2m
  'C', // XMM2n
  'C', // XMM2o
  'C', // XMM2p
  'C', // XMM3
  'C', // XMM3b
  'C', // XMM3c
  'C', // XMM3d
  'C', // XMM3e
  'C', // XMM3f
  'C', // XMM3g
  'C', // XMM3h
  'C', // XMM3i
  'C', // XMM3j
  'C', // XMM3k
  'C', // XMM3l
  'C', // XMM3m
  'C', // XMM3n
  'C', // XMM3o
  'C', // XMM3p
  'C', // XMM4
  'C', // XMM4b
  'C', // XMM4c
  'C', // XMM4d
  'C', // XMM4e
  'C', // XMM4f
  'C', // XMM4g
  'C', // XMM4h
  'C', // XMM4i
  'C', // XMM4j
  'C', // XMM4k
  'C', // XMM4l
  'C', // XMM4m
  'C', // XMM4n
  'C', // XMM4o
  'C', // XMM4p
  'C', // XMM5
  'C', // XMM5b
  'C', // XMM5c
  'C', // XMM5d
  'C', // XMM5e
  'C', // XMM5f
  'C', // XMM5g
  'C', // XMM5h
  'C', // XMM5i
  'C', // XMM5j
  'C', // XMM5k
  'C', // XMM5l
  'C', // XMM5m
  'C', // XMM5n
  'C', // XMM5o
  'C', // XMM5p
  'C', // XMM6
  'C', // XMM6b
  'C', // XMM6c
  'C', // XMM6d
  'C', // XMM6e
  'C', // XMM6f
  'C', // XMM6g
  'C', // XMM6h
  'C', // XMM6i
  'C', // XMM6j
  'C', // XMM6k
  'C', // XMM6l
  'C', // XMM6m
  'C', // XMM6n
  'C', // XMM6o
  'C', // XMM6p
  'C', // XMM7
  'C', // XMM7b
  'C', // XMM7c
  'C', // XMM7d
  'C', // XMM7e
  'C', // XMM7f
  'C', // XMM7g
  'C', // XMM7h
  'C', // XMM7i
  'C', // XMM7j
  'C', // XMM7k
  'C', // XMM7l
  'C', // XMM7m
  'C', // XMM7n
  'C', // XMM7o
  'C', // XMM7p
  'C', // K7
  'C', // K7_H
  'C', // K6
  'C', // K6_H
  'C', // K5
  'C', // K5_H
  'C', // K4
  'C', // K4_H
  'C', // K3
  'C', // K3_H
  'C', // K2
  'C', // K2_H
  'C', // K1
  'C', // K1_H
  'C' // no trailing comma // RFLAGS
};

// Map from machine-independent register number to c_reg_save_policy
const        char c_reg_save_policy[] = {
  'C', // ECX
  'E', // EBX
  'E', // EBP
  'E', // EDI
  'C', // EAX
  'C', // EDX
  'E', // ESI
  'N', // ESP
  'C', // FPR0L
  'C', // FPR0H
  'C', // FPR1L
  'C', // FPR1H
  'C', // FPR2L
  'C', // FPR2H
  'C', // FPR3L
  'C', // FPR3H
  'C', // FPR4L
  'C', // FPR4H
  'C', // FPR5L
  'C', // FPR5H
  'C', // FPR6L
  'C', // FPR6H
  'C', // FPR7L
  'C', // FPR7H
  'C', // FILL0
  'C', // FILL1
  'C', // FILL2
  'C', // FILL3
  'C', // FILL4
  'C', // FILL5
  'C', // FILL6
  'C', // FILL7
  'C', // XMM0
  'C', // XMM0b
  'C', // XMM0c
  'C', // XMM0d
  'C', // XMM0e
  'C', // XMM0f
  'C', // XMM0g
  'C', // XMM0h
  'C', // XMM0i
  'C', // XMM0j
  'C', // XMM0k
  'C', // XMM0l
  'C', // XMM0m
  'C', // XMM0n
  'C', // XMM0o
  'C', // XMM0p
  'C', // XMM1
  'C', // XMM1b
  'C', // XMM1c
  'C', // XMM1d
  'C', // XMM1e
  'C', // XMM1f
  'C', // XMM1g
  'C', // XMM1h
  'C', // XMM1i
  'C', // XMM1j
  'C', // XMM1k
  'C', // XMM1l
  'C', // XMM1m
  'C', // XMM1n
  'C', // XMM1o
  'C', // XMM1p
  'C', // XMM2
  'C', // XMM2b
  'C', // XMM2c
  'C', // XMM2d
  'C', // XMM2e
  'C', // XMM2f
  'C', // XMM2g
  'C', // XMM2h
  'C', // XMM2i
  'C', // XMM2j
  'C', // XMM2k
  'C', // XMM2l
  'C', // XMM2m
  'C', // XMM2n
  'C', // XMM2o
  'C', // XMM2p
  'C', // XMM3
  'C', // XMM3b
  'C', // XMM3c
  'C', // XMM3d
  'C', // XMM3e
  'C', // XMM3f
  'C', // XMM3g
  'C', // XMM3h
  'C', // XMM3i
  'C', // XMM3j
  'C', // XMM3k
  'C', // XMM3l
  'C', // XMM3m
  'C', // XMM3n
  'C', // XMM3o
  'C', // XMM3p
  'C', // XMM4
  'C', // XMM4b
  'C', // XMM4c
  'C', // XMM4d
  'C', // XMM4e
  'C', // XMM4f
  'C', // XMM4g
  'C', // XMM4h
  'C', // XMM4i
  'C', // XMM4j
  'C', // XMM4k
  'C', // XMM4l
  'C', // XMM4m
  'C', // XMM4n
  'C', // XMM4o
  'C', // XMM4p
  'C', // XMM5
  'C', // XMM5b
  'C', // XMM5c
  'C', // XMM5d
  'C', // XMM5e
  'C', // XMM5f
  'C', // XMM5g
  'C', // XMM5h
  'C', // XMM5i
  'C', // XMM5j
  'C', // XMM5k
  'C', // XMM5l
  'C', // XMM5m
  'C', // XMM5n
  'C', // XMM5o
  'C', // XMM5p
  'C', // XMM6
  'C', // XMM6b
  'C', // XMM6c
  'C', // XMM6d
  'C', // XMM6e
  'C', // XMM6f
  'C', // XMM6g
  'C', // XMM6h
  'C', // XMM6i
  'C', // XMM6j
  'C', // XMM6k
  'C', // XMM6l
  'C', // XMM6m
  'C', // XMM6n
  'C', // XMM6o
  'C', // XMM6p
  'C', // XMM7
  'C', // XMM7b
  'C', // XMM7c
  'C', // XMM7d
  'C', // XMM7e
  'C', // XMM7f
  'C', // XMM7g
  'C', // XMM7h
  'C', // XMM7i
  'C', // XMM7j
  'C', // XMM7k
  'C', // XMM7l
  'C', // XMM7m
  'C', // XMM7n
  'C', // XMM7o
  'C', // XMM7p
  'C', // K7
  'C', // K7_H
  'C', // K6
  'C', // K6_H
  'C', // K5
  'C', // K5_H
  'C', // K4
  'C', // K4_H
  'C', // K3
  'C', // K3_H
  'C', // K2
  'C', // K2_H
  'C', // K1
  'C', // K1_H
  'C' // no trailing comma // RFLAGS
};

// Map from machine-independent register number to register_save_type
const        int register_save_type[] = {
  Op_RegI,
  Op_RegI,
  Op_RegI,
  Op_RegI,
  Op_RegI,
  Op_RegI,
  Op_RegI,
  Op_RegI,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegI,
  Op_RegI,
  Op_RegI,
  Op_RegI,
  Op_RegI,
  Op_RegI,
  Op_RegI,
  Op_RegI,
  Op_RegI,
  Op_RegI,
  Op_RegI,
  Op_RegI,
  Op_RegI,
  Op_RegI,
  0 // no trailing comma
};


const        int   reduceOp[] = {
  /*    0 */  0,
  /*    1 */  0,
  /*    2 */  0,
  /*    3 */  0,
  /*    4 */  0,
  /*    5 */  0,
  /*    6 */  0,
  /*    7 */  0,
  /*    8 */  immI_rule,
  /*    9 */  immI_0_rule,
  /*   10 */  immI_1_rule,
  /*   11 */  immI_M1_rule,
  /*   12 */  immI2_rule,
  /*   13 */  immI8_rule,
  /*   14 */  immU8_rule,
  /*   15 */  immI16_rule,
  /*   16 */  immU31_rule,
  /*   17 */  immI_32_rule,
  /*   18 */  immI_1_31_rule,
  /*   19 */  immI_32_63_rule,
  /*   20 */  immI_2_rule,
  /*   21 */  immI_3_rule,
  /*   22 */  immI_4_rule,
  /*   23 */  immI_8_rule,
  /*   24 */  immP_rule,
  /*   25 */  immP0_rule,
  /*   26 */  immL_rule,
  /*   27 */  immL0_rule,
  /*   28 */  immL_M1_rule,
  /*   29 */  immL_127_rule,
  /*   30 */  immL_32bits_rule,
  /*   31 */  immL32_rule,
  /*   32 */  immDPR0_rule,
  /*   33 */  immDPR1_rule,
  /*   34 */  immDPR_rule,
  /*   35 */  immD_rule,
  /*   36 */  immD0_rule,
  /*   37 */  immFPR0_rule,
  /*   38 */  immFPR1_rule,
  /*   39 */  immFPR_rule,
  /*   40 */  immF_rule,
  /*   41 */  immF0_rule,
  /*   42 */  immI_16_rule,
  /*   43 */  immI_24_rule,
  /*   44 */  immI_255_rule,
  /*   45 */  immI_65535_rule,
  /*   46 */  kReg_rule,
  /*   47 */  kReg_K1_rule,
  /*   48 */  kReg_K2_rule,
  /*   49 */  kReg_K3_rule,
  /*   50 */  kReg_K4_rule,
  /*   51 */  kReg_K5_rule,
  /*   52 */  kReg_K6_rule,
  /*   53 */  kReg_K7_rule,
  /*   54 */  rRegI_rule,
  /*   55 */  xRegI_rule,
  /*   56 */  eAXRegI_rule,
  /*   57 */  eBXRegI_rule,
  /*   58 */  eCXRegI_rule,
  /*   59 */  eDXRegI_rule,
  /*   60 */  eDIRegI_rule,
  /*   61 */  naxRegI_rule,
  /*   62 */  nadxRegI_rule,
  /*   63 */  ncxRegI_rule,
  /*   64 */  eSIRegI_rule,
  /*   65 */  anyRegP_rule,
  /*   66 */  eRegP_rule,
  /*   67 */  rRegP_rule,
  /*   68 */  eRegP_no_EBP_rule,
  /*   69 */  naxRegP_rule,
  /*   70 */  nabxRegP_rule,
  /*   71 */  pRegP_rule,
  /*   72 */  eAXRegP_rule,
  /*   73 */  eBXRegP_rule,
  /*   74 */  eCXRegP_rule,
  /*   75 */  eDXRegP_rule,
  /*   76 */  eSIRegP_rule,
  /*   77 */  eDIRegP_rule,
  /*   78 */  eRegL_rule,
  /*   79 */  eADXRegL_rule,
  /*   80 */  eBCXRegL_rule,
  /*   81 */  eBDPRegL_rule,
  /*   82 */  eADXRegL_low_only_rule,
  /*   83 */  rFlagsReg_rule,
  /*   84 */  eFlagsReg_rule,
  /*   85 */  eFlagsRegU_rule,
  /*   86 */  eFlagsRegUCF_rule,
  /*   87 */  flagsReg_long_LTGE_rule,
  /*   88 */  flagsReg_long_EQNE_rule,
  /*   89 */  flagsReg_long_LEGT_rule,
  /*   90 */  flagsReg_ulong_LTGE_rule,
  /*   91 */  flagsReg_ulong_EQNE_rule,
  /*   92 */  flagsReg_ulong_LEGT_rule,
  /*   93 */  regDPR_rule,
  /*   94 */  regDPR1_rule,
  /*   95 */  regDPR2_rule,
  /*   96 */  regnotDPR1_rule,
  /*   97 */  regFPR_rule,
  /*   98 */  regFPR1_rule,
  /*   99 */  regF_rule,
  /*  100 */  legRegF_rule,
  /*  101 */  vlRegF_rule,
  /*  102 */  regD_rule,
  /*  103 */  legRegD_rule,
  /*  104 */  vlRegD_rule,
  /*  105 */  direct_rule,
  /*  106 */  indirect_rule,
  /*  107 */  indOffset8_rule,
  /*  108 */  indOffset32_rule,
  /*  109 */  indOffset32X_rule,
  /*  110 */  indIndexOffset_rule,
  /*  111 */  indIndex_rule,
  /*  112 */  indIndexScale_rule,
  /*  113 */  indIndexScaleOffset_rule,
  /*  114 */  load_long_RegP_rule,
  /*  115 */  load_long_indirect_rule,
  /*  116 */  load_long_indOffset32_rule,
  /*  117 */  stackSlotP_rule,
  /*  118 */  stackSlotI_rule,
  /*  119 */  stackSlotF_rule,
  /*  120 */  stackSlotD_rule,
  /*  121 */  stackSlotL_rule,
  /*  122 */  cmpOp_rule,
  /*  123 */  cmpOpU_rule,
  /*  124 */  cmpOpUCF_rule,
  /*  125 */  cmpOpUCF2_rule,
  /*  126 */  cmpOp_fcmov_rule,
  /*  127 */  cmpOp_commute_rule,
  /*  128 */  cmpOpU_commute_rule,
  /*  129 */  rxmm0_rule,
  /*  130 */  vec_rule,
  /*  131 */  legVec_rule,
  /*  132 */  vecS_rule,
  /*  133 */  legVecS_rule,
  /*  134 */  vecD_rule,
  /*  135 */  legVecD_rule,
  /*  136 */  vecX_rule,
  /*  137 */  legVecX_rule,
  /*  138 */  vecY_rule,
  /*  139 */  legVecY_rule,
  /*  140 */  vecZ_rule,
  /*  141 */  legVecZ_rule,
  // last operand
  /*  142 */  load_long_memory_rule,
  /*  143 */  memory_rule,
  /*  144 */  long_memory_rule,
  // last operand class
  /*  145 */  _AddP_eRegP_rRegI_rule,
  /*  146 */  _LShiftI_rRegI_immI2_rule,
  /*  147 */  _AddP_eRegP__LShiftI_rRegI_immI2_rule,
  /*  148 */  _LoadI_memory__rule,
  /*  149 */  _LoadL_memory__rule,
  /*  150 */  _LoadB_memory__rule,
  /*  151 */  _LoadUB_memory__rule,
  /*  152 */  _AndI__LoadUB_memory__immI_rule,
  /*  153 */  _LoadS_memory__rule,
  /*  154 */  _LShiftI__LoadS_memory__immI_24_rule,
  /*  155 */  _LoadUS_memory__rule,
  /*  156 */  _LShiftI__LoadUS_memory__immI_24_rule,
  /*  157 */  _AndI__LoadUS_memory__immI_255_rule,
  /*  158 */  _AndI__LoadUS_memory__immI_rule,
  /*  159 */  _LShiftI__LoadI_memory__immI_24_rule,
  /*  160 */  _LShiftI__LoadI_memory__immI_16_rule,
  /*  161 */  _AndI__LoadI_memory__immI_255_rule,
  /*  162 */  _AndI__LoadI_memory__immI_65535_rule,
  /*  163 */  _AndI__LoadI_memory__immU31_rule,
  /*  164 */  _ConvI2L__LoadI_memory___rule,
  /*  165 */  _ConvL2I_eRegL__rule,
  /*  166 */  _RoundDouble_regDPR1__rule,
  /*  167 */  _RoundFloat_regFPR1__rule,
  /*  168 */  _ConvD2F_regDPR1__rule,
  /*  169 */  _Binary_cmpOp_eFlagsReg_rule,
  /*  170 */  _Binary_rRegI_rRegI_rule,
  /*  171 */  _Binary_cmpOpU_eFlagsRegU_rule,
  /*  172 */  _Binary_cmpOpUCF_eFlagsRegUCF_rule,
  /*  173 */  _Binary_rRegI__LoadI_memory__rule,
  /*  174 */  _Binary_eRegP_eRegP_rule,
  /*  175 */  _Binary_cmpOp_fcmov_eFlagsRegU_rule,
  /*  176 */  _Binary_regDPR1_regDPR_rule,
  /*  177 */  _Binary_regFPR1_regFPR_rule,
  /*  178 */  _Binary_regDPR_regDPR_rule,
  /*  179 */  _Binary_regFPR_regFPR_rule,
  /*  180 */  _Binary_regF_regF_rule,
  /*  181 */  _Binary_regD_regD_rule,
  /*  182 */  _Binary_eRegL_eRegL_rule,
  /*  183 */  _AddI__LoadI_memory__rRegI_rule,
  /*  184 */  _AddI_rRegI__LoadI_memory__rule,
  /*  185 */  _AddI__LoadI_memory__immI_rule,
  /*  186 */  _AddI__LoadI_memory__immI_1_rule,
  /*  187 */  _AddI__LoadI_memory__immI_M1_rule,
  /*  188 */  _Binary_eADXRegL_eBCXRegL_rule,
  /*  189 */  _Binary_eAXRegP_eCXRegP_rule,
  /*  190 */  _Binary_eAXRegI_eCXRegI_rule,
  /*  191 */  _SubI__LoadI_memory__rRegI_rule,
  /*  192 */  _SubI_immI_0_rRegI_rule,
  /*  193 */  _ConvI2L_nadxRegI__rule,
  /*  194 */  _MulL__ConvI2L_nadxRegI__eADXRegL_low_only_rule,
  /*  195 */  _RShiftL__MulL__ConvI2L_nadxRegI__eADXRegL_low_only_immI_32_rule,
  /*  196 */  _MulL_eADXRegL_low_only__ConvI2L_nadxRegI__rule,
  /*  197 */  _RShiftL__MulL_eADXRegL_low_only__ConvI2L_nadxRegI__immI_32_rule,
  /*  198 */  _RShiftL__MulL__ConvI2L_nadxRegI__eADXRegL_low_only_immI_32_63_rule,
  /*  199 */  _RShiftL__MulL_eADXRegL_low_only__ConvI2L_nadxRegI__immI_32_63_rule,
  /*  200 */  _ConvI2L_eAXRegI__rule,
  /*  201 */  _AndL__ConvI2L_eAXRegI__immL_32bits_rule,
  /*  202 */  _AndL__ConvI2L_nadxRegI__immL_32bits_rule,
  /*  203 */  _RShiftI__LoadI_memory__immI_1_rule,
  /*  204 */  _RShiftI__LoadI_memory__immI8_rule,
  /*  205 */  _LShiftI_xRegI_immI_24_rule,
  /*  206 */  _LShiftI_xRegI_immI_16_rule,
  /*  207 */  _AndI__LoadI_memory__rRegI_rule,
  /*  208 */  _AndI_rRegI__LoadI_memory__rule,
  /*  209 */  _AndI__LoadI_memory__immI_rule,
  /*  210 */  _XorI_rRegI_immI_M1_rule,
  /*  211 */  _SubI_immI_0__LoadI_memory__rule,
  /*  212 */  _AddI_rRegI_immI_M1_rule,
  /*  213 */  _CastP2X_eRegP__rule,
  /*  214 */  _OrI__LoadI_memory__rRegI_rule,
  /*  215 */  _OrI_rRegI__LoadI_memory__rule,
  /*  216 */  _OrI__LoadI_memory__immI_rule,
  /*  217 */  _LShiftI_rRegI_immI_1_rule,
  /*  218 */  _URShiftI_rRegI_immI_M1_rule,
  /*  219 */  _LShiftI_rRegI_immI8_rule,
  /*  220 */  _URShiftI_rRegI_immI8_rule,
  /*  221 */  _LShiftI_ncxRegI_eCXRegI_rule,
  /*  222 */  _SubI_immI_0_eCXRegI_rule,
  /*  223 */  _URShiftI_ncxRegI__SubI_immI_0_eCXRegI_rule,
  /*  224 */  _SubI_immI_32_eCXRegI_rule,
  /*  225 */  _URShiftI_ncxRegI__SubI_immI_32_eCXRegI_rule,
  /*  226 */  _URShiftI_rRegI_immI_1_rule,
  /*  227 */  _LShiftI_rRegI_immI_M1_rule,
  /*  228 */  _URShiftI_ncxRegI_eCXRegI_rule,
  /*  229 */  _LShiftI_ncxRegI__SubI_immI_0_eCXRegI_rule,
  /*  230 */  _LShiftI_ncxRegI__SubI_immI_32_eCXRegI_rule,
  /*  231 */  _XorI__LoadI_memory__rRegI_rule,
  /*  232 */  _XorI_rRegI__LoadI_memory__rule,
  /*  233 */  _XorI__LoadI_memory__immI_rule,
  /*  234 */  _CmpLTMask_rRegI_rRegI_rule,
  /*  235 */  _AndI__CmpLTMask_rRegI_rRegI_rRegI_rule,
  /*  236 */  _SubI_rRegI_rRegI_rule,
  /*  237 */  _AndI_rRegI__CmpLTMask_rRegI_rRegI_rule,
  /*  238 */  _LoadL_load_long_memory__rule,
  /*  239 */  _XorL_eRegL_immL_M1_rule,
  /*  240 */  _SubL_immL0_eRegL_rule,
  /*  241 */  _SubL_immL0__LoadL_memory__rule,
  /*  242 */  _AddL_eRegL_immL_M1_rule,
  /*  243 */  _AddL__LoadL_memory__immL_M1_rule,
  /*  244 */  _LoadD_memory__rule,
  /*  245 */  _SubD_regDPR_regDPR_rule,
  /*  246 */  _AddD_regDPR_regDPR_rule,
  /*  247 */  _AddD__LoadD_memory__regDPR_rule,
  /*  248 */  _RoundDouble__AddD__LoadD_memory__regDPR__rule,
  /*  249 */  _AddD_regDPR__LoadD_memory__rule,
  /*  250 */  _RoundDouble__AddD_regDPR__LoadD_memory___rule,
  /*  251 */  _AddD_regDPR_immDPR_rule,
  /*  252 */  _MulD_regDPR_regDPR_rule,
  /*  253 */  _LoadF_memory__rule,
  /*  254 */  _MulF__LoadF_memory__regFPR_rule,
  /*  255 */  _MulF_regFPR__LoadF_memory__rule,
  /*  256 */  _MulF_regFPR_regFPR_rule,
  /*  257 */  _SubF_regFPR_regFPR_rule,
  /*  258 */  _ConvI2L_rRegI__rule,
  /*  259 */  _Binary_eDIRegP_eCXRegI_rule,
  /*  260 */  _Binary_eSIRegP_eDXRegI_rule,
  /*  261 */  _Binary_eDIRegP_eSIRegP_rule,
  /*  262 */  _Binary_eDIRegP_eDXRegI_rule,
  /*  263 */  _Binary_eSIRegP_immI_rule,
  /*  264 */  _Binary_eSIRegP_eAXRegI_rule,
  /*  265 */  _AndI_rRegI_immI_rule,
  /*  266 */  _AndI_rRegI_memory_rule,
  /*  267 */  _LoadP_memory__rule,
  /*  268 */  _Binary_nadxRegI_eAXRegI_rule,
  /*  269 */  _PartialSubtypeCheck_eSIRegP_eAXRegP_rule,
  /*  270 */  _Binary_cmpOp_flagsReg_long_LTGE_rule,
  /*  271 */  _Binary_eRegL__LoadL_load_long_memory__rule,
  /*  272 */  _Binary_cmpOpU_flagsReg_ulong_LTGE_rule,
  /*  273 */  _Binary_cmpOp_flagsReg_long_EQNE_rule,
  /*  274 */  _Binary_cmpOpU_flagsReg_ulong_EQNE_rule,
  /*  275 */  _Binary_cmpOp_commute_flagsReg_long_LEGT_rule,
  /*  276 */  _Binary_cmpOpU_commute_flagsReg_ulong_LEGT_rule,
  /*  277 */  _ConvF2HF_regF__rule,
  /*  278 */  _VectorCastF2HF_vec__rule,
  /*  279 */  _LoadVector_memory__rule,
  /*  280 */  _Binary_vec_kReg_rule,
  /*  281 */  _Binary_vec_vec_rule,
  /*  282 */  _Binary_vec__Binary_vec_kReg_rule,
  /*  283 */  _Binary_vec_rRegI_rule,
  /*  284 */  _Binary_vec_regF_rule,
  /*  285 */  _LShiftCntV_immI8__rule,
  /*  286 */  _RShiftCntV_immI8__rule,
  /*  287 */  _Binary_legVec_legVec_rule,
  /*  288 */  _Binary_vec__LoadVector_memory__rule,
  /*  289 */  _MulAddVS2VI_vec_vec_rule,
  /*  290 */  _Binary_vec_immU8_rule,
  /*  291 */  _Binary__LoadVector_memory__immU8_rule,
  /*  292 */  _Binary_vec_immI8_rule,
  /*  293 */  _Binary_vec__LShiftCntV_immI8__rule,
  /*  294 */  _Binary_vec__RShiftCntV_immI8__rule,
  /*  295 */  _Binary__LoadVector_memory__kReg_rule,
  /*  296 */  _Binary_immI8_kReg_rule,
  /*  297 */  _Binary_immU8_kReg_rule,
  /*  298 */  _Binary_vec__Binary_immU8_kReg_rule,
  /*  299 */  _Binary_vec__Binary_vec__Binary_immU8_kReg_rule,
  /*  300 */  _Binary_memory__Binary_immU8_kReg_rule,
  /*  301 */  _Binary_vec__Binary_memory__Binary_immU8_kReg_rule,
  /*  302 */  _Binary_eAXRegP_eRegP_rule,
  // last internally defined operand
  /*  303 */  stackSlotI_rule,
  /*  304 */  stackSlotP_rule,
  /*  305 */  stackSlotL_rule,
  /*  306 */  legRegF_rule,
  /*  307 */  regF_rule,
  /*  308 */  vlRegF_rule,
  /*  309 */  regF_rule,
  /*  310 */  legRegD_rule,
  /*  311 */  regD_rule,
  /*  312 */  vlRegD_rule,
  /*  313 */  regD_rule,
  /*  314 */  eRegP_rule,
  /*  315 */  eRegP_rule,
  /*  316 */  eRegP_rule,
  /*  317 */  eRegP_rule,
  /*  318 */  eRegP_rule,
  /*  319 */  rRegI_rule,
  /*  320 */  rRegI_rule,
  /*  321 */  eRegP_rule,
  /*  322 */  eRegL_rule,
  /*  323 */  eRegL_rule,
  /*  324 */  regFPR_rule,
  /*  325 */  regFPR_rule,
  /*  326 */  regFPR_rule,
  /*  327 */  regF_rule,
  /*  328 */  regF_rule,
  /*  329 */  regDPR_rule,
  /*  330 */  regDPR_rule,
  /*  331 */  regDPR_rule,
  /*  332 */  regD_rule,
  /*  333 */  regD_rule,
  /*  334 */  rRegI_rule,
  /*  335 */  eRegL_rule,
  /*  336 */  eRegP_rule,
  /*  337 */  regFPR_rule,
  /*  338 */  regDPR_rule,
  /*  339 */  eADXRegL_low_only_rule,
  /*  340 */  legVec_rule,
  /*  341 */  vec_rule,
  /*  342 */  Universe_rule,
  /*  343 */  eAXRegP_rule,
  /*  344 */  rRegI_rule,
  /*  345 */  eRegP_rule,
  /*  346 */  eRegP_rule,
  /*  347 */  rRegI_rule,
  /*  348 */  eRegL_rule,
  /*  349 */  regF_rule,
  /*  350 */  regD_rule,
  /*  351 */  regFPR_rule,
  /*  352 */  regDPR_rule,
  /*  353 */  0,
  /*  354 */  0,
  /*  355 */  eFlagsReg_rule,
  /*  356 */  eFlagsReg_rule,
  /*  357 */  eFlagsReg_rule,
  /*  358 */  eFlagsReg_rule,
  /*  359 */  eFlagsReg_rule,
  /*  360 */  eFlagsReg_rule,
  /*  361 */  eFlagsReg_rule,
  /*  362 */  eFlagsRegU_rule,
  /*  363 */  eFlagsRegUCF_rule,
  /*  364 */  eFlagsRegU_rule,
  /*  365 */  eFlagsRegU_rule,
  /*  366 */  eFlagsRegUCF_rule,
  /*  367 */  eFlagsRegU_rule,
  /*  368 */  eFlagsRegUCF_rule,
  /*  369 */  regDPR1_rule,
  /*  370 */  regDPR1_rule,
  /*  371 */  eFlagsRegU_rule,
  /*  372 */  eFlagsRegUCF_rule,
  /*  373 */  eFlagsRegU_rule,
  /*  374 */  eFlagsRegU_rule,
  /*  375 */  eFlagsRegUCF_rule,
  /*  376 */  eFlagsRegU_rule,
  /*  377 */  eFlagsRegUCF_rule,
  /*  378 */  regFPR1_rule,
  /*  379 */  regFPR1_rule,
  /*  380 */  rRegI_rule,
  /*  381 */  rRegI_rule,
  /*  382 */  regFPR_rule,
  /*  383 */  regF_rule,
  /*  384 */  regF_rule,
  /*  385 */  eRegL_rule,
  /*  386 */  regDPR_rule,
  /*  387 */  regD_rule,
  /*  388 */  regD_rule,
  /*  389 */  eFlagsReg_rule,
  /*  390 */  eFlagsReg_rule,
  /*  391 */  eFlagsReg_rule,
  /*  392 */  eFlagsReg_rule,
  /*  393 */  eFlagsReg_rule,
  /*  394 */  eFlagsReg_rule,
  /*  395 */  eFlagsRegU_rule,
  /*  396 */  eFlagsRegU_rule,
  /*  397 */  eFlagsRegU_rule,
  /*  398 */  eFlagsRegU_rule,
  /*  399 */  eFlagsRegU_rule,
  /*  400 */  eFlagsRegU_rule,
  /*  401 */  eFlagsRegU_rule,
  /*  402 */  eFlagsRegU_rule,
  /*  403 */  eFlagsReg_rule,
  /*  404 */  eFlagsReg_rule,
  /*  405 */  eFlagsReg_rule,
  /*  406 */  flagsReg_long_LTGE_rule,
  /*  407 */  flagsReg_long_LTGE_rule,
  /*  408 */  flagsReg_ulong_LTGE_rule,
  /*  409 */  flagsReg_ulong_LTGE_rule,
  /*  410 */  flagsReg_long_EQNE_rule,
  /*  411 */  flagsReg_long_EQNE_rule,
  /*  412 */  flagsReg_ulong_EQNE_rule,
  /*  413 */  flagsReg_ulong_EQNE_rule,
  /*  414 */  flagsReg_long_LEGT_rule,
  /*  415 */  flagsReg_long_LEGT_rule,
  /*  416 */  flagsReg_ulong_LEGT_rule,
  /*  417 */  flagsReg_ulong_LEGT_rule,
  /*  418 */  eFlagsReg_rule,
  /*  419 */  eFlagsReg_rule,
  /*  420 */  eFlagsReg_rule,
  /*  421 */  regF_rule,
  /*  422 */  vlRegF_rule,
  /*  423 */  regD_rule,
  /*  424 */  vlRegD_rule,
  /*  425 */  regF_rule,
  /*  426 */  vlRegF_rule,
  /*  427 */  regD_rule,
  /*  428 */  vlRegD_rule,
  /*  429 */  vec_rule,
  /*  430 */  regF_rule,
  /*  431 */  vec_rule,
  /*  432 */  vec_rule,
  /*  433 */  kReg_rule,
  /*  434 */  vec_rule,
  /*  435 */  legVec_rule,
  /*  436 */  legVec_rule,
  /*  437 */  vec_rule,
  /*  438 */  vec_rule,
  /*  439 */  vec_rule,
  /*  440 */  vec_rule,
  /*  441 */  vec_rule,
  /*  442 */  vec_rule,
  /*  443 */  vec_rule,
  /*  444 */  vec_rule,
  /*  445 */  vec_rule,
  /*  446 */  vec_rule,
  /*  447 */  vec_rule,
  /*  448 */  vec_rule,
  /*  449 */  vec_rule,
  /*  450 */  vec_rule,
  /*  451 */  vec_rule,
  /*  452 */  vec_rule,
  /*  453 */  vec_rule,
  /*  454 */  vec_rule,
  /*  455 */  vec_rule,
  /*  456 */  vec_rule,
  /*  457 */  vec_rule,
  /*  458 */  vec_rule,
  /*  459 */  vec_rule,
  /*  460 */  vec_rule,
  /*  461 */  vec_rule,
  /*  462 */  vec_rule,
  /*  463 */  vec_rule,
  /*  464 */  vec_rule,
  /*  465 */  vec_rule,
  /*  466 */  vec_rule,
  /*  467 */  vec_rule,
  /*  468 */  vec_rule,
  /*  469 */  vec_rule,
  /*  470 */  vec_rule,
  /*  471 */  vec_rule,
  /*  472 */  vec_rule,
  /*  473 */  vec_rule,
  /*  474 */  vec_rule,
  /*  475 */  vec_rule,
  /*  476 */  vec_rule,
  /*  477 */  vec_rule,
  /*  478 */  vec_rule,
  /*  479 */  vec_rule,
  /*  480 */  vec_rule,
  /*  481 */  vec_rule,
  /*  482 */  vec_rule,
  /*  483 */  vec_rule,
  /*  484 */  vec_rule,
  /*  485 */  vec_rule,
  /*  486 */  vec_rule,
  /*  487 */  vec_rule,
  /*  488 */  vec_rule,
  /*  489 */  vec_rule,
  /*  490 */  vec_rule,
  /*  491 */  vec_rule,
  /*  492 */  vec_rule,
  /*  493 */  vec_rule,
  /*  494 */  vec_rule,
  /*  495 */  vec_rule,
  /*  496 */  vec_rule,
  /*  497 */  kReg_rule,
  /*  498 */  vec_rule,
  /*  499 */  vec_rule,
  /*  500 */  vec_rule,
  /*  501 */  vec_rule,
  /*  502 */  vec_rule,
  /*  503 */  vec_rule,
  /*  504 */  vec_rule,
  /*  505 */  vec_rule,
  /*  506 */  vec_rule,
  /*  507 */  vec_rule,
  /*  508 */  kReg_rule,
  /*  509 */  vec_rule,
  /*  510 */  legVec_rule,
  /*  511 */  rRegI_rule,
  /*  512 */  eRegL_rule,
  /*  513 */  rRegI_rule,
  /*  514 */  rRegI_rule,
  /*  515 */  rRegI_rule,
  /*  516 */  rRegI_rule,
  /*  517 */  rRegI_rule,
  /*  518 */  rRegI_rule,
  /*  519 */  rRegI_rule,
  /*  520 */  rRegI_rule,
  /*  521 */  rRegI_rule,
  /*  522 */  rRegI_rule,
  /*  523 */  rRegI_rule,
  /*  524 */  rRegI_rule,
  /*  525 */  rRegI_rule,
  /*  526 */  rRegI_rule,
  /*  527 */  xRegI_rule,
  /*  528 */  eRegL_rule,
  /*  529 */  xRegI_rule,
  /*  530 */  eRegL_rule,
  /*  531 */  eRegL_rule,
  /*  532 */  rRegI_rule,
  /*  533 */  rRegI_rule,
  /*  534 */  eRegL_rule,
  /*  535 */  rRegI_rule,
  /*  536 */  rRegI_rule,
  /*  537 */  eRegL_rule,
  /*  538 */  eRegL_rule,
  /*  539 */  eRegL_rule,
  /*  540 */  rRegI_rule,
  /*  541 */  rRegI_rule,
  /*  542 */  rRegI_rule,
  /*  543 */  rRegI_rule,
  /*  544 */  rRegI_rule,
  /*  545 */  eRegL_rule,
  /*  546 */  eRegL_rule,
  /*  547 */  eRegL_rule,
  /*  548 */  eRegL_rule,
  /*  549 */  eRegL_rule,
  /*  550 */  eRegL_rule,
  /*  551 */  stackSlotL_rule,
  /*  552 */  stackSlotL_rule,
  /*  553 */  eRegL_rule,
  /*  554 */  rRegI_rule,
  /*  555 */  eRegP_rule,
  /*  556 */  eRegP_rule,
  /*  557 */  regDPR_rule,
  /*  558 */  regD_rule,
  /*  559 */  regD_rule,
  /*  560 */  regF_rule,
  /*  561 */  regFPR_rule,
  /*  562 */  Universe_rule,
  /*  563 */  Universe_rule,
  /*  564 */  Universe_rule,
  /*  565 */  Universe_rule,
  /*  566 */  Universe_rule,
  /*  567 */  Universe_rule,
  /*  568 */  Universe_rule,
  /*  569 */  Universe_rule,
  /*  570 */  Universe_rule,
  /*  571 */  Universe_rule,
  /*  572 */  Universe_rule,
  /*  573 */  Universe_rule,
  /*  574 */  Universe_rule,
  /*  575 */  Universe_rule,
  /*  576 */  Universe_rule,
  /*  577 */  Universe_rule,
  /*  578 */  Universe_rule,
  /*  579 */  Universe_rule,
  /*  580 */  Universe_rule,
  /*  581 */  Universe_rule,
  /*  582 */  Universe_rule,
  /*  583 */  Universe_rule,
  /*  584 */  Universe_rule,
  /*  585 */  Universe_rule,
  /*  586 */  Universe_rule,
  /*  587 */  Universe_rule,
  /*  588 */  Universe_rule,
  /*  589 */  Universe_rule,
  /*  590 */  Universe_rule,
  /*  591 */  Universe_rule,
  /*  592 */  Universe_rule,
  /*  593 */  Universe_rule,
  /*  594 */  Universe_rule,
  /*  595 */  Universe_rule,
  /*  596 */  Universe_rule,
  /*  597 */  Universe_rule,
  /*  598 */  Universe_rule,
  /*  599 */  rRegI_rule,
  /*  600 */  rRegI_rule,
  /*  601 */  rRegI_rule,
  /*  602 */  rRegI_rule,
  /*  603 */  rRegI_rule,
  /*  604 */  rRegI_rule,
  /*  605 */  rRegI_rule,
  /*  606 */  rRegI_rule,
  /*  607 */  eRegP_rule,
  /*  608 */  eRegP_rule,
  /*  609 */  eRegP_rule,
  /*  610 */  eRegP_rule,
  /*  611 */  regDPR1_rule,
  /*  612 */  regFPR1_rule,
  /*  613 */  regDPR_rule,
  /*  614 */  regFPR_rule,
  /*  615 */  regF_rule,
  /*  616 */  regD_rule,
  /*  617 */  regF_rule,
  /*  618 */  regF_rule,
  /*  619 */  regD_rule,
  /*  620 */  regD_rule,
  /*  621 */  eRegL_rule,
  /*  622 */  eRegL_rule,
  /*  623 */  eRegL_rule,
  /*  624 */  rRegI_rule,
  /*  625 */  rRegI_rule,
  /*  626 */  rRegI_rule,
  /*  627 */  rRegI_rule,
  /*  628 */  eRegP_rule,
  /*  629 */  rRegI_rule,
  /*  630 */  eRegP_rule,
  /*  631 */  eRegP_rule,
  /*  632 */  rRegI_rule,
  /*  633 */  rRegI_rule,
  /*  634 */  Universe_rule,
  /*  635 */  Universe_rule,
  /*  636 */  Universe_rule,
  /*  637 */  Universe_rule,
  /*  638 */  Universe_rule,
  /*  639 */  rRegI_rule,
  /*  640 */  rRegI_rule,
  /*  641 */  rRegI_rule,
  /*  642 */  rRegI_rule,
  /*  643 */  rRegI_rule,
  /*  644 */  rRegI_rule,
  /*  645 */  rRegI_rule,
  /*  646 */  rRegI_rule,
  /*  647 */  rRegI_rule,
  /*  648 */  rRegI_rule,
  /*  649 */  eADXRegL_rule,
  /*  650 */  eAXRegP_rule,
  /*  651 */  eAXRegI_rule,
  /*  652 */  eAXRegI_rule,
  /*  653 */  eAXRegI_rule,
  /*  654 */  Universe_rule,
  /*  655 */  xRegI_rule,
  /*  656 */  Universe_rule,
  /*  657 */  rRegI_rule,
  /*  658 */  Universe_rule,
  /*  659 */  rRegI_rule,
  /*  660 */  xRegI_rule,
  /*  661 */  rRegI_rule,
  /*  662 */  rRegI_rule,
  /*  663 */  pRegP_rule,
  /*  664 */  rRegI_rule,
  /*  665 */  rRegI_rule,
  /*  666 */  rRegI_rule,
  /*  667 */  Universe_rule,
  /*  668 */  eRegP_rule,
  /*  669 */  rRegI_rule,
  /*  670 */  rRegI_rule,
  /*  671 */  rRegI_rule,
  /*  672 */  eDXRegI_rule,
  /*  673 */  eDXRegI_rule,
  /*  674 */  eDXRegI_rule,
  /*  675 */  eDXRegI_rule,
  /*  676 */  rRegI_rule,
  /*  677 */  rRegI_rule,
  /*  678 */  rRegI_rule,
  /*  679 */  rRegI_rule,
  /*  680 */  eADXRegL_rule,
  /*  681 */  eADXRegL_rule,
  /*  682 */  eADXRegL_rule,
  /*  683 */  eADXRegL_rule,
  /*  684 */  eADXRegL_rule,
  /*  685 */  eADXRegL_rule,
  /*  686 */  eADXRegL_rule,
  /*  687 */  eADXRegL_rule,
  /*  688 */  eADXRegL_rule,
  /*  689 */  eAXRegI_rule,
  /*  690 */  eADXRegL_rule,
  /*  691 */  Universe_rule,
  /*  692 */  eDXRegI_rule,
  /*  693 */  eADXRegL_rule,
  /*  694 */  eADXRegL_rule,
  /*  695 */  eADXRegL_rule,
  /*  696 */  rRegI_rule,
  /*  697 */  rRegI_rule,
  /*  698 */  rRegI_rule,
  /*  699 */  rRegI_rule,
  /*  700 */  Universe_rule,
  /*  701 */  rRegI_rule,
  /*  702 */  Universe_rule,
  /*  703 */  rRegI_rule,
  /*  704 */  rRegI_rule,
  /*  705 */  rRegI_rule,
  /*  706 */  rRegI_rule,
  /*  707 */  rRegI_rule,
  /*  708 */  rRegI_rule,
  /*  709 */  rRegI_rule,
  /*  710 */  rRegI_rule,
  /*  711 */  rRegI_rule,
  /*  712 */  rRegI_rule,
  /*  713 */  Universe_rule,
  /*  714 */  Universe_rule,
  /*  715 */  Universe_rule,
  /*  716 */  rRegI_rule,
  /*  717 */  rRegI_rule,
  /*  718 */  rRegI_rule,
  /*  719 */  rRegI_rule,
  /*  720 */  rRegI_rule,
  /*  721 */  rRegI_rule,
  /*  722 */  rRegI_rule,
  /*  723 */  rRegI_rule,
  /*  724 */  rRegI_rule,
  /*  725 */  rRegI_rule,
  /*  726 */  rRegI_rule,
  /*  727 */  rRegI_rule,
  /*  728 */  rRegI_rule,
  /*  729 */  rRegI_rule,
  /*  730 */  rRegI_rule,
  /*  731 */  rRegI_rule,
  /*  732 */  rRegI_rule,
  /*  733 */  rRegI_rule,
  /*  734 */  rRegI_rule,
  /*  735 */  rRegI_rule,
  /*  736 */  rRegI_rule,
  /*  737 */  rRegI_rule,
  /*  738 */  Universe_rule,
  /*  739 */  Universe_rule,
  /*  740 */  Universe_rule,
  /*  741 */  0,
  /*  742 */  0,
  /*  743 */  0,
  /*  744 */  rRegI_rule,
  /*  745 */  rRegI_rule,
  /*  746 */  rRegI_rule,
  /*  747 */  rRegI_rule,
  /*  748 */  ncxRegI_rule,
  /*  749 */  ncxRegI_rule,
  /*  750 */  ncxRegI_rule,
  /*  751 */  ncxRegI_rule,
  /*  752 */  0,
  /*  753 */  0,
  /*  754 */  0,
  /*  755 */  rRegI_rule,
  /*  756 */  rRegI_rule,
  /*  757 */  rRegI_rule,
  /*  758 */  rRegI_rule,
  /*  759 */  ncxRegI_rule,
  /*  760 */  ncxRegI_rule,
  /*  761 */  ncxRegI_rule,
  /*  762 */  ncxRegI_rule,
  /*  763 */  rRegI_rule,
  /*  764 */  rRegI_rule,
  /*  765 */  rRegI_rule,
  /*  766 */  rRegI_rule,
  /*  767 */  rRegI_rule,
  /*  768 */  Universe_rule,
  /*  769 */  Universe_rule,
  /*  770 */  Universe_rule,
  /*  771 */  0,
  /*  772 */  rRegI_rule,
  /*  773 */  0,
  /*  774 */  rRegI_rule,
  /*  775 */  eCXRegI_rule,
  /*  776 */  rRegI_rule,
  /*  777 */  rRegI_rule,
  /*  778 */  rRegI_rule,
  /*  779 */  rRegI_rule,
  /*  780 */  rRegI_rule,
  /*  781 */  rRegI_rule,
  /*  782 */  rRegI_rule,
  /*  783 */  rRegI_rule,
  /*  784 */  eRegL_rule,
  /*  785 */  eRegL_rule,
  /*  786 */  eRegL_rule,
  /*  787 */  eRegL_rule,
  /*  788 */  eRegL_rule,
  /*  789 */  eRegL_rule,
  /*  790 */  eRegL_rule,
  /*  791 */  eRegL_rule,
  /*  792 */  eRegL_rule,
  /*  793 */  eRegL_rule,
  /*  794 */  eRegL_rule,
  /*  795 */  eRegL_rule,
  /*  796 */  eRegL_rule,
  /*  797 */  eRegL_rule,
  /*  798 */  eRegL_rule,
  /*  799 */  eRegL_rule,
  /*  800 */  eRegL_rule,
  /*  801 */  eRegL_rule,
  /*  802 */  eRegL_rule,
  /*  803 */  eRegL_rule,
  /*  804 */  eRegL_rule,
  /*  805 */  eRegL_rule,
  /*  806 */  eRegL_rule,
  /*  807 */  eRegL_rule,
  /*  808 */  eRegL_rule,
  /*  809 */  eRegL_rule,
  /*  810 */  eRegL_rule,
  /*  811 */  eRegL_rule,
  /*  812 */  eRegL_rule,
  /*  813 */  eRegL_rule,
  /*  814 */  eRegL_rule,
  /*  815 */  eRegL_rule,
  /*  816 */  eRegL_rule,
  /*  817 */  eRegL_rule,
  /*  818 */  eRegL_rule,
  /*  819 */  eRegL_rule,
  /*  820 */  eRegL_rule,
  /*  821 */  eRegL_rule,
  /*  822 */  eRegL_rule,
  /*  823 */  eRegL_rule,
  /*  824 */  eRegL_rule,
  /*  825 */  eRegL_rule,
  /*  826 */  eRegL_rule,
  /*  827 */  eRegL_rule,
  /*  828 */  eRegL_rule,
  /*  829 */  eRegL_rule,
  /*  830 */  eRegL_rule,
  /*  831 */  eRegL_rule,
  /*  832 */  eRegL_rule,
  /*  833 */  rRegI_rule,
  /*  834 */  rRegI_rule,
  /*  835 */  xRegI_rule,
  /*  836 */  xRegI_rule,
  /*  837 */  regDPR_rule,
  /*  838 */  stackSlotD_rule,
  /*  839 */  regDPR_rule,
  /*  840 */  regDPR_rule,
  /*  841 */  stackSlotD_rule,
  /*  842 */  regDPR_rule,
  /*  843 */  regDPR_rule,
  /*  844 */  Universe_rule,
  /*  845 */  Universe_rule,
  /*  846 */  regDPR_rule,
  /*  847 */  regDPR_rule,
  /*  848 */  stackSlotD_rule,
  /*  849 */  regDPR_rule,
  /*  850 */  regDPR1_rule,
  /*  851 */  regDPR_rule,
  /*  852 */  regDPR_rule,
  /*  853 */  regDPR_rule,
  /*  854 */  regDPR_rule,
  /*  855 */  regDPR_rule,
  /*  856 */  regDPR_rule,
  /*  857 */  regDPR_rule,
  /*  858 */  regDPR_rule,
  /*  859 */  regDPR_rule,
  /*  860 */  regDPR1_rule,
  /*  861 */  regDPR_rule,
  /*  862 */  regD_rule,
  /*  863 */  regDPR_rule,
  /*  864 */  regD_rule,
  /*  865 */  regDPR_rule,
  /*  866 */  rRegI_rule,
  /*  867 */  rRegI_rule,
  /*  868 */  xRegI_rule,
  /*  869 */  xRegI_rule,
  /*  870 */  stackSlotF_rule,
  /*  871 */  regFPR_rule,
  /*  872 */  stackSlotF_rule,
  /*  873 */  regFPR_rule,
  /*  874 */  stackSlotF_rule,
  /*  875 */  stackSlotF_rule,
  /*  876 */  regFPR_rule,
  /*  877 */  regFPR_rule,
  /*  878 */  stackSlotF_rule,
  /*  879 */  stackSlotF_rule,
  /*  880 */  stackSlotF_rule,
  /*  881 */  stackSlotF_rule,
  /*  882 */  stackSlotF_rule,
  /*  883 */  regFPR_rule,
  /*  884 */  stackSlotF_rule,
  /*  885 */  regFPR_rule,
  /*  886 */  stackSlotF_rule,
  /*  887 */  stackSlotF_rule,
  /*  888 */  regFPR_rule,
  /*  889 */  regFPR_rule,
  /*  890 */  stackSlotF_rule,
  /*  891 */  stackSlotF_rule,
  /*  892 */  regFPR_rule,
  /*  893 */  regFPR_rule,
  /*  894 */  regFPR_rule,
  /*  895 */  regFPR_rule,
  /*  896 */  regFPR_rule,
  /*  897 */  regFPR_rule,
  /*  898 */  regFPR_rule,
  /*  899 */  regFPR_rule,
  /*  900 */  regFPR_rule,
  /*  901 */  regFPR_rule,
  /*  902 */  stackSlotF_rule,
  /*  903 */  regFPR_rule,
  /*  904 */  stackSlotF_rule,
  /*  905 */  regFPR_rule,
  /*  906 */  regF_rule,
  /*  907 */  stackSlotF_rule,
  /*  908 */  stackSlotD_rule,
  /*  909 */  stackSlotF_rule,
  /*  910 */  regF_rule,
  /*  911 */  regF_rule,
  /*  912 */  regDPR_rule,
  /*  913 */  stackSlotD_rule,
  /*  914 */  regDPR_rule,
  /*  915 */  regD_rule,
  /*  916 */  eAXRegI_rule,
  /*  917 */  eAXRegI_rule,
  /*  918 */  eADXRegL_rule,
  /*  919 */  eADXRegL_rule,
  /*  920 */  eAXRegI_rule,
  /*  921 */  eAXRegI_rule,
  /*  922 */  eADXRegL_rule,
  /*  923 */  eADXRegL_rule,
  /*  924 */  regDPR_rule,
  /*  925 */  regD_rule,
  /*  926 */  regD_rule,
  /*  927 */  regD_rule,
  /*  928 */  regDPR_rule,
  /*  929 */  regFPR_rule,
  /*  930 */  stackSlotF_rule,
  /*  931 */  stackSlotF_rule,
  /*  932 */  regFPR_rule,
  /*  933 */  regFPR_rule,
  /*  934 */  regF_rule,
  /*  935 */  regF_rule,
  /*  936 */  eRegL_rule,
  /*  937 */  eRegL_rule,
  /*  938 */  eRegL_rule,
  /*  939 */  stackSlotD_rule,
  /*  940 */  regD_rule,
  /*  941 */  regF_rule,
  /*  942 */  stackSlotF_rule,
  /*  943 */  rRegI_rule,
  /*  944 */  stackSlotI_rule,
  /*  945 */  stackSlotI_rule,
  /*  946 */  stackSlotF_rule,
  /*  947 */  stackSlotL_rule,
  /*  948 */  stackSlotL_rule,
  /*  949 */  eRegL_rule,
  /*  950 */  stackSlotD_rule,
  /*  951 */  regD_rule,
  /*  952 */  eADXRegL_rule,
  /*  953 */  eADXRegL_rule,
  /*  954 */  Universe_rule,
  /*  955 */  Universe_rule,
  /*  956 */  Universe_rule,
  /*  957 */  Universe_rule,
  /*  958 */  Universe_rule,
  /*  959 */  eAXRegI_rule,
  /*  960 */  eAXRegI_rule,
  /*  961 */  eAXRegI_rule,
  /*  962 */  eAXRegI_rule,
  /*  963 */  eAXRegI_rule,
  /*  964 */  eAXRegI_rule,
  /*  965 */  eAXRegI_rule,
  /*  966 */  eAXRegI_rule,
  /*  967 */  eAXRegI_rule,
  /*  968 */  eAXRegI_rule,
  /*  969 */  eBXRegI_rule,
  /*  970 */  eBXRegI_rule,
  /*  971 */  eBXRegI_rule,
  /*  972 */  eBXRegI_rule,
  /*  973 */  eBXRegI_rule,
  /*  974 */  eBXRegI_rule,
  /*  975 */  eBXRegI_rule,
  /*  976 */  eBXRegI_rule,
  /*  977 */  eAXRegI_rule,
  /*  978 */  eAXRegI_rule,
  /*  979 */  eAXRegI_rule,
  /*  980 */  eAXRegI_rule,
  /*  981 */  eAXRegI_rule,
  /*  982 */  eAXRegI_rule,
  /*  983 */  eAXRegI_rule,
  /*  984 */  eAXRegI_rule,
  /*  985 */  Universe_rule,
  /*  986 */  Universe_rule,
  /*  987 */  eAXRegI_rule,
  /*  988 */  eAXRegI_rule,
  /*  989 */  rRegI_rule,
  /*  990 */  rRegI_rule,
  /*  991 */  eAXRegI_rule,
  /*  992 */  Universe_rule,
  /*  993 */  Universe_rule,
  /*  994 */  Universe_rule,
  /*  995 */  Universe_rule,
  /*  996 */  Universe_rule,
  /*  997 */  Universe_rule,
  /*  998 */  Universe_rule,
  /*  999 */  eDIRegP_rule,
  /* 1000 */  Universe_rule,
  /* 1001 */  Universe_rule,
  /* 1002 */  Universe_rule,
  /* 1003 */  Universe_rule,
  /* 1004 */  Universe_rule,
  /* 1005 */  Universe_rule,
  /* 1006 */  eSIRegI_rule,
  /* 1007 */  Universe_rule,
  /* 1008 */  Universe_rule,
  /* 1009 */  eRegL_rule,
  /* 1010 */  eRegL_rule,
  /* 1011 */  eRegL_rule,
  /* 1012 */  eRegL_rule,
  /* 1013 */  rRegI_rule,
  /* 1014 */  rRegI_rule,
  /* 1015 */  rRegI_rule,
  /* 1016 */  rRegI_rule,
  /* 1017 */  eRegP_rule,
  /* 1018 */  eRegP_rule,
  /* 1019 */  regDPR_rule,
  /* 1020 */  regD_rule,
  /* 1021 */  regFPR_rule,
  /* 1022 */  regF_rule,
  /* 1023 */  Universe_rule,
  /* 1024 */  Universe_rule,
  /* 1025 */  eRegL_rule,
  /* 1026 */  eRegL_rule,
  /* 1027 */  rRegI_rule,
  /* 1028 */  rRegI_rule,
  /* 1029 */  rRegI_rule,
  /* 1030 */  rRegI_rule,
  /* 1031 */  eRegP_rule,
  /* 1032 */  eRegP_rule,
  /* 1033 */  regDPR_rule,
  /* 1034 */  regD_rule,
  /* 1035 */  regFPR_rule,
  /* 1036 */  regF_rule,
  /* 1037 */  Universe_rule,
  /* 1038 */  Universe_rule,
  /* 1039 */  eRegL_rule,
  /* 1040 */  eRegL_rule,
  /* 1041 */  eRegL_rule,
  /* 1042 */  eRegL_rule,
  /* 1043 */  rRegI_rule,
  /* 1044 */  rRegI_rule,
  /* 1045 */  rRegI_rule,
  /* 1046 */  rRegI_rule,
  /* 1047 */  eRegP_rule,
  /* 1048 */  eRegP_rule,
  /* 1049 */  regDPR_rule,
  /* 1050 */  regD_rule,
  /* 1051 */  regFPR_rule,
  /* 1052 */  regF_rule,
  /* 1053 */  Universe_rule,
  /* 1054 */  Universe_rule,
  /* 1055 */  Universe_rule,
  /* 1056 */  Universe_rule,
  /* 1057 */  Universe_rule,
  /* 1058 */  Universe_rule,
  /* 1059 */  Universe_rule,
  /* 1060 */  Universe_rule,
  /* 1061 */  eAXRegP_rule,
  /* 1062 */  Universe_rule,
  /* 1063 */  kReg_rule,
  /* 1064 */  kReg_rule,
  /* 1065 */  kReg_rule,
  /* 1066 */  Universe_rule,
  /* 1067 */  eRegP_rule,
  /* 1068 */  Universe_rule,
  /* 1069 */  regF_rule,
  /* 1070 */  regF_rule,
  /* 1071 */  regF_rule,
  /* 1072 */  regF_rule,
  /* 1073 */  regF_rule,
  /* 1074 */  regF_rule,
  /* 1075 */  regF_rule,
  /* 1076 */  regF_rule,
  /* 1077 */  regD_rule,
  /* 1078 */  regD_rule,
  /* 1079 */  regD_rule,
  /* 1080 */  regD_rule,
  /* 1081 */  regD_rule,
  /* 1082 */  regD_rule,
  /* 1083 */  regD_rule,
  /* 1084 */  regD_rule,
  /* 1085 */  regF_rule,
  /* 1086 */  regF_rule,
  /* 1087 */  regF_rule,
  /* 1088 */  regF_rule,
  /* 1089 */  regF_rule,
  /* 1090 */  regF_rule,
  /* 1091 */  regD_rule,
  /* 1092 */  regD_rule,
  /* 1093 */  regD_rule,
  /* 1094 */  regD_rule,
  /* 1095 */  regD_rule,
  /* 1096 */  regD_rule,
  /* 1097 */  regF_rule,
  /* 1098 */  regF_rule,
  /* 1099 */  regF_rule,
  /* 1100 */  regF_rule,
  /* 1101 */  regF_rule,
  /* 1102 */  regF_rule,
  /* 1103 */  regF_rule,
  /* 1104 */  regF_rule,
  /* 1105 */  regD_rule,
  /* 1106 */  regD_rule,
  /* 1107 */  regD_rule,
  /* 1108 */  regD_rule,
  /* 1109 */  regD_rule,
  /* 1110 */  regD_rule,
  /* 1111 */  regD_rule,
  /* 1112 */  regD_rule,
  /* 1113 */  regF_rule,
  /* 1114 */  regF_rule,
  /* 1115 */  regF_rule,
  /* 1116 */  regF_rule,
  /* 1117 */  regF_rule,
  /* 1118 */  regF_rule,
  /* 1119 */  regD_rule,
  /* 1120 */  regD_rule,
  /* 1121 */  regD_rule,
  /* 1122 */  regD_rule,
  /* 1123 */  regD_rule,
  /* 1124 */  regD_rule,
  /* 1125 */  regF_rule,
  /* 1126 */  regD_rule,
  /* 1127 */  rRegI_rule,
  /* 1128 */  Universe_rule,
  /* 1129 */  Universe_rule,
  /* 1130 */  kReg_rule,
  /* 1131 */  kReg_rule,
  /* 1132 */  kReg_rule,
  /* 1133 */  vec_rule,
  /* 1134 */  Universe_rule,
  /* 1135 */  regD_rule,
  /* 1136 */  regF_rule,
  /* 1137 */  vec_rule,
  /* 1138 */  Universe_rule,
  /* 1139 */  legVec_rule,
  /* 1140 */  vec_rule,
  /* 1141 */  vec_rule,
  /* 1142 */  Universe_rule,
  /* 1143 */  Universe_rule,
  /* 1144 */  vec_rule,
  /* 1145 */  legVec_rule,
  /* 1146 */  vec_rule,
  /* 1147 */  vec_rule,
  /* 1148 */  vec_rule,
  /* 1149 */  vec_rule,
  /* 1150 */  vec_rule,
  /* 1151 */  rRegI_rule,
  /* 1152 */  rRegI_rule,
  /* 1153 */  rRegI_rule,
  /* 1154 */  rRegI_rule,
  /* 1155 */  rRegI_rule,
  /* 1156 */  rRegI_rule,
  /* 1157 */  rRegI_rule,
  /* 1158 */  regF_rule,
  /* 1159 */  regF_rule,
  /* 1160 */  regF_rule,
  /* 1161 */  regF_rule,
  /* 1162 */  regF_rule,
  /* 1163 */  regF_rule,
  /* 1164 */  regD_rule,
  /* 1165 */  regD_rule,
  /* 1166 */  regD_rule,
  /* 1167 */  regD_rule,
  /* 1168 */  regD_rule,
  /* 1169 */  regD_rule,
  /* 1170 */  rRegI_rule,
  /* 1171 */  rRegI_rule,
  /* 1172 */  rRegI_rule,
  /* 1173 */  rRegI_rule,
  /* 1174 */  rRegI_rule,
  /* 1175 */  rRegI_rule,
  /* 1176 */  rRegI_rule,
  /* 1177 */  rRegI_rule,
  /* 1178 */  rRegI_rule,
  /* 1179 */  legRegF_rule,
  /* 1180 */  legRegF_rule,
  /* 1181 */  legRegF_rule,
  /* 1182 */  legRegF_rule,
  /* 1183 */  legRegF_rule,
  /* 1184 */  legRegF_rule,
  /* 1185 */  legRegF_rule,
  /* 1186 */  legRegF_rule,
  /* 1187 */  legRegD_rule,
  /* 1188 */  legRegD_rule,
  /* 1189 */  legRegD_rule,
  /* 1190 */  legRegD_rule,
  /* 1191 */  legRegD_rule,
  /* 1192 */  legRegD_rule,
  /* 1193 */  legRegD_rule,
  /* 1194 */  legRegD_rule,
  /* 1195 */  vec_rule,
  /* 1196 */  vec_rule,
  /* 1197 */  vec_rule,
  /* 1198 */  vec_rule,
  /* 1199 */  vec_rule,
  /* 1200 */  vec_rule,
  /* 1201 */  vec_rule,
  /* 1202 */  vec_rule,
  /* 1203 */  vec_rule,
  /* 1204 */  vec_rule,
  /* 1205 */  vec_rule,
  /* 1206 */  vec_rule,
  /* 1207 */  vec_rule,
  /* 1208 */  vec_rule,
  /* 1209 */  vec_rule,
  /* 1210 */  vec_rule,
  /* 1211 */  vec_rule,
  /* 1212 */  vec_rule,
  /* 1213 */  vec_rule,
  /* 1214 */  vec_rule,
  /* 1215 */  vec_rule,
  /* 1216 */  vec_rule,
  /* 1217 */  vec_rule,
  /* 1218 */  vec_rule,
  /* 1219 */  vec_rule,
  /* 1220 */  vec_rule,
  /* 1221 */  vec_rule,
  /* 1222 */  vec_rule,
  /* 1223 */  vec_rule,
  /* 1224 */  vec_rule,
  /* 1225 */  vec_rule,
  /* 1226 */  vec_rule,
  /* 1227 */  vec_rule,
  /* 1228 */  vec_rule,
  /* 1229 */  vec_rule,
  /* 1230 */  vec_rule,
  /* 1231 */  vec_rule,
  /* 1232 */  vec_rule,
  /* 1233 */  vec_rule,
  /* 1234 */  vec_rule,
  /* 1235 */  vec_rule,
  /* 1236 */  vec_rule,
  /* 1237 */  vec_rule,
  /* 1238 */  vec_rule,
  /* 1239 */  vec_rule,
  /* 1240 */  vec_rule,
  /* 1241 */  vec_rule,
  /* 1242 */  vec_rule,
  /* 1243 */  vec_rule,
  /* 1244 */  vec_rule,
  /* 1245 */  vec_rule,
  /* 1246 */  vec_rule,
  /* 1247 */  vec_rule,
  /* 1248 */  vec_rule,
  /* 1249 */  vec_rule,
  /* 1250 */  vec_rule,
  /* 1251 */  vec_rule,
  /* 1252 */  vec_rule,
  /* 1253 */  vec_rule,
  /* 1254 */  vec_rule,
  /* 1255 */  vec_rule,
  /* 1256 */  vec_rule,
  /* 1257 */  vec_rule,
  /* 1258 */  vec_rule,
  /* 1259 */  vec_rule,
  /* 1260 */  vec_rule,
  /* 1261 */  vec_rule,
  /* 1262 */  vec_rule,
  /* 1263 */  vec_rule,
  /* 1264 */  vec_rule,
  /* 1265 */  vec_rule,
  /* 1266 */  vec_rule,
  /* 1267 */  vec_rule,
  /* 1268 */  vec_rule,
  /* 1269 */  vec_rule,
  /* 1270 */  vec_rule,
  /* 1271 */  vec_rule,
  /* 1272 */  vec_rule,
  /* 1273 */  legVec_rule,
  /* 1274 */  legVec_rule,
  /* 1275 */  vec_rule,
  /* 1276 */  vec_rule,
  /* 1277 */  legVec_rule,
  /* 1278 */  legVec_rule,
  /* 1279 */  vec_rule,
  /* 1280 */  vec_rule,
  /* 1281 */  regF_rule,
  /* 1282 */  regD_rule,
  /* 1283 */  vec_rule,
  /* 1284 */  vec_rule,
  /* 1285 */  vec_rule,
  /* 1286 */  vec_rule,
  /* 1287 */  rRegI_rule,
  /* 1288 */  rRegI_rule,
  /* 1289 */  rRegI_rule,
  /* 1290 */  rRegI_rule,
  /* 1291 */  vec_rule,
  /* 1292 */  vec_rule,
  /* 1293 */  vec_rule,
  /* 1294 */  vec_rule,
  /* 1295 */  vec_rule,
  /* 1296 */  vec_rule,
  /* 1297 */  vec_rule,
  /* 1298 */  vec_rule,
  /* 1299 */  vec_rule,
  /* 1300 */  vec_rule,
  /* 1301 */  vec_rule,
  /* 1302 */  vec_rule,
  /* 1303 */  vec_rule,
  /* 1304 */  vec_rule,
  /* 1305 */  vec_rule,
  /* 1306 */  vec_rule,
  /* 1307 */  vec_rule,
  /* 1308 */  vec_rule,
  /* 1309 */  vec_rule,
  /* 1310 */  vec_rule,
  /* 1311 */  vec_rule,
  /* 1312 */  vec_rule,
  /* 1313 */  vec_rule,
  /* 1314 */  vec_rule,
  /* 1315 */  vec_rule,
  /* 1316 */  vec_rule,
  /* 1317 */  vec_rule,
  /* 1318 */  vec_rule,
  /* 1319 */  vec_rule,
  /* 1320 */  vec_rule,
  /* 1321 */  vec_rule,
  /* 1322 */  vec_rule,
  /* 1323 */  vec_rule,
  /* 1324 */  vec_rule,
  /* 1325 */  vec_rule,
  /* 1326 */  vec_rule,
  /* 1327 */  vec_rule,
  /* 1328 */  vec_rule,
  /* 1329 */  vec_rule,
  /* 1330 */  vec_rule,
  /* 1331 */  vec_rule,
  /* 1332 */  vec_rule,
  /* 1333 */  vec_rule,
  /* 1334 */  vec_rule,
  /* 1335 */  vec_rule,
  /* 1336 */  vec_rule,
  /* 1337 */  vec_rule,
  /* 1338 */  vec_rule,
  /* 1339 */  vec_rule,
  /* 1340 */  vec_rule,
  /* 1341 */  vec_rule,
  /* 1342 */  vec_rule,
  /* 1343 */  vec_rule,
  /* 1344 */  vec_rule,
  /* 1345 */  vec_rule,
  /* 1346 */  vec_rule,
  /* 1347 */  vec_rule,
  /* 1348 */  vec_rule,
  /* 1349 */  vec_rule,
  /* 1350 */  vec_rule,
  /* 1351 */  vec_rule,
  /* 1352 */  vec_rule,
  /* 1353 */  vec_rule,
  /* 1354 */  vec_rule,
  /* 1355 */  vec_rule,
  /* 1356 */  vec_rule,
  /* 1357 */  vec_rule,
  /* 1358 */  vec_rule,
  /* 1359 */  vec_rule,
  /* 1360 */  vec_rule,
  /* 1361 */  vec_rule,
  /* 1362 */  vec_rule,
  /* 1363 */  vec_rule,
  /* 1364 */  vec_rule,
  /* 1365 */  vec_rule,
  /* 1366 */  vec_rule,
  /* 1367 */  vec_rule,
  /* 1368 */  vec_rule,
  /* 1369 */  vec_rule,
  /* 1370 */  legVec_rule,
  /* 1371 */  vec_rule,
  /* 1372 */  kReg_rule,
  /* 1373 */  legVec_rule,
  /* 1374 */  legVec_rule,
  /* 1375 */  legVec_rule,
  /* 1376 */  vec_rule,
  /* 1377 */  kReg_rule,
  /* 1378 */  rRegI_rule,
  /* 1379 */  rRegI_rule,
  /* 1380 */  rRegI_rule,
  /* 1381 */  rRegI_rule,
  /* 1382 */  legRegF_rule,
  /* 1383 */  legRegF_rule,
  /* 1384 */  legRegD_rule,
  /* 1385 */  legRegD_rule,
  /* 1386 */  vec_rule,
  /* 1387 */  legVec_rule,
  /* 1388 */  legVec_rule,
  /* 1389 */  vec_rule,
  /* 1390 */  vec_rule,
  /* 1391 */  legVec_rule,
  /* 1392 */  kReg_rule,
  /* 1393 */  kReg_rule,
  /* 1394 */  vec_rule,
  /* 1395 */  vec_rule,
  /* 1396 */  vec_rule,
  /* 1397 */  vec_rule,
  /* 1398 */  vec_rule,
  /* 1399 */  vec_rule,
  /* 1400 */  vec_rule,
  /* 1401 */  vec_rule,
  /* 1402 */  vec_rule,
  /* 1403 */  vec_rule,
  /* 1404 */  legVec_rule,
  /* 1405 */  vec_rule,
  /* 1406 */  vec_rule,
  /* 1407 */  vec_rule,
  /* 1408 */  vec_rule,
  /* 1409 */  legVec_rule,
  /* 1410 */  vec_rule,
  /* 1411 */  vec_rule,
  /* 1412 */  vec_rule,
  /* 1413 */  vec_rule,
  /* 1414 */  vec_rule,
  /* 1415 */  vec_rule,
  /* 1416 */  vec_rule,
  /* 1417 */  vec_rule,
  /* 1418 */  vec_rule,
  /* 1419 */  vec_rule,
  /* 1420 */  vec_rule,
  /* 1421 */  vec_rule,
  /* 1422 */  vec_rule,
  /* 1423 */  vec_rule,
  /* 1424 */  vec_rule,
  /* 1425 */  vec_rule,
  /* 1426 */  vec_rule,
  /* 1427 */  vec_rule,
  /* 1428 */  vec_rule,
  /* 1429 */  vec_rule,
  /* 1430 */  vec_rule,
  /* 1431 */  vec_rule,
  /* 1432 */  vec_rule,
  /* 1433 */  vec_rule,
  /* 1434 */  vec_rule,
  /* 1435 */  vec_rule,
  /* 1436 */  vec_rule,
  /* 1437 */  vec_rule,
  /* 1438 */  vec_rule,
  /* 1439 */  vec_rule,
  /* 1440 */  vec_rule,
  /* 1441 */  Universe_rule,
  /* 1442 */  Universe_rule,
  /* 1443 */  vec_rule,
  /* 1444 */  vec_rule,
  /* 1445 */  vec_rule,
  /* 1446 */  vec_rule,
  /* 1447 */  vec_rule,
  /* 1448 */  vec_rule,
  /* 1449 */  vec_rule,
  /* 1450 */  vec_rule,
  /* 1451 */  vec_rule,
  /* 1452 */  vec_rule,
  /* 1453 */  vec_rule,
  /* 1454 */  vec_rule,
  /* 1455 */  vec_rule,
  /* 1456 */  vec_rule,
  /* 1457 */  vec_rule,
  /* 1458 */  vec_rule,
  /* 1459 */  vec_rule,
  /* 1460 */  vec_rule,
  /* 1461 */  vec_rule,
  /* 1462 */  vec_rule,
  /* 1463 */  vec_rule,
  /* 1464 */  vec_rule,
  /* 1465 */  vec_rule,
  /* 1466 */  vec_rule,
  /* 1467 */  vec_rule,
  /* 1468 */  vec_rule,
  /* 1469 */  vec_rule,
  /* 1470 */  vec_rule,
  /* 1471 */  vec_rule,
  /* 1472 */  vec_rule,
  /* 1473 */  vec_rule,
  /* 1474 */  vec_rule,
  /* 1475 */  vec_rule,
  /* 1476 */  vec_rule,
  /* 1477 */  vec_rule,
  /* 1478 */  vec_rule,
  /* 1479 */  vec_rule,
  /* 1480 */  vec_rule,
  /* 1481 */  vec_rule,
  /* 1482 */  vec_rule,
  /* 1483 */  vec_rule,
  /* 1484 */  vec_rule,
  /* 1485 */  vec_rule,
  /* 1486 */  vec_rule,
  /* 1487 */  vec_rule,
  /* 1488 */  vec_rule,
  /* 1489 */  vec_rule,
  /* 1490 */  vec_rule,
  /* 1491 */  vec_rule,
  /* 1492 */  vec_rule,
  /* 1493 */  vec_rule,
  /* 1494 */  vec_rule,
  /* 1495 */  vec_rule,
  /* 1496 */  vec_rule,
  /* 1497 */  vec_rule,
  /* 1498 */  vec_rule,
  /* 1499 */  vec_rule,
  /* 1500 */  vec_rule,
  /* 1501 */  vec_rule,
  /* 1502 */  vec_rule,
  /* 1503 */  vec_rule,
  /* 1504 */  vec_rule,
  /* 1505 */  vec_rule,
  /* 1506 */  vec_rule,
  /* 1507 */  vec_rule,
  /* 1508 */  vec_rule,
  /* 1509 */  vec_rule,
  /* 1510 */  vec_rule,
  /* 1511 */  vec_rule,
  /* 1512 */  vec_rule,
  /* 1513 */  vec_rule,
  /* 1514 */  vec_rule,
  /* 1515 */  vec_rule,
  /* 1516 */  vec_rule,
  /* 1517 */  vec_rule,
  /* 1518 */  vec_rule,
  /* 1519 */  vec_rule,
  /* 1520 */  vec_rule,
  /* 1521 */  vec_rule,
  /* 1522 */  vec_rule,
  /* 1523 */  vec_rule,
  /* 1524 */  vec_rule,
  /* 1525 */  vec_rule,
  /* 1526 */  vec_rule,
  /* 1527 */  vec_rule,
  /* 1528 */  vec_rule,
  /* 1529 */  vec_rule,
  /* 1530 */  vec_rule,
  /* 1531 */  vec_rule,
  /* 1532 */  vec_rule,
  /* 1533 */  vec_rule,
  /* 1534 */  vec_rule,
  /* 1535 */  vec_rule,
  /* 1536 */  vec_rule,
  /* 1537 */  vec_rule,
  /* 1538 */  vec_rule,
  /* 1539 */  vec_rule,
  /* 1540 */  vec_rule,
  /* 1541 */  vec_rule,
  /* 1542 */  vec_rule,
  /* 1543 */  vec_rule,
  /* 1544 */  vec_rule,
  /* 1545 */  vec_rule,
  /* 1546 */  vec_rule,
  /* 1547 */  vec_rule,
  /* 1548 */  vec_rule,
  /* 1549 */  vec_rule,
  /* 1550 */  vec_rule,
  /* 1551 */  kReg_rule,
  /* 1552 */  kReg_rule,
  /* 1553 */  kReg_rule,
  /* 1554 */  kReg_rule,
  /* 1555 */  kReg_rule,
  /* 1556 */  vec_rule,
  /* 1557 */  vec_rule,
  /* 1558 */  rRegI_rule,
  /* 1559 */  rRegI_rule,
  /* 1560 */  rRegI_rule,
  /* 1561 */  rRegI_rule,
  /* 1562 */  eAXRegP_rule,
  // last instruction
  0 // no trailing comma
};

const        int   leftOp[] = {
  /*    0 */  0,
  /*    1 */  0,
  /*    2 */  0,
  /*    3 */  0,
  /*    4 */  0,
  /*    5 */  0,
  /*    6 */  0,
  /*    7 */  0,
  /*    8 */  0,
  /*    9 */  0,
  /*   10 */  0,
  /*   11 */  0,
  /*   12 */  0,
  /*   13 */  0,
  /*   14 */  0,
  /*   15 */  0,
  /*   16 */  0,
  /*   17 */  0,
  /*   18 */  0,
  /*   19 */  0,
  /*   20 */  0,
  /*   21 */  0,
  /*   22 */  0,
  /*   23 */  0,
  /*   24 */  0,
  /*   25 */  0,
  /*   26 */  0,
  /*   27 */  0,
  /*   28 */  0,
  /*   29 */  0,
  /*   30 */  0,
  /*   31 */  0,
  /*   32 */  0,
  /*   33 */  0,
  /*   34 */  0,
  /*   35 */  0,
  /*   36 */  0,
  /*   37 */  0,
  /*   38 */  0,
  /*   39 */  0,
  /*   40 */  0,
  /*   41 */  0,
  /*   42 */  0,
  /*   43 */  0,
  /*   44 */  0,
  /*   45 */  0,
  /*   46 */  0,
  /*   47 */  0,
  /*   48 */  0,
  /*   49 */  0,
  /*   50 */  0,
  /*   51 */  0,
  /*   52 */  0,
  /*   53 */  0,
  /*   54 */  0,
  /*   55 */  0,
  /*   56 */  0,
  /*   57 */  0,
  /*   58 */  0,
  /*   59 */  0,
  /*   60 */  0,
  /*   61 */  0,
  /*   62 */  0,
  /*   63 */  0,
  /*   64 */  0,
  /*   65 */  0,
  /*   66 */  0,
  /*   67 */  0,
  /*   68 */  0,
  /*   69 */  0,
  /*   70 */  0,
  /*   71 */  0,
  /*   72 */  0,
  /*   73 */  0,
  /*   74 */  0,
  /*   75 */  0,
  /*   76 */  0,
  /*   77 */  0,
  /*   78 */  0,
  /*   79 */  0,
  /*   80 */  0,
  /*   81 */  0,
  /*   82 */  0,
  /*   83 */  0,
  /*   84 */  0,
  /*   85 */  0,
  /*   86 */  0,
  /*   87 */  0,
  /*   88 */  0,
  /*   89 */  0,
  /*   90 */  0,
  /*   91 */  0,
  /*   92 */  0,
  /*   93 */  0,
  /*   94 */  0,
  /*   95 */  0,
  /*   96 */  0,
  /*   97 */  0,
  /*   98 */  0,
  /*   99 */  0,
  /*  100 */  0,
  /*  101 */  0,
  /*  102 */  0,
  /*  103 */  0,
  /*  104 */  0,
  /*  105 */  0,
  /*  106 */  0,
  /*  107 */  eRegP_rule,
  /*  108 */  eRegP_rule,
  /*  109 */  immP_rule,
  /*  110 */  _AddP_eRegP_rRegI_rule,
  /*  111 */  eRegP_rule,
  /*  112 */  eRegP_rule,
  /*  113 */  _AddP_eRegP__LShiftI_rRegI_immI2_rule,
  /*  114 */  0,
  /*  115 */  0,
  /*  116 */  load_long_RegP_rule,
  /*  117 */  0,
  /*  118 */  0,
  /*  119 */  0,
  /*  120 */  0,
  /*  121 */  0,
  /*  122 */  0,
  /*  123 */  0,
  /*  124 */  0,
  /*  125 */  0,
  /*  126 */  0,
  /*  127 */  0,
  /*  128 */  0,
  /*  129 */  0,
  /*  130 */  0,
  /*  131 */  0,
  /*  132 */  0,
  /*  133 */  0,
  /*  134 */  0,
  /*  135 */  0,
  /*  136 */  0,
  /*  137 */  0,
  /*  138 */  0,
  /*  139 */  0,
  /*  140 */  0,
  /*  141 */  0,
  // last operand
  /*  142 */  0,
  /*  143 */  0,
  /*  144 */  0,
  // last operand class
  /*  145 */  eRegP_rule,
  /*  146 */  rRegI_rule,
  /*  147 */  eRegP_rule,
  /*  148 */  memory_rule,
  /*  149 */  memory_rule,
  /*  150 */  memory_rule,
  /*  151 */  memory_rule,
  /*  152 */  _LoadUB_memory__rule,
  /*  153 */  memory_rule,
  /*  154 */  _LoadS_memory__rule,
  /*  155 */  memory_rule,
  /*  156 */  _LoadUS_memory__rule,
  /*  157 */  _LoadUS_memory__rule,
  /*  158 */  _LoadUS_memory__rule,
  /*  159 */  _LoadI_memory__rule,
  /*  160 */  _LoadI_memory__rule,
  /*  161 */  _LoadI_memory__rule,
  /*  162 */  _LoadI_memory__rule,
  /*  163 */  _LoadI_memory__rule,
  /*  164 */  _LoadI_memory__rule,
  /*  165 */  eRegL_rule,
  /*  166 */  regDPR1_rule,
  /*  167 */  regFPR1_rule,
  /*  168 */  regDPR1_rule,
  /*  169 */  cmpOp_rule,
  /*  170 */  rRegI_rule,
  /*  171 */  cmpOpU_rule,
  /*  172 */  cmpOpUCF_rule,
  /*  173 */  rRegI_rule,
  /*  174 */  eRegP_rule,
  /*  175 */  cmpOp_fcmov_rule,
  /*  176 */  regDPR1_rule,
  /*  177 */  regFPR1_rule,
  /*  178 */  regDPR_rule,
  /*  179 */  regFPR_rule,
  /*  180 */  regF_rule,
  /*  181 */  regD_rule,
  /*  182 */  eRegL_rule,
  /*  183 */  _LoadI_memory__rule,
  /*  184 */  rRegI_rule,
  /*  185 */  _LoadI_memory__rule,
  /*  186 */  _LoadI_memory__rule,
  /*  187 */  _LoadI_memory__rule,
  /*  188 */  eADXRegL_rule,
  /*  189 */  eAXRegP_rule,
  /*  190 */  eAXRegI_rule,
  /*  191 */  _LoadI_memory__rule,
  /*  192 */  immI_0_rule,
  /*  193 */  nadxRegI_rule,
  /*  194 */  _ConvI2L_nadxRegI__rule,
  /*  195 */  _MulL__ConvI2L_nadxRegI__eADXRegL_low_only_rule,
  /*  196 */  eADXRegL_low_only_rule,
  /*  197 */  _MulL_eADXRegL_low_only__ConvI2L_nadxRegI__rule,
  /*  198 */  _MulL__ConvI2L_nadxRegI__eADXRegL_low_only_rule,
  /*  199 */  _MulL_eADXRegL_low_only__ConvI2L_nadxRegI__rule,
  /*  200 */  eAXRegI_rule,
  /*  201 */  _ConvI2L_eAXRegI__rule,
  /*  202 */  _ConvI2L_nadxRegI__rule,
  /*  203 */  _LoadI_memory__rule,
  /*  204 */  _LoadI_memory__rule,
  /*  205 */  xRegI_rule,
  /*  206 */  xRegI_rule,
  /*  207 */  _LoadI_memory__rule,
  /*  208 */  rRegI_rule,
  /*  209 */  _LoadI_memory__rule,
  /*  210 */  rRegI_rule,
  /*  211 */  immI_0_rule,
  /*  212 */  rRegI_rule,
  /*  213 */  eRegP_rule,
  /*  214 */  _LoadI_memory__rule,
  /*  215 */  rRegI_rule,
  /*  216 */  _LoadI_memory__rule,
  /*  217 */  rRegI_rule,
  /*  218 */  rRegI_rule,
  /*  219 */  rRegI_rule,
  /*  220 */  rRegI_rule,
  /*  221 */  ncxRegI_rule,
  /*  222 */  immI_0_rule,
  /*  223 */  ncxRegI_rule,
  /*  224 */  immI_32_rule,
  /*  225 */  ncxRegI_rule,
  /*  226 */  rRegI_rule,
  /*  227 */  rRegI_rule,
  /*  228 */  ncxRegI_rule,
  /*  229 */  ncxRegI_rule,
  /*  230 */  ncxRegI_rule,
  /*  231 */  _LoadI_memory__rule,
  /*  232 */  rRegI_rule,
  /*  233 */  _LoadI_memory__rule,
  /*  234 */  rRegI_rule,
  /*  235 */  _CmpLTMask_rRegI_rRegI_rule,
  /*  236 */  rRegI_rule,
  /*  237 */  rRegI_rule,
  /*  238 */  load_long_memory_rule,
  /*  239 */  eRegL_rule,
  /*  240 */  immL0_rule,
  /*  241 */  immL0_rule,
  /*  242 */  eRegL_rule,
  /*  243 */  _LoadL_memory__rule,
  /*  244 */  memory_rule,
  /*  245 */  regDPR_rule,
  /*  246 */  regDPR_rule,
  /*  247 */  _LoadD_memory__rule,
  /*  248 */  _AddD__LoadD_memory__regDPR_rule,
  /*  249 */  regDPR_rule,
  /*  250 */  _AddD_regDPR__LoadD_memory__rule,
  /*  251 */  regDPR_rule,
  /*  252 */  regDPR_rule,
  /*  253 */  memory_rule,
  /*  254 */  _LoadF_memory__rule,
  /*  255 */  regFPR_rule,
  /*  256 */  regFPR_rule,
  /*  257 */  regFPR_rule,
  /*  258 */  rRegI_rule,
  /*  259 */  eDIRegP_rule,
  /*  260 */  eSIRegP_rule,
  /*  261 */  eDIRegP_rule,
  /*  262 */  eDIRegP_rule,
  /*  263 */  eSIRegP_rule,
  /*  264 */  eSIRegP_rule,
  /*  265 */  rRegI_rule,
  /*  266 */  rRegI_rule,
  /*  267 */  memory_rule,
  /*  268 */  nadxRegI_rule,
  /*  269 */  eSIRegP_rule,
  /*  270 */  cmpOp_rule,
  /*  271 */  eRegL_rule,
  /*  272 */  cmpOpU_rule,
  /*  273 */  cmpOp_rule,
  /*  274 */  cmpOpU_rule,
  /*  275 */  cmpOp_commute_rule,
  /*  276 */  cmpOpU_commute_rule,
  /*  277 */  regF_rule,
  /*  278 */  vec_rule,
  /*  279 */  memory_rule,
  /*  280 */  vec_rule,
  /*  281 */  vec_rule,
  /*  282 */  vec_rule,
  /*  283 */  vec_rule,
  /*  284 */  vec_rule,
  /*  285 */  immI8_rule,
  /*  286 */  immI8_rule,
  /*  287 */  legVec_rule,
  /*  288 */  vec_rule,
  /*  289 */  vec_rule,
  /*  290 */  vec_rule,
  /*  291 */  _LoadVector_memory__rule,
  /*  292 */  vec_rule,
  /*  293 */  vec_rule,
  /*  294 */  vec_rule,
  /*  295 */  _LoadVector_memory__rule,
  /*  296 */  immI8_rule,
  /*  297 */  immU8_rule,
  /*  298 */  vec_rule,
  /*  299 */  vec_rule,
  /*  300 */  memory_rule,
  /*  301 */  vec_rule,
  /*  302 */  eAXRegP_rule,
  // last internally defined operand
  /*  303 */  rRegI_rule,
  /*  304 */  eRegP_rule,
  /*  305 */  eRegL_rule,
  /*  306 */  regF_rule,
  /*  307 */  legRegF_rule,
  /*  308 */  regF_rule,
  /*  309 */  vlRegF_rule,
  /*  310 */  regD_rule,
  /*  311 */  legRegD_rule,
  /*  312 */  regD_rule,
  /*  313 */  vlRegD_rule,
  /*  314 */  indOffset8_rule,
  /*  315 */  indOffset32_rule,
  /*  316 */  indIndexOffset_rule,
  /*  317 */  indIndexScale_rule,
  /*  318 */  indIndexScaleOffset_rule,
  /*  319 */  immI_rule,
  /*  320 */  immI_0_rule,
  /*  321 */  immP_rule,
  /*  322 */  immL_rule,
  /*  323 */  immL0_rule,
  /*  324 */  immFPR_rule,
  /*  325 */  immFPR0_rule,
  /*  326 */  immFPR1_rule,
  /*  327 */  immF_rule,
  /*  328 */  immF0_rule,
  /*  329 */  immDPR_rule,
  /*  330 */  immDPR0_rule,
  /*  331 */  immDPR1_rule,
  /*  332 */  immD_rule,
  /*  333 */  immD0_rule,
  /*  334 */  stackSlotI_rule,
  /*  335 */  stackSlotL_rule,
  /*  336 */  stackSlotP_rule,
  /*  337 */  stackSlotF_rule,
  /*  338 */  stackSlotD_rule,
  /*  339 */  immL32_rule,
  /*  340 */  vec_rule,
  /*  341 */  legVec_rule,
  /*  342 */  0,
  /*  343 */  eAXRegI_rule,
  /*  344 */  eRegP_rule,
  /*  345 */  eRegP_rule,
  /*  346 */  eRegP_rule,
  /*  347 */  rRegI_rule,
  /*  348 */  eRegL_rule,
  /*  349 */  regF_rule,
  /*  350 */  regD_rule,
  /*  351 */  regFPR_rule,
  /*  352 */  regDPR_rule,
  /*  353 */  0,
  /*  354 */  0,
  /*  355 */  eAXRegI_rule,
  /*  356 */  eAXRegI_rule,
  /*  357 */  rRegI_rule,
  /*  358 */  rRegI_rule,
  /*  359 */  immI_0_rule,
  /*  360 */  eAXRegI_rule,
  /*  361 */  rRegI_rule,
  /*  362 */  regDPR_rule,
  /*  363 */  regDPR_rule,
  /*  364 */  regDPR_rule,
  /*  365 */  regD_rule,
  /*  366 */  regD_rule,
  /*  367 */  regD_rule,
  /*  368 */  regD_rule,
  /*  369 */  regDPR1_rule,
  /*  370 */  regDPR1_rule,
  /*  371 */  regFPR_rule,
  /*  372 */  regFPR_rule,
  /*  373 */  regFPR_rule,
  /*  374 */  regF_rule,
  /*  375 */  regF_rule,
  /*  376 */  regF_rule,
  /*  377 */  regF_rule,
  /*  378 */  regFPR1_rule,
  /*  379 */  regFPR1_rule,
  /*  380 */  stackSlotF_rule,
  /*  381 */  regF_rule,
  /*  382 */  stackSlotI_rule,
  /*  383 */  stackSlotI_rule,
  /*  384 */  rRegI_rule,
  /*  385 */  stackSlotD_rule,
  /*  386 */  stackSlotL_rule,
  /*  387 */  stackSlotL_rule,
  /*  388 */  stackSlotL_rule,
  /*  389 */  rRegI_rule,
  /*  390 */  rRegI_rule,
  /*  391 */  rRegI_rule,
  /*  392 */  rRegI_rule,
  /*  393 */  _AndI_rRegI_immI_rule,
  /*  394 */  _AndI_rRegI_memory_rule,
  /*  395 */  rRegI_rule,
  /*  396 */  rRegI_rule,
  /*  397 */  rRegI_rule,
  /*  398 */  rRegI_rule,
  /*  399 */  eRegP_rule,
  /*  400 */  eRegP_rule,
  /*  401 */  eRegP_rule,
  /*  402 */  eRegP_rule,
  /*  403 */  eRegP_rule,
  /*  404 */  _LoadP_memory__rule,
  /*  405 */  _PartialSubtypeCheck_eSIRegP_eAXRegP_rule,
  /*  406 */  eRegL_rule,
  /*  407 */  eRegL_rule,
  /*  408 */  eRegL_rule,
  /*  409 */  eRegL_rule,
  /*  410 */  eRegL_rule,
  /*  411 */  eRegL_rule,
  /*  412 */  eRegL_rule,
  /*  413 */  eRegL_rule,
  /*  414 */  eRegL_rule,
  /*  415 */  eRegL_rule,
  /*  416 */  eRegL_rule,
  /*  417 */  eRegL_rule,
  /*  418 */  eRegP_rule,
  /*  419 */  eRegP_rule,
  /*  420 */  eRegP_rule,
  /*  421 */  regF_rule,
  /*  422 */  vlRegF_rule,
  /*  423 */  regD_rule,
  /*  424 */  vlRegD_rule,
  /*  425 */  regF_rule,
  /*  426 */  vlRegF_rule,
  /*  427 */  regD_rule,
  /*  428 */  vlRegD_rule,
  /*  429 */  vec_rule,
  /*  430 */  rRegI_rule,
  /*  431 */  _LoadVector_memory__rule,
  /*  432 */  vec_rule,
  /*  433 */  kReg_rule,
  /*  434 */  vec_rule,
  /*  435 */  vec_rule,
  /*  436 */  vec_rule,
  /*  437 */  legVec_rule,
  /*  438 */  rRegI_rule,
  /*  439 */  rRegI_rule,
  /*  440 */  _LoadB_memory__rule,
  /*  441 */  rRegI_rule,
  /*  442 */  rRegI_rule,
  /*  443 */  _LoadS_memory__rule,
  /*  444 */  rRegI_rule,
  /*  445 */  _LoadI_memory__rule,
  /*  446 */  immI_rule,
  /*  447 */  immI_rule,
  /*  448 */  immI_rule,
  /*  449 */  immI_0_rule,
  /*  450 */  immI_0_rule,
  /*  451 */  immI_0_rule,
  /*  452 */  immI_M1_rule,
  /*  453 */  immI_M1_rule,
  /*  454 */  immI_M1_rule,
  /*  455 */  _LoadL_memory__rule,
  /*  456 */  immL_rule,
  /*  457 */  immL0_rule,
  /*  458 */  immL_M1_rule,
  /*  459 */  vlRegF_rule,
  /*  460 */  vlRegF_rule,
  /*  461 */  _LoadF_memory__rule,
  /*  462 */  immF_rule,
  /*  463 */  immF0_rule,
  /*  464 */  vlRegD_rule,
  /*  465 */  vlRegD_rule,
  /*  466 */  _LoadD_memory__rule,
  /*  467 */  immD_rule,
  /*  468 */  immD0_rule,
  /*  469 */  vec_rule,
  /*  470 */  _LoadVector_memory__rule,
  /*  471 */  vec_rule,
  /*  472 */  _LoadVector_memory__rule,
  /*  473 */  rRegI_rule,
  /*  474 */  rRegI_rule,
  /*  475 */  vec_rule,
  /*  476 */  vec_rule,
  /*  477 */  vec_rule,
  /*  478 */  vec_rule,
  /*  479 */  vec_rule,
  /*  480 */  vec_rule,
  /*  481 */  vec_rule,
  /*  482 */  vec_rule,
  /*  483 */  vec_rule,
  /*  484 */  vec_rule,
  /*  485 */  vec_rule,
  /*  486 */  vec_rule,
  /*  487 */  vec_rule,
  /*  488 */  vec_rule,
  /*  489 */  vec_rule,
  /*  490 */  vec_rule,
  /*  491 */  vec_rule,
  /*  492 */  vec_rule,
  /*  493 */  vec_rule,
  /*  494 */  vec_rule,
  /*  495 */  vec_rule,
  /*  496 */  vec_rule,
  /*  497 */  kReg_rule,
  /*  498 */  vec_rule,
  /*  499 */  vec_rule,
  /*  500 */  immI_0_rule,
  /*  501 */  vec_rule,
  /*  502 */  vec_rule,
  /*  503 */  vec_rule,
  /*  504 */  vec_rule,
  /*  505 */  vec_rule,
  /*  506 */  vec_rule,
  /*  507 */  vec_rule,
  /*  508 */  kReg_rule,
  /*  509 */  vec_rule,
  /*  510 */  legVec_rule,
  /*  511 */  rRegI_rule,
  /*  512 */  eRegL_rule,
  /*  513 */  rRegI_rule,
  /*  514 */  rRegI_rule,
  /*  515 */  rRegI_rule,
  /*  516 */  rRegI_rule,
  /*  517 */  eRegL_rule,
  /*  518 */  eRegL_rule,
  /*  519 */  rRegI_rule,
  /*  520 */  rRegI_rule,
  /*  521 */  eRegL_rule,
  /*  522 */  eRegL_rule,
  /*  523 */  rRegI_rule,
  /*  524 */  _LoadI_memory__rule,
  /*  525 */  eRegL_rule,
  /*  526 */  _LoadL_memory__rule,
  /*  527 */  memory_rule,
  /*  528 */  _LoadB_memory__rule,
  /*  529 */  memory_rule,
  /*  530 */  _LoadUB_memory__rule,
  /*  531 */  _AndI__LoadUB_memory__immI_rule,
  /*  532 */  memory_rule,
  /*  533 */  _LShiftI__LoadS_memory__immI_24_rule,
  /*  534 */  _LoadS_memory__rule,
  /*  535 */  memory_rule,
  /*  536 */  _LShiftI__LoadUS_memory__immI_24_rule,
  /*  537 */  _LoadUS_memory__rule,
  /*  538 */  _AndI__LoadUS_memory__immI_255_rule,
  /*  539 */  _AndI__LoadUS_memory__immI_rule,
  /*  540 */  memory_rule,
  /*  541 */  _LShiftI__LoadI_memory__immI_24_rule,
  /*  542 */  _LoadI_memory__rule,
  /*  543 */  _LShiftI__LoadI_memory__immI_16_rule,
  /*  544 */  _LoadI_memory__rule,
  /*  545 */  _LoadI_memory__rule,
  /*  546 */  _AndI__LoadI_memory__immI_255_rule,
  /*  547 */  _AndI__LoadI_memory__immI_65535_rule,
  /*  548 */  _AndI__LoadI_memory__immU31_rule,
  /*  549 */  _ConvI2L__LoadI_memory___rule,
  /*  550 */  load_long_memory_rule,
  /*  551 */  memory_rule,
  /*  552 */  memory_rule,
  /*  553 */  memory_rule,
  /*  554 */  memory_rule,
  /*  555 */  memory_rule,
  /*  556 */  memory_rule,
  /*  557 */  memory_rule,
  /*  558 */  memory_rule,
  /*  559 */  memory_rule,
  /*  560 */  memory_rule,
  /*  561 */  memory_rule,
  /*  562 */  memory_rule,
  /*  563 */  memory_rule,
  /*  564 */  memory_rule,
  /*  565 */  memory_rule,
  /*  566 */  memory_rule,
  /*  567 */  memory_rule,
  /*  568 */  memory_rule,
  /*  569 */  memory_rule,
  /*  570 */  long_memory_rule,
  /*  571 */  memory_rule,
  /*  572 */  memory_rule,
  /*  573 */  memory_rule,
  /*  574 */  memory_rule,
  /*  575 */  memory_rule,
  /*  576 */  memory_rule,
  /*  577 */  memory_rule,
  /*  578 */  memory_rule,
  /*  579 */  memory_rule,
  /*  580 */  memory_rule,
  /*  581 */  memory_rule,
  /*  582 */  memory_rule,
  /*  583 */  memory_rule,
  /*  584 */  memory_rule,
  /*  585 */  memory_rule,
  /*  586 */  memory_rule,
  /*  587 */  memory_rule,
  /*  588 */  memory_rule,
  /*  589 */  memory_rule,
  /*  590 */  0,
  /*  591 */  0,
  /*  592 */  0,
  /*  593 */  0,
  /*  594 */  0,
  /*  595 */  0,
  /*  596 */  0,
  /*  597 */  0,
  /*  598 */  0,
  /*  599 */  _Binary_cmpOp_eFlagsReg_rule,
  /*  600 */  _Binary_cmpOpU_eFlagsRegU_rule,
  /*  601 */  _Binary_cmpOp_eFlagsReg_rule,
  /*  602 */  _Binary_cmpOpU_eFlagsRegU_rule,
  /*  603 */  _Binary_cmpOpUCF_eFlagsRegUCF_rule,
  /*  604 */  _Binary_cmpOp_eFlagsReg_rule,
  /*  605 */  _Binary_cmpOpU_eFlagsRegU_rule,
  /*  606 */  _Binary_cmpOpUCF_eFlagsRegUCF_rule,
  /*  607 */  _Binary_cmpOp_eFlagsReg_rule,
  /*  608 */  _Binary_cmpOp_eFlagsReg_rule,
  /*  609 */  _Binary_cmpOpU_eFlagsRegU_rule,
  /*  610 */  _Binary_cmpOpUCF_eFlagsRegUCF_rule,
  /*  611 */  _Binary_cmpOp_fcmov_eFlagsRegU_rule,
  /*  612 */  _Binary_cmpOp_fcmov_eFlagsRegU_rule,
  /*  613 */  _Binary_cmpOp_eFlagsReg_rule,
  /*  614 */  _Binary_cmpOp_eFlagsReg_rule,
  /*  615 */  _Binary_cmpOp_eFlagsReg_rule,
  /*  616 */  _Binary_cmpOp_eFlagsReg_rule,
  /*  617 */  _Binary_cmpOpU_eFlagsRegU_rule,
  /*  618 */  _Binary_cmpOpUCF_eFlagsRegUCF_rule,
  /*  619 */  _Binary_cmpOpU_eFlagsRegU_rule,
  /*  620 */  _Binary_cmpOpUCF_eFlagsRegUCF_rule,
  /*  621 */  _Binary_cmpOp_eFlagsReg_rule,
  /*  622 */  _Binary_cmpOpU_eFlagsRegU_rule,
  /*  623 */  _Binary_cmpOpUCF_eFlagsRegUCF_rule,
  /*  624 */  rRegI_rule,
  /*  625 */  rRegI_rule,
  /*  626 */  rRegI_rule,
  /*  627 */  rRegI_rule,
  /*  628 */  eRegP_rule,
  /*  629 */  rRegI_rule,
  /*  630 */  eRegP_rule,
  /*  631 */  eRegP_rule,
  /*  632 */  rRegI_rule,
  /*  633 */  _LoadI_memory__rule,
  /*  634 */  memory_rule,
  /*  635 */  memory_rule,
  /*  636 */  memory_rule,
  /*  637 */  memory_rule,
  /*  638 */  memory_rule,
  /*  639 */  eSIRegP_rule,
  /*  640 */  eSIRegP_rule,
  /*  641 */  pRegP_rule,
  /*  642 */  pRegP_rule,
  /*  643 */  pRegP_rule,
  /*  644 */  pRegP_rule,
  /*  645 */  pRegP_rule,
  /*  646 */  pRegP_rule,
  /*  647 */  pRegP_rule,
  /*  648 */  pRegP_rule,
  /*  649 */  eSIRegP_rule,
  /*  650 */  pRegP_rule,
  /*  651 */  pRegP_rule,
  /*  652 */  pRegP_rule,
  /*  653 */  pRegP_rule,
  /*  654 */  memory_rule,
  /*  655 */  memory_rule,
  /*  656 */  memory_rule,
  /*  657 */  memory_rule,
  /*  658 */  memory_rule,
  /*  659 */  memory_rule,
  /*  660 */  memory_rule,
  /*  661 */  memory_rule,
  /*  662 */  memory_rule,
  /*  663 */  memory_rule,
  /*  664 */  rRegI_rule,
  /*  665 */  rRegI_rule,
  /*  666 */  rRegI_rule,
  /*  667 */  memory_rule,
  /*  668 */  eRegP_rule,
  /*  669 */  immI_0_rule,
  /*  670 */  rRegI_rule,
  /*  671 */  rRegI_rule,
  /*  672 */  _RShiftL__MulL__ConvI2L_nadxRegI__eADXRegL_low_only_immI_32_rule,
  /*  673 */  _RShiftL__MulL_eADXRegL_low_only__ConvI2L_nadxRegI__immI_32_rule,
  /*  674 */  _RShiftL__MulL__ConvI2L_nadxRegI__eADXRegL_low_only_immI_32_63_rule,
  /*  675 */  _RShiftL__MulL_eADXRegL_low_only__ConvI2L_nadxRegI__immI_32_63_rule,
  /*  676 */  _LoadI_memory__rule,
  /*  677 */  rRegI_rule,
  /*  678 */  _LoadI_memory__rule,
  /*  679 */  _Binary_rRegI_rRegI_rule,
  /*  680 */  _ConvI2L_eAXRegI__rule,
  /*  681 */  _ConvI2L_nadxRegI__rule,
  /*  682 */  _AndL__ConvI2L_eAXRegI__immL_32bits_rule,
  /*  683 */  _AndL__ConvI2L_nadxRegI__immL_32bits_rule,
  /*  684 */  eADXRegL_rule,
  /*  685 */  eADXRegL_rule,
  /*  686 */  eADXRegL_rule,
  /*  687 */  eADXRegL_rule,
  /*  688 */  eADXRegL_rule,
  /*  689 */  eAXRegI_rule,
  /*  690 */  eRegL_rule,
  /*  691 */  eAXRegI_rule,
  /*  692 */  eAXRegI_rule,
  /*  693 */  eRegL_rule,
  /*  694 */  eADXRegL_rule,
  /*  695 */  eADXRegL_rule,
  /*  696 */  rRegI_rule,
  /*  697 */  rRegI_rule,
  /*  698 */  rRegI_rule,
  /*  699 */  rRegI_rule,
  /*  700 */  memory_rule,
  /*  701 */  rRegI_rule,
  /*  702 */  memory_rule,
  /*  703 */  rRegI_rule,
  /*  704 */  rRegI_rule,
  /*  705 */  rRegI_rule,
  /*  706 */  _LShiftI_xRegI_immI_24_rule,
  /*  707 */  _LShiftI_xRegI_immI_16_rule,
  /*  708 */  rRegI_rule,
  /*  709 */  rRegI_rule,
  /*  710 */  rRegI_rule,
  /*  711 */  rRegI_rule,
  /*  712 */  _LoadI_memory__rule,
  /*  713 */  memory_rule,
  /*  714 */  memory_rule,
  /*  715 */  memory_rule,
  /*  716 */  _XorI_rRegI_immI_M1_rule,
  /*  717 */  rRegI_rule,
  /*  718 */  _XorI_rRegI_immI_M1_rule,
  /*  719 */  _LoadI_memory__rule,
  /*  720 */  _SubI_immI_0_rRegI_rule,
  /*  721 */  rRegI_rule,
  /*  722 */  _SubI_immI_0__LoadI_memory__rule,
  /*  723 */  _LoadI_memory__rule,
  /*  724 */  _AddI_rRegI_immI_M1_rule,
  /*  725 */  rRegI_rule,
  /*  726 */  _AddI__LoadI_memory__immI_M1_rule,
  /*  727 */  _LoadI_memory__rule,
  /*  728 */  _AddI_rRegI_immI_M1_rule,
  /*  729 */  rRegI_rule,
  /*  730 */  _AddI__LoadI_memory__immI_M1_rule,
  /*  731 */  _LoadI_memory__rule,
  /*  732 */  rRegI_rule,
  /*  733 */  rRegI_rule,
  /*  734 */  _CastP2X_eRegP__rule,
  /*  735 */  rRegI_rule,
  /*  736 */  rRegI_rule,
  /*  737 */  _LoadI_memory__rule,
  /*  738 */  memory_rule,
  /*  739 */  memory_rule,
  /*  740 */  memory_rule,
  /*  741 */  0,
  /*  742 */  0,
  /*  743 */  0,
  /*  744 */  _LShiftI_rRegI_immI_1_rule,
  /*  745 */  _URShiftI_rRegI_immI_M1_rule,
  /*  746 */  _LShiftI_rRegI_immI8_rule,
  /*  747 */  _URShiftI_rRegI_immI8_rule,
  /*  748 */  _LShiftI_ncxRegI_eCXRegI_rule,
  /*  749 */  _URShiftI_ncxRegI__SubI_immI_0_eCXRegI_rule,
  /*  750 */  _LShiftI_ncxRegI_eCXRegI_rule,
  /*  751 */  _URShiftI_ncxRegI__SubI_immI_32_eCXRegI_rule,
  /*  752 */  0,
  /*  753 */  0,
  /*  754 */  0,
  /*  755 */  _URShiftI_rRegI_immI_1_rule,
  /*  756 */  _LShiftI_rRegI_immI_M1_rule,
  /*  757 */  _URShiftI_rRegI_immI8_rule,
  /*  758 */  _LShiftI_rRegI_immI8_rule,
  /*  759 */  _URShiftI_ncxRegI_eCXRegI_rule,
  /*  760 */  _LShiftI_ncxRegI__SubI_immI_0_eCXRegI_rule,
  /*  761 */  _URShiftI_ncxRegI_eCXRegI_rule,
  /*  762 */  _LShiftI_ncxRegI__SubI_immI_32_eCXRegI_rule,
  /*  763 */  rRegI_rule,
  /*  764 */  rRegI_rule,
  /*  765 */  rRegI_rule,
  /*  766 */  rRegI_rule,
  /*  767 */  _LoadI_memory__rule,
  /*  768 */  memory_rule,
  /*  769 */  memory_rule,
  /*  770 */  memory_rule,
  /*  771 */  0,
  /*  772 */  rRegI_rule,
  /*  773 */  0,
  /*  774 */  eRegP_rule,
  /*  775 */  ncxRegI_rule,
  /*  776 */  rRegI_rule,
  /*  777 */  _AndI__CmpLTMask_rRegI_rRegI_rRegI_rule,
  /*  778 */  _SubI_rRegI_rRegI_rule,
  /*  779 */  _AndI_rRegI__CmpLTMask_rRegI_rRegI_rule,
  /*  780 */  _SubI_rRegI_rRegI_rule,
  /*  781 */  _CmpLTMask_rRegI_rRegI_rule,
  /*  782 */  rRegI_rule,
  /*  783 */  rRegI_rule,
  /*  784 */  eRegL_rule,
  /*  785 */  eRegL_rule,
  /*  786 */  eRegL_rule,
  /*  787 */  _LoadL_load_long_memory__rule,
  /*  788 */  eRegL_rule,
  /*  789 */  eRegL_rule,
  /*  790 */  eRegL_rule,
  /*  791 */  immL0_rule,
  /*  792 */  eRegL_rule,
  /*  793 */  eRegL_rule,
  /*  794 */  eRegL_rule,
  /*  795 */  _LoadL_load_long_memory__rule,
  /*  796 */  _XorL_eRegL_immL_M1_rule,
  /*  797 */  eRegL_rule,
  /*  798 */  _XorL_eRegL_immL_M1_rule,
  /*  799 */  _LoadL_memory__rule,
  /*  800 */  _SubL_immL0_eRegL_rule,
  /*  801 */  eRegL_rule,
  /*  802 */  _SubL_immL0__LoadL_memory__rule,
  /*  803 */  _LoadL_memory__rule,
  /*  804 */  _AddL_eRegL_immL_M1_rule,
  /*  805 */  eRegL_rule,
  /*  806 */  _AddL__LoadL_memory__immL_M1_rule,
  /*  807 */  _LoadL_memory__rule,
  /*  808 */  _AddL_eRegL_immL_M1_rule,
  /*  809 */  eRegL_rule,
  /*  810 */  _AddL__LoadL_memory__immL_M1_rule,
  /*  811 */  _LoadL_memory__rule,
  /*  812 */  eRegL_rule,
  /*  813 */  eRegL_rule,
  /*  814 */  eRegL_rule,
  /*  815 */  _LoadL_load_long_memory__rule,
  /*  816 */  eRegL_rule,
  /*  817 */  eRegL_rule,
  /*  818 */  eRegL_rule,
  /*  819 */  eRegL_rule,
  /*  820 */  _LoadL_load_long_memory__rule,
  /*  821 */  eRegL_rule,
  /*  822 */  eRegL_rule,
  /*  823 */  eRegL_rule,
  /*  824 */  eRegL_rule,
  /*  825 */  eRegL_rule,
  /*  826 */  eRegL_rule,
  /*  827 */  eRegL_rule,
  /*  828 */  eRegL_rule,
  /*  829 */  eRegL_rule,
  /*  830 */  eRegL_rule,
  /*  831 */  eRegL_rule,
  /*  832 */  eRegL_rule,
  /*  833 */  regDPR_rule,
  /*  834 */  regDPR_rule,
  /*  835 */  regD_rule,
  /*  836 */  regD_rule,
  /*  837 */  regDPR_rule,
  /*  838 */  _SubD_regDPR_regDPR_rule,
  /*  839 */  regDPR_rule,
  /*  840 */  regDPR_rule,
  /*  841 */  _AddD_regDPR_regDPR_rule,
  /*  842 */  regDPR_rule,
  /*  843 */  _LoadD_memory__rule,
  /*  844 */  memory_rule,
  /*  845 */  memory_rule,
  /*  846 */  regDPR_rule,
  /*  847 */  regDPR_rule,
  /*  848 */  _AddD_regDPR_immDPR_rule,
  /*  849 */  regDPR_rule,
  /*  850 */  regDPR1_rule,
  /*  851 */  regDPR_rule,
  /*  852 */  regDPR_rule,
  /*  853 */  _LoadD_memory__rule,
  /*  854 */  regDPR_rule,
  /*  855 */  _LoadD_memory__rule,
  /*  856 */  _MulD_regDPR_regDPR_rule,
  /*  857 */  regDPR_rule,
  /*  858 */  _MulD_regDPR_regDPR_rule,
  /*  859 */  regDPR_rule,
  /*  860 */  regDPR1_rule,
  /*  861 */  regDPR_rule,
  /*  862 */  regD_rule,
  /*  863 */  regDPR_rule,
  /*  864 */  regD_rule,
  /*  865 */  regDPR_rule,
  /*  866 */  regFPR_rule,
  /*  867 */  regFPR_rule,
  /*  868 */  regF_rule,
  /*  869 */  regF_rule,
  /*  870 */  regFPR_rule,
  /*  871 */  regFPR_rule,
  /*  872 */  regFPR_rule,
  /*  873 */  regFPR_rule,
  /*  874 */  regFPR_rule,
  /*  875 */  _LoadF_memory__rule,
  /*  876 */  regFPR_rule,
  /*  877 */  _LoadF_memory__rule,
  /*  878 */  memory_rule,
  /*  879 */  memory_rule,
  /*  880 */  _LoadF_memory__rule,
  /*  881 */  memory_rule,
  /*  882 */  regFPR_rule,
  /*  883 */  regFPR_rule,
  /*  884 */  regFPR_rule,
  /*  885 */  regFPR_rule,
  /*  886 */  regFPR_rule,
  /*  887 */  _LoadF_memory__rule,
  /*  888 */  regFPR_rule,
  /*  889 */  _LoadF_memory__rule,
  /*  890 */  memory_rule,
  /*  891 */  regFPR_rule,
  /*  892 */  regFPR_rule,
  /*  893 */  _LoadF_memory__rule,
  /*  894 */  regFPR_rule,
  /*  895 */  _MulF__LoadF_memory__regFPR_rule,
  /*  896 */  regFPR_rule,
  /*  897 */  _MulF_regFPR__LoadF_memory__rule,
  /*  898 */  regFPR_rule,
  /*  899 */  _MulF_regFPR_regFPR_rule,
  /*  900 */  regFPR_rule,
  /*  901 */  _SubF_regFPR_regFPR_rule,
  /*  902 */  regFPR_rule,
  /*  903 */  regFPR_rule,
  /*  904 */  regFPR_rule,
  /*  905 */  regFPR_rule,
  /*  906 */  regF_rule,
  /*  907 */  regFPR_rule,
  /*  908 */  regDPR_rule,
  /*  909 */  regDPR_rule,
  /*  910 */  regDPR_rule,
  /*  911 */  regD_rule,
  /*  912 */  regFPR_rule,
  /*  913 */  regFPR_rule,
  /*  914 */  regF_rule,
  /*  915 */  regF_rule,
  /*  916 */  regDPR_rule,
  /*  917 */  regD_rule,
  /*  918 */  regDPR_rule,
  /*  919 */  regD_rule,
  /*  920 */  regFPR_rule,
  /*  921 */  regF_rule,
  /*  922 */  regFPR_rule,
  /*  923 */  regF_rule,
  /*  924 */  stackSlotI_rule,
  /*  925 */  rRegI_rule,
  /*  926 */  _LoadI_memory__rule,
  /*  927 */  rRegI_rule,
  /*  928 */  _LoadI_memory__rule,
  /*  929 */  stackSlotI_rule,
  /*  930 */  stackSlotI_rule,
  /*  931 */  _LoadI_memory__rule,
  /*  932 */  stackSlotI_rule,
  /*  933 */  _LoadI_memory__rule,
  /*  934 */  rRegI_rule,
  /*  935 */  rRegI_rule,
  /*  936 */  rRegI_rule,
  /*  937 */  _ConvI2L_rRegI__rule,
  /*  938 */  eRegL_rule,
  /*  939 */  eRegL_rule,
  /*  940 */  eRegL_rule,
  /*  941 */  eRegL_rule,
  /*  942 */  eRegL_rule,
  /*  943 */  eRegL_rule,
  /*  944 */  regFPR_rule,
  /*  945 */  regF_rule,
  /*  946 */  rRegI_rule,
  /*  947 */  regDPR_rule,
  /*  948 */  regD_rule,
  /*  949 */  regD_rule,
  /*  950 */  eRegL_rule,
  /*  951 */  eRegL_rule,
  /*  952 */  eBCXRegL_rule,
  /*  953 */  eBCXRegL_rule,
  /*  954 */  eCXRegI_rule,
  /*  955 */  eCXRegI_rule,
  /*  956 */  eCXRegI_rule,
  /*  957 */  eCXRegI_rule,
  /*  958 */  immI_rule,
  /*  959 */  _Binary_eDIRegP_eCXRegI_rule,
  /*  960 */  _Binary_eDIRegP_eCXRegI_rule,
  /*  961 */  _Binary_eDIRegP_eCXRegI_rule,
  /*  962 */  _Binary_eDIRegP_eCXRegI_rule,
  /*  963 */  _Binary_eDIRegP_eCXRegI_rule,
  /*  964 */  _Binary_eDIRegP_eCXRegI_rule,
  /*  965 */  _Binary_eSIRegP_eDXRegI_rule,
  /*  966 */  _Binary_eSIRegP_eDXRegI_rule,
  /*  967 */  _Binary_eDIRegP_eSIRegP_rule,
  /*  968 */  _Binary_eDIRegP_eSIRegP_rule,
  /*  969 */  _Binary_eDIRegP_eDXRegI_rule,
  /*  970 */  _Binary_eDIRegP_eDXRegI_rule,
  /*  971 */  _Binary_eDIRegP_eDXRegI_rule,
  /*  972 */  _Binary_eDIRegP_eDXRegI_rule,
  /*  973 */  _Binary_eDIRegP_eDXRegI_rule,
  /*  974 */  _Binary_eDIRegP_eDXRegI_rule,
  /*  975 */  _Binary_eDIRegP_eDXRegI_rule,
  /*  976 */  _Binary_eDIRegP_eDXRegI_rule,
  /*  977 */  eDIRegP_rule,
  /*  978 */  eDIRegP_rule,
  /*  979 */  eDIRegP_rule,
  /*  980 */  eDIRegP_rule,
  /*  981 */  eSIRegP_rule,
  /*  982 */  eSIRegP_rule,
  /*  983 */  eSIRegP_rule,
  /*  984 */  eSIRegP_rule,
  /*  985 */  eSIRegP_rule,
  /*  986 */  eSIRegP_rule,
  /*  987 */  eSIRegP_rule,
  /*  988 */  eSIRegP_rule,
  /*  989 */  rRegI_rule,
  /*  990 */  rRegI_rule,
  /*  991 */  _Binary_nadxRegI_eAXRegI_rule,
  /*  992 */  rRegI_rule,
  /*  993 */  0,
  /*  994 */  cmpOp_rule,
  /*  995 */  cmpOp_rule,
  /*  996 */  cmpOpU_rule,
  /*  997 */  cmpOpUCF_rule,
  /*  998 */  cmpOpUCF2_rule,
  /*  999 */  eSIRegP_rule,
  /* 1000 */  0,
  /* 1001 */  cmpOp_rule,
  /* 1002 */  cmpOp_rule,
  /* 1003 */  cmpOpU_rule,
  /* 1004 */  cmpOpUCF_rule,
  /* 1005 */  cmpOpUCF2_rule,
  /* 1006 */  eRegL_rule,
  /* 1007 */  cmpOp_rule,
  /* 1008 */  cmpOpU_rule,
  /* 1009 */  _Binary_cmpOp_flagsReg_long_LTGE_rule,
  /* 1010 */  _Binary_cmpOp_flagsReg_long_LTGE_rule,
  /* 1011 */  _Binary_cmpOpU_flagsReg_ulong_LTGE_rule,
  /* 1012 */  _Binary_cmpOpU_flagsReg_ulong_LTGE_rule,
  /* 1013 */  _Binary_cmpOp_flagsReg_long_LTGE_rule,
  /* 1014 */  _Binary_cmpOp_flagsReg_long_LTGE_rule,
  /* 1015 */  _Binary_cmpOpU_flagsReg_ulong_LTGE_rule,
  /* 1016 */  _Binary_cmpOpU_flagsReg_ulong_LTGE_rule,
  /* 1017 */  _Binary_cmpOp_flagsReg_long_LTGE_rule,
  /* 1018 */  _Binary_cmpOpU_flagsReg_ulong_LTGE_rule,
  /* 1019 */  _Binary_cmpOp_flagsReg_long_LTGE_rule,
  /* 1020 */  _Binary_cmpOp_flagsReg_long_LTGE_rule,
  /* 1021 */  _Binary_cmpOp_flagsReg_long_LTGE_rule,
  /* 1022 */  _Binary_cmpOp_flagsReg_long_LTGE_rule,
  /* 1023 */  cmpOp_rule,
  /* 1024 */  cmpOpU_rule,
  /* 1025 */  _Binary_cmpOp_flagsReg_long_EQNE_rule,
  /* 1026 */  _Binary_cmpOp_flagsReg_long_EQNE_rule,
  /* 1027 */  _Binary_cmpOp_flagsReg_long_EQNE_rule,
  /* 1028 */  _Binary_cmpOp_flagsReg_long_EQNE_rule,
  /* 1029 */  _Binary_cmpOpU_flagsReg_ulong_EQNE_rule,
  /* 1030 */  _Binary_cmpOpU_flagsReg_ulong_EQNE_rule,
  /* 1031 */  _Binary_cmpOp_flagsReg_long_EQNE_rule,
  /* 1032 */  _Binary_cmpOpU_flagsReg_ulong_EQNE_rule,
  /* 1033 */  _Binary_cmpOp_flagsReg_long_EQNE_rule,
  /* 1034 */  _Binary_cmpOp_flagsReg_long_EQNE_rule,
  /* 1035 */  _Binary_cmpOp_flagsReg_long_EQNE_rule,
  /* 1036 */  _Binary_cmpOp_flagsReg_long_EQNE_rule,
  /* 1037 */  cmpOp_commute_rule,
  /* 1038 */  cmpOpU_commute_rule,
  /* 1039 */  _Binary_cmpOp_commute_flagsReg_long_LEGT_rule,
  /* 1040 */  _Binary_cmpOp_commute_flagsReg_long_LEGT_rule,
  /* 1041 */  _Binary_cmpOpU_commute_flagsReg_ulong_LEGT_rule,
  /* 1042 */  _Binary_cmpOpU_commute_flagsReg_ulong_LEGT_rule,
  /* 1043 */  _Binary_cmpOp_commute_flagsReg_long_LEGT_rule,
  /* 1044 */  _Binary_cmpOp_commute_flagsReg_long_LEGT_rule,
  /* 1045 */  _Binary_cmpOpU_commute_flagsReg_ulong_LEGT_rule,
  /* 1046 */  _Binary_cmpOpU_commute_flagsReg_ulong_LEGT_rule,
  /* 1047 */  _Binary_cmpOp_commute_flagsReg_long_LEGT_rule,
  /* 1048 */  _Binary_cmpOpU_commute_flagsReg_ulong_LEGT_rule,
  /* 1049 */  _Binary_cmpOp_commute_flagsReg_long_LEGT_rule,
  /* 1050 */  _Binary_cmpOp_commute_flagsReg_long_LEGT_rule,
  /* 1051 */  _Binary_cmpOp_commute_flagsReg_long_LEGT_rule,
  /* 1052 */  _Binary_cmpOp_commute_flagsReg_long_LEGT_rule,
  /* 1053 */  0,
  /* 1054 */  0,
  /* 1055 */  0,
  /* 1056 */  0,
  /* 1057 */  0,
  /* 1058 */  0,
  /* 1059 */  eRegP_no_EBP_rule,
  /* 1060 */  eRegP_no_EBP_rule,
  /* 1061 */  0,
  /* 1062 */  0,
  /* 1063 */  eRegL_rule,
  /* 1064 */  eRegL_rule,
  /* 1065 */  rRegI_rule,
  /* 1066 */  eRegP_no_EBP_rule,
  /* 1067 */  0,
  /* 1068 */  0,
  /* 1069 */  regF_rule,
  /* 1070 */  regF_rule,
  /* 1071 */  _LoadF_memory__rule,
  /* 1072 */  regF_rule,
  /* 1073 */  regF_rule,
  /* 1074 */  regF_rule,
  /* 1075 */  _LoadF_memory__rule,
  /* 1076 */  regF_rule,
  /* 1077 */  regD_rule,
  /* 1078 */  regD_rule,
  /* 1079 */  _LoadD_memory__rule,
  /* 1080 */  regD_rule,
  /* 1081 */  regD_rule,
  /* 1082 */  regD_rule,
  /* 1083 */  _LoadD_memory__rule,
  /* 1084 */  regD_rule,
  /* 1085 */  regF_rule,
  /* 1086 */  regF_rule,
  /* 1087 */  regF_rule,
  /* 1088 */  regF_rule,
  /* 1089 */  regF_rule,
  /* 1090 */  regF_rule,
  /* 1091 */  regD_rule,
  /* 1092 */  regD_rule,
  /* 1093 */  regD_rule,
  /* 1094 */  regD_rule,
  /* 1095 */  regD_rule,
  /* 1096 */  regD_rule,
  /* 1097 */  regF_rule,
  /* 1098 */  regF_rule,
  /* 1099 */  _LoadF_memory__rule,
  /* 1100 */  regF_rule,
  /* 1101 */  regF_rule,
  /* 1102 */  regF_rule,
  /* 1103 */  _LoadF_memory__rule,
  /* 1104 */  regF_rule,
  /* 1105 */  regD_rule,
  /* 1106 */  regD_rule,
  /* 1107 */  _LoadD_memory__rule,
  /* 1108 */  regD_rule,
  /* 1109 */  regD_rule,
  /* 1110 */  regD_rule,
  /* 1111 */  _LoadD_memory__rule,
  /* 1112 */  regD_rule,
  /* 1113 */  regF_rule,
  /* 1114 */  regF_rule,
  /* 1115 */  regF_rule,
  /* 1116 */  regF_rule,
  /* 1117 */  regF_rule,
  /* 1118 */  regF_rule,
  /* 1119 */  regD_rule,
  /* 1120 */  regD_rule,
  /* 1121 */  regD_rule,
  /* 1122 */  regD_rule,
  /* 1123 */  regD_rule,
  /* 1124 */  regD_rule,
  /* 1125 */  regF_rule,
  /* 1126 */  regD_rule,
  /* 1127 */  regF_rule,
  /* 1128 */  memory_rule,
  /* 1129 */  memory_rule,
  /* 1130 */  kReg_rule,
  /* 1131 */  kReg_rule,
  /* 1132 */  kReg_rule,
  /* 1133 */  vec_rule,
  /* 1134 */  0,
  /* 1135 */  regD_rule,
  /* 1136 */  regF_rule,
  /* 1137 */  memory_rule,
  /* 1138 */  memory_rule,
  /* 1139 */  memory_rule,
  /* 1140 */  memory_rule,
  /* 1141 */  memory_rule,
  /* 1142 */  memory_rule,
  /* 1143 */  memory_rule,
  /* 1144 */  eRegL_rule,
  /* 1145 */  eRegL_rule,
  /* 1146 */  _Binary_vec_rRegI_rule,
  /* 1147 */  _Binary_vec_rRegI_rule,
  /* 1148 */  _Binary_vec_rRegI_rule,
  /* 1149 */  _Binary_vec_regF_rule,
  /* 1150 */  _Binary_vec_regF_rule,
  /* 1151 */  rRegI_rule,
  /* 1152 */  rRegI_rule,
  /* 1153 */  rRegI_rule,
  /* 1154 */  rRegI_rule,
  /* 1155 */  rRegI_rule,
  /* 1156 */  rRegI_rule,
  /* 1157 */  rRegI_rule,
  /* 1158 */  regF_rule,
  /* 1159 */  regF_rule,
  /* 1160 */  regF_rule,
  /* 1161 */  regF_rule,
  /* 1162 */  regF_rule,
  /* 1163 */  regF_rule,
  /* 1164 */  regD_rule,
  /* 1165 */  regD_rule,
  /* 1166 */  regD_rule,
  /* 1167 */  regD_rule,
  /* 1168 */  regD_rule,
  /* 1169 */  regD_rule,
  /* 1170 */  rRegI_rule,
  /* 1171 */  rRegI_rule,
  /* 1172 */  rRegI_rule,
  /* 1173 */  rRegI_rule,
  /* 1174 */  rRegI_rule,
  /* 1175 */  rRegI_rule,
  /* 1176 */  rRegI_rule,
  /* 1177 */  rRegI_rule,
  /* 1178 */  rRegI_rule,
  /* 1179 */  immF_rule,
  /* 1180 */  immF_rule,
  /* 1181 */  immF_rule,
  /* 1182 */  immF_rule,
  /* 1183 */  legRegF_rule,
  /* 1184 */  legRegF_rule,
  /* 1185 */  legRegF_rule,
  /* 1186 */  legRegF_rule,
  /* 1187 */  immD_rule,
  /* 1188 */  immD_rule,
  /* 1189 */  immD_rule,
  /* 1190 */  immD_rule,
  /* 1191 */  legRegD_rule,
  /* 1192 */  legRegD_rule,
  /* 1193 */  legRegD_rule,
  /* 1194 */  legRegD_rule,
  /* 1195 */  vec_rule,
  /* 1196 */  vec_rule,
  /* 1197 */  vec_rule,
  /* 1198 */  _LoadVector_memory__rule,
  /* 1199 */  vec_rule,
  /* 1200 */  vec_rule,
  /* 1201 */  vec_rule,
  /* 1202 */  _LoadVector_memory__rule,
  /* 1203 */  vec_rule,
  /* 1204 */  vec_rule,
  /* 1205 */  vec_rule,
  /* 1206 */  _LoadVector_memory__rule,
  /* 1207 */  vec_rule,
  /* 1208 */  vec_rule,
  /* 1209 */  vec_rule,
  /* 1210 */  _LoadVector_memory__rule,
  /* 1211 */  vec_rule,
  /* 1212 */  vec_rule,
  /* 1213 */  vec_rule,
  /* 1214 */  _LoadVector_memory__rule,
  /* 1215 */  vec_rule,
  /* 1216 */  vec_rule,
  /* 1217 */  vec_rule,
  /* 1218 */  _LoadVector_memory__rule,
  /* 1219 */  vec_rule,
  /* 1220 */  vec_rule,
  /* 1221 */  vec_rule,
  /* 1222 */  vec_rule,
  /* 1223 */  vec_rule,
  /* 1224 */  vec_rule,
  /* 1225 */  vec_rule,
  /* 1226 */  vec_rule,
  /* 1227 */  vec_rule,
  /* 1228 */  vec_rule,
  /* 1229 */  vec_rule,
  /* 1230 */  vec_rule,
  /* 1231 */  vec_rule,
  /* 1232 */  vec_rule,
  /* 1233 */  vec_rule,
  /* 1234 */  vec_rule,
  /* 1235 */  vec_rule,
  /* 1236 */  vec_rule,
  /* 1237 */  vec_rule,
  /* 1238 */  vec_rule,
  /* 1239 */  vec_rule,
  /* 1240 */  vec_rule,
  /* 1241 */  vec_rule,
  /* 1242 */  vec_rule,
  /* 1243 */  _LoadVector_memory__rule,
  /* 1244 */  vec_rule,
  /* 1245 */  vec_rule,
  /* 1246 */  vec_rule,
  /* 1247 */  _LoadVector_memory__rule,
  /* 1248 */  vec_rule,
  /* 1249 */  vec_rule,
  /* 1250 */  _LoadVector_memory__rule,
  /* 1251 */  vec_rule,
  /* 1252 */  vec_rule,
  /* 1253 */  vec_rule,
  /* 1254 */  vec_rule,
  /* 1255 */  vec_rule,
  /* 1256 */  _LoadVector_memory__rule,
  /* 1257 */  vec_rule,
  /* 1258 */  vec_rule,
  /* 1259 */  vec_rule,
  /* 1260 */  _LoadVector_memory__rule,
  /* 1261 */  vec_rule,
  /* 1262 */  vec_rule,
  /* 1263 */  vec_rule,
  /* 1264 */  vec_rule,
  /* 1265 */  vec_rule,
  /* 1266 */  vec_rule,
  /* 1267 */  vec_rule,
  /* 1268 */  vec_rule,
  /* 1269 */  vec_rule,
  /* 1270 */  vec_rule,
  /* 1271 */  vec_rule,
  /* 1272 */  vec_rule,
  /* 1273 */  legVec_rule,
  /* 1274 */  legVec_rule,
  /* 1275 */  vec_rule,
  /* 1276 */  vec_rule,
  /* 1277 */  legVec_rule,
  /* 1278 */  legVec_rule,
  /* 1279 */  vec_rule,
  /* 1280 */  vec_rule,
  /* 1281 */  regF_rule,
  /* 1282 */  regD_rule,
  /* 1283 */  vec_rule,
  /* 1284 */  vec_rule,
  /* 1285 */  vec_rule,
  /* 1286 */  vec_rule,
  /* 1287 */  rRegI_rule,
  /* 1288 */  rRegI_rule,
  /* 1289 */  rRegI_rule,
  /* 1290 */  rRegI_rule,
  /* 1291 */  vec_rule,
  /* 1292 */  vec_rule,
  /* 1293 */  vec_rule,
  /* 1294 */  vec_rule,
  /* 1295 */  vec_rule,
  /* 1296 */  vec_rule,
  /* 1297 */  vec_rule,
  /* 1298 */  vec_rule,
  /* 1299 */  vec_rule,
  /* 1300 */  vec_rule,
  /* 1301 */  vec_rule,
  /* 1302 */  vec_rule,
  /* 1303 */  vec_rule,
  /* 1304 */  vec_rule,
  /* 1305 */  vec_rule,
  /* 1306 */  vec_rule,
  /* 1307 */  vec_rule,
  /* 1308 */  vec_rule,
  /* 1309 */  vec_rule,
  /* 1310 */  vec_rule,
  /* 1311 */  vec_rule,
  /* 1312 */  vec_rule,
  /* 1313 */  vec_rule,
  /* 1314 */  vec_rule,
  /* 1315 */  vec_rule,
  /* 1316 */  vec_rule,
  /* 1317 */  vec_rule,
  /* 1318 */  vec_rule,
  /* 1319 */  vec_rule,
  /* 1320 */  vec_rule,
  /* 1321 */  vec_rule,
  /* 1322 */  vec_rule,
  /* 1323 */  vec_rule,
  /* 1324 */  vec_rule,
  /* 1325 */  vec_rule,
  /* 1326 */  vec_rule,
  /* 1327 */  vec_rule,
  /* 1328 */  vec_rule,
  /* 1329 */  vec_rule,
  /* 1330 */  vec_rule,
  /* 1331 */  vec_rule,
  /* 1332 */  vec_rule,
  /* 1333 */  vec_rule,
  /* 1334 */  vec_rule,
  /* 1335 */  vec_rule,
  /* 1336 */  vec_rule,
  /* 1337 */  vec_rule,
  /* 1338 */  vec_rule,
  /* 1339 */  vec_rule,
  /* 1340 */  vec_rule,
  /* 1341 */  vec_rule,
  /* 1342 */  vec_rule,
  /* 1343 */  vec_rule,
  /* 1344 */  vec_rule,
  /* 1345 */  vec_rule,
  /* 1346 */  vec_rule,
  /* 1347 */  vec_rule,
  /* 1348 */  vec_rule,
  /* 1349 */  vec_rule,
  /* 1350 */  vec_rule,
  /* 1351 */  vec_rule,
  /* 1352 */  vec_rule,
  /* 1353 */  vec_rule,
  /* 1354 */  vec_rule,
  /* 1355 */  _LoadVector_memory__rule,
  /* 1356 */  vec_rule,
  /* 1357 */  vec_rule,
  /* 1358 */  vec_rule,
  /* 1359 */  _LoadVector_memory__rule,
  /* 1360 */  vec_rule,
  /* 1361 */  vec_rule,
  /* 1362 */  vec_rule,
  /* 1363 */  _LoadVector_memory__rule,
  /* 1364 */  vec_rule,
  /* 1365 */  vec_rule,
  /* 1366 */  vec_rule,
  /* 1367 */  vec_rule,
  /* 1368 */  vec_rule,
  /* 1369 */  vec_rule,
  /* 1370 */  _Binary_legVec_legVec_rule,
  /* 1371 */  _Binary_vec_vec_rule,
  /* 1372 */  _Binary_vec_vec_rule,
  /* 1373 */  _Binary_legVec_legVec_rule,
  /* 1374 */  _Binary_legVec_legVec_rule,
  /* 1375 */  _Binary_legVec_legVec_rule,
  /* 1376 */  _Binary_vec_vec_rule,
  /* 1377 */  _Binary_vec_vec_rule,
  /* 1378 */  legVec_rule,
  /* 1379 */  legVec_rule,
  /* 1380 */  legVec_rule,
  /* 1381 */  legVec_rule,
  /* 1382 */  legVec_rule,
  /* 1383 */  legVec_rule,
  /* 1384 */  legVec_rule,
  /* 1385 */  legVec_rule,
  /* 1386 */  _Binary_vec_vec_rule,
  /* 1387 */  _Binary_legVec_legVec_rule,
  /* 1388 */  _Binary_legVec_legVec_rule,
  /* 1389 */  _Binary_vec_vec_rule,
  /* 1390 */  _Binary_vec_vec_rule,
  /* 1391 */  legVec_rule,
  /* 1392 */  vec_rule,
  /* 1393 */  vec_rule,
  /* 1394 */  vec_rule,
  /* 1395 */  vec_rule,
  /* 1396 */  vec_rule,
  /* 1397 */  vec_rule,
  /* 1398 */  vec_rule,
  /* 1399 */  vec_rule,
  /* 1400 */  vec_rule,
  /* 1401 */  kReg_rule,
  /* 1402 */  kReg_rule,
  /* 1403 */  vec_rule,
  /* 1404 */  legVec_rule,
  /* 1405 */  vec_rule,
  /* 1406 */  vec_rule,
  /* 1407 */  vec_rule,
  /* 1408 */  vec_rule,
  /* 1409 */  legVec_rule,
  /* 1410 */  vec_rule,
  /* 1411 */  vec_rule,
  /* 1412 */  vec_rule,
  /* 1413 */  vec_rule,
  /* 1414 */  vec_rule,
  /* 1415 */  vec_rule,
  /* 1416 */  vec_rule,
  /* 1417 */  vec_rule,
  /* 1418 */  vec_rule,
  /* 1419 */  vec_rule,
  /* 1420 */  vec_rule,
  /* 1421 */  vec_rule,
  /* 1422 */  vec_rule,
  /* 1423 */  _MulAddVS2VI_vec_vec_rule,
  /* 1424 */  vec_rule,
  /* 1425 */  vec_rule,
  /* 1426 */  vec_rule,
  /* 1427 */  vec_rule,
  /* 1428 */  vec_rule,
  /* 1429 */  vec_rule,
  /* 1430 */  vec_rule,
  /* 1431 */  vec_rule,
  /* 1432 */  vec_rule,
  /* 1433 */  _Binary_vec_vec_rule,
  /* 1434 */  _Binary_vec_vec_rule,
  /* 1435 */  vec_rule,
  /* 1436 */  vec_rule,
  /* 1437 */  vec_rule,
  /* 1438 */  vec_rule,
  /* 1439 */  memory_rule,
  /* 1440 */  memory_rule,
  /* 1441 */  memory_rule,
  /* 1442 */  memory_rule,
  /* 1443 */  vec_rule,
  /* 1444 */  vec_rule,
  /* 1445 */  vec_rule,
  /* 1446 */  vec_rule,
  /* 1447 */  vec_rule,
  /* 1448 */  vec_rule,
  /* 1449 */  vec_rule,
  /* 1450 */  vec_rule,
  /* 1451 */  vec_rule,
  /* 1452 */  _Binary_vec_vec_rule,
  /* 1453 */  _Binary_vec_vec_rule,
  /* 1454 */  _Binary_vec_vec_rule,
  /* 1455 */  _Binary_vec_vec_rule,
  /* 1456 */  _Binary_vec_vec_rule,
  /* 1457 */  _Binary_vec_vec_rule,
  /* 1458 */  _Binary_vec__LoadVector_memory__rule,
  /* 1459 */  _Binary_vec__LoadVector_memory__rule,
  /* 1460 */  _Binary_vec__LoadVector_memory__rule,
  /* 1461 */  _Binary_vec__LoadVector_memory__rule,
  /* 1462 */  _Binary_vec__LoadVector_memory__rule,
  /* 1463 */  _Binary_vec__LoadVector_memory__rule,
  /* 1464 */  _Binary_vec_vec_rule,
  /* 1465 */  _Binary_vec__LoadVector_memory__rule,
  /* 1466 */  _Binary_vec_vec_rule,
  /* 1467 */  _Binary_vec__LoadVector_memory__rule,
  /* 1468 */  _Binary_vec_vec_rule,
  /* 1469 */  _Binary_vec__LoadVector_memory__rule,
  /* 1470 */  _Binary_vec_vec_rule,
  /* 1471 */  _Binary_vec_vec_rule,
  /* 1472 */  _Binary_vec_vec_rule,
  /* 1473 */  _Binary_vec_vec_rule,
  /* 1474 */  _Binary_vec_vec_rule,
  /* 1475 */  _Binary_vec_vec_rule,
  /* 1476 */  _Binary_vec__LoadVector_memory__rule,
  /* 1477 */  _Binary_vec__LoadVector_memory__rule,
  /* 1478 */  _Binary_vec__LoadVector_memory__rule,
  /* 1479 */  _Binary_vec__LoadVector_memory__rule,
  /* 1480 */  _Binary_vec__LoadVector_memory__rule,
  /* 1481 */  _Binary_vec__LoadVector_memory__rule,
  /* 1482 */  _Binary_vec_vec_rule,
  /* 1483 */  _Binary_vec_vec_rule,
  /* 1484 */  _Binary_vec_vec_rule,
  /* 1485 */  _Binary_vec_vec_rule,
  /* 1486 */  _Binary_vec_vec_rule,
  /* 1487 */  _Binary_vec__LoadVector_memory__rule,
  /* 1488 */  _Binary_vec__LoadVector_memory__rule,
  /* 1489 */  _Binary_vec__LoadVector_memory__rule,
  /* 1490 */  _Binary_vec__LoadVector_memory__rule,
  /* 1491 */  _Binary_vec__LoadVector_memory__rule,
  /* 1492 */  vec_rule,
  /* 1493 */  vec_rule,
  /* 1494 */  _Binary_vec_vec_rule,
  /* 1495 */  _Binary_vec_vec_rule,
  /* 1496 */  _Binary_vec__LoadVector_memory__rule,
  /* 1497 */  _Binary_vec__LoadVector_memory__rule,
  /* 1498 */  _Binary_vec_immI8_rule,
  /* 1499 */  _Binary_vec_immI8_rule,
  /* 1500 */  _Binary_vec_vec_rule,
  /* 1501 */  _Binary_vec_vec_rule,
  /* 1502 */  _Binary_vec__LShiftCntV_immI8__rule,
  /* 1503 */  _Binary_vec__LShiftCntV_immI8__rule,
  /* 1504 */  _Binary_vec__LShiftCntV_immI8__rule,
  /* 1505 */  _Binary_vec_vec_rule,
  /* 1506 */  _Binary_vec_vec_rule,
  /* 1507 */  _Binary_vec_vec_rule,
  /* 1508 */  _Binary_vec_vec_rule,
  /* 1509 */  _Binary_vec_vec_rule,
  /* 1510 */  _Binary_vec_vec_rule,
  /* 1511 */  _Binary_vec__LoadVector_memory__rule,
  /* 1512 */  _Binary_vec__LoadVector_memory__rule,
  /* 1513 */  _Binary_vec__LoadVector_memory__rule,
  /* 1514 */  _Binary_vec__RShiftCntV_immI8__rule,
  /* 1515 */  _Binary_vec__RShiftCntV_immI8__rule,
  /* 1516 */  _Binary_vec__RShiftCntV_immI8__rule,
  /* 1517 */  _Binary_vec_vec_rule,
  /* 1518 */  _Binary_vec_vec_rule,
  /* 1519 */  _Binary_vec_vec_rule,
  /* 1520 */  _Binary_vec_vec_rule,
  /* 1521 */  _Binary_vec_vec_rule,
  /* 1522 */  _Binary_vec_vec_rule,
  /* 1523 */  _Binary_vec__LoadVector_memory__rule,
  /* 1524 */  _Binary_vec__LoadVector_memory__rule,
  /* 1525 */  _Binary_vec__LoadVector_memory__rule,
  /* 1526 */  _Binary_vec__RShiftCntV_immI8__rule,
  /* 1527 */  _Binary_vec__RShiftCntV_immI8__rule,
  /* 1528 */  _Binary_vec__RShiftCntV_immI8__rule,
  /* 1529 */  _Binary_vec_vec_rule,
  /* 1530 */  _Binary_vec_vec_rule,
  /* 1531 */  _Binary_vec_vec_rule,
  /* 1532 */  _Binary_vec_vec_rule,
  /* 1533 */  _Binary_vec_vec_rule,
  /* 1534 */  _Binary_vec_vec_rule,
  /* 1535 */  _Binary_vec__LoadVector_memory__rule,
  /* 1536 */  _Binary_vec__LoadVector_memory__rule,
  /* 1537 */  _Binary_vec__LoadVector_memory__rule,
  /* 1538 */  _Binary_vec_vec_rule,
  /* 1539 */  _Binary_vec__LoadVector_memory__rule,
  /* 1540 */  _Binary_vec_vec_rule,
  /* 1541 */  _Binary_vec__LoadVector_memory__rule,
  /* 1542 */  _Binary_vec_vec_rule,
  /* 1543 */  vec_rule,
  /* 1544 */  vec_rule,
  /* 1545 */  vec_rule,
  /* 1546 */  vec_rule,
  /* 1547 */  _Binary_vec_vec_rule,
  /* 1548 */  _Binary_vec_vec_rule,
  /* 1549 */  _Binary_vec_vec_rule,
  /* 1550 */  _Binary_vec_vec_rule,
  /* 1551 */  _Binary_vec_vec_rule,
  /* 1552 */  rRegI_rule,
  /* 1553 */  kReg_rule,
  /* 1554 */  kReg_rule,
  /* 1555 */  kReg_rule,
  /* 1556 */  vec_rule,
  /* 1557 */  vec_rule,
  /* 1558 */  regF_rule,
  /* 1559 */  regD_rule,
  /* 1560 */  memory_rule,
  /* 1561 */  memory_rule,
  /* 1562 */  memory_rule,
  // last instruction
  0 // no trailing comma
};

const        int   rightOp[] = {
  /*    0 */  0,
  /*    1 */  0,
  /*    2 */  0,
  /*    3 */  0,
  /*    4 */  0,
  /*    5 */  0,
  /*    6 */  0,
  /*    7 */  0,
  /*    8 */  0,
  /*    9 */  0,
  /*   10 */  0,
  /*   11 */  0,
  /*   12 */  0,
  /*   13 */  0,
  /*   14 */  0,
  /*   15 */  0,
  /*   16 */  0,
  /*   17 */  0,
  /*   18 */  0,
  /*   19 */  0,
  /*   20 */  0,
  /*   21 */  0,
  /*   22 */  0,
  /*   23 */  0,
  /*   24 */  0,
  /*   25 */  0,
  /*   26 */  0,
  /*   27 */  0,
  /*   28 */  0,
  /*   29 */  0,
  /*   30 */  0,
  /*   31 */  0,
  /*   32 */  0,
  /*   33 */  0,
  /*   34 */  0,
  /*   35 */  0,
  /*   36 */  0,
  /*   37 */  0,
  /*   38 */  0,
  /*   39 */  0,
  /*   40 */  0,
  /*   41 */  0,
  /*   42 */  0,
  /*   43 */  0,
  /*   44 */  0,
  /*   45 */  0,
  /*   46 */  0,
  /*   47 */  0,
  /*   48 */  0,
  /*   49 */  0,
  /*   50 */  0,
  /*   51 */  0,
  /*   52 */  0,
  /*   53 */  0,
  /*   54 */  0,
  /*   55 */  0,
  /*   56 */  0,
  /*   57 */  0,
  /*   58 */  0,
  /*   59 */  0,
  /*   60 */  0,
  /*   61 */  0,
  /*   62 */  0,
  /*   63 */  0,
  /*   64 */  0,
  /*   65 */  0,
  /*   66 */  0,
  /*   67 */  0,
  /*   68 */  0,
  /*   69 */  0,
  /*   70 */  0,
  /*   71 */  0,
  /*   72 */  0,
  /*   73 */  0,
  /*   74 */  0,
  /*   75 */  0,
  /*   76 */  0,
  /*   77 */  0,
  /*   78 */  0,
  /*   79 */  0,
  /*   80 */  0,
  /*   81 */  0,
  /*   82 */  0,
  /*   83 */  0,
  /*   84 */  0,
  /*   85 */  0,
  /*   86 */  0,
  /*   87 */  0,
  /*   88 */  0,
  /*   89 */  0,
  /*   90 */  0,
  /*   91 */  0,
  /*   92 */  0,
  /*   93 */  0,
  /*   94 */  0,
  /*   95 */  0,
  /*   96 */  0,
  /*   97 */  0,
  /*   98 */  0,
  /*   99 */  0,
  /*  100 */  0,
  /*  101 */  0,
  /*  102 */  0,
  /*  103 */  0,
  /*  104 */  0,
  /*  105 */  0,
  /*  106 */  0,
  /*  107 */  immI8_rule,
  /*  108 */  immI_rule,
  /*  109 */  rRegI_rule,
  /*  110 */  immI_rule,
  /*  111 */  rRegI_rule,
  /*  112 */  _LShiftI_rRegI_immI2_rule,
  /*  113 */  immI_rule,
  /*  114 */  0,
  /*  115 */  0,
  /*  116 */  immI_rule,
  /*  117 */  0,
  /*  118 */  0,
  /*  119 */  0,
  /*  120 */  0,
  /*  121 */  0,
  /*  122 */  0,
  /*  123 */  0,
  /*  124 */  0,
  /*  125 */  0,
  /*  126 */  0,
  /*  127 */  0,
  /*  128 */  0,
  /*  129 */  0,
  /*  130 */  0,
  /*  131 */  0,
  /*  132 */  0,
  /*  133 */  0,
  /*  134 */  0,
  /*  135 */  0,
  /*  136 */  0,
  /*  137 */  0,
  /*  138 */  0,
  /*  139 */  0,
  /*  140 */  0,
  /*  141 */  0,
  // last operand
  /*  142 */  0,
  /*  143 */  0,
  /*  144 */  0,
  // last operand class
  /*  145 */  rRegI_rule,
  /*  146 */  immI2_rule,
  /*  147 */  _LShiftI_rRegI_immI2_rule,
  /*  148 */  0,
  /*  149 */  0,
  /*  150 */  0,
  /*  151 */  0,
  /*  152 */  immI_rule,
  /*  153 */  0,
  /*  154 */  immI_24_rule,
  /*  155 */  0,
  /*  156 */  immI_24_rule,
  /*  157 */  immI_255_rule,
  /*  158 */  immI_rule,
  /*  159 */  immI_24_rule,
  /*  160 */  immI_16_rule,
  /*  161 */  immI_255_rule,
  /*  162 */  immI_65535_rule,
  /*  163 */  immU31_rule,
  /*  164 */  0,
  /*  165 */  0,
  /*  166 */  0,
  /*  167 */  0,
  /*  168 */  0,
  /*  169 */  eFlagsReg_rule,
  /*  170 */  rRegI_rule,
  /*  171 */  eFlagsRegU_rule,
  /*  172 */  eFlagsRegUCF_rule,
  /*  173 */  _LoadI_memory__rule,
  /*  174 */  eRegP_rule,
  /*  175 */  eFlagsRegU_rule,
  /*  176 */  regDPR_rule,
  /*  177 */  regFPR_rule,
  /*  178 */  regDPR_rule,
  /*  179 */  regFPR_rule,
  /*  180 */  regF_rule,
  /*  181 */  regD_rule,
  /*  182 */  eRegL_rule,
  /*  183 */  rRegI_rule,
  /*  184 */  _LoadI_memory__rule,
  /*  185 */  immI_rule,
  /*  186 */  immI_1_rule,
  /*  187 */  immI_M1_rule,
  /*  188 */  eBCXRegL_rule,
  /*  189 */  eCXRegP_rule,
  /*  190 */  eCXRegI_rule,
  /*  191 */  rRegI_rule,
  /*  192 */  rRegI_rule,
  /*  193 */  0,
  /*  194 */  eADXRegL_low_only_rule,
  /*  195 */  immI_32_rule,
  /*  196 */  _ConvI2L_nadxRegI__rule,
  /*  197 */  immI_32_rule,
  /*  198 */  immI_32_63_rule,
  /*  199 */  immI_32_63_rule,
  /*  200 */  0,
  /*  201 */  immL_32bits_rule,
  /*  202 */  immL_32bits_rule,
  /*  203 */  immI_1_rule,
  /*  204 */  immI8_rule,
  /*  205 */  immI_24_rule,
  /*  206 */  immI_16_rule,
  /*  207 */  rRegI_rule,
  /*  208 */  _LoadI_memory__rule,
  /*  209 */  immI_rule,
  /*  210 */  immI_M1_rule,
  /*  211 */  _LoadI_memory__rule,
  /*  212 */  immI_M1_rule,
  /*  213 */  0,
  /*  214 */  rRegI_rule,
  /*  215 */  _LoadI_memory__rule,
  /*  216 */  immI_rule,
  /*  217 */  immI_1_rule,
  /*  218 */  immI_M1_rule,
  /*  219 */  immI8_rule,
  /*  220 */  immI8_rule,
  /*  221 */  eCXRegI_rule,
  /*  222 */  eCXRegI_rule,
  /*  223 */  _SubI_immI_0_eCXRegI_rule,
  /*  224 */  eCXRegI_rule,
  /*  225 */  _SubI_immI_32_eCXRegI_rule,
  /*  226 */  immI_1_rule,
  /*  227 */  immI_M1_rule,
  /*  228 */  eCXRegI_rule,
  /*  229 */  _SubI_immI_0_eCXRegI_rule,
  /*  230 */  _SubI_immI_32_eCXRegI_rule,
  /*  231 */  rRegI_rule,
  /*  232 */  _LoadI_memory__rule,
  /*  233 */  immI_rule,
  /*  234 */  rRegI_rule,
  /*  235 */  rRegI_rule,
  /*  236 */  rRegI_rule,
  /*  237 */  _CmpLTMask_rRegI_rRegI_rule,
  /*  238 */  0,
  /*  239 */  immL_M1_rule,
  /*  240 */  eRegL_rule,
  /*  241 */  _LoadL_memory__rule,
  /*  242 */  immL_M1_rule,
  /*  243 */  immL_M1_rule,
  /*  244 */  0,
  /*  245 */  regDPR_rule,
  /*  246 */  regDPR_rule,
  /*  247 */  regDPR_rule,
  /*  248 */  0,
  /*  249 */  _LoadD_memory__rule,
  /*  250 */  0,
  /*  251 */  immDPR_rule,
  /*  252 */  regDPR_rule,
  /*  253 */  0,
  /*  254 */  regFPR_rule,
  /*  255 */  _LoadF_memory__rule,
  /*  256 */  regFPR_rule,
  /*  257 */  regFPR_rule,
  /*  258 */  0,
  /*  259 */  eCXRegI_rule,
  /*  260 */  eDXRegI_rule,
  /*  261 */  eSIRegP_rule,
  /*  262 */  eDXRegI_rule,
  /*  263 */  immI_rule,
  /*  264 */  eAXRegI_rule,
  /*  265 */  immI_rule,
  /*  266 */  memory_rule,
  /*  267 */  0,
  /*  268 */  eAXRegI_rule,
  /*  269 */  eAXRegP_rule,
  /*  270 */  flagsReg_long_LTGE_rule,
  /*  271 */  _LoadL_load_long_memory__rule,
  /*  272 */  flagsReg_ulong_LTGE_rule,
  /*  273 */  flagsReg_long_EQNE_rule,
  /*  274 */  flagsReg_ulong_EQNE_rule,
  /*  275 */  flagsReg_long_LEGT_rule,
  /*  276 */  flagsReg_ulong_LEGT_rule,
  /*  277 */  0,
  /*  278 */  0,
  /*  279 */  0,
  /*  280 */  kReg_rule,
  /*  281 */  vec_rule,
  /*  282 */  _Binary_vec_kReg_rule,
  /*  283 */  rRegI_rule,
  /*  284 */  regF_rule,
  /*  285 */  0,
  /*  286 */  0,
  /*  287 */  legVec_rule,
  /*  288 */  _LoadVector_memory__rule,
  /*  289 */  vec_rule,
  /*  290 */  immU8_rule,
  /*  291 */  immU8_rule,
  /*  292 */  immI8_rule,
  /*  293 */  _LShiftCntV_immI8__rule,
  /*  294 */  _RShiftCntV_immI8__rule,
  /*  295 */  kReg_rule,
  /*  296 */  kReg_rule,
  /*  297 */  kReg_rule,
  /*  298 */  _Binary_immU8_kReg_rule,
  /*  299 */  _Binary_vec__Binary_immU8_kReg_rule,
  /*  300 */  _Binary_immU8_kReg_rule,
  /*  301 */  _Binary_memory__Binary_immU8_kReg_rule,
  /*  302 */  eRegP_rule,
  // last internally defined operand
  /*  303 */  0,
  /*  304 */  0,
  /*  305 */  0,
  /*  306 */  0,
  /*  307 */  0,
  /*  308 */  0,
  /*  309 */  0,
  /*  310 */  0,
  /*  311 */  0,
  /*  312 */  0,
  /*  313 */  0,
  /*  314 */  0,
  /*  315 */  0,
  /*  316 */  0,
  /*  317 */  0,
  /*  318 */  0,
  /*  319 */  0,
  /*  320 */  0,
  /*  321 */  0,
  /*  322 */  0,
  /*  323 */  0,
  /*  324 */  0,
  /*  325 */  0,
  /*  326 */  0,
  /*  327 */  0,
  /*  328 */  0,
  /*  329 */  0,
  /*  330 */  0,
  /*  331 */  0,
  /*  332 */  0,
  /*  333 */  0,
  /*  334 */  0,
  /*  335 */  0,
  /*  336 */  0,
  /*  337 */  0,
  /*  338 */  0,
  /*  339 */  0,
  /*  340 */  0,
  /*  341 */  0,
  /*  342 */  0,
  /*  343 */  0,
  /*  344 */  0,
  /*  345 */  0,
  /*  346 */  0,
  /*  347 */  0,
  /*  348 */  0,
  /*  349 */  0,
  /*  350 */  0,
  /*  351 */  0,
  /*  352 */  0,
  /*  353 */  0,
  /*  354 */  0,
  /*  355 */  rRegI_rule,
  /*  356 */  immI_rule,
  /*  357 */  rRegI_rule,
  /*  358 */  immI_rule,
  /*  359 */  eAXRegI_rule,
  /*  360 */  rRegI_rule,
  /*  361 */  immI_rule,
  /*  362 */  regDPR_rule,
  /*  363 */  regDPR_rule,
  /*  364 */  regDPR_rule,
  /*  365 */  regD_rule,
  /*  366 */  regD_rule,
  /*  367 */  _LoadD_memory__rule,
  /*  368 */  _LoadD_memory__rule,
  /*  369 */  0,
  /*  370 */  0,
  /*  371 */  regFPR_rule,
  /*  372 */  regFPR_rule,
  /*  373 */  regFPR_rule,
  /*  374 */  regF_rule,
  /*  375 */  regF_rule,
  /*  376 */  _LoadF_memory__rule,
  /*  377 */  _LoadF_memory__rule,
  /*  378 */  0,
  /*  379 */  0,
  /*  380 */  0,
  /*  381 */  0,
  /*  382 */  0,
  /*  383 */  0,
  /*  384 */  0,
  /*  385 */  0,
  /*  386 */  0,
  /*  387 */  0,
  /*  388 */  0,
  /*  389 */  rRegI_rule,
  /*  390 */  immI_rule,
  /*  391 */  _LoadI_memory__rule,
  /*  392 */  immI_0_rule,
  /*  393 */  immI_0_rule,
  /*  394 */  immI_0_rule,
  /*  395 */  rRegI_rule,
  /*  396 */  immI_rule,
  /*  397 */  _LoadI_memory__rule,
  /*  398 */  immI_0_rule,
  /*  399 */  eRegP_rule,
  /*  400 */  immP_rule,
  /*  401 */  _LoadP_memory__rule,
  /*  402 */  _LoadP_memory__rule,
  /*  403 */  immP0_rule,
  /*  404 */  immI_0_rule,
  /*  405 */  immP0_rule,
  /*  406 */  immL0_rule,
  /*  407 */  eRegL_rule,
  /*  408 */  immL0_rule,
  /*  409 */  eRegL_rule,
  /*  410 */  immL0_rule,
  /*  411 */  eRegL_rule,
  /*  412 */  immL0_rule,
  /*  413 */  eRegL_rule,
  /*  414 */  immL0_rule,
  /*  415 */  eRegL_rule,
  /*  416 */  immL0_rule,
  /*  417 */  eRegL_rule,
  /*  418 */  eBXRegP_rule,
  /*  419 */  eBXRegP_rule,
  /*  420 */  eAXRegP_rule,
  /*  421 */  0,
  /*  422 */  0,
  /*  423 */  0,
  /*  424 */  0,
  /*  425 */  0,
  /*  426 */  0,
  /*  427 */  0,
  /*  428 */  0,
  /*  429 */  0,
  /*  430 */  0,
  /*  431 */  0,
  /*  432 */  0,
  /*  433 */  0,
  /*  434 */  0,
  /*  435 */  0,
  /*  436 */  0,
  /*  437 */  0,
  /*  438 */  0,
  /*  439 */  0,
  /*  440 */  0,
  /*  441 */  0,
  /*  442 */  0,
  /*  443 */  0,
  /*  444 */  0,
  /*  445 */  0,
  /*  446 */  0,
  /*  447 */  0,
  /*  448 */  0,
  /*  449 */  0,
  /*  450 */  0,
  /*  451 */  0,
  /*  452 */  0,
  /*  453 */  0,
  /*  454 */  0,
  /*  455 */  0,
  /*  456 */  0,
  /*  457 */  0,
  /*  458 */  0,
  /*  459 */  0,
  /*  460 */  0,
  /*  461 */  0,
  /*  462 */  0,
  /*  463 */  0,
  /*  464 */  0,
  /*  465 */  0,
  /*  466 */  0,
  /*  467 */  0,
  /*  468 */  0,
  /*  469 */  0,
  /*  470 */  0,
  /*  471 */  0,
  /*  472 */  0,
  /*  473 */  0,
  /*  474 */  0,
  /*  475 */  0,
  /*  476 */  0,
  /*  477 */  0,
  /*  478 */  0,
  /*  479 */  0,
  /*  480 */  0,
  /*  481 */  0,
  /*  482 */  0,
  /*  483 */  0,
  /*  484 */  0,
  /*  485 */  0,
  /*  486 */  0,
  /*  487 */  0,
  /*  488 */  0,
  /*  489 */  0,
  /*  490 */  0,
  /*  491 */  0,
  /*  492 */  0,
  /*  493 */  0,
  /*  494 */  0,
  /*  495 */  0,
  /*  496 */  0,
  /*  497 */  0,
  /*  498 */  0,
  /*  499 */  0,
  /*  500 */  0,
  /*  501 */  0,
  /*  502 */  0,
  /*  503 */  0,
  /*  504 */  0,
  /*  505 */  0,
  /*  506 */  0,
  /*  507 */  0,
  /*  508 */  0,
  /*  509 */  0,
  /*  510 */  0,
  /*  511 */  0,
  /*  512 */  0,
  /*  513 */  0,
  /*  514 */  0,
  /*  515 */  0,
  /*  516 */  0,
  /*  517 */  0,
  /*  518 */  0,
  /*  519 */  0,
  /*  520 */  0,
  /*  521 */  0,
  /*  522 */  0,
  /*  523 */  0,
  /*  524 */  0,
  /*  525 */  0,
  /*  526 */  0,
  /*  527 */  0,
  /*  528 */  0,
  /*  529 */  0,
  /*  530 */  0,
  /*  531 */  0,
  /*  532 */  0,
  /*  533 */  immI_24_rule,
  /*  534 */  0,
  /*  535 */  0,
  /*  536 */  immI_24_rule,
  /*  537 */  0,
  /*  538 */  0,
  /*  539 */  0,
  /*  540 */  0,
  /*  541 */  immI_24_rule,
  /*  542 */  immI_255_rule,
  /*  543 */  immI_16_rule,
  /*  544 */  immI_65535_rule,
  /*  545 */  0,
  /*  546 */  0,
  /*  547 */  0,
  /*  548 */  0,
  /*  549 */  immL_32bits_rule,
  /*  550 */  0,
  /*  551 */  0,
  /*  552 */  0,
  /*  553 */  0,
  /*  554 */  0,
  /*  555 */  0,
  /*  556 */  0,
  /*  557 */  0,
  /*  558 */  0,
  /*  559 */  0,
  /*  560 */  0,
  /*  561 */  0,
  /*  562 */  0,
  /*  563 */  0,
  /*  564 */  0,
  /*  565 */  0,
  /*  566 */  0,
  /*  567 */  xRegI_rule,
  /*  568 */  rRegI_rule,
  /*  569 */  rRegI_rule,
  /*  570 */  eRegL_rule,
  /*  571 */  _ConvL2I_eRegL__rule,
  /*  572 */  stackSlotL_rule,
  /*  573 */  stackSlotL_rule,
  /*  574 */  eRegL_rule,
  /*  575 */  anyRegP_rule,
  /*  576 */  immI_rule,
  /*  577 */  immI16_rule,
  /*  578 */  immP_rule,
  /*  579 */  immI8_rule,
  /*  580 */  immI8_rule,
  /*  581 */  regDPR1_rule,
  /*  582 */  _RoundDouble_regDPR1__rule,
  /*  583 */  regD_rule,
  /*  584 */  regF_rule,
  /*  585 */  regFPR1_rule,
  /*  586 */  _RoundFloat_regFPR1__rule,
  /*  587 */  _ConvD2F_regDPR1__rule,
  /*  588 */  immFPR_rule,
  /*  589 */  immF_rule,
  /*  590 */  0,
  /*  591 */  0,
  /*  592 */  0,
  /*  593 */  0,
  /*  594 */  0,
  /*  595 */  0,
  /*  596 */  0,
  /*  597 */  0,
  /*  598 */  0,
  /*  599 */  _Binary_rRegI_rRegI_rule,
  /*  600 */  _Binary_rRegI_rRegI_rule,
  /*  601 */  _Binary_rRegI_rRegI_rule,
  /*  602 */  _Binary_rRegI_rRegI_rule,
  /*  603 */  _Binary_rRegI_rRegI_rule,
  /*  604 */  _Binary_rRegI__LoadI_memory__rule,
  /*  605 */  _Binary_rRegI__LoadI_memory__rule,
  /*  606 */  _Binary_rRegI__LoadI_memory__rule,
  /*  607 */  _Binary_eRegP_eRegP_rule,
  /*  608 */  _Binary_eRegP_eRegP_rule,
  /*  609 */  _Binary_eRegP_eRegP_rule,
  /*  610 */  _Binary_eRegP_eRegP_rule,
  /*  611 */  _Binary_regDPR1_regDPR_rule,
  /*  612 */  _Binary_regFPR1_regFPR_rule,
  /*  613 */  _Binary_regDPR_regDPR_rule,
  /*  614 */  _Binary_regFPR_regFPR_rule,
  /*  615 */  _Binary_regF_regF_rule,
  /*  616 */  _Binary_regD_regD_rule,
  /*  617 */  _Binary_regF_regF_rule,
  /*  618 */  _Binary_regF_regF_rule,
  /*  619 */  _Binary_regD_regD_rule,
  /*  620 */  _Binary_regD_regD_rule,
  /*  621 */  _Binary_eRegL_eRegL_rule,
  /*  622 */  _Binary_eRegL_eRegL_rule,
  /*  623 */  _Binary_eRegL_eRegL_rule,
  /*  624 */  rRegI_rule,
  /*  625 */  immI_rule,
  /*  626 */  immI_1_rule,
  /*  627 */  immI_rule,
  /*  628 */  immI_rule,
  /*  629 */  immI_M1_rule,
  /*  630 */  rRegI_rule,
  /*  631 */  immI_rule,
  /*  632 */  _LoadI_memory__rule,
  /*  633 */  rRegI_rule,
  /*  634 */  _AddI__LoadI_memory__rRegI_rule,
  /*  635 */  _AddI_rRegI__LoadI_memory__rule,
  /*  636 */  _AddI__LoadI_memory__immI_rule,
  /*  637 */  _AddI__LoadI_memory__immI_1_rule,
  /*  638 */  _AddI__LoadI_memory__immI_M1_rule,
  /*  639 */  _Binary_eADXRegL_eBCXRegL_rule,
  /*  640 */  _Binary_eADXRegL_eBCXRegL_rule,
  /*  641 */  _Binary_eAXRegP_eCXRegP_rule,
  /*  642 */  _Binary_eAXRegP_eCXRegP_rule,
  /*  643 */  _Binary_eAXRegI_eCXRegI_rule,
  /*  644 */  _Binary_eAXRegI_eCXRegI_rule,
  /*  645 */  _Binary_eAXRegI_eCXRegI_rule,
  /*  646 */  _Binary_eAXRegI_eCXRegI_rule,
  /*  647 */  _Binary_eAXRegI_eCXRegI_rule,
  /*  648 */  _Binary_eAXRegI_eCXRegI_rule,
  /*  649 */  _Binary_eADXRegL_eBCXRegL_rule,
  /*  650 */  _Binary_eAXRegP_eCXRegP_rule,
  /*  651 */  _Binary_eAXRegI_eCXRegI_rule,
  /*  652 */  _Binary_eAXRegI_eCXRegI_rule,
  /*  653 */  _Binary_eAXRegI_eCXRegI_rule,
  /*  654 */  immI_rule,
  /*  655 */  xRegI_rule,
  /*  656 */  immI_rule,
  /*  657 */  rRegI_rule,
  /*  658 */  immI_rule,
  /*  659 */  rRegI_rule,
  /*  660 */  xRegI_rule,
  /*  661 */  rRegI_rule,
  /*  662 */  rRegI_rule,
  /*  663 */  pRegP_rule,
  /*  664 */  rRegI_rule,
  /*  665 */  immI_rule,
  /*  666 */  _LoadI_memory__rule,
  /*  667 */  _SubI__LoadI_memory__rRegI_rule,
  /*  668 */  _SubI_immI_0_rRegI_rule,
  /*  669 */  rRegI_rule,
  /*  670 */  rRegI_rule,
  /*  671 */  immI_rule,
  /*  672 */  0,
  /*  673 */  0,
  /*  674 */  0,
  /*  675 */  0,
  /*  676 */  immI_rule,
  /*  677 */  _LoadI_memory__rule,
  /*  678 */  rRegI_rule,
  /*  679 */  _Binary_rRegI_rRegI_rule,
  /*  680 */  _ConvI2L_nadxRegI__rule,
  /*  681 */  _ConvI2L_eAXRegI__rule,
  /*  682 */  _AndL__ConvI2L_nadxRegI__immL_32bits_rule,
  /*  683 */  _AndL__ConvI2L_eAXRegI__immL_32bits_rule,
  /*  684 */  eRegL_rule,
  /*  685 */  eRegL_rule,
  /*  686 */  eRegL_rule,
  /*  687 */  eRegL_rule,
  /*  688 */  immL_127_rule,
  /*  689 */  eCXRegI_rule,
  /*  690 */  eRegL_rule,
  /*  691 */  eCXRegI_rule,
  /*  692 */  eCXRegI_rule,
  /*  693 */  eRegL_rule,
  /*  694 */  immL32_rule,
  /*  695 */  immL32_rule,
  /*  696 */  immI_1_rule,
  /*  697 */  immI8_rule,
  /*  698 */  eCXRegI_rule,
  /*  699 */  immI_1_rule,
  /*  700 */  _RShiftI__LoadI_memory__immI_1_rule,
  /*  701 */  immI8_rule,
  /*  702 */  _RShiftI__LoadI_memory__immI8_rule,
  /*  703 */  eCXRegI_rule,
  /*  704 */  immI_1_rule,
  /*  705 */  immI8_rule,
  /*  706 */  immI_24_rule,
  /*  707 */  immI_16_rule,
  /*  708 */  eCXRegI_rule,
  /*  709 */  rRegI_rule,
  /*  710 */  immI_rule,
  /*  711 */  _LoadI_memory__rule,
  /*  712 */  rRegI_rule,
  /*  713 */  _AndI__LoadI_memory__rRegI_rule,
  /*  714 */  _AndI_rRegI__LoadI_memory__rule,
  /*  715 */  _AndI__LoadI_memory__immI_rule,
  /*  716 */  rRegI_rule,
  /*  717 */  _XorI_rRegI_immI_M1_rule,
  /*  718 */  _LoadI_memory__rule,
  /*  719 */  _XorI_rRegI_immI_M1_rule,
  /*  720 */  rRegI_rule,
  /*  721 */  _SubI_immI_0_rRegI_rule,
  /*  722 */  _LoadI_memory__rule,
  /*  723 */  _SubI_immI_0__LoadI_memory__rule,
  /*  724 */  rRegI_rule,
  /*  725 */  _AddI_rRegI_immI_M1_rule,
  /*  726 */  _LoadI_memory__rule,
  /*  727 */  _AddI__LoadI_memory__immI_M1_rule,
  /*  728 */  rRegI_rule,
  /*  729 */  _AddI_rRegI_immI_M1_rule,
  /*  730 */  _LoadI_memory__rule,
  /*  731 */  _AddI__LoadI_memory__immI_M1_rule,
  /*  732 */  rRegI_rule,
  /*  733 */  _CastP2X_eRegP__rule,
  /*  734 */  rRegI_rule,
  /*  735 */  immI_rule,
  /*  736 */  _LoadI_memory__rule,
  /*  737 */  rRegI_rule,
  /*  738 */  _OrI__LoadI_memory__rRegI_rule,
  /*  739 */  _OrI_rRegI__LoadI_memory__rule,
  /*  740 */  _OrI__LoadI_memory__immI_rule,
  /*  741 */  0,
  /*  742 */  0,
  /*  743 */  0,
  /*  744 */  _URShiftI_rRegI_immI_M1_rule,
  /*  745 */  _LShiftI_rRegI_immI_1_rule,
  /*  746 */  _URShiftI_rRegI_immI8_rule,
  /*  747 */  _LShiftI_rRegI_immI8_rule,
  /*  748 */  _URShiftI_ncxRegI__SubI_immI_0_eCXRegI_rule,
  /*  749 */  _LShiftI_ncxRegI_eCXRegI_rule,
  /*  750 */  _URShiftI_ncxRegI__SubI_immI_32_eCXRegI_rule,
  /*  751 */  _LShiftI_ncxRegI_eCXRegI_rule,
  /*  752 */  0,
  /*  753 */  0,
  /*  754 */  0,
  /*  755 */  _LShiftI_rRegI_immI_M1_rule,
  /*  756 */  _URShiftI_rRegI_immI_1_rule,
  /*  757 */  _LShiftI_rRegI_immI8_rule,
  /*  758 */  _URShiftI_rRegI_immI8_rule,
  /*  759 */  _LShiftI_ncxRegI__SubI_immI_0_eCXRegI_rule,
  /*  760 */  _URShiftI_ncxRegI_eCXRegI_rule,
  /*  761 */  _LShiftI_ncxRegI__SubI_immI_32_eCXRegI_rule,
  /*  762 */  _URShiftI_ncxRegI_eCXRegI_rule,
  /*  763 */  rRegI_rule,
  /*  764 */  immI_M1_rule,
  /*  765 */  immI_rule,
  /*  766 */  _LoadI_memory__rule,
  /*  767 */  rRegI_rule,
  /*  768 */  _XorI__LoadI_memory__rRegI_rule,
  /*  769 */  _XorI_rRegI__LoadI_memory__rule,
  /*  770 */  _XorI__LoadI_memory__immI_rule,
  /*  771 */  0,
  /*  772 */  0,
  /*  773 */  0,
  /*  774 */  0,
  /*  775 */  ncxRegI_rule,
  /*  776 */  immI_0_rule,
  /*  777 */  _SubI_rRegI_rRegI_rule,
  /*  778 */  _AndI__CmpLTMask_rRegI_rRegI_rRegI_rule,
  /*  779 */  _SubI_rRegI_rRegI_rule,
  /*  780 */  _AndI_rRegI__CmpLTMask_rRegI_rRegI_rule,
  /*  781 */  rRegI_rule,
  /*  782 */  _CmpLTMask_rRegI_rRegI_rule,
  /*  783 */  0,
  /*  784 */  eRegL_rule,
  /*  785 */  immL_rule,
  /*  786 */  _LoadL_load_long_memory__rule,
  /*  787 */  eRegL_rule,
  /*  788 */  eRegL_rule,
  /*  789 */  immL_rule,
  /*  790 */  _LoadL_load_long_memory__rule,
  /*  791 */  eRegL_rule,
  /*  792 */  eRegL_rule,
  /*  793 */  immL_rule,
  /*  794 */  _LoadL_load_long_memory__rule,
  /*  795 */  eRegL_rule,
  /*  796 */  eRegL_rule,
  /*  797 */  _XorL_eRegL_immL_M1_rule,
  /*  798 */  _LoadL_memory__rule,
  /*  799 */  _XorL_eRegL_immL_M1_rule,
  /*  800 */  eRegL_rule,
  /*  801 */  _SubL_immL0_eRegL_rule,
  /*  802 */  _LoadL_memory__rule,
  /*  803 */  _SubL_immL0__LoadL_memory__rule,
  /*  804 */  eRegL_rule,
  /*  805 */  _AddL_eRegL_immL_M1_rule,
  /*  806 */  _LoadL_memory__rule,
  /*  807 */  _AddL__LoadL_memory__immL_M1_rule,
  /*  808 */  eRegL_rule,
  /*  809 */  _AddL_eRegL_immL_M1_rule,
  /*  810 */  _LoadL_memory__rule,
  /*  811 */  _AddL__LoadL_memory__immL_M1_rule,
  /*  812 */  eRegL_rule,
  /*  813 */  immL_rule,
  /*  814 */  _LoadL_load_long_memory__rule,
  /*  815 */  eRegL_rule,
  /*  816 */  eRegL_rule,
  /*  817 */  immL_M1_rule,
  /*  818 */  immL_rule,
  /*  819 */  _LoadL_load_long_memory__rule,
  /*  820 */  eRegL_rule,
  /*  821 */  immI_1_rule,
  /*  822 */  immI_2_rule,
  /*  823 */  immI_3_rule,
  /*  824 */  immI_1_31_rule,
  /*  825 */  immI_32_63_rule,
  /*  826 */  eCXRegI_rule,
  /*  827 */  immI_1_31_rule,
  /*  828 */  immI_32_63_rule,
  /*  829 */  eCXRegI_rule,
  /*  830 */  immI_1_31_rule,
  /*  831 */  immI_32_63_rule,
  /*  832 */  eCXRegI_rule,
  /*  833 */  immDPR0_rule,
  /*  834 */  regDPR_rule,
  /*  835 */  regD_rule,
  /*  836 */  _LoadD_memory__rule,
  /*  837 */  regDPR_rule,
  /*  838 */  0,
  /*  839 */  _LoadD_memory__rule,
  /*  840 */  regDPR_rule,
  /*  841 */  0,
  /*  842 */  _LoadD_memory__rule,
  /*  843 */  regDPR_rule,
  /*  844 */  _RoundDouble__AddD__LoadD_memory__regDPR__rule,
  /*  845 */  _RoundDouble__AddD_regDPR__LoadD_memory___rule,
  /*  846 */  immDPR1_rule,
  /*  847 */  immDPR_rule,
  /*  848 */  0,
  /*  849 */  regDPR_rule,
  /*  850 */  regnotDPR1_rule,
  /*  851 */  immDPR_rule,
  /*  852 */  _LoadD_memory__rule,
  /*  853 */  regDPR_rule,
  /*  854 */  _LoadD_memory__rule,
  /*  855 */  regDPR_rule,
  /*  856 */  regDPR_rule,
  /*  857 */  _MulD_regDPR_regDPR_rule,
  /*  858 */  regDPR_rule,
  /*  859 */  regDPR_rule,
  /*  860 */  regnotDPR1_rule,
  /*  861 */  regDPR_rule,
  /*  862 */  regD_rule,
  /*  863 */  regDPR_rule,
  /*  864 */  regD_rule,
  /*  865 */  0,
  /*  866 */  immFPR0_rule,
  /*  867 */  regFPR_rule,
  /*  868 */  regF_rule,
  /*  869 */  _LoadF_memory__rule,
  /*  870 */  regFPR_rule,
  /*  871 */  regFPR_rule,
  /*  872 */  regFPR_rule,
  /*  873 */  regFPR_rule,
  /*  874 */  _LoadF_memory__rule,
  /*  875 */  regFPR_rule,
  /*  876 */  _LoadF_memory__rule,
  /*  877 */  regFPR_rule,
  /*  878 */  regFPR_rule,
  /*  879 */  _LoadF_memory__rule,
  /*  880 */  memory_rule,
  /*  881 */  memory_rule,
  /*  882 */  immFPR_rule,
  /*  883 */  immFPR_rule,
  /*  884 */  regFPR_rule,
  /*  885 */  regFPR_rule,
  /*  886 */  _LoadF_memory__rule,
  /*  887 */  regFPR_rule,
  /*  888 */  _LoadF_memory__rule,
  /*  889 */  regFPR_rule,
  /*  890 */  memory_rule,
  /*  891 */  immFPR_rule,
  /*  892 */  immFPR_rule,
  /*  893 */  regFPR_rule,
  /*  894 */  _LoadF_memory__rule,
  /*  895 */  regFPR_rule,
  /*  896 */  _MulF__LoadF_memory__regFPR_rule,
  /*  897 */  regFPR_rule,
  /*  898 */  _MulF_regFPR__LoadF_memory__rule,
  /*  899 */  regFPR_rule,
  /*  900 */  _MulF_regFPR_regFPR_rule,
  /*  901 */  regFPR_rule,
  /*  902 */  regFPR_rule,
  /*  903 */  regFPR_rule,
  /*  904 */  regFPR_rule,
  /*  905 */  regFPR_rule,
  /*  906 */  regF_rule,
  /*  907 */  0,
  /*  908 */  0,
  /*  909 */  0,
  /*  910 */  0,
  /*  911 */  0,
  /*  912 */  0,
  /*  913 */  0,
  /*  914 */  0,
  /*  915 */  0,
  /*  916 */  0,
  /*  917 */  0,
  /*  918 */  0,
  /*  919 */  0,
  /*  920 */  0,
  /*  921 */  0,
  /*  922 */  0,
  /*  923 */  0,
  /*  924 */  0,
  /*  925 */  0,
  /*  926 */  0,
  /*  927 */  0,
  /*  928 */  0,
  /*  929 */  0,
  /*  930 */  0,
  /*  931 */  0,
  /*  932 */  0,
  /*  933 */  0,
  /*  934 */  0,
  /*  935 */  0,
  /*  936 */  0,
  /*  937 */  immL_32bits_rule,
  /*  938 */  immL_32bits_rule,
  /*  939 */  0,
  /*  940 */  0,
  /*  941 */  0,
  /*  942 */  0,
  /*  943 */  0,
  /*  944 */  0,
  /*  945 */  0,
  /*  946 */  0,
  /*  947 */  0,
  /*  948 */  0,
  /*  949 */  0,
  /*  950 */  0,
  /*  951 */  0,
  /*  952 */  eBDPRegL_rule,
  /*  953 */  eBDPRegL_rule,
  /*  954 */  eDIRegP_rule,
  /*  955 */  eDIRegP_rule,
  /*  956 */  eDIRegP_rule,
  /*  957 */  eDIRegP_rule,
  /*  958 */  eRegP_rule,
  /*  959 */  _Binary_eSIRegP_eDXRegI_rule,
  /*  960 */  _Binary_eSIRegP_eDXRegI_rule,
  /*  961 */  _Binary_eSIRegP_eDXRegI_rule,
  /*  962 */  _Binary_eSIRegP_eDXRegI_rule,
  /*  963 */  _Binary_eSIRegP_eDXRegI_rule,
  /*  964 */  _Binary_eSIRegP_eDXRegI_rule,
  /*  965 */  _Binary_eDIRegP_eCXRegI_rule,
  /*  966 */  _Binary_eDIRegP_eCXRegI_rule,
  /*  967 */  eCXRegI_rule,
  /*  968 */  eCXRegI_rule,
  /*  969 */  _Binary_eSIRegP_immI_rule,
  /*  970 */  _Binary_eSIRegP_immI_rule,
  /*  971 */  _Binary_eSIRegP_immI_rule,
  /*  972 */  _Binary_eSIRegP_eAXRegI_rule,
  /*  973 */  _Binary_eSIRegP_eAXRegI_rule,
  /*  974 */  _Binary_eSIRegP_eAXRegI_rule,
  /*  975 */  eAXRegI_rule,
  /*  976 */  eAXRegI_rule,
  /*  977 */  eSIRegP_rule,
  /*  978 */  eSIRegP_rule,
  /*  979 */  eSIRegP_rule,
  /*  980 */  eSIRegP_rule,
  /*  981 */  eCXRegI_rule,
  /*  982 */  eCXRegI_rule,
  /*  983 */  _Binary_eDIRegP_eDXRegI_rule,
  /*  984 */  _Binary_eDIRegP_eDXRegI_rule,
  /*  985 */  _Binary_eDIRegP_eDXRegI_rule,
  /*  986 */  _Binary_eDIRegP_eDXRegI_rule,
  /*  987 */  _Binary_eDIRegP_eDXRegI_rule,
  /*  988 */  _Binary_eDIRegP_eDXRegI_rule,
  /*  989 */  rRegI_rule,
  /*  990 */  rRegI_rule,
  /*  991 */  immI_rule,
  /*  992 */  0,
  /*  993 */  0,
  /*  994 */  eFlagsReg_rule,
  /*  995 */  eFlagsReg_rule,
  /*  996 */  eFlagsRegU_rule,
  /*  997 */  eFlagsRegUCF_rule,
  /*  998 */  eFlagsRegUCF_rule,
  /*  999 */  eAXRegP_rule,
  /* 1000 */  0,
  /* 1001 */  eFlagsReg_rule,
  /* 1002 */  eFlagsReg_rule,
  /* 1003 */  eFlagsRegU_rule,
  /* 1004 */  eFlagsRegUCF_rule,
  /* 1005 */  eFlagsRegUCF_rule,
  /* 1006 */  eRegL_rule,
  /* 1007 */  flagsReg_long_LTGE_rule,
  /* 1008 */  flagsReg_ulong_LTGE_rule,
  /* 1009 */  _Binary_eRegL_eRegL_rule,
  /* 1010 */  _Binary_eRegL__LoadL_load_long_memory__rule,
  /* 1011 */  _Binary_eRegL_eRegL_rule,
  /* 1012 */  _Binary_eRegL__LoadL_load_long_memory__rule,
  /* 1013 */  _Binary_rRegI_rRegI_rule,
  /* 1014 */  _Binary_rRegI__LoadI_memory__rule,
  /* 1015 */  _Binary_rRegI_rRegI_rule,
  /* 1016 */  _Binary_rRegI__LoadI_memory__rule,
  /* 1017 */  _Binary_eRegP_eRegP_rule,
  /* 1018 */  _Binary_eRegP_eRegP_rule,
  /* 1019 */  _Binary_regDPR_regDPR_rule,
  /* 1020 */  _Binary_regD_regD_rule,
  /* 1021 */  _Binary_regFPR_regFPR_rule,
  /* 1022 */  _Binary_regF_regF_rule,
  /* 1023 */  flagsReg_long_EQNE_rule,
  /* 1024 */  flagsReg_ulong_EQNE_rule,
  /* 1025 */  _Binary_eRegL_eRegL_rule,
  /* 1026 */  _Binary_eRegL__LoadL_load_long_memory__rule,
  /* 1027 */  _Binary_rRegI_rRegI_rule,
  /* 1028 */  _Binary_rRegI__LoadI_memory__rule,
  /* 1029 */  _Binary_rRegI_rRegI_rule,
  /* 1030 */  _Binary_rRegI__LoadI_memory__rule,
  /* 1031 */  _Binary_eRegP_eRegP_rule,
  /* 1032 */  _Binary_eRegP_eRegP_rule,
  /* 1033 */  _Binary_regDPR_regDPR_rule,
  /* 1034 */  _Binary_regD_regD_rule,
  /* 1035 */  _Binary_regFPR_regFPR_rule,
  /* 1036 */  _Binary_regF_regF_rule,
  /* 1037 */  flagsReg_long_LEGT_rule,
  /* 1038 */  flagsReg_ulong_LEGT_rule,
  /* 1039 */  _Binary_eRegL_eRegL_rule,
  /* 1040 */  _Binary_eRegL__LoadL_load_long_memory__rule,
  /* 1041 */  _Binary_eRegL_eRegL_rule,
  /* 1042 */  _Binary_eRegL__LoadL_load_long_memory__rule,
  /* 1043 */  _Binary_rRegI_rRegI_rule,
  /* 1044 */  _Binary_rRegI__LoadI_memory__rule,
  /* 1045 */  _Binary_rRegI_rRegI_rule,
  /* 1046 */  _Binary_rRegI__LoadI_memory__rule,
  /* 1047 */  _Binary_eRegP_eRegP_rule,
  /* 1048 */  _Binary_eRegP_eRegP_rule,
  /* 1049 */  _Binary_regDPR_regDPR_rule,
  /* 1050 */  _Binary_regD_regD_rule,
  /* 1051 */  _Binary_regFPR_regFPR_rule,
  /* 1052 */  _Binary_regF_regF_rule,
  /* 1053 */  0,
  /* 1054 */  0,
  /* 1055 */  0,
  /* 1056 */  0,
  /* 1057 */  0,
  /* 1058 */  0,
  /* 1059 */  eBXRegP_rule,
  /* 1060 */  eAXRegP_rule,
  /* 1061 */  0,
  /* 1062 */  0,
  /* 1063 */  0,
  /* 1064 */  0,
  /* 1065 */  0,
  /* 1066 */  0,
  /* 1067 */  0,
  /* 1068 */  0,
  /* 1069 */  regF_rule,
  /* 1070 */  _LoadF_memory__rule,
  /* 1071 */  regF_rule,
  /* 1072 */  immF_rule,
  /* 1073 */  regF_rule,
  /* 1074 */  _LoadF_memory__rule,
  /* 1075 */  regF_rule,
  /* 1076 */  immF_rule,
  /* 1077 */  regD_rule,
  /* 1078 */  _LoadD_memory__rule,
  /* 1079 */  regD_rule,
  /* 1080 */  immD_rule,
  /* 1081 */  regD_rule,
  /* 1082 */  _LoadD_memory__rule,
  /* 1083 */  regD_rule,
  /* 1084 */  immD_rule,
  /* 1085 */  regF_rule,
  /* 1086 */  _LoadF_memory__rule,
  /* 1087 */  immF_rule,
  /* 1088 */  regF_rule,
  /* 1089 */  _LoadF_memory__rule,
  /* 1090 */  immF_rule,
  /* 1091 */  regD_rule,
  /* 1092 */  _LoadD_memory__rule,
  /* 1093 */  immD_rule,
  /* 1094 */  regD_rule,
  /* 1095 */  _LoadD_memory__rule,
  /* 1096 */  immD_rule,
  /* 1097 */  regF_rule,
  /* 1098 */  _LoadF_memory__rule,
  /* 1099 */  regF_rule,
  /* 1100 */  immF_rule,
  /* 1101 */  regF_rule,
  /* 1102 */  _LoadF_memory__rule,
  /* 1103 */  regF_rule,
  /* 1104 */  immF_rule,
  /* 1105 */  regD_rule,
  /* 1106 */  _LoadD_memory__rule,
  /* 1107 */  regD_rule,
  /* 1108 */  immD_rule,
  /* 1109 */  regD_rule,
  /* 1110 */  _LoadD_memory__rule,
  /* 1111 */  regD_rule,
  /* 1112 */  immD_rule,
  /* 1113 */  regF_rule,
  /* 1114 */  _LoadF_memory__rule,
  /* 1115 */  immF_rule,
  /* 1116 */  regF_rule,
  /* 1117 */  _LoadF_memory__rule,
  /* 1118 */  immF_rule,
  /* 1119 */  regD_rule,
  /* 1120 */  _LoadD_memory__rule,
  /* 1121 */  immD_rule,
  /* 1122 */  regD_rule,
  /* 1123 */  _LoadD_memory__rule,
  /* 1124 */  immD_rule,
  /* 1125 */  0,
  /* 1126 */  0,
  /* 1127 */  0,
  /* 1128 */  _ConvF2HF_regF__rule,
  /* 1129 */  _VectorCastF2HF_vec__rule,
  /* 1130 */  0,
  /* 1131 */  0,
  /* 1132 */  0,
  /* 1133 */  0,
  /* 1134 */  0,
  /* 1135 */  _Binary_regD_regD_rule,
  /* 1136 */  _Binary_regF_regF_rule,
  /* 1137 */  0,
  /* 1138 */  vec_rule,
  /* 1139 */  legVec_rule,
  /* 1140 */  vec_rule,
  /* 1141 */  _Binary_vec_kReg_rule,
  /* 1142 */  _Binary_vec_vec_rule,
  /* 1143 */  _Binary_vec__Binary_vec_kReg_rule,
  /* 1144 */  0,
  /* 1145 */  0,
  /* 1146 */  immU8_rule,
  /* 1147 */  immU8_rule,
  /* 1148 */  immU8_rule,
  /* 1149 */  immU8_rule,
  /* 1150 */  immU8_rule,
  /* 1151 */  legVec_rule,
  /* 1152 */  legVec_rule,
  /* 1153 */  legVec_rule,
  /* 1154 */  legVec_rule,
  /* 1155 */  legVec_rule,
  /* 1156 */  legVec_rule,
  /* 1157 */  legVec_rule,
  /* 1158 */  vec_rule,
  /* 1159 */  vec_rule,
  /* 1160 */  vec_rule,
  /* 1161 */  vec_rule,
  /* 1162 */  legVec_rule,
  /* 1163 */  legVec_rule,
  /* 1164 */  vec_rule,
  /* 1165 */  vec_rule,
  /* 1166 */  vec_rule,
  /* 1167 */  vec_rule,
  /* 1168 */  legVec_rule,
  /* 1169 */  legVec_rule,
  /* 1170 */  legVec_rule,
  /* 1171 */  legVec_rule,
  /* 1172 */  legVec_rule,
  /* 1173 */  legVec_rule,
  /* 1174 */  legVec_rule,
  /* 1175 */  legVec_rule,
  /* 1176 */  legVec_rule,
  /* 1177 */  vec_rule,
  /* 1178 */  legVec_rule,
  /* 1179 */  legVec_rule,
  /* 1180 */  legVec_rule,
  /* 1181 */  legVec_rule,
  /* 1182 */  legVec_rule,
  /* 1183 */  legVec_rule,
  /* 1184 */  legVec_rule,
  /* 1185 */  legVec_rule,
  /* 1186 */  legVec_rule,
  /* 1187 */  legVec_rule,
  /* 1188 */  legVec_rule,
  /* 1189 */  legVec_rule,
  /* 1190 */  legVec_rule,
  /* 1191 */  legVec_rule,
  /* 1192 */  legVec_rule,
  /* 1193 */  legVec_rule,
  /* 1194 */  legVec_rule,
  /* 1195 */  vec_rule,
  /* 1196 */  vec_rule,
  /* 1197 */  _LoadVector_memory__rule,
  /* 1198 */  vec_rule,
  /* 1199 */  vec_rule,
  /* 1200 */  vec_rule,
  /* 1201 */  _LoadVector_memory__rule,
  /* 1202 */  vec_rule,
  /* 1203 */  vec_rule,
  /* 1204 */  vec_rule,
  /* 1205 */  _LoadVector_memory__rule,
  /* 1206 */  vec_rule,
  /* 1207 */  vec_rule,
  /* 1208 */  vec_rule,
  /* 1209 */  _LoadVector_memory__rule,
  /* 1210 */  vec_rule,
  /* 1211 */  vec_rule,
  /* 1212 */  vec_rule,
  /* 1213 */  _LoadVector_memory__rule,
  /* 1214 */  vec_rule,
  /* 1215 */  vec_rule,
  /* 1216 */  vec_rule,
  /* 1217 */  _LoadVector_memory__rule,
  /* 1218 */  vec_rule,
  /* 1219 */  vec_rule,
  /* 1220 */  vec_rule,
  /* 1221 */  _LoadVector_memory__rule,
  /* 1222 */  vec_rule,
  /* 1223 */  vec_rule,
  /* 1224 */  _LoadVector_memory__rule,
  /* 1225 */  vec_rule,
  /* 1226 */  vec_rule,
  /* 1227 */  _LoadVector_memory__rule,
  /* 1228 */  vec_rule,
  /* 1229 */  vec_rule,
  /* 1230 */  _LoadVector_memory__rule,
  /* 1231 */  vec_rule,
  /* 1232 */  vec_rule,
  /* 1233 */  _LoadVector_memory__rule,
  /* 1234 */  vec_rule,
  /* 1235 */  vec_rule,
  /* 1236 */  _LoadVector_memory__rule,
  /* 1237 */  vec_rule,
  /* 1238 */  vec_rule,
  /* 1239 */  vec_rule,
  /* 1240 */  vec_rule,
  /* 1241 */  vec_rule,
  /* 1242 */  _LoadVector_memory__rule,
  /* 1243 */  vec_rule,
  /* 1244 */  vec_rule,
  /* 1245 */  vec_rule,
  /* 1246 */  _LoadVector_memory__rule,
  /* 1247 */  vec_rule,
  /* 1248 */  vec_rule,
  /* 1249 */  _LoadVector_memory__rule,
  /* 1250 */  vec_rule,
  /* 1251 */  vec_rule,
  /* 1252 */  vec_rule,
  /* 1253 */  vec_rule,
  /* 1254 */  vec_rule,
  /* 1255 */  _LoadVector_memory__rule,
  /* 1256 */  vec_rule,
  /* 1257 */  vec_rule,
  /* 1258 */  vec_rule,
  /* 1259 */  _LoadVector_memory__rule,
  /* 1260 */  vec_rule,
  /* 1261 */  vec_rule,
  /* 1262 */  vec_rule,
  /* 1263 */  _LoadVector_memory__rule,
  /* 1264 */  vec_rule,
  /* 1265 */  vec_rule,
  /* 1266 */  _LoadVector_memory__rule,
  /* 1267 */  vec_rule,
  /* 1268 */  vec_rule,
  /* 1269 */  vec_rule,
  /* 1270 */  vec_rule,
  /* 1271 */  vec_rule,
  /* 1272 */  vec_rule,
  /* 1273 */  legVec_rule,
  /* 1274 */  legVec_rule,
  /* 1275 */  vec_rule,
  /* 1276 */  vec_rule,
  /* 1277 */  legVec_rule,
  /* 1278 */  legVec_rule,
  /* 1279 */  vec_rule,
  /* 1280 */  vec_rule,
  /* 1281 */  _Binary_regF_regF_rule,
  /* 1282 */  _Binary_regD_regD_rule,
  /* 1283 */  _Binary_vec_vec_rule,
  /* 1284 */  _Binary_vec_vec_rule,
  /* 1285 */  _Binary_vec_vec_rule,
  /* 1286 */  _Binary_vec_vec_rule,
  /* 1287 */  rRegI_rule,
  /* 1288 */  rRegI_rule,
  /* 1289 */  _LoadI_memory__rule,
  /* 1290 */  _LoadI_memory__rule,
  /* 1291 */  vec_rule,
  /* 1292 */  vec_rule,
  /* 1293 */  vec_rule,
  /* 1294 */  vec_rule,
  /* 1295 */  vec_rule,
  /* 1296 */  vec_rule,
  /* 1297 */  vec_rule,
  /* 1298 */  vec_rule,
  /* 1299 */  vec_rule,
  /* 1300 */  vec_rule,
  /* 1301 */  vec_rule,
  /* 1302 */  vec_rule,
  /* 1303 */  vec_rule,
  /* 1304 */  vec_rule,
  /* 1305 */  vec_rule,
  /* 1306 */  vec_rule,
  /* 1307 */  vec_rule,
  /* 1308 */  vec_rule,
  /* 1309 */  vec_rule,
  /* 1310 */  vec_rule,
  /* 1311 */  vec_rule,
  /* 1312 */  _LShiftCntV_immI8__rule,
  /* 1313 */  _RShiftCntV_immI8__rule,
  /* 1314 */  _RShiftCntV_immI8__rule,
  /* 1315 */  vec_rule,
  /* 1316 */  vec_rule,
  /* 1317 */  _LShiftCntV_immI8__rule,
  /* 1318 */  _RShiftCntV_immI8__rule,
  /* 1319 */  vec_rule,
  /* 1320 */  vec_rule,
  /* 1321 */  vec_rule,
  /* 1322 */  vec_rule,
  /* 1323 */  vec_rule,
  /* 1324 */  vec_rule,
  /* 1325 */  vec_rule,
  /* 1326 */  vec_rule,
  /* 1327 */  vec_rule,
  /* 1328 */  vec_rule,
  /* 1329 */  vec_rule,
  /* 1330 */  vec_rule,
  /* 1331 */  vec_rule,
  /* 1332 */  vec_rule,
  /* 1333 */  vec_rule,
  /* 1334 */  vec_rule,
  /* 1335 */  vec_rule,
  /* 1336 */  vec_rule,
  /* 1337 */  vec_rule,
  /* 1338 */  vec_rule,
  /* 1339 */  vec_rule,
  /* 1340 */  vec_rule,
  /* 1341 */  vec_rule,
  /* 1342 */  vec_rule,
  /* 1343 */  vec_rule,
  /* 1344 */  vec_rule,
  /* 1345 */  vec_rule,
  /* 1346 */  vec_rule,
  /* 1347 */  vec_rule,
  /* 1348 */  vec_rule,
  /* 1349 */  vec_rule,
  /* 1350 */  vec_rule,
  /* 1351 */  vec_rule,
  /* 1352 */  vec_rule,
  /* 1353 */  vec_rule,
  /* 1354 */  _LoadVector_memory__rule,
  /* 1355 */  vec_rule,
  /* 1356 */  vec_rule,
  /* 1357 */  vec_rule,
  /* 1358 */  _LoadVector_memory__rule,
  /* 1359 */  vec_rule,
  /* 1360 */  vec_rule,
  /* 1361 */  vec_rule,
  /* 1362 */  _LoadVector_memory__rule,
  /* 1363 */  vec_rule,
  /* 1364 */  0,
  /* 1365 */  0,
  /* 1366 */  0,
  /* 1367 */  0,
  /* 1368 */  0,
  /* 1369 */  0,
  /* 1370 */  immI8_rule,
  /* 1371 */  immI8_rule,
  /* 1372 */  immI8_rule,
  /* 1373 */  immI8_rule,
  /* 1374 */  immI8_rule,
  /* 1375 */  immI8_rule,
  /* 1376 */  immI8_rule,
  /* 1377 */  immI8_rule,
  /* 1378 */  immU8_rule,
  /* 1379 */  immU8_rule,
  /* 1380 */  immI_rule,
  /* 1381 */  immI_rule,
  /* 1382 */  immU8_rule,
  /* 1383 */  immU8_rule,
  /* 1384 */  immU8_rule,
  /* 1385 */  immU8_rule,
  /* 1386 */  vec_rule,
  /* 1387 */  legVec_rule,
  /* 1388 */  legVec_rule,
  /* 1389 */  vec_rule,
  /* 1390 */  kReg_rule,
  /* 1391 */  0,
  /* 1392 */  0,
  /* 1393 */  0,
  /* 1394 */  immI_1_rule,
  /* 1395 */  immI_2_rule,
  /* 1396 */  immI_4_rule,
  /* 1397 */  immI_8_rule,
  /* 1398 */  immI_8_rule,
  /* 1399 */  immI_4_rule,
  /* 1400 */  immI_8_rule,
  /* 1401 */  immI_rule,
  /* 1402 */  immI_rule,
  /* 1403 */  vec_rule,
  /* 1404 */  vec_rule,
  /* 1405 */  vec_rule,
  /* 1406 */  vec_rule,
  /* 1407 */  0,
  /* 1408 */  vec_rule,
  /* 1409 */  vec_rule,
  /* 1410 */  vec_rule,
  /* 1411 */  0,
  /* 1412 */  vec_rule,
  /* 1413 */  vec_rule,
  /* 1414 */  0,
  /* 1415 */  vec_rule,
  /* 1416 */  vec_rule,
  /* 1417 */  _Binary_vec_vec_rule,
  /* 1418 */  _Binary_vec__LoadVector_memory__rule,
  /* 1419 */  _Binary_vec_vec_rule,
  /* 1420 */  _Binary_vec__LoadVector_memory__rule,
  /* 1421 */  vec_rule,
  /* 1422 */  vec_rule,
  /* 1423 */  vec_rule,
  /* 1424 */  _MulAddVS2VI_vec_vec_rule,
  /* 1425 */  kReg_rule,
  /* 1426 */  kReg_rule,
  /* 1427 */  0,
  /* 1428 */  0,
  /* 1429 */  0,
  /* 1430 */  0,
  /* 1431 */  0,
  /* 1432 */  0,
  /* 1433 */  _Binary_vec_immU8_rule,
  /* 1434 */  _Binary__LoadVector_memory__immU8_rule,
  /* 1435 */  immI8_rule,
  /* 1436 */  immI8_rule,
  /* 1437 */  vec_rule,
  /* 1438 */  vec_rule,
  /* 1439 */  vec_rule,
  /* 1440 */  kReg_rule,
  /* 1441 */  _Binary_vec_vec_rule,
  /* 1442 */  _Binary_vec_kReg_rule,
  /* 1443 */  0,
  /* 1444 */  0,
  /* 1445 */  0,
  /* 1446 */  0,
  /* 1447 */  kReg_rule,
  /* 1448 */  0,
  /* 1449 */  0,
  /* 1450 */  0,
  /* 1451 */  0,
  /* 1452 */  kReg_rule,
  /* 1453 */  kReg_rule,
  /* 1454 */  kReg_rule,
  /* 1455 */  kReg_rule,
  /* 1456 */  kReg_rule,
  /* 1457 */  kReg_rule,
  /* 1458 */  kReg_rule,
  /* 1459 */  kReg_rule,
  /* 1460 */  kReg_rule,
  /* 1461 */  kReg_rule,
  /* 1462 */  kReg_rule,
  /* 1463 */  kReg_rule,
  /* 1464 */  kReg_rule,
  /* 1465 */  kReg_rule,
  /* 1466 */  kReg_rule,
  /* 1467 */  kReg_rule,
  /* 1468 */  kReg_rule,
  /* 1469 */  kReg_rule,
  /* 1470 */  kReg_rule,
  /* 1471 */  kReg_rule,
  /* 1472 */  kReg_rule,
  /* 1473 */  kReg_rule,
  /* 1474 */  kReg_rule,
  /* 1475 */  kReg_rule,
  /* 1476 */  kReg_rule,
  /* 1477 */  kReg_rule,
  /* 1478 */  kReg_rule,
  /* 1479 */  kReg_rule,
  /* 1480 */  kReg_rule,
  /* 1481 */  kReg_rule,
  /* 1482 */  kReg_rule,
  /* 1483 */  kReg_rule,
  /* 1484 */  kReg_rule,
  /* 1485 */  kReg_rule,
  /* 1486 */  kReg_rule,
  /* 1487 */  kReg_rule,
  /* 1488 */  kReg_rule,
  /* 1489 */  kReg_rule,
  /* 1490 */  kReg_rule,
  /* 1491 */  kReg_rule,
  /* 1492 */  kReg_rule,
  /* 1493 */  kReg_rule,
  /* 1494 */  kReg_rule,
  /* 1495 */  kReg_rule,
  /* 1496 */  kReg_rule,
  /* 1497 */  kReg_rule,
  /* 1498 */  kReg_rule,
  /* 1499 */  kReg_rule,
  /* 1500 */  kReg_rule,
  /* 1501 */  kReg_rule,
  /* 1502 */  kReg_rule,
  /* 1503 */  kReg_rule,
  /* 1504 */  kReg_rule,
  /* 1505 */  kReg_rule,
  /* 1506 */  kReg_rule,
  /* 1507 */  kReg_rule,
  /* 1508 */  kReg_rule,
  /* 1509 */  kReg_rule,
  /* 1510 */  kReg_rule,
  /* 1511 */  kReg_rule,
  /* 1512 */  kReg_rule,
  /* 1513 */  kReg_rule,
  /* 1514 */  kReg_rule,
  /* 1515 */  kReg_rule,
  /* 1516 */  kReg_rule,
  /* 1517 */  kReg_rule,
  /* 1518 */  kReg_rule,
  /* 1519 */  kReg_rule,
  /* 1520 */  kReg_rule,
  /* 1521 */  kReg_rule,
  /* 1522 */  kReg_rule,
  /* 1523 */  kReg_rule,
  /* 1524 */  kReg_rule,
  /* 1525 */  kReg_rule,
  /* 1526 */  kReg_rule,
  /* 1527 */  kReg_rule,
  /* 1528 */  kReg_rule,
  /* 1529 */  kReg_rule,
  /* 1530 */  kReg_rule,
  /* 1531 */  kReg_rule,
  /* 1532 */  kReg_rule,
  /* 1533 */  kReg_rule,
  /* 1534 */  kReg_rule,
  /* 1535 */  kReg_rule,
  /* 1536 */  kReg_rule,
  /* 1537 */  kReg_rule,
  /* 1538 */  kReg_rule,
  /* 1539 */  kReg_rule,
  /* 1540 */  kReg_rule,
  /* 1541 */  kReg_rule,
  /* 1542 */  kReg_rule,
  /* 1543 */  kReg_rule,
  /* 1544 */  kReg_rule,
  /* 1545 */  kReg_rule,
  /* 1546 */  kReg_rule,
  /* 1547 */  _Binary_vec_kReg_rule,
  /* 1548 */  _Binary_vec_kReg_rule,
  /* 1549 */  _Binary__LoadVector_memory__kReg_rule,
  /* 1550 */  _Binary__LoadVector_memory__kReg_rule,
  /* 1551 */  _Binary_immI8_kReg_rule,
  /* 1552 */  0,
  /* 1553 */  kReg_rule,
  /* 1554 */  kReg_rule,
  /* 1555 */  kReg_rule,
  /* 1556 */  _Binary_vec__Binary_vec__Binary_immU8_kReg_rule,
  /* 1557 */  _Binary_vec__Binary_memory__Binary_immU8_kReg_rule,
  /* 1558 */  0,
  /* 1559 */  0,
  /* 1560 */  _Binary_eAXRegP_eRegP_rule,
  /* 1561 */  _Binary_eAXRegP_eRegP_rule,
  /* 1562 */  _Binary_eAXRegP_eRegP_rule,
  // last instruction
  0 // no trailing comma
};

const char        *ruleName[] = {
  /*    0 */  "UNIVERSE",
  /*    1 */  "LABEL",
  /*    2 */  "SREGI",
  /*    3 */  "SREGP",
  /*    4 */  "SREGF",
  /*    5 */  "SREGD",
  /*    6 */  "SREGL",
  /*    7 */  "METHOD",
  /*    8 */  "IMMI",
  /*    9 */  "IMMI_0",
  /*   10 */  "IMMI_1",
  /*   11 */  "IMMI_M1",
  /*   12 */  "IMMI2",
  /*   13 */  "IMMI8",
  /*   14 */  "IMMU8",
  /*   15 */  "IMMI16",
  /*   16 */  "IMMU31",
  /*   17 */  "IMMI_32",
  /*   18 */  "IMMI_1_31",
  /*   19 */  "IMMI_32_63",
  /*   20 */  "IMMI_2",
  /*   21 */  "IMMI_3",
  /*   22 */  "IMMI_4",
  /*   23 */  "IMMI_8",
  /*   24 */  "IMMP",
  /*   25 */  "IMMP0",
  /*   26 */  "IMML",
  /*   27 */  "IMML0",
  /*   28 */  "IMML_M1",
  /*   29 */  "IMML_127",
  /*   30 */  "IMML_32BITS",
  /*   31 */  "IMML32",
  /*   32 */  "IMMDPR0",
  /*   33 */  "IMMDPR1",
  /*   34 */  "IMMDPR",
  /*   35 */  "IMMD",
  /*   36 */  "IMMD0",
  /*   37 */  "IMMFPR0",
  /*   38 */  "IMMFPR1",
  /*   39 */  "IMMFPR",
  /*   40 */  "IMMF",
  /*   41 */  "IMMF0",
  /*   42 */  "IMMI_16",
  /*   43 */  "IMMI_24",
  /*   44 */  "IMMI_255",
  /*   45 */  "IMMI_65535",
  /*   46 */  "KREG",
  /*   47 */  "KREG_K1",
  /*   48 */  "KREG_K2",
  /*   49 */  "KREG_K3",
  /*   50 */  "KREG_K4",
  /*   51 */  "KREG_K5",
  /*   52 */  "KREG_K6",
  /*   53 */  "KREG_K7",
  /*   54 */  "RREGI",
  /*   55 */  "XREGI",
  /*   56 */  "EAXREGI",
  /*   57 */  "EBXREGI",
  /*   58 */  "ECXREGI",
  /*   59 */  "EDXREGI",
  /*   60 */  "EDIREGI",
  /*   61 */  "NAXREGI",
  /*   62 */  "NADXREGI",
  /*   63 */  "NCXREGI",
  /*   64 */  "ESIREGI",
  /*   65 */  "ANYREGP",
  /*   66 */  "EREGP",
  /*   67 */  "RREGP",
  /*   68 */  "EREGP_NO_EBP",
  /*   69 */  "NAXREGP",
  /*   70 */  "NABXREGP",
  /*   71 */  "PREGP",
  /*   72 */  "EAXREGP",
  /*   73 */  "EBXREGP",
  /*   74 */  "ECXREGP",
  /*   75 */  "EDXREGP",
  /*   76 */  "ESIREGP",
  /*   77 */  "EDIREGP",
  /*   78 */  "EREGL",
  /*   79 */  "EADXREGL",
  /*   80 */  "EBCXREGL",
  /*   81 */  "EBDPREGL",
  /*   82 */  "EADXREGL_LOW_ONLY",
  /*   83 */  "RFLAGSREG",
  /*   84 */  "EFLAGSREG",
  /*   85 */  "EFLAGSREGU",
  /*   86 */  "EFLAGSREGUCF",
  /*   87 */  "FLAGSREG_LONG_LTGE",
  /*   88 */  "FLAGSREG_LONG_EQNE",
  /*   89 */  "FLAGSREG_LONG_LEGT",
  /*   90 */  "FLAGSREG_ULONG_LTGE",
  /*   91 */  "FLAGSREG_ULONG_EQNE",
  /*   92 */  "FLAGSREG_ULONG_LEGT",
  /*   93 */  "REGDPR",
  /*   94 */  "REGDPR1",
  /*   95 */  "REGDPR2",
  /*   96 */  "REGNOTDPR1",
  /*   97 */  "REGFPR",
  /*   98 */  "REGFPR1",
  /*   99 */  "REGF",
  /*  100 */  "LEGREGF",
  /*  101 */  "VLREGF",
  /*  102 */  "REGD",
  /*  103 */  "LEGREGD",
  /*  104 */  "VLREGD",
  /*  105 */  "DIRECT",
  /*  106 */  "INDIRECT",
  /*  107 */  "INDOFFSET8",
  /*  108 */  "INDOFFSET32",
  /*  109 */  "INDOFFSET32X",
  /*  110 */  "INDINDEXOFFSET",
  /*  111 */  "INDINDEX",
  /*  112 */  "INDINDEXSCALE",
  /*  113 */  "INDINDEXSCALEOFFSET",
  /*  114 */  "LOAD_LONG_REGP",
  /*  115 */  "LOAD_LONG_INDIRECT",
  /*  116 */  "LOAD_LONG_INDOFFSET32",
  /*  117 */  "STACKSLOTP",
  /*  118 */  "STACKSLOTI",
  /*  119 */  "STACKSLOTF",
  /*  120 */  "STACKSLOTD",
  /*  121 */  "STACKSLOTL",
  /*  122 */  "CMPOP",
  /*  123 */  "CMPOPU",
  /*  124 */  "CMPOPUCF",
  /*  125 */  "CMPOPUCF2",
  /*  126 */  "CMPOP_FCMOV",
  /*  127 */  "CMPOP_COMMUTE",
  /*  128 */  "CMPOPU_COMMUTE",
  /*  129 */  "RXMM0",
  /*  130 */  "VEC",
  /*  131 */  "LEGVEC",
  /*  132 */  "VECS",
  /*  133 */  "LEGVECS",
  /*  134 */  "VECD",
  /*  135 */  "LEGVECD",
  /*  136 */  "VECX",
  /*  137 */  "LEGVECX",
  /*  138 */  "VECY",
  /*  139 */  "LEGVECY",
  /*  140 */  "VECZ",
  /*  141 */  "LEGVECZ",
  // last operand
  /*  142 */  "LOAD_LONG_MEMORY",
  /*  143 */  "MEMORY",
  /*  144 */  "LONG_MEMORY",
  // last operand class
  /*  145 */  "_AddP_eRegP_rRegI",
  /*  146 */  "_LShiftI_rRegI_immI2",
  /*  147 */  "_AddP_eRegP__LShiftI_rRegI_immI2",
  /*  148 */  "_LoadI_memory_",
  /*  149 */  "_LoadL_memory_",
  /*  150 */  "_LoadB_memory_",
  /*  151 */  "_LoadUB_memory_",
  /*  152 */  "_AndI__LoadUB_memory__immI",
  /*  153 */  "_LoadS_memory_",
  /*  154 */  "_LShiftI__LoadS_memory__immI_24",
  /*  155 */  "_LoadUS_memory_",
  /*  156 */  "_LShiftI__LoadUS_memory__immI_24",
  /*  157 */  "_AndI__LoadUS_memory__immI_255",
  /*  158 */  "_AndI__LoadUS_memory__immI",
  /*  159 */  "_LShiftI__LoadI_memory__immI_24",
  /*  160 */  "_LShiftI__LoadI_memory__immI_16",
  /*  161 */  "_AndI__LoadI_memory__immI_255",
  /*  162 */  "_AndI__LoadI_memory__immI_65535",
  /*  163 */  "_AndI__LoadI_memory__immU31",
  /*  164 */  "_ConvI2L__LoadI_memory__",
  /*  165 */  "_ConvL2I_eRegL_",
  /*  166 */  "_RoundDouble_regDPR1_",
  /*  167 */  "_RoundFloat_regFPR1_",
  /*  168 */  "_ConvD2F_regDPR1_",
  /*  169 */  "_Binary_cmpOp_eFlagsReg",
  /*  170 */  "_Binary_rRegI_rRegI",
  /*  171 */  "_Binary_cmpOpU_eFlagsRegU",
  /*  172 */  "_Binary_cmpOpUCF_eFlagsRegUCF",
  /*  173 */  "_Binary_rRegI__LoadI_memory_",
  /*  174 */  "_Binary_eRegP_eRegP",
  /*  175 */  "_Binary_cmpOp_fcmov_eFlagsRegU",
  /*  176 */  "_Binary_regDPR1_regDPR",
  /*  177 */  "_Binary_regFPR1_regFPR",
  /*  178 */  "_Binary_regDPR_regDPR",
  /*  179 */  "_Binary_regFPR_regFPR",
  /*  180 */  "_Binary_regF_regF",
  /*  181 */  "_Binary_regD_regD",
  /*  182 */  "_Binary_eRegL_eRegL",
  /*  183 */  "_AddI__LoadI_memory__rRegI",
  /*  184 */  "_AddI_rRegI__LoadI_memory_",
  /*  185 */  "_AddI__LoadI_memory__immI",
  /*  186 */  "_AddI__LoadI_memory__immI_1",
  /*  187 */  "_AddI__LoadI_memory__immI_M1",
  /*  188 */  "_Binary_eADXRegL_eBCXRegL",
  /*  189 */  "_Binary_eAXRegP_eCXRegP",
  /*  190 */  "_Binary_eAXRegI_eCXRegI",
  /*  191 */  "_SubI__LoadI_memory__rRegI",
  /*  192 */  "_SubI_immI_0_rRegI",
  /*  193 */  "_ConvI2L_nadxRegI_",
  /*  194 */  "_MulL__ConvI2L_nadxRegI__eADXRegL_low_only",
  /*  195 */  "_RShiftL__MulL__ConvI2L_nadxRegI__eADXRegL_low_only_immI_32",
  /*  196 */  "_MulL_eADXRegL_low_only__ConvI2L_nadxRegI_",
  /*  197 */  "_RShiftL__MulL_eADXRegL_low_only__ConvI2L_nadxRegI__immI_32",
  /*  198 */  "_RShiftL__MulL__ConvI2L_nadxRegI__eADXRegL_low_only_immI_32_63",
  /*  199 */  "_RShiftL__MulL_eADXRegL_low_only__ConvI2L_nadxRegI__immI_32_63",
  /*  200 */  "_ConvI2L_eAXRegI_",
  /*  201 */  "_AndL__ConvI2L_eAXRegI__immL_32bits",
  /*  202 */  "_AndL__ConvI2L_nadxRegI__immL_32bits",
  /*  203 */  "_RShiftI__LoadI_memory__immI_1",
  /*  204 */  "_RShiftI__LoadI_memory__immI8",
  /*  205 */  "_LShiftI_xRegI_immI_24",
  /*  206 */  "_LShiftI_xRegI_immI_16",
  /*  207 */  "_AndI__LoadI_memory__rRegI",
  /*  208 */  "_AndI_rRegI__LoadI_memory_",
  /*  209 */  "_AndI__LoadI_memory__immI",
  /*  210 */  "_XorI_rRegI_immI_M1",
  /*  211 */  "_SubI_immI_0__LoadI_memory_",
  /*  212 */  "_AddI_rRegI_immI_M1",
  /*  213 */  "_CastP2X_eRegP_",
  /*  214 */  "_OrI__LoadI_memory__rRegI",
  /*  215 */  "_OrI_rRegI__LoadI_memory_",
  /*  216 */  "_OrI__LoadI_memory__immI",
  /*  217 */  "_LShiftI_rRegI_immI_1",
  /*  218 */  "_URShiftI_rRegI_immI_M1",
  /*  219 */  "_LShiftI_rRegI_immI8",
  /*  220 */  "_URShiftI_rRegI_immI8",
  /*  221 */  "_LShiftI_ncxRegI_eCXRegI",
  /*  222 */  "_SubI_immI_0_eCXRegI",
  /*  223 */  "_URShiftI_ncxRegI__SubI_immI_0_eCXRegI",
  /*  224 */  "_SubI_immI_32_eCXRegI",
  /*  225 */  "_URShiftI_ncxRegI__SubI_immI_32_eCXRegI",
  /*  226 */  "_URShiftI_rRegI_immI_1",
  /*  227 */  "_LShiftI_rRegI_immI_M1",
  /*  228 */  "_URShiftI_ncxRegI_eCXRegI",
  /*  229 */  "_LShiftI_ncxRegI__SubI_immI_0_eCXRegI",
  /*  230 */  "_LShiftI_ncxRegI__SubI_immI_32_eCXRegI",
  /*  231 */  "_XorI__LoadI_memory__rRegI",
  /*  232 */  "_XorI_rRegI__LoadI_memory_",
  /*  233 */  "_XorI__LoadI_memory__immI",
  /*  234 */  "_CmpLTMask_rRegI_rRegI",
  /*  235 */  "_AndI__CmpLTMask_rRegI_rRegI_rRegI",
  /*  236 */  "_SubI_rRegI_rRegI",
  /*  237 */  "_AndI_rRegI__CmpLTMask_rRegI_rRegI",
  /*  238 */  "_LoadL_load_long_memory_",
  /*  239 */  "_XorL_eRegL_immL_M1",
  /*  240 */  "_SubL_immL0_eRegL",
  /*  241 */  "_SubL_immL0__LoadL_memory_",
  /*  242 */  "_AddL_eRegL_immL_M1",
  /*  243 */  "_AddL__LoadL_memory__immL_M1",
  /*  244 */  "_LoadD_memory_",
  /*  245 */  "_SubD_regDPR_regDPR",
  /*  246 */  "_AddD_regDPR_regDPR",
  /*  247 */  "_AddD__LoadD_memory__regDPR",
  /*  248 */  "_RoundDouble__AddD__LoadD_memory__regDPR_",
  /*  249 */  "_AddD_regDPR__LoadD_memory_",
  /*  250 */  "_RoundDouble__AddD_regDPR__LoadD_memory__",
  /*  251 */  "_AddD_regDPR_immDPR",
  /*  252 */  "_MulD_regDPR_regDPR",
  /*  253 */  "_LoadF_memory_",
  /*  254 */  "_MulF__LoadF_memory__regFPR",
  /*  255 */  "_MulF_regFPR__LoadF_memory_",
  /*  256 */  "_MulF_regFPR_regFPR",
  /*  257 */  "_SubF_regFPR_regFPR",
  /*  258 */  "_ConvI2L_rRegI_",
  /*  259 */  "_Binary_eDIRegP_eCXRegI",
  /*  260 */  "_Binary_eSIRegP_eDXRegI",
  /*  261 */  "_Binary_eDIRegP_eSIRegP",
  /*  262 */  "_Binary_eDIRegP_eDXRegI",
  /*  263 */  "_Binary_eSIRegP_immI",
  /*  264 */  "_Binary_eSIRegP_eAXRegI",
  /*  265 */  "_AndI_rRegI_immI",
  /*  266 */  "_AndI_rRegI_memory",
  /*  267 */  "_LoadP_memory_",
  /*  268 */  "_Binary_nadxRegI_eAXRegI",
  /*  269 */  "_PartialSubtypeCheck_eSIRegP_eAXRegP",
  /*  270 */  "_Binary_cmpOp_flagsReg_long_LTGE",
  /*  271 */  "_Binary_eRegL__LoadL_load_long_memory_",
  /*  272 */  "_Binary_cmpOpU_flagsReg_ulong_LTGE",
  /*  273 */  "_Binary_cmpOp_flagsReg_long_EQNE",
  /*  274 */  "_Binary_cmpOpU_flagsReg_ulong_EQNE",
  /*  275 */  "_Binary_cmpOp_commute_flagsReg_long_LEGT",
  /*  276 */  "_Binary_cmpOpU_commute_flagsReg_ulong_LEGT",
  /*  277 */  "_ConvF2HF_regF_",
  /*  278 */  "_VectorCastF2HF_vec_",
  /*  279 */  "_LoadVector_memory_",
  /*  280 */  "_Binary_vec_kReg",
  /*  281 */  "_Binary_vec_vec",
  /*  282 */  "_Binary_vec__Binary_vec_kReg",
  /*  283 */  "_Binary_vec_rRegI",
  /*  284 */  "_Binary_vec_regF",
  /*  285 */  "_LShiftCntV_immI8_",
  /*  286 */  "_RShiftCntV_immI8_",
  /*  287 */  "_Binary_legVec_legVec",
  /*  288 */  "_Binary_vec__LoadVector_memory_",
  /*  289 */  "_MulAddVS2VI_vec_vec",
  /*  290 */  "_Binary_vec_immU8",
  /*  291 */  "_Binary__LoadVector_memory__immU8",
  /*  292 */  "_Binary_vec_immI8",
  /*  293 */  "_Binary_vec__LShiftCntV_immI8_",
  /*  294 */  "_Binary_vec__RShiftCntV_immI8_",
  /*  295 */  "_Binary__LoadVector_memory__kReg",
  /*  296 */  "_Binary_immI8_kReg",
  /*  297 */  "_Binary_immU8_kReg",
  /*  298 */  "_Binary_vec__Binary_immU8_kReg",
  /*  299 */  "_Binary_vec__Binary_vec__Binary_immU8_kReg",
  /*  300 */  "_Binary_memory__Binary_immU8_kReg",
  /*  301 */  "_Binary_vec__Binary_memory__Binary_immU8_kReg",
  /*  302 */  "_Binary_eAXRegP_eRegP",
  // last internally defined operand
  /*  303 */  "storeSSI",
  /*  304 */  "storeSSP",
  /*  305 */  "storeSSL",
  /*  306 */  "MoveF2LEG",
  /*  307 */  "MoveLEG2F",
  /*  308 */  "MoveF2VL",
  /*  309 */  "MoveVL2F",
  /*  310 */  "MoveD2LEG",
  /*  311 */  "MoveLEG2D",
  /*  312 */  "MoveD2VL",
  /*  313 */  "MoveVL2D",
  /*  314 */  "leaP8",
  /*  315 */  "leaP32",
  /*  316 */  "leaPIdxOff",
  /*  317 */  "leaPIdxScale",
  /*  318 */  "leaPIdxScaleOff",
  /*  319 */  "loadConI",
  /*  320 */  "loadConI0",
  /*  321 */  "loadConP",
  /*  322 */  "loadConL",
  /*  323 */  "loadConL0",
  /*  324 */  "loadConFPR",
  /*  325 */  "loadConFPR0",
  /*  326 */  "loadConFPR1",
  /*  327 */  "loadConF",
  /*  328 */  "loadConF0",
  /*  329 */  "loadConDPR",
  /*  330 */  "loadConDPR0",
  /*  331 */  "loadConDPR1",
  /*  332 */  "loadConD",
  /*  333 */  "loadConD0",
  /*  334 */  "loadSSI",
  /*  335 */  "loadSSL",
  /*  336 */  "loadSSP",
  /*  337 */  "loadSSF",
  /*  338 */  "loadSSD",
  /*  339 */  "loadConL_low_only",
  /*  340 */  "MoveVec2Leg",
  /*  341 */  "MoveLeg2Vec",
  /*  342 */  "membar_volatile",
  /*  343 */  "castX2P",
  /*  344 */  "castP2X",
  /*  345 */  "checkCastPP",
  /*  346 */  "castPP",
  /*  347 */  "castII",
  /*  348 */  "castLL",
  /*  349 */  "castFF",
  /*  350 */  "castDD",
  /*  351 */  "castFF_PR",
  /*  352 */  "castDD_PR",
  /*  353 */  "movI_nocopy",
  /*  354 */  "movP_nocopy",
  /*  355 */  "overflowAddI_eReg",
  /*  356 */  "overflowAddI_rReg_imm",
  /*  357 */  "overflowSubI_rReg",
  /*  358 */  "overflowSubI_rReg_imm",
  /*  359 */  "overflowNegI_rReg",
  /*  360 */  "overflowMulI_rReg",
  /*  361 */  "overflowMulI_rReg_imm",
  /*  362 */  "cmpDPR_cc_P6",
  /*  363 */  "cmpDPR_cc_P6CF",
  /*  364 */  "cmpDPR_cc",
  /*  365 */  "cmpD_cc",
  /*  366 */  "cmpD_ccCF",
  /*  367 */  "cmpD_ccmem",
  /*  368 */  "cmpD_ccmemCF",
  /*  369 */  "absDPR_reg",
  /*  370 */  "negDPR_reg",
  /*  371 */  "cmpFPR_cc_P6",
  /*  372 */  "cmpFPR_cc_P6CF",
  /*  373 */  "cmpFPR_cc",
  /*  374 */  "cmpF_cc",
  /*  375 */  "cmpF_ccCF",
  /*  376 */  "cmpF_ccmem",
  /*  377 */  "cmpF_ccmemCF",
  /*  378 */  "absFPR_reg",
  /*  379 */  "negFPR_reg",
  /*  380 */  "MoveF2I_stack_reg",
  /*  381 */  "MoveF2I_reg_reg_sse",
  /*  382 */  "MoveI2FPR_stack_reg",
  /*  383 */  "MoveI2F_stack_reg_sse",
  /*  384 */  "MoveI2F_reg_reg_sse",
  /*  385 */  "MoveD2L_stack_reg",
  /*  386 */  "MoveL2DPR_stack_reg",
  /*  387 */  "MoveL2D_stack_reg_sse",
  /*  388 */  "MoveL2D_stack_reg_sse_partial",
  /*  389 */  "compI_eReg",
  /*  390 */  "compI_eReg_imm",
  /*  391 */  "compI_eReg_mem",
  /*  392 */  "testI_reg",
  /*  393 */  "testI_reg_imm",
  /*  394 */  "testI_reg_mem",
  /*  395 */  "compU_eReg",
  /*  396 */  "compU_eReg_imm",
  /*  397 */  "compU_eReg_mem",
  /*  398 */  "testU_reg",
  /*  399 */  "compP_eReg",
  /*  400 */  "compP_eReg_imm",
  /*  401 */  "compP_eReg_mem",
  /*  402 */  "compP_mem_eReg",
  /*  403 */  "testP_reg",
  /*  404 */  "testP_Reg_mem",
  /*  405 */  "partialSubtypeCheck_vs_Zero",
  /*  406 */  "cmpL_zero_flags_LTGE",
  /*  407 */  "cmpL_reg_flags_LTGE",
  /*  408 */  "cmpUL_zero_flags_LTGE",
  /*  409 */  "cmpUL_reg_flags_LTGE",
  /*  410 */  "cmpL_zero_flags_EQNE",
  /*  411 */  "cmpL_reg_flags_EQNE",
  /*  412 */  "cmpUL_zero_flags_EQNE",
  /*  413 */  "cmpUL_reg_flags_EQNE",
  /*  414 */  "cmpL_zero_flags_LEGT",
  /*  415 */  "cmpL_reg_flags_LEGT",
  /*  416 */  "cmpUL_zero_flags_LEGT",
  /*  417 */  "cmpUL_reg_flags_LEGT",
  /*  418 */  "cmpFastLockRTM",
  /*  419 */  "cmpFastLock",
  /*  420 */  "cmpFastUnlock",
  /*  421 */  "absF_reg",
  /*  422 */  "absF_reg_reg",
  /*  423 */  "absD_reg",
  /*  424 */  "absD_reg_reg",
  /*  425 */  "negF_reg",
  /*  426 */  "negF_reg_reg",
  /*  427 */  "negD_reg",
  /*  428 */  "negD_reg_reg",
  /*  429 */  "vconvF2HF",
  /*  430 */  "convHF2F_reg_reg",
  /*  431 */  "vconvHF2F_reg_mem",
  /*  432 */  "vconvHF2F",
  /*  433 */  "reinterpret_mask",
  /*  434 */  "reinterpret",
  /*  435 */  "vreinterpret_expand4",
  /*  436 */  "vreinterpret_expand",
  /*  437 */  "reinterpret_shrink",
  /*  438 */  "vReplB_reg",
  /*  439 */  "ReplB_reg",
  /*  440 */  "ReplB_mem",
  /*  441 */  "vReplS_reg",
  /*  442 */  "ReplS_reg",
  /*  443 */  "ReplS_mem",
  /*  444 */  "ReplI_reg",
  /*  445 */  "ReplI_mem",
  /*  446 */  "ReplI_imm",
  /*  447 */  "ReplI_imm_0",
  /*  448 */  "ReplI_imm_1",
  /*  449 */  "ReplI_zero",
  /*  450 */  "ReplI_zero_0",
  /*  451 */  "ReplI_zero_1",
  /*  452 */  "ReplI_M1",
  /*  453 */  "ReplI_M1_0",
  /*  454 */  "ReplI_M1_1",
  /*  455 */  "ReplL_mem",
  /*  456 */  "ReplL_imm",
  /*  457 */  "ReplL_zero",
  /*  458 */  "ReplL_M1",
  /*  459 */  "vReplF_reg",
  /*  460 */  "ReplF_reg",
  /*  461 */  "ReplF_mem",
  /*  462 */  "ReplF_imm",
  /*  463 */  "ReplF_zero",
  /*  464 */  "vReplD_reg",
  /*  465 */  "ReplD_reg",
  /*  466 */  "ReplD_mem",
  /*  467 */  "ReplD_imm",
  /*  468 */  "ReplD_zero",
  /*  469 */  "vsqrtF_reg",
  /*  470 */  "vsqrtF_mem",
  /*  471 */  "vsqrtD_reg",
  /*  472 */  "vsqrtD_mem",
  /*  473 */  "vshiftcnt",
  /*  474 */  "vshiftcnt_0",
  /*  475 */  "vcastBtoX",
  /*  476 */  "castStoX",
  /*  477 */  "vcastStoX_evex",
  /*  478 */  "castItoX",
  /*  479 */  "vcastItoX_evex",
  /*  480 */  "vcastLtoBS",
  /*  481 */  "vcastLtoX_evex",
  /*  482 */  "vcastFtoD_reg",
  /*  483 */  "vcastDtoF_reg",
  /*  484 */  "vucast",
  /*  485 */  "vucast_0",
  /*  486 */  "vucast_1",
  /*  487 */  "vabsB_reg",
  /*  488 */  "vabsS_reg",
  /*  489 */  "vabsI_reg",
  /*  490 */  "vabsL_reg",
  /*  491 */  "vabsnegF",
  /*  492 */  "vabsnegF_0",
  /*  493 */  "vabsneg4F",
  /*  494 */  "vabsneg4F_0",
  /*  495 */  "vabsnegD",
  /*  496 */  "vabsnegD_0",
  /*  497 */  "vmaskcast_evex",
  /*  498 */  "vmaskcast",
  /*  499 */  "vmaskcast_avx",
  /*  500 */  "loadIotaIndices",
  /*  501 */  "loadShuffleB",
  /*  502 */  "loadShuffleS_evex",
  /*  503 */  "loadShuffleI_avx",
  /*  504 */  "loadShuffleL_evex",
  /*  505 */  "vpopcount_integral_reg_evex",
  /*  506 */  "vpopcount_integral_reg_evex_0",
  /*  507 */  "vcount_leading_zeros_IL_reg_evex",
  /*  508 */  "castMM",
  /*  509 */  "castVV",
  /*  510 */  "castVVLeg",
  /*  511 */  "bytes_reverse_int",
  /*  512 */  "bytes_reverse_long",
  /*  513 */  "bytes_reverse_unsigned_short",
  /*  514 */  "bytes_reverse_short",
  /*  515 */  "countLeadingZerosI",
  /*  516 */  "countLeadingZerosI_bsr",
  /*  517 */  "countLeadingZerosL",
  /*  518 */  "countLeadingZerosL_bsr",
  /*  519 */  "countTrailingZerosI",
  /*  520 */  "countTrailingZerosI_bsf",
  /*  521 */  "countTrailingZerosL",
  /*  522 */  "countTrailingZerosL_bsf",
  /*  523 */  "popCountI",
  /*  524 */  "popCountI_mem",
  /*  525 */  "popCountL",
  /*  526 */  "popCountL_mem",
  /*  527 */  "loadB",
  /*  528 */  "loadB2L",
  /*  529 */  "loadUB",
  /*  530 */  "loadUB2L",
  /*  531 */  "loadUB2L_immI",
  /*  532 */  "loadS",
  /*  533 */  "loadS2B",
  /*  534 */  "loadS2L",
  /*  535 */  "loadUS",
  /*  536 */  "loadUS2B",
  /*  537 */  "loadUS2L",
  /*  538 */  "loadUS2L_immI_255",
  /*  539 */  "loadUS2L_immI",
  /*  540 */  "loadI",
  /*  541 */  "loadI2B",
  /*  542 */  "loadI2UB",
  /*  543 */  "loadI2S",
  /*  544 */  "loadI2US",
  /*  545 */  "loadI2L",
  /*  546 */  "loadI2L_immI_255",
  /*  547 */  "loadI2L_immI_65535",
  /*  548 */  "loadI2L_immU31",
  /*  549 */  "loadUI2L",
  /*  550 */  "loadL",
  /*  551 */  "loadL_volatile",
  /*  552 */  "loadLX_volatile",
  /*  553 */  "loadLX_reg_volatile",
  /*  554 */  "loadRange",
  /*  555 */  "loadP",
  /*  556 */  "loadKlass",
  /*  557 */  "loadDPR",
  /*  558 */  "loadD",
  /*  559 */  "loadD_partial",
  /*  560 */  "loadF",
  /*  561 */  "loadFPR",
  /*  562 */  "prefetchAlloc0",
  /*  563 */  "prefetchAlloc",
  /*  564 */  "prefetchAllocNTA",
  /*  565 */  "prefetchAllocT0",
  /*  566 */  "prefetchAllocT2",
  /*  567 */  "storeB",
  /*  568 */  "storeC",
  /*  569 */  "storeI",
  /*  570 */  "storeL",
  /*  571 */  "storeL2I",
  /*  572 */  "storeL_volatile",
  /*  573 */  "storeLX_volatile",
  /*  574 */  "storeLX_reg_volatile",
  /*  575 */  "storeP",
  /*  576 */  "storeImmI",
  /*  577 */  "storeImmI16",
  /*  578 */  "storeImmP",
  /*  579 */  "storeImmB",
  /*  580 */  "storeImmCM",
  /*  581 */  "storeDPR",
  /*  582 */  "storeDPR_rounded",
  /*  583 */  "storeD",
  /*  584 */  "storeF",
  /*  585 */  "storeFPR",
  /*  586 */  "storeFPR_rounded",
  /*  587 */  "storeFPR_Drounded",
  /*  588 */  "storeFPR_imm",
  /*  589 */  "storeF_imm",
  /*  590 */  "membar_acquire",
  /*  591 */  "membar_acquire_0",
  /*  592 */  "membar_acquire_lock",
  /*  593 */  "membar_release",
  /*  594 */  "membar_release_0",
  /*  595 */  "membar_release_lock",
  /*  596 */  "unnecessary_membar_volatile",
  /*  597 */  "membar_storestore",
  /*  598 */  "membar_storestore_0",
  /*  599 */  "jmovI_reg",
  /*  600 */  "jmovI_regU",
  /*  601 */  "cmovI_reg",
  /*  602 */  "cmovI_regU",
  /*  603 */  "cmovI_regUCF",
  /*  604 */  "cmovI_mem",
  /*  605 */  "cmovI_memU",
  /*  606 */  "cmovI_memUCF",
  /*  607 */  "cmovP_reg",
  /*  608 */  "cmovP_reg_nonP6",
  /*  609 */  "cmovP_regU",
  /*  610 */  "cmovP_regUCF",
  /*  611 */  "fcmovDPR_regU",
  /*  612 */  "fcmovFPR_regU",
  /*  613 */  "fcmovDPR_regS",
  /*  614 */  "fcmovFPR_regS",
  /*  615 */  "fcmovF_regS",
  /*  616 */  "fcmovD_regS",
  /*  617 */  "fcmovF_regU",
  /*  618 */  "fcmovF_regUCF",
  /*  619 */  "fcmovD_regU",
  /*  620 */  "fcmovD_regUCF",
  /*  621 */  "cmovL_reg",
  /*  622 */  "cmovL_regU",
  /*  623 */  "cmovL_regUCF",
  /*  624 */  "addI_eReg",
  /*  625 */  "addI_eReg_imm",
  /*  626 */  "incI_eReg",
  /*  627 */  "leaI_eReg_immI",
  /*  628 */  "leaP_eReg_immI",
  /*  629 */  "decI_eReg",
  /*  630 */  "addP_eReg",
  /*  631 */  "addP_eReg_imm",
  /*  632 */  "addI_eReg_mem",
  /*  633 */  "addI_eReg_mem_0",
  /*  634 */  "addI_mem_eReg",
  /*  635 */  "addI_mem_eReg_0",
  /*  636 */  "addI_mem_imm",
  /*  637 */  "incI_mem",
  /*  638 */  "decI_mem",
  /*  639 */  "compareAndSwapL",
  /*  640 */  "compareAndSwapL_0",
  /*  641 */  "compareAndSwapP",
  /*  642 */  "compareAndSwapP_0",
  /*  643 */  "compareAndSwapB",
  /*  644 */  "compareAndSwapB_0",
  /*  645 */  "compareAndSwapS",
  /*  646 */  "compareAndSwapS_0",
  /*  647 */  "compareAndSwapI",
  /*  648 */  "compareAndSwapI_0",
  /*  649 */  "compareAndExchangeL",
  /*  650 */  "compareAndExchangeP",
  /*  651 */  "compareAndExchangeB",
  /*  652 */  "compareAndExchangeS",
  /*  653 */  "compareAndExchangeI",
  /*  654 */  "xaddB_no_res",
  /*  655 */  "xaddB",
  /*  656 */  "xaddS_no_res",
  /*  657 */  "xaddS",
  /*  658 */  "xaddI_no_res",
  /*  659 */  "xaddI",
  /*  660 */  "xchgB",
  /*  661 */  "xchgS",
  /*  662 */  "xchgI",
  /*  663 */  "xchgP",
  /*  664 */  "subI_eReg",
  /*  665 */  "subI_eReg_imm",
  /*  666 */  "subI_eReg_mem",
  /*  667 */  "subI_mem_eReg",
  /*  668 */  "subP_eReg",
  /*  669 */  "negI_eReg",
  /*  670 */  "mulI_eReg",
  /*  671 */  "mulI_eReg_imm",
  /*  672 */  "mulI_imm_high",
  /*  673 */  "mulI_imm_high_0",
  /*  674 */  "mulI_imm_RShift_high",
  /*  675 */  "mulI_imm_RShift_high_0",
  /*  676 */  "mulI_mem_imm",
  /*  677 */  "mulI",
  /*  678 */  "mulI_0",
  /*  679 */  "mulAddS2I_rReg",
  /*  680 */  "mulI2L",
  /*  681 */  "mulI2L_0",
  /*  682 */  "mulIS_eReg",
  /*  683 */  "mulIS_eReg_0",
  /*  684 */  "mulL_eReg",
  /*  685 */  "mulL_eReg_lhi0",
  /*  686 */  "mulL_eReg_rhi0",
  /*  687 */  "mulL_eReg_hi0",
  /*  688 */  "mulL_eReg_con",
  /*  689 */  "divI_eReg",
  /*  690 */  "divL_eReg",
  /*  691 */  "divModI_eReg_divmod",
  /*  692 */  "modI_eReg",
  /*  693 */  "modL_eReg",
  /*  694 */  "divL_eReg_imm32",
  /*  695 */  "modL_eReg_imm32",
  /*  696 */  "shlI_eReg_1",
  /*  697 */  "salI_eReg_imm",
  /*  698 */  "salI_eReg_CL",
  /*  699 */  "sarI_eReg_1",
  /*  700 */  "sarI_mem_1",
  /*  701 */  "sarI_eReg_imm",
  /*  702 */  "sarI_mem_imm",
  /*  703 */  "sarI_eReg_CL",
  /*  704 */  "shrI_eReg_1",
  /*  705 */  "shrI_eReg_imm",
  /*  706 */  "i2b",
  /*  707 */  "i2s",
  /*  708 */  "shrI_eReg_CL",
  /*  709 */  "andI_eReg",
  /*  710 */  "andI_eReg_imm",
  /*  711 */  "andI_eReg_mem",
  /*  712 */  "andI_eReg_mem_0",
  /*  713 */  "andI_mem_eReg",
  /*  714 */  "andI_mem_eReg_0",
  /*  715 */  "andI_mem_imm",
  /*  716 */  "andnI_rReg_rReg_rReg",
  /*  717 */  "andnI_rReg_rReg_rReg_0",
  /*  718 */  "andnI_rReg_rReg_mem",
  /*  719 */  "andnI_rReg_rReg_mem_0",
  /*  720 */  "blsiI_rReg_rReg",
  /*  721 */  "blsiI_rReg_rReg_0",
  /*  722 */  "blsiI_rReg_mem",
  /*  723 */  "blsiI_rReg_mem_0",
  /*  724 */  "blsmskI_rReg_rReg",
  /*  725 */  "blsmskI_rReg_rReg_0",
  /*  726 */  "blsmskI_rReg_mem",
  /*  727 */  "blsmskI_rReg_mem_0",
  /*  728 */  "blsrI_rReg_rReg",
  /*  729 */  "blsrI_rReg_rReg_0",
  /*  730 */  "blsrI_rReg_mem",
  /*  731 */  "blsrI_rReg_mem_0",
  /*  732 */  "orI_eReg",
  /*  733 */  "orI_eReg_castP2X",
  /*  734 */  "orI_eReg_castP2X_0",
  /*  735 */  "orI_eReg_imm",
  /*  736 */  "orI_eReg_mem",
  /*  737 */  "orI_eReg_mem_0",
  /*  738 */  "orI_mem_eReg",
  /*  739 */  "orI_mem_eReg_0",
  /*  740 */  "orI_mem_imm",
  /*  741 */  "rolI_eReg_imm1",
  /*  742 */  "rolI_eReg_imm8",
  /*  743 */  "rolI_eReg_CL",
  /*  744 */  "rolI_eReg_i1",
  /*  745 */  "rolI_eReg_i1_0",
  /*  746 */  "rolI_eReg_i8",
  /*  747 */  "rolI_eReg_i8_0",
  /*  748 */  "rolI_eReg_Var_C0",
  /*  749 */  "rolI_eReg_Var_C0_0",
  /*  750 */  "rolI_eReg_Var_C32",
  /*  751 */  "rolI_eReg_Var_C32_0",
  /*  752 */  "rorI_eReg_imm1",
  /*  753 */  "rorI_eReg_imm8",
  /*  754 */  "rorI_eReg_CL",
  /*  755 */  "rorI_eReg_i1",
  /*  756 */  "rorI_eReg_i1_0",
  /*  757 */  "rorI_eReg_i8",
  /*  758 */  "rorI_eReg_i8_0",
  /*  759 */  "rorI_eReg_Var_C0",
  /*  760 */  "rorI_eReg_Var_C0_0",
  /*  761 */  "rorI_eReg_Var_C32",
  /*  762 */  "rorI_eReg_Var_C32_0",
  /*  763 */  "xorI_eReg",
  /*  764 */  "xorI_eReg_im1",
  /*  765 */  "xorI_eReg_imm",
  /*  766 */  "xorI_eReg_mem",
  /*  767 */  "xorI_eReg_mem_0",
  /*  768 */  "xorI_mem_eReg",
  /*  769 */  "xorI_mem_eReg_0",
  /*  770 */  "xorI_mem_imm",
  /*  771 */  "ci2b",
  /*  772 */  "convI2B",
  /*  773 */  "cp2b",
  /*  774 */  "convP2B",
  /*  775 */  "cmpLTMask",
  /*  776 */  "cmpLTMask0",
  /*  777 */  "cadd_cmpLTMask",
  /*  778 */  "cadd_cmpLTMask_1",
  /*  779 */  "cadd_cmpLTMask_0",
  /*  780 */  "cadd_cmpLTMask_2",
  /*  781 */  "and_cmpLTMask",
  /*  782 */  "and_cmpLTMask_0",
  /*  783 */  "absI_rReg",
  /*  784 */  "addL_eReg",
  /*  785 */  "addL_eReg_imm",
  /*  786 */  "addL_eReg_mem",
  /*  787 */  "addL_eReg_mem_0",
  /*  788 */  "subL_eReg",
  /*  789 */  "subL_eReg_imm",
  /*  790 */  "subL_eReg_mem",
  /*  791 */  "negL_eReg",
  /*  792 */  "andL_eReg",
  /*  793 */  "andL_eReg_imm",
  /*  794 */  "andL_eReg_mem",
  /*  795 */  "andL_eReg_mem_0",
  /*  796 */  "andnL_eReg_eReg_eReg",
  /*  797 */  "andnL_eReg_eReg_eReg_0",
  /*  798 */  "andnL_eReg_eReg_mem",
  /*  799 */  "andnL_eReg_eReg_mem_0",
  /*  800 */  "blsiL_eReg_eReg",
  /*  801 */  "blsiL_eReg_eReg_0",
  /*  802 */  "blsiL_eReg_mem",
  /*  803 */  "blsiL_eReg_mem_0",
  /*  804 */  "blsmskL_eReg_eReg",
  /*  805 */  "blsmskL_eReg_eReg_0",
  /*  806 */  "blsmskL_eReg_mem",
  /*  807 */  "blsmskL_eReg_mem_0",
  /*  808 */  "blsrL_eReg_eReg",
  /*  809 */  "blsrL_eReg_eReg_0",
  /*  810 */  "blsrL_eReg_mem",
  /*  811 */  "blsrL_eReg_mem_0",
  /*  812 */  "orl_eReg",
  /*  813 */  "orl_eReg_imm",
  /*  814 */  "orl_eReg_mem",
  /*  815 */  "orl_eReg_mem_0",
  /*  816 */  "xorl_eReg",
  /*  817 */  "xorl_eReg_im1",
  /*  818 */  "xorl_eReg_imm",
  /*  819 */  "xorl_eReg_mem",
  /*  820 */  "xorl_eReg_mem_0",
  /*  821 */  "shlL_eReg_1",
  /*  822 */  "shlL_eReg_2",
  /*  823 */  "shlL_eReg_3",
  /*  824 */  "shlL_eReg_1_31",
  /*  825 */  "shlL_eReg_32_63",
  /*  826 */  "salL_eReg_CL",
  /*  827 */  "shrL_eReg_1_31",
  /*  828 */  "shrL_eReg_32_63",
  /*  829 */  "shrL_eReg_CL",
  /*  830 */  "sarL_eReg_1_31",
  /*  831 */  "sarL_eReg_32_63",
  /*  832 */  "sarL_eReg_CL",
  /*  833 */  "cmpDPR_0",
  /*  834 */  "cmpDPR_reg",
  /*  835 */  "cmpD_reg",
  /*  836 */  "cmpD_regmem",
  /*  837 */  "subDPR_reg",
  /*  838 */  "subDPR_reg_round",
  /*  839 */  "subDPR_reg_mem",
  /*  840 */  "addDPR_reg",
  /*  841 */  "addDPR_reg_round",
  /*  842 */  "addDPR_reg_mem",
  /*  843 */  "addDPR_reg_mem_0",
  /*  844 */  "addDPR_mem_reg",
  /*  845 */  "addDPR_mem_reg_0",
  /*  846 */  "addDPR_reg_imm1",
  /*  847 */  "addDPR_reg_imm",
  /*  848 */  "addDPR_reg_imm_round",
  /*  849 */  "mulDPR_reg",
  /*  850 */  "strictfp_mulDPR_reg",
  /*  851 */  "mulDPR_reg_imm",
  /*  852 */  "mulDPR_reg_mem",
  /*  853 */  "mulDPR_reg_mem_0",
  /*  854 */  "mulDPR_reg_mem_cisc",
  /*  855 */  "mulDPR_reg_mem_cisc_0",
  /*  856 */  "addDPR_mulDPR_reg",
  /*  857 */  "addDPR_mulDPR_reg_0",
  /*  858 */  "subDPR_mulDPR_reg",
  /*  859 */  "divDPR_reg",
  /*  860 */  "strictfp_divDPR_reg",
  /*  861 */  "modDPR_reg",
  /*  862 */  "modD_reg",
  /*  863 */  "atanDPR_reg",
  /*  864 */  "atanD_reg",
  /*  865 */  "sqrtDPR_reg",
  /*  866 */  "cmpFPR_0",
  /*  867 */  "cmpFPR_reg",
  /*  868 */  "cmpF_reg",
  /*  869 */  "cmpF_regmem",
  /*  870 */  "subFPR24_reg",
  /*  871 */  "subFPR_reg",
  /*  872 */  "addFPR24_reg",
  /*  873 */  "addFPR_reg",
  /*  874 */  "addFPR24_reg_mem",
  /*  875 */  "addFPR24_reg_mem_0",
  /*  876 */  "addFPR_reg_mem",
  /*  877 */  "addFPR_reg_mem_0",
  /*  878 */  "addFPR24_mem_reg",
  /*  879 */  "addFPR24_mem_cisc",
  /*  880 */  "addFPR24_mem_cisc_0",
  /*  881 */  "addFPR24_mem_mem",
  /*  882 */  "addFPR24_reg_imm",
  /*  883 */  "addFPR_reg_imm",
  /*  884 */  "mulFPR24_reg",
  /*  885 */  "mulFPR_reg",
  /*  886 */  "mulFPR24_reg_mem",
  /*  887 */  "mulFPR24_reg_mem_0",
  /*  888 */  "mulFPR_reg_mem",
  /*  889 */  "mulFPR_reg_mem_0",
  /*  890 */  "mulFPR24_mem_mem",
  /*  891 */  "mulFPR24_reg_imm",
  /*  892 */  "mulFPR_reg_imm",
  /*  893 */  "mulFPR_reg_load1",
  /*  894 */  "mulFPR_reg_load1_0",
  /*  895 */  "addFPR_mulFPR_reg_load1",
  /*  896 */  "addFPR_mulFPR_reg_load1_1",
  /*  897 */  "addFPR_mulFPR_reg_load1_0",
  /*  898 */  "addFPR_mulFPR_reg_load1_2",
  /*  899 */  "addFPR_mulFPR_reg",
  /*  900 */  "addFPR_mulFPR_reg_0",
  /*  901 */  "subFPR_divFPR_reg",
  /*  902 */  "divFPR24_reg",
  /*  903 */  "divFPR_reg",
  /*  904 */  "modFPR24_reg",
  /*  905 */  "modFPR_reg",
  /*  906 */  "modF_reg",
  /*  907 */  "roundFloat_mem_reg",
  /*  908 */  "roundDouble_mem_reg",
  /*  909 */  "convDPR2FPR_reg",
  /*  910 */  "convDPR2F_reg",
  /*  911 */  "convD2F_reg",
  /*  912 */  "convFPR2DPR_reg_reg",
  /*  913 */  "convFPR2D_reg",
  /*  914 */  "convF2DPR_reg",
  /*  915 */  "convF2D_reg",
  /*  916 */  "convDPR2I_reg_reg",
  /*  917 */  "convD2I_reg_reg",
  /*  918 */  "convDPR2L_reg_reg",
  /*  919 */  "convD2L_reg_reg",
  /*  920 */  "convFPR2I_reg_reg",
  /*  921 */  "convF2I_reg",
  /*  922 */  "convFPR2L_reg_reg",
  /*  923 */  "convF2L_reg_reg",
  /*  924 */  "convI2DPR_reg",
  /*  925 */  "convI2D_reg",
  /*  926 */  "convI2D_mem",
  /*  927 */  "convXI2D_reg",
  /*  928 */  "convI2DPR_mem",
  /*  929 */  "conv24I2FPR_reg",
  /*  930 */  "convI2FPR_SSF",
  /*  931 */  "convI2FPR_SSF_mem",
  /*  932 */  "convI2FPR_reg",
  /*  933 */  "convI2FPR_mem",
  /*  934 */  "convI2F_reg",
  /*  935 */  "convXI2F_reg",
  /*  936 */  "convI2L_reg",
  /*  937 */  "convI2L_reg_zex",
  /*  938 */  "zerox_long",
  /*  939 */  "convL2DPR_reg",
  /*  940 */  "convL2D_reg",
  /*  941 */  "convL2F_reg",
  /*  942 */  "convL2FPR_reg",
  /*  943 */  "convL2I_reg",
  /*  944 */  "MoveFPR2I_reg_stack",
  /*  945 */  "MoveF2I_reg_stack_sse",
  /*  946 */  "MoveI2F_reg_stack",
  /*  947 */  "MoveDPR2L_reg_stack",
  /*  948 */  "MoveD2L_reg_stack_sse",
  /*  949 */  "MoveD2L_reg_reg_sse",
  /*  950 */  "MoveL2D_reg_stack",
  /*  951 */  "MoveL2D_reg_reg_sse",
  /*  952 */  "compressBitsL_reg",
  /*  953 */  "expandBitsL_reg",
  /*  954 */  "rep_stos",
  /*  955 */  "rep_stos_evex",
  /*  956 */  "rep_stos_large",
  /*  957 */  "rep_stos_large_evex",
  /*  958 */  "rep_stos_im",
  /*  959 */  "string_compareL",
  /*  960 */  "string_compareL_evex",
  /*  961 */  "string_compareU",
  /*  962 */  "string_compareU_evex",
  /*  963 */  "string_compareLU",
  /*  964 */  "string_compareLU_evex",
  /*  965 */  "string_compareUL",
  /*  966 */  "string_compareUL_evex",
  /*  967 */  "string_equals",
  /*  968 */  "string_equals_evex",
  /*  969 */  "string_indexof_conL",
  /*  970 */  "string_indexof_conU",
  /*  971 */  "string_indexof_conUL",
  /*  972 */  "string_indexofL",
  /*  973 */  "string_indexofU",
  /*  974 */  "string_indexofUL",
  /*  975 */  "string_indexof_char",
  /*  976 */  "stringL_indexof_char",
  /*  977 */  "array_equalsB",
  /*  978 */  "array_equalsB_evex",
  /*  979 */  "array_equalsC",
  /*  980 */  "array_equalsC_evex",
  /*  981 */  "count_positives",
  /*  982 */  "count_positives_evex",
  /*  983 */  "string_compress",
  /*  984 */  "string_compress_evex",
  /*  985 */  "string_inflate",
  /*  986 */  "string_inflate_evex",
  /*  987 */  "encode_iso_array",
  /*  988 */  "encode_ascii_array",
  /*  989 */  "minI_eReg",
  /*  990 */  "maxI_eReg",
  /*  991 */  "loopLimit_eReg",
  /*  992 */  "jumpXtnd",
  /*  993 */  "jmpDir",
  /*  994 */  "jmpCon",
  /*  995 */  "jmpLoopEnd",
  /*  996 */  "jmpConU",
  /*  997 */  "jmpConUCF",
  /*  998 */  "jmpConUCF2",
  /*  999 */  "partialSubtypeCheck",
  /* 1000 */  "jmpDir_short",
  /* 1001 */  "jmpCon_short",
  /* 1002 */  "jmpLoopEnd_short",
  /* 1003 */  "jmpConU_short",
  /* 1004 */  "jmpConUCF_short",
  /* 1005 */  "jmpConUCF2_short",
  /* 1006 */  "cmpL3_reg_reg",
  /* 1007 */  "cmpL_LTGE",
  /* 1008 */  "cmpUL_LTGE",
  /* 1009 */  "cmovLL_reg_LTGE",
  /* 1010 */  "cmovLL_mem_LTGE",
  /* 1011 */  "cmovLL_reg_LTGE_U",
  /* 1012 */  "cmovLL_mem_LTGE_U",
  /* 1013 */  "cmovII_reg_LTGE",
  /* 1014 */  "cmovII_mem_LTGE",
  /* 1015 */  "cmovII_reg_LTGE_U",
  /* 1016 */  "cmovII_mem_LTGE_U",
  /* 1017 */  "cmovPP_reg_LTGE",
  /* 1018 */  "cmovPP_reg_LTGE_U",
  /* 1019 */  "cmovDDPR_reg_LTGE",
  /* 1020 */  "cmovDD_reg_LTGE",
  /* 1021 */  "cmovFFPR_reg_LTGE",
  /* 1022 */  "cmovFF_reg_LTGE",
  /* 1023 */  "cmpL_EQNE",
  /* 1024 */  "cmpUL_EQNE",
  /* 1025 */  "cmovLL_reg_EQNE",
  /* 1026 */  "cmovLL_mem_EQNE",
  /* 1027 */  "cmovII_reg_EQNE",
  /* 1028 */  "cmovII_mem_EQNE",
  /* 1029 */  "cmovII_reg_EQNE_U",
  /* 1030 */  "cmovII_mem_EQNE_U",
  /* 1031 */  "cmovPP_reg_EQNE",
  /* 1032 */  "cmovPP_reg_EQNE_U",
  /* 1033 */  "cmovDDPR_reg_EQNE",
  /* 1034 */  "cmovDD_reg_EQNE",
  /* 1035 */  "cmovFFPR_reg_EQNE",
  /* 1036 */  "cmovFF_reg_EQNE",
  /* 1037 */  "cmpL_LEGT",
  /* 1038 */  "cmpUL_LEGT",
  /* 1039 */  "cmovLL_reg_LEGT",
  /* 1040 */  "cmovLL_mem_LEGT",
  /* 1041 */  "cmovLL_reg_LEGT_U",
  /* 1042 */  "cmovLL_mem_LEGT_U",
  /* 1043 */  "cmovII_reg_LEGT",
  /* 1044 */  "cmovII_mem_LEGT",
  /* 1045 */  "cmovII_reg_LEGT_U",
  /* 1046 */  "cmovII_mem_LEGT_U",
  /* 1047 */  "cmovPP_reg_LEGT",
  /* 1048 */  "cmovPP_reg_LEGT_U",
  /* 1049 */  "cmovDDPR_reg_LEGT",
  /* 1050 */  "cmovDD_reg_LEGT",
  /* 1051 */  "cmovFFPR_reg_LEGT",
  /* 1052 */  "cmovFF_reg_LEGT",
  /* 1053 */  "CallStaticJavaDirect",
  /* 1054 */  "CallDynamicJavaDirect",
  /* 1055 */  "CallRuntimeDirect",
  /* 1056 */  "CallLeafDirect",
  /* 1057 */  "CallLeafNoFPDirect",
  /* 1058 */  "Ret",
  /* 1059 */  "TailCalljmpInd",
  /* 1060 */  "tailjmpInd",
  /* 1061 */  "CreateException",
  /* 1062 */  "RethrowException",
  /* 1063 */  "mask_all_evexL_LT32",
  /* 1064 */  "mask_all_evexL_GT32",
  /* 1065 */  "mask_all_evexI_GT32",
  /* 1066 */  "safePoint_poll_tls",
  /* 1067 */  "tlsLoadP",
  /* 1068 */  "ShouldNotReachHere",
  /* 1069 */  "addF_reg",
  /* 1070 */  "addF_mem",
  /* 1071 */  "addF_mem_0",
  /* 1072 */  "addF_imm",
  /* 1073 */  "addF_reg_reg",
  /* 1074 */  "addF_reg_mem",
  /* 1075 */  "addF_reg_mem_0",
  /* 1076 */  "addF_reg_imm",
  /* 1077 */  "addD_reg",
  /* 1078 */  "addD_mem",
  /* 1079 */  "addD_mem_0",
  /* 1080 */  "addD_imm",
  /* 1081 */  "addD_reg_reg",
  /* 1082 */  "addD_reg_mem",
  /* 1083 */  "addD_reg_mem_0",
  /* 1084 */  "addD_reg_imm",
  /* 1085 */  "subF_reg",
  /* 1086 */  "subF_mem",
  /* 1087 */  "subF_imm",
  /* 1088 */  "subF_reg_reg",
  /* 1089 */  "subF_reg_mem",
  /* 1090 */  "subF_reg_imm",
  /* 1091 */  "subD_reg",
  /* 1092 */  "subD_mem",
  /* 1093 */  "subD_imm",
  /* 1094 */  "subD_reg_reg",
  /* 1095 */  "subD_reg_mem",
  /* 1096 */  "subD_reg_imm",
  /* 1097 */  "mulF_reg",
  /* 1098 */  "mulF_mem",
  /* 1099 */  "mulF_mem_0",
  /* 1100 */  "mulF_imm",
  /* 1101 */  "mulF_reg_reg",
  /* 1102 */  "mulF_reg_mem",
  /* 1103 */  "mulF_reg_mem_0",
  /* 1104 */  "mulF_reg_imm",
  /* 1105 */  "mulD_reg",
  /* 1106 */  "mulD_mem",
  /* 1107 */  "mulD_mem_0",
  /* 1108 */  "mulD_imm",
  /* 1109 */  "mulD_reg_reg",
  /* 1110 */  "mulD_reg_mem",
  /* 1111 */  "mulD_reg_mem_0",
  /* 1112 */  "mulD_reg_imm",
  /* 1113 */  "divF_reg",
  /* 1114 */  "divF_mem",
  /* 1115 */  "divF_imm",
  /* 1116 */  "divF_reg_reg",
  /* 1117 */  "divF_reg_mem",
  /* 1118 */  "divF_reg_imm",
  /* 1119 */  "divD_reg",
  /* 1120 */  "divD_mem",
  /* 1121 */  "divD_imm",
  /* 1122 */  "divD_reg_reg",
  /* 1123 */  "divD_reg_mem",
  /* 1124 */  "divD_reg_imm",
  /* 1125 */  "sqrtF_reg",
  /* 1126 */  "sqrtD_reg",
  /* 1127 */  "convF2HF_reg_reg",
  /* 1128 */  "convF2HF_mem_reg",
  /* 1129 */  "vconvF2HF_mem_reg",
  /* 1130 */  "reinterpret_mask_W2B",
  /* 1131 */  "reinterpret_mask_D2B",
  /* 1132 */  "reinterpret_mask_Q2B",
  /* 1133 */  "reinterpret_expand",
  /* 1134 */  "onspinwait",
  /* 1135 */  "fmaD_reg",
  /* 1136 */  "fmaF_reg",
  /* 1137 */  "loadV",
  /* 1138 */  "storeV",
  /* 1139 */  "gather",
  /* 1140 */  "evgather",
  /* 1141 */  "evgather_masked",
  /* 1142 */  "scatter",
  /* 1143 */  "scatter_masked",
  /* 1144 */  "ReplL_reg",
  /* 1145 */  "ReplL_reg_leg",
  /* 1146 */  "insert",
  /* 1147 */  "insert32",
  /* 1148 */  "insert64",
  /* 1149 */  "insertF",
  /* 1150 */  "vinsertF",
  /* 1151 */  "reductionI",
  /* 1152 */  "reductionI_0",
  /* 1153 */  "reductionI_1",
  /* 1154 */  "reductionI_2",
  /* 1155 */  "reductionI_3",
  /* 1156 */  "reductionI_4",
  /* 1157 */  "reductionI_5",
  /* 1158 */  "reductionF128",
  /* 1159 */  "reductionF128_0",
  /* 1160 */  "reduction8F",
  /* 1161 */  "reduction8F_0",
  /* 1162 */  "reduction16F",
  /* 1163 */  "reduction16F_0",
  /* 1164 */  "reduction2D",
  /* 1165 */  "reduction2D_0",
  /* 1166 */  "reduction4D",
  /* 1167 */  "reduction4D_0",
  /* 1168 */  "reduction8D",
  /* 1169 */  "reduction8D_0",
  /* 1170 */  "reductionS",
  /* 1171 */  "reductionS_0",
  /* 1172 */  "reductionS_1",
  /* 1173 */  "reductionS_2",
  /* 1174 */  "reductionS_3",
  /* 1175 */  "reductionS_4",
  /* 1176 */  "reductionS_5",
  /* 1177 */  "mul_reductionB",
  /* 1178 */  "mul_reduction64B",
  /* 1179 */  "minmax_reduction2F",
  /* 1180 */  "minmax_reduction2F_0",
  /* 1181 */  "minmax_reductionF",
  /* 1182 */  "minmax_reductionF_0",
  /* 1183 */  "minmax_reduction2F_av",
  /* 1184 */  "minmax_reduction2F_av_0",
  /* 1185 */  "minmax_reductionF_av",
  /* 1186 */  "minmax_reductionF_av_0",
  /* 1187 */  "minmax_reduction2D",
  /* 1188 */  "minmax_reduction2D_0",
  /* 1189 */  "minmax_reductionD",
  /* 1190 */  "minmax_reductionD_0",
  /* 1191 */  "minmax_reduction2D_av",
  /* 1192 */  "minmax_reduction2D_av_0",
  /* 1193 */  "minmax_reductionD_av",
  /* 1194 */  "minmax_reductionD_av_0",
  /* 1195 */  "vaddB",
  /* 1196 */  "vaddB_reg",
  /* 1197 */  "vaddB_mem",
  /* 1198 */  "vaddB_mem_0",
  /* 1199 */  "vaddS",
  /* 1200 */  "vaddS_reg",
  /* 1201 */  "vaddS_mem",
  /* 1202 */  "vaddS_mem_0",
  /* 1203 */  "vaddI",
  /* 1204 */  "vaddI_reg",
  /* 1205 */  "vaddI_mem",
  /* 1206 */  "vaddI_mem_0",
  /* 1207 */  "vaddL",
  /* 1208 */  "vaddL_reg",
  /* 1209 */  "vaddL_mem",
  /* 1210 */  "vaddL_mem_0",
  /* 1211 */  "vaddF",
  /* 1212 */  "vaddF_reg",
  /* 1213 */  "vaddF_mem",
  /* 1214 */  "vaddF_mem_0",
  /* 1215 */  "vaddD",
  /* 1216 */  "vaddD_reg",
  /* 1217 */  "vaddD_mem",
  /* 1218 */  "vaddD_mem_0",
  /* 1219 */  "vsubB",
  /* 1220 */  "vsubB_reg",
  /* 1221 */  "vsubB_mem",
  /* 1222 */  "vsubS",
  /* 1223 */  "vsubS_reg",
  /* 1224 */  "vsubS_mem",
  /* 1225 */  "vsubI",
  /* 1226 */  "vsubI_reg",
  /* 1227 */  "vsubI_mem",
  /* 1228 */  "vsubL",
  /* 1229 */  "vsubL_reg",
  /* 1230 */  "vsubL_mem",
  /* 1231 */  "vsubF",
  /* 1232 */  "vsubF_reg",
  /* 1233 */  "vsubF_mem",
  /* 1234 */  "vsubD",
  /* 1235 */  "vsubD_reg",
  /* 1236 */  "vsubD_mem",
  /* 1237 */  "vmul8B",
  /* 1238 */  "vmulB",
  /* 1239 */  "vmulB_reg",
  /* 1240 */  "vmulS",
  /* 1241 */  "vmulS_reg",
  /* 1242 */  "vmulS_mem",
  /* 1243 */  "vmulS_mem_0",
  /* 1244 */  "vmulI",
  /* 1245 */  "vmulI_reg",
  /* 1246 */  "vmulI_mem",
  /* 1247 */  "vmulI_mem_0",
  /* 1248 */  "evmulL_reg",
  /* 1249 */  "evmulL_mem",
  /* 1250 */  "evmulL_mem_0",
  /* 1251 */  "vmulL",
  /* 1252 */  "vmulL_reg",
  /* 1253 */  "vmulF",
  /* 1254 */  "vmulF_reg",
  /* 1255 */  "vmulF_mem",
  /* 1256 */  "vmulF_mem_0",
  /* 1257 */  "vmulD",
  /* 1258 */  "vmulD_reg",
  /* 1259 */  "vmulD_mem",
  /* 1260 */  "vmulD_mem_0",
  /* 1261 */  "vdivF",
  /* 1262 */  "vdivF_reg",
  /* 1263 */  "vdivF_mem",
  /* 1264 */  "vdivD",
  /* 1265 */  "vdivD_reg",
  /* 1266 */  "vdivD_mem",
  /* 1267 */  "minmax_reg_sse",
  /* 1268 */  "minmax_reg_sse_0",
  /* 1269 */  "vminmax_reg",
  /* 1270 */  "vminmax_reg_0",
  /* 1271 */  "minmaxL_reg_sse",
  /* 1272 */  "minmaxL_reg_sse_0",
  /* 1273 */  "vminmaxL_reg_avx",
  /* 1274 */  "vminmaxL_reg_avx_0",
  /* 1275 */  "vminmaxL_reg_evex",
  /* 1276 */  "vminmaxL_reg_evex_0",
  /* 1277 */  "minmaxFP_reg",
  /* 1278 */  "minmaxFP_reg_0",
  /* 1279 */  "evminmaxFP_reg_eavx",
  /* 1280 */  "evminmaxFP_reg_eavx_0",
  /* 1281 */  "signumF_reg",
  /* 1282 */  "signumD_reg",
  /* 1283 */  "signumV_reg_avx",
  /* 1284 */  "signumV_reg_avx_0",
  /* 1285 */  "signumV_reg_evex",
  /* 1286 */  "signumV_reg_evex_0",
  /* 1287 */  "compressBitsI_reg",
  /* 1288 */  "expandBitsI_reg",
  /* 1289 */  "compressBitsI_mem",
  /* 1290 */  "expandBitsI_mem",
  /* 1291 */  "vshiftB",
  /* 1292 */  "vshiftB_0",
  /* 1293 */  "vshiftB_1",
  /* 1294 */  "vshift16B",
  /* 1295 */  "vshift16B_0",
  /* 1296 */  "vshift16B_1",
  /* 1297 */  "vshift16B_avx",
  /* 1298 */  "vshift16B_avx_0",
  /* 1299 */  "vshift16B_avx_1",
  /* 1300 */  "vshift32B_avx",
  /* 1301 */  "vshift32B_avx_0",
  /* 1302 */  "vshift32B_avx_1",
  /* 1303 */  "vshift64B_avx",
  /* 1304 */  "vshift64B_avx_0",
  /* 1305 */  "vshift64B_avx_1",
  /* 1306 */  "vshiftS",
  /* 1307 */  "vshiftS_0",
  /* 1308 */  "vshiftS_1",
  /* 1309 */  "vshiftI",
  /* 1310 */  "vshiftI_0",
  /* 1311 */  "vshiftI_1",
  /* 1312 */  "vshiftI_imm",
  /* 1313 */  "vshiftI_imm_0",
  /* 1314 */  "vshiftI_imm_1",
  /* 1315 */  "vshiftL",
  /* 1316 */  "vshiftL_0",
  /* 1317 */  "vshiftL_imm",
  /* 1318 */  "vshiftL_imm_0",
  /* 1319 */  "vshiftL_arith_reg",
  /* 1320 */  "vshiftL_arith_reg_evex",
  /* 1321 */  "vshift8B_var_nobw",
  /* 1322 */  "vshift8B_var_nobw_0",
  /* 1323 */  "vshift8B_var_nobw_1",
  /* 1324 */  "vshift16B_var_nobw",
  /* 1325 */  "vshift16B_var_nobw_0",
  /* 1326 */  "vshift16B_var_nobw_1",
  /* 1327 */  "vshift32B_var_nobw",
  /* 1328 */  "vshift32B_var_nobw_0",
  /* 1329 */  "vshift32B_var_nobw_1",
  /* 1330 */  "vshiftB_var_evex_bw",
  /* 1331 */  "vshiftB_var_evex_bw_0",
  /* 1332 */  "vshiftB_var_evex_bw_1",
  /* 1333 */  "vshift64B_var_evex_bw",
  /* 1334 */  "vshift64B_var_evex_bw_0",
  /* 1335 */  "vshift64B_var_evex_bw_1",
  /* 1336 */  "vshift8S_var_nobw",
  /* 1337 */  "vshift8S_var_nobw_0",
  /* 1338 */  "vshift8S_var_nobw_1",
  /* 1339 */  "vshift16S_var_nobw",
  /* 1340 */  "vshift16S_var_nobw_0",
  /* 1341 */  "vshift16S_var_nobw_1",
  /* 1342 */  "vshift16S_var_evex_bw",
  /* 1343 */  "vshift16S_var_evex_bw_0",
  /* 1344 */  "vshift16S_var_evex_bw_1",
  /* 1345 */  "vshiftI_var",
  /* 1346 */  "vshiftI_var_0",
  /* 1347 */  "vshiftI_var_1",
  /* 1348 */  "vshiftL_var",
  /* 1349 */  "vshiftL_var_0",
  /* 1350 */  "vshiftL_arith_var",
  /* 1351 */  "vshiftL_arith_var_evex",
  /* 1352 */  "vand",
  /* 1353 */  "vand_reg",
  /* 1354 */  "vand_mem",
  /* 1355 */  "vand_mem_0",
  /* 1356 */  "vor",
  /* 1357 */  "vor_reg",
  /* 1358 */  "vor_mem",
  /* 1359 */  "vor_mem_0",
  /* 1360 */  "vxor",
  /* 1361 */  "vxor_reg",
  /* 1362 */  "vxor_mem",
  /* 1363 */  "vxor_mem_0",
  /* 1364 */  "vcastStoX",
  /* 1365 */  "vcastItoX",
  /* 1366 */  "castFtoX_reg_avx",
  /* 1367 */  "castFtoX_reg_evex",
  /* 1368 */  "castDtoX_reg_avx",
  /* 1369 */  "castDtoX_reg_evex",
  /* 1370 */  "vcmpFD",
  /* 1371 */  "evcmpFD64",
  /* 1372 */  "evcmpFD",
  /* 1373 */  "vcmp_direct",
  /* 1374 */  "vcmp_negate",
  /* 1375 */  "vcmpu",
  /* 1376 */  "vcmp64",
  /* 1377 */  "evcmp",
  /* 1378 */  "extractI",
  /* 1379 */  "extractI_0",
  /* 1380 */  "vextractI",
  /* 1381 */  "vextractI_0",
  /* 1382 */  "extractF",
  /* 1383 */  "vextractF",
  /* 1384 */  "extractD",
  /* 1385 */  "vextractD",
  /* 1386 */  "blendvp",
  /* 1387 */  "vblendvpI",
  /* 1388 */  "vblendvpFD",
  /* 1389 */  "evblendvp64",
  /* 1390 */  "evblendvp64_masked",
  /* 1391 */  "loadMask",
  /* 1392 */  "loadMask64",
  /* 1393 */  "loadMask_evex",
  /* 1394 */  "vstoreMask1B",
  /* 1395 */  "vstoreMask2B",
  /* 1396 */  "vstoreMask4B",
  /* 1397 */  "storeMask8B",
  /* 1398 */  "storeMask8B_avx",
  /* 1399 */  "vstoreMask4B_evex_novectmask",
  /* 1400 */  "vstoreMask8B_evex_novectmask",
  /* 1401 */  "vstoreMask_evex_vectmask",
  /* 1402 */  "vstoreMask_evex",
  /* 1403 */  "rearrangeB",
  /* 1404 */  "rearrangeB_avx",
  /* 1405 */  "rearrangeB_evex",
  /* 1406 */  "rearrangeB_evex_vbmi",
  /* 1407 */  "loadShuffleS",
  /* 1408 */  "rearrangeS",
  /* 1409 */  "rearrangeS_avx",
  /* 1410 */  "rearrangeS_evex",
  /* 1411 */  "loadShuffleI",
  /* 1412 */  "rearrangeI",
  /* 1413 */  "rearrangeI_avx",
  /* 1414 */  "loadShuffleL",
  /* 1415 */  "rearrangeL",
  /* 1416 */  "rearrangeL_evex",
  /* 1417 */  "vfmaF_reg",
  /* 1418 */  "vfmaF_mem",
  /* 1419 */  "vfmaD_reg",
  /* 1420 */  "vfmaD_mem",
  /* 1421 */  "vmuladdS2I_reg_sse",
  /* 1422 */  "vmuladdS2I_reg_avx",
  /* 1423 */  "vmuladdaddS2I_reg",
  /* 1424 */  "vmuladdaddS2I_reg_0",
  /* 1425 */  "vpopcount_integral_reg_evex_masked",
  /* 1426 */  "vpopcount_integral_reg_evex_masked_0",
  /* 1427 */  "vpopcount_avx_reg",
  /* 1428 */  "vpopcount_avx_reg_0",
  /* 1429 */  "vcount_trailing_zeros_reg_evex",
  /* 1430 */  "vcount_trailing_zeros_short_reg_evex",
  /* 1431 */  "vcount_trailing_zeros_byte_reg_evex",
  /* 1432 */  "vcount_trailing_zeros_reg_avx",
  /* 1433 */  "vpternlog",
  /* 1434 */  "vpternlog_mem",
  /* 1435 */  "vprotate_immI8",
  /* 1436 */  "vprotate_immI8_0",
  /* 1437 */  "vprorate",
  /* 1438 */  "vprorate_0",
  /* 1439 */  "vmasked_load_avx_non_subword",
  /* 1440 */  "vmasked_load_evex",
  /* 1441 */  "vmasked_store_avx_non_subword",
  /* 1442 */  "vmasked_store_evex",
  /* 1443 */  "vreverse_reg",
  /* 1444 */  "vreverse_reg_gfni",
  /* 1445 */  "vreverse_byte_reg",
  /* 1446 */  "vreverse_byte64_reg",
  /* 1447 */  "vcount_leading_zeros_IL_reg_evex_masked",
  /* 1448 */  "vcount_leading_zeros_short_reg_evex",
  /* 1449 */  "vcount_leading_zeros_byte_reg_evex",
  /* 1450 */  "vcount_leading_zeros_int_reg_avx",
  /* 1451 */  "vcount_leading_zeros_reg_avx",
  /* 1452 */  "vadd_reg_masked",
  /* 1453 */  "vadd_reg_masked_0",
  /* 1454 */  "vadd_reg_masked_1",
  /* 1455 */  "vadd_reg_masked_2",
  /* 1456 */  "vadd_reg_masked_3",
  /* 1457 */  "vadd_reg_masked_4",
  /* 1458 */  "vadd_mem_masked",
  /* 1459 */  "vadd_mem_masked_0",
  /* 1460 */  "vadd_mem_masked_1",
  /* 1461 */  "vadd_mem_masked_2",
  /* 1462 */  "vadd_mem_masked_3",
  /* 1463 */  "vadd_mem_masked_4",
  /* 1464 */  "vxor_reg_masked",
  /* 1465 */  "vxor_mem_masked",
  /* 1466 */  "vor_reg_masked",
  /* 1467 */  "vor_mem_masked",
  /* 1468 */  "vand_reg_masked",
  /* 1469 */  "vand_mem_masked",
  /* 1470 */  "vsub_reg_masked",
  /* 1471 */  "vsub_reg_masked_0",
  /* 1472 */  "vsub_reg_masked_1",
  /* 1473 */  "vsub_reg_masked_2",
  /* 1474 */  "vsub_reg_masked_3",
  /* 1475 */  "vsub_reg_masked_4",
  /* 1476 */  "vsub_mem_masked",
  /* 1477 */  "vsub_mem_masked_0",
  /* 1478 */  "vsub_mem_masked_1",
  /* 1479 */  "vsub_mem_masked_2",
  /* 1480 */  "vsub_mem_masked_3",
  /* 1481 */  "vsub_mem_masked_4",
  /* 1482 */  "vmul_reg_masked",
  /* 1483 */  "vmul_reg_masked_0",
  /* 1484 */  "vmul_reg_masked_1",
  /* 1485 */  "vmul_reg_masked_2",
  /* 1486 */  "vmul_reg_masked_3",
  /* 1487 */  "vmul_mem_masked",
  /* 1488 */  "vmul_mem_masked_0",
  /* 1489 */  "vmul_mem_masked_1",
  /* 1490 */  "vmul_mem_masked_2",
  /* 1491 */  "vmul_mem_masked_3",
  /* 1492 */  "vsqrt_reg_masked",
  /* 1493 */  "vsqrt_reg_masked_0",
  /* 1494 */  "vdiv_reg_masked",
  /* 1495 */  "vdiv_reg_masked_0",
  /* 1496 */  "vdiv_mem_masked",
  /* 1497 */  "vdiv_mem_masked_0",
  /* 1498 */  "vrol_imm_masked",
  /* 1499 */  "vrol_imm_masked_0",
  /* 1500 */  "vrol_reg_masked",
  /* 1501 */  "vrol_reg_masked_0",
  /* 1502 */  "vlshift_imm_masked",
  /* 1503 */  "vlshift_imm_masked_0",
  /* 1504 */  "vlshift_imm_masked_1",
  /* 1505 */  "vlshift_reg_masked",
  /* 1506 */  "vlshift_reg_masked_0",
  /* 1507 */  "vlshift_reg_masked_1",
  /* 1508 */  "vlshiftv_reg_masked",
  /* 1509 */  "vlshiftv_reg_masked_0",
  /* 1510 */  "vlshiftv_reg_masked_1",
  /* 1511 */  "vlshift_mem_masked",
  /* 1512 */  "vlshift_mem_masked_0",
  /* 1513 */  "vlshift_mem_masked_1",
  /* 1514 */  "vrshift_imm_masked",
  /* 1515 */  "vrshift_imm_masked_0",
  /* 1516 */  "vrshift_imm_masked_1",
  /* 1517 */  "vrshift_reg_masked",
  /* 1518 */  "vrshift_reg_masked_0",
  /* 1519 */  "vrshift_reg_masked_1",
  /* 1520 */  "vrshiftv_reg_masked",
  /* 1521 */  "vrshiftv_reg_masked_0",
  /* 1522 */  "vrshiftv_reg_masked_1",
  /* 1523 */  "vrshift_mem_masked",
  /* 1524 */  "vrshift_mem_masked_0",
  /* 1525 */  "vrshift_mem_masked_1",
  /* 1526 */  "vurshift_imm_masked",
  /* 1527 */  "vurshift_imm_masked_0",
  /* 1528 */  "vurshift_imm_masked_1",
  /* 1529 */  "vurshift_reg_masked",
  /* 1530 */  "vurshift_reg_masked_0",
  /* 1531 */  "vurshift_reg_masked_1",
  /* 1532 */  "vurshiftv_reg_masked",
  /* 1533 */  "vurshiftv_reg_masked_0",
  /* 1534 */  "vurshiftv_reg_masked_1",
  /* 1535 */  "vurshift_mem_masked",
  /* 1536 */  "vurshift_mem_masked_0",
  /* 1537 */  "vurshift_mem_masked_1",
  /* 1538 */  "vmaxv_reg_masked",
  /* 1539 */  "vmaxv_mem_masked",
  /* 1540 */  "vminv_reg_masked",
  /* 1541 */  "vminv_mem_masked",
  /* 1542 */  "vrearrangev_reg_masked",
  /* 1543 */  "vabs_masked",
  /* 1544 */  "vabs_masked_0",
  /* 1545 */  "vabs_masked_1",
  /* 1546 */  "vabs_masked_2",
  /* 1547 */  "vfma_reg_masked",
  /* 1548 */  "vfma_reg_masked_0",
  /* 1549 */  "vfma_mem_masked",
  /* 1550 */  "vfma_mem_masked_0",
  /* 1551 */  "evcmp_masked",
  /* 1552 */  "mask_all_evexI_LE32",
  /* 1553 */  "mask_opers_evex",
  /* 1554 */  "mask_opers_evex_0",
  /* 1555 */  "mask_opers_evex_1",
  /* 1556 */  "vternlog_reg_masked",
  /* 1557 */  "vternlogd_mem_masked",
  /* 1558 */  "FloatClassCheck_reg_reg_vfpclass",
  /* 1559 */  "DoubleClassCheck_reg_reg_vfpclass",
  /* 1560 */  "compareAndSwapP_shenandoah",
  /* 1561 */  "compareAndSwapP_shenandoah_0",
  /* 1562 */  "compareAndExchangeP_shenandoah",
  // last instruction
  "invalid rule name" // no trailing comma
};

const        bool  swallowed[] = {
  /*    0 */  false,
  /*    1 */  false,
  /*    2 */  false,
  /*    3 */  false,
  /*    4 */  false,
  /*    5 */  false,
  /*    6 */  false,
  /*    7 */  false,
  /*    8 */  true,
  /*    9 */  true,
  /*   10 */  true,
  /*   11 */  true,
  /*   12 */  true,
  /*   13 */  true,
  /*   14 */  true,
  /*   15 */  true,
  /*   16 */  true,
  /*   17 */  true,
  /*   18 */  true,
  /*   19 */  true,
  /*   20 */  true,
  /*   21 */  true,
  /*   22 */  true,
  /*   23 */  true,
  /*   24 */  true,
  /*   25 */  true,
  /*   26 */  true,
  /*   27 */  true,
  /*   28 */  true,
  /*   29 */  true,
  /*   30 */  true,
  /*   31 */  true,
  /*   32 */  true,
  /*   33 */  true,
  /*   34 */  true,
  /*   35 */  true,
  /*   36 */  true,
  /*   37 */  true,
  /*   38 */  true,
  /*   39 */  true,
  /*   40 */  true,
  /*   41 */  true,
  /*   42 */  true,
  /*   43 */  true,
  /*   44 */  true,
  /*   45 */  true,
  /*   46 */  false,
  /*   47 */  false,
  /*   48 */  false,
  /*   49 */  false,
  /*   50 */  false,
  /*   51 */  false,
  /*   52 */  false,
  /*   53 */  false,
  /*   54 */  false,
  /*   55 */  false,
  /*   56 */  false,
  /*   57 */  false,
  /*   58 */  false,
  /*   59 */  false,
  /*   60 */  false,
  /*   61 */  false,
  /*   62 */  false,
  /*   63 */  false,
  /*   64 */  false,
  /*   65 */  false,
  /*   66 */  false,
  /*   67 */  false,
  /*   68 */  false,
  /*   69 */  false,
  /*   70 */  false,
  /*   71 */  false,
  /*   72 */  false,
  /*   73 */  false,
  /*   74 */  false,
  /*   75 */  false,
  /*   76 */  false,
  /*   77 */  false,
  /*   78 */  false,
  /*   79 */  false,
  /*   80 */  false,
  /*   81 */  false,
  /*   82 */  false,
  /*   83 */  false,
  /*   84 */  false,
  /*   85 */  false,
  /*   86 */  false,
  /*   87 */  false,
  /*   88 */  false,
  /*   89 */  false,
  /*   90 */  false,
  /*   91 */  false,
  /*   92 */  false,
  /*   93 */  false,
  /*   94 */  false,
  /*   95 */  false,
  /*   96 */  false,
  /*   97 */  false,
  /*   98 */  false,
  /*   99 */  false,
  /*  100 */  false,
  /*  101 */  false,
  /*  102 */  false,
  /*  103 */  false,
  /*  104 */  false,
  /*  105 */  true,
  /*  106 */  false,
  /*  107 */  false,
  /*  108 */  false,
  /*  109 */  false,
  /*  110 */  false,
  /*  111 */  false,
  /*  112 */  false,
  /*  113 */  false,
  /*  114 */  false,
  /*  115 */  false,
  /*  116 */  false,
  /*  117 */  false,
  /*  118 */  false,
  /*  119 */  false,
  /*  120 */  false,
  /*  121 */  false,
  /*  122 */  true,
  /*  123 */  true,
  /*  124 */  true,
  /*  125 */  true,
  /*  126 */  true,
  /*  127 */  true,
  /*  128 */  true,
  /*  129 */  false,
  /*  130 */  false,
  /*  131 */  false,
  /*  132 */  false,
  /*  133 */  false,
  /*  134 */  false,
  /*  135 */  false,
  /*  136 */  false,
  /*  137 */  false,
  /*  138 */  false,
  /*  139 */  false,
  /*  140 */  false,
  /*  141 */  false,
  // last operand
  /*  142 */  false,
  /*  143 */  false,
  /*  144 */  false,
  // last operand class
  /*  145 */  false,
  /*  146 */  false,
  /*  147 */  false,
  /*  148 */  false,
  /*  149 */  false,
  /*  150 */  false,
  /*  151 */  false,
  /*  152 */  false,
  /*  153 */  false,
  /*  154 */  false,
  /*  155 */  false,
  /*  156 */  false,
  /*  157 */  false,
  /*  158 */  false,
  /*  159 */  false,
  /*  160 */  false,
  /*  161 */  false,
  /*  162 */  false,
  /*  163 */  false,
  /*  164 */  false,
  /*  165 */  false,
  /*  166 */  false,
  /*  167 */  false,
  /*  168 */  false,
  /*  169 */  false,
  /*  170 */  false,
  /*  171 */  false,
  /*  172 */  false,
  /*  173 */  false,
  /*  174 */  false,
  /*  175 */  false,
  /*  176 */  false,
  /*  177 */  false,
  /*  178 */  false,
  /*  179 */  false,
  /*  180 */  false,
  /*  181 */  false,
  /*  182 */  false,
  /*  183 */  false,
  /*  184 */  false,
  /*  185 */  false,
  /*  186 */  false,
  /*  187 */  false,
  /*  188 */  false,
  /*  189 */  false,
  /*  190 */  false,
  /*  191 */  false,
  /*  192 */  false,
  /*  193 */  false,
  /*  194 */  false,
  /*  195 */  false,
  /*  196 */  false,
  /*  197 */  false,
  /*  198 */  false,
  /*  199 */  false,
  /*  200 */  false,
  /*  201 */  false,
  /*  202 */  false,
  /*  203 */  false,
  /*  204 */  false,
  /*  205 */  false,
  /*  206 */  false,
  /*  207 */  false,
  /*  208 */  false,
  /*  209 */  false,
  /*  210 */  false,
  /*  211 */  false,
  /*  212 */  false,
  /*  213 */  false,
  /*  214 */  false,
  /*  215 */  false,
  /*  216 */  false,
  /*  217 */  false,
  /*  218 */  false,
  /*  219 */  false,
  /*  220 */  false,
  /*  221 */  false,
  /*  222 */  false,
  /*  223 */  false,
  /*  224 */  false,
  /*  225 */  false,
  /*  226 */  false,
  /*  227 */  false,
  /*  228 */  false,
  /*  229 */  false,
  /*  230 */  false,
  /*  231 */  false,
  /*  232 */  false,
  /*  233 */  false,
  /*  234 */  false,
  /*  235 */  false,
  /*  236 */  false,
  /*  237 */  false,
  /*  238 */  false,
  /*  239 */  false,
  /*  240 */  false,
  /*  241 */  false,
  /*  242 */  false,
  /*  243 */  false,
  /*  244 */  false,
  /*  245 */  false,
  /*  246 */  false,
  /*  247 */  false,
  /*  248 */  false,
  /*  249 */  false,
  /*  250 */  false,
  /*  251 */  false,
  /*  252 */  false,
  /*  253 */  false,
  /*  254 */  false,
  /*  255 */  false,
  /*  256 */  false,
  /*  257 */  false,
  /*  258 */  false,
  /*  259 */  false,
  /*  260 */  false,
  /*  261 */  false,
  /*  262 */  false,
  /*  263 */  false,
  /*  264 */  false,
  /*  265 */  false,
  /*  266 */  false,
  /*  267 */  false,
  /*  268 */  false,
  /*  269 */  false,
  /*  270 */  false,
  /*  271 */  false,
  /*  272 */  false,
  /*  273 */  false,
  /*  274 */  false,
  /*  275 */  false,
  /*  276 */  false,
  /*  277 */  false,
  /*  278 */  false,
  /*  279 */  false,
  /*  280 */  false,
  /*  281 */  false,
  /*  282 */  false,
  /*  283 */  false,
  /*  284 */  false,
  /*  285 */  false,
  /*  286 */  false,
  /*  287 */  false,
  /*  288 */  false,
  /*  289 */  false,
  /*  290 */  false,
  /*  291 */  false,
  /*  292 */  false,
  /*  293 */  false,
  /*  294 */  false,
  /*  295 */  false,
  /*  296 */  false,
  /*  297 */  false,
  /*  298 */  false,
  /*  299 */  false,
  /*  300 */  false,
  /*  301 */  false,
  /*  302 */  false,
  // last internally defined operand
  /*  303 */  false,
  /*  304 */  false,
  /*  305 */  false,
  /*  306 */  false,
  /*  307 */  false,
  /*  308 */  false,
  /*  309 */  false,
  /*  310 */  false,
  /*  311 */  false,
  /*  312 */  false,
  /*  313 */  false,
  /*  314 */  false,
  /*  315 */  false,
  /*  316 */  false,
  /*  317 */  false,
  /*  318 */  false,
  /*  319 */  false,
  /*  320 */  false,
  /*  321 */  false,
  /*  322 */  false,
  /*  323 */  false,
  /*  324 */  false,
  /*  325 */  false,
  /*  326 */  false,
  /*  327 */  false,
  /*  328 */  false,
  /*  329 */  false,
  /*  330 */  false,
  /*  331 */  false,
  /*  332 */  false,
  /*  333 */  false,
  /*  334 */  false,
  /*  335 */  false,
  /*  336 */  false,
  /*  337 */  false,
  /*  338 */  false,
  /*  339 */  false,
  /*  340 */  false,
  /*  341 */  false,
  /*  342 */  false,
  /*  343 */  false,
  /*  344 */  false,
  /*  345 */  false,
  /*  346 */  false,
  /*  347 */  false,
  /*  348 */  false,
  /*  349 */  false,
  /*  350 */  false,
  /*  351 */  false,
  /*  352 */  false,
  /*  353 */  false,
  /*  354 */  false,
  /*  355 */  false,
  /*  356 */  false,
  /*  357 */  false,
  /*  358 */  false,
  /*  359 */  false,
  /*  360 */  false,
  /*  361 */  false,
  /*  362 */  false,
  /*  363 */  false,
  /*  364 */  false,
  /*  365 */  false,
  /*  366 */  false,
  /*  367 */  false,
  /*  368 */  false,
  /*  369 */  false,
  /*  370 */  false,
  /*  371 */  false,
  /*  372 */  false,
  /*  373 */  false,
  /*  374 */  false,
  /*  375 */  false,
  /*  376 */  false,
  /*  377 */  false,
  /*  378 */  false,
  /*  379 */  false,
  /*  380 */  false,
  /*  381 */  false,
  /*  382 */  false,
  /*  383 */  false,
  /*  384 */  false,
  /*  385 */  false,
  /*  386 */  false,
  /*  387 */  false,
  /*  388 */  false,
  /*  389 */  false,
  /*  390 */  false,
  /*  391 */  false,
  /*  392 */  false,
  /*  393 */  false,
  /*  394 */  false,
  /*  395 */  false,
  /*  396 */  false,
  /*  397 */  false,
  /*  398 */  false,
  /*  399 */  false,
  /*  400 */  false,
  /*  401 */  false,
  /*  402 */  false,
  /*  403 */  false,
  /*  404 */  false,
  /*  405 */  false,
  /*  406 */  false,
  /*  407 */  false,
  /*  408 */  false,
  /*  409 */  false,
  /*  410 */  false,
  /*  411 */  false,
  /*  412 */  false,
  /*  413 */  false,
  /*  414 */  false,
  /*  415 */  false,
  /*  416 */  false,
  /*  417 */  false,
  /*  418 */  false,
  /*  419 */  false,
  /*  420 */  false,
  /*  421 */  false,
  /*  422 */  false,
  /*  423 */  false,
  /*  424 */  false,
  /*  425 */  false,
  /*  426 */  false,
  /*  427 */  false,
  /*  428 */  false,
  /*  429 */  false,
  /*  430 */  false,
  /*  431 */  false,
  /*  432 */  false,
  /*  433 */  false,
  /*  434 */  false,
  /*  435 */  false,
  /*  436 */  false,
  /*  437 */  false,
  /*  438 */  false,
  /*  439 */  false,
  /*  440 */  false,
  /*  441 */  false,
  /*  442 */  false,
  /*  443 */  false,
  /*  444 */  false,
  /*  445 */  false,
  /*  446 */  false,
  /*  447 */  false,
  /*  448 */  false,
  /*  449 */  false,
  /*  450 */  false,
  /*  451 */  false,
  /*  452 */  false,
  /*  453 */  false,
  /*  454 */  false,
  /*  455 */  false,
  /*  456 */  false,
  /*  457 */  false,
  /*  458 */  false,
  /*  459 */  false,
  /*  460 */  false,
  /*  461 */  false,
  /*  462 */  false,
  /*  463 */  false,
  /*  464 */  false,
  /*  465 */  false,
  /*  466 */  false,
  /*  467 */  false,
  /*  468 */  false,
  /*  469 */  false,
  /*  470 */  false,
  /*  471 */  false,
  /*  472 */  false,
  /*  473 */  false,
  /*  474 */  false,
  /*  475 */  false,
  /*  476 */  false,
  /*  477 */  false,
  /*  478 */  false,
  /*  479 */  false,
  /*  480 */  false,
  /*  481 */  false,
  /*  482 */  false,
  /*  483 */  false,
  /*  484 */  false,
  /*  485 */  false,
  /*  486 */  false,
  /*  487 */  false,
  /*  488 */  false,
  /*  489 */  false,
  /*  490 */  false,
  /*  491 */  false,
  /*  492 */  false,
  /*  493 */  false,
  /*  494 */  false,
  /*  495 */  false,
  /*  496 */  false,
  /*  497 */  false,
  /*  498 */  false,
  /*  499 */  false,
  /*  500 */  false,
  /*  501 */  false,
  /*  502 */  false,
  /*  503 */  false,
  /*  504 */  false,
  /*  505 */  false,
  /*  506 */  false,
  /*  507 */  false,
  /*  508 */  false,
  /*  509 */  false,
  /*  510 */  false,
  /*  511 */  false,
  /*  512 */  false,
  /*  513 */  false,
  /*  514 */  false,
  /*  515 */  false,
  /*  516 */  false,
  /*  517 */  false,
  /*  518 */  false,
  /*  519 */  false,
  /*  520 */  false,
  /*  521 */  false,
  /*  522 */  false,
  /*  523 */  false,
  /*  524 */  false,
  /*  525 */  false,
  /*  526 */  false,
  /*  527 */  false,
  /*  528 */  false,
  /*  529 */  false,
  /*  530 */  false,
  /*  531 */  false,
  /*  532 */  false,
  /*  533 */  false,
  /*  534 */  false,
  /*  535 */  false,
  /*  536 */  false,
  /*  537 */  false,
  /*  538 */  false,
  /*  539 */  false,
  /*  540 */  false,
  /*  541 */  false,
  /*  542 */  false,
  /*  543 */  false,
  /*  544 */  false,
  /*  545 */  false,
  /*  546 */  false,
  /*  547 */  false,
  /*  548 */  false,
  /*  549 */  false,
  /*  550 */  false,
  /*  551 */  false,
  /*  552 */  false,
  /*  553 */  false,
  /*  554 */  false,
  /*  555 */  false,
  /*  556 */  false,
  /*  557 */  false,
  /*  558 */  false,
  /*  559 */  false,
  /*  560 */  false,
  /*  561 */  false,
  /*  562 */  false,
  /*  563 */  false,
  /*  564 */  false,
  /*  565 */  false,
  /*  566 */  false,
  /*  567 */  false,
  /*  568 */  false,
  /*  569 */  false,
  /*  570 */  false,
  /*  571 */  false,
  /*  572 */  false,
  /*  573 */  false,
  /*  574 */  false,
  /*  575 */  false,
  /*  576 */  false,
  /*  577 */  false,
  /*  578 */  false,
  /*  579 */  false,
  /*  580 */  false,
  /*  581 */  false,
  /*  582 */  false,
  /*  583 */  false,
  /*  584 */  false,
  /*  585 */  false,
  /*  586 */  false,
  /*  587 */  false,
  /*  588 */  false,
  /*  589 */  false,
  /*  590 */  false,
  /*  591 */  false,
  /*  592 */  false,
  /*  593 */  false,
  /*  594 */  false,
  /*  595 */  false,
  /*  596 */  false,
  /*  597 */  false,
  /*  598 */  false,
  /*  599 */  false,
  /*  600 */  false,
  /*  601 */  false,
  /*  602 */  false,
  /*  603 */  false,
  /*  604 */  false,
  /*  605 */  false,
  /*  606 */  false,
  /*  607 */  false,
  /*  608 */  false,
  /*  609 */  false,
  /*  610 */  false,
  /*  611 */  false,
  /*  612 */  false,
  /*  613 */  false,
  /*  614 */  false,
  /*  615 */  false,
  /*  616 */  false,
  /*  617 */  false,
  /*  618 */  false,
  /*  619 */  false,
  /*  620 */  false,
  /*  621 */  false,
  /*  622 */  false,
  /*  623 */  false,
  /*  624 */  false,
  /*  625 */  false,
  /*  626 */  false,
  /*  627 */  false,
  /*  628 */  false,
  /*  629 */  false,
  /*  630 */  false,
  /*  631 */  false,
  /*  632 */  false,
  /*  633 */  false,
  /*  634 */  false,
  /*  635 */  false,
  /*  636 */  false,
  /*  637 */  false,
  /*  638 */  false,
  /*  639 */  false,
  /*  640 */  false,
  /*  641 */  false,
  /*  642 */  false,
  /*  643 */  false,
  /*  644 */  false,
  /*  645 */  false,
  /*  646 */  false,
  /*  647 */  false,
  /*  648 */  false,
  /*  649 */  false,
  /*  650 */  false,
  /*  651 */  false,
  /*  652 */  false,
  /*  653 */  false,
  /*  654 */  false,
  /*  655 */  false,
  /*  656 */  false,
  /*  657 */  false,
  /*  658 */  false,
  /*  659 */  false,
  /*  660 */  false,
  /*  661 */  false,
  /*  662 */  false,
  /*  663 */  false,
  /*  664 */  false,
  /*  665 */  false,
  /*  666 */  false,
  /*  667 */  false,
  /*  668 */  false,
  /*  669 */  false,
  /*  670 */  false,
  /*  671 */  false,
  /*  672 */  false,
  /*  673 */  false,
  /*  674 */  false,
  /*  675 */  false,
  /*  676 */  false,
  /*  677 */  false,
  /*  678 */  false,
  /*  679 */  false,
  /*  680 */  false,
  /*  681 */  false,
  /*  682 */  false,
  /*  683 */  false,
  /*  684 */  false,
  /*  685 */  false,
  /*  686 */  false,
  /*  687 */  false,
  /*  688 */  false,
  /*  689 */  false,
  /*  690 */  false,
  /*  691 */  false,
  /*  692 */  false,
  /*  693 */  false,
  /*  694 */  false,
  /*  695 */  false,
  /*  696 */  false,
  /*  697 */  false,
  /*  698 */  false,
  /*  699 */  false,
  /*  700 */  false,
  /*  701 */  false,
  /*  702 */  false,
  /*  703 */  false,
  /*  704 */  false,
  /*  705 */  false,
  /*  706 */  false,
  /*  707 */  false,
  /*  708 */  false,
  /*  709 */  false,
  /*  710 */  false,
  /*  711 */  false,
  /*  712 */  false,
  /*  713 */  false,
  /*  714 */  false,
  /*  715 */  false,
  /*  716 */  false,
  /*  717 */  false,
  /*  718 */  false,
  /*  719 */  false,
  /*  720 */  false,
  /*  721 */  false,
  /*  722 */  false,
  /*  723 */  false,
  /*  724 */  false,
  /*  725 */  false,
  /*  726 */  false,
  /*  727 */  false,
  /*  728 */  false,
  /*  729 */  false,
  /*  730 */  false,
  /*  731 */  false,
  /*  732 */  false,
  /*  733 */  false,
  /*  734 */  false,
  /*  735 */  false,
  /*  736 */  false,
  /*  737 */  false,
  /*  738 */  false,
  /*  739 */  false,
  /*  740 */  false,
  /*  741 */  false,
  /*  742 */  false,
  /*  743 */  false,
  /*  744 */  false,
  /*  745 */  false,
  /*  746 */  false,
  /*  747 */  false,
  /*  748 */  false,
  /*  749 */  false,
  /*  750 */  false,
  /*  751 */  false,
  /*  752 */  false,
  /*  753 */  false,
  /*  754 */  false,
  /*  755 */  false,
  /*  756 */  false,
  /*  757 */  false,
  /*  758 */  false,
  /*  759 */  false,
  /*  760 */  false,
  /*  761 */  false,
  /*  762 */  false,
  /*  763 */  false,
  /*  764 */  false,
  /*  765 */  false,
  /*  766 */  false,
  /*  767 */  false,
  /*  768 */  false,
  /*  769 */  false,
  /*  770 */  false,
  /*  771 */  false,
  /*  772 */  false,
  /*  773 */  false,
  /*  774 */  false,
  /*  775 */  false,
  /*  776 */  false,
  /*  777 */  false,
  /*  778 */  false,
  /*  779 */  false,
  /*  780 */  false,
  /*  781 */  false,
  /*  782 */  false,
  /*  783 */  false,
  /*  784 */  false,
  /*  785 */  false,
  /*  786 */  false,
  /*  787 */  false,
  /*  788 */  false,
  /*  789 */  false,
  /*  790 */  false,
  /*  791 */  false,
  /*  792 */  false,
  /*  793 */  false,
  /*  794 */  false,
  /*  795 */  false,
  /*  796 */  false,
  /*  797 */  false,
  /*  798 */  false,
  /*  799 */  false,
  /*  800 */  false,
  /*  801 */  false,
  /*  802 */  false,
  /*  803 */  false,
  /*  804 */  false,
  /*  805 */  false,
  /*  806 */  false,
  /*  807 */  false,
  /*  808 */  false,
  /*  809 */  false,
  /*  810 */  false,
  /*  811 */  false,
  /*  812 */  false,
  /*  813 */  false,
  /*  814 */  false,
  /*  815 */  false,
  /*  816 */  false,
  /*  817 */  false,
  /*  818 */  false,
  /*  819 */  false,
  /*  820 */  false,
  /*  821 */  false,
  /*  822 */  false,
  /*  823 */  false,
  /*  824 */  false,
  /*  825 */  false,
  /*  826 */  false,
  /*  827 */  false,
  /*  828 */  false,
  /*  829 */  false,
  /*  830 */  false,
  /*  831 */  false,
  /*  832 */  false,
  /*  833 */  false,
  /*  834 */  false,
  /*  835 */  false,
  /*  836 */  false,
  /*  837 */  false,
  /*  838 */  false,
  /*  839 */  false,
  /*  840 */  false,
  /*  841 */  false,
  /*  842 */  false,
  /*  843 */  false,
  /*  844 */  false,
  /*  845 */  false,
  /*  846 */  false,
  /*  847 */  false,
  /*  848 */  false,
  /*  849 */  false,
  /*  850 */  false,
  /*  851 */  false,
  /*  852 */  false,
  /*  853 */  false,
  /*  854 */  false,
  /*  855 */  false,
  /*  856 */  false,
  /*  857 */  false,
  /*  858 */  false,
  /*  859 */  false,
  /*  860 */  false,
  /*  861 */  false,
  /*  862 */  false,
  /*  863 */  false,
  /*  864 */  false,
  /*  865 */  false,
  /*  866 */  false,
  /*  867 */  false,
  /*  868 */  false,
  /*  869 */  false,
  /*  870 */  false,
  /*  871 */  false,
  /*  872 */  false,
  /*  873 */  false,
  /*  874 */  false,
  /*  875 */  false,
  /*  876 */  false,
  /*  877 */  false,
  /*  878 */  false,
  /*  879 */  false,
  /*  880 */  false,
  /*  881 */  false,
  /*  882 */  false,
  /*  883 */  false,
  /*  884 */  false,
  /*  885 */  false,
  /*  886 */  false,
  /*  887 */  false,
  /*  888 */  false,
  /*  889 */  false,
  /*  890 */  false,
  /*  891 */  false,
  /*  892 */  false,
  /*  893 */  false,
  /*  894 */  false,
  /*  895 */  false,
  /*  896 */  false,
  /*  897 */  false,
  /*  898 */  false,
  /*  899 */  false,
  /*  900 */  false,
  /*  901 */  false,
  /*  902 */  false,
  /*  903 */  false,
  /*  904 */  false,
  /*  905 */  false,
  /*  906 */  false,
  /*  907 */  false,
  /*  908 */  false,
  /*  909 */  false,
  /*  910 */  false,
  /*  911 */  false,
  /*  912 */  false,
  /*  913 */  false,
  /*  914 */  false,
  /*  915 */  false,
  /*  916 */  false,
  /*  917 */  false,
  /*  918 */  false,
  /*  919 */  false,
  /*  920 */  false,
  /*  921 */  false,
  /*  922 */  false,
  /*  923 */  false,
  /*  924 */  false,
  /*  925 */  false,
  /*  926 */  false,
  /*  927 */  false,
  /*  928 */  false,
  /*  929 */  false,
  /*  930 */  false,
  /*  931 */  false,
  /*  932 */  false,
  /*  933 */  false,
  /*  934 */  false,
  /*  935 */  false,
  /*  936 */  false,
  /*  937 */  false,
  /*  938 */  false,
  /*  939 */  false,
  /*  940 */  false,
  /*  941 */  false,
  /*  942 */  false,
  /*  943 */  false,
  /*  944 */  false,
  /*  945 */  false,
  /*  946 */  false,
  /*  947 */  false,
  /*  948 */  false,
  /*  949 */  false,
  /*  950 */  false,
  /*  951 */  false,
  /*  952 */  false,
  /*  953 */  false,
  /*  954 */  false,
  /*  955 */  false,
  /*  956 */  false,
  /*  957 */  false,
  /*  958 */  false,
  /*  959 */  false,
  /*  960 */  false,
  /*  961 */  false,
  /*  962 */  false,
  /*  963 */  false,
  /*  964 */  false,
  /*  965 */  false,
  /*  966 */  false,
  /*  967 */  false,
  /*  968 */  false,
  /*  969 */  false,
  /*  970 */  false,
  /*  971 */  false,
  /*  972 */  false,
  /*  973 */  false,
  /*  974 */  false,
  /*  975 */  false,
  /*  976 */  false,
  /*  977 */  false,
  /*  978 */  false,
  /*  979 */  false,
  /*  980 */  false,
  /*  981 */  false,
  /*  982 */  false,
  /*  983 */  false,
  /*  984 */  false,
  /*  985 */  false,
  /*  986 */  false,
  /*  987 */  false,
  /*  988 */  false,
  /*  989 */  false,
  /*  990 */  false,
  /*  991 */  false,
  /*  992 */  false,
  /*  993 */  false,
  /*  994 */  false,
  /*  995 */  false,
  /*  996 */  false,
  /*  997 */  false,
  /*  998 */  false,
  /*  999 */  false,
  /* 1000 */  false,
  /* 1001 */  false,
  /* 1002 */  false,
  /* 1003 */  false,
  /* 1004 */  false,
  /* 1005 */  false,
  /* 1006 */  false,
  /* 1007 */  false,
  /* 1008 */  false,
  /* 1009 */  false,
  /* 1010 */  false,
  /* 1011 */  false,
  /* 1012 */  false,
  /* 1013 */  false,
  /* 1014 */  false,
  /* 1015 */  false,
  /* 1016 */  false,
  /* 1017 */  false,
  /* 1018 */  false,
  /* 1019 */  false,
  /* 1020 */  false,
  /* 1021 */  false,
  /* 1022 */  false,
  /* 1023 */  false,
  /* 1024 */  false,
  /* 1025 */  false,
  /* 1026 */  false,
  /* 1027 */  false,
  /* 1028 */  false,
  /* 1029 */  false,
  /* 1030 */  false,
  /* 1031 */  false,
  /* 1032 */  false,
  /* 1033 */  false,
  /* 1034 */  false,
  /* 1035 */  false,
  /* 1036 */  false,
  /* 1037 */  false,
  /* 1038 */  false,
  /* 1039 */  false,
  /* 1040 */  false,
  /* 1041 */  false,
  /* 1042 */  false,
  /* 1043 */  false,
  /* 1044 */  false,
  /* 1045 */  false,
  /* 1046 */  false,
  /* 1047 */  false,
  /* 1048 */  false,
  /* 1049 */  false,
  /* 1050 */  false,
  /* 1051 */  false,
  /* 1052 */  false,
  /* 1053 */  false,
  /* 1054 */  false,
  /* 1055 */  false,
  /* 1056 */  false,
  /* 1057 */  false,
  /* 1058 */  false,
  /* 1059 */  false,
  /* 1060 */  false,
  /* 1061 */  false,
  /* 1062 */  false,
  /* 1063 */  false,
  /* 1064 */  false,
  /* 1065 */  false,
  /* 1066 */  false,
  /* 1067 */  false,
  /* 1068 */  false,
  /* 1069 */  false,
  /* 1070 */  false,
  /* 1071 */  false,
  /* 1072 */  false,
  /* 1073 */  false,
  /* 1074 */  false,
  /* 1075 */  false,
  /* 1076 */  false,
  /* 1077 */  false,
  /* 1078 */  false,
  /* 1079 */  false,
  /* 1080 */  false,
  /* 1081 */  false,
  /* 1082 */  false,
  /* 1083 */  false,
  /* 1084 */  false,
  /* 1085 */  false,
  /* 1086 */  false,
  /* 1087 */  false,
  /* 1088 */  false,
  /* 1089 */  false,
  /* 1090 */  false,
  /* 1091 */  false,
  /* 1092 */  false,
  /* 1093 */  false,
  /* 1094 */  false,
  /* 1095 */  false,
  /* 1096 */  false,
  /* 1097 */  false,
  /* 1098 */  false,
  /* 1099 */  false,
  /* 1100 */  false,
  /* 1101 */  false,
  /* 1102 */  false,
  /* 1103 */  false,
  /* 1104 */  false,
  /* 1105 */  false,
  /* 1106 */  false,
  /* 1107 */  false,
  /* 1108 */  false,
  /* 1109 */  false,
  /* 1110 */  false,
  /* 1111 */  false,
  /* 1112 */  false,
  /* 1113 */  false,
  /* 1114 */  false,
  /* 1115 */  false,
  /* 1116 */  false,
  /* 1117 */  false,
  /* 1118 */  false,
  /* 1119 */  false,
  /* 1120 */  false,
  /* 1121 */  false,
  /* 1122 */  false,
  /* 1123 */  false,
  /* 1124 */  false,
  /* 1125 */  false,
  /* 1126 */  false,
  /* 1127 */  false,
  /* 1128 */  false,
  /* 1129 */  false,
  /* 1130 */  false,
  /* 1131 */  false,
  /* 1132 */  false,
  /* 1133 */  false,
  /* 1134 */  false,
  /* 1135 */  false,
  /* 1136 */  false,
  /* 1137 */  false,
  /* 1138 */  false,
  /* 1139 */  false,
  /* 1140 */  false,
  /* 1141 */  false,
  /* 1142 */  false,
  /* 1143 */  false,
  /* 1144 */  false,
  /* 1145 */  false,
  /* 1146 */  false,
  /* 1147 */  false,
  /* 1148 */  false,
  /* 1149 */  false,
  /* 1150 */  false,
  /* 1151 */  false,
  /* 1152 */  false,
  /* 1153 */  false,
  /* 1154 */  false,
  /* 1155 */  false,
  /* 1156 */  false,
  /* 1157 */  false,
  /* 1158 */  false,
  /* 1159 */  false,
  /* 1160 */  false,
  /* 1161 */  false,
  /* 1162 */  false,
  /* 1163 */  false,
  /* 1164 */  false,
  /* 1165 */  false,
  /* 1166 */  false,
  /* 1167 */  false,
  /* 1168 */  false,
  /* 1169 */  false,
  /* 1170 */  false,
  /* 1171 */  false,
  /* 1172 */  false,
  /* 1173 */  false,
  /* 1174 */  false,
  /* 1175 */  false,
  /* 1176 */  false,
  /* 1177 */  false,
  /* 1178 */  false,
  /* 1179 */  false,
  /* 1180 */  false,
  /* 1181 */  false,
  /* 1182 */  false,
  /* 1183 */  false,
  /* 1184 */  false,
  /* 1185 */  false,
  /* 1186 */  false,
  /* 1187 */  false,
  /* 1188 */  false,
  /* 1189 */  false,
  /* 1190 */  false,
  /* 1191 */  false,
  /* 1192 */  false,
  /* 1193 */  false,
  /* 1194 */  false,
  /* 1195 */  false,
  /* 1196 */  false,
  /* 1197 */  false,
  /* 1198 */  false,
  /* 1199 */  false,
  /* 1200 */  false,
  /* 1201 */  false,
  /* 1202 */  false,
  /* 1203 */  false,
  /* 1204 */  false,
  /* 1205 */  false,
  /* 1206 */  false,
  /* 1207 */  false,
  /* 1208 */  false,
  /* 1209 */  false,
  /* 1210 */  false,
  /* 1211 */  false,
  /* 1212 */  false,
  /* 1213 */  false,
  /* 1214 */  false,
  /* 1215 */  false,
  /* 1216 */  false,
  /* 1217 */  false,
  /* 1218 */  false,
  /* 1219 */  false,
  /* 1220 */  false,
  /* 1221 */  false,
  /* 1222 */  false,
  /* 1223 */  false,
  /* 1224 */  false,
  /* 1225 */  false,
  /* 1226 */  false,
  /* 1227 */  false,
  /* 1228 */  false,
  /* 1229 */  false,
  /* 1230 */  false,
  /* 1231 */  false,
  /* 1232 */  false,
  /* 1233 */  false,
  /* 1234 */  false,
  /* 1235 */  false,
  /* 1236 */  false,
  /* 1237 */  false,
  /* 1238 */  false,
  /* 1239 */  false,
  /* 1240 */  false,
  /* 1241 */  false,
  /* 1242 */  false,
  /* 1243 */  false,
  /* 1244 */  false,
  /* 1245 */  false,
  /* 1246 */  false,
  /* 1247 */  false,
  /* 1248 */  false,
  /* 1249 */  false,
  /* 1250 */  false,
  /* 1251 */  false,
  /* 1252 */  false,
  /* 1253 */  false,
  /* 1254 */  false,
  /* 1255 */  false,
  /* 1256 */  false,
  /* 1257 */  false,
  /* 1258 */  false,
  /* 1259 */  false,
  /* 1260 */  false,
  /* 1261 */  false,
  /* 1262 */  false,
  /* 1263 */  false,
  /* 1264 */  false,
  /* 1265 */  false,
  /* 1266 */  false,
  /* 1267 */  false,
  /* 1268 */  false,
  /* 1269 */  false,
  /* 1270 */  false,
  /* 1271 */  false,
  /* 1272 */  false,
  /* 1273 */  false,
  /* 1274 */  false,
  /* 1275 */  false,
  /* 1276 */  false,
  /* 1277 */  false,
  /* 1278 */  false,
  /* 1279 */  false,
  /* 1280 */  false,
  /* 1281 */  false,
  /* 1282 */  false,
  /* 1283 */  false,
  /* 1284 */  false,
  /* 1285 */  false,
  /* 1286 */  false,
  /* 1287 */  false,
  /* 1288 */  false,
  /* 1289 */  false,
  /* 1290 */  false,
  /* 1291 */  false,
  /* 1292 */  false,
  /* 1293 */  false,
  /* 1294 */  false,
  /* 1295 */  false,
  /* 1296 */  false,
  /* 1297 */  false,
  /* 1298 */  false,
  /* 1299 */  false,
  /* 1300 */  false,
  /* 1301 */  false,
  /* 1302 */  false,
  /* 1303 */  false,
  /* 1304 */  false,
  /* 1305 */  false,
  /* 1306 */  false,
  /* 1307 */  false,
  /* 1308 */  false,
  /* 1309 */  false,
  /* 1310 */  false,
  /* 1311 */  false,
  /* 1312 */  false,
  /* 1313 */  false,
  /* 1314 */  false,
  /* 1315 */  false,
  /* 1316 */  false,
  /* 1317 */  false,
  /* 1318 */  false,
  /* 1319 */  false,
  /* 1320 */  false,
  /* 1321 */  false,
  /* 1322 */  false,
  /* 1323 */  false,
  /* 1324 */  false,
  /* 1325 */  false,
  /* 1326 */  false,
  /* 1327 */  false,
  /* 1328 */  false,
  /* 1329 */  false,
  /* 1330 */  false,
  /* 1331 */  false,
  /* 1332 */  false,
  /* 1333 */  false,
  /* 1334 */  false,
  /* 1335 */  false,
  /* 1336 */  false,
  /* 1337 */  false,
  /* 1338 */  false,
  /* 1339 */  false,
  /* 1340 */  false,
  /* 1341 */  false,
  /* 1342 */  false,
  /* 1343 */  false,
  /* 1344 */  false,
  /* 1345 */  false,
  /* 1346 */  false,
  /* 1347 */  false,
  /* 1348 */  false,
  /* 1349 */  false,
  /* 1350 */  false,
  /* 1351 */  false,
  /* 1352 */  false,
  /* 1353 */  false,
  /* 1354 */  false,
  /* 1355 */  false,
  /* 1356 */  false,
  /* 1357 */  false,
  /* 1358 */  false,
  /* 1359 */  false,
  /* 1360 */  false,
  /* 1361 */  false,
  /* 1362 */  false,
  /* 1363 */  false,
  /* 1364 */  false,
  /* 1365 */  false,
  /* 1366 */  false,
  /* 1367 */  false,
  /* 1368 */  false,
  /* 1369 */  false,
  /* 1370 */  false,
  /* 1371 */  false,
  /* 1372 */  false,
  /* 1373 */  false,
  /* 1374 */  false,
  /* 1375 */  false,
  /* 1376 */  false,
  /* 1377 */  false,
  /* 1378 */  false,
  /* 1379 */  false,
  /* 1380 */  false,
  /* 1381 */  false,
  /* 1382 */  false,
  /* 1383 */  false,
  /* 1384 */  false,
  /* 1385 */  false,
  /* 1386 */  false,
  /* 1387 */  false,
  /* 1388 */  false,
  /* 1389 */  false,
  /* 1390 */  false,
  /* 1391 */  false,
  /* 1392 */  false,
  /* 1393 */  false,
  /* 1394 */  false,
  /* 1395 */  false,
  /* 1396 */  false,
  /* 1397 */  false,
  /* 1398 */  false,
  /* 1399 */  false,
  /* 1400 */  false,
  /* 1401 */  false,
  /* 1402 */  false,
  /* 1403 */  false,
  /* 1404 */  false,
  /* 1405 */  false,
  /* 1406 */  false,
  /* 1407 */  false,
  /* 1408 */  false,
  /* 1409 */  false,
  /* 1410 */  false,
  /* 1411 */  false,
  /* 1412 */  false,
  /* 1413 */  false,
  /* 1414 */  false,
  /* 1415 */  false,
  /* 1416 */  false,
  /* 1417 */  false,
  /* 1418 */  false,
  /* 1419 */  false,
  /* 1420 */  false,
  /* 1421 */  false,
  /* 1422 */  false,
  /* 1423 */  false,
  /* 1424 */  false,
  /* 1425 */  false,
  /* 1426 */  false,
  /* 1427 */  false,
  /* 1428 */  false,
  /* 1429 */  false,
  /* 1430 */  false,
  /* 1431 */  false,
  /* 1432 */  false,
  /* 1433 */  false,
  /* 1434 */  false,
  /* 1435 */  false,
  /* 1436 */  false,
  /* 1437 */  false,
  /* 1438 */  false,
  /* 1439 */  false,
  /* 1440 */  false,
  /* 1441 */  false,
  /* 1442 */  false,
  /* 1443 */  false,
  /* 1444 */  false,
  /* 1445 */  false,
  /* 1446 */  false,
  /* 1447 */  false,
  /* 1448 */  false,
  /* 1449 */  false,
  /* 1450 */  false,
  /* 1451 */  false,
  /* 1452 */  false,
  /* 1453 */  false,
  /* 1454 */  false,
  /* 1455 */  false,
  /* 1456 */  false,
  /* 1457 */  false,
  /* 1458 */  false,
  /* 1459 */  false,
  /* 1460 */  false,
  /* 1461 */  false,
  /* 1462 */  false,
  /* 1463 */  false,
  /* 1464 */  false,
  /* 1465 */  false,
  /* 1466 */  false,
  /* 1467 */  false,
  /* 1468 */  false,
  /* 1469 */  false,
  /* 1470 */  false,
  /* 1471 */  false,
  /* 1472 */  false,
  /* 1473 */  false,
  /* 1474 */  false,
  /* 1475 */  false,
  /* 1476 */  false,
  /* 1477 */  false,
  /* 1478 */  false,
  /* 1479 */  false,
  /* 1480 */  false,
  /* 1481 */  false,
  /* 1482 */  false,
  /* 1483 */  false,
  /* 1484 */  false,
  /* 1485 */  false,
  /* 1486 */  false,
  /* 1487 */  false,
  /* 1488 */  false,
  /* 1489 */  false,
  /* 1490 */  false,
  /* 1491 */  false,
  /* 1492 */  false,
  /* 1493 */  false,
  /* 1494 */  false,
  /* 1495 */  false,
  /* 1496 */  false,
  /* 1497 */  false,
  /* 1498 */  false,
  /* 1499 */  false,
  /* 1500 */  false,
  /* 1501 */  false,
  /* 1502 */  false,
  /* 1503 */  false,
  /* 1504 */  false,
  /* 1505 */  false,
  /* 1506 */  false,
  /* 1507 */  false,
  /* 1508 */  false,
  /* 1509 */  false,
  /* 1510 */  false,
  /* 1511 */  false,
  /* 1512 */  false,
  /* 1513 */  false,
  /* 1514 */  false,
  /* 1515 */  false,
  /* 1516 */  false,
  /* 1517 */  false,
  /* 1518 */  false,
  /* 1519 */  false,
  /* 1520 */  false,
  /* 1521 */  false,
  /* 1522 */  false,
  /* 1523 */  false,
  /* 1524 */  false,
  /* 1525 */  false,
  /* 1526 */  false,
  /* 1527 */  false,
  /* 1528 */  false,
  /* 1529 */  false,
  /* 1530 */  false,
  /* 1531 */  false,
  /* 1532 */  false,
  /* 1533 */  false,
  /* 1534 */  false,
  /* 1535 */  false,
  /* 1536 */  false,
  /* 1537 */  false,
  /* 1538 */  false,
  /* 1539 */  false,
  /* 1540 */  false,
  /* 1541 */  false,
  /* 1542 */  false,
  /* 1543 */  false,
  /* 1544 */  false,
  /* 1545 */  false,
  /* 1546 */  false,
  /* 1547 */  false,
  /* 1548 */  false,
  /* 1549 */  false,
  /* 1550 */  false,
  /* 1551 */  false,
  /* 1552 */  false,
  /* 1553 */  false,
  /* 1554 */  false,
  /* 1555 */  false,
  /* 1556 */  false,
  /* 1557 */  false,
  /* 1558 */  false,
  /* 1559 */  false,
  /* 1560 */  false,
  /* 1561 */  false,
  /* 1562 */  false,
  // last instruction
  false // no trailing comma
};

// Mapping from machine-independent opcode to boolean
const        char must_clone[] = {
  0, // Node: 0
  0, // Set: 1
  0, // RegN: 2
  0, // RegI: 3
  0, // RegP: 4
  0, // RegF: 5
  0, // RegD: 6
  0, // RegL: 7
  0, // VecA: 8
  0, // VecS: 9
  0, // VecD: 10
  0, // VecX: 11
  0, // VecY: 12
  0, // VecZ: 13
  0, // RegVectMask: 14
  0, // RegFlags: 15
  0, // _last_machine_leaf: 16
  0, // AbsD: 17
  0, // AbsF: 18
  0, // AbsI: 19
  0, // AbsL: 20
  0, // AddD: 21
  0, // AddF: 22
  0, // AddI: 23
  0, // AddL: 24
  0, // AddP: 25
  0, // Allocate: 26
  0, // AllocateArray: 27
  0, // AndI: 28
  0, // AndL: 29
  0, // ArrayCopy: 30
  0, // AryEq: 31
  0, // AtanD: 32
  1, // Binary: 33
  0, // Blackhole: 34
  1, // Bool: 35
  0, // BoxLock: 36
  0, // ReverseBytesI: 37
  0, // ReverseBytesL: 38
  0, // ReverseBytesUS: 39
  0, // ReverseBytesS: 40
  0, // ReverseBytesV: 41
  0, // CProj: 42
  0, // CacheWB: 43
  0, // CacheWBPreSync: 44
  0, // CacheWBPostSync: 45
  0, // CallDynamicJava: 46
  0, // CallJava: 47
  0, // CallLeaf: 48
  0, // CallLeafNoFP: 49
  0, // CallLeafVector: 50
  0, // CallRuntime: 51
  0, // CallStaticJava: 52
  0, // CastDD: 53
  0, // CastFF: 54
  0, // CastII: 55
  0, // CastLL: 56
  0, // CastVV: 57
  0, // CastX2P: 58
  0, // CastP2X: 59
  0, // CastPP: 60
  0, // Catch: 61
  0, // CatchProj: 62
  0, // CheckCastPP: 63
  0, // ClearArray: 64
  0, // CompressBits: 65
  0, // ExpandBits: 66
  0, // CompressBitsV: 67
  0, // ExpandBitsV: 68
  0, // ConstraintCast: 69
  0, // CMoveD: 70
  0, // CMoveF: 71
  0, // CMoveI: 72
  0, // CMoveL: 73
  0, // CMoveP: 74
  0, // CMoveN: 75
  1, // CmpN: 76
  1, // CmpD: 77
  0, // CmpD3: 78
  1, // CmpF: 79
  0, // CmpF3: 80
  1, // CmpI: 81
  1, // CmpL: 82
  0, // CmpL3: 83
  0, // CmpLTMask: 84
  1, // CmpP: 85
  1, // CmpU: 86
  0, // CmpU3: 87
  1, // CmpUL: 88
  0, // CmpUL3: 89
  0, // CompareAndSwapB: 90
  0, // CompareAndSwapS: 91
  0, // CompareAndSwapI: 92
  0, // CompareAndSwapL: 93
  0, // CompareAndSwapP: 94
  0, // CompareAndSwapN: 95
  0, // WeakCompareAndSwapB: 96
  0, // WeakCompareAndSwapS: 97
  0, // WeakCompareAndSwapI: 98
  0, // WeakCompareAndSwapL: 99
  0, // WeakCompareAndSwapP: 100
  0, // WeakCompareAndSwapN: 101
  0, // CompareAndExchangeB: 102
  0, // CompareAndExchangeS: 103
  0, // CompareAndExchangeI: 104
  0, // CompareAndExchangeL: 105
  0, // CompareAndExchangeP: 106
  0, // CompareAndExchangeN: 107
  0, // GetAndAddB: 108
  0, // GetAndAddS: 109
  0, // GetAndAddI: 110
  0, // GetAndAddL: 111
  0, // GetAndSetB: 112
  0, // GetAndSetS: 113
  0, // GetAndSetI: 114
  0, // GetAndSetL: 115
  0, // GetAndSetP: 116
  0, // GetAndSetN: 117
  0, // Con: 118
  0, // ConN: 119
  0, // ConNKlass: 120
  0, // ConD: 121
  0, // ConF: 122
  0, // ConI: 123
  0, // ConL: 124
  0, // ConP: 125
  0, // Conv2B: 126
  0, // ConvD2F: 127
  0, // ConvD2I: 128
  0, // ConvD2L: 129
  0, // ConvF2D: 130
  0, // ConvF2I: 131
  0, // ConvF2L: 132
  0, // ConvI2D: 133
  0, // ConvI2F: 134
  0, // ConvI2L: 135
  0, // ConvL2D: 136
  0, // ConvL2F: 137
  0, // ConvL2I: 138
  0, // ConvF2HF: 139
  0, // ConvHF2F: 140
  0, // CountedLoop: 141
  0, // CountedLoopEnd: 142
  0, // OuterStripMinedLoop: 143
  0, // OuterStripMinedLoopEnd: 144
  0, // LongCountedLoop: 145
  0, // LongCountedLoopEnd: 146
  0, // CountLeadingZerosI: 147
  0, // CountLeadingZerosL: 148
  0, // CountLeadingZerosV: 149
  0, // CountTrailingZerosI: 150
  0, // CountTrailingZerosL: 151
  0, // CountTrailingZerosV: 152
  0, // CreateEx: 153
  0, // DecodeN: 154
  0, // DecodeNKlass: 155
  0, // DivD: 156
  0, // DivF: 157
  0, // DivI: 158
  0, // DivL: 159
  0, // UDivI: 160
  0, // UDivL: 161
  0, // DivMod: 162
  0, // DivModI: 163
  0, // DivModL: 164
  0, // UDivModI: 165
  0, // UDivModL: 166
  0, // EncodeISOArray: 167
  0, // EncodeP: 168
  0, // EncodePKlass: 169
  1, // FastLock: 170
  1, // FastUnlock: 171
  0, // FmaD: 172
  0, // FmaF: 173
  0, // Goto: 174
  0, // Halt: 175
  0, // CountPositives: 176
  0, // If: 177
  0, // RangeCheck: 178
  0, // IfFalse: 179
  0, // IfTrue: 180
  0, // Initialize: 181
  0, // JProj: 182
  0, // Jump: 183
  0, // JumpProj: 184
  0, // LShiftI: 185
  0, // LShiftL: 186
  0, // LoadB: 187
  0, // LoadUB: 188
  0, // LoadUS: 189
  0, // LoadD: 190
  0, // LoadD_unaligned: 191
  0, // LoadF: 192
  0, // LoadI: 193
  0, // LoadKlass: 194
  0, // LoadNKlass: 195
  0, // LoadL: 196
  0, // LoadL_unaligned: 197
  0, // LoadP: 198
  0, // LoadN: 199
  0, // LoadRange: 200
  0, // LoadS: 201
  0, // Lock: 202
  0, // Loop: 203
  0, // LoopLimit: 204
  0, // Mach: 205
  0, // MachNullCheck: 206
  0, // MachProj: 207
  0, // MulAddS2I: 208
  0, // MaxI: 209
  0, // MaxL: 210
  0, // MaxD: 211
  0, // MaxF: 212
  0, // MemBarAcquire: 213
  0, // LoadFence: 214
  0, // MemBarAcquireLock: 215
  0, // MemBarCPUOrder: 216
  0, // MemBarRelease: 217
  0, // StoreFence: 218
  0, // StoreStoreFence: 219
  0, // MemBarReleaseLock: 220
  0, // MemBarVolatile: 221
  0, // MemBarStoreStore: 222
  0, // MergeMem: 223
  0, // MinI: 224
  0, // MinL: 225
  0, // MinF: 226
  0, // MinD: 227
  0, // ModD: 228
  0, // ModF: 229
  0, // ModI: 230
  0, // ModL: 231
  0, // UModI: 232
  0, // UModL: 233
  0, // MoveI2F: 234
  0, // MoveF2I: 235
  0, // MoveL2D: 236
  0, // MoveD2L: 237
  0, // IsInfiniteF: 238
  0, // IsFiniteF: 239
  0, // IsInfiniteD: 240
  0, // IsFiniteD: 241
  0, // MulD: 242
  0, // MulF: 243
  0, // MulHiL: 244
  0, // UMulHiL: 245
  0, // MulI: 246
  0, // MulL: 247
  0, // Multi: 248
  0, // NegI: 249
  0, // NegL: 250
  0, // NegD: 251
  0, // NegF: 252
  0, // NeverBranch: 253
  0, // OnSpinWait: 254
  0, // Opaque1: 255
  0, // OpaqueLoopInit: 256
  0, // OpaqueLoopStride: 257
  0, // OpaqueZeroTripGuard: 258
  0, // Opaque3: 259
  0, // Opaque4: 260
  0, // ProfileBoolean: 261
  0, // OrI: 262
  0, // OrL: 263
  1, // OverflowAddI: 264
  1, // OverflowSubI: 265
  1, // OverflowMulI: 266
  1, // OverflowAddL: 267
  1, // OverflowSubL: 268
  1, // OverflowMulL: 269
  0, // PCTable: 270
  0, // Parm: 271
  0, // ParsePredicate: 272
  0, // PartialSubtypeCheck: 273
  0, // SubTypeCheck: 274
  0, // Phi: 275
  0, // PopCountI: 276
  0, // PopCountL: 277
  0, // PopCountVI: 278
  0, // PopCountVL: 279
  0, // PopulateIndex: 280
  0, // PrefetchAllocation: 281
  0, // Proj: 282
  0, // RShiftI: 283
  0, // RShiftL: 284
  0, // Region: 285
  0, // Rethrow: 286
  0, // Return: 287
  0, // ReverseI: 288
  0, // ReverseL: 289
  0, // ReverseV: 290
  0, // Root: 291
  0, // RoundDouble: 292
  0, // RoundDoubleMode: 293
  0, // RoundDoubleModeV: 294
  0, // RoundFloat: 295
  0, // RotateLeft: 296
  0, // RotateLeftV: 297
  0, // RotateRight: 298
  0, // RotateRightV: 299
  0, // SafePoint: 300
  0, // SafePointScalarObject: 301
  0, // ShenandoahCompareAndExchangeP: 302
  0, // ShenandoahCompareAndExchangeN: 303
  0, // ShenandoahCompareAndSwapN: 304
  0, // ShenandoahCompareAndSwapP: 305
  0, // ShenandoahWeakCompareAndSwapN: 306
  0, // ShenandoahWeakCompareAndSwapP: 307
  0, // ShenandoahIUBarrier: 308
  0, // ShenandoahLoadReferenceBarrier: 309
  0, // SCMemProj: 310
  0, // CopySignD: 311
  0, // CopySignF: 312
  0, // SignumD: 313
  0, // SignumF: 314
  0, // SignumVF: 315
  0, // SignumVD: 316
  0, // SqrtD: 317
  0, // SqrtF: 318
  0, // RoundF: 319
  0, // RoundD: 320
  0, // Start: 321
  0, // StartOSR: 322
  0, // StoreB: 323
  0, // StoreC: 324
  0, // StoreCM: 325
  0, // StoreD: 326
  0, // StoreF: 327
  0, // StoreI: 328
  0, // StoreL: 329
  0, // StoreP: 330
  0, // StoreN: 331
  0, // StoreNKlass: 332
  0, // StrComp: 333
  0, // StrCompressedCopy: 334
  0, // StrEquals: 335
  0, // StrIndexOf: 336
  0, // StrIndexOfChar: 337
  0, // StrInflatedCopy: 338
  0, // SubD: 339
  0, // SubF: 340
  0, // SubI: 341
  0, // SubL: 342
  0, // TailCall: 343
  0, // TailJump: 344
  0, // MacroLogicV: 345
  0, // ThreadLocal: 346
  0, // Unlock: 347
  0, // URShiftB: 348
  0, // URShiftS: 349
  0, // URShiftI: 350
  0, // URShiftL: 351
  0, // XorI: 352
  0, // XorL: 353
  0, // Vector: 354
  0, // AddVB: 355
  0, // AddVS: 356
  0, // AddVI: 357
  0, // AddReductionVI: 358
  0, // AddVL: 359
  0, // AddReductionVL: 360
  0, // AddVF: 361
  0, // AddReductionVF: 362
  0, // AddVD: 363
  0, // AddReductionVD: 364
  0, // SubVB: 365
  0, // SubVS: 366
  0, // SubVI: 367
  0, // SubVL: 368
  0, // SubVF: 369
  0, // SubVD: 370
  0, // MulVB: 371
  0, // MulVS: 372
  0, // MulVI: 373
  0, // MulReductionVI: 374
  0, // MulVL: 375
  0, // MulReductionVL: 376
  0, // MulVF: 377
  0, // MulReductionVF: 378
  0, // MulVD: 379
  0, // MulReductionVD: 380
  0, // MulAddVS2VI: 381
  0, // FmaVD: 382
  0, // FmaVF: 383
  0, // DivVF: 384
  0, // DivVD: 385
  0, // AbsVB: 386
  0, // AbsVS: 387
  0, // AbsVI: 388
  0, // AbsVL: 389
  0, // AbsVF: 390
  0, // AbsVD: 391
  0, // NegVI: 392
  0, // NegVL: 393
  0, // NegVF: 394
  0, // NegVD: 395
  0, // SqrtVD: 396
  0, // SqrtVF: 397
  0, // LShiftCntV: 398
  0, // RShiftCntV: 399
  0, // LShiftVB: 400
  0, // LShiftVS: 401
  0, // LShiftVI: 402
  0, // LShiftVL: 403
  0, // RShiftVB: 404
  0, // RShiftVS: 405
  0, // RShiftVI: 406
  0, // RShiftVL: 407
  0, // URShiftVB: 408
  0, // URShiftVS: 409
  0, // URShiftVI: 410
  0, // URShiftVL: 411
  0, // AndV: 412
  0, // AndReductionV: 413
  0, // OrV: 414
  0, // OrReductionV: 415
  0, // XorV: 416
  0, // XorReductionV: 417
  0, // MinV: 418
  0, // MaxV: 419
  0, // MinReductionV: 420
  0, // MaxReductionV: 421
  0, // CompressV: 422
  0, // CompressM: 423
  0, // ExpandV: 424
  0, // LoadVector: 425
  0, // LoadVectorGather: 426
  0, // LoadVectorGatherMasked: 427
  0, // StoreVector: 428
  0, // StoreVectorScatter: 429
  0, // StoreVectorScatterMasked: 430
  0, // LoadVectorMasked: 431
  0, // StoreVectorMasked: 432
  0, // VectorCmpMasked: 433
  0, // VectorMaskGen: 434
  0, // VectorMaskOp: 435
  0, // VectorMaskTrueCount: 436
  0, // VectorMaskFirstTrue: 437
  0, // VectorMaskLastTrue: 438
  0, // VectorMaskToLong: 439
  0, // VectorLongToMask: 440
  0, // Pack: 441
  0, // PackB: 442
  0, // PackS: 443
  0, // PackI: 444
  0, // PackL: 445
  0, // PackF: 446
  0, // PackD: 447
  0, // Pack2L: 448
  0, // Pack2D: 449
  0, // ReplicateB: 450
  0, // ReplicateS: 451
  0, // ReplicateI: 452
  0, // ReplicateL: 453
  0, // ReplicateF: 454
  0, // ReplicateD: 455
  0, // RoundVF: 456
  0, // RoundVD: 457
  0, // Extract: 458
  0, // ExtractB: 459
  0, // ExtractUB: 460
  0, // ExtractC: 461
  0, // ExtractS: 462
  0, // ExtractI: 463
  0, // ExtractL: 464
  0, // ExtractF: 465
  0, // ExtractD: 466
  0, // Digit: 467
  0, // LowerCase: 468
  0, // UpperCase: 469
  0, // Whitespace: 470
  0, // VectorBox: 471
  0, // VectorBoxAllocate: 472
  0, // VectorUnbox: 473
  0, // VectorMaskWrapper: 474
  0, // VectorMaskCmp: 475
  0, // VectorMaskCast: 476
  1, // VectorTest: 477
  0, // VectorBlend: 478
  0, // VectorRearrange: 479
  0, // VectorLoadMask: 480
  0, // VectorLoadShuffle: 481
  0, // VectorLoadConst: 482
  0, // VectorStoreMask: 483
  0, // VectorReinterpret: 484
  0, // VectorCast: 485
  0, // VectorCastB2X: 486
  0, // VectorCastS2X: 487
  0, // VectorCastI2X: 488
  0, // VectorCastL2X: 489
  0, // VectorCastF2X: 490
  0, // VectorCastD2X: 491
  0, // VectorCastF2HF: 492
  0, // VectorCastHF2F: 493
  0, // VectorUCastB2X: 494
  0, // VectorUCastS2X: 495
  0, // VectorUCastI2X: 496
  0, // VectorizedHashCode: 497
  0, // VectorInsert: 498
  0, // MaskAll: 499
  0, // AndVMask: 500
  0, // OrVMask: 501
  0 // no trailing comma // XorVMask: 502
};
//  The following instructions can cisc-spill
//  popCountI can cisc-spill operand 1 to popCountI_mem
//  popCountL can cisc-spill operand 1 to popCountL_mem
//  cmovI_reg can cisc-spill operand 4 to cmovI_mem
//  cmovI_regU can cisc-spill operand 4 to cmovI_memU
//  cmovI_regUCF can cisc-spill operand 4 to cmovI_memUCF
//  addI_eReg can cisc-spill operand 2 to addI_eReg_mem
//  subI_eReg can cisc-spill operand 2 to subI_eReg_mem
//  mulI_eReg can cisc-spill operand 2 to mulI
//  mulI_eReg_imm can cisc-spill operand 1 to mulI_mem_imm
//  i2b can cisc-spill operand 1 to loadI2B
//  i2s can cisc-spill operand 1 to loadI2S
//  andI_eReg can cisc-spill operand 2 to andI_eReg_mem
//  andnI_rReg_rReg_rReg can cisc-spill operand 3 to andnI_rReg_rReg_mem
//  andnI_rReg_rReg_rReg_0 can cisc-spill operand 1 to andnI_rReg_rReg_mem_0
//  orI_eReg can cisc-spill operand 2 to orI_eReg_mem
//  xorI_eReg can cisc-spill operand 2 to xorI_eReg_mem
//  addL_eReg can cisc-spill operand 2 to addL_eReg_mem
//  subL_eReg can cisc-spill operand 2 to subL_eReg_mem
//  andL_eReg can cisc-spill operand 2 to andL_eReg_mem
//  andnL_eReg_eReg_eReg can cisc-spill operand 3 to andnL_eReg_eReg_mem
//  andnL_eReg_eReg_eReg_0 can cisc-spill operand 1 to andnL_eReg_eReg_mem_0
//  orl_eReg can cisc-spill operand 2 to orl_eReg_mem
//  xorl_eReg can cisc-spill operand 2 to xorl_eReg_mem
//  cmpD_cc can cisc-spill operand 2 to cmpD_ccmem
//  cmpD_ccCF can cisc-spill operand 2 to cmpD_ccmemCF
//  cmpD_reg can cisc-spill operand 2 to cmpD_regmem
//  subDPR_reg can cisc-spill operand 2 to subDPR_reg_mem
//  addDPR_reg can cisc-spill operand 2 to addDPR_reg_mem
//  mulDPR_reg can cisc-spill operand 2 to mulDPR_reg_mem
//  cmpF_cc can cisc-spill operand 2 to cmpF_ccmem
//  cmpF_ccCF can cisc-spill operand 2 to cmpF_ccmemCF
//  cmpF_reg can cisc-spill operand 2 to cmpF_regmem
//  addFPR24_reg can cisc-spill operand 2 to addFPR24_reg_mem
//  addFPR_reg can cisc-spill operand 2 to addFPR_reg_mem
//  addFPR24_reg_mem can cisc-spill operand 1 to addFPR24_mem_cisc
//  addFPR24_reg_mem_0 can cisc-spill operand 2 to addFPR24_mem_cisc_0
//  addFPR24_mem_reg can cisc-spill operand 2 to addFPR24_mem_cisc
//  mulFPR24_reg can cisc-spill operand 2 to mulFPR24_reg_mem
//  mulFPR_reg can cisc-spill operand 2 to mulFPR_reg_mem
//  addFPR_mulFPR_reg can cisc-spill operand 1 to addFPR_mulFPR_reg_load1
//  addFPR_mulFPR_reg_0 can cisc-spill operand 2 to addFPR_mulFPR_reg_load1_1
//  convI2L_reg can cisc-spill operand 1 to loadI2L
//  convI2L_reg_zex can cisc-spill operand 1 to loadUI2L
//  compI_eReg can cisc-spill operand 2 to compI_eReg_mem
//  compU_eReg can cisc-spill operand 2 to compU_eReg_mem
//  compP_eReg can cisc-spill operand 2 to compP_eReg_mem
//  cmovLL_reg_LTGE can cisc-spill operand 4 to cmovLL_mem_LTGE
//  cmovLL_reg_LTGE_U can cisc-spill operand 4 to cmovLL_mem_LTGE_U
//  cmovII_reg_LTGE can cisc-spill operand 4 to cmovII_mem_LTGE
//  cmovII_reg_LTGE_U can cisc-spill operand 4 to cmovII_mem_LTGE_U
//  cmovLL_reg_EQNE can cisc-spill operand 4 to cmovLL_mem_EQNE
//  cmovII_reg_EQNE can cisc-spill operand 4 to cmovII_mem_EQNE
//  cmovII_reg_EQNE_U can cisc-spill operand 4 to cmovII_mem_EQNE_U
//  cmovLL_reg_LEGT can cisc-spill operand 4 to cmovLL_mem_LEGT
//  cmovLL_reg_LEGT_U can cisc-spill operand 4 to cmovLL_mem_LEGT_U
//  cmovII_reg_LEGT can cisc-spill operand 4 to cmovII_mem_LEGT
//  cmovII_reg_LEGT_U can cisc-spill operand 4 to cmovII_mem_LEGT_U
//  addF_reg can cisc-spill operand 2 to addF_mem
//  addF_reg_reg can cisc-spill operand 2 to addF_reg_mem
//  addD_reg can cisc-spill operand 2 to addD_mem
//  addD_reg_reg can cisc-spill operand 2 to addD_reg_mem
//  subF_reg can cisc-spill operand 2 to subF_mem
//  subF_reg_reg can cisc-spill operand 2 to subF_reg_mem
//  subD_reg can cisc-spill operand 2 to subD_mem
//  subD_reg_reg can cisc-spill operand 2 to subD_reg_mem
//  mulF_reg can cisc-spill operand 2 to mulF_mem
//  mulF_reg_reg can cisc-spill operand 2 to mulF_reg_mem
//  mulD_reg can cisc-spill operand 2 to mulD_mem
//  mulD_reg_reg can cisc-spill operand 2 to mulD_reg_mem
//  divF_reg can cisc-spill operand 2 to divF_mem
//  divF_reg_reg can cisc-spill operand 2 to divF_reg_mem
//  divD_reg can cisc-spill operand 2 to divD_mem
//  divD_reg_reg can cisc-spill operand 2 to divD_reg_mem
//  ReplI_reg can cisc-spill operand 1 to ReplI_mem
//  vReplF_reg can cisc-spill operand 1 to ReplF_mem
//  vReplD_reg can cisc-spill operand 1 to ReplD_mem
//  compressBitsI_reg can cisc-spill operand 2 to compressBitsI_mem
//  expandBitsI_reg can cisc-spill operand 2 to expandBitsI_mem



// An array of character pointers to machine register names.
const char *Matcher::regName[REG_COUNT] = {
  "ECX",
  "EBX",
  "EBP",
  "EDI",
  "EAX",
  "EDX",
  "ESI",
  "ESP",
  "FPR0L",
  "FPR0H",
  "FPR1L",
  "FPR1H",
  "FPR2L",
  "FPR2H",
  "FPR3L",
  "FPR3H",
  "FPR4L",
  "FPR4H",
  "FPR5L",
  "FPR5H",
  "FPR6L",
  "FPR6H",
  "FPR7L",
  "FPR7H",
  "FILL0",
  "FILL1",
  "FILL2",
  "FILL3",
  "FILL4",
  "FILL5",
  "FILL6",
  "FILL7",
  "XMM0",
  "XMM0b",
  "XMM0c",
  "XMM0d",
  "XMM0e",
  "XMM0f",
  "XMM0g",
  "XMM0h",
  "XMM0i",
  "XMM0j",
  "XMM0k",
  "XMM0l",
  "XMM0m",
  "XMM0n",
  "XMM0o",
  "XMM0p",
  "XMM1",
  "XMM1b",
  "XMM1c",
  "XMM1d",
  "XMM1e",
  "XMM1f",
  "XMM1g",
  "XMM1h",
  "XMM1i",
  "XMM1j",
  "XMM1k",
  "XMM1l",
  "XMM1m",
  "XMM1n",
  "XMM1o",
  "XMM1p",
  "XMM2",
  "XMM2b",
  "XMM2c",
  "XMM2d",
  "XMM2e",
  "XMM2f",
  "XMM2g",
  "XMM2h",
  "XMM2i",
  "XMM2j",
  "XMM2k",
  "XMM2l",
  "XMM2m",
  "XMM2n",
  "XMM2o",
  "XMM2p",
  "XMM3",
  "XMM3b",
  "XMM3c",
  "XMM3d",
  "XMM3e",
  "XMM3f",
  "XMM3g",
  "XMM3h",
  "XMM3i",
  "XMM3j",
  "XMM3k",
  "XMM3l",
  "XMM3m",
  "XMM3n",
  "XMM3o",
  "XMM3p",
  "XMM4",
  "XMM4b",
  "XMM4c",
  "XMM4d",
  "XMM4e",
  "XMM4f",
  "XMM4g",
  "XMM4h",
  "XMM4i",
  "XMM4j",
  "XMM4k",
  "XMM4l",
  "XMM4m",
  "XMM4n",
  "XMM4o",
  "XMM4p",
  "XMM5",
  "XMM5b",
  "XMM5c",
  "XMM5d",
  "XMM5e",
  "XMM5f",
  "XMM5g",
  "XMM5h",
  "XMM5i",
  "XMM5j",
  "XMM5k",
  "XMM5l",
  "XMM5m",
  "XMM5n",
  "XMM5o",
  "XMM5p",
  "XMM6",
  "XMM6b",
  "XMM6c",
  "XMM6d",
  "XMM6e",
  "XMM6f",
  "XMM6g",
  "XMM6h",
  "XMM6i",
  "XMM6j",
  "XMM6k",
  "XMM6l",
  "XMM6m",
  "XMM6n",
  "XMM6o",
  "XMM6p",
  "XMM7",
  "XMM7b",
  "XMM7c",
  "XMM7d",
  "XMM7e",
  "XMM7f",
  "XMM7g",
  "XMM7h",
  "XMM7i",
  "XMM7j",
  "XMM7k",
  "XMM7l",
  "XMM7m",
  "XMM7n",
  "XMM7o",
  "XMM7p",
  "K7",
  "K7_H",
  "K6",
  "K6_H",
  "K5",
  "K5_H",
  "K4",
  "K4_H",
  "K3",
  "K3_H",
  "K2",
  "K2_H",
  "K1",
  "K1_H",
  "RFLAGS" // no trailing comma
};

// An array of character pointers to machine register names.
const VMReg OptoReg::opto2vm[REG_COUNT] = {
	rcx->as_VMReg(),
	rbx->as_VMReg(),
	rbp->as_VMReg(),
	rdi->as_VMReg(),
	rax->as_VMReg(),
	rdx->as_VMReg(),
	rsi->as_VMReg(),
	rsp->as_VMReg(),
	VMRegImpl::Bad(),
	VMRegImpl::Bad(),
	as_FloatRegister(0)->as_VMReg(),
	as_FloatRegister(0)->as_VMReg()->next(),
	as_FloatRegister(1)->as_VMReg(),
	as_FloatRegister(1)->as_VMReg()->next(),
	as_FloatRegister(2)->as_VMReg(),
	as_FloatRegister(2)->as_VMReg()->next(),
	as_FloatRegister(3)->as_VMReg(),
	as_FloatRegister(3)->as_VMReg()->next(),
	as_FloatRegister(4)->as_VMReg(),
	as_FloatRegister(4)->as_VMReg()->next(),
	as_FloatRegister(5)->as_VMReg(),
	as_FloatRegister(5)->as_VMReg()->next(),
	as_FloatRegister(6)->as_VMReg(),
	as_FloatRegister(6)->as_VMReg()->next(),
	VMRegImpl::Bad(),
	VMRegImpl::Bad(),
	VMRegImpl::Bad(),
	VMRegImpl::Bad(),
	VMRegImpl::Bad(),
	VMRegImpl::Bad(),
	VMRegImpl::Bad(),
	VMRegImpl::Bad(),
	xmm0->as_VMReg(),
	xmm0->as_VMReg()->next(1),
	xmm0->as_VMReg()->next(2),
	xmm0->as_VMReg()->next(3),
	xmm0->as_VMReg()->next(4),
	xmm0->as_VMReg()->next(5),
	xmm0->as_VMReg()->next(6),
	xmm0->as_VMReg()->next(7),
	xmm0->as_VMReg()->next(8),
	xmm0->as_VMReg()->next(9),
	xmm0->as_VMReg()->next(10),
	xmm0->as_VMReg()->next(11),
	xmm0->as_VMReg()->next(12),
	xmm0->as_VMReg()->next(13),
	xmm0->as_VMReg()->next(14),
	xmm0->as_VMReg()->next(15),
	xmm1->as_VMReg(),
	xmm1->as_VMReg()->next(1),
	xmm1->as_VMReg()->next(2),
	xmm1->as_VMReg()->next(3),
	xmm1->as_VMReg()->next(4),
	xmm1->as_VMReg()->next(5),
	xmm1->as_VMReg()->next(6),
	xmm1->as_VMReg()->next(7),
	xmm1->as_VMReg()->next(8),
	xmm1->as_VMReg()->next(9),
	xmm1->as_VMReg()->next(10),
	xmm1->as_VMReg()->next(11),
	xmm1->as_VMReg()->next(12),
	xmm1->as_VMReg()->next(13),
	xmm1->as_VMReg()->next(14),
	xmm1->as_VMReg()->next(15),
	xmm2->as_VMReg(),
	xmm2->as_VMReg()->next(1),
	xmm2->as_VMReg()->next(2),
	xmm2->as_VMReg()->next(3),
	xmm2->as_VMReg()->next(4),
	xmm2->as_VMReg()->next(5),
	xmm2->as_VMReg()->next(6),
	xmm2->as_VMReg()->next(7),
	xmm2->as_VMReg()->next(8),
	xmm2->as_VMReg()->next(9),
	xmm2->as_VMReg()->next(10),
	xmm2->as_VMReg()->next(11),
	xmm2->as_VMReg()->next(12),
	xmm2->as_VMReg()->next(13),
	xmm2->as_VMReg()->next(14),
	xmm2->as_VMReg()->next(15),
	xmm3->as_VMReg(),
	xmm3->as_VMReg()->next(1),
	xmm3->as_VMReg()->next(2),
	xmm3->as_VMReg()->next(3),
	xmm3->as_VMReg()->next(4),
	xmm3->as_VMReg()->next(5),
	xmm3->as_VMReg()->next(6),
	xmm3->as_VMReg()->next(7),
	xmm3->as_VMReg()->next(8),
	xmm3->as_VMReg()->next(9),
	xmm3->as_VMReg()->next(10),
	xmm3->as_VMReg()->next(11),
	xmm3->as_VMReg()->next(12),
	xmm3->as_VMReg()->next(13),
	xmm3->as_VMReg()->next(14),
	xmm3->as_VMReg()->next(15),
	xmm4->as_VMReg(),
	xmm4->as_VMReg()->next(1),
	xmm4->as_VMReg()->next(2),
	xmm4->as_VMReg()->next(3),
	xmm4->as_VMReg()->next(4),
	xmm4->as_VMReg()->next(5),
	xmm4->as_VMReg()->next(6),
	xmm4->as_VMReg()->next(7),
	xmm4->as_VMReg()->next(8),
	xmm4->as_VMReg()->next(9),
	xmm4->as_VMReg()->next(10),
	xmm4->as_VMReg()->next(11),
	xmm4->as_VMReg()->next(12),
	xmm4->as_VMReg()->next(13),
	xmm4->as_VMReg()->next(14),
	xmm4->as_VMReg()->next(15),
	xmm5->as_VMReg(),
	xmm5->as_VMReg()->next(1),
	xmm5->as_VMReg()->next(2),
	xmm5->as_VMReg()->next(3),
	xmm5->as_VMReg()->next(4),
	xmm5->as_VMReg()->next(5),
	xmm5->as_VMReg()->next(6),
	xmm5->as_VMReg()->next(7),
	xmm5->as_VMReg()->next(8),
	xmm5->as_VMReg()->next(9),
	xmm5->as_VMReg()->next(10),
	xmm5->as_VMReg()->next(11),
	xmm5->as_VMReg()->next(12),
	xmm5->as_VMReg()->next(13),
	xmm5->as_VMReg()->next(14),
	xmm5->as_VMReg()->next(15),
	xmm6->as_VMReg(),
	xmm6->as_VMReg()->next(1),
	xmm6->as_VMReg()->next(2),
	xmm6->as_VMReg()->next(3),
	xmm6->as_VMReg()->next(4),
	xmm6->as_VMReg()->next(5),
	xmm6->as_VMReg()->next(6),
	xmm6->as_VMReg()->next(7),
	xmm6->as_VMReg()->next(8),
	xmm6->as_VMReg()->next(9),
	xmm6->as_VMReg()->next(10),
	xmm6->as_VMReg()->next(11),
	xmm6->as_VMReg()->next(12),
	xmm6->as_VMReg()->next(13),
	xmm6->as_VMReg()->next(14),
	xmm6->as_VMReg()->next(15),
	xmm7->as_VMReg(),
	xmm7->as_VMReg()->next(1),
	xmm7->as_VMReg()->next(2),
	xmm7->as_VMReg()->next(3),
	xmm7->as_VMReg()->next(4),
	xmm7->as_VMReg()->next(5),
	xmm7->as_VMReg()->next(6),
	xmm7->as_VMReg()->next(7),
	xmm7->as_VMReg()->next(8),
	xmm7->as_VMReg()->next(9),
	xmm7->as_VMReg()->next(10),
	xmm7->as_VMReg()->next(11),
	xmm7->as_VMReg()->next(12),
	xmm7->as_VMReg()->next(13),
	xmm7->as_VMReg()->next(14),
	xmm7->as_VMReg()->next(15),
	k7->as_VMReg(),
	k7->as_VMReg()->next(),
	k6->as_VMReg(),
	k6->as_VMReg()->next(),
	k5->as_VMReg(),
	k5->as_VMReg()->next(),
	k4->as_VMReg(),
	k4->as_VMReg()->next(),
	k3->as_VMReg(),
	k3->as_VMReg()->next(),
	k2->as_VMReg(),
	k2->as_VMReg()->next(),
	k1->as_VMReg(),
	k1->as_VMReg()->next(),
	VMRegImpl::Bad() // no trailing comma
	};

 OptoReg::Name OptoReg::vm2opto[ConcreteRegisterImpl::number_of_registers];

// An array of the machine register encode values
const unsigned char Matcher::_regEncode[REG_COUNT] = {
  (unsigned char)'\x1',  // ECX
  (unsigned char)'\x3',  // EBX
  (unsigned char)'\x5',  // EBP
  (unsigned char)'\x7',  // EDI
  (unsigned char)'\x0',  // EAX
  (unsigned char)'\x2',  // EDX
  (unsigned char)'\x6',  // ESI
  (unsigned char)'\x4',  // ESP
  (unsigned char)'\x0',  // FPR0L
  (unsigned char)'\x0',  // FPR0H
  (unsigned char)'\x1',  // FPR1L
  (unsigned char)'\x1',  // FPR1H
  (unsigned char)'\x2',  // FPR2L
  (unsigned char)'\x2',  // FPR2H
  (unsigned char)'\x3',  // FPR3L
  (unsigned char)'\x3',  // FPR3H
  (unsigned char)'\x4',  // FPR4L
  (unsigned char)'\x4',  // FPR4H
  (unsigned char)'\x5',  // FPR5L
  (unsigned char)'\x5',  // FPR5H
  (unsigned char)'\x6',  // FPR6L
  (unsigned char)'\x6',  // FPR6H
  (unsigned char)'\x7',  // FPR7L
  (unsigned char)'\x7',  // FPR7H
  (unsigned char)'\x8',  // FILL0
  (unsigned char)'\x9',  // FILL1
  (unsigned char)'\xA',  // FILL2
  (unsigned char)'\xB',  // FILL3
  (unsigned char)'\xC',  // FILL4
  (unsigned char)'\xD',  // FILL5
  (unsigned char)'\xE',  // FILL6
  (unsigned char)'\xF',  // FILL7
  (unsigned char)'\x0',  // XMM0
  (unsigned char)'\x0',  // XMM0b
  (unsigned char)'\x0',  // XMM0c
  (unsigned char)'\x0',  // XMM0d
  (unsigned char)'\x0',  // XMM0e
  (unsigned char)'\x0',  // XMM0f
  (unsigned char)'\x0',  // XMM0g
  (unsigned char)'\x0',  // XMM0h
  (unsigned char)'\x0',  // XMM0i
  (unsigned char)'\x0',  // XMM0j
  (unsigned char)'\x0',  // XMM0k
  (unsigned char)'\x0',  // XMM0l
  (unsigned char)'\x0',  // XMM0m
  (unsigned char)'\x0',  // XMM0n
  (unsigned char)'\x0',  // XMM0o
  (unsigned char)'\x0',  // XMM0p
  (unsigned char)'\x1',  // XMM1
  (unsigned char)'\x1',  // XMM1b
  (unsigned char)'\x1',  // XMM1c
  (unsigned char)'\x1',  // XMM1d
  (unsigned char)'\x1',  // XMM1e
  (unsigned char)'\x1',  // XMM1f
  (unsigned char)'\x1',  // XMM1g
  (unsigned char)'\x1',  // XMM1h
  (unsigned char)'\x1',  // XMM1i
  (unsigned char)'\x1',  // XMM1j
  (unsigned char)'\x1',  // XMM1k
  (unsigned char)'\x1',  // XMM1l
  (unsigned char)'\x1',  // XMM1m
  (unsigned char)'\x1',  // XMM1n
  (unsigned char)'\x1',  // XMM1o
  (unsigned char)'\x1',  // XMM1p
  (unsigned char)'\x2',  // XMM2
  (unsigned char)'\x2',  // XMM2b
  (unsigned char)'\x2',  // XMM2c
  (unsigned char)'\x2',  // XMM2d
  (unsigned char)'\x2',  // XMM2e
  (unsigned char)'\x2',  // XMM2f
  (unsigned char)'\x2',  // XMM2g
  (unsigned char)'\x2',  // XMM2h
  (unsigned char)'\x2',  // XMM2i
  (unsigned char)'\x2',  // XMM2j
  (unsigned char)'\x2',  // XMM2k
  (unsigned char)'\x2',  // XMM2l
  (unsigned char)'\x2',  // XMM2m
  (unsigned char)'\x2',  // XMM2n
  (unsigned char)'\x2',  // XMM2o
  (unsigned char)'\x2',  // XMM2p
  (unsigned char)'\x3',  // XMM3
  (unsigned char)'\x3',  // XMM3b
  (unsigned char)'\x3',  // XMM3c
  (unsigned char)'\x3',  // XMM3d
  (unsigned char)'\x3',  // XMM3e
  (unsigned char)'\x3',  // XMM3f
  (unsigned char)'\x3',  // XMM3g
  (unsigned char)'\x3',  // XMM3h
  (unsigned char)'\x3',  // XMM3i
  (unsigned char)'\x3',  // XMM3j
  (unsigned char)'\x3',  // XMM3k
  (unsigned char)'\x3',  // XMM3l
  (unsigned char)'\x3',  // XMM3m
  (unsigned char)'\x3',  // XMM3n
  (unsigned char)'\x3',  // XMM3o
  (unsigned char)'\x3',  // XMM3p
  (unsigned char)'\x4',  // XMM4
  (unsigned char)'\x4',  // XMM4b
  (unsigned char)'\x4',  // XMM4c
  (unsigned char)'\x4',  // XMM4d
  (unsigned char)'\x4',  // XMM4e
  (unsigned char)'\x4',  // XMM4f
  (unsigned char)'\x4',  // XMM4g
  (unsigned char)'\x4',  // XMM4h
  (unsigned char)'\x4',  // XMM4i
  (unsigned char)'\x4',  // XMM4j
  (unsigned char)'\x4',  // XMM4k
  (unsigned char)'\x4',  // XMM4l
  (unsigned char)'\x4',  // XMM4m
  (unsigned char)'\x4',  // XMM4n
  (unsigned char)'\x4',  // XMM4o
  (unsigned char)'\x4',  // XMM4p
  (unsigned char)'\x5',  // XMM5
  (unsigned char)'\x5',  // XMM5b
  (unsigned char)'\x5',  // XMM5c
  (unsigned char)'\x5',  // XMM5d
  (unsigned char)'\x5',  // XMM5e
  (unsigned char)'\x5',  // XMM5f
  (unsigned char)'\x5',  // XMM5g
  (unsigned char)'\x5',  // XMM5h
  (unsigned char)'\x5',  // XMM5i
  (unsigned char)'\x5',  // XMM5j
  (unsigned char)'\x5',  // XMM5k
  (unsigned char)'\x5',  // XMM5l
  (unsigned char)'\x5',  // XMM5m
  (unsigned char)'\x5',  // XMM5n
  (unsigned char)'\x5',  // XMM5o
  (unsigned char)'\x5',  // XMM5p
  (unsigned char)'\x6',  // XMM6
  (unsigned char)'\x6',  // XMM6b
  (unsigned char)'\x6',  // XMM6c
  (unsigned char)'\x6',  // XMM6d
  (unsigned char)'\x6',  // XMM6e
  (unsigned char)'\x6',  // XMM6f
  (unsigned char)'\x6',  // XMM6g
  (unsigned char)'\x6',  // XMM6h
  (unsigned char)'\x6',  // XMM6i
  (unsigned char)'\x6',  // XMM6j
  (unsigned char)'\x6',  // XMM6k
  (unsigned char)'\x6',  // XMM6l
  (unsigned char)'\x6',  // XMM6m
  (unsigned char)'\x6',  // XMM6n
  (unsigned char)'\x6',  // XMM6o
  (unsigned char)'\x6',  // XMM6p
  (unsigned char)'\x7',  // XMM7
  (unsigned char)'\x7',  // XMM7b
  (unsigned char)'\x7',  // XMM7c
  (unsigned char)'\x7',  // XMM7d
  (unsigned char)'\x7',  // XMM7e
  (unsigned char)'\x7',  // XMM7f
  (unsigned char)'\x7',  // XMM7g
  (unsigned char)'\x7',  // XMM7h
  (unsigned char)'\x7',  // XMM7i
  (unsigned char)'\x7',  // XMM7j
  (unsigned char)'\x7',  // XMM7k
  (unsigned char)'\x7',  // XMM7l
  (unsigned char)'\x7',  // XMM7m
  (unsigned char)'\x7',  // XMM7n
  (unsigned char)'\x7',  // XMM7o
  (unsigned char)'\x7',  // XMM7p
  (unsigned char)'\x7',  // K7
  (unsigned char)'\x7',  // K7_H
  (unsigned char)'\x6',  // K6
  (unsigned char)'\x6',  // K6_H
  (unsigned char)'\x5',  // K5
  (unsigned char)'\x5',  // K5_H
  (unsigned char)'\x4',  // K4
  (unsigned char)'\x4',  // K4_H
  (unsigned char)'\x3',  // K3
  (unsigned char)'\x3',  // K3_H
  (unsigned char)'\x2',  // K2
  (unsigned char)'\x2',  // K2_H
  (unsigned char)'\x1',  // K1
  (unsigned char)'\x1',  // K1_H
  (unsigned char)'\x8' // no trailing comma  // RFLAGS
};


//------------------Define classes derived from MachOper---------------------
MachOper  *labelOper::clone() const {
  return  new labelOper(_label, _block_num);
}
uint labelOper::opcode() const { return LABEL; }

const RegMask *sRegIOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &(Compile::current()->FIRST_STACK_mask());
}

const RegMask *sRegPOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &(Compile::current()->FIRST_STACK_mask());
}

const RegMask *sRegFOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &(Compile::current()->FIRST_STACK_mask());
}

const RegMask *sRegDOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &(Compile::current()->FIRST_STACK_mask());
}

const RegMask *sRegLOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &(Compile::current()->FIRST_STACK_mask());
}

MachOper  *methodOper::clone() const {
  return  new methodOper(_method);
}
uint methodOper::opcode() const { return METHOD; }

const RegMask *kRegOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &VECTMASK_REG_mask();
}

const RegMask *kReg_K1Oper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &VECTMASK_REG_K1_mask();
}

const RegMask *kReg_K2Oper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &VECTMASK_REG_K2_mask();
}

const RegMask *kReg_K3Oper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &VECTMASK_REG_K3_mask();
}

const RegMask *kReg_K4Oper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &VECTMASK_REG_K4_mask();
}

const RegMask *kReg_K5Oper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &VECTMASK_REG_K5_mask();
}

const RegMask *kReg_K6Oper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &VECTMASK_REG_K6_mask();
}

const RegMask *kReg_K7Oper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &VECTMASK_REG_K7_mask();
}

const RegMask *rRegIOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &INT_REG_mask();
}

const RegMask *xRegIOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &INT_X_REG_mask();
}

const RegMask *eAXRegIOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &EAX_REG_mask();
}

const RegMask *eBXRegIOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &EBX_REG_mask();
}

const RegMask *eCXRegIOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &ECX_REG_mask();
}

const RegMask *eDXRegIOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &EDX_REG_mask();
}

const RegMask *eDIRegIOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &EDI_REG_mask();
}

const RegMask *naxRegIOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &NAX_REG_mask();
}

const RegMask *nadxRegIOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &NADX_REG_mask();
}

const RegMask *ncxRegIOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &NCX_REG_mask();
}

const RegMask *eSIRegIOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &ESI_REG_mask();
}

const RegMask *anyRegPOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &ANY_REG_mask();
}

const RegMask *eRegPOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &INT_REG_mask();
}

const RegMask *rRegPOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &INT_REG_mask();
}

const RegMask *eRegP_no_EBPOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &INT_REG_NO_EBP_mask();
}

const RegMask *naxRegPOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &NAX_REG_mask();
}

const RegMask *nabxRegPOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &NABX_REG_mask();
}

const RegMask *pRegPOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &P_REG_mask();
}

const RegMask *eAXRegPOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &EAX_REG_mask();
}

const RegMask *eBXRegPOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &EBX_REG_mask();
}

const RegMask *eCXRegPOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &ECX_REG_mask();
}

const RegMask *eDXRegPOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &EDX_REG_mask();
}

const RegMask *eSIRegPOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &ESI_REG_mask();
}

const RegMask *eDIRegPOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &EDI_REG_mask();
}

const RegMask *eRegLOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &LONG_REG_mask();
}

const RegMask *eADXRegLOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &EADX_REG_mask();
}

const RegMask *eBCXRegLOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &EBCX_REG_mask();
}

const RegMask *eBDPRegLOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &EBPD_REG_mask();
}

const RegMask *eADXRegL_low_onlyOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &EADX_REG_mask();
}

const RegMask *rFlagsRegOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &INT_FLAGS_mask();
}

const RegMask *eFlagsRegOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &INT_FLAGS_mask();
}

const RegMask *eFlagsRegUOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &INT_FLAGS_mask();
}

const RegMask *eFlagsRegUCFOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &INT_FLAGS_mask();
}

const RegMask *flagsReg_long_LTGEOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &INT_FLAGS_mask();
}

const RegMask *flagsReg_long_EQNEOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &INT_FLAGS_mask();
}

const RegMask *flagsReg_long_LEGTOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &INT_FLAGS_mask();
}

const RegMask *flagsReg_ulong_LTGEOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &INT_FLAGS_mask();
}

const RegMask *flagsReg_ulong_EQNEOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &INT_FLAGS_mask();
}

const RegMask *flagsReg_ulong_LEGTOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &INT_FLAGS_mask();
}

const RegMask *regDPROper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &FP_DBL_REG_mask();
}

const RegMask *regDPR1Oper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &FP_DBL_REG0_mask();
}

const RegMask *regDPR2Oper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &FP_DBL_REG1_mask();
}

const RegMask *regnotDPR1Oper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &FP_DBL_NOTREG0_mask();
}

const RegMask *regFPROper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &FP_FLT_REG_mask();
}

const RegMask *regFPR1Oper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &FP_FLT_REG0_mask();
}

const RegMask *regFOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &FLOAT_REG_LEGACY_mask();
}

const RegMask *legRegFOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &FLOAT_REG_LEGACY_mask();
}

const RegMask *vlRegFOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &FLOAT_REG_VL_mask();
}

const RegMask *regDOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &DOUBLE_REG_LEGACY_mask();
}

const RegMask *legRegDOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &DOUBLE_REG_LEGACY_mask();
}

const RegMask *vlRegDOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &DOUBLE_REG_VL_mask();
}

const RegMask *indirectOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &INT_REG_mask();
}

const RegMask *indOffset8Oper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &INT_REG_mask();
}

const RegMask *indOffset32Oper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &INT_REG_mask();
}

const RegMask *indOffset32XOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &INT_REG_mask();
}

const RegMask *indIndexOffsetOper::in_RegMask(int index) const {
  assert(0 <= index && index < 2, "index out of range");
  return &INT_REG_mask();
}

const RegMask *indIndexOper::in_RegMask(int index) const {
  assert(0 <= index && index < 2, "index out of range");
  return &INT_REG_mask();
}

const RegMask *indIndexScaleOper::in_RegMask(int index) const {
  assert(0 <= index && index < 2, "index out of range");
  return &INT_REG_mask();
}

const RegMask *indIndexScaleOffsetOper::in_RegMask(int index) const {
  assert(0 <= index && index < 2, "index out of range");
  return &INT_REG_mask();
}

const RegMask *load_long_RegPOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &ESI_REG_mask();
}

const RegMask *load_long_indirectOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &ESI_REG_mask();
}

const RegMask *load_long_indOffset32Oper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &ESI_REG_mask();
}

const RegMask *stackSlotPOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &(Compile::current()->FIRST_STACK_mask());
}

const RegMask *stackSlotIOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &(Compile::current()->FIRST_STACK_mask());
}

const RegMask *stackSlotFOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &(Compile::current()->FIRST_STACK_mask());
}

const RegMask *stackSlotDOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &(Compile::current()->FIRST_STACK_mask());
}

const RegMask *stackSlotLOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &(Compile::current()->FIRST_STACK_mask());
}

const RegMask *rxmm0Oper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &XMM0_REG_mask();
}

const RegMask *vecOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &RegMask::Empty;
}

const RegMask *legVecOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &RegMask::Empty;
}

const RegMask *vecSOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &VECTORS_REG_VLBWDQ_mask();
}

const RegMask *legVecSOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &VECTORS_REG_LEGACY_mask();
}

const RegMask *vecDOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &VECTORD_REG_VLBWDQ_mask();
}

const RegMask *legVecDOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &VECTORD_REG_LEGACY_mask();
}

const RegMask *vecXOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &VECTORX_REG_VLBWDQ_mask();
}

const RegMask *legVecXOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &VECTORX_REG_LEGACY_mask();
}

const RegMask *vecYOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &VECTORY_REG_VLBWDQ_mask();
}

const RegMask *legVecYOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &VECTORY_REG_LEGACY_mask();
}

const RegMask *vecZOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &VECTORZ_REG_mask();
}

const RegMask *legVecZOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &VECTORZ_REG_LEGACY_mask();
}

//------------------Define members for classes derived from MachNode----------

void popCountINode::use_cisc_RegMask() {
  _cisc_RegMask = &STACK_OR_INT_REG_mask();
}

// Build CISC version of this instruction
MachNode *popCountINode::cisc_version(int offset) {
  popCountI_memNode *node = new popCountI_memNode();

  // Copy _idx, inputs and operands to new node
  fill_new_machnode(node);
  // Construct operand to access [stack_pointer + offset]
  node->set_opnd_array(cisc_operand(), new indOffset32Oper(offset));

  return node;
}


void popCountLNode::use_cisc_RegMask() {
  _cisc_RegMask = &STACK_OR_LONG_REG_mask();
}

// Build CISC version of this instruction
MachNode *popCountLNode::cisc_version(int offset) {
  popCountL_memNode *node = new popCountL_memNode();

  // Copy _idx, inputs and operands to new node
  fill_new_machnode(node);
  // Construct operand to access [stack_pointer + offset]
  node->set_opnd_array(cisc_operand(), new indOffset32Oper(offset));

  return node;
}


void cmovI_regNode::use_cisc_RegMask() {
  _cisc_RegMask = &STACK_OR_INT_REG_mask();
}

// Build CISC version of this instruction
MachNode *cmovI_regNode::cisc_version(int offset) {
  cmovI_memNode *node = new cmovI_memNode();

  // Copy _idx, inputs and operands to new node
  fill_new_machnode(node);
  // Construct operand to access [stack_pointer + offset]
  node->set_opnd_array(cisc_operand(), new indOffset32Oper(offset));

  return node;
}


void cmovI_regUNode::use_cisc_RegMask() {
  _cisc_RegMask = &STACK_OR_INT_REG_mask();
}

// Build CISC version of this instruction
MachNode *cmovI_regUNode::cisc_version(int offset) {
  cmovI_memUNode *node = new cmovI_memUNode();

  // Copy _idx, inputs and operands to new node
  fill_new_machnode(node);
  // Construct operand to access [stack_pointer + offset]
  node->set_opnd_array(cisc_operand(), new indOffset32Oper(offset));

  return node;
}


void cmovI_regUCFNode::use_cisc_RegMask() {
  _cisc_RegMask = &STACK_OR_INT_REG_mask();
}

// Build CISC version of this instruction
MachNode *cmovI_regUCFNode::cisc_version(int offset) {
  cmovI_memUCFNode *node = new cmovI_memUCFNode();

  // Copy _idx, inputs and operands to new node
  fill_new_machnode(node);
  // Construct operand to access [stack_pointer + offset]
  node->set_opnd_array(cisc_operand(), new indOffset32Oper(offset));

  return node;
}


void addI_eRegNode::use_cisc_RegMask() {
  _cisc_RegMask = &STACK_OR_INT_REG_mask();
}

// Build CISC version of this instruction
MachNode *addI_eRegNode::cisc_version(int offset) {
  addI_eReg_memNode *node = new addI_eReg_memNode();

  // Copy _idx, inputs and operands to new node
  fill_new_machnode(node);
  // Construct operand to access [stack_pointer + offset]
  node->set_opnd_array(cisc_operand(), new indOffset32Oper(offset));

  return node;
}


void subI_eRegNode::use_cisc_RegMask() {
  _cisc_RegMask = &STACK_OR_INT_REG_mask();
}

// Build CISC version of this instruction
MachNode *subI_eRegNode::cisc_version(int offset) {
  subI_eReg_memNode *node = new subI_eReg_memNode();

  // Copy _idx, inputs and operands to new node
  fill_new_machnode(node);
  // Construct operand to access [stack_pointer + offset]
  node->set_opnd_array(cisc_operand(), new indOffset32Oper(offset));

  return node;
}


void mulI_eRegNode::use_cisc_RegMask() {
  _cisc_RegMask = &STACK_OR_INT_REG_mask();
}

// Build CISC version of this instruction
MachNode *mulI_eRegNode::cisc_version(int offset) {
  mulINode *node = new mulINode();

  // Copy _idx, inputs and operands to new node
  fill_new_machnode(node);
  // Construct operand to access [stack_pointer + offset]
  node->set_opnd_array(cisc_operand(), new indOffset32Oper(offset));

  return node;
}


void mulI_eReg_immNode::use_cisc_RegMask() {
  _cisc_RegMask = &STACK_OR_INT_REG_mask();
}

// Build CISC version of this instruction
MachNode *mulI_eReg_immNode::cisc_version(int offset) {
  mulI_mem_immNode *node = new mulI_mem_immNode();

  // Copy _idx, inputs and operands to new node
  fill_new_machnode(node);
  // Construct operand to access [stack_pointer + offset]
  node->set_opnd_array(cisc_operand(), new indOffset32Oper(offset));

  return node;
}


void i2bNode::use_cisc_RegMask() {
  _cisc_RegMask = &STACK_OR_INT_X_REG_mask();
}

// Build CISC version of this instruction
MachNode *i2bNode::cisc_version(int offset) {
  loadI2BNode *node = new loadI2BNode();
  node->_num_opnds = 3;

  // Copy _idx, inputs and operands to new node
  fill_new_machnode(node);
  // Construct operand to access [stack_pointer + offset]
  node->set_opnd_array(cisc_operand(), new indOffset32Oper(offset));

  return node;
}


void i2sNode::use_cisc_RegMask() {
  _cisc_RegMask = &STACK_OR_INT_X_REG_mask();
}

// Build CISC version of this instruction
MachNode *i2sNode::cisc_version(int offset) {
  loadI2SNode *node = new loadI2SNode();
  node->_num_opnds = 3;

  // Copy _idx, inputs and operands to new node
  fill_new_machnode(node);
  // Construct operand to access [stack_pointer + offset]
  node->set_opnd_array(cisc_operand(), new indOffset32Oper(offset));

  return node;
}


void andI_eRegNode::use_cisc_RegMask() {
  _cisc_RegMask = &STACK_OR_INT_REG_mask();
}

// Build CISC version of this instruction
MachNode *andI_eRegNode::cisc_version(int offset) {
  andI_eReg_memNode *node = new andI_eReg_memNode();

  // Copy _idx, inputs and operands to new node
  fill_new_machnode(node);
  // Construct operand to access [stack_pointer + offset]
  node->set_opnd_array(cisc_operand(), new indOffset32Oper(offset));

  return node;
}


void andnI_rReg_rReg_rRegNode::use_cisc_RegMask() {
  _cisc_RegMask = &STACK_OR_INT_REG_mask();
}

// Build CISC version of this instruction
MachNode *andnI_rReg_rReg_rRegNode::cisc_version(int offset) {
  andnI_rReg_rReg_memNode *node = new andnI_rReg_rReg_memNode();

  // Copy _idx, inputs and operands to new node
  fill_new_machnode(node);
  // Construct operand to access [stack_pointer + offset]
  node->set_opnd_array(cisc_operand(), new indOffset32Oper(offset));

  return node;
}


void andnI_rReg_rReg_rReg_0Node::use_cisc_RegMask() {
  _cisc_RegMask = &STACK_OR_INT_REG_mask();
}

// Build CISC version of this instruction
MachNode *andnI_rReg_rReg_rReg_0Node::cisc_version(int offset) {
  andnI_rReg_rReg_mem_0Node *node = new andnI_rReg_rReg_mem_0Node();

  // Copy _idx, inputs and operands to new node
  fill_new_machnode(node);
  // Construct operand to access [stack_pointer + offset]
  node->set_opnd_array(cisc_operand(), new indOffset32Oper(offset));

  return node;
}


void orI_eRegNode::use_cisc_RegMask() {
  _cisc_RegMask = &STACK_OR_INT_REG_mask();
}

// Build CISC version of this instruction
MachNode *orI_eRegNode::cisc_version(int offset) {
  orI_eReg_memNode *node = new orI_eReg_memNode();

  // Copy _idx, inputs and operands to new node
  fill_new_machnode(node);
  // Construct operand to access [stack_pointer + offset]
  node->set_opnd_array(cisc_operand(), new indOffset32Oper(offset));

  return node;
}


void xorI_eRegNode::use_cisc_RegMask() {
  _cisc_RegMask = &STACK_OR_INT_REG_mask();
}

// Build CISC version of this instruction
MachNode *xorI_eRegNode::cisc_version(int offset) {
  xorI_eReg_memNode *node = new xorI_eReg_memNode();

  // Copy _idx, inputs and operands to new node
  fill_new_machnode(node);
  // Construct operand to access [stack_pointer + offset]
  node->set_opnd_array(cisc_operand(), new indOffset32Oper(offset));

  return node;
}


void addL_eRegNode::use_cisc_RegMask() {
  _cisc_RegMask = &STACK_OR_LONG_REG_mask();
}

// Build CISC version of this instruction
MachNode *addL_eRegNode::cisc_version(int offset) {
  addL_eReg_memNode *node = new addL_eReg_memNode();

  // Copy _idx, inputs and operands to new node
  fill_new_machnode(node);
  // Construct operand to access [stack_pointer + offset]
  node->set_opnd_array(cisc_operand(), new indOffset32Oper(offset));

  return node;
}


void subL_eRegNode::use_cisc_RegMask() {
  _cisc_RegMask = &STACK_OR_LONG_REG_mask();
}

// Build CISC version of this instruction
MachNode *subL_eRegNode::cisc_version(int offset) {
  subL_eReg_memNode *node = new subL_eReg_memNode();

  // Copy _idx, inputs and operands to new node
  fill_new_machnode(node);
  // Construct operand to access [stack_pointer + offset]
  node->set_opnd_array(cisc_operand(), new indOffset32Oper(offset));

  return node;
}


void andL_eRegNode::use_cisc_RegMask() {
  _cisc_RegMask = &STACK_OR_LONG_REG_mask();
}

// Build CISC version of this instruction
MachNode *andL_eRegNode::cisc_version(int offset) {
  andL_eReg_memNode *node = new andL_eReg_memNode();

  // Copy _idx, inputs and operands to new node
  fill_new_machnode(node);
  // Construct operand to access [stack_pointer + offset]
  node->set_opnd_array(cisc_operand(), new indOffset32Oper(offset));

  return node;
}


void andnL_eReg_eReg_eRegNode::use_cisc_RegMask() {
  _cisc_RegMask = &STACK_OR_LONG_REG_mask();
}

// Build CISC version of this instruction
MachNode *andnL_eReg_eReg_eRegNode::cisc_version(int offset) {
  andnL_eReg_eReg_memNode *node = new andnL_eReg_eReg_memNode();

  // Copy _idx, inputs and operands to new node
  fill_new_machnode(node);
  // Construct operand to access [stack_pointer + offset]
  node->set_opnd_array(cisc_operand(), new indOffset32Oper(offset));

  return node;
}


void andnL_eReg_eReg_eReg_0Node::use_cisc_RegMask() {
  _cisc_RegMask = &STACK_OR_LONG_REG_mask();
}

// Build CISC version of this instruction
MachNode *andnL_eReg_eReg_eReg_0Node::cisc_version(int offset) {
  andnL_eReg_eReg_mem_0Node *node = new andnL_eReg_eReg_mem_0Node();

  // Copy _idx, inputs and operands to new node
  fill_new_machnode(node);
  // Construct operand to access [stack_pointer + offset]
  node->set_opnd_array(cisc_operand(), new indOffset32Oper(offset));

  return node;
}


void orl_eRegNode::use_cisc_RegMask() {
  _cisc_RegMask = &STACK_OR_LONG_REG_mask();
}

// Build CISC version of this instruction
MachNode *orl_eRegNode::cisc_version(int offset) {
  orl_eReg_memNode *node = new orl_eReg_memNode();

  // Copy _idx, inputs and operands to new node
  fill_new_machnode(node);
  // Construct operand to access [stack_pointer + offset]
  node->set_opnd_array(cisc_operand(), new indOffset32Oper(offset));

  return node;
}


void xorl_eRegNode::use_cisc_RegMask() {
  _cisc_RegMask = &STACK_OR_LONG_REG_mask();
}

// Build CISC version of this instruction
MachNode *xorl_eRegNode::cisc_version(int offset) {
  xorl_eReg_memNode *node = new xorl_eReg_memNode();

  // Copy _idx, inputs and operands to new node
  fill_new_machnode(node);
  // Construct operand to access [stack_pointer + offset]
  node->set_opnd_array(cisc_operand(), new indOffset32Oper(offset));

  return node;
}


void cmpD_ccNode::use_cisc_RegMask() {
  _cisc_RegMask = &STACK_OR_DOUBLE_REG_LEGACY_mask();
}

// Build CISC version of this instruction
MachNode *cmpD_ccNode::cisc_version(int offset) {
  cmpD_ccmemNode *node = new cmpD_ccmemNode();

  // Copy _idx, inputs and operands to new node
  fill_new_machnode(node);
  // Construct operand to access [stack_pointer + offset]
  node->set_opnd_array(cisc_operand(), new indOffset32Oper(offset));

  return node;
}


void cmpD_ccCFNode::use_cisc_RegMask() {
  _cisc_RegMask = &STACK_OR_DOUBLE_REG_LEGACY_mask();
}

// Build CISC version of this instruction
MachNode *cmpD_ccCFNode::cisc_version(int offset) {
  cmpD_ccmemCFNode *node = new cmpD_ccmemCFNode();

  // Copy _idx, inputs and operands to new node
  fill_new_machnode(node);
  // Construct operand to access [stack_pointer + offset]
  node->set_opnd_array(cisc_operand(), new indOffset32Oper(offset));

  return node;
}


void cmpD_regNode::use_cisc_RegMask() {
  _cisc_RegMask = &STACK_OR_DOUBLE_REG_LEGACY_mask();
}

// Build CISC version of this instruction
MachNode *cmpD_regNode::cisc_version(int offset) {
  cmpD_regmemNode *node = new cmpD_regmemNode();

  // Copy _idx, inputs and operands to new node
  fill_new_machnode(node);
  // Construct operand to access [stack_pointer + offset]
  node->set_opnd_array(cisc_operand(), new indOffset32Oper(offset));

  return node;
}


void subDPR_regNode::use_cisc_RegMask() {
  _cisc_RegMask = &STACK_OR_FP_DBL_REG_mask();
}

// Build CISC version of this instruction
MachNode *subDPR_regNode::cisc_version(int offset) {
  subDPR_reg_memNode *node = new subDPR_reg_memNode();

  // Copy _idx, inputs and operands to new node
  fill_new_machnode(node);
  // Construct operand to access [stack_pointer + offset]
  node->set_opnd_array(cisc_operand(), new indOffset32Oper(offset));

  return node;
}


void addDPR_regNode::use_cisc_RegMask() {
  _cisc_RegMask = &STACK_OR_FP_DBL_REG_mask();
}

// Build CISC version of this instruction
MachNode *addDPR_regNode::cisc_version(int offset) {
  addDPR_reg_memNode *node = new addDPR_reg_memNode();

  // Copy _idx, inputs and operands to new node
  fill_new_machnode(node);
  // Construct operand to access [stack_pointer + offset]
  node->set_opnd_array(cisc_operand(), new indOffset32Oper(offset));

  return node;
}


void mulDPR_regNode::use_cisc_RegMask() {
  _cisc_RegMask = &STACK_OR_FP_DBL_REG_mask();
}

// Build CISC version of this instruction
MachNode *mulDPR_regNode::cisc_version(int offset) {
  mulDPR_reg_memNode *node = new mulDPR_reg_memNode();

  // Copy _idx, inputs and operands to new node
  fill_new_machnode(node);
  // Construct operand to access [stack_pointer + offset]
  node->set_opnd_array(cisc_operand(), new indOffset32Oper(offset));

  return node;
}


void cmpF_ccNode::use_cisc_RegMask() {
  _cisc_RegMask = &STACK_OR_FLOAT_REG_LEGACY_mask();
}

// Build CISC version of this instruction
MachNode *cmpF_ccNode::cisc_version(int offset) {
  cmpF_ccmemNode *node = new cmpF_ccmemNode();

  // Copy _idx, inputs and operands to new node
  fill_new_machnode(node);
  // Construct operand to access [stack_pointer + offset]
  node->set_opnd_array(cisc_operand(), new indOffset32Oper(offset));

  return node;
}


void cmpF_ccCFNode::use_cisc_RegMask() {
  _cisc_RegMask = &STACK_OR_FLOAT_REG_LEGACY_mask();
}

// Build CISC version of this instruction
MachNode *cmpF_ccCFNode::cisc_version(int offset) {
  cmpF_ccmemCFNode *node = new cmpF_ccmemCFNode();

  // Copy _idx, inputs and operands to new node
  fill_new_machnode(node);
  // Construct operand to access [stack_pointer + offset]
  node->set_opnd_array(cisc_operand(), new indOffset32Oper(offset));

  return node;
}


void cmpF_regNode::use_cisc_RegMask() {
  _cisc_RegMask = &STACK_OR_FLOAT_REG_LEGACY_mask();
}

// Build CISC version of this instruction
MachNode *cmpF_regNode::cisc_version(int offset) {
  cmpF_regmemNode *node = new cmpF_regmemNode();

  // Copy _idx, inputs and operands to new node
  fill_new_machnode(node);
  // Construct operand to access [stack_pointer + offset]
  node->set_opnd_array(cisc_operand(), new indOffset32Oper(offset));

  return node;
}


void addFPR24_regNode::use_cisc_RegMask() {
  _cisc_RegMask = &STACK_OR_FP_FLT_REG_mask();
}

// Build CISC version of this instruction
MachNode *addFPR24_regNode::cisc_version(int offset) {
  addFPR24_reg_memNode *node = new addFPR24_reg_memNode();

  // Copy _idx, inputs and operands to new node
  fill_new_machnode(node);
  // Construct operand to access [stack_pointer + offset]
  node->set_opnd_array(cisc_operand(), new indOffset32Oper(offset));

  return node;
}


void addFPR_regNode::use_cisc_RegMask() {
  _cisc_RegMask = &STACK_OR_FP_FLT_REG_mask();
}

// Build CISC version of this instruction
MachNode *addFPR_regNode::cisc_version(int offset) {
  addFPR_reg_memNode *node = new addFPR_reg_memNode();

  // Copy _idx, inputs and operands to new node
  fill_new_machnode(node);
  // Construct operand to access [stack_pointer + offset]
  node->set_opnd_array(cisc_operand(), new indOffset32Oper(offset));

  return node;
}


void addFPR24_reg_memNode::use_cisc_RegMask() {
  _cisc_RegMask = &STACK_OR_FP_FLT_REG_mask();
}

// Build CISC version of this instruction
MachNode *addFPR24_reg_memNode::cisc_version(int offset) {
  addFPR24_mem_ciscNode *node = new addFPR24_mem_ciscNode();

  // Copy _idx, inputs and operands to new node
  fill_new_machnode(node);
  // Construct operand to access [stack_pointer + offset]
  node->set_opnd_array(cisc_operand(), new indOffset32Oper(offset));

  return node;
}


void addFPR24_reg_mem_0Node::use_cisc_RegMask() {
  _cisc_RegMask = &STACK_OR_FP_FLT_REG_mask();
}

// Build CISC version of this instruction
MachNode *addFPR24_reg_mem_0Node::cisc_version(int offset) {
  addFPR24_mem_cisc_0Node *node = new addFPR24_mem_cisc_0Node();

  // Copy _idx, inputs and operands to new node
  fill_new_machnode(node);
  // Construct operand to access [stack_pointer + offset]
  node->set_opnd_array(cisc_operand(), new indOffset32Oper(offset));

  return node;
}


void addFPR24_mem_regNode::use_cisc_RegMask() {
  _cisc_RegMask = &STACK_OR_FP_FLT_REG_mask();
}

// Build CISC version of this instruction
MachNode *addFPR24_mem_regNode::cisc_version(int offset) {
  addFPR24_mem_ciscNode *node = new addFPR24_mem_ciscNode();

  // Copy _idx, inputs and operands to new node
  fill_new_machnode(node);
  // Construct operand to access [stack_pointer + offset]
  node->set_opnd_array(cisc_operand(), new indOffset32Oper(offset));

  return node;
}


void mulFPR24_regNode::use_cisc_RegMask() {
  _cisc_RegMask = &STACK_OR_FP_FLT_REG_mask();
}

// Build CISC version of this instruction
MachNode *mulFPR24_regNode::cisc_version(int offset) {
  mulFPR24_reg_memNode *node = new mulFPR24_reg_memNode();

  // Copy _idx, inputs and operands to new node
  fill_new_machnode(node);
  // Construct operand to access [stack_pointer + offset]
  node->set_opnd_array(cisc_operand(), new indOffset32Oper(offset));

  return node;
}


void mulFPR_regNode::use_cisc_RegMask() {
  _cisc_RegMask = &STACK_OR_FP_FLT_REG_mask();
}

// Build CISC version of this instruction
MachNode *mulFPR_regNode::cisc_version(int offset) {
  mulFPR_reg_memNode *node = new mulFPR_reg_memNode();

  // Copy _idx, inputs and operands to new node
  fill_new_machnode(node);
  // Construct operand to access [stack_pointer + offset]
  node->set_opnd_array(cisc_operand(), new indOffset32Oper(offset));

  return node;
}


void addFPR_mulFPR_regNode::use_cisc_RegMask() {
  _cisc_RegMask = &STACK_OR_FP_FLT_REG_mask();
}

// Build CISC version of this instruction
MachNode *addFPR_mulFPR_regNode::cisc_version(int offset) {
  addFPR_mulFPR_reg_load1Node *node = new addFPR_mulFPR_reg_load1Node();

  // Copy _idx, inputs and operands to new node
  fill_new_machnode(node);
  // Construct operand to access [stack_pointer + offset]
  node->set_opnd_array(cisc_operand(), new indOffset32Oper(offset));

  return node;
}


void addFPR_mulFPR_reg_0Node::use_cisc_RegMask() {
  _cisc_RegMask = &STACK_OR_FP_FLT_REG_mask();
}

// Build CISC version of this instruction
MachNode *addFPR_mulFPR_reg_0Node::cisc_version(int offset) {
  addFPR_mulFPR_reg_load1_1Node *node = new addFPR_mulFPR_reg_load1_1Node();

  // Copy _idx, inputs and operands to new node
  fill_new_machnode(node);
  // Construct operand to access [stack_pointer + offset]
  node->set_opnd_array(cisc_operand(), new indOffset32Oper(offset));

  return node;
}


void convI2L_regNode::use_cisc_RegMask() {
  _cisc_RegMask = &STACK_OR_INT_REG_mask();
}

// Build CISC version of this instruction
MachNode *convI2L_regNode::cisc_version(int offset) {
  loadI2LNode *node = new loadI2LNode();

  // Copy _idx, inputs and operands to new node
  fill_new_machnode(node);
  // Construct operand to access [stack_pointer + offset]
  node->set_opnd_array(cisc_operand(), new indOffset32Oper(offset));

  return node;
}


void convI2L_reg_zexNode::use_cisc_RegMask() {
  _cisc_RegMask = &STACK_OR_INT_REG_mask();
}

// Build CISC version of this instruction
MachNode *convI2L_reg_zexNode::cisc_version(int offset) {
  loadUI2LNode *node = new loadUI2LNode();

  // Copy _idx, inputs and operands to new node
  fill_new_machnode(node);
  // Construct operand to access [stack_pointer + offset]
  node->set_opnd_array(cisc_operand(), new indOffset32Oper(offset));

  return node;
}


void compI_eRegNode::use_cisc_RegMask() {
  _cisc_RegMask = &STACK_OR_INT_REG_mask();
}

// Build CISC version of this instruction
MachNode *compI_eRegNode::cisc_version(int offset) {
  compI_eReg_memNode *node = new compI_eReg_memNode();

  // Copy _idx, inputs and operands to new node
  fill_new_machnode(node);
  // Construct operand to access [stack_pointer + offset]
  node->set_opnd_array(cisc_operand(), new indOffset32Oper(offset));

  return node;
}


void compU_eRegNode::use_cisc_RegMask() {
  _cisc_RegMask = &STACK_OR_INT_REG_mask();
}

// Build CISC version of this instruction
MachNode *compU_eRegNode::cisc_version(int offset) {
  compU_eReg_memNode *node = new compU_eReg_memNode();

  // Copy _idx, inputs and operands to new node
  fill_new_machnode(node);
  // Construct operand to access [stack_pointer + offset]
  node->set_opnd_array(cisc_operand(), new indOffset32Oper(offset));

  return node;
}


void compP_eRegNode::use_cisc_RegMask() {
  _cisc_RegMask = &STACK_OR_INT_REG_mask();
}

// Build CISC version of this instruction
MachNode *compP_eRegNode::cisc_version(int offset) {
  compP_eReg_memNode *node = new compP_eReg_memNode();

  // Copy _idx, inputs and operands to new node
  fill_new_machnode(node);
  // Construct operand to access [stack_pointer + offset]
  node->set_opnd_array(cisc_operand(), new indOffset32Oper(offset));

  return node;
}

// Build short branch version of this instruction
MachNode *jmpDirNode::short_branch_version() {
  jmpDir_shortNode *node = new jmpDir_shortNode();

  // Copy _idx, inputs and operands to new node
  fill_new_machnode(node);
  return node;
}

// Build short branch version of this instruction
MachNode *jmpConNode::short_branch_version() {
  jmpCon_shortNode *node = new jmpCon_shortNode();
  node->_prob = _prob;
  node->_fcnt = _fcnt;

  // Copy _idx, inputs and operands to new node
  fill_new_machnode(node);
  return node;
}

// Build short branch version of this instruction
MachNode *jmpLoopEndNode::short_branch_version() {
  jmpLoopEnd_shortNode *node = new jmpLoopEnd_shortNode();
  node->_prob = _prob;
  node->_fcnt = _fcnt;

  // Copy _idx, inputs and operands to new node
  fill_new_machnode(node);
  return node;
}

// Build short branch version of this instruction
MachNode *jmpConUNode::short_branch_version() {
  jmpConU_shortNode *node = new jmpConU_shortNode();
  node->_prob = _prob;
  node->_fcnt = _fcnt;

  // Copy _idx, inputs and operands to new node
  fill_new_machnode(node);
  return node;
}

// Build short branch version of this instruction
MachNode *jmpConUCFNode::short_branch_version() {
  jmpConUCF_shortNode *node = new jmpConUCF_shortNode();
  node->_prob = _prob;
  node->_fcnt = _fcnt;

  // Copy _idx, inputs and operands to new node
  fill_new_machnode(node);
  return node;
}

// Build short branch version of this instruction
MachNode *jmpConUCF2Node::short_branch_version() {
  jmpConUCF2_shortNode *node = new jmpConUCF2_shortNode();
  node->_prob = _prob;
  node->_fcnt = _fcnt;

  // Copy _idx, inputs and operands to new node
  fill_new_machnode(node);
  return node;
}


void cmovLL_reg_LTGENode::use_cisc_RegMask() {
  _cisc_RegMask = &STACK_OR_LONG_REG_mask();
}

// Build CISC version of this instruction
MachNode *cmovLL_reg_LTGENode::cisc_version(int offset) {
  cmovLL_mem_LTGENode *node = new cmovLL_mem_LTGENode();

  // Copy _idx, inputs and operands to new node
  fill_new_machnode(node);
  // Construct operand to access [stack_pointer + offset]
  node->set_opnd_array(cisc_operand(), new indOffset32Oper(offset));

  return node;
}


void cmovLL_reg_LTGE_UNode::use_cisc_RegMask() {
  _cisc_RegMask = &STACK_OR_LONG_REG_mask();
}

// Build CISC version of this instruction
MachNode *cmovLL_reg_LTGE_UNode::cisc_version(int offset) {
  cmovLL_mem_LTGE_UNode *node = new cmovLL_mem_LTGE_UNode();

  // Copy _idx, inputs and operands to new node
  fill_new_machnode(node);
  // Construct operand to access [stack_pointer + offset]
  node->set_opnd_array(cisc_operand(), new indOffset32Oper(offset));

  return node;
}


void cmovII_reg_LTGENode::use_cisc_RegMask() {
  _cisc_RegMask = &STACK_OR_INT_REG_mask();
}

// Build CISC version of this instruction
MachNode *cmovII_reg_LTGENode::cisc_version(int offset) {
  cmovII_mem_LTGENode *node = new cmovII_mem_LTGENode();

  // Copy _idx, inputs and operands to new node
  fill_new_machnode(node);
  // Construct operand to access [stack_pointer + offset]
  node->set_opnd_array(cisc_operand(), new indOffset32Oper(offset));

  return node;
}


void cmovII_reg_LTGE_UNode::use_cisc_RegMask() {
  _cisc_RegMask = &STACK_OR_INT_REG_mask();
}

// Build CISC version of this instruction
MachNode *cmovII_reg_LTGE_UNode::cisc_version(int offset) {
  cmovII_mem_LTGE_UNode *node = new cmovII_mem_LTGE_UNode();

  // Copy _idx, inputs and operands to new node
  fill_new_machnode(node);
  // Construct operand to access [stack_pointer + offset]
  node->set_opnd_array(cisc_operand(), new indOffset32Oper(offset));

  return node;
}


void cmovLL_reg_EQNENode::use_cisc_RegMask() {
  _cisc_RegMask = &STACK_OR_LONG_REG_mask();
}

// Build CISC version of this instruction
MachNode *cmovLL_reg_EQNENode::cisc_version(int offset) {
  cmovLL_mem_EQNENode *node = new cmovLL_mem_EQNENode();

  // Copy _idx, inputs and operands to new node
  fill_new_machnode(node);
  // Construct operand to access [stack_pointer + offset]
  node->set_opnd_array(cisc_operand(), new indOffset32Oper(offset));

  return node;
}


void cmovII_reg_EQNENode::use_cisc_RegMask() {
  _cisc_RegMask = &STACK_OR_INT_REG_mask();
}

// Build CISC version of this instruction
MachNode *cmovII_reg_EQNENode::cisc_version(int offset) {
  cmovII_mem_EQNENode *node = new cmovII_mem_EQNENode();

  // Copy _idx, inputs and operands to new node
  fill_new_machnode(node);
  // Construct operand to access [stack_pointer + offset]
  node->set_opnd_array(cisc_operand(), new indOffset32Oper(offset));

  return node;
}


void cmovII_reg_EQNE_UNode::use_cisc_RegMask() {
  _cisc_RegMask = &STACK_OR_INT_REG_mask();
}

// Build CISC version of this instruction
MachNode *cmovII_reg_EQNE_UNode::cisc_version(int offset) {
  cmovII_mem_EQNE_UNode *node = new cmovII_mem_EQNE_UNode();

  // Copy _idx, inputs and operands to new node
  fill_new_machnode(node);
  // Construct operand to access [stack_pointer + offset]
  node->set_opnd_array(cisc_operand(), new indOffset32Oper(offset));

  return node;
}


void cmovLL_reg_LEGTNode::use_cisc_RegMask() {
  _cisc_RegMask = &STACK_OR_LONG_REG_mask();
}

// Build CISC version of this instruction
MachNode *cmovLL_reg_LEGTNode::cisc_version(int offset) {
  cmovLL_mem_LEGTNode *node = new cmovLL_mem_LEGTNode();

  // Copy _idx, inputs and operands to new node
  fill_new_machnode(node);
  // Construct operand to access [stack_pointer + offset]
  node->set_opnd_array(cisc_operand(), new indOffset32Oper(offset));

  return node;
}


void cmovLL_reg_LEGT_UNode::use_cisc_RegMask() {
  _cisc_RegMask = &STACK_OR_LONG_REG_mask();
}

// Build CISC version of this instruction
MachNode *cmovLL_reg_LEGT_UNode::cisc_version(int offset) {
  cmovLL_mem_LEGT_UNode *node = new cmovLL_mem_LEGT_UNode();

  // Copy _idx, inputs and operands to new node
  fill_new_machnode(node);
  // Construct operand to access [stack_pointer + offset]
  node->set_opnd_array(cisc_operand(), new indOffset32Oper(offset));

  return node;
}


void cmovII_reg_LEGTNode::use_cisc_RegMask() {
  _cisc_RegMask = &STACK_OR_INT_REG_mask();
}

// Build CISC version of this instruction
MachNode *cmovII_reg_LEGTNode::cisc_version(int offset) {
  cmovII_mem_LEGTNode *node = new cmovII_mem_LEGTNode();

  // Copy _idx, inputs and operands to new node
  fill_new_machnode(node);
  // Construct operand to access [stack_pointer + offset]
  node->set_opnd_array(cisc_operand(), new indOffset32Oper(offset));

  return node;
}


void cmovII_reg_LEGT_UNode::use_cisc_RegMask() {
  _cisc_RegMask = &STACK_OR_INT_REG_mask();
}

// Build CISC version of this instruction
MachNode *cmovII_reg_LEGT_UNode::cisc_version(int offset) {
  cmovII_mem_LEGT_UNode *node = new cmovII_mem_LEGT_UNode();

  // Copy _idx, inputs and operands to new node
  fill_new_machnode(node);
  // Construct operand to access [stack_pointer + offset]
  node->set_opnd_array(cisc_operand(), new indOffset32Oper(offset));

  return node;
}


void addF_regNode::use_cisc_RegMask() {
  _cisc_RegMask = &STACK_OR_FLOAT_REG_LEGACY_mask();
}

// Build CISC version of this instruction
MachNode *addF_regNode::cisc_version(int offset) {
  addF_memNode *node = new addF_memNode();

  // Copy _idx, inputs and operands to new node
  fill_new_machnode(node);
  // Construct operand to access [stack_pointer + offset]
  node->set_opnd_array(cisc_operand(), new indOffset32Oper(offset));

  return node;
}


void addF_reg_regNode::use_cisc_RegMask() {
  _cisc_RegMask = &STACK_OR_FLOAT_REG_LEGACY_mask();
}

// Build CISC version of this instruction
MachNode *addF_reg_regNode::cisc_version(int offset) {
  addF_reg_memNode *node = new addF_reg_memNode();

  // Copy _idx, inputs and operands to new node
  fill_new_machnode(node);
  // Construct operand to access [stack_pointer + offset]
  node->set_opnd_array(cisc_operand(), new indOffset32Oper(offset));

  return node;
}


void addD_regNode::use_cisc_RegMask() {
  _cisc_RegMask = &STACK_OR_DOUBLE_REG_LEGACY_mask();
}

// Build CISC version of this instruction
MachNode *addD_regNode::cisc_version(int offset) {
  addD_memNode *node = new addD_memNode();

  // Copy _idx, inputs and operands to new node
  fill_new_machnode(node);
  // Construct operand to access [stack_pointer + offset]
  node->set_opnd_array(cisc_operand(), new indOffset32Oper(offset));

  return node;
}


void addD_reg_regNode::use_cisc_RegMask() {
  _cisc_RegMask = &STACK_OR_DOUBLE_REG_LEGACY_mask();
}

// Build CISC version of this instruction
MachNode *addD_reg_regNode::cisc_version(int offset) {
  addD_reg_memNode *node = new addD_reg_memNode();

  // Copy _idx, inputs and operands to new node
  fill_new_machnode(node);
  // Construct operand to access [stack_pointer + offset]
  node->set_opnd_array(cisc_operand(), new indOffset32Oper(offset));

  return node;
}


void subF_regNode::use_cisc_RegMask() {
  _cisc_RegMask = &STACK_OR_FLOAT_REG_LEGACY_mask();
}

// Build CISC version of this instruction
MachNode *subF_regNode::cisc_version(int offset) {
  subF_memNode *node = new subF_memNode();

  // Copy _idx, inputs and operands to new node
  fill_new_machnode(node);
  // Construct operand to access [stack_pointer + offset]
  node->set_opnd_array(cisc_operand(), new indOffset32Oper(offset));

  return node;
}


void subF_reg_regNode::use_cisc_RegMask() {
  _cisc_RegMask = &STACK_OR_FLOAT_REG_LEGACY_mask();
}

// Build CISC version of this instruction
MachNode *subF_reg_regNode::cisc_version(int offset) {
  subF_reg_memNode *node = new subF_reg_memNode();

  // Copy _idx, inputs and operands to new node
  fill_new_machnode(node);
  // Construct operand to access [stack_pointer + offset]
  node->set_opnd_array(cisc_operand(), new indOffset32Oper(offset));

  return node;
}


void subD_regNode::use_cisc_RegMask() {
  _cisc_RegMask = &STACK_OR_DOUBLE_REG_LEGACY_mask();
}

// Build CISC version of this instruction
MachNode *subD_regNode::cisc_version(int offset) {
  subD_memNode *node = new subD_memNode();

  // Copy _idx, inputs and operands to new node
  fill_new_machnode(node);
  // Construct operand to access [stack_pointer + offset]
  node->set_opnd_array(cisc_operand(), new indOffset32Oper(offset));

  return node;
}


void subD_reg_regNode::use_cisc_RegMask() {
  _cisc_RegMask = &STACK_OR_DOUBLE_REG_LEGACY_mask();
}

// Build CISC version of this instruction
MachNode *subD_reg_regNode::cisc_version(int offset) {
  subD_reg_memNode *node = new subD_reg_memNode();

  // Copy _idx, inputs and operands to new node
  fill_new_machnode(node);
  // Construct operand to access [stack_pointer + offset]
  node->set_opnd_array(cisc_operand(), new indOffset32Oper(offset));

  return node;
}


void mulF_regNode::use_cisc_RegMask() {
  _cisc_RegMask = &STACK_OR_FLOAT_REG_LEGACY_mask();
}

// Build CISC version of this instruction
MachNode *mulF_regNode::cisc_version(int offset) {
  mulF_memNode *node = new mulF_memNode();

  // Copy _idx, inputs and operands to new node
  fill_new_machnode(node);
  // Construct operand to access [stack_pointer + offset]
  node->set_opnd_array(cisc_operand(), new indOffset32Oper(offset));

  return node;
}


void mulF_reg_regNode::use_cisc_RegMask() {
  _cisc_RegMask = &STACK_OR_FLOAT_REG_LEGACY_mask();
}

// Build CISC version of this instruction
MachNode *mulF_reg_regNode::cisc_version(int offset) {
  mulF_reg_memNode *node = new mulF_reg_memNode();

  // Copy _idx, inputs and operands to new node
  fill_new_machnode(node);
  // Construct operand to access [stack_pointer + offset]
  node->set_opnd_array(cisc_operand(), new indOffset32Oper(offset));

  return node;
}


void mulD_regNode::use_cisc_RegMask() {
  _cisc_RegMask = &STACK_OR_DOUBLE_REG_LEGACY_mask();
}

// Build CISC version of this instruction
MachNode *mulD_regNode::cisc_version(int offset) {
  mulD_memNode *node = new mulD_memNode();

  // Copy _idx, inputs and operands to new node
  fill_new_machnode(node);
  // Construct operand to access [stack_pointer + offset]
  node->set_opnd_array(cisc_operand(), new indOffset32Oper(offset));

  return node;
}


void mulD_reg_regNode::use_cisc_RegMask() {
  _cisc_RegMask = &STACK_OR_DOUBLE_REG_LEGACY_mask();
}

// Build CISC version of this instruction
MachNode *mulD_reg_regNode::cisc_version(int offset) {
  mulD_reg_memNode *node = new mulD_reg_memNode();

  // Copy _idx, inputs and operands to new node
  fill_new_machnode(node);
  // Construct operand to access [stack_pointer + offset]
  node->set_opnd_array(cisc_operand(), new indOffset32Oper(offset));

  return node;
}


void divF_regNode::use_cisc_RegMask() {
  _cisc_RegMask = &STACK_OR_FLOAT_REG_LEGACY_mask();
}

// Build CISC version of this instruction
MachNode *divF_regNode::cisc_version(int offset) {
  divF_memNode *node = new divF_memNode();

  // Copy _idx, inputs and operands to new node
  fill_new_machnode(node);
  // Construct operand to access [stack_pointer + offset]
  node->set_opnd_array(cisc_operand(), new indOffset32Oper(offset));

  return node;
}


void divF_reg_regNode::use_cisc_RegMask() {
  _cisc_RegMask = &STACK_OR_FLOAT_REG_LEGACY_mask();
}

// Build CISC version of this instruction
MachNode *divF_reg_regNode::cisc_version(int offset) {
  divF_reg_memNode *node = new divF_reg_memNode();

  // Copy _idx, inputs and operands to new node
  fill_new_machnode(node);
  // Construct operand to access [stack_pointer + offset]
  node->set_opnd_array(cisc_operand(), new indOffset32Oper(offset));

  return node;
}


void divD_regNode::use_cisc_RegMask() {
  _cisc_RegMask = &STACK_OR_DOUBLE_REG_LEGACY_mask();
}

// Build CISC version of this instruction
MachNode *divD_regNode::cisc_version(int offset) {
  divD_memNode *node = new divD_memNode();

  // Copy _idx, inputs and operands to new node
  fill_new_machnode(node);
  // Construct operand to access [stack_pointer + offset]
  node->set_opnd_array(cisc_operand(), new indOffset32Oper(offset));

  return node;
}


void divD_reg_regNode::use_cisc_RegMask() {
  _cisc_RegMask = &STACK_OR_DOUBLE_REG_LEGACY_mask();
}

// Build CISC version of this instruction
MachNode *divD_reg_regNode::cisc_version(int offset) {
  divD_reg_memNode *node = new divD_reg_memNode();

  // Copy _idx, inputs and operands to new node
  fill_new_machnode(node);
  // Construct operand to access [stack_pointer + offset]
  node->set_opnd_array(cisc_operand(), new indOffset32Oper(offset));

  return node;
}


void ReplI_regNode::use_cisc_RegMask() {
  _cisc_RegMask = &STACK_OR_INT_REG_mask();
}

// Build CISC version of this instruction
MachNode *ReplI_regNode::cisc_version(int offset) {
  ReplI_memNode *node = new ReplI_memNode();
  node->_bottom_type = bottom_type();

  // Copy _idx, inputs and operands to new node
  fill_new_machnode(node);
  // Construct operand to access [stack_pointer + offset]
  node->set_opnd_array(cisc_operand(), new indOffset32Oper(offset));

  return node;
}


void vReplF_regNode::use_cisc_RegMask() {
  _cisc_RegMask = &STACK_OR_FLOAT_REG_VL_mask();
}

// Build CISC version of this instruction
MachNode *vReplF_regNode::cisc_version(int offset) {
  ReplF_memNode *node = new ReplF_memNode();
  node->_bottom_type = bottom_type();

  // Copy _idx, inputs and operands to new node
  fill_new_machnode(node);
  // Construct operand to access [stack_pointer + offset]
  node->set_opnd_array(cisc_operand(), new indOffset32Oper(offset));

  return node;
}


void vReplD_regNode::use_cisc_RegMask() {
  _cisc_RegMask = &STACK_OR_DOUBLE_REG_VL_mask();
}

// Build CISC version of this instruction
MachNode *vReplD_regNode::cisc_version(int offset) {
  ReplD_memNode *node = new ReplD_memNode();
  node->_bottom_type = bottom_type();

  // Copy _idx, inputs and operands to new node
  fill_new_machnode(node);
  // Construct operand to access [stack_pointer + offset]
  node->set_opnd_array(cisc_operand(), new indOffset32Oper(offset));

  return node;
}


void compressBitsI_regNode::use_cisc_RegMask() {
  _cisc_RegMask = &STACK_OR_INT_REG_mask();
}

// Build CISC version of this instruction
MachNode *compressBitsI_regNode::cisc_version(int offset) {
  compressBitsI_memNode *node = new compressBitsI_memNode();

  // Copy _idx, inputs and operands to new node
  fill_new_machnode(node);
  // Construct operand to access [stack_pointer + offset]
  node->set_opnd_array(cisc_operand(), new indOffset32Oper(offset));

  return node;
}


void expandBitsI_regNode::use_cisc_RegMask() {
  _cisc_RegMask = &STACK_OR_INT_REG_mask();
}

// Build CISC version of this instruction
MachNode *expandBitsI_regNode::cisc_version(int offset) {
  expandBitsI_memNode *node = new expandBitsI_memNode();

  // Copy _idx, inputs and operands to new node
  fill_new_machnode(node);
  // Construct operand to access [stack_pointer + offset]
  node->set_opnd_array(cisc_operand(), new indOffset32Oper(offset));

  return node;
}


// Copy _idx, inputs and operands to new node
void MachNode::fill_new_machnode(MachNode* node) const {
  // New node must use same node index
  node->set_idx( _idx );
  // Copy machine-independent inputs
  for( uint j = 0; j < req(); j++ ) {
    node->add_req(in(j));
  }
  // Copy my operands, except for cisc position
  int nopnds = num_opnds();
  assert( node->num_opnds() == (uint)nopnds, "Must have same number of operands");
  MachOper **to = node->_opnds;
  for( int i = 0; i < nopnds; i++ ) {
    if( i != cisc_operand() ) 
      to[i] = _opnds[i]->clone();
  }
  // Do not increment node index counter, since node reuses my index
  Compile* C = Compile::current();
  C->set_unique(C->unique() - 1);
}

void jmpDirNode::label_set( Label* label, uint block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(1));
  oper->_label     = label;
  oper->_block_num = block_num;
}
void jmpDirNode::save_label( Label** label, uint* block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(1));
  *label = oper->_label;
  *block_num = oper->_block_num;
}
void jmpConNode::label_set( Label* label, uint block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(3));
  oper->_label     = label;
  oper->_block_num = block_num;
}
void jmpConNode::save_label( Label** label, uint* block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(3));
  *label = oper->_label;
  *block_num = oper->_block_num;
}
void jmpLoopEndNode::label_set( Label* label, uint block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(3));
  oper->_label     = label;
  oper->_block_num = block_num;
}
void jmpLoopEndNode::save_label( Label** label, uint* block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(3));
  *label = oper->_label;
  *block_num = oper->_block_num;
}
void jmpConUNode::label_set( Label* label, uint block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(3));
  oper->_label     = label;
  oper->_block_num = block_num;
}
void jmpConUNode::save_label( Label** label, uint* block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(3));
  *label = oper->_label;
  *block_num = oper->_block_num;
}
void jmpConUCFNode::label_set( Label* label, uint block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(3));
  oper->_label     = label;
  oper->_block_num = block_num;
}
void jmpConUCFNode::save_label( Label** label, uint* block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(3));
  *label = oper->_label;
  *block_num = oper->_block_num;
}
void jmpConUCF2Node::label_set( Label* label, uint block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(3));
  oper->_label     = label;
  oper->_block_num = block_num;
}
void jmpConUCF2Node::save_label( Label** label, uint* block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(3));
  *label = oper->_label;
  *block_num = oper->_block_num;
}
void jmpDir_shortNode::label_set( Label* label, uint block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(1));
  oper->_label     = label;
  oper->_block_num = block_num;
}
void jmpDir_shortNode::save_label( Label** label, uint* block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(1));
  *label = oper->_label;
  *block_num = oper->_block_num;
}
void jmpCon_shortNode::label_set( Label* label, uint block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(3));
  oper->_label     = label;
  oper->_block_num = block_num;
}
void jmpCon_shortNode::save_label( Label** label, uint* block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(3));
  *label = oper->_label;
  *block_num = oper->_block_num;
}
void jmpLoopEnd_shortNode::label_set( Label* label, uint block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(3));
  oper->_label     = label;
  oper->_block_num = block_num;
}
void jmpLoopEnd_shortNode::save_label( Label** label, uint* block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(3));
  *label = oper->_label;
  *block_num = oper->_block_num;
}
void jmpConU_shortNode::label_set( Label* label, uint block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(3));
  oper->_label     = label;
  oper->_block_num = block_num;
}
void jmpConU_shortNode::save_label( Label** label, uint* block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(3));
  *label = oper->_label;
  *block_num = oper->_block_num;
}
void jmpConUCF_shortNode::label_set( Label* label, uint block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(3));
  oper->_label     = label;
  oper->_block_num = block_num;
}
void jmpConUCF_shortNode::save_label( Label** label, uint* block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(3));
  *label = oper->_label;
  *block_num = oper->_block_num;
}
void jmpConUCF2_shortNode::label_set( Label* label, uint block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(3));
  oper->_label     = label;
  oper->_block_num = block_num;
}
void jmpConUCF2_shortNode::save_label( Label** label, uint* block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(3));
  *label = oper->_label;
  *block_num = oper->_block_num;
}
void cmpL_LTGENode::label_set( Label* label, uint block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(3));
  oper->_label     = label;
  oper->_block_num = block_num;
}
void cmpL_LTGENode::save_label( Label** label, uint* block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(3));
  *label = oper->_label;
  *block_num = oper->_block_num;
}
void cmpUL_LTGENode::label_set( Label* label, uint block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(3));
  oper->_label     = label;
  oper->_block_num = block_num;
}
void cmpUL_LTGENode::save_label( Label** label, uint* block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(3));
  *label = oper->_label;
  *block_num = oper->_block_num;
}
void cmpL_EQNENode::label_set( Label* label, uint block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(3));
  oper->_label     = label;
  oper->_block_num = block_num;
}
void cmpL_EQNENode::save_label( Label** label, uint* block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(3));
  *label = oper->_label;
  *block_num = oper->_block_num;
}
void cmpUL_EQNENode::label_set( Label* label, uint block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(3));
  oper->_label     = label;
  oper->_block_num = block_num;
}
void cmpUL_EQNENode::save_label( Label** label, uint* block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(3));
  *label = oper->_label;
  *block_num = oper->_block_num;
}
void cmpL_LEGTNode::label_set( Label* label, uint block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(3));
  oper->_label     = label;
  oper->_block_num = block_num;
}
void cmpL_LEGTNode::save_label( Label** label, uint* block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(3));
  *label = oper->_label;
  *block_num = oper->_block_num;
}
void cmpUL_LEGTNode::label_set( Label* label, uint block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(3));
  oper->_label     = label;
  oper->_block_num = block_num;
}
void cmpUL_LEGTNode::save_label( Label** label, uint* block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(3));
  *label = oper->_label;
  *block_num = oper->_block_num;
}
void CallStaticJavaDirectNode::method_set( intptr_t method ) {
  ((methodOper*)opnd_array(1))->_method = method;
}

void CallDynamicJavaDirectNode::method_set( intptr_t method ) {
  ((methodOper*)opnd_array(1))->_method = method;
}

void CallRuntimeDirectNode::method_set( intptr_t method ) {
  ((methodOper*)opnd_array(1))->_method = method;
}

void CallLeafDirectNode::method_set( intptr_t method ) {
  ((methodOper*)opnd_array(1))->_method = method;
}

void CallLeafNoFPDirectNode::method_set( intptr_t method ) {
  ((methodOper*)opnd_array(1))->_method = method;
}

int popCountI_memNode::reloc() const {
  return 1;
}

int popCountL_memNode::reloc() const {
  return 1;
}

int loadBNode::reloc() const {
  return 1;
}

int loadB2LNode::reloc() const {
  return 1;
}

int loadUBNode::reloc() const {
  return 1;
}

int loadUB2LNode::reloc() const {
  return 1;
}

int loadUB2L_immINode::reloc() const {
  return 1;
}

int loadSNode::reloc() const {
  return 1;
}

int loadS2BNode::reloc() const {
  return 1;
}

int loadS2LNode::reloc() const {
  return 1;
}

int loadUSNode::reloc() const {
  return 1;
}

int loadUS2BNode::reloc() const {
  return 1;
}

int loadUS2LNode::reloc() const {
  return 1;
}

int loadUS2L_immI_255Node::reloc() const {
  return 1;
}

int loadUS2L_immINode::reloc() const {
  return 1;
}

int loadINode::reloc() const {
  return 1;
}

int loadI2BNode::reloc() const {
  return 1;
}

int loadI2UBNode::reloc() const {
  return 1;
}

int loadI2SNode::reloc() const {
  return 1;
}

int loadI2USNode::reloc() const {
  return 1;
}

int loadI2LNode::reloc() const {
  return 1;
}

int loadI2L_immI_255Node::reloc() const {
  return 1;
}

int loadI2L_immI_65535Node::reloc() const {
  return 1;
}

int loadI2L_immU31Node::reloc() const {
  return 1;
}

int loadUI2LNode::reloc() const {
  return 1;
}

int loadLNode::reloc() const {
  return 1;
}

int loadL_volatileNode::reloc() const {
  return 1;
}

int loadLX_volatileNode::reloc() const {
  return 1;
}

int loadLX_reg_volatileNode::reloc() const {
  return 1;
}

int loadRangeNode::reloc() const {
  return 1;
}

int loadPNode::reloc() const {
  return 1;
}

int loadKlassNode::reloc() const {
  return 1;
}

int loadDPRNode::reloc() const {
  return 1;
}

int loadDNode::reloc() const {
  return 1;
}

int loadD_partialNode::reloc() const {
  return 1;
}

int loadFNode::reloc() const {
  return 1;
}

int loadFPRNode::reloc() const {
  return 1;
}

int loadConPNode::reloc() const {
  return 1;
}

int loadConFPRNode::reloc() const {
  return 2;
}

int loadConFPR0Node::reloc() const {
  return 2;
}

int loadConFPR1Node::reloc() const {
  return 2;
}

int loadConFNode::reloc() const {
  return 2;
}

int loadConF0Node::reloc() const {
  return 2;
}

int loadConDPRNode::reloc() const {
  return 2;
}

int loadConDPR0Node::reloc() const {
  return 2;
}

int loadConDPR1Node::reloc() const {
  return 2;
}

int loadConDNode::reloc() const {
  return 2;
}

int loadConD0Node::reloc() const {
  return 2;
}

int prefetchAlloc0Node::reloc() const {
  return 1;
}

int prefetchAllocNode::reloc() const {
  return 1;
}

int prefetchAllocNTANode::reloc() const {
  return 1;
}

int prefetchAllocT0Node::reloc() const {
  return 1;
}

int prefetchAllocT2Node::reloc() const {
  return 1;
}

int storeBNode::reloc() const {
  return 2;
}

int storeCNode::reloc() const {
  return 2;
}

int storeINode::reloc() const {
  return 2;
}

int storeLNode::reloc() const {
  return 2;
}

int storeL2INode::reloc() const {
  return 2;
}

int storeL_volatileNode::reloc() const {
  return 2;
}

int storeLX_volatileNode::reloc() const {
  return 2;
}

int storeLX_reg_volatileNode::reloc() const {
  return 2;
}

int storePNode::reloc() const {
  return 2;
}

int storeImmINode::reloc() const {
  return 2;
}

int storeImmI16Node::reloc() const {
  return 2;
}

int storeImmPNode::reloc() const {
  return 3;
}

int storeImmBNode::reloc() const {
  return 2;
}

int storeImmCMNode::reloc() const {
  return 2;
}

int storeDPRNode::reloc() const {
  return 2;
}

int storeDPR_roundedNode::reloc() const {
  return 2;
}

int storeDNode::reloc() const {
  return 2;
}

int storeFNode::reloc() const {
  return 2;
}

int storeFPRNode::reloc() const {
  return 2;
}

int storeFPR_roundedNode::reloc() const {
  return 2;
}

int storeFPR_DroundedNode::reloc() const {
  return 2;
}

int storeFPR_immNode::reloc() const {
  return 3;
}

int storeF_immNode::reloc() const {
  return 3;
}

int cmovI_memNode::reloc() const {
  return 1;
}

int cmovI_memUNode::reloc() const {
  return 1;
}

int cmovI_memUCFNode::reloc() const {
  return 1;
}

int addI_eReg_memNode::reloc() const {
  return 1;
}

int addI_eReg_mem_0Node::reloc() const {
  return 1;
}

int addI_mem_eRegNode::reloc() const {
  return 3;
}

int addI_mem_eReg_0Node::reloc() const {
  return 3;
}

int addI_mem_immNode::reloc() const {
  return 3;
}

int incI_memNode::reloc() const {
  return 3;
}

int decI_memNode::reloc() const {
  return 3;
}

int xaddB_no_resNode::reloc() const {
  return 1;
}

int xaddBNode::reloc() const {
  return 1;
}

int xaddS_no_resNode::reloc() const {
  return 1;
}

int xaddSNode::reloc() const {
  return 1;
}

int xaddI_no_resNode::reloc() const {
  return 1;
}

int xaddINode::reloc() const {
  return 1;
}

int xchgBNode::reloc() const {
  return 1;
}

int xchgSNode::reloc() const {
  return 1;
}

int xchgINode::reloc() const {
  return 1;
}

int xchgPNode::reloc() const {
  return 1;
}

int subI_eReg_memNode::reloc() const {
  return 1;
}

int subI_mem_eRegNode::reloc() const {
  return 3;
}

int mulI_mem_immNode::reloc() const {
  return 1;
}

int mulINode::reloc() const {
  return 1;
}

int mulI_0Node::reloc() const {
  return 1;
}

int sarI_mem_1Node::reloc() const {
  return 3;
}

int sarI_mem_immNode::reloc() const {
  return 3;
}

int andI_eReg_memNode::reloc() const {
  return 1;
}

int andI_eReg_mem_0Node::reloc() const {
  return 1;
}

int andI_mem_eRegNode::reloc() const {
  return 3;
}

int andI_mem_eReg_0Node::reloc() const {
  return 3;
}

int andI_mem_immNode::reloc() const {
  return 3;
}

int andnI_rReg_rReg_memNode::reloc() const {
  return 1;
}

int andnI_rReg_rReg_mem_0Node::reloc() const {
  return 1;
}

int blsiI_rReg_memNode::reloc() const {
  return 2;
}

int blsiI_rReg_mem_0Node::reloc() const {
  return 2;
}

int blsmskI_rReg_memNode::reloc() const {
  return 2;
}

int blsmskI_rReg_mem_0Node::reloc() const {
  return 2;
}

int blsrI_rReg_memNode::reloc() const {
  return 2;
}

int blsrI_rReg_mem_0Node::reloc() const {
  return 2;
}

int orI_eReg_memNode::reloc() const {
  return 1;
}

int orI_eReg_mem_0Node::reloc() const {
  return 1;
}

int orI_mem_eRegNode::reloc() const {
  return 3;
}

int orI_mem_eReg_0Node::reloc() const {
  return 3;
}

int orI_mem_immNode::reloc() const {
  return 3;
}

int xorI_eReg_memNode::reloc() const {
  return 1;
}

int xorI_eReg_mem_0Node::reloc() const {
  return 1;
}

int xorI_mem_eRegNode::reloc() const {
  return 3;
}

int xorI_mem_eReg_0Node::reloc() const {
  return 3;
}

int xorI_mem_immNode::reloc() const {
  return 3;
}

int addL_eReg_memNode::reloc() const {
  return 1;
}

int addL_eReg_mem_0Node::reloc() const {
  return 1;
}

int subL_eReg_memNode::reloc() const {
  return 1;
}

int andL_eReg_memNode::reloc() const {
  return 1;
}

int andL_eReg_mem_0Node::reloc() const {
  return 1;
}

int andnL_eReg_eReg_memNode::reloc() const {
  return 1;
}

int andnL_eReg_eReg_mem_0Node::reloc() const {
  return 1;
}

int blsiL_eReg_memNode::reloc() const {
  return 2;
}

int blsiL_eReg_mem_0Node::reloc() const {
  return 2;
}

int blsmskL_eReg_memNode::reloc() const {
  return 2;
}

int blsmskL_eReg_mem_0Node::reloc() const {
  return 2;
}

int blsrL_eReg_memNode::reloc() const {
  return 2;
}

int blsrL_eReg_mem_0Node::reloc() const {
  return 2;
}

int orl_eReg_memNode::reloc() const {
  return 1;
}

int orl_eReg_mem_0Node::reloc() const {
  return 1;
}

int xorl_eReg_memNode::reloc() const {
  return 1;
}

int xorl_eReg_mem_0Node::reloc() const {
  return 1;
}

int cmpDPR_0Node::reloc() const {
  return 1;
}

int cmpD_ccmemNode::reloc() const {
  return 1;
}

int cmpD_ccmemCFNode::reloc() const {
  return 1;
}

int cmpD_regmemNode::reloc() const {
  return 1;
}

int subDPR_reg_memNode::reloc() const {
  return 1;
}

int addDPR_reg_memNode::reloc() const {
  return 1;
}

int addDPR_reg_mem_0Node::reloc() const {
  return 1;
}

int addDPR_mem_regNode::reloc() const {
  return 3;
}

int addDPR_mem_reg_0Node::reloc() const {
  return 3;
}

int addDPR_reg_imm1Node::reloc() const {
  return 1;
}

int addDPR_reg_immNode::reloc() const {
  return 1;
}

int addDPR_reg_imm_roundNode::reloc() const {
  return 1;
}

int mulDPR_reg_immNode::reloc() const {
  return 1;
}

int mulDPR_reg_memNode::reloc() const {
  return 1;
}

int mulDPR_reg_mem_0Node::reloc() const {
  return 1;
}

int mulDPR_reg_mem_ciscNode::reloc() const {
  return 1;
}

int mulDPR_reg_mem_cisc_0Node::reloc() const {
  return 1;
}

int cmpFPR_0Node::reloc() const {
  return 1;
}

int cmpF_ccmemNode::reloc() const {
  return 1;
}

int cmpF_ccmemCFNode::reloc() const {
  return 1;
}

int cmpF_regmemNode::reloc() const {
  return 1;
}

int addFPR24_reg_memNode::reloc() const {
  return 1;
}

int addFPR24_reg_mem_0Node::reloc() const {
  return 1;
}

int addFPR_reg_memNode::reloc() const {
  return 1;
}

int addFPR_reg_mem_0Node::reloc() const {
  return 1;
}

int addFPR24_mem_regNode::reloc() const {
  return 1;
}

int addFPR24_mem_ciscNode::reloc() const {
  return 2;
}

int addFPR24_mem_cisc_0Node::reloc() const {
  return 2;
}

int addFPR24_mem_memNode::reloc() const {
  return 2;
}

int addFPR24_reg_immNode::reloc() const {
  return 1;
}

int addFPR_reg_immNode::reloc() const {
  return 1;
}

int mulFPR24_reg_memNode::reloc() const {
  return 1;
}

int mulFPR24_reg_mem_0Node::reloc() const {
  return 1;
}

int mulFPR_reg_memNode::reloc() const {
  return 1;
}

int mulFPR_reg_mem_0Node::reloc() const {
  return 1;
}

int mulFPR24_mem_memNode::reloc() const {
  return 2;
}

int mulFPR24_reg_immNode::reloc() const {
  return 1;
}

int mulFPR_reg_immNode::reloc() const {
  return 1;
}

int mulFPR_reg_load1Node::reloc() const {
  return 1;
}

int mulFPR_reg_load1_0Node::reloc() const {
  return 1;
}

int addFPR_mulFPR_reg_load1Node::reloc() const {
  return 1;
}

int addFPR_mulFPR_reg_load1_1Node::reloc() const {
  return 1;
}

int addFPR_mulFPR_reg_load1_0Node::reloc() const {
  return 1;
}

int addFPR_mulFPR_reg_load1_2Node::reloc() const {
  return 1;
}

int convI2D_memNode::reloc() const {
  return 1;
}

int convI2DPR_memNode::reloc() const {
  return 1;
}

int convI2FPR_SSF_memNode::reloc() const {
  return 1;
}

int convI2FPR_memNode::reloc() const {
  return 1;
}

int compI_eReg_memNode::reloc() const {
  return 1;
}

int testI_reg_memNode::reloc() const {
  return 1;
}

int compU_eReg_memNode::reloc() const {
  return 1;
}

int compP_eReg_immNode::reloc() const {
  return 1;
}

int compP_eReg_memNode::reloc() const {
  return 1;
}

int compP_mem_eRegNode::reloc() const {
  return 1;
}

int testP_regNode::reloc() const {
  return 1;
}

int testP_Reg_memNode::reloc() const {
  return 1;
}

int partialSubtypeCheck_vs_ZeroNode::reloc() const {
  return 1;
}

int cmovLL_mem_LTGENode::reloc() const {
  return 1;
}

int cmovLL_mem_LTGE_UNode::reloc() const {
  return 1;
}

int cmovII_mem_LTGENode::reloc() const {
  return 1;
}

int cmovII_mem_LTGE_UNode::reloc() const {
  return 1;
}

int cmovLL_mem_EQNENode::reloc() const {
  return 1;
}

int cmovII_mem_EQNENode::reloc() const {
  return 1;
}

int cmovII_mem_EQNE_UNode::reloc() const {
  return 1;
}

int cmovLL_mem_LEGTNode::reloc() const {
  return 1;
}

int cmovLL_mem_LEGT_UNode::reloc() const {
  return 1;
}

int cmovII_mem_LEGTNode::reloc() const {
  return 1;
}

int cmovII_mem_LEGT_UNode::reloc() const {
  return 1;
}

int CallStaticJavaDirectNode::reloc() const {
  return 1;
}

int CallDynamicJavaDirectNode::reloc() const {
  return 1;
}

int CallRuntimeDirectNode::reloc() const {
  return 1;
}

int CallLeafDirectNode::reloc() const {
  return 1;
}

int CallLeafNoFPDirectNode::reloc() const {
  return 1;
}

int RetNode::reloc() const {
  return 1;
}

int TailCalljmpIndNode::reloc() const {
  return 1;
}

int tailjmpIndNode::reloc() const {
  return 1;
}

int RethrowExceptionNode::reloc() const {
  return 1;
}

int safePoint_poll_tlsNode::reloc() const {
  return 1;
}

int addF_memNode::reloc() const {
  return 1;
}

int addF_mem_0Node::reloc() const {
  return 1;
}

int addF_immNode::reloc() const {
  return 1;
}

int addF_reg_memNode::reloc() const {
  return 1;
}

int addF_reg_mem_0Node::reloc() const {
  return 1;
}

int addF_reg_immNode::reloc() const {
  return 1;
}

int addD_memNode::reloc() const {
  return 1;
}

int addD_mem_0Node::reloc() const {
  return 1;
}

int addD_immNode::reloc() const {
  return 1;
}

int addD_reg_memNode::reloc() const {
  return 1;
}

int addD_reg_mem_0Node::reloc() const {
  return 1;
}

int addD_reg_immNode::reloc() const {
  return 1;
}

int subF_memNode::reloc() const {
  return 1;
}

int subF_immNode::reloc() const {
  return 1;
}

int subF_reg_memNode::reloc() const {
  return 1;
}

int subF_reg_immNode::reloc() const {
  return 1;
}

int subD_memNode::reloc() const {
  return 1;
}

int subD_immNode::reloc() const {
  return 1;
}

int subD_reg_memNode::reloc() const {
  return 1;
}

int subD_reg_immNode::reloc() const {
  return 1;
}

int mulF_memNode::reloc() const {
  return 1;
}

int mulF_mem_0Node::reloc() const {
  return 1;
}

int mulF_immNode::reloc() const {
  return 1;
}

int mulF_reg_memNode::reloc() const {
  return 1;
}

int mulF_reg_mem_0Node::reloc() const {
  return 1;
}

int mulF_reg_immNode::reloc() const {
  return 1;
}

int mulD_memNode::reloc() const {
  return 1;
}

int mulD_mem_0Node::reloc() const {
  return 1;
}

int mulD_immNode::reloc() const {
  return 1;
}

int mulD_reg_memNode::reloc() const {
  return 1;
}

int mulD_reg_mem_0Node::reloc() const {
  return 1;
}

int mulD_reg_immNode::reloc() const {
  return 1;
}

int divF_memNode::reloc() const {
  return 1;
}

int divF_immNode::reloc() const {
  return 1;
}

int divF_reg_memNode::reloc() const {
  return 1;
}

int divF_reg_immNode::reloc() const {
  return 1;
}

int divD_memNode::reloc() const {
  return 1;
}

int divD_immNode::reloc() const {
  return 1;
}

int divD_reg_memNode::reloc() const {
  return 1;
}

int divD_reg_immNode::reloc() const {
  return 1;
}

int convF2HF_mem_regNode::reloc() const {
  return 2;
}

int vconvF2HF_mem_regNode::reloc() const {
  return 2;
}

int vconvHF2F_reg_memNode::reloc() const {
  return 1;
}

int loadVNode::reloc() const {
  return 1;
}

int storeVNode::reloc() const {
  return 2;
}

int gatherNode::reloc() const {
  return 1;
}

int evgatherNode::reloc() const {
  return 1;
}

int evgather_maskedNode::reloc() const {
  return 1;
}

int scatterNode::reloc() const {
  return 2;
}

int scatter_maskedNode::reloc() const {
  return 2;
}

int ReplB_memNode::reloc() const {
  return 1;
}

int ReplS_memNode::reloc() const {
  return 1;
}

int ReplI_memNode::reloc() const {
  return 1;
}

int ReplL_memNode::reloc() const {
  return 1;
}

int ReplF_memNode::reloc() const {
  return 1;
}

int ReplF_immNode::reloc() const {
  return 1;
}

int ReplF_zeroNode::reloc() const {
  return 1;
}

int ReplD_memNode::reloc() const {
  return 1;
}

int ReplD_immNode::reloc() const {
  return 1;
}

int ReplD_zeroNode::reloc() const {
  return 1;
}

int minmax_reduction2FNode::reloc() const {
  return 1;
}

int minmax_reduction2F_0Node::reloc() const {
  return 1;
}

int minmax_reductionFNode::reloc() const {
  return 1;
}

int minmax_reductionF_0Node::reloc() const {
  return 1;
}

int minmax_reduction2DNode::reloc() const {
  return 1;
}

int minmax_reduction2D_0Node::reloc() const {
  return 1;
}

int minmax_reductionDNode::reloc() const {
  return 1;
}

int minmax_reductionD_0Node::reloc() const {
  return 1;
}

int vaddB_memNode::reloc() const {
  return 1;
}

int vaddB_mem_0Node::reloc() const {
  return 1;
}

int vaddS_memNode::reloc() const {
  return 1;
}

int vaddS_mem_0Node::reloc() const {
  return 1;
}

int vaddI_memNode::reloc() const {
  return 1;
}

int vaddI_mem_0Node::reloc() const {
  return 1;
}

int vaddL_memNode::reloc() const {
  return 1;
}

int vaddL_mem_0Node::reloc() const {
  return 1;
}

int vaddF_memNode::reloc() const {
  return 1;
}

int vaddF_mem_0Node::reloc() const {
  return 1;
}

int vaddD_memNode::reloc() const {
  return 1;
}

int vaddD_mem_0Node::reloc() const {
  return 1;
}

int vsubB_memNode::reloc() const {
  return 1;
}

int vsubS_memNode::reloc() const {
  return 1;
}

int vsubI_memNode::reloc() const {
  return 1;
}

int vsubL_memNode::reloc() const {
  return 1;
}

int vsubF_memNode::reloc() const {
  return 1;
}

int vsubD_memNode::reloc() const {
  return 1;
}

int vmulS_memNode::reloc() const {
  return 1;
}

int vmulS_mem_0Node::reloc() const {
  return 1;
}

int vmulI_memNode::reloc() const {
  return 1;
}

int vmulI_mem_0Node::reloc() const {
  return 1;
}

int evmulL_memNode::reloc() const {
  return 1;
}

int evmulL_mem_0Node::reloc() const {
  return 1;
}

int vmulF_memNode::reloc() const {
  return 1;
}

int vmulF_mem_0Node::reloc() const {
  return 1;
}

int vmulD_memNode::reloc() const {
  return 1;
}

int vmulD_mem_0Node::reloc() const {
  return 1;
}

int vdivF_memNode::reloc() const {
  return 1;
}

int vdivD_memNode::reloc() const {
  return 1;
}

int compressBitsI_memNode::reloc() const {
  return 1;
}

int expandBitsI_memNode::reloc() const {
  return 1;
}

int vsqrtF_memNode::reloc() const {
  return 1;
}

int vsqrtD_memNode::reloc() const {
  return 1;
}

int vand_memNode::reloc() const {
  return 1;
}

int vand_mem_0Node::reloc() const {
  return 1;
}

int vor_memNode::reloc() const {
  return 1;
}

int vor_mem_0Node::reloc() const {
  return 1;
}

int vxor_memNode::reloc() const {
  return 1;
}

int vxor_mem_0Node::reloc() const {
  return 1;
}

int vfmaF_memNode::reloc() const {
  return 1;
}

int vfmaD_memNode::reloc() const {
  return 1;
}

int vpternlog_memNode::reloc() const {
  return 1;
}

int vmasked_load_avx_non_subwordNode::reloc() const {
  return 1;
}

int vmasked_load_evexNode::reloc() const {
  return 1;
}

int vmasked_store_avx_non_subwordNode::reloc() const {
  return 2;
}

int vmasked_store_evexNode::reloc() const {
  return 2;
}

int vadd_mem_maskedNode::reloc() const {
  return 1;
}

int vadd_mem_masked_0Node::reloc() const {
  return 1;
}

int vadd_mem_masked_1Node::reloc() const {
  return 1;
}

int vadd_mem_masked_2Node::reloc() const {
  return 1;
}

int vadd_mem_masked_3Node::reloc() const {
  return 1;
}

int vadd_mem_masked_4Node::reloc() const {
  return 1;
}

int vxor_mem_maskedNode::reloc() const {
  return 1;
}

int vor_mem_maskedNode::reloc() const {
  return 1;
}

int vand_mem_maskedNode::reloc() const {
  return 1;
}

int vsub_mem_maskedNode::reloc() const {
  return 1;
}

int vsub_mem_masked_0Node::reloc() const {
  return 1;
}

int vsub_mem_masked_1Node::reloc() const {
  return 1;
}

int vsub_mem_masked_2Node::reloc() const {
  return 1;
}

int vsub_mem_masked_3Node::reloc() const {
  return 1;
}

int vsub_mem_masked_4Node::reloc() const {
  return 1;
}

int vmul_mem_maskedNode::reloc() const {
  return 1;
}

int vmul_mem_masked_0Node::reloc() const {
  return 1;
}

int vmul_mem_masked_1Node::reloc() const {
  return 1;
}

int vmul_mem_masked_2Node::reloc() const {
  return 1;
}

int vmul_mem_masked_3Node::reloc() const {
  return 1;
}

int vdiv_mem_maskedNode::reloc() const {
  return 1;
}

int vdiv_mem_masked_0Node::reloc() const {
  return 1;
}

int vlshift_mem_maskedNode::reloc() const {
  return 1;
}

int vlshift_mem_masked_0Node::reloc() const {
  return 1;
}

int vlshift_mem_masked_1Node::reloc() const {
  return 1;
}

int vrshift_mem_maskedNode::reloc() const {
  return 1;
}

int vrshift_mem_masked_0Node::reloc() const {
  return 1;
}

int vrshift_mem_masked_1Node::reloc() const {
  return 1;
}

int vurshift_mem_maskedNode::reloc() const {
  return 1;
}

int vurshift_mem_masked_0Node::reloc() const {
  return 1;
}

int vurshift_mem_masked_1Node::reloc() const {
  return 1;
}

int vmaxv_mem_maskedNode::reloc() const {
  return 1;
}

int vminv_mem_maskedNode::reloc() const {
  return 1;
}

int vfma_mem_maskedNode::reloc() const {
  return 1;
}

int vfma_mem_masked_0Node::reloc() const {
  return 1;
}

int vternlogd_mem_maskedNode::reloc() const {
  return 1;
}

int compareAndSwapP_shenandoahNode::reloc() const {
  return 1;
}

int compareAndSwapP_shenandoah_0Node::reloc() const {
  return 1;
}

int compareAndExchangeP_shenandoahNode::reloc() const {
  return 1;
}


void MoveF2LEGNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 5068 "src/hotspot/cpu/x86/x86_32.ad"

    ShouldNotReachHere();
  
#line 15968 "ad_x86.cpp"
  }
}

void MoveLEG2FNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 5078 "src/hotspot/cpu/x86/x86_32.ad"

    ShouldNotReachHere();
  
#line 15984 "ad_x86.cpp"
  }
}

void MoveF2VLNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 5088 "src/hotspot/cpu/x86/x86_32.ad"

    ShouldNotReachHere();
  
#line 16000 "ad_x86.cpp"
  }
}

void MoveVL2FNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 5098 "src/hotspot/cpu/x86/x86_32.ad"

    ShouldNotReachHere();
  
#line 16016 "ad_x86.cpp"
  }
}

void MoveD2LEGNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 5110 "src/hotspot/cpu/x86/x86_32.ad"

    ShouldNotReachHere();
  
#line 16032 "ad_x86.cpp"
  }
}

void MoveLEG2DNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 5120 "src/hotspot/cpu/x86/x86_32.ad"

    ShouldNotReachHere();
  
#line 16048 "ad_x86.cpp"
  }
}

void MoveD2VLNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 5130 "src/hotspot/cpu/x86/x86_32.ad"

    ShouldNotReachHere();
  
#line 16064 "ad_x86.cpp"
  }
}

void MoveVL2DNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 5140 "src/hotspot/cpu/x86/x86_32.ad"

    ShouldNotReachHere();
  
#line 16080 "ad_x86.cpp"
  }
}

void bytes_reverse_intNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  {

#line 1544 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0x0F /*primary()*/));
  
#line 16095 "ad_x86.cpp"
  }
  {

#line 1679 "src/hotspot/cpu/x86/x86_32.ad"

    emit_cc(cbuf, (0xC8 /*secondary()*/), opnd_array(1)->reg(ra_,this,idx1)/* dst */ );
  
#line 16103 "ad_x86.cpp"
  }
}

void bytes_reverse_longNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  {

#line 1683 "src/hotspot/cpu/x86/x86_32.ad"

    int destlo = opnd_array(1)->reg(ra_,this,idx1)/* dst */;
    int desthi = HIGH_FROM_LOW_ENC(destlo);
    // bswap lo
    emit_opcode(cbuf, 0x0F);
    emit_cc(cbuf, 0xC8, destlo);
    // bswap hi
    emit_opcode(cbuf, 0x0F);
    emit_cc(cbuf, 0xC8, desthi);
    // xchg lo and hi
    emit_opcode(cbuf, 0x87);
    emit_rm(cbuf, 0x3, destlo, desthi);
  
#line 16128 "ad_x86.cpp"
  }
}

void bytes_reverse_unsigned_shortNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  {
    C2_MacroAssembler _masm(&cbuf);

#line 5174 "src/hotspot/cpu/x86/x86_32.ad"

    __ bswapl(opnd_array(1)->as_Register(ra_,this,idx1)/* dst */);
    __ shrl(opnd_array(1)->as_Register(ra_,this,idx1)/* dst */, 16);
  
#line 16145 "ad_x86.cpp"
  }
}

void bytes_reverse_shortNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  {
    C2_MacroAssembler _masm(&cbuf);

#line 5187 "src/hotspot/cpu/x86/x86_32.ad"

    __ bswapl(opnd_array(1)->as_Register(ra_,this,idx1)/* dst */);
    __ sarl(opnd_array(1)->as_Register(ra_,this,idx1)/* dst */, 16);
  
#line 16162 "ad_x86.cpp"
  }
}

void countLeadingZerosINode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 5203 "src/hotspot/cpu/x86/x86_32.ad"

    __ lzcntl(opnd_array(0)->as_Register(ra_,this)/* dst */, opnd_array(1)->as_Register(ra_,this,idx1)/* src */);
  
#line 16178 "ad_x86.cpp"
  }
}

void countLeadingZerosI_bsrNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 5220 "src/hotspot/cpu/x86/x86_32.ad"

    Register Rdst = opnd_array(0)->as_Register(ra_,this)/* dst */;
    Register Rsrc = opnd_array(1)->as_Register(ra_,this,idx1)/* src */;
    Label skip;
    __ bsrl(Rdst, Rsrc);
    __ jccb(Assembler::notZero, skip);
    __ movl(Rdst, -1);
    __ bind(skip);
    __ negl(Rdst);
    __ addl(Rdst, BitsPerInt - 1);
  
#line 16202 "ad_x86.cpp"
  }
}

void countLeadingZerosLNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// dst
  {
    C2_MacroAssembler _masm(&cbuf);

#line 5244 "src/hotspot/cpu/x86/x86_32.ad"

    Register Rdst = opnd_array(2)->as_Register(ra_,this,idx2)/* dst */;
    Register Rsrc = opnd_array(1)->as_Register(ra_,this,idx1)/* src */;
    Label done;
    __ lzcntl(Rdst, HIGH_FROM_LOW(Rsrc));
    __ jccb(Assembler::carryClear, done);
    __ lzcntl(Rdst, Rsrc);
    __ addl(Rdst, BitsPerInt);
    __ bind(done);
  
#line 16226 "ad_x86.cpp"
  }
}

void countLeadingZerosL_bsrNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// dst
  {
    C2_MacroAssembler _masm(&cbuf);

#line 5273 "src/hotspot/cpu/x86/x86_32.ad"

    Register Rdst = opnd_array(2)->as_Register(ra_,this,idx2)/* dst */;
    Register Rsrc = opnd_array(1)->as_Register(ra_,this,idx1)/* src */;
    Label msw_is_zero;
    Label not_zero;
    __ bsrl(Rdst, HIGH_FROM_LOW(Rsrc));
    __ jccb(Assembler::zero, msw_is_zero);
    __ addl(Rdst, BitsPerInt);
    __ jmpb(not_zero);
    __ bind(msw_is_zero);
    __ bsrl(Rdst, Rsrc);
    __ jccb(Assembler::notZero, not_zero);
    __ movl(Rdst, -1);
    __ bind(not_zero);
    __ negl(Rdst);
    __ addl(Rdst, BitsPerLong - 1);
  
#line 16257 "ad_x86.cpp"
  }
}

void countTrailingZerosINode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 5299 "src/hotspot/cpu/x86/x86_32.ad"

    __ tzcntl(opnd_array(0)->as_Register(ra_,this)/* dst */, opnd_array(1)->as_Register(ra_,this,idx1)/* src */);
  
#line 16273 "ad_x86.cpp"
  }
}

void countTrailingZerosI_bsfNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 5314 "src/hotspot/cpu/x86/x86_32.ad"

    Register Rdst = opnd_array(0)->as_Register(ra_,this)/* dst */;
    Label done;
    __ bsfl(Rdst, opnd_array(1)->as_Register(ra_,this,idx1)/* src */);
    __ jccb(Assembler::notZero, done);
    __ movl(Rdst, BitsPerInt);
    __ bind(done);
  
#line 16294 "ad_x86.cpp"
  }
}

void countTrailingZerosLNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// dst
  {
    C2_MacroAssembler _masm(&cbuf);

#line 5335 "src/hotspot/cpu/x86/x86_32.ad"

    Register Rdst = opnd_array(2)->as_Register(ra_,this,idx2)/* dst */;
    Register Rsrc = opnd_array(1)->as_Register(ra_,this,idx1)/* src */;
    Label done;
    __ tzcntl(Rdst, Rsrc);
    __ jccb(Assembler::carryClear, done);
    __ tzcntl(Rdst, HIGH_FROM_LOW(Rsrc));
    __ addl(Rdst, BitsPerInt);
    __ bind(done);
  
#line 16318 "ad_x86.cpp"
  }
}

void countTrailingZerosL_bsfNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// dst
  {
    C2_MacroAssembler _masm(&cbuf);

#line 5361 "src/hotspot/cpu/x86/x86_32.ad"

    Register Rdst = opnd_array(2)->as_Register(ra_,this,idx2)/* dst */;
    Register Rsrc = opnd_array(1)->as_Register(ra_,this,idx1)/* src */;
    Label msw_not_zero;
    Label done;
    __ bsfl(Rdst, Rsrc);
    __ jccb(Assembler::notZero, done);
    __ bsfl(Rdst, HIGH_FROM_LOW(Rsrc));
    __ jccb(Assembler::notZero, msw_not_zero);
    __ movl(Rdst, BitsPerInt);
    __ bind(msw_not_zero);
    __ addl(Rdst, BitsPerInt);
    __ bind(done);
  
#line 16346 "ad_x86.cpp"
  }
}

void popCountINode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 5387 "src/hotspot/cpu/x86/x86_32.ad"

    __ popcntl(opnd_array(0)->as_Register(ra_,this)/* dst */, opnd_array(1)->as_Register(ra_,this,idx1)/* src */);
  
#line 16362 "ad_x86.cpp"
  }
}

void popCountI_memNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  {
    C2_MacroAssembler _masm(&cbuf);

#line 5399 "src/hotspot/cpu/x86/x86_32.ad"

    __ popcntl(opnd_array(0)->as_Register(ra_,this)/* dst */, Address::make_raw(opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1), opnd_array(1)->disp_reloc()));
  
#line 16378 "ad_x86.cpp"
  }
}

void popCountLNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// dst
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// tmp
  {
    C2_MacroAssembler _masm(&cbuf);

#line 5414 "src/hotspot/cpu/x86/x86_32.ad"

    __ popcntl(opnd_array(2)->as_Register(ra_,this,idx2)/* dst */, opnd_array(1)->as_Register(ra_,this,idx1)/* src */);
    __ popcntl(opnd_array(3)->as_Register(ra_,this,idx3)/* tmp */, HIGH_FROM_LOW(opnd_array(1)->as_Register(ra_,this,idx1)/* src */));
    __ addl(opnd_array(2)->as_Register(ra_,this,idx2)/* dst */, opnd_array(3)->as_Register(ra_,this,idx3)/* tmp */);
  
#line 16398 "ad_x86.cpp"
  }
}

void popCountL_memNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// dst
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// tmp
  {
    C2_MacroAssembler _masm(&cbuf);

#line 5431 "src/hotspot/cpu/x86/x86_32.ad"

    //__ popcntl($dst$$Register, $mem$$Address$$first);
    //__ popcntl($tmp$$Register, $mem$$Address$$second);
    __ popcntl(opnd_array(2)->as_Register(ra_,this,idx2)/* dst */, Address::make_raw(opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1), relocInfo::none));
    __ popcntl(opnd_array(3)->as_Register(ra_,this,idx3)/* tmp */, Address::make_raw(opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1)+ 4, relocInfo::none));
    __ addl(opnd_array(2)->as_Register(ra_,this,idx2)/* dst */, opnd_array(3)->as_Register(ra_,this,idx3)/* tmp */);
  
#line 16420 "ad_x86.cpp"
  }
}

void loadBNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  {
    C2_MacroAssembler _masm(&cbuf);

#line 5451 "src/hotspot/cpu/x86/x86_32.ad"

    __ movsbl(opnd_array(0)->as_Register(ra_,this)/* dst */, Address::make_raw(opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1), opnd_array(1)->disp_reloc()));
  
#line 16436 "ad_x86.cpp"
  }
}

void loadB2LNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  {
    C2_MacroAssembler _masm(&cbuf);

#line 5468 "src/hotspot/cpu/x86/x86_32.ad"

    __ movsbl(opnd_array(0)->as_Register(ra_,this)/* dst */, Address::make_raw(opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1), opnd_array(1)->disp_reloc()));
    __ movl(HIGH_FROM_LOW(opnd_array(0)->as_Register(ra_,this)/* dst */), opnd_array(0)->as_Register(ra_,this)/* dst */); // This is always a different register.
    __ sarl(HIGH_FROM_LOW(opnd_array(0)->as_Register(ra_,this)/* dst */), 7); // 24+1 MSB are already signed extended.
  
#line 16454 "ad_x86.cpp"
  }
}

void loadUBNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  {
    C2_MacroAssembler _masm(&cbuf);

#line 5484 "src/hotspot/cpu/x86/x86_32.ad"

    __ movzbl(opnd_array(0)->as_Register(ra_,this)/* dst */, Address::make_raw(opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1), opnd_array(1)->disp_reloc()));
  
#line 16470 "ad_x86.cpp"
  }
}

void loadUB2LNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  {
    C2_MacroAssembler _masm(&cbuf);

#line 5500 "src/hotspot/cpu/x86/x86_32.ad"

    Register Rdst = opnd_array(0)->as_Register(ra_,this)/* dst */;
    __ movzbl(Rdst, Address::make_raw(opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1), opnd_array(1)->disp_reloc()));
    __ xorl(HIGH_FROM_LOW(Rdst), HIGH_FROM_LOW(Rdst));
  
#line 16488 "ad_x86.cpp"
  }
}

void loadUB2L_immINode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// mask
  {
    C2_MacroAssembler _masm(&cbuf);

#line 5517 "src/hotspot/cpu/x86/x86_32.ad"

    Register Rdst = opnd_array(0)->as_Register(ra_,this)/* dst */;
    __ movzbl(Rdst, Address::make_raw(opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1), opnd_array(1)->disp_reloc()));
    __ xorl(HIGH_FROM_LOW(Rdst), HIGH_FROM_LOW(Rdst));
    __ andl(Rdst, opnd_array(2)->constant()& right_n_bits(8));
  
#line 16508 "ad_x86.cpp"
  }
}

void loadSNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  {
    C2_MacroAssembler _masm(&cbuf);

#line 5533 "src/hotspot/cpu/x86/x86_32.ad"

    __ movswl(opnd_array(0)->as_Register(ra_,this)/* dst */, Address::make_raw(opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1), opnd_array(1)->disp_reloc()));
  
#line 16524 "ad_x86.cpp"
  }
}

void loadS2BNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// twentyfour
  {
    C2_MacroAssembler _masm(&cbuf);

#line 5546 "src/hotspot/cpu/x86/x86_32.ad"

    __ movsbl(opnd_array(0)->as_Register(ra_,this)/* dst */, Address::make_raw(opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1), opnd_array(1)->disp_reloc()));
  
#line 16541 "ad_x86.cpp"
  }
}

void loadS2LNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  {
    C2_MacroAssembler _masm(&cbuf);

#line 5562 "src/hotspot/cpu/x86/x86_32.ad"

    __ movswl(opnd_array(0)->as_Register(ra_,this)/* dst */, Address::make_raw(opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1), opnd_array(1)->disp_reloc()));
    __ movl(HIGH_FROM_LOW(opnd_array(0)->as_Register(ra_,this)/* dst */), opnd_array(0)->as_Register(ra_,this)/* dst */); // This is always a different register.
    __ sarl(HIGH_FROM_LOW(opnd_array(0)->as_Register(ra_,this)/* dst */), 15); // 16+1 MSB are already signed extended.
  
#line 16559 "ad_x86.cpp"
  }
}

void loadUSNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  {
    C2_MacroAssembler _masm(&cbuf);

#line 5578 "src/hotspot/cpu/x86/x86_32.ad"

    __ movzwl(opnd_array(0)->as_Register(ra_,this)/* dst */, Address::make_raw(opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1), opnd_array(1)->disp_reloc()));
  
#line 16575 "ad_x86.cpp"
  }
}

void loadUS2BNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// twentyfour
  {
    C2_MacroAssembler _masm(&cbuf);

#line 5591 "src/hotspot/cpu/x86/x86_32.ad"

    __ movsbl(opnd_array(0)->as_Register(ra_,this)/* dst */, Address::make_raw(opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1), opnd_array(1)->disp_reloc()));
  
#line 16592 "ad_x86.cpp"
  }
}

void loadUS2LNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  {
    C2_MacroAssembler _masm(&cbuf);

#line 5606 "src/hotspot/cpu/x86/x86_32.ad"

    __ movzwl(opnd_array(0)->as_Register(ra_,this)/* dst */, Address::make_raw(opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1), opnd_array(1)->disp_reloc()));
    __ xorl(HIGH_FROM_LOW(opnd_array(0)->as_Register(ra_,this)/* dst */), HIGH_FROM_LOW(opnd_array(0)->as_Register(ra_,this)/* dst */));
  
#line 16609 "ad_x86.cpp"
  }
}

void loadUS2L_immI_255Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// mask
  {
    C2_MacroAssembler _masm(&cbuf);

#line 5621 "src/hotspot/cpu/x86/x86_32.ad"

    Register Rdst = opnd_array(0)->as_Register(ra_,this)/* dst */;
    __ movzbl(Rdst, Address::make_raw(opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1), opnd_array(1)->disp_reloc()));
    __ xorl(HIGH_FROM_LOW(Rdst), HIGH_FROM_LOW(Rdst));
  
#line 16628 "ad_x86.cpp"
  }
}

void loadUS2L_immINode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// mask
  {
    C2_MacroAssembler _masm(&cbuf);

#line 5637 "src/hotspot/cpu/x86/x86_32.ad"

    Register Rdst = opnd_array(0)->as_Register(ra_,this)/* dst */;
    __ movzwl(Rdst, Address::make_raw(opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1), opnd_array(1)->disp_reloc()));
    __ xorl(HIGH_FROM_LOW(Rdst), HIGH_FROM_LOW(Rdst));
    __ andl(Rdst, opnd_array(2)->constant()& right_n_bits(16));
  
#line 16648 "ad_x86.cpp"
  }
}

void loadINode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  {
    C2_MacroAssembler _masm(&cbuf);

#line 5653 "src/hotspot/cpu/x86/x86_32.ad"

    __ movl(opnd_array(0)->as_Register(ra_,this)/* dst */, Address::make_raw(opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1), opnd_array(1)->disp_reloc()));
  
#line 16664 "ad_x86.cpp"
  }
}

void loadI2BNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// twentyfour
  {
    C2_MacroAssembler _masm(&cbuf);

#line 5666 "src/hotspot/cpu/x86/x86_32.ad"

    __ movsbl(opnd_array(0)->as_Register(ra_,this)/* dst */, Address::make_raw(opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1), opnd_array(1)->disp_reloc()));
  
#line 16681 "ad_x86.cpp"
  }
}

void loadI2UBNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// mask
  {
    C2_MacroAssembler _masm(&cbuf);

#line 5678 "src/hotspot/cpu/x86/x86_32.ad"

    __ movzbl(opnd_array(0)->as_Register(ra_,this)/* dst */, Address::make_raw(opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1), opnd_array(1)->disp_reloc()));
  
#line 16698 "ad_x86.cpp"
  }
}

void loadI2SNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// sixteen
  {
    C2_MacroAssembler _masm(&cbuf);

#line 5690 "src/hotspot/cpu/x86/x86_32.ad"

    __ movswl(opnd_array(0)->as_Register(ra_,this)/* dst */, Address::make_raw(opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1), opnd_array(1)->disp_reloc()));
  
#line 16715 "ad_x86.cpp"
  }
}

void loadI2USNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// mask
  {
    C2_MacroAssembler _masm(&cbuf);

#line 5702 "src/hotspot/cpu/x86/x86_32.ad"

    __ movzwl(opnd_array(0)->as_Register(ra_,this)/* dst */, Address::make_raw(opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1), opnd_array(1)->disp_reloc()));
  
#line 16732 "ad_x86.cpp"
  }
}

void loadI2LNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  {
    C2_MacroAssembler _masm(&cbuf);

#line 5718 "src/hotspot/cpu/x86/x86_32.ad"

    __ movl(opnd_array(0)->as_Register(ra_,this)/* dst */, Address::make_raw(opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1), opnd_array(1)->disp_reloc()));
    __ movl(HIGH_FROM_LOW(opnd_array(0)->as_Register(ra_,this)/* dst */), opnd_array(0)->as_Register(ra_,this)/* dst */); // This is always a different register.
    __ sarl(HIGH_FROM_LOW(opnd_array(0)->as_Register(ra_,this)/* dst */), 31);
  
#line 16750 "ad_x86.cpp"
  }
}

void loadI2L_immI_255Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// mask
  {
    C2_MacroAssembler _masm(&cbuf);

#line 5734 "src/hotspot/cpu/x86/x86_32.ad"

    Register Rdst = opnd_array(0)->as_Register(ra_,this)/* dst */;
    __ movzbl(Rdst, Address::make_raw(opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1), opnd_array(1)->disp_reloc()));
    __ xorl(HIGH_FROM_LOW(Rdst), HIGH_FROM_LOW(Rdst));
  
#line 16769 "ad_x86.cpp"
  }
}

void loadI2L_immI_65535Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// mask
  {
    C2_MacroAssembler _masm(&cbuf);

#line 5749 "src/hotspot/cpu/x86/x86_32.ad"

    Register Rdst = opnd_array(0)->as_Register(ra_,this)/* dst */;
    __ movzwl(Rdst, Address::make_raw(opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1), opnd_array(1)->disp_reloc()));
    __ xorl(HIGH_FROM_LOW(Rdst), HIGH_FROM_LOW(Rdst));
  
#line 16788 "ad_x86.cpp"
  }
}

void loadI2L_immU31Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// mask
  {
    C2_MacroAssembler _masm(&cbuf);

#line 5765 "src/hotspot/cpu/x86/x86_32.ad"

    Register Rdst = opnd_array(0)->as_Register(ra_,this)/* dst */;
    __ movl(Rdst, Address::make_raw(opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1), opnd_array(1)->disp_reloc()));
    __ xorl(HIGH_FROM_LOW(Rdst), HIGH_FROM_LOW(Rdst));
    __ andl(Rdst, opnd_array(2)->constant());
  
#line 16808 "ad_x86.cpp"
  }
}

void loadUI2LNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// mask
  {
    C2_MacroAssembler _masm(&cbuf);

#line 5783 "src/hotspot/cpu/x86/x86_32.ad"

    __ movl(opnd_array(0)->as_Register(ra_,this)/* dst */, Address::make_raw(opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1), opnd_array(1)->disp_reloc()));
    __ xorl(HIGH_FROM_LOW(opnd_array(0)->as_Register(ra_,this)/* dst */), HIGH_FROM_LOW(opnd_array(0)->as_Register(ra_,this)/* dst */));
  
#line 16826 "ad_x86.cpp"
  }
}

void loadLNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  {
    C2_MacroAssembler _masm(&cbuf);

#line 5801 "src/hotspot/cpu/x86/x86_32.ad"

    Address Amemlo = Address::make_raw(opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1), relocInfo::none);
    Address Amemhi = Address::make_raw(opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1)+ 4, relocInfo::none);
    __ movl(opnd_array(0)->as_Register(ra_,this)/* dst */, Amemlo);
    __ movl(HIGH_FROM_LOW(opnd_array(0)->as_Register(ra_,this)/* dst */), Amemhi);
  
#line 16845 "ad_x86.cpp"
  }
}

void loadL_volatileNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  {

#line 3063 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf,0xDF);
    int rm_byte_opcode = 0x05;
    int base     = opnd_array(1)->base(ra_,this,idx1);
    int index    = opnd_array(1)->index(ra_,this,idx1);
    int scale    = opnd_array(1)->scale();
    int displace = opnd_array(1)->disp(ra_,this,idx1);
    relocInfo::relocType disp_reloc = opnd_array(1)->disp_reloc(); // disp-as-oop when working with static globals
    encode_RegMem(cbuf, rm_byte_opcode, base, index, scale, displace, disp_reloc);
    store_to_stackslot( cbuf, 0x0DF, 0x07, opnd_array(0)->disp(ra_,this,0) );
  
#line 16868 "ad_x86.cpp"
  }
}

void loadLX_volatileNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// tmp
  {
    C2_MacroAssembler _masm(&cbuf);

#line 5832 "src/hotspot/cpu/x86/x86_32.ad"

    __ movdbl(opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* tmp */, Address::make_raw(opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1), opnd_array(1)->disp_reloc()));
    __ movdbl(Address(rsp, opnd_array(0)->disp(ra_,this,0)), opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* tmp */);
  
#line 16886 "ad_x86.cpp"
  }
}

void loadLX_reg_volatileNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// tmp
  {
    C2_MacroAssembler _masm(&cbuf);

#line 5848 "src/hotspot/cpu/x86/x86_32.ad"

    __ movdbl(opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* tmp */, Address::make_raw(opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1), opnd_array(1)->disp_reloc()));
    __ movdl(opnd_array(0)->as_Register(ra_,this)/* dst */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* tmp */);
    __ psrlq(opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* tmp */, 32);
    __ movdl(HIGH_FROM_LOW(opnd_array(0)->as_Register(ra_,this)/* dst */), opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* tmp */);
  
#line 16906 "ad_x86.cpp"
  }
}

void loadRangeNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  {

#line 1544 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0x8B /*primary()*/));
  
#line 16921 "ad_x86.cpp"
  }
  {

#line 2096 "src/hotspot/cpu/x86/x86_32.ad"

    int reg_encoding = opnd_array(0)->reg(ra_,this)/* dst */;
    int base  = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int displace = opnd_array(1)->disp(ra_,this,idx1);
    relocInfo::relocType disp_reloc = opnd_array(1)->disp_reloc();
    encode_RegMem(cbuf, reg_encoding, base, index, scale, displace, disp_reloc);
  
#line 16935 "ad_x86.cpp"
  }
}

void loadPNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  {

#line 1544 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0x8B /*primary()*/));
  
#line 16950 "ad_x86.cpp"
  }
  {

#line 2096 "src/hotspot/cpu/x86/x86_32.ad"

    int reg_encoding = opnd_array(0)->reg(ra_,this)/* dst */;
    int base  = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int displace = opnd_array(1)->disp(ra_,this,idx1);
    relocInfo::relocType disp_reloc = opnd_array(1)->disp_reloc();
    encode_RegMem(cbuf, reg_encoding, base, index, scale, displace, disp_reloc);
  
#line 16964 "ad_x86.cpp"
  }
}

void loadKlassNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  {

#line 1544 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0x8B /*primary()*/));
  
#line 16979 "ad_x86.cpp"
  }
  {

#line 2096 "src/hotspot/cpu/x86/x86_32.ad"

    int reg_encoding = opnd_array(0)->reg(ra_,this)/* dst */;
    int base  = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int displace = opnd_array(1)->disp(ra_,this,idx1);
    relocInfo::relocType disp_reloc = opnd_array(1)->disp_reloc();
    encode_RegMem(cbuf, reg_encoding, base, index, scale, displace, disp_reloc);
  
#line 16993 "ad_x86.cpp"
  }
}

void loadDPRNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  {

#line 1544 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0xDD /*primary()*/));
  
#line 17008 "ad_x86.cpp"
  }
  {

#line 2186 "src/hotspot/cpu/x86/x86_32.ad"

    int rm_byte_opcode = (0x00);
    int base     = opnd_array(1)->base(ra_,this,idx1);
    int index    = opnd_array(1)->index(ra_,this,idx1);
    int scale    = opnd_array(1)->scale();
    int displace = opnd_array(1)->disp(ra_,this,idx1);
    relocInfo::relocType disp_reloc = opnd_array(1)->disp_reloc(); // disp-as-oop when working with static globals
    encode_RegMem(cbuf, rm_byte_opcode, base, index, scale, displace, disp_reloc);
  
#line 17022 "ad_x86.cpp"
  }
  {

#line 2367 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode( cbuf, 0xDD );           // FSTP   ST(i)
    emit_d8( cbuf, 0xD8+opnd_array(0)->reg(ra_,this)/* dst */ );
  
#line 17031 "ad_x86.cpp"
  }
}

void loadDNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  {
    C2_MacroAssembler _masm(&cbuf);

#line 5911 "src/hotspot/cpu/x86/x86_32.ad"

    __ movdbl (opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, Address::make_raw(opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1), opnd_array(1)->disp_reloc()));
  
#line 17047 "ad_x86.cpp"
  }
}

void loadD_partialNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  {
    C2_MacroAssembler _masm(&cbuf);

#line 5922 "src/hotspot/cpu/x86/x86_32.ad"

    __ movdbl (opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, Address::make_raw(opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1), opnd_array(1)->disp_reloc()));
  
#line 17063 "ad_x86.cpp"
  }
}

void loadFNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  {
    C2_MacroAssembler _masm(&cbuf);

#line 5935 "src/hotspot/cpu/x86/x86_32.ad"

    __ movflt (opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, Address::make_raw(opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1), opnd_array(1)->disp_reloc()));
  
#line 17079 "ad_x86.cpp"
  }
}

void loadFPRNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  {

#line 1544 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0xD9 /*primary()*/));
  
#line 17094 "ad_x86.cpp"
  }
  {

#line 2186 "src/hotspot/cpu/x86/x86_32.ad"

    int rm_byte_opcode = (0x00);
    int base     = opnd_array(1)->base(ra_,this,idx1);
    int index    = opnd_array(1)->index(ra_,this,idx1);
    int scale    = opnd_array(1)->scale();
    int displace = opnd_array(1)->disp(ra_,this,idx1);
    relocInfo::relocType disp_reloc = opnd_array(1)->disp_reloc(); // disp-as-oop when working with static globals
    encode_RegMem(cbuf, rm_byte_opcode, base, index, scale, displace, disp_reloc);
  
#line 17108 "ad_x86.cpp"
  }
  {

#line 2422 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode( cbuf, 0xDD );           // FSTP   ST(i)
    emit_d8( cbuf, 0xD8+opnd_array(0)->reg(ra_,this)/* dst */ );
  
#line 17117 "ad_x86.cpp"
  }
}

void leaP8Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  {

#line 1544 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0x8D /*primary()*/));
  
#line 17132 "ad_x86.cpp"
  }
  {

#line 2096 "src/hotspot/cpu/x86/x86_32.ad"

    int reg_encoding = opnd_array(0)->reg(ra_,this)/* dst */;
    int base  = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int displace = opnd_array(1)->disp(ra_,this,idx1);
    relocInfo::relocType disp_reloc = opnd_array(1)->disp_reloc();
    encode_RegMem(cbuf, reg_encoding, base, index, scale, displace, disp_reloc);
  
#line 17146 "ad_x86.cpp"
  }
}

void leaP32Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  {

#line 1544 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0x8D /*primary()*/));
  
#line 17161 "ad_x86.cpp"
  }
  {

#line 2096 "src/hotspot/cpu/x86/x86_32.ad"

    int reg_encoding = opnd_array(0)->reg(ra_,this)/* dst */;
    int base  = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int displace = opnd_array(1)->disp(ra_,this,idx1);
    relocInfo::relocType disp_reloc = opnd_array(1)->disp_reloc();
    encode_RegMem(cbuf, reg_encoding, base, index, scale, displace, disp_reloc);
  
#line 17175 "ad_x86.cpp"
  }
}

void leaPIdxOffNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  {

#line 1544 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0x8D /*primary()*/));
  
#line 17190 "ad_x86.cpp"
  }
  {

#line 2096 "src/hotspot/cpu/x86/x86_32.ad"

    int reg_encoding = opnd_array(0)->reg(ra_,this)/* dst */;
    int base  = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int displace = opnd_array(1)->disp(ra_,this,idx1);
    relocInfo::relocType disp_reloc = opnd_array(1)->disp_reloc();
    encode_RegMem(cbuf, reg_encoding, base, index, scale, displace, disp_reloc);
  
#line 17204 "ad_x86.cpp"
  }
}

void leaPIdxScaleNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  {

#line 1544 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0x8D /*primary()*/));
  
#line 17219 "ad_x86.cpp"
  }
  {

#line 2096 "src/hotspot/cpu/x86/x86_32.ad"

    int reg_encoding = opnd_array(0)->reg(ra_,this)/* dst */;
    int base  = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int displace = opnd_array(1)->disp(ra_,this,idx1);
    relocInfo::relocType disp_reloc = opnd_array(1)->disp_reloc();
    encode_RegMem(cbuf, reg_encoding, base, index, scale, displace, disp_reloc);
  
#line 17233 "ad_x86.cpp"
  }
}

void leaPIdxScaleOffNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  {

#line 1544 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0x8D /*primary()*/));
  
#line 17248 "ad_x86.cpp"
  }
  {

#line 2096 "src/hotspot/cpu/x86/x86_32.ad"

    int reg_encoding = opnd_array(0)->reg(ra_,this)/* dst */;
    int base  = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int displace = opnd_array(1)->disp(ra_,this,idx1);
    relocInfo::relocType disp_reloc = opnd_array(1)->disp_reloc();
    encode_RegMem(cbuf, reg_encoding, base, index, scale, displace, disp_reloc);
  
#line 17262 "ad_x86.cpp"
  }
}

void loadConINode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {

#line 1905 "src/hotspot/cpu/x86/x86_32.ad"

    // Load immediate does not have a zero or sign extended version
    // for 8-bit immediates
    emit_opcode(cbuf, 0xB8 + opnd_array(0)->reg(ra_,this)/* dst */);
    emit_d32(cbuf, opnd_array(1)->constant());
  
#line 17280 "ad_x86.cpp"
  }
}

void loadConI0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {

#line 1544 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0x33 /*primary()*/));
  
#line 17295 "ad_x86.cpp"
  }
  {

#line 1959 "src/hotspot/cpu/x86/x86_32.ad"

    emit_rm(cbuf, 0x3, opnd_array(0)->reg(ra_,this)/* dst */, opnd_array(0)->reg(ra_,this)/* dst */);
  
#line 17303 "ad_x86.cpp"
  }
}

void loadConPNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {

#line 1912 "src/hotspot/cpu/x86/x86_32.ad"

    // Load immediate does not have a zero or sign extended version
    // for 8-bit immediates
    emit_opcode(cbuf, (0xB8 /*primary()*/) + opnd_array(0)->reg(ra_,this)/* dst */);
    
if ( opnd_array(1)->constant_reloc() != relocInfo::none ) {
  emit_d32_reloc(cbuf, opnd_array(1)->constant(), opnd_array(1)->constant_reloc(), 0);
} else {
  emit_d32(cbuf, opnd_array(1)->constant());
};
  
#line 17326 "ad_x86.cpp"
  }
}

void loadConLNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {

#line 1919 "src/hotspot/cpu/x86/x86_32.ad"

    // Load immediate does not have a zero or sign extended version
    // for 8-bit immediates
    int dst_enc = opnd_array(0)->reg(ra_,this)/* dst */;
    int src_con = opnd_array(1)->constantL() & 0x0FFFFFFFFL;
    if (src_con == 0) {
      // xor dst, dst
      emit_opcode(cbuf, 0x33);
      emit_rm(cbuf, 0x3, dst_enc, dst_enc);
    } else {
      emit_opcode(cbuf, (0xB8 /*primary()*/) + dst_enc);
      emit_d32(cbuf, src_con);
    }
  
#line 17352 "ad_x86.cpp"
  }
  {

#line 1934 "src/hotspot/cpu/x86/x86_32.ad"

    // Load immediate does not have a zero or sign extended version
    // for 8-bit immediates
    int dst_enc = opnd_array(0)->reg(ra_,this)/* dst */ + 2;
    int src_con = ((julong)(opnd_array(1)->constantL())) >> 32;
    if (src_con == 0) {
      // xor dst, dst
      emit_opcode(cbuf, 0x33);
      emit_rm(cbuf, 0x3, dst_enc, dst_enc);
    } else {
      emit_opcode(cbuf, (0xB8 /*primary()*/) + dst_enc);
      emit_d32(cbuf, src_con);
    }
  
#line 17371 "ad_x86.cpp"
  }
}

void loadConL0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {

#line 1963 "src/hotspot/cpu/x86/x86_32.ad"

    emit_d8(cbuf, (0x33 /*primary()*/));
    emit_rm(cbuf, 0x3, opnd_array(0)->reg(ra_,this)/* dst */, opnd_array(0)->reg(ra_,this)/* dst */);
  
#line 17387 "ad_x86.cpp"
  }
  {

#line 1968 "src/hotspot/cpu/x86/x86_32.ad"

    emit_d8(cbuf, (0x33 /*secondary()*/));
    emit_rm(cbuf, 0x3, HIGH_FROM_LOW_ENC(opnd_array(0)->reg(ra_,this)/* dst */), HIGH_FROM_LOW_ENC(opnd_array(0)->reg(ra_,this)/* dst */));
  
#line 17396 "ad_x86.cpp"
  }
}

void loadConFPRNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// con
  {
    C2_MacroAssembler _masm(&cbuf);

#line 6064 "src/hotspot/cpu/x86/x86_32.ad"

    __ fld_s(InternalAddress(__ code()->consts()->start() + constant_offset()));
    __ fstp_d(opnd_array(0)->reg(ra_,this)/* dst */);
  
#line 17413 "ad_x86.cpp"
  }
}

void loadConFPRNode::eval_constant(Compile* C) {
  {

#line 6065 "src/hotspot/cpu/x86/x86_32.ad"
    _constant = C->output()->constant_table().add(this, opnd_array(1));
#line 17422 "ad_x86.cpp"
  }
}
void loadConFPR0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// con
  {
    C2_MacroAssembler _masm(&cbuf);

#line 6077 "src/hotspot/cpu/x86/x86_32.ad"

    __ fldz();
    __ fstp_d(opnd_array(0)->reg(ra_,this)/* dst */);
  
#line 17438 "ad_x86.cpp"
  }
}

void loadConFPR1Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// con
  {
    C2_MacroAssembler _masm(&cbuf);

#line 6090 "src/hotspot/cpu/x86/x86_32.ad"

    __ fld1();
    __ fstp_d(opnd_array(0)->reg(ra_,this)/* dst */);
  
#line 17455 "ad_x86.cpp"
  }
}

void loadConFNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// con
  {
    C2_MacroAssembler _masm(&cbuf);

#line 6102 "src/hotspot/cpu/x86/x86_32.ad"

    __ movflt(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, InternalAddress(__ code()->consts()->start() + constant_offset()));
  
#line 17471 "ad_x86.cpp"
  }
}

void loadConFNode::eval_constant(Compile* C) {
  {

#line 6103 "src/hotspot/cpu/x86/x86_32.ad"
    _constant = C->output()->constant_table().add(this, opnd_array(1));
#line 17480 "ad_x86.cpp"
  }
}
void loadConF0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 6113 "src/hotspot/cpu/x86/x86_32.ad"

    __ xorps(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(0)->as_XMMRegister(ra_,this)/* dst */);
  
#line 17495 "ad_x86.cpp"
  }
}

void loadConDPRNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// con
  {
    C2_MacroAssembler _masm(&cbuf);

#line 6126 "src/hotspot/cpu/x86/x86_32.ad"

    __ fld_d(InternalAddress(__ code()->consts()->start() + constant_offset()));
    __ fstp_d(opnd_array(0)->reg(ra_,this)/* dst */);
  
#line 17512 "ad_x86.cpp"
  }
}

void loadConDPRNode::eval_constant(Compile* C) {
  {

#line 6127 "src/hotspot/cpu/x86/x86_32.ad"
    _constant = C->output()->constant_table().add(this, opnd_array(1));
#line 17521 "ad_x86.cpp"
  }
}
void loadConDPR0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// con
  {
    C2_MacroAssembler _masm(&cbuf);

#line 6140 "src/hotspot/cpu/x86/x86_32.ad"

    __ fldz();
    __ fstp_d(opnd_array(0)->reg(ra_,this)/* dst */);
  
#line 17537 "ad_x86.cpp"
  }
}

void loadConDPR1Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// con
  {
    C2_MacroAssembler _masm(&cbuf);

#line 6154 "src/hotspot/cpu/x86/x86_32.ad"

    __ fld1();
    __ fstp_d(opnd_array(0)->reg(ra_,this)/* dst */);
  
#line 17554 "ad_x86.cpp"
  }
}

void loadConDNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// con
  {
    C2_MacroAssembler _masm(&cbuf);

#line 6166 "src/hotspot/cpu/x86/x86_32.ad"

    __ movdbl(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, InternalAddress(__ code()->consts()->start() + constant_offset()));
  
#line 17570 "ad_x86.cpp"
  }
}

void loadConDNode::eval_constant(Compile* C) {
  {

#line 6167 "src/hotspot/cpu/x86/x86_32.ad"
    _constant = C->output()->constant_table().add(this, opnd_array(1));
#line 17579 "ad_x86.cpp"
  }
}
void loadConD0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 6177 "src/hotspot/cpu/x86/x86_32.ad"

    __ xorpd (opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(0)->as_XMMRegister(ra_,this)/* dst */);
  
#line 17594 "ad_x86.cpp"
  }
}

void loadSSINode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {

#line 1544 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0x8B /*primary()*/));
  
#line 17609 "ad_x86.cpp"
  }
  {

#line 2096 "src/hotspot/cpu/x86/x86_32.ad"

    int reg_encoding = opnd_array(0)->reg(ra_,this)/* dst */;
    int base  = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int displace = opnd_array(1)->disp(ra_,this,idx1);
    relocInfo::relocType disp_reloc = opnd_array(1)->disp_reloc();
    encode_RegMem(cbuf, reg_encoding, base, index, scale, displace, disp_reloc);
  
#line 17623 "ad_x86.cpp"
  }
}

void loadSSLNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {

#line 1544 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0x8B /*primary()*/));
  
#line 17638 "ad_x86.cpp"
  }
  {

#line 2096 "src/hotspot/cpu/x86/x86_32.ad"

    int reg_encoding = opnd_array(0)->reg(ra_,this)/* dst */;
    int base  = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int displace = opnd_array(1)->disp(ra_,this,idx1);
    relocInfo::relocType disp_reloc = opnd_array(1)->disp_reloc();
    encode_RegMem(cbuf, reg_encoding, base, index, scale, displace, disp_reloc);
  
#line 17652 "ad_x86.cpp"
  }
  {

#line 1549 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0x8B /*secondary()*/));
  
#line 17660 "ad_x86.cpp"
  }
  {

#line 2106 "src/hotspot/cpu/x86/x86_32.ad"

    int reg_encoding = HIGH_FROM_LOW_ENC(opnd_array(0)->reg(ra_,this)/* dst */);  // Hi register of pair, computed from lo
    int base  = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int displace = opnd_array(1)->disp(ra_,this,idx1) + 4;      // Offset is 4 further in memory
    assert( opnd_array(1)->disp_reloc() == relocInfo::none, "Cannot add 4 to oop" );
    encode_RegMem(cbuf, reg_encoding, base, index, scale, displace, relocInfo::none);
  
#line 17674 "ad_x86.cpp"
  }
}

void loadSSPNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {

#line 1544 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0x8B /*primary()*/));
  
#line 17689 "ad_x86.cpp"
  }
  {

#line 2096 "src/hotspot/cpu/x86/x86_32.ad"

    int reg_encoding = opnd_array(0)->reg(ra_,this)/* dst */;
    int base  = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int displace = opnd_array(1)->disp(ra_,this,idx1);
    relocInfo::relocType disp_reloc = opnd_array(1)->disp_reloc();
    encode_RegMem(cbuf, reg_encoding, base, index, scale, displace, disp_reloc);
  
#line 17703 "ad_x86.cpp"
  }
}

void loadSSFNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {

#line 1544 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0xD9 /*primary()*/));
  
#line 17718 "ad_x86.cpp"
  }
  {

#line 2176 "src/hotspot/cpu/x86/x86_32.ad"

    int rm_byte_opcode = (0x00);
    int base     = opnd_array(1)->base(ra_,this,idx1);
    int index    = opnd_array(1)->index(ra_,this,idx1);
    int scale    = opnd_array(1)->scale();
    int displace = opnd_array(1)->disp(ra_,this,idx1);
    assert( opnd_array(1)->disp_reloc() == relocInfo::none, "No oops here because no reloc info allowed" );
    encode_RegMem(cbuf, rm_byte_opcode, base, index, scale, displace, relocInfo::none);
  
#line 17732 "ad_x86.cpp"
  }
  {

#line 2422 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode( cbuf, 0xDD );           // FSTP   ST(i)
    emit_d8( cbuf, 0xD8+opnd_array(0)->reg(ra_,this)/* dst */ );
  
#line 17741 "ad_x86.cpp"
  }
}

void loadSSDNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {

#line 1544 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0xDD /*primary()*/));
  
#line 17756 "ad_x86.cpp"
  }
  {

#line 2176 "src/hotspot/cpu/x86/x86_32.ad"

    int rm_byte_opcode = (0x00);
    int base     = opnd_array(1)->base(ra_,this,idx1);
    int index    = opnd_array(1)->index(ra_,this,idx1);
    int scale    = opnd_array(1)->scale();
    int displace = opnd_array(1)->disp(ra_,this,idx1);
    assert( opnd_array(1)->disp_reloc() == relocInfo::none, "No oops here because no reloc info allowed" );
    encode_RegMem(cbuf, rm_byte_opcode, base, index, scale, displace, relocInfo::none);
  
#line 17770 "ad_x86.cpp"
  }
  {

#line 2367 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode( cbuf, 0xDD );           // FSTP   ST(i)
    emit_d8( cbuf, 0xD8+opnd_array(0)->reg(ra_,this)/* dst */ );
  
#line 17779 "ad_x86.cpp"
  }
}

void prefetchAlloc0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// 
  // User did not define which encode class to use.
}

uint prefetchAlloc0Node::size(PhaseRegAlloc *ra_) const {
  assert(VerifyOops || MachNode::size(ra_) <= 0, "bad fixed size");
  return (VerifyOops ? MachNode::size(ra_) : 0);
}

void prefetchAllocNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// 
  {
    C2_MacroAssembler _masm(&cbuf);

#line 6261 "src/hotspot/cpu/x86/x86_32.ad"

    __ prefetchw(Address::make_raw(opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1), opnd_array(1)->disp_reloc()));
  
#line 17808 "ad_x86.cpp"
  }
}

void prefetchAllocNTANode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// 
  {
    C2_MacroAssembler _masm(&cbuf);

#line 6273 "src/hotspot/cpu/x86/x86_32.ad"

    __ prefetchnta(Address::make_raw(opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1), opnd_array(1)->disp_reloc()));
  
#line 17824 "ad_x86.cpp"
  }
}

void prefetchAllocT0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// 
  {
    C2_MacroAssembler _masm(&cbuf);

#line 6285 "src/hotspot/cpu/x86/x86_32.ad"

    __ prefetcht0(Address::make_raw(opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1), opnd_array(1)->disp_reloc()));
  
#line 17840 "ad_x86.cpp"
  }
}

void prefetchAllocT2Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// 
  {
    C2_MacroAssembler _masm(&cbuf);

#line 6297 "src/hotspot/cpu/x86/x86_32.ad"

    __ prefetcht2(Address::make_raw(opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1), opnd_array(1)->disp_reloc()));
  
#line 17856 "ad_x86.cpp"
  }
}

void storeBNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {

#line 1544 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0x88 /*primary()*/));
  
#line 17872 "ad_x86.cpp"
  }
  {

#line 2096 "src/hotspot/cpu/x86/x86_32.ad"

    int reg_encoding = opnd_array(2)->reg(ra_,this,idx2)/* src */;
    int base  = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int displace = opnd_array(1)->disp(ra_,this,idx1);
    relocInfo::relocType disp_reloc = opnd_array(1)->disp_reloc();
    encode_RegMem(cbuf, reg_encoding, base, index, scale, displace, disp_reloc);
  
#line 17886 "ad_x86.cpp"
  }
}

void storeCNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {

#line 1549 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0x66 /*secondary()*/));
  
#line 17902 "ad_x86.cpp"
  }
  {

#line 1544 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0x89 /*primary()*/));
  
#line 17910 "ad_x86.cpp"
  }
  {

#line 2096 "src/hotspot/cpu/x86/x86_32.ad"

    int reg_encoding = opnd_array(2)->reg(ra_,this,idx2)/* src */;
    int base  = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int displace = opnd_array(1)->disp(ra_,this,idx1);
    relocInfo::relocType disp_reloc = opnd_array(1)->disp_reloc();
    encode_RegMem(cbuf, reg_encoding, base, index, scale, displace, disp_reloc);
  
#line 17924 "ad_x86.cpp"
  }
}

void storeINode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {

#line 1544 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0x89 /*primary()*/));
  
#line 17940 "ad_x86.cpp"
  }
  {

#line 2096 "src/hotspot/cpu/x86/x86_32.ad"

    int reg_encoding = opnd_array(2)->reg(ra_,this,idx2)/* src */;
    int base  = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int displace = opnd_array(1)->disp(ra_,this,idx1);
    relocInfo::relocType disp_reloc = opnd_array(1)->disp_reloc();
    encode_RegMem(cbuf, reg_encoding, base, index, scale, displace, disp_reloc);
  
#line 17954 "ad_x86.cpp"
  }
}

void storeLNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {

#line 1544 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0x89 /*primary()*/));
  
#line 17970 "ad_x86.cpp"
  }
  {

#line 2096 "src/hotspot/cpu/x86/x86_32.ad"

    int reg_encoding = opnd_array(2)->reg(ra_,this,idx2)/* src */;
    int base  = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int displace = opnd_array(1)->disp(ra_,this,idx1);
    relocInfo::relocType disp_reloc = opnd_array(1)->disp_reloc();
    encode_RegMem(cbuf, reg_encoding, base, index, scale, displace, disp_reloc);
  
#line 17984 "ad_x86.cpp"
  }
  {

#line 1549 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0x89 /*secondary()*/));
  
#line 17992 "ad_x86.cpp"
  }
  {

#line 2106 "src/hotspot/cpu/x86/x86_32.ad"

    int reg_encoding = HIGH_FROM_LOW_ENC(opnd_array(2)->reg(ra_,this,idx2)/* src */);  // Hi register of pair, computed from lo
    int base  = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int displace = opnd_array(1)->disp(ra_,this,idx1) + 4;      // Offset is 4 further in memory
    assert( opnd_array(1)->disp_reloc() == relocInfo::none, "Cannot add 4 to oop" );
    encode_RegMem(cbuf, reg_encoding, base, index, scale, displace, relocInfo::none);
  
#line 18006 "ad_x86.cpp"
  }
}

void storeL2INode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 6356 "src/hotspot/cpu/x86/x86_32.ad"

    __ movl(Address::make_raw(opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1), opnd_array(1)->disp_reloc()), opnd_array(2)->as_Register(ra_,this,idx2)/* src */);
  
#line 18023 "ad_x86.cpp"
  }
}

void storeL_volatileNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {

#line 1544 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0x3B /*primary()*/));
  
#line 18039 "ad_x86.cpp"
  }
  {

#line 2096 "src/hotspot/cpu/x86/x86_32.ad"

    int reg_encoding = (EAX_enc);
    int base  = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int displace = opnd_array(1)->disp(ra_,this,idx1);
    relocInfo::relocType disp_reloc = opnd_array(1)->disp_reloc();
    encode_RegMem(cbuf, reg_encoding, base, index, scale, displace, disp_reloc);
  
#line 18053 "ad_x86.cpp"
  }
  {

#line 3079 "src/hotspot/cpu/x86/x86_32.ad"

    store_to_stackslot( cbuf, 0x0DF, 0x05, opnd_array(2)->disp(ra_,this,idx2) );
    cbuf.set_insts_mark();            // Mark start of FIST in case $mem has an oop
    emit_opcode(cbuf,0xDF);
    int rm_byte_opcode = 0x07;
    int base     = opnd_array(1)->base(ra_,this,idx1);
    int index    = opnd_array(1)->index(ra_,this,idx1);
    int scale    = opnd_array(1)->scale();
    int displace = opnd_array(1)->disp(ra_,this,idx1);
    relocInfo::relocType disp_reloc = opnd_array(1)->disp_reloc(); // disp-as-oop when working with static globals
    encode_RegMem(cbuf, rm_byte_opcode, base, index, scale, displace, disp_reloc);
  
#line 18070 "ad_x86.cpp"
  }
}

void storeLX_volatileNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// tmp
  {
    C2_MacroAssembler _masm(&cbuf);

#line 6387 "src/hotspot/cpu/x86/x86_32.ad"

    __ cmpl(rax, Address::make_raw(opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1), opnd_array(1)->disp_reloc()));
    __ movdbl(opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* tmp */, Address(rsp, opnd_array(2)->disp(ra_,this,idx2)));
    __ movdbl(Address::make_raw(opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1), opnd_array(1)->disp_reloc()), opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* tmp */);
  
#line 18090 "ad_x86.cpp"
  }
}

void storeLX_reg_volatileNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// tmp2
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// tmp
  {
    C2_MacroAssembler _masm(&cbuf);

#line 6405 "src/hotspot/cpu/x86/x86_32.ad"

    __ cmpl(rax, Address::make_raw(opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1), opnd_array(1)->disp_reloc()));
    __ movdl(opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* tmp */, opnd_array(2)->as_Register(ra_,this,idx2)/* src */);
    __ movdl(opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* tmp2 */, HIGH_FROM_LOW(opnd_array(2)->as_Register(ra_,this,idx2)/* src */));
    __ punpckldq(opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* tmp */, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* tmp2 */);
    __ movdbl(Address::make_raw(opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1), opnd_array(1)->disp_reloc()), opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* tmp */);
  
#line 18113 "ad_x86.cpp"
  }
}

void storePNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {

#line 1544 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0x89 /*primary()*/));
  
#line 18129 "ad_x86.cpp"
  }
  {

#line 2096 "src/hotspot/cpu/x86/x86_32.ad"

    int reg_encoding = opnd_array(2)->reg(ra_,this,idx2)/* src */;
    int base  = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int displace = opnd_array(1)->disp(ra_,this,idx1);
    relocInfo::relocType disp_reloc = opnd_array(1)->disp_reloc();
    encode_RegMem(cbuf, reg_encoding, base, index, scale, displace, disp_reloc);
  
#line 18143 "ad_x86.cpp"
  }
}

void storeImmINode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {

#line 1544 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0xC7 /*primary()*/));
  
#line 18159 "ad_x86.cpp"
  }
  {

#line 2186 "src/hotspot/cpu/x86/x86_32.ad"

    int rm_byte_opcode = (0x00);
    int base     = opnd_array(1)->base(ra_,this,idx1);
    int index    = opnd_array(1)->index(ra_,this,idx1);
    int scale    = opnd_array(1)->scale();
    int displace = opnd_array(1)->disp(ra_,this,idx1);
    relocInfo::relocType disp_reloc = opnd_array(1)->disp_reloc(); // disp-as-oop when working with static globals
    encode_RegMem(cbuf, rm_byte_opcode, base, index, scale, displace, disp_reloc);
  
#line 18173 "ad_x86.cpp"
  }
  {

#line 1985 "src/hotspot/cpu/x86/x86_32.ad"

    // Output immediate
    emit_d32(cbuf, opnd_array(2)->constant());
  
#line 18182 "ad_x86.cpp"
  }
}

void storeImmI16Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {

#line 1558 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf,0x66);
  
#line 18198 "ad_x86.cpp"
  }
  {

#line 1544 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0xC7 /*primary()*/));
  
#line 18206 "ad_x86.cpp"
  }
  {

#line 2186 "src/hotspot/cpu/x86/x86_32.ad"

    int rm_byte_opcode = (0x00);
    int base     = opnd_array(1)->base(ra_,this,idx1);
    int index    = opnd_array(1)->index(ra_,this,idx1);
    int scale    = opnd_array(1)->scale();
    int displace = opnd_array(1)->disp(ra_,this,idx1);
    relocInfo::relocType disp_reloc = opnd_array(1)->disp_reloc(); // disp-as-oop when working with static globals
    encode_RegMem(cbuf, rm_byte_opcode, base, index, scale, displace, disp_reloc);
  
#line 18220 "ad_x86.cpp"
  }
  {

#line 2004 "src/hotspot/cpu/x86/x86_32.ad"

    // Output immediate
    emit_d16(cbuf, opnd_array(2)->constant());
  
#line 18229 "ad_x86.cpp"
  }
}

void storeImmPNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {

#line 1544 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0xC7 /*primary()*/));
  
#line 18245 "ad_x86.cpp"
  }
  {

#line 2186 "src/hotspot/cpu/x86/x86_32.ad"

    int rm_byte_opcode = (0x00);
    int base     = opnd_array(1)->base(ra_,this,idx1);
    int index    = opnd_array(1)->index(ra_,this,idx1);
    int scale    = opnd_array(1)->scale();
    int displace = opnd_array(1)->disp(ra_,this,idx1);
    relocInfo::relocType disp_reloc = opnd_array(1)->disp_reloc(); // disp-as-oop when working with static globals
    encode_RegMem(cbuf, rm_byte_opcode, base, index, scale, displace, disp_reloc);
  
#line 18259 "ad_x86.cpp"
  }
  {

#line 1985 "src/hotspot/cpu/x86/x86_32.ad"

    // Output immediate
    
if ( opnd_array(2)->constant_reloc() != relocInfo::none ) {
  emit_d32_reloc(cbuf, opnd_array(2)->constant(), opnd_array(2)->constant_reloc(), 0);
} else {
  emit_d32(cbuf, opnd_array(2)->constant());
};
  
#line 18273 "ad_x86.cpp"
  }
}

void storeImmBNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {

#line 1544 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0xC6 /*primary()*/));
  
#line 18289 "ad_x86.cpp"
  }
  {

#line 2186 "src/hotspot/cpu/x86/x86_32.ad"

    int rm_byte_opcode = (0x00);
    int base     = opnd_array(1)->base(ra_,this,idx1);
    int index    = opnd_array(1)->index(ra_,this,idx1);
    int scale    = opnd_array(1)->scale();
    int displace = opnd_array(1)->disp(ra_,this,idx1);
    relocInfo::relocType disp_reloc = opnd_array(1)->disp_reloc(); // disp-as-oop when working with static globals
    encode_RegMem(cbuf, rm_byte_opcode, base, index, scale, displace, disp_reloc);
  
#line 18303 "ad_x86.cpp"
  }
  {

#line 1646 "src/hotspot/cpu/x86/x86_32.ad"

    // Check for 8-bit immediate, and set sign extend bit in opcode
    if ((opnd_array(2)->constant() >= -128) && (opnd_array(2)->constant() <= 127)) {
      emit_d8(cbuf, opnd_array(2)->constant());
    }
    else {                          // If 32-bit immediate
      // Output immediate
      emit_d32(cbuf, opnd_array(2)->constant());
    }
  
#line 18318 "ad_x86.cpp"
  }
}

void storeImmCMNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {

#line 1544 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0xC6 /*primary()*/));
  
#line 18334 "ad_x86.cpp"
  }
  {

#line 2186 "src/hotspot/cpu/x86/x86_32.ad"

    int rm_byte_opcode = (0x00);
    int base     = opnd_array(1)->base(ra_,this,idx1);
    int index    = opnd_array(1)->index(ra_,this,idx1);
    int scale    = opnd_array(1)->scale();
    int displace = opnd_array(1)->disp(ra_,this,idx1);
    relocInfo::relocType disp_reloc = opnd_array(1)->disp_reloc(); // disp-as-oop when working with static globals
    encode_RegMem(cbuf, rm_byte_opcode, base, index, scale, displace, disp_reloc);
  
#line 18348 "ad_x86.cpp"
  }
  {

#line 1646 "src/hotspot/cpu/x86/x86_32.ad"

    // Check for 8-bit immediate, and set sign extend bit in opcode
    if ((opnd_array(2)->constant() >= -128) && (opnd_array(2)->constant() <= 127)) {
      emit_d8(cbuf, opnd_array(2)->constant());
    }
    else {                          // If 32-bit immediate
      // Output immediate
      emit_d32(cbuf, opnd_array(2)->constant());
    }
  
#line 18363 "ad_x86.cpp"
  }
}

void storeDPRNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {

#line 2230 "src/hotspot/cpu/x86/x86_32.ad"

    // If src is FPR1, we can just FST to store it.
    // Else we need to FLD it to FPR1, then FSTP to store/pop it.
    int reg_encoding = 0x2; // Just store
    int base  = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int displace = opnd_array(1)->disp(ra_,this,idx1);
    relocInfo::relocType disp_reloc = opnd_array(1)->disp_reloc(); // disp-as-oop when working with static globals
    if( opnd_array(2)->reg(ra_,this,idx2)/* src */ != FPR1L_enc ) {
      reg_encoding = 0x3;  // Store & pop
      emit_opcode( cbuf, 0xD9 ); // FLD (i.e., push it)
      emit_d8( cbuf, 0xC0-1+opnd_array(2)->reg(ra_,this,idx2)/* src */ );
    }
    cbuf.set_insts_mark();       // Mark start of opcode for reloc info in mem operand
    emit_opcode(cbuf,(0xDD /*primary()*/));
    encode_RegMem(cbuf, reg_encoding, base, index, scale, displace, disp_reloc);
  
#line 18394 "ad_x86.cpp"
  }
}

void storeDPR_roundedNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {

#line 2230 "src/hotspot/cpu/x86/x86_32.ad"

    // If src is FPR1, we can just FST to store it.
    // Else we need to FLD it to FPR1, then FSTP to store/pop it.
    int reg_encoding = 0x2; // Just store
    int base  = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int displace = opnd_array(1)->disp(ra_,this,idx1);
    relocInfo::relocType disp_reloc = opnd_array(1)->disp_reloc(); // disp-as-oop when working with static globals
    if( opnd_array(2)->reg(ra_,this,idx2)/* src */ != FPR1L_enc ) {
      reg_encoding = 0x3;  // Store & pop
      emit_opcode( cbuf, 0xD9 ); // FLD (i.e., push it)
      emit_d8( cbuf, 0xC0-1+opnd_array(2)->reg(ra_,this,idx2)/* src */ );
    }
    cbuf.set_insts_mark();       // Mark start of opcode for reloc info in mem operand
    emit_opcode(cbuf,(0xDD /*primary()*/));
    encode_RegMem(cbuf, reg_encoding, base, index, scale, displace, disp_reloc);
  
#line 18425 "ad_x86.cpp"
  }
}

void storeDNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 6514 "src/hotspot/cpu/x86/x86_32.ad"

    __ movdbl(Address::make_raw(opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1), opnd_array(1)->disp_reloc()), opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src */);
  
#line 18442 "ad_x86.cpp"
  }
}

void storeFNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 6527 "src/hotspot/cpu/x86/x86_32.ad"

    __ movflt(Address::make_raw(opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1), opnd_array(1)->disp_reloc()), opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src */);
  
#line 18459 "ad_x86.cpp"
  }
}

void storeFPRNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {

#line 2230 "src/hotspot/cpu/x86/x86_32.ad"

    // If src is FPR1, we can just FST to store it.
    // Else we need to FLD it to FPR1, then FSTP to store/pop it.
    int reg_encoding = 0x2; // Just store
    int base  = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int displace = opnd_array(1)->disp(ra_,this,idx1);
    relocInfo::relocType disp_reloc = opnd_array(1)->disp_reloc(); // disp-as-oop when working with static globals
    if( opnd_array(2)->reg(ra_,this,idx2)/* src */ != FPR1L_enc ) {
      reg_encoding = 0x3;  // Store & pop
      emit_opcode( cbuf, 0xD9 ); // FLD (i.e., push it)
      emit_d8( cbuf, 0xC0-1+opnd_array(2)->reg(ra_,this,idx2)/* src */ );
    }
    cbuf.set_insts_mark();       // Mark start of opcode for reloc info in mem operand
    emit_opcode(cbuf,(0xD9 /*primary()*/));
    encode_RegMem(cbuf, reg_encoding, base, index, scale, displace, disp_reloc);
  
#line 18490 "ad_x86.cpp"
  }
}

void storeFPR_roundedNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {

#line 2230 "src/hotspot/cpu/x86/x86_32.ad"

    // If src is FPR1, we can just FST to store it.
    // Else we need to FLD it to FPR1, then FSTP to store/pop it.
    int reg_encoding = 0x2; // Just store
    int base  = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int displace = opnd_array(1)->disp(ra_,this,idx1);
    relocInfo::relocType disp_reloc = opnd_array(1)->disp_reloc(); // disp-as-oop when working with static globals
    if( opnd_array(2)->reg(ra_,this,idx2)/* src */ != FPR1L_enc ) {
      reg_encoding = 0x3;  // Store & pop
      emit_opcode( cbuf, 0xD9 ); // FLD (i.e., push it)
      emit_d8( cbuf, 0xC0-1+opnd_array(2)->reg(ra_,this,idx2)/* src */ );
    }
    cbuf.set_insts_mark();       // Mark start of opcode for reloc info in mem operand
    emit_opcode(cbuf,(0xD9 /*primary()*/));
    encode_RegMem(cbuf, reg_encoding, base, index, scale, displace, disp_reloc);
  
#line 18521 "ad_x86.cpp"
  }
}

void storeFPR_DroundedNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {

#line 2230 "src/hotspot/cpu/x86/x86_32.ad"

    // If src is FPR1, we can just FST to store it.
    // Else we need to FLD it to FPR1, then FSTP to store/pop it.
    int reg_encoding = 0x2; // Just store
    int base  = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int displace = opnd_array(1)->disp(ra_,this,idx1);
    relocInfo::relocType disp_reloc = opnd_array(1)->disp_reloc(); // disp-as-oop when working with static globals
    if( opnd_array(2)->reg(ra_,this,idx2)/* src */ != FPR1L_enc ) {
      reg_encoding = 0x3;  // Store & pop
      emit_opcode( cbuf, 0xD9 ); // FLD (i.e., push it)
      emit_d8( cbuf, 0xC0-1+opnd_array(2)->reg(ra_,this,idx2)/* src */ );
    }
    cbuf.set_insts_mark();       // Mark start of opcode for reloc info in mem operand
    emit_opcode(cbuf,(0xD9 /*primary()*/));
    encode_RegMem(cbuf, reg_encoding, base, index, scale, displace, disp_reloc);
  
#line 18552 "ad_x86.cpp"
  }
}

void storeFPR_immNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {

#line 1544 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0xC7 /*primary()*/));
  
#line 18568 "ad_x86.cpp"
  }
  {

#line 2186 "src/hotspot/cpu/x86/x86_32.ad"

    int rm_byte_opcode = (0x00);
    int base     = opnd_array(1)->base(ra_,this,idx1);
    int index    = opnd_array(1)->index(ra_,this,idx1);
    int scale    = opnd_array(1)->scale();
    int displace = opnd_array(1)->disp(ra_,this,idx1);
    relocInfo::relocType disp_reloc = opnd_array(1)->disp_reloc(); // disp-as-oop when working with static globals
    encode_RegMem(cbuf, rm_byte_opcode, base, index, scale, displace, disp_reloc);
  
#line 18582 "ad_x86.cpp"
  }
  {

#line 1990 "src/hotspot/cpu/x86/x86_32.ad"

    // Output Float immediate bits
    jfloat jf = opnd_array(2)->constantF();
    int    jf_as_bits = jint_cast( jf );
    emit_d32(cbuf, jf_as_bits);
  
#line 18593 "ad_x86.cpp"
  }
}

void storeF_immNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {

#line 1544 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0xC7 /*primary()*/));
  
#line 18609 "ad_x86.cpp"
  }
  {

#line 2186 "src/hotspot/cpu/x86/x86_32.ad"

    int rm_byte_opcode = (0x00);
    int base     = opnd_array(1)->base(ra_,this,idx1);
    int index    = opnd_array(1)->index(ra_,this,idx1);
    int scale    = opnd_array(1)->scale();
    int displace = opnd_array(1)->disp(ra_,this,idx1);
    relocInfo::relocType disp_reloc = opnd_array(1)->disp_reloc(); // disp-as-oop when working with static globals
    encode_RegMem(cbuf, rm_byte_opcode, base, index, scale, displace, disp_reloc);
  
#line 18623 "ad_x86.cpp"
  }
  {

#line 1997 "src/hotspot/cpu/x86/x86_32.ad"

    // Output Float immediate bits
    jfloat jf = opnd_array(2)->constantF();
    int    jf_as_bits = jint_cast( jf );
    emit_d32(cbuf, jf_as_bits);
  
#line 18634 "ad_x86.cpp"
  }
}

void storeSSINode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {

#line 2394 "src/hotspot/cpu/x86/x86_32.ad"

    store_to_stackslot( cbuf, (0x89 /*primary()*/), opnd_array(1)->reg(ra_,this,idx1)/* src */, opnd_array(0)->disp(ra_,this,0) );
  
#line 18649 "ad_x86.cpp"
  }
}

void storeSSPNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {

#line 2394 "src/hotspot/cpu/x86/x86_32.ad"

    store_to_stackslot( cbuf, (0x89 /*primary()*/), opnd_array(1)->reg(ra_,this,idx1)/* src */, opnd_array(0)->disp(ra_,this,0) );
  
#line 18664 "ad_x86.cpp"
  }
}

void storeSSLNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {

#line 1544 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0x89 /*primary()*/));
  
#line 18679 "ad_x86.cpp"
  }
  {

#line 2096 "src/hotspot/cpu/x86/x86_32.ad"

    int reg_encoding = opnd_array(1)->reg(ra_,this,idx1)/* src */;
    int base  = opnd_array(0)->base(ra_,this,idx0);
    int index = opnd_array(0)->index(ra_,this,idx0);
    int scale = opnd_array(0)->scale();
    int displace = opnd_array(0)->disp(ra_,this,0);
    relocInfo::relocType disp_reloc = opnd_array(0)->disp_reloc();
    encode_RegMem(cbuf, reg_encoding, base, index, scale, displace, disp_reloc);
  
#line 18693 "ad_x86.cpp"
  }
  {

#line 1549 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0x89 /*secondary()*/));
  
#line 18701 "ad_x86.cpp"
  }
  {

#line 2106 "src/hotspot/cpu/x86/x86_32.ad"

    int reg_encoding = HIGH_FROM_LOW_ENC(opnd_array(1)->reg(ra_,this,idx1)/* src */);  // Hi register of pair, computed from lo
    int base  = opnd_array(0)->base(ra_,this,idx0);
    int index = opnd_array(0)->index(ra_,this,idx0);
    int scale = opnd_array(0)->scale();
    int displace = opnd_array(0)->disp(ra_,this,0) + 4;      // Offset is 4 further in memory
    assert( opnd_array(0)->disp_reloc() == relocInfo::none, "Cannot add 4 to oop" );
    encode_RegMem(cbuf, reg_encoding, base, index, scale, displace, relocInfo::none);
  
#line 18715 "ad_x86.cpp"
  }
}

void membar_acquireNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// 
  // User did not define which encode class to use.
}

uint membar_acquireNode::size(PhaseRegAlloc *ra_) const {
  assert(VerifyOops || MachNode::size(ra_) <= 0, "bad fixed size");
  return (VerifyOops ? MachNode::size(ra_) : 0);
}

void membar_acquire_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// 
  // User did not define which encode class to use.
}

uint membar_acquire_0Node::size(PhaseRegAlloc *ra_) const {
  assert(VerifyOops || MachNode::size(ra_) <= 0, "bad fixed size");
  return (VerifyOops ? MachNode::size(ra_) : 0);
}

void membar_acquire_lockNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// 
  // User did not define which encode class to use.
}

uint membar_acquire_lockNode::size(PhaseRegAlloc *ra_) const {
  assert(VerifyOops || MachNode::size(ra_) <= 0, "bad fixed size");
  return (VerifyOops ? MachNode::size(ra_) : 0);
}

void membar_releaseNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// 
  // User did not define which encode class to use.
}

uint membar_releaseNode::size(PhaseRegAlloc *ra_) const {
  assert(VerifyOops || MachNode::size(ra_) <= 0, "bad fixed size");
  return (VerifyOops ? MachNode::size(ra_) : 0);
}

void membar_release_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// 
  // User did not define which encode class to use.
}

uint membar_release_0Node::size(PhaseRegAlloc *ra_) const {
  assert(VerifyOops || MachNode::size(ra_) <= 0, "bad fixed size");
  return (VerifyOops ? MachNode::size(ra_) : 0);
}

void membar_release_lockNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// 
  // User did not define which encode class to use.
}

uint membar_release_lockNode::size(PhaseRegAlloc *ra_) const {
  assert(VerifyOops || MachNode::size(ra_) <= 0, "bad fixed size");
  return (VerifyOops ? MachNode::size(ra_) : 0);
}

void membar_volatileNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// 
  {
    C2_MacroAssembler _masm(&cbuf);

#line 6682 "src/hotspot/cpu/x86/x86_32.ad"

    __ membar(Assembler::StoreLoad);
  
#line 18809 "ad_x86.cpp"
  }
}

void unnecessary_membar_volatileNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// 
  // User did not define which encode class to use.
}

uint unnecessary_membar_volatileNode::size(PhaseRegAlloc *ra_) const {
  assert(VerifyOops || MachNode::size(ra_) <= 0, "bad fixed size");
  return (VerifyOops ? MachNode::size(ra_) : 0);
}

void membar_storestoreNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// 
  // User did not define which encode class to use.
}

uint membar_storestoreNode::size(PhaseRegAlloc *ra_) const {
  assert(VerifyOops || MachNode::size(ra_) <= 0, "bad fixed size");
  return (VerifyOops ? MachNode::size(ra_) : 0);
}

void membar_storestore_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// 
  // User did not define which encode class to use.
}

uint membar_storestore_0Node::size(PhaseRegAlloc *ra_) const {
  assert(VerifyOops || MachNode::size(ra_) <= 0, "bad fixed size");
  return (VerifyOops ? MachNode::size(ra_) : 0);
}

void castX2PNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  // User did not define which encode class to use.
}

void castP2XNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {

#line 1951 "src/hotspot/cpu/x86/x86_32.ad"

    encode_Copy( cbuf, opnd_array(0)->reg(ra_,this)/* dst */, opnd_array(1)->reg(ra_,this,idx1)/* src */ );
  
#line 18871 "ad_x86.cpp"
  }
}

void jmovI_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// cop
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// cr
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// dst
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 6736 "src/hotspot/cpu/x86/x86_32.ad"

    Label Lskip;
    // Invert sense of branch from sense of CMOV
    __ jccb((Assembler::Condition)(opnd_array(1)->ccode()^1), Lskip);
    __ movl(opnd_array(3)->as_Register(ra_,this,idx3)/* dst */, opnd_array(4)->as_Register(ra_,this,idx4)/* src */);
    __ bind(Lskip);
  
#line 18894 "ad_x86.cpp"
  }
}

void jmovI_regUNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// cop
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// cr
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// dst
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 6753 "src/hotspot/cpu/x86/x86_32.ad"

    Label Lskip;
    // Invert sense of branch from sense of CMOV
    __ jccb((Assembler::Condition)(opnd_array(1)->ccode()^1), Lskip);
    __ movl(opnd_array(3)->as_Register(ra_,this,idx3)/* dst */, opnd_array(4)->as_Register(ra_,this,idx4)/* src */);
    __ bind(Lskip);
  
#line 18917 "ad_x86.cpp"
  }
}

void cmovI_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// cop
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// cr
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// dst
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// src
  {

#line 1701 "src/hotspot/cpu/x86/x86_32.ad"

    emit_d8(cbuf, (0x0F /*primary()*/));
    emit_cc(cbuf, (0x40 /*secondary()*/), opnd_array(1)->ccode());
  
#line 18936 "ad_x86.cpp"
  }
  {

#line 1959 "src/hotspot/cpu/x86/x86_32.ad"

    emit_rm(cbuf, 0x3, opnd_array(3)->reg(ra_,this,idx3)/* dst */, opnd_array(4)->reg(ra_,this,idx4)/* src */);
  
#line 18944 "ad_x86.cpp"
  }
}

void cmovI_regUNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// cop
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// cr
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// dst
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// src
  {

#line 1701 "src/hotspot/cpu/x86/x86_32.ad"

    emit_d8(cbuf, (0x0F /*primary()*/));
    emit_cc(cbuf, (0x40 /*secondary()*/), opnd_array(1)->ccode());
  
#line 18963 "ad_x86.cpp"
  }
  {

#line 1959 "src/hotspot/cpu/x86/x86_32.ad"

    emit_rm(cbuf, 0x3, opnd_array(3)->reg(ra_,this,idx3)/* dst */, opnd_array(4)->reg(ra_,this,idx4)/* src */);
  
#line 18971 "ad_x86.cpp"
  }
}

void cmovI_memNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// cop
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// cr
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// dst
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// src
  {

#line 1701 "src/hotspot/cpu/x86/x86_32.ad"

    emit_d8(cbuf, (0x0F /*primary()*/));
    emit_cc(cbuf, (0x40 /*secondary()*/), opnd_array(1)->ccode());
  
#line 18990 "ad_x86.cpp"
  }
  {

#line 2096 "src/hotspot/cpu/x86/x86_32.ad"

    int reg_encoding = opnd_array(3)->reg(ra_,this,idx3)/* dst */;
    int base  = opnd_array(4)->base(ra_,this,idx4);
    int index = opnd_array(4)->index(ra_,this,idx4);
    int scale = opnd_array(4)->scale();
    int displace = opnd_array(4)->disp(ra_,this,idx4);
    relocInfo::relocType disp_reloc = opnd_array(4)->disp_reloc();
    encode_RegMem(cbuf, reg_encoding, base, index, scale, displace, disp_reloc);
  
#line 19004 "ad_x86.cpp"
  }
}

void cmovI_memUNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// cop
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// cr
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// dst
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// src
  {

#line 1701 "src/hotspot/cpu/x86/x86_32.ad"

    emit_d8(cbuf, (0x0F /*primary()*/));
    emit_cc(cbuf, (0x40 /*secondary()*/), opnd_array(1)->ccode());
  
#line 19023 "ad_x86.cpp"
  }
  {

#line 2096 "src/hotspot/cpu/x86/x86_32.ad"

    int reg_encoding = opnd_array(3)->reg(ra_,this,idx3)/* dst */;
    int base  = opnd_array(4)->base(ra_,this,idx4);
    int index = opnd_array(4)->index(ra_,this,idx4);
    int scale = opnd_array(4)->scale();
    int displace = opnd_array(4)->disp(ra_,this,idx4);
    relocInfo::relocType disp_reloc = opnd_array(4)->disp_reloc();
    encode_RegMem(cbuf, reg_encoding, base, index, scale, displace, disp_reloc);
  
#line 19037 "ad_x86.cpp"
  }
}

void cmovP_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// cop
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// cr
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// dst
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// src
  {

#line 1701 "src/hotspot/cpu/x86/x86_32.ad"

    emit_d8(cbuf, (0x0F /*primary()*/));
    emit_cc(cbuf, (0x40 /*secondary()*/), opnd_array(1)->ccode());
  
#line 19056 "ad_x86.cpp"
  }
  {

#line 1959 "src/hotspot/cpu/x86/x86_32.ad"

    emit_rm(cbuf, 0x3, opnd_array(3)->reg(ra_,this,idx3)/* dst */, opnd_array(4)->reg(ra_,this,idx4)/* src */);
  
#line 19064 "ad_x86.cpp"
  }
}

void cmovP_reg_nonP6Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// cop
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// cr
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// dst
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// src
  {

#line 1713 "src/hotspot/cpu/x86/x86_32.ad"

    // Invert sense of branch from sense of CMOV
    emit_cc( cbuf, 0x70, (opnd_array(1)->ccode()^1) );
    emit_d8( cbuf, (0x2) );
  
#line 19084 "ad_x86.cpp"
  }
  {

#line 1544 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0x8b /*primary()*/));
  
#line 19092 "ad_x86.cpp"
  }
  {

#line 1959 "src/hotspot/cpu/x86/x86_32.ad"

    emit_rm(cbuf, 0x3, opnd_array(3)->reg(ra_,this,idx3)/* dst */, opnd_array(4)->reg(ra_,this,idx4)/* src */);
  
#line 19100 "ad_x86.cpp"
  }
}

void cmovP_regUNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// cop
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// cr
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// dst
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// src
  {

#line 1701 "src/hotspot/cpu/x86/x86_32.ad"

    emit_d8(cbuf, (0x0F /*primary()*/));
    emit_cc(cbuf, (0x40 /*secondary()*/), opnd_array(1)->ccode());
  
#line 19119 "ad_x86.cpp"
  }
  {

#line 1959 "src/hotspot/cpu/x86/x86_32.ad"

    emit_rm(cbuf, 0x3, opnd_array(3)->reg(ra_,this,idx3)/* dst */, opnd_array(4)->reg(ra_,this,idx4)/* src */);
  
#line 19127 "ad_x86.cpp"
  }
}

void fcmovDPR_regUNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// cop
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// cr
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// dst
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// src
  {

#line 1706 "src/hotspot/cpu/x86/x86_32.ad"

    int op = 0xDA00 + opnd_array(1)->ccode() + (opnd_array(4)->reg(ra_,this,idx4)/* src */-1);
    emit_d8(cbuf, op >> 8 );
    emit_d8(cbuf, op & 255);
  
#line 19147 "ad_x86.cpp"
  }
}

void fcmovFPR_regUNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// cop
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// cr
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// dst
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// src
  {

#line 1706 "src/hotspot/cpu/x86/x86_32.ad"

    int op = 0xDA00 + opnd_array(1)->ccode() + (opnd_array(4)->reg(ra_,this,idx4)/* src */-1);
    emit_d8(cbuf, op >> 8 );
    emit_d8(cbuf, op & 255);
  
#line 19167 "ad_x86.cpp"
  }
}

void fcmovDPR_regSNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// cop
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// cr
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// dst
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// src
  {

#line 1713 "src/hotspot/cpu/x86/x86_32.ad"

    // Invert sense of branch from sense of CMOV
    emit_cc( cbuf, 0x70, (opnd_array(1)->ccode()^1) );
    emit_d8( cbuf, (0x4) );
  
#line 19187 "ad_x86.cpp"
  }
  {

#line 2372 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode( cbuf, 0xD9 );
    emit_d8( cbuf, 0xC0-1+opnd_array(4)->reg(ra_,this,idx4)/* src */ );   // FLD ST(i-1)
  
#line 19196 "ad_x86.cpp"
  }
  {

#line 1544 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0xdd /*primary()*/));
  
#line 19204 "ad_x86.cpp"
  }
  {

#line 1697 "src/hotspot/cpu/x86/x86_32.ad"

    emit_rm(cbuf, 0x3, (0x3 /*secondary()*/), opnd_array(3)->reg(ra_,this,idx3)/* dst */ );
  
#line 19212 "ad_x86.cpp"
  }
}

void fcmovFPR_regSNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// cop
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// cr
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// dst
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// src
  {

#line 1713 "src/hotspot/cpu/x86/x86_32.ad"

    // Invert sense of branch from sense of CMOV
    emit_cc( cbuf, 0x70, (opnd_array(1)->ccode()^1) );
    emit_d8( cbuf, (0x4) );
  
#line 19232 "ad_x86.cpp"
  }
  {

#line 2427 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode( cbuf, 0xD9 );           // FLD    ST(i-1)
    emit_d8( cbuf, 0xC0-1+opnd_array(4)->reg(ra_,this,idx4)/* src */ );
  
#line 19241 "ad_x86.cpp"
  }
  {

#line 1544 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0xdd /*primary()*/));
  
#line 19249 "ad_x86.cpp"
  }
  {

#line 1697 "src/hotspot/cpu/x86/x86_32.ad"

    emit_rm(cbuf, 0x3, (0x3 /*secondary()*/), opnd_array(3)->reg(ra_,this,idx3)/* dst */ );
  
#line 19257 "ad_x86.cpp"
  }
}

void fcmovF_regSNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// cop
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// cr
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// dst
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 6952 "src/hotspot/cpu/x86/x86_32.ad"

    Label skip;
    // Invert sense of branch from sense of CMOV
    __ jccb((Assembler::Condition)(opnd_array(1)->ccode()^1), skip);
    __ movflt(opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* src */);
    __ bind(skip);
  
#line 19280 "ad_x86.cpp"
  }
}

void fcmovD_regSNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// cop
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// cr
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// dst
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 6970 "src/hotspot/cpu/x86/x86_32.ad"

    Label skip;
    // Invert sense of branch from sense of CMOV
    __ jccb((Assembler::Condition)(opnd_array(1)->ccode()^1), skip);
    __ movdbl(opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* src */);
    __ bind(skip);
  
#line 19303 "ad_x86.cpp"
  }
}

void fcmovF_regUNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// cop
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// cr
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// dst
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 6988 "src/hotspot/cpu/x86/x86_32.ad"

    Label skip;
    // Invert sense of branch from sense of CMOV
    __ jccb((Assembler::Condition)(opnd_array(1)->ccode()^1), skip);
    __ movflt(opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* src */);
    __ bind(skip);
  
#line 19326 "ad_x86.cpp"
  }
}

void fcmovD_regUNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// cop
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// cr
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// dst
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 7015 "src/hotspot/cpu/x86/x86_32.ad"

    Label skip;
    // Invert sense of branch from sense of CMOV
    __ jccb((Assembler::Condition)(opnd_array(1)->ccode()^1), skip);
    __ movdbl(opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* src */);
    __ bind(skip);
  
#line 19349 "ad_x86.cpp"
  }
}

void cmovL_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// cop
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// cr
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// dst
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// src
  {

#line 1701 "src/hotspot/cpu/x86/x86_32.ad"

    emit_d8(cbuf, (0x0F /*primary()*/));
    emit_cc(cbuf, (0x40 /*secondary()*/), opnd_array(1)->ccode());
  
#line 19368 "ad_x86.cpp"
  }
  {

#line 1973 "src/hotspot/cpu/x86/x86_32.ad"

    emit_rm(cbuf, 0x3, opnd_array(3)->reg(ra_,this,idx3)/* dst */, opnd_array(4)->reg(ra_,this,idx4)/* src */);
  
#line 19376 "ad_x86.cpp"
  }
  {

#line 1701 "src/hotspot/cpu/x86/x86_32.ad"

    emit_d8(cbuf, (0x0F /*primary()*/));
    emit_cc(cbuf, (0x40 /*secondary()*/), opnd_array(1)->ccode());
  
#line 19385 "ad_x86.cpp"
  }
  {

#line 1977 "src/hotspot/cpu/x86/x86_32.ad"

    emit_rm(cbuf, 0x3, HIGH_FROM_LOW_ENC(opnd_array(3)->reg(ra_,this,idx3)/* dst */), HIGH_FROM_LOW_ENC(opnd_array(4)->reg(ra_,this,idx4)/* src */));
  
#line 19393 "ad_x86.cpp"
  }
}

void cmovL_regUNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// cop
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// cr
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// dst
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// src
  {

#line 1701 "src/hotspot/cpu/x86/x86_32.ad"

    emit_d8(cbuf, (0x0F /*primary()*/));
    emit_cc(cbuf, (0x40 /*secondary()*/), opnd_array(1)->ccode());
  
#line 19412 "ad_x86.cpp"
  }
  {

#line 1973 "src/hotspot/cpu/x86/x86_32.ad"

    emit_rm(cbuf, 0x3, opnd_array(3)->reg(ra_,this,idx3)/* dst */, opnd_array(4)->reg(ra_,this,idx4)/* src */);
  
#line 19420 "ad_x86.cpp"
  }
  {

#line 1701 "src/hotspot/cpu/x86/x86_32.ad"

    emit_d8(cbuf, (0x0F /*primary()*/));
    emit_cc(cbuf, (0x40 /*secondary()*/), opnd_array(1)->ccode());
  
#line 19429 "ad_x86.cpp"
  }
  {

#line 1977 "src/hotspot/cpu/x86/x86_32.ad"

    emit_rm(cbuf, 0x3, HIGH_FROM_LOW_ENC(opnd_array(3)->reg(ra_,this,idx3)/* dst */), HIGH_FROM_LOW_ENC(opnd_array(4)->reg(ra_,this,idx4)/* src */));
  
#line 19437 "ad_x86.cpp"
  }
}

void addI_eRegNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {

#line 1544 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0x03 /*primary()*/));
  
#line 19453 "ad_x86.cpp"
  }
  {

#line 1959 "src/hotspot/cpu/x86/x86_32.ad"

    emit_rm(cbuf, 0x3, opnd_array(1)->reg(ra_,this,idx1)/* dst */, opnd_array(2)->reg(ra_,this,idx2)/* src */);
  
#line 19461 "ad_x86.cpp"
  }
}

uint addI_eRegNode::size(PhaseRegAlloc *ra_) const {
  assert(VerifyOops || MachNode::size(ra_) <= 2, "bad fixed size");
  return (VerifyOops ? MachNode::size(ra_) : 2);
}

void addI_eReg_immNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {

#line 1634 "src/hotspot/cpu/x86/x86_32.ad"

    // Emit primary opcode and set sign-extend bit
    // Check for 8-bit immediate, and set sign extend bit in opcode
    if ((opnd_array(2)->constant() >= -128) && (opnd_array(2)->constant() <= 127)) {
      emit_opcode(cbuf, (0x81 /*primary()*/) | 0x02);    }
    else {                          // If 32-bit immediate
      emit_opcode(cbuf, (0x81 /*primary()*/));
    }
    // Emit r/m byte with secondary opcode, after primary opcode.
    emit_rm(cbuf, 0x3, (0x00 /*secondary()*/), opnd_array(1)->reg(ra_,this,idx1)/* dst */);
  
#line 19490 "ad_x86.cpp"
  }
  {

#line 1646 "src/hotspot/cpu/x86/x86_32.ad"

    // Check for 8-bit immediate, and set sign extend bit in opcode
    if ((opnd_array(2)->constant() >= -128) && (opnd_array(2)->constant() <= 127)) {
      emit_d8(cbuf, opnd_array(2)->constant());
    }
    else {                          // If 32-bit immediate
      // Output immediate
      emit_d32(cbuf, opnd_array(2)->constant());
    }
  
#line 19505 "ad_x86.cpp"
  }
}

void incI_eRegNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {

#line 1618 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0x40 /*primary()*/) + opnd_array(1)->reg(ra_,this,idx1)/* dst */);
  
#line 19521 "ad_x86.cpp"
  }
}

uint incI_eRegNode::size(PhaseRegAlloc *ra_) const {
  assert(VerifyOops || MachNode::size(ra_) <= 1, "bad fixed size");
  return (VerifyOops ? MachNode::size(ra_) : 1);
}

void leaI_eReg_immINode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src0
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src1
  {

#line 1544 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0x8D /*primary()*/));
  
#line 19542 "ad_x86.cpp"
  }
  {

#line 2196 "src/hotspot/cpu/x86/x86_32.ad"

    int reg_encoding = opnd_array(0)->reg(ra_,this)/* dst */;
    int base         = opnd_array(1)->reg(ra_,this,idx1)/* src0 */;      // 0xFFFFFFFF indicates no base
    int index        = 0x04;            // 0x04 indicates no index
    int scale        = 0x00;            // 0x00 indicates no scale
    int displace     = opnd_array(2)->constant(); // 0x00 indicates no displacement
    relocInfo::relocType disp_reloc = relocInfo::none;
    encode_RegMem(cbuf, reg_encoding, base, index, scale, displace, disp_reloc);
  
#line 19556 "ad_x86.cpp"
  }
}

void leaP_eReg_immINode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// src0
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src1
  {

#line 1544 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0x8D /*primary()*/));
  
#line 19572 "ad_x86.cpp"
  }
  {

#line 2196 "src/hotspot/cpu/x86/x86_32.ad"

    int reg_encoding = opnd_array(0)->reg(ra_,this)/* dst */;
    int base         = opnd_array(1)->reg(ra_,this,idx1)/* src0 */;      // 0xFFFFFFFF indicates no base
    int index        = 0x04;            // 0x04 indicates no index
    int scale        = 0x00;            // 0x00 indicates no scale
    int displace     = opnd_array(2)->constant(); // 0x00 indicates no displacement
    relocInfo::relocType disp_reloc = relocInfo::none;
    encode_RegMem(cbuf, reg_encoding, base, index, scale, displace, disp_reloc);
  
#line 19586 "ad_x86.cpp"
  }
}

void decI_eRegNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {

#line 1618 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0x48 /*primary()*/) + opnd_array(1)->reg(ra_,this,idx1)/* dst */);
  
#line 19602 "ad_x86.cpp"
  }
}

uint decI_eRegNode::size(PhaseRegAlloc *ra_) const {
  assert(VerifyOops || MachNode::size(ra_) <= 1, "bad fixed size");
  return (VerifyOops ? MachNode::size(ra_) : 1);
}

void addP_eRegNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {

#line 1544 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0x03 /*primary()*/));
  
#line 19623 "ad_x86.cpp"
  }
  {

#line 1959 "src/hotspot/cpu/x86/x86_32.ad"

    emit_rm(cbuf, 0x3, opnd_array(1)->reg(ra_,this,idx1)/* dst */, opnd_array(2)->reg(ra_,this,idx2)/* src */);
  
#line 19631 "ad_x86.cpp"
  }
}

uint addP_eRegNode::size(PhaseRegAlloc *ra_) const {
  assert(VerifyOops || MachNode::size(ra_) <= 2, "bad fixed size");
  return (VerifyOops ? MachNode::size(ra_) : 2);
}

void addP_eReg_immNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {

#line 1634 "src/hotspot/cpu/x86/x86_32.ad"

    // Emit primary opcode and set sign-extend bit
    // Check for 8-bit immediate, and set sign extend bit in opcode
    if ((opnd_array(2)->constant() >= -128) && (opnd_array(2)->constant() <= 127)) {
      emit_opcode(cbuf, (0x81 /*primary()*/) | 0x02);    }
    else {                          // If 32-bit immediate
      emit_opcode(cbuf, (0x81 /*primary()*/));
    }
    // Emit r/m byte with secondary opcode, after primary opcode.
    emit_rm(cbuf, 0x3, (0x00 /*secondary()*/), opnd_array(1)->reg(ra_,this,idx1)/* dst */);
  
#line 19660 "ad_x86.cpp"
  }
  {

#line 1646 "src/hotspot/cpu/x86/x86_32.ad"

    // Check for 8-bit immediate, and set sign extend bit in opcode
    if ((opnd_array(2)->constant() >= -128) && (opnd_array(2)->constant() <= 127)) {
      emit_d8(cbuf, opnd_array(2)->constant());
    }
    else {                          // If 32-bit immediate
      // Output immediate
      emit_d32(cbuf, opnd_array(2)->constant());
    }
  
#line 19675 "ad_x86.cpp"
  }
}

void addI_eReg_memNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {

#line 1544 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0x03 /*primary()*/));
  
#line 19691 "ad_x86.cpp"
  }
  {

#line 2096 "src/hotspot/cpu/x86/x86_32.ad"

    int reg_encoding = opnd_array(1)->reg(ra_,this,idx1)/* dst */;
    int base  = opnd_array(2)->base(ra_,this,idx2);
    int index = opnd_array(2)->index(ra_,this,idx2);
    int scale = opnd_array(2)->scale();
    int displace = opnd_array(2)->disp(ra_,this,idx2);
    relocInfo::relocType disp_reloc = opnd_array(2)->disp_reloc();
    encode_RegMem(cbuf, reg_encoding, base, index, scale, displace, disp_reloc);
  
#line 19705 "ad_x86.cpp"
  }
}

void addI_eReg_mem_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// dst
  {

#line 1544 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0x03 /*primary()*/));
  
#line 19721 "ad_x86.cpp"
  }
  {

#line 2096 "src/hotspot/cpu/x86/x86_32.ad"

    int reg_encoding = opnd_array(2)->reg(ra_,this,idx2)/* dst */;
    int base  = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int displace = opnd_array(1)->disp(ra_,this,idx1);
    relocInfo::relocType disp_reloc = opnd_array(1)->disp_reloc();
    encode_RegMem(cbuf, reg_encoding, base, index, scale, displace, disp_reloc);
  
#line 19735 "ad_x86.cpp"
  }
}

void addI_mem_eRegNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {

#line 1544 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0x01 /*primary()*/));
  
#line 19751 "ad_x86.cpp"
  }
  {

#line 2096 "src/hotspot/cpu/x86/x86_32.ad"

    int reg_encoding = opnd_array(2)->reg(ra_,this,idx2)/* src */;
    int base  = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int displace = opnd_array(1)->disp(ra_,this,idx1);
    relocInfo::relocType disp_reloc = opnd_array(1)->disp_reloc();
    encode_RegMem(cbuf, reg_encoding, base, index, scale, displace, disp_reloc);
  
#line 19765 "ad_x86.cpp"
  }
}

void addI_mem_eReg_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {

#line 1544 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0x01 /*primary()*/));
  
#line 19781 "ad_x86.cpp"
  }
  {

#line 2096 "src/hotspot/cpu/x86/x86_32.ad"

    int reg_encoding = opnd_array(2)->reg(ra_,this,idx2)/* src */;
    int base  = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int displace = opnd_array(1)->disp(ra_,this,idx1);
    relocInfo::relocType disp_reloc = opnd_array(1)->disp_reloc();
    encode_RegMem(cbuf, reg_encoding, base, index, scale, displace, disp_reloc);
  
#line 19795 "ad_x86.cpp"
  }
}

void addI_mem_immNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {

#line 1624 "src/hotspot/cpu/x86/x86_32.ad"

    // Check for 8-bit immediate, and set sign extend bit in opcode
    if ((opnd_array(2)->constant() >= -128) && (opnd_array(2)->constant() <= 127)) {
      emit_opcode(cbuf, (0x81 /*primary()*/) | 0x02);
    }
    else {                          // If 32-bit immediate
      emit_opcode(cbuf, (0x81 /*primary()*/));
    }
  
#line 19817 "ad_x86.cpp"
  }
  {

#line 2186 "src/hotspot/cpu/x86/x86_32.ad"

    int rm_byte_opcode = (0x00);
    int base     = opnd_array(1)->base(ra_,this,idx1);
    int index    = opnd_array(1)->index(ra_,this,idx1);
    int scale    = opnd_array(1)->scale();
    int displace = opnd_array(1)->disp(ra_,this,idx1);
    relocInfo::relocType disp_reloc = opnd_array(1)->disp_reloc(); // disp-as-oop when working with static globals
    encode_RegMem(cbuf, rm_byte_opcode, base, index, scale, displace, disp_reloc);
  
#line 19831 "ad_x86.cpp"
  }
  {

#line 1646 "src/hotspot/cpu/x86/x86_32.ad"

    // Check for 8-bit immediate, and set sign extend bit in opcode
    if ((opnd_array(2)->constant() >= -128) && (opnd_array(2)->constant() <= 127)) {
      emit_d8(cbuf, opnd_array(2)->constant());
    }
    else {                          // If 32-bit immediate
      // Output immediate
      emit_d32(cbuf, opnd_array(2)->constant());
    }
  
#line 19846 "ad_x86.cpp"
  }
}

void incI_memNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {

#line 1544 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0xFF /*primary()*/));
  
#line 19862 "ad_x86.cpp"
  }
  {

#line 2186 "src/hotspot/cpu/x86/x86_32.ad"

    int rm_byte_opcode = (0x00);
    int base     = opnd_array(1)->base(ra_,this,idx1);
    int index    = opnd_array(1)->index(ra_,this,idx1);
    int scale    = opnd_array(1)->scale();
    int displace = opnd_array(1)->disp(ra_,this,idx1);
    relocInfo::relocType disp_reloc = opnd_array(1)->disp_reloc(); // disp-as-oop when working with static globals
    encode_RegMem(cbuf, rm_byte_opcode, base, index, scale, displace, disp_reloc);
  
#line 19876 "ad_x86.cpp"
  }
}

void decI_memNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {

#line 1544 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0xFF /*primary()*/));
  
#line 19892 "ad_x86.cpp"
  }
  {

#line 2186 "src/hotspot/cpu/x86/x86_32.ad"

    int rm_byte_opcode = (0x01);
    int base     = opnd_array(1)->base(ra_,this,idx1);
    int index    = opnd_array(1)->index(ra_,this,idx1);
    int scale    = opnd_array(1)->scale();
    int displace = opnd_array(1)->disp(ra_,this,idx1);
    relocInfo::relocType disp_reloc = opnd_array(1)->disp_reloc(); // disp-as-oop when working with static globals
    encode_RegMem(cbuf, rm_byte_opcode, base, index, scale, displace, disp_reloc);
  
#line 19906 "ad_x86.cpp"
  }
}

void checkCastPPNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  // User did not define which encode class to use.
}

uint checkCastPPNode::size(PhaseRegAlloc *ra_) const {
  assert(VerifyOops || MachNode::size(ra_) <= 0, "bad fixed size");
  return (VerifyOops ? MachNode::size(ra_) : 0);
}

void castPPNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  // User did not define which encode class to use.
}

void castIINode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  // User did not define which encode class to use.
}

void castLLNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  // User did not define which encode class to use.
}

void castFFNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  // User did not define which encode class to use.
}

void castDDNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  // User did not define which encode class to use.
}

void castFF_PRNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  // User did not define which encode class to use.
}

void castDD_PRNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  // User did not define which encode class to use.
}

void compareAndSwapLNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem_ptr
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// oldval
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// newval
  {

#line 2028 "src/hotspot/cpu/x86/x86_32.ad"


    // XCHG  rbx,ecx
    emit_opcode(cbuf,0x87);
    emit_opcode(cbuf,0xD9);
    // [Lock]
    emit_opcode(cbuf,0xF0);
    // CMPXCHG8 [Eptr]
    emit_opcode(cbuf,0x0F);
    emit_opcode(cbuf,0xC7);
    emit_rm( cbuf, 0x0, 1, opnd_array(1)->reg(ra_,this,idx1)/* mem_ptr */ );
    // XCHG  rbx,ecx
    emit_opcode(cbuf,0x87);
    emit_opcode(cbuf,0xD9);
  
#line 20004 "ad_x86.cpp"
  }
  {

#line 2077 "src/hotspot/cpu/x86/x86_32.ad"

    int res_encoding = opnd_array(0)->reg(ra_,this)/* res */;

    // MOV  res,0
    emit_opcode( cbuf, 0xB8 + res_encoding);
    emit_d32( cbuf, 0 );
    // JNE,s  fail
    emit_opcode(cbuf,0x75);
    emit_d8(cbuf, 5 );
    // MOV  res,1
    emit_opcode( cbuf, 0xB8 + res_encoding);
    emit_d32( cbuf, 1 );
    // fail:
  
#line 20023 "ad_x86.cpp"
  }
}

void compareAndSwapL_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem_ptr
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// oldval
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// newval
  {

#line 2028 "src/hotspot/cpu/x86/x86_32.ad"


    // XCHG  rbx,ecx
    emit_opcode(cbuf,0x87);
    emit_opcode(cbuf,0xD9);
    // [Lock]
    emit_opcode(cbuf,0xF0);
    // CMPXCHG8 [Eptr]
    emit_opcode(cbuf,0x0F);
    emit_opcode(cbuf,0xC7);
    emit_rm( cbuf, 0x0, 1, opnd_array(1)->reg(ra_,this,idx1)/* mem_ptr */ );
    // XCHG  rbx,ecx
    emit_opcode(cbuf,0x87);
    emit_opcode(cbuf,0xD9);
  
#line 20052 "ad_x86.cpp"
  }
  {

#line 2077 "src/hotspot/cpu/x86/x86_32.ad"

    int res_encoding = opnd_array(0)->reg(ra_,this)/* res */;

    // MOV  res,0
    emit_opcode( cbuf, 0xB8 + res_encoding);
    emit_d32( cbuf, 0 );
    // JNE,s  fail
    emit_opcode(cbuf,0x75);
    emit_d8(cbuf, 5 );
    // MOV  res,1
    emit_opcode( cbuf, 0xB8 + res_encoding);
    emit_d32( cbuf, 1 );
    // fail:
  
#line 20071 "ad_x86.cpp"
  }
}

void compareAndSwapPNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem_ptr
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// oldval
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// newval
  {

#line 2044 "src/hotspot/cpu/x86/x86_32.ad"

    // [Lock]
    emit_opcode(cbuf,0xF0);

    // CMPXCHG [Eptr]
    emit_opcode(cbuf,0x0F);
    emit_opcode(cbuf,0xB1);
    emit_rm( cbuf, 0x0, 1, opnd_array(1)->reg(ra_,this,idx1)/* mem_ptr */ );
  
#line 20094 "ad_x86.cpp"
  }
  {

#line 2077 "src/hotspot/cpu/x86/x86_32.ad"

    int res_encoding = opnd_array(0)->reg(ra_,this)/* res */;

    // MOV  res,0
    emit_opcode( cbuf, 0xB8 + res_encoding);
    emit_d32( cbuf, 0 );
    // JNE,s  fail
    emit_opcode(cbuf,0x75);
    emit_d8(cbuf, 5 );
    // MOV  res,1
    emit_opcode( cbuf, 0xB8 + res_encoding);
    emit_d32( cbuf, 1 );
    // fail:
  
#line 20113 "ad_x86.cpp"
  }
}

void compareAndSwapP_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem_ptr
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// oldval
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// newval
  {

#line 2044 "src/hotspot/cpu/x86/x86_32.ad"

    // [Lock]
    emit_opcode(cbuf,0xF0);

    // CMPXCHG [Eptr]
    emit_opcode(cbuf,0x0F);
    emit_opcode(cbuf,0xB1);
    emit_rm( cbuf, 0x0, 1, opnd_array(1)->reg(ra_,this,idx1)/* mem_ptr */ );
  
#line 20136 "ad_x86.cpp"
  }
  {

#line 2077 "src/hotspot/cpu/x86/x86_32.ad"

    int res_encoding = opnd_array(0)->reg(ra_,this)/* res */;

    // MOV  res,0
    emit_opcode( cbuf, 0xB8 + res_encoding);
    emit_d32( cbuf, 0 );
    // JNE,s  fail
    emit_opcode(cbuf,0x75);
    emit_d8(cbuf, 5 );
    // MOV  res,1
    emit_opcode( cbuf, 0xB8 + res_encoding);
    emit_d32( cbuf, 1 );
    // fail:
  
#line 20155 "ad_x86.cpp"
  }
}

void compareAndSwapBNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem_ptr
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// oldval
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// newval
  {

#line 2054 "src/hotspot/cpu/x86/x86_32.ad"

    // [Lock]
    emit_opcode(cbuf,0xF0);

    // CMPXCHGB [Eptr]
    emit_opcode(cbuf,0x0F);
    emit_opcode(cbuf,0xB0);
    emit_rm( cbuf, 0x0, 1, opnd_array(1)->reg(ra_,this,idx1)/* mem_ptr */ );
  
#line 20178 "ad_x86.cpp"
  }
  {

#line 2077 "src/hotspot/cpu/x86/x86_32.ad"

    int res_encoding = opnd_array(0)->reg(ra_,this)/* res */;

    // MOV  res,0
    emit_opcode( cbuf, 0xB8 + res_encoding);
    emit_d32( cbuf, 0 );
    // JNE,s  fail
    emit_opcode(cbuf,0x75);
    emit_d8(cbuf, 5 );
    // MOV  res,1
    emit_opcode( cbuf, 0xB8 + res_encoding);
    emit_d32( cbuf, 1 );
    // fail:
  
#line 20197 "ad_x86.cpp"
  }
}

void compareAndSwapB_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem_ptr
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// oldval
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// newval
  {

#line 2054 "src/hotspot/cpu/x86/x86_32.ad"

    // [Lock]
    emit_opcode(cbuf,0xF0);

    // CMPXCHGB [Eptr]
    emit_opcode(cbuf,0x0F);
    emit_opcode(cbuf,0xB0);
    emit_rm( cbuf, 0x0, 1, opnd_array(1)->reg(ra_,this,idx1)/* mem_ptr */ );
  
#line 20220 "ad_x86.cpp"
  }
  {

#line 2077 "src/hotspot/cpu/x86/x86_32.ad"

    int res_encoding = opnd_array(0)->reg(ra_,this)/* res */;

    // MOV  res,0
    emit_opcode( cbuf, 0xB8 + res_encoding);
    emit_d32( cbuf, 0 );
    // JNE,s  fail
    emit_opcode(cbuf,0x75);
    emit_d8(cbuf, 5 );
    // MOV  res,1
    emit_opcode( cbuf, 0xB8 + res_encoding);
    emit_d32( cbuf, 1 );
    // fail:
  
#line 20239 "ad_x86.cpp"
  }
}

void compareAndSwapSNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem_ptr
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// oldval
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// newval
  {

#line 2064 "src/hotspot/cpu/x86/x86_32.ad"

    // [Lock]
    emit_opcode(cbuf,0xF0);

    // 16-bit mode
    emit_opcode(cbuf, 0x66);

    // CMPXCHGW [Eptr]
    emit_opcode(cbuf,0x0F);
    emit_opcode(cbuf,0xB1);
    emit_rm( cbuf, 0x0, 1, opnd_array(1)->reg(ra_,this,idx1)/* mem_ptr */ );
  
#line 20265 "ad_x86.cpp"
  }
  {

#line 2077 "src/hotspot/cpu/x86/x86_32.ad"

    int res_encoding = opnd_array(0)->reg(ra_,this)/* res */;

    // MOV  res,0
    emit_opcode( cbuf, 0xB8 + res_encoding);
    emit_d32( cbuf, 0 );
    // JNE,s  fail
    emit_opcode(cbuf,0x75);
    emit_d8(cbuf, 5 );
    // MOV  res,1
    emit_opcode( cbuf, 0xB8 + res_encoding);
    emit_d32( cbuf, 1 );
    // fail:
  
#line 20284 "ad_x86.cpp"
  }
}

void compareAndSwapS_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem_ptr
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// oldval
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// newval
  {

#line 2064 "src/hotspot/cpu/x86/x86_32.ad"

    // [Lock]
    emit_opcode(cbuf,0xF0);

    // 16-bit mode
    emit_opcode(cbuf, 0x66);

    // CMPXCHGW [Eptr]
    emit_opcode(cbuf,0x0F);
    emit_opcode(cbuf,0xB1);
    emit_rm( cbuf, 0x0, 1, opnd_array(1)->reg(ra_,this,idx1)/* mem_ptr */ );
  
#line 20310 "ad_x86.cpp"
  }
  {

#line 2077 "src/hotspot/cpu/x86/x86_32.ad"

    int res_encoding = opnd_array(0)->reg(ra_,this)/* res */;

    // MOV  res,0
    emit_opcode( cbuf, 0xB8 + res_encoding);
    emit_d32( cbuf, 0 );
    // JNE,s  fail
    emit_opcode(cbuf,0x75);
    emit_d8(cbuf, 5 );
    // MOV  res,1
    emit_opcode( cbuf, 0xB8 + res_encoding);
    emit_d32( cbuf, 1 );
    // fail:
  
#line 20329 "ad_x86.cpp"
  }
}

void compareAndSwapINode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem_ptr
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// oldval
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// newval
  {

#line 2044 "src/hotspot/cpu/x86/x86_32.ad"

    // [Lock]
    emit_opcode(cbuf,0xF0);

    // CMPXCHG [Eptr]
    emit_opcode(cbuf,0x0F);
    emit_opcode(cbuf,0xB1);
    emit_rm( cbuf, 0x0, 1, opnd_array(1)->reg(ra_,this,idx1)/* mem_ptr */ );
  
#line 20352 "ad_x86.cpp"
  }
  {

#line 2077 "src/hotspot/cpu/x86/x86_32.ad"

    int res_encoding = opnd_array(0)->reg(ra_,this)/* res */;

    // MOV  res,0
    emit_opcode( cbuf, 0xB8 + res_encoding);
    emit_d32( cbuf, 0 );
    // JNE,s  fail
    emit_opcode(cbuf,0x75);
    emit_d8(cbuf, 5 );
    // MOV  res,1
    emit_opcode( cbuf, 0xB8 + res_encoding);
    emit_d32( cbuf, 1 );
    // fail:
  
#line 20371 "ad_x86.cpp"
  }
}

void compareAndSwapI_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem_ptr
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// oldval
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// newval
  {

#line 2044 "src/hotspot/cpu/x86/x86_32.ad"

    // [Lock]
    emit_opcode(cbuf,0xF0);

    // CMPXCHG [Eptr]
    emit_opcode(cbuf,0x0F);
    emit_opcode(cbuf,0xB1);
    emit_rm( cbuf, 0x0, 1, opnd_array(1)->reg(ra_,this,idx1)/* mem_ptr */ );
  
#line 20394 "ad_x86.cpp"
  }
  {

#line 2077 "src/hotspot/cpu/x86/x86_32.ad"

    int res_encoding = opnd_array(0)->reg(ra_,this)/* res */;

    // MOV  res,0
    emit_opcode( cbuf, 0xB8 + res_encoding);
    emit_d32( cbuf, 0 );
    // JNE,s  fail
    emit_opcode(cbuf,0x75);
    emit_d8(cbuf, 5 );
    // MOV  res,1
    emit_opcode( cbuf, 0xB8 + res_encoding);
    emit_d32( cbuf, 1 );
    // fail:
  
#line 20413 "ad_x86.cpp"
  }
}

void compareAndExchangeLNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem_ptr
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// oldval
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// newval
  {

#line 2028 "src/hotspot/cpu/x86/x86_32.ad"


    // XCHG  rbx,ecx
    emit_opcode(cbuf,0x87);
    emit_opcode(cbuf,0xD9);
    // [Lock]
    emit_opcode(cbuf,0xF0);
    // CMPXCHG8 [Eptr]
    emit_opcode(cbuf,0x0F);
    emit_opcode(cbuf,0xC7);
    emit_rm( cbuf, 0x0, 1, opnd_array(1)->reg(ra_,this,idx1)/* mem_ptr */ );
    // XCHG  rbx,ecx
    emit_opcode(cbuf,0x87);
    emit_opcode(cbuf,0xD9);
  
#line 20442 "ad_x86.cpp"
  }
}

void compareAndExchangePNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem_ptr
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// oldval
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// newval
  {

#line 2044 "src/hotspot/cpu/x86/x86_32.ad"

    // [Lock]
    emit_opcode(cbuf,0xF0);

    // CMPXCHG [Eptr]
    emit_opcode(cbuf,0x0F);
    emit_opcode(cbuf,0xB1);
    emit_rm( cbuf, 0x0, 1, opnd_array(1)->reg(ra_,this,idx1)/* mem_ptr */ );
  
#line 20465 "ad_x86.cpp"
  }
}

void compareAndExchangeBNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem_ptr
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// oldval
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// newval
  {

#line 2054 "src/hotspot/cpu/x86/x86_32.ad"

    // [Lock]
    emit_opcode(cbuf,0xF0);

    // CMPXCHGB [Eptr]
    emit_opcode(cbuf,0x0F);
    emit_opcode(cbuf,0xB0);
    emit_rm( cbuf, 0x0, 1, opnd_array(1)->reg(ra_,this,idx1)/* mem_ptr */ );
  
#line 20488 "ad_x86.cpp"
  }
}

void compareAndExchangeSNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem_ptr
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// oldval
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// newval
  {

#line 2064 "src/hotspot/cpu/x86/x86_32.ad"

    // [Lock]
    emit_opcode(cbuf,0xF0);

    // 16-bit mode
    emit_opcode(cbuf, 0x66);

    // CMPXCHGW [Eptr]
    emit_opcode(cbuf,0x0F);
    emit_opcode(cbuf,0xB1);
    emit_rm( cbuf, 0x0, 1, opnd_array(1)->reg(ra_,this,idx1)/* mem_ptr */ );
  
#line 20514 "ad_x86.cpp"
  }
}

void compareAndExchangeINode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem_ptr
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// oldval
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// newval
  {

#line 2044 "src/hotspot/cpu/x86/x86_32.ad"

    // [Lock]
    emit_opcode(cbuf,0xF0);

    // CMPXCHG [Eptr]
    emit_opcode(cbuf,0x0F);
    emit_opcode(cbuf,0xB1);
    emit_rm( cbuf, 0x0, 1, opnd_array(1)->reg(ra_,this,idx1)/* mem_ptr */ );
  
#line 20537 "ad_x86.cpp"
  }
}

void xaddB_no_resNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// add
  {
    C2_MacroAssembler _masm(&cbuf);

#line 7398 "src/hotspot/cpu/x86/x86_32.ad"

    __ lock();
    __ addb(Address::make_raw(opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1), opnd_array(1)->disp_reloc()), opnd_array(2)->constant());
  
#line 20555 "ad_x86.cpp"
  }
}

void xaddBNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// newval
  {
    C2_MacroAssembler _masm(&cbuf);

#line 7410 "src/hotspot/cpu/x86/x86_32.ad"

    __ lock();
    __ xaddb(Address::make_raw(opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1), opnd_array(1)->disp_reloc()), opnd_array(2)->as_Register(ra_,this,idx2)/* newval */);
  
#line 20573 "ad_x86.cpp"
  }
}

void xaddS_no_resNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// add
  {
    C2_MacroAssembler _masm(&cbuf);

#line 7422 "src/hotspot/cpu/x86/x86_32.ad"

    __ lock();
    __ addw(Address::make_raw(opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1), opnd_array(1)->disp_reloc()), opnd_array(2)->constant());
  
#line 20591 "ad_x86.cpp"
  }
}

void xaddSNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// newval
  {
    C2_MacroAssembler _masm(&cbuf);

#line 7433 "src/hotspot/cpu/x86/x86_32.ad"

    __ lock();
    __ xaddw(Address::make_raw(opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1), opnd_array(1)->disp_reloc()), opnd_array(2)->as_Register(ra_,this,idx2)/* newval */);
  
#line 20609 "ad_x86.cpp"
  }
}

void xaddI_no_resNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// add
  {
    C2_MacroAssembler _masm(&cbuf);

#line 7445 "src/hotspot/cpu/x86/x86_32.ad"

    __ lock();
    __ addl(Address::make_raw(opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1), opnd_array(1)->disp_reloc()), opnd_array(2)->constant());
  
#line 20627 "ad_x86.cpp"
  }
}

void xaddINode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// newval
  {
    C2_MacroAssembler _masm(&cbuf);

#line 7456 "src/hotspot/cpu/x86/x86_32.ad"

    __ lock();
    __ xaddl(Address::make_raw(opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1), opnd_array(1)->disp_reloc()), opnd_array(2)->as_Register(ra_,this,idx2)/* newval */);
  
#line 20645 "ad_x86.cpp"
  }
}

void xchgBNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// newval
  {
    C2_MacroAssembler _masm(&cbuf);

#line 7467 "src/hotspot/cpu/x86/x86_32.ad"

    __ xchgb(opnd_array(2)->as_Register(ra_,this,idx2)/* newval */, Address::make_raw(opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1), opnd_array(1)->disp_reloc()));
  
#line 20662 "ad_x86.cpp"
  }
}

void xchgSNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// newval
  {
    C2_MacroAssembler _masm(&cbuf);

#line 7476 "src/hotspot/cpu/x86/x86_32.ad"

    __ xchgw(opnd_array(2)->as_Register(ra_,this,idx2)/* newval */, Address::make_raw(opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1), opnd_array(1)->disp_reloc()));
  
#line 20679 "ad_x86.cpp"
  }
}

void xchgINode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// newval
  {
    C2_MacroAssembler _masm(&cbuf);

#line 7485 "src/hotspot/cpu/x86/x86_32.ad"

    __ xchgl(opnd_array(2)->as_Register(ra_,this,idx2)/* newval */, Address::make_raw(opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1), opnd_array(1)->disp_reloc()));
  
#line 20696 "ad_x86.cpp"
  }
}

void xchgPNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// newval
  {
    C2_MacroAssembler _masm(&cbuf);

#line 7494 "src/hotspot/cpu/x86/x86_32.ad"

    __ xchgl(opnd_array(2)->as_Register(ra_,this,idx2)/* newval */, Address::make_raw(opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1), opnd_array(1)->disp_reloc()));
  
#line 20713 "ad_x86.cpp"
  }
}

void subI_eRegNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {

#line 1544 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0x2B /*primary()*/));
  
#line 20729 "ad_x86.cpp"
  }
  {

#line 1959 "src/hotspot/cpu/x86/x86_32.ad"

    emit_rm(cbuf, 0x3, opnd_array(1)->reg(ra_,this,idx1)/* dst */, opnd_array(2)->reg(ra_,this,idx2)/* src */);
  
#line 20737 "ad_x86.cpp"
  }
}

uint subI_eRegNode::size(PhaseRegAlloc *ra_) const {
  assert(VerifyOops || MachNode::size(ra_) <= 2, "bad fixed size");
  return (VerifyOops ? MachNode::size(ra_) : 2);
}

void subI_eReg_immNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {

#line 1634 "src/hotspot/cpu/x86/x86_32.ad"

    // Emit primary opcode and set sign-extend bit
    // Check for 8-bit immediate, and set sign extend bit in opcode
    if ((opnd_array(2)->constant() >= -128) && (opnd_array(2)->constant() <= 127)) {
      emit_opcode(cbuf, (0x81 /*primary()*/) | 0x02);    }
    else {                          // If 32-bit immediate
      emit_opcode(cbuf, (0x81 /*primary()*/));
    }
    // Emit r/m byte with secondary opcode, after primary opcode.
    emit_rm(cbuf, 0x3, (0x05 /*secondary()*/), opnd_array(1)->reg(ra_,this,idx1)/* dst */);
  
#line 20766 "ad_x86.cpp"
  }
  {

#line 1646 "src/hotspot/cpu/x86/x86_32.ad"

    // Check for 8-bit immediate, and set sign extend bit in opcode
    if ((opnd_array(2)->constant() >= -128) && (opnd_array(2)->constant() <= 127)) {
      emit_d8(cbuf, opnd_array(2)->constant());
    }
    else {                          // If 32-bit immediate
      // Output immediate
      emit_d32(cbuf, opnd_array(2)->constant());
    }
  
#line 20781 "ad_x86.cpp"
  }
}

void subI_eReg_memNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {

#line 1544 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0x2B /*primary()*/));
  
#line 20797 "ad_x86.cpp"
  }
  {

#line 2096 "src/hotspot/cpu/x86/x86_32.ad"

    int reg_encoding = opnd_array(1)->reg(ra_,this,idx1)/* dst */;
    int base  = opnd_array(2)->base(ra_,this,idx2);
    int index = opnd_array(2)->index(ra_,this,idx2);
    int scale = opnd_array(2)->scale();
    int displace = opnd_array(2)->disp(ra_,this,idx2);
    relocInfo::relocType disp_reloc = opnd_array(2)->disp_reloc();
    encode_RegMem(cbuf, reg_encoding, base, index, scale, displace, disp_reloc);
  
#line 20811 "ad_x86.cpp"
  }
}

void subI_mem_eRegNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {

#line 1544 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0x29 /*primary()*/));
  
#line 20827 "ad_x86.cpp"
  }
  {

#line 2096 "src/hotspot/cpu/x86/x86_32.ad"

    int reg_encoding = opnd_array(2)->reg(ra_,this,idx2)/* src */;
    int base  = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int displace = opnd_array(1)->disp(ra_,this,idx1);
    relocInfo::relocType disp_reloc = opnd_array(1)->disp_reloc();
    encode_RegMem(cbuf, reg_encoding, base, index, scale, displace, disp_reloc);
  
#line 20841 "ad_x86.cpp"
  }
}

void subP_eRegNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// zero
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// src
  {

#line 1544 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0x2B /*primary()*/));
  
#line 20858 "ad_x86.cpp"
  }
  {

#line 1959 "src/hotspot/cpu/x86/x86_32.ad"

    emit_rm(cbuf, 0x3, opnd_array(1)->reg(ra_,this,idx1)/* dst */, opnd_array(3)->reg(ra_,this,idx3)/* src */);
  
#line 20866 "ad_x86.cpp"
  }
}

uint subP_eRegNode::size(PhaseRegAlloc *ra_) const {
  assert(VerifyOops || MachNode::size(ra_) <= 2, "bad fixed size");
  return (VerifyOops ? MachNode::size(ra_) : 2);
}

void negI_eRegNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// zero
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// dst
  {

#line 1544 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0xF7 /*primary()*/));
  
#line 20887 "ad_x86.cpp"
  }
  {

#line 1697 "src/hotspot/cpu/x86/x86_32.ad"

    emit_rm(cbuf, 0x3, (0x03 /*secondary()*/), opnd_array(2)->reg(ra_,this,idx2)/* dst */ );
  
#line 20895 "ad_x86.cpp"
  }
}

uint negI_eRegNode::size(PhaseRegAlloc *ra_) const {
  assert(VerifyOops || MachNode::size(ra_) <= 2, "bad fixed size");
  return (VerifyOops ? MachNode::size(ra_) : 2);
}

void mulI_eRegNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {

#line 1549 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0x0F /*secondary()*/));
  
#line 20916 "ad_x86.cpp"
  }
  {

#line 1544 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0xAF /*primary()*/));
  
#line 20924 "ad_x86.cpp"
  }
  {

#line 1959 "src/hotspot/cpu/x86/x86_32.ad"

    emit_rm(cbuf, 0x3, opnd_array(1)->reg(ra_,this,idx1)/* dst */, opnd_array(2)->reg(ra_,this,idx2)/* src */);
  
#line 20932 "ad_x86.cpp"
  }
}

uint mulI_eRegNode::size(PhaseRegAlloc *ra_) const {
  assert(VerifyOops || MachNode::size(ra_) <= 3, "bad fixed size");
  return (VerifyOops ? MachNode::size(ra_) : 3);
}

void mulI_eReg_immNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// imm
  {

#line 1624 "src/hotspot/cpu/x86/x86_32.ad"

    // Check for 8-bit immediate, and set sign extend bit in opcode
    if ((opnd_array(2)->constant() >= -128) && (opnd_array(2)->constant() <= 127)) {
      emit_opcode(cbuf, (0x69 /*primary()*/) | 0x02);
    }
    else {                          // If 32-bit immediate
      emit_opcode(cbuf, (0x69 /*primary()*/));
    }
  
#line 20959 "ad_x86.cpp"
  }
  {

#line 1959 "src/hotspot/cpu/x86/x86_32.ad"

    emit_rm(cbuf, 0x3, opnd_array(0)->reg(ra_,this)/* dst */, opnd_array(1)->reg(ra_,this,idx1)/* src */);
  
#line 20967 "ad_x86.cpp"
  }
  {

#line 1646 "src/hotspot/cpu/x86/x86_32.ad"

    // Check for 8-bit immediate, and set sign extend bit in opcode
    if ((opnd_array(2)->constant() >= -128) && (opnd_array(2)->constant() <= 127)) {
      emit_d8(cbuf, opnd_array(2)->constant());
    }
    else {                          // If 32-bit immediate
      // Output immediate
      emit_d32(cbuf, opnd_array(2)->constant());
    }
  
#line 20982 "ad_x86.cpp"
  }
}

void loadConL_low_onlyNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {

#line 1919 "src/hotspot/cpu/x86/x86_32.ad"

    // Load immediate does not have a zero or sign extended version
    // for 8-bit immediates
    int dst_enc = opnd_array(0)->reg(ra_,this)/* dst */;
    int src_con = opnd_array(1)->constantL() & 0x0FFFFFFFFL;
    if (src_con == 0) {
      // xor dst, dst
      emit_opcode(cbuf, 0x33);
      emit_rm(cbuf, 0x3, dst_enc, dst_enc);
    } else {
      emit_opcode(cbuf, (0xB8 /*primary()*/) + dst_enc);
      emit_d32(cbuf, src_con);
    }
  
#line 21008 "ad_x86.cpp"
  }
}

void mulI_imm_highNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// cnt
  {

#line 2718 "src/hotspot/cpu/x86/x86_32.ad"

    // IMUL   EDX:EAX,$src1
    emit_opcode( cbuf, 0xF7 );
    emit_rm( cbuf, 0x3, 0x5, opnd_array(1)->reg(ra_,this,idx1)/* src1 */ );
    // SAR    EDX,$cnt-32
    int shift_count = ((int)opnd_array(3)->constant()) - 32;
    if (shift_count > 0) {
      emit_opcode(cbuf, 0xC1);
      emit_rm(cbuf, 0x3, 7, opnd_array(0)->reg(ra_,this)/* dst */ );
      emit_d8(cbuf, shift_count);
    }
  
#line 21034 "ad_x86.cpp"
  }
}

void mulI_imm_high_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src2
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src1
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// cnt
  {

#line 2718 "src/hotspot/cpu/x86/x86_32.ad"

    // IMUL   EDX:EAX,$src1
    emit_opcode( cbuf, 0xF7 );
    emit_rm( cbuf, 0x3, 0x5, opnd_array(2)->reg(ra_,this,idx2)/* src1 */ );
    // SAR    EDX,$cnt-32
    int shift_count = ((int)opnd_array(3)->constant()) - 32;
    if (shift_count > 0) {
      emit_opcode(cbuf, 0xC1);
      emit_rm(cbuf, 0x3, 7, opnd_array(0)->reg(ra_,this)/* dst */ );
      emit_d8(cbuf, shift_count);
    }
  
#line 21060 "ad_x86.cpp"
  }
}

void mulI_imm_RShift_highNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// cnt
  {

#line 2718 "src/hotspot/cpu/x86/x86_32.ad"

    // IMUL   EDX:EAX,$src1
    emit_opcode( cbuf, 0xF7 );
    emit_rm( cbuf, 0x3, 0x5, opnd_array(1)->reg(ra_,this,idx1)/* src1 */ );
    // SAR    EDX,$cnt-32
    int shift_count = ((int)opnd_array(3)->constant()) - 32;
    if (shift_count > 0) {
      emit_opcode(cbuf, 0xC1);
      emit_rm(cbuf, 0x3, 7, opnd_array(0)->reg(ra_,this)/* dst */ );
      emit_d8(cbuf, shift_count);
    }
  
#line 21086 "ad_x86.cpp"
  }
}

void mulI_imm_RShift_high_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src2
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src1
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// cnt
  {

#line 2718 "src/hotspot/cpu/x86/x86_32.ad"

    // IMUL   EDX:EAX,$src1
    emit_opcode( cbuf, 0xF7 );
    emit_rm( cbuf, 0x3, 0x5, opnd_array(2)->reg(ra_,this,idx2)/* src1 */ );
    // SAR    EDX,$cnt-32
    int shift_count = ((int)opnd_array(3)->constant()) - 32;
    if (shift_count > 0) {
      emit_opcode(cbuf, 0xC1);
      emit_rm(cbuf, 0x3, 7, opnd_array(0)->reg(ra_,this)/* dst */ );
      emit_d8(cbuf, shift_count);
    }
  
#line 21112 "ad_x86.cpp"
  }
}

void mulI_mem_immNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// imm
  {

#line 1624 "src/hotspot/cpu/x86/x86_32.ad"

    // Check for 8-bit immediate, and set sign extend bit in opcode
    if ((opnd_array(2)->constant() >= -128) && (opnd_array(2)->constant() <= 127)) {
      emit_opcode(cbuf, (0x69 /*primary()*/) | 0x02);
    }
    else {                          // If 32-bit immediate
      emit_opcode(cbuf, (0x69 /*primary()*/));
    }
  
#line 21134 "ad_x86.cpp"
  }
  {

#line 2096 "src/hotspot/cpu/x86/x86_32.ad"

    int reg_encoding = opnd_array(0)->reg(ra_,this)/* dst */;
    int base  = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int displace = opnd_array(1)->disp(ra_,this,idx1);
    relocInfo::relocType disp_reloc = opnd_array(1)->disp_reloc();
    encode_RegMem(cbuf, reg_encoding, base, index, scale, displace, disp_reloc);
  
#line 21148 "ad_x86.cpp"
  }
  {

#line 1646 "src/hotspot/cpu/x86/x86_32.ad"

    // Check for 8-bit immediate, and set sign extend bit in opcode
    if ((opnd_array(2)->constant() >= -128) && (opnd_array(2)->constant() <= 127)) {
      emit_d8(cbuf, opnd_array(2)->constant());
    }
    else {                          // If 32-bit immediate
      // Output immediate
      emit_d32(cbuf, opnd_array(2)->constant());
    }
  
#line 21163 "ad_x86.cpp"
  }
}

void mulINode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {

#line 1549 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0x0F /*secondary()*/));
  
#line 21179 "ad_x86.cpp"
  }
  {

#line 1544 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0xAF /*primary()*/));
  
#line 21187 "ad_x86.cpp"
  }
  {

#line 2096 "src/hotspot/cpu/x86/x86_32.ad"

    int reg_encoding = opnd_array(1)->reg(ra_,this,idx1)/* dst */;
    int base  = opnd_array(2)->base(ra_,this,idx2);
    int index = opnd_array(2)->index(ra_,this,idx2);
    int scale = opnd_array(2)->scale();
    int displace = opnd_array(2)->disp(ra_,this,idx2);
    relocInfo::relocType disp_reloc = opnd_array(2)->disp_reloc();
    encode_RegMem(cbuf, reg_encoding, base, index, scale, displace, disp_reloc);
  
#line 21201 "ad_x86.cpp"
  }
}

void mulI_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// dst
  {

#line 1549 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0x0F /*secondary()*/));
  
#line 21217 "ad_x86.cpp"
  }
  {

#line 1544 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0xAF /*primary()*/));
  
#line 21225 "ad_x86.cpp"
  }
  {

#line 2096 "src/hotspot/cpu/x86/x86_32.ad"

    int reg_encoding = opnd_array(2)->reg(ra_,this,idx2)/* dst */;
    int base  = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int displace = opnd_array(1)->disp(ra_,this,idx1);
    relocInfo::relocType disp_reloc = opnd_array(1)->disp_reloc();
    encode_RegMem(cbuf, reg_encoding, base, index, scale, displace, disp_reloc);
  
#line 21239 "ad_x86.cpp"
  }
}

void mulI2LNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src1
  {

#line 2744 "src/hotspot/cpu/x86/x86_32.ad"

    // Basic idea: long = (long)int * (long)int
    // IMUL EDX:EAX, src
    emit_opcode( cbuf, 0xF7 );
    emit_rm( cbuf, 0x3, 0x5, opnd_array(2)->reg(ra_,this,idx2)/* src1 */);
  
#line 21258 "ad_x86.cpp"
  }
}

void mulI2L_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {

#line 2744 "src/hotspot/cpu/x86/x86_32.ad"

    // Basic idea: long = (long)int * (long)int
    // IMUL EDX:EAX, src
    emit_opcode( cbuf, 0xF7 );
    emit_rm( cbuf, 0x3, 0x5, opnd_array(1)->reg(ra_,this,idx1)/* src1 */);
  
#line 21277 "ad_x86.cpp"
  }
}

void mulIS_eRegNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// mask
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// src1
  {

#line 2751 "src/hotspot/cpu/x86/x86_32.ad"

    // Basic Idea:  long = (int & 0xffffffffL) * (int & 0xffffffffL)
    // MUL EDX:EAX, src
    emit_opcode( cbuf, 0xF7 );
    emit_rm( cbuf, 0x3, 0x4, opnd_array(3)->reg(ra_,this,idx3)/* src1 */);
  
#line 21297 "ad_x86.cpp"
  }
}

void mulIS_eReg_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// mask
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// src
  {

#line 2751 "src/hotspot/cpu/x86/x86_32.ad"

    // Basic Idea:  long = (int & 0xffffffffL) * (int & 0xffffffffL)
    // MUL EDX:EAX, src
    emit_opcode( cbuf, 0xF7 );
    emit_rm( cbuf, 0x3, 0x4, opnd_array(1)->reg(ra_,this,idx1)/* src1 */);
  
#line 21317 "ad_x86.cpp"
  }
}

void mulL_eRegNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// tmp
  {

#line 2758 "src/hotspot/cpu/x86/x86_32.ad"

    // Basic idea: lo(result) = lo(x_lo * y_lo)
    //             hi(result) = hi(x_lo * y_lo) + lo(x_hi * y_lo) + lo(x_lo * y_hi)
    // MOV    $tmp,$src.lo
    encode_Copy( cbuf, opnd_array(3)->reg(ra_,this,idx3)/* tmp */, opnd_array(2)->reg(ra_,this,idx2)/* src */ );
    // IMUL   $tmp,EDX
    emit_opcode( cbuf, 0x0F );
    emit_opcode( cbuf, 0xAF );
    emit_rm( cbuf, 0x3, opnd_array(3)->reg(ra_,this,idx3)/* tmp */, HIGH_FROM_LOW_ENC(opnd_array(1)->reg(ra_,this,idx1)/* dst */) );
    // MOV    EDX,$src.hi
    encode_Copy( cbuf, HIGH_FROM_LOW_ENC(opnd_array(1)->reg(ra_,this,idx1)/* dst */), HIGH_FROM_LOW_ENC(opnd_array(2)->reg(ra_,this,idx2)/* src */) );
    // IMUL   EDX,EAX
    emit_opcode( cbuf, 0x0F );
    emit_opcode( cbuf, 0xAF );
    emit_rm( cbuf, 0x3, HIGH_FROM_LOW_ENC(opnd_array(1)->reg(ra_,this,idx1)/* dst */), opnd_array(1)->reg(ra_,this,idx1)/* dst */ );
    // ADD    $tmp,EDX
    emit_opcode( cbuf, 0x03 );
    emit_rm( cbuf, 0x3, opnd_array(3)->reg(ra_,this,idx3)/* tmp */, HIGH_FROM_LOW_ENC(opnd_array(1)->reg(ra_,this,idx1)/* dst */) );
    // MUL   EDX:EAX,$src.lo
    emit_opcode( cbuf, 0xF7 );
    emit_rm( cbuf, 0x3, 0x4, opnd_array(2)->reg(ra_,this,idx2)/* src */ );
    // ADD    EDX,ESI
    emit_opcode( cbuf, 0x03 );
    emit_rm( cbuf, 0x3, HIGH_FROM_LOW_ENC(opnd_array(1)->reg(ra_,this,idx1)/* dst */), opnd_array(3)->reg(ra_,this,idx3)/* tmp */ );
  
#line 21356 "ad_x86.cpp"
  }
}

void mulL_eReg_lhi0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// tmp
  {
    C2_MacroAssembler _masm(&cbuf);

#line 7730 "src/hotspot/cpu/x86/x86_32.ad"

    __ movl(opnd_array(3)->as_Register(ra_,this,idx3)/* tmp */, HIGH_FROM_LOW(opnd_array(2)->as_Register(ra_,this,idx2)/* src */));
    __ imull(opnd_array(3)->as_Register(ra_,this,idx3)/* tmp */, rax);
    __ mull(opnd_array(2)->as_Register(ra_,this,idx2)/* src */);
    __ addl(rdx, opnd_array(3)->as_Register(ra_,this,idx3)/* tmp */);
  
#line 21377 "ad_x86.cpp"
  }
}

void mulL_eReg_rhi0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// tmp
  {
    C2_MacroAssembler _masm(&cbuf);

#line 7751 "src/hotspot/cpu/x86/x86_32.ad"

    __ movl(opnd_array(3)->as_Register(ra_,this,idx3)/* tmp */, opnd_array(2)->as_Register(ra_,this,idx2)/* src */);
    __ imull(opnd_array(3)->as_Register(ra_,this,idx3)/* tmp */, rdx);
    __ mull(opnd_array(2)->as_Register(ra_,this,idx2)/* src */);
    __ addl(rdx, opnd_array(3)->as_Register(ra_,this,idx3)/* tmp */);
  
#line 21398 "ad_x86.cpp"
  }
}

void mulL_eReg_hi0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 7769 "src/hotspot/cpu/x86/x86_32.ad"

    __ mull(opnd_array(2)->as_Register(ra_,this,idx2)/* src */);
  
#line 21415 "ad_x86.cpp"
  }
}

void mulL_eReg_conNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// tmp
  {

#line 2784 "src/hotspot/cpu/x86/x86_32.ad"

    // Basic idea: lo(result) = lo(src * y_lo)
    //             hi(result) = hi(src * y_lo) + lo(src * y_hi)
    // IMUL   $tmp,EDX,$src
    emit_opcode( cbuf, 0x6B );
    emit_rm( cbuf, 0x3, opnd_array(3)->reg(ra_,this,idx3)/* tmp */, HIGH_FROM_LOW_ENC(opnd_array(1)->reg(ra_,this,idx1)/* dst */) );
    emit_d8( cbuf, (int)opnd_array(2)->constantL() );
    // MOV    EDX,$src
    emit_opcode(cbuf, 0xB8 + EDX_enc);
    emit_d32( cbuf, (int)opnd_array(2)->constantL() );
    // MUL   EDX:EAX,EDX
    emit_opcode( cbuf, 0xF7 );
    emit_rm( cbuf, 0x3, 0x4, EDX_enc );
    // ADD    EDX,ESI
    emit_opcode( cbuf, 0x03 );
    emit_rm( cbuf, 0x3, EDX_enc, opnd_array(3)->reg(ra_,this,idx3)/* tmp */ );
  
#line 21446 "ad_x86.cpp"
  }
}

uint mulL_eReg_conNode::size(PhaseRegAlloc *ra_) const {
  assert(VerifyOops || MachNode::size(ra_) <= 12, "bad fixed size");
  return (VerifyOops ? MachNode::size(ra_) : 12);
}

void divI_eRegNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// rax
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// div
  {

#line 1576 "src/hotspot/cpu/x86/x86_32.ad"

    // Full implementation of Java idiv and irem; checks for
    // special case as described in JVM spec., p.243 & p.271.
    //
    //         normal case                           special case
    //
    // input : rax,: dividend                         min_int
    //         reg: divisor                          -1
    //
    // output: rax,: quotient  (= rax, idiv reg)       min_int
    //         rdx: remainder (= rax, irem reg)       0
    //
    //  Code sequnce:
    //
    //  81 F8 00 00 00 80    cmp         rax,80000000h
    //  0F 85 0B 00 00 00    jne         normal_case
    //  33 D2                xor         rdx,edx
    //  83 F9 FF             cmp         rcx,0FFh
    //  0F 84 03 00 00 00    je          done
    //                  normal_case:
    //  99                   cdq
    //  F7 F9                idiv        rax,ecx
    //                  done:
    //
    emit_opcode(cbuf,0x81); emit_d8(cbuf,0xF8);
    emit_opcode(cbuf,0x00); emit_d8(cbuf,0x00);
    emit_opcode(cbuf,0x00); emit_d8(cbuf,0x80);                     // cmp rax,80000000h
    emit_opcode(cbuf,0x0F); emit_d8(cbuf,0x85);
    emit_opcode(cbuf,0x0B); emit_d8(cbuf,0x00);
    emit_opcode(cbuf,0x00); emit_d8(cbuf,0x00);                     // jne normal_case
    emit_opcode(cbuf,0x33); emit_d8(cbuf,0xD2);                     // xor rdx,edx
    emit_opcode(cbuf,0x83); emit_d8(cbuf,0xF9); emit_d8(cbuf,0xFF); // cmp rcx,0FFh
    emit_opcode(cbuf,0x0F); emit_d8(cbuf,0x84);
    emit_opcode(cbuf,0x03); emit_d8(cbuf,0x00);
    emit_opcode(cbuf,0x00); emit_d8(cbuf,0x00);                     // je done
    // normal_case:
    emit_opcode(cbuf,0x99);                                         // cdq
    // idiv (note: must be emitted by the user of this rule)
    // normal:
  
#line 21504 "ad_x86.cpp"
  }
  {

#line 1544 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0xF7 /*primary()*/));
  
#line 21512 "ad_x86.cpp"
  }
  {

#line 1697 "src/hotspot/cpu/x86/x86_32.ad"

    emit_rm(cbuf, 0x3, (0x7 /*secondary()*/), opnd_array(2)->reg(ra_,this,idx2)/* div */ );
  
#line 21520 "ad_x86.cpp"
  }
}

uint divI_eRegNode::size(PhaseRegAlloc *ra_) const {
  assert(VerifyOops || MachNode::size(ra_) <= 26, "bad fixed size");
  return (VerifyOops ? MachNode::size(ra_) : 26);
}

void divL_eRegNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {

#line 2802 "src/hotspot/cpu/x86/x86_32.ad"

    // PUSH src1.hi
    emit_opcode(cbuf, HIGH_FROM_LOW_ENC(0x50+opnd_array(1)->reg(ra_,this,idx1)/* src1 */) );
    // PUSH src1.lo
    emit_opcode(cbuf,               0x50+opnd_array(1)->reg(ra_,this,idx1)/* src1 */  );
    // PUSH src2.hi
    emit_opcode(cbuf, HIGH_FROM_LOW_ENC(0x50+opnd_array(2)->reg(ra_,this,idx2)/* src2 */) );
    // PUSH src2.lo
    emit_opcode(cbuf,               0x50+opnd_array(2)->reg(ra_,this,idx2)/* src2 */  );
    // CALL directly to the runtime
    MacroAssembler _masm(&cbuf);
    cbuf.set_insts_mark();
    emit_opcode(cbuf,0xE8);       // Call into runtime
    emit_d32_reloc(cbuf, (CAST_FROM_FN_PTR(address, SharedRuntime::ldiv) - cbuf.insts_end()) - 4, runtime_call_Relocation::spec(), RELOC_IMM32 );
    __ post_call_nop();
    // Restore stack
    emit_opcode(cbuf, 0x83); // add  SP, #framesize
    emit_rm(cbuf, 0x3, 0x00, ESP_enc);
    emit_d8(cbuf, 4*4);
  
#line 21558 "ad_x86.cpp"
  }
}

void divModI_eReg_divmodNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// div
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// rdx
  {

#line 1576 "src/hotspot/cpu/x86/x86_32.ad"

    // Full implementation of Java idiv and irem; checks for
    // special case as described in JVM spec., p.243 & p.271.
    //
    //         normal case                           special case
    //
    // input : rax,: dividend                         min_int
    //         reg: divisor                          -1
    //
    // output: rax,: quotient  (= rax, idiv reg)       min_int
    //         rdx: remainder (= rax, irem reg)       0
    //
    //  Code sequnce:
    //
    //  81 F8 00 00 00 80    cmp         rax,80000000h
    //  0F 85 0B 00 00 00    jne         normal_case
    //  33 D2                xor         rdx,edx
    //  83 F9 FF             cmp         rcx,0FFh
    //  0F 84 03 00 00 00    je          done
    //                  normal_case:
    //  99                   cdq
    //  F7 F9                idiv        rax,ecx
    //                  done:
    //
    emit_opcode(cbuf,0x81); emit_d8(cbuf,0xF8);
    emit_opcode(cbuf,0x00); emit_d8(cbuf,0x00);
    emit_opcode(cbuf,0x00); emit_d8(cbuf,0x80);                     // cmp rax,80000000h
    emit_opcode(cbuf,0x0F); emit_d8(cbuf,0x85);
    emit_opcode(cbuf,0x0B); emit_d8(cbuf,0x00);
    emit_opcode(cbuf,0x00); emit_d8(cbuf,0x00);                     // jne normal_case
    emit_opcode(cbuf,0x33); emit_d8(cbuf,0xD2);                     // xor rdx,edx
    emit_opcode(cbuf,0x83); emit_d8(cbuf,0xF9); emit_d8(cbuf,0xFF); // cmp rcx,0FFh
    emit_opcode(cbuf,0x0F); emit_d8(cbuf,0x84);
    emit_opcode(cbuf,0x03); emit_d8(cbuf,0x00);
    emit_opcode(cbuf,0x00); emit_d8(cbuf,0x00);                     // je done
    // normal_case:
    emit_opcode(cbuf,0x99);                                         // cdq
    // idiv (note: must be emitted by the user of this rule)
    // normal:
  
#line 21611 "ad_x86.cpp"
  }
  {

#line 1544 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0xF7 /*primary()*/));
  
#line 21619 "ad_x86.cpp"
  }
  {

#line 1697 "src/hotspot/cpu/x86/x86_32.ad"

    emit_rm(cbuf, 0x3, (0x7 /*secondary()*/), opnd_array(2)->reg(ra_,this,idx2)/* div */ );
  
#line 21627 "ad_x86.cpp"
  }
}

uint divModI_eReg_divmodNode::size(PhaseRegAlloc *ra_) const {
  assert(VerifyOops || MachNode::size(ra_) <= 26, "bad fixed size");
  return (VerifyOops ? MachNode::size(ra_) : 26);
}

void modI_eRegNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// rax
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// div
  {

#line 1576 "src/hotspot/cpu/x86/x86_32.ad"

    // Full implementation of Java idiv and irem; checks for
    // special case as described in JVM spec., p.243 & p.271.
    //
    //         normal case                           special case
    //
    // input : rax,: dividend                         min_int
    //         reg: divisor                          -1
    //
    // output: rax,: quotient  (= rax, idiv reg)       min_int
    //         rdx: remainder (= rax, irem reg)       0
    //
    //  Code sequnce:
    //
    //  81 F8 00 00 00 80    cmp         rax,80000000h
    //  0F 85 0B 00 00 00    jne         normal_case
    //  33 D2                xor         rdx,edx
    //  83 F9 FF             cmp         rcx,0FFh
    //  0F 84 03 00 00 00    je          done
    //                  normal_case:
    //  99                   cdq
    //  F7 F9                idiv        rax,ecx
    //                  done:
    //
    emit_opcode(cbuf,0x81); emit_d8(cbuf,0xF8);
    emit_opcode(cbuf,0x00); emit_d8(cbuf,0x00);
    emit_opcode(cbuf,0x00); emit_d8(cbuf,0x80);                     // cmp rax,80000000h
    emit_opcode(cbuf,0x0F); emit_d8(cbuf,0x85);
    emit_opcode(cbuf,0x0B); emit_d8(cbuf,0x00);
    emit_opcode(cbuf,0x00); emit_d8(cbuf,0x00);                     // jne normal_case
    emit_opcode(cbuf,0x33); emit_d8(cbuf,0xD2);                     // xor rdx,edx
    emit_opcode(cbuf,0x83); emit_d8(cbuf,0xF9); emit_d8(cbuf,0xFF); // cmp rcx,0FFh
    emit_opcode(cbuf,0x0F); emit_d8(cbuf,0x84);
    emit_opcode(cbuf,0x03); emit_d8(cbuf,0x00);
    emit_opcode(cbuf,0x00); emit_d8(cbuf,0x00);                     // je done
    // normal_case:
    emit_opcode(cbuf,0x99);                                         // cdq
    // idiv (note: must be emitted by the user of this rule)
    // normal:
  
#line 21685 "ad_x86.cpp"
  }
  {

#line 1544 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0xF7 /*primary()*/));
  
#line 21693 "ad_x86.cpp"
  }
  {

#line 1697 "src/hotspot/cpu/x86/x86_32.ad"

    emit_rm(cbuf, 0x3, (0x7 /*secondary()*/), opnd_array(2)->reg(ra_,this,idx2)/* div */ );
  
#line 21701 "ad_x86.cpp"
  }
}

uint modI_eRegNode::size(PhaseRegAlloc *ra_) const {
  assert(VerifyOops || MachNode::size(ra_) <= 26, "bad fixed size");
  return (VerifyOops ? MachNode::size(ra_) : 26);
}

void modL_eRegNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {

#line 2823 "src/hotspot/cpu/x86/x86_32.ad"

    // PUSH src1.hi
    emit_opcode(cbuf, HIGH_FROM_LOW_ENC(0x50+opnd_array(1)->reg(ra_,this,idx1)/* src1 */) );
    // PUSH src1.lo
    emit_opcode(cbuf,               0x50+opnd_array(1)->reg(ra_,this,idx1)/* src1 */  );
    // PUSH src2.hi
    emit_opcode(cbuf, HIGH_FROM_LOW_ENC(0x50+opnd_array(2)->reg(ra_,this,idx2)/* src2 */) );
    // PUSH src2.lo
    emit_opcode(cbuf,               0x50+opnd_array(2)->reg(ra_,this,idx2)/* src2 */  );
    // CALL directly to the runtime
    MacroAssembler _masm(&cbuf);
    cbuf.set_insts_mark();
    emit_opcode(cbuf,0xE8);       // Call into runtime
    emit_d32_reloc(cbuf, (CAST_FROM_FN_PTR(address, SharedRuntime::lrem ) - cbuf.insts_end()) - 4, runtime_call_Relocation::spec(), RELOC_IMM32 );
    __ post_call_nop();
    // Restore stack
    emit_opcode(cbuf, 0x83); // add  SP, #framesize
    emit_rm(cbuf, 0x3, 0x00, ESP_enc);
    emit_d8(cbuf, 4*4);
  
#line 21739 "ad_x86.cpp"
  }
}

void divL_eReg_imm32Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// imm
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// tmp
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// tmp2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 7900 "src/hotspot/cpu/x86/x86_32.ad"

    int con = (int)opnd_array(2)->constantL();
    assert(con != 0 && con != -1 && con != min_jint, "wrong divisor");
    int pcon = (con > 0) ? con : -con;
    Label Lfast, Lpos, Ldone;

    __ movl(opnd_array(3)->as_Register(ra_,this,idx3)/* tmp */, pcon);
    __ xorl(opnd_array(4)->as_Register(ra_,this,idx4)/* tmp2 */,opnd_array(4)->as_Register(ra_,this,idx4)/* tmp2 */);
    __ cmpl(opnd_array(3)->as_Register(ra_,this,idx3)/* tmp */, HIGH_FROM_LOW(opnd_array(1)->as_Register(ra_,this,idx1)/* dst */));
    __ jccb(Assembler::above, Lfast); // result fits into 32 bit

    __ movl(opnd_array(4)->as_Register(ra_,this,idx4)/* tmp2 */, opnd_array(1)->as_Register(ra_,this,idx1)/* dst */); // save
    __ movl(opnd_array(1)->as_Register(ra_,this,idx1)/* dst */, HIGH_FROM_LOW(opnd_array(1)->as_Register(ra_,this,idx1)/* dst */));
    __ movl(HIGH_FROM_LOW(opnd_array(1)->as_Register(ra_,this,idx1)/* dst */),0); // preserve flags
    __ jccb(Assembler::lessEqual, Lpos); // result is positive

    // Negative dividend.
    // convert value to positive to use unsigned division
    __ lneg(opnd_array(1)->as_Register(ra_,this,idx1)/* dst */, opnd_array(4)->as_Register(ra_,this,idx4)/* tmp2 */);
    __ divl(opnd_array(3)->as_Register(ra_,this,idx3)/* tmp */);
    __ xchgl(opnd_array(1)->as_Register(ra_,this,idx1)/* dst */, opnd_array(4)->as_Register(ra_,this,idx4)/* tmp2 */);
    __ divl(opnd_array(3)->as_Register(ra_,this,idx3)/* tmp */);
    // revert result back to negative
    __ lneg(opnd_array(4)->as_Register(ra_,this,idx4)/* tmp2 */, opnd_array(1)->as_Register(ra_,this,idx1)/* dst */);
    __ jmpb(Ldone);

    __ bind(Lpos);
    __ divl(opnd_array(3)->as_Register(ra_,this,idx3)/* tmp */); // Use unsigned division
    __ xchgl(opnd_array(1)->as_Register(ra_,this,idx1)/* dst */, opnd_array(4)->as_Register(ra_,this,idx4)/* tmp2 */);
    // Fallthrow for final divide, tmp2 has 32 bit hi result

    __ bind(Lfast);
    // fast path: src is positive
    __ divl(opnd_array(3)->as_Register(ra_,this,idx3)/* tmp */); // Use unsigned division

    __ bind(Ldone);
    __ movl(HIGH_FROM_LOW(opnd_array(1)->as_Register(ra_,this,idx1)/* dst */),opnd_array(4)->as_Register(ra_,this,idx4)/* tmp2 */);
    if (con < 0) {
      __ lneg(HIGH_FROM_LOW(opnd_array(1)->as_Register(ra_,this,idx1)/* dst */), opnd_array(1)->as_Register(ra_,this,idx1)/* dst */);
    }
  
#line 21796 "ad_x86.cpp"
  }
}

void modL_eReg_imm32Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// imm
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// tmp
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// tmp2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 7970 "src/hotspot/cpu/x86/x86_32.ad"

    int con = (int)opnd_array(2)->constantL();
    assert(con != 0 && con != -1 && con != min_jint, "wrong divisor");
    int pcon = (con > 0) ? con : -con;
    Label  Lfast, Lpos, Ldone;

    __ movl(opnd_array(3)->as_Register(ra_,this,idx3)/* tmp */, pcon);
    __ cmpl(opnd_array(3)->as_Register(ra_,this,idx3)/* tmp */, HIGH_FROM_LOW(opnd_array(1)->as_Register(ra_,this,idx1)/* dst */));
    __ jccb(Assembler::above, Lfast); // src is positive and result fits into 32 bit

    __ movl(opnd_array(4)->as_Register(ra_,this,idx4)/* tmp2 */, opnd_array(1)->as_Register(ra_,this,idx1)/* dst */); // save
    __ movl(opnd_array(1)->as_Register(ra_,this,idx1)/* dst */, HIGH_FROM_LOW(opnd_array(1)->as_Register(ra_,this,idx1)/* dst */));
    __ movl(HIGH_FROM_LOW(opnd_array(1)->as_Register(ra_,this,idx1)/* dst */),0); // preserve flags
    __ jccb(Assembler::lessEqual, Lpos); // result is positive

    // Negative dividend.
    // convert value to positive to use unsigned division
    __ lneg(opnd_array(1)->as_Register(ra_,this,idx1)/* dst */, opnd_array(4)->as_Register(ra_,this,idx4)/* tmp2 */);
    __ divl(opnd_array(3)->as_Register(ra_,this,idx3)/* tmp */);
    __ movl(opnd_array(1)->as_Register(ra_,this,idx1)/* dst */, opnd_array(4)->as_Register(ra_,this,idx4)/* tmp2 */);
    __ divl(opnd_array(3)->as_Register(ra_,this,idx3)/* tmp */);
    // revert remainder back to negative
    __ negl(HIGH_FROM_LOW(opnd_array(1)->as_Register(ra_,this,idx1)/* dst */));
    __ jmpb(Ldone);

    __ bind(Lpos);
    __ divl(opnd_array(3)->as_Register(ra_,this,idx3)/* tmp */);
    __ movl(opnd_array(1)->as_Register(ra_,this,idx1)/* dst */, opnd_array(4)->as_Register(ra_,this,idx4)/* tmp2 */);

    __ bind(Lfast);
    // fast path: src is positive
    __ divl(opnd_array(3)->as_Register(ra_,this,idx3)/* tmp */);

    __ bind(Ldone);
    __ movl(opnd_array(1)->as_Register(ra_,this,idx1)/* dst */, HIGH_FROM_LOW(opnd_array(1)->as_Register(ra_,this,idx1)/* dst */));
    __ sarl(HIGH_FROM_LOW(opnd_array(1)->as_Register(ra_,this,idx1)/* dst */), 31); // result sign

  
#line 21850 "ad_x86.cpp"
  }
}

void shlI_eReg_1Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// shift
  {

#line 1544 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0xD1 /*primary()*/));
  
#line 21866 "ad_x86.cpp"
  }
  {

#line 1697 "src/hotspot/cpu/x86/x86_32.ad"

    emit_rm(cbuf, 0x3, (0x4 /*secondary()*/), opnd_array(1)->reg(ra_,this,idx1)/* dst */ );
  
#line 21874 "ad_x86.cpp"
  }
}

uint shlI_eReg_1Node::size(PhaseRegAlloc *ra_) const {
  assert(VerifyOops || MachNode::size(ra_) <= 2, "bad fixed size");
  return (VerifyOops ? MachNode::size(ra_) : 2);
}

void salI_eReg_immNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// shift
  {

#line 1899 "src/hotspot/cpu/x86/x86_32.ad"

    emit_d8(cbuf, (0xC1 /*primary()*/));
    emit_rm(cbuf, 0x3, (0x4 /*secondary()*/), opnd_array(1)->reg(ra_,this,idx1)/* dst */);
    emit_d8(cbuf, opnd_array(2)->constant());
  
#line 21897 "ad_x86.cpp"
  }
}

uint salI_eReg_immNode::size(PhaseRegAlloc *ra_) const {
  assert(VerifyOops || MachNode::size(ra_) <= 3, "bad fixed size");
  return (VerifyOops ? MachNode::size(ra_) : 3);
}

void salI_eReg_CLNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// shift
  {

#line 1544 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0xD3 /*primary()*/));
  
#line 21918 "ad_x86.cpp"
  }
  {

#line 1697 "src/hotspot/cpu/x86/x86_32.ad"

    emit_rm(cbuf, 0x3, (0x4 /*secondary()*/), opnd_array(1)->reg(ra_,this,idx1)/* dst */ );
  
#line 21926 "ad_x86.cpp"
  }
}

uint salI_eReg_CLNode::size(PhaseRegAlloc *ra_) const {
  assert(VerifyOops || MachNode::size(ra_) <= 2, "bad fixed size");
  return (VerifyOops ? MachNode::size(ra_) : 2);
}

void sarI_eReg_1Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// shift
  {

#line 1544 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0xD1 /*primary()*/));
  
#line 21947 "ad_x86.cpp"
  }
  {

#line 1697 "src/hotspot/cpu/x86/x86_32.ad"

    emit_rm(cbuf, 0x3, (0x7 /*secondary()*/), opnd_array(1)->reg(ra_,this,idx1)/* dst */ );
  
#line 21955 "ad_x86.cpp"
  }
}

uint sarI_eReg_1Node::size(PhaseRegAlloc *ra_) const {
  assert(VerifyOops || MachNode::size(ra_) <= 2, "bad fixed size");
  return (VerifyOops ? MachNode::size(ra_) : 2);
}

void sarI_mem_1Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// shift
  {

#line 1544 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0xD1 /*primary()*/));
  
#line 21976 "ad_x86.cpp"
  }
  {

#line 2186 "src/hotspot/cpu/x86/x86_32.ad"

    int rm_byte_opcode = (0x7 /*secondary()*/);
    int base     = opnd_array(1)->base(ra_,this,idx1);
    int index    = opnd_array(1)->index(ra_,this,idx1);
    int scale    = opnd_array(1)->scale();
    int displace = opnd_array(1)->disp(ra_,this,idx1);
    relocInfo::relocType disp_reloc = opnd_array(1)->disp_reloc(); // disp-as-oop when working with static globals
    encode_RegMem(cbuf, rm_byte_opcode, base, index, scale, displace, disp_reloc);
  
#line 21990 "ad_x86.cpp"
  }
}

void sarI_eReg_immNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// shift
  {

#line 1899 "src/hotspot/cpu/x86/x86_32.ad"

    emit_d8(cbuf, (0xC1 /*primary()*/));
    emit_rm(cbuf, 0x3, (0x7 /*secondary()*/), opnd_array(1)->reg(ra_,this,idx1)/* dst */);
    emit_d8(cbuf, opnd_array(2)->constant());
  
#line 22008 "ad_x86.cpp"
  }
}

uint sarI_eReg_immNode::size(PhaseRegAlloc *ra_) const {
  assert(VerifyOops || MachNode::size(ra_) <= 3, "bad fixed size");
  return (VerifyOops ? MachNode::size(ra_) : 3);
}

void sarI_mem_immNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// shift
  {

#line 1544 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0xC1 /*primary()*/));
  
#line 22029 "ad_x86.cpp"
  }
  {

#line 2186 "src/hotspot/cpu/x86/x86_32.ad"

    int rm_byte_opcode = (0x7 /*secondary()*/);
    int base     = opnd_array(1)->base(ra_,this,idx1);
    int index    = opnd_array(1)->index(ra_,this,idx1);
    int scale    = opnd_array(1)->scale();
    int displace = opnd_array(1)->disp(ra_,this,idx1);
    relocInfo::relocType disp_reloc = opnd_array(1)->disp_reloc(); // disp-as-oop when working with static globals
    encode_RegMem(cbuf, rm_byte_opcode, base, index, scale, displace, disp_reloc);
  
#line 22043 "ad_x86.cpp"
  }
  {

#line 1646 "src/hotspot/cpu/x86/x86_32.ad"

    // Check for 8-bit immediate, and set sign extend bit in opcode
    if ((opnd_array(2)->constant() >= -128) && (opnd_array(2)->constant() <= 127)) {
      emit_d8(cbuf, opnd_array(2)->constant());
    }
    else {                          // If 32-bit immediate
      // Output immediate
      emit_d32(cbuf, opnd_array(2)->constant());
    }
  
#line 22058 "ad_x86.cpp"
  }
}

void sarI_eReg_CLNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// shift
  {

#line 1544 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0xD3 /*primary()*/));
  
#line 22074 "ad_x86.cpp"
  }
  {

#line 1697 "src/hotspot/cpu/x86/x86_32.ad"

    emit_rm(cbuf, 0x3, (0x7 /*secondary()*/), opnd_array(1)->reg(ra_,this,idx1)/* dst */ );
  
#line 22082 "ad_x86.cpp"
  }
}

uint sarI_eReg_CLNode::size(PhaseRegAlloc *ra_) const {
  assert(VerifyOops || MachNode::size(ra_) <= 2, "bad fixed size");
  return (VerifyOops ? MachNode::size(ra_) : 2);
}

void shrI_eReg_1Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// shift
  {

#line 1544 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0xD1 /*primary()*/));
  
#line 22103 "ad_x86.cpp"
  }
  {

#line 1697 "src/hotspot/cpu/x86/x86_32.ad"

    emit_rm(cbuf, 0x3, (0x5 /*secondary()*/), opnd_array(1)->reg(ra_,this,idx1)/* dst */ );
  
#line 22111 "ad_x86.cpp"
  }
}

uint shrI_eReg_1Node::size(PhaseRegAlloc *ra_) const {
  assert(VerifyOops || MachNode::size(ra_) <= 2, "bad fixed size");
  return (VerifyOops ? MachNode::size(ra_) : 2);
}

void shrI_eReg_immNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// shift
  {

#line 1899 "src/hotspot/cpu/x86/x86_32.ad"

    emit_d8(cbuf, (0xC1 /*primary()*/));
    emit_rm(cbuf, 0x3, (0x5 /*secondary()*/), opnd_array(1)->reg(ra_,this,idx1)/* dst */);
    emit_d8(cbuf, opnd_array(2)->constant());
  
#line 22134 "ad_x86.cpp"
  }
}

uint shrI_eReg_immNode::size(PhaseRegAlloc *ra_) const {
  assert(VerifyOops || MachNode::size(ra_) <= 3, "bad fixed size");
  return (VerifyOops ? MachNode::size(ra_) : 3);
}

void i2bNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// twentyfour
  {
    C2_MacroAssembler _masm(&cbuf);

#line 8137 "src/hotspot/cpu/x86/x86_32.ad"

    __ movsbl(opnd_array(0)->as_Register(ra_,this)/* dst */, opnd_array(1)->as_Register(ra_,this,idx1)/* src */);
  
#line 22156 "ad_x86.cpp"
  }
}

uint i2bNode::size(PhaseRegAlloc *ra_) const {
  assert(VerifyOops || MachNode::size(ra_) <= 3, "bad fixed size");
  return (VerifyOops ? MachNode::size(ra_) : 3);
}

void i2sNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// sixteen
  {
    C2_MacroAssembler _masm(&cbuf);

#line 8150 "src/hotspot/cpu/x86/x86_32.ad"

    __ movswl(opnd_array(0)->as_Register(ra_,this)/* dst */, opnd_array(1)->as_Register(ra_,this,idx1)/* src */);
  
#line 22178 "ad_x86.cpp"
  }
}

uint i2sNode::size(PhaseRegAlloc *ra_) const {
  assert(VerifyOops || MachNode::size(ra_) <= 3, "bad fixed size");
  return (VerifyOops ? MachNode::size(ra_) : 3);
}

void shrI_eReg_CLNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// shift
  {

#line 1544 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0xD3 /*primary()*/));
  
#line 22199 "ad_x86.cpp"
  }
  {

#line 1697 "src/hotspot/cpu/x86/x86_32.ad"

    emit_rm(cbuf, 0x3, (0x5 /*secondary()*/), opnd_array(1)->reg(ra_,this,idx1)/* dst */ );
  
#line 22207 "ad_x86.cpp"
  }
}

uint shrI_eReg_CLNode::size(PhaseRegAlloc *ra_) const {
  assert(VerifyOops || MachNode::size(ra_) <= 2, "bad fixed size");
  return (VerifyOops ? MachNode::size(ra_) : 2);
}

void andI_eRegNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {

#line 1544 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0x23 /*primary()*/));
  
#line 22228 "ad_x86.cpp"
  }
  {

#line 1959 "src/hotspot/cpu/x86/x86_32.ad"

    emit_rm(cbuf, 0x3, opnd_array(1)->reg(ra_,this,idx1)/* dst */, opnd_array(2)->reg(ra_,this,idx2)/* src */);
  
#line 22236 "ad_x86.cpp"
  }
}

uint andI_eRegNode::size(PhaseRegAlloc *ra_) const {
  assert(VerifyOops || MachNode::size(ra_) <= 2, "bad fixed size");
  return (VerifyOops ? MachNode::size(ra_) : 2);
}

void andI_eReg_immNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {

#line 1634 "src/hotspot/cpu/x86/x86_32.ad"

    // Emit primary opcode and set sign-extend bit
    // Check for 8-bit immediate, and set sign extend bit in opcode
    if ((opnd_array(2)->constant() >= -128) && (opnd_array(2)->constant() <= 127)) {
      emit_opcode(cbuf, (0x81 /*primary()*/) | 0x02);    }
    else {                          // If 32-bit immediate
      emit_opcode(cbuf, (0x81 /*primary()*/));
    }
    // Emit r/m byte with secondary opcode, after primary opcode.
    emit_rm(cbuf, 0x3, (0x04 /*secondary()*/), opnd_array(1)->reg(ra_,this,idx1)/* dst */);
  
#line 22265 "ad_x86.cpp"
  }
  {

#line 1646 "src/hotspot/cpu/x86/x86_32.ad"

    // Check for 8-bit immediate, and set sign extend bit in opcode
    if ((opnd_array(2)->constant() >= -128) && (opnd_array(2)->constant() <= 127)) {
      emit_d8(cbuf, opnd_array(2)->constant());
    }
    else {                          // If 32-bit immediate
      // Output immediate
      emit_d32(cbuf, opnd_array(2)->constant());
    }
  
#line 22280 "ad_x86.cpp"
  }
}

void andI_eReg_memNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {

#line 1544 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0x23 /*primary()*/));
  
#line 22296 "ad_x86.cpp"
  }
  {

#line 2096 "src/hotspot/cpu/x86/x86_32.ad"

    int reg_encoding = opnd_array(1)->reg(ra_,this,idx1)/* dst */;
    int base  = opnd_array(2)->base(ra_,this,idx2);
    int index = opnd_array(2)->index(ra_,this,idx2);
    int scale = opnd_array(2)->scale();
    int displace = opnd_array(2)->disp(ra_,this,idx2);
    relocInfo::relocType disp_reloc = opnd_array(2)->disp_reloc();
    encode_RegMem(cbuf, reg_encoding, base, index, scale, displace, disp_reloc);
  
#line 22310 "ad_x86.cpp"
  }
}

void andI_eReg_mem_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// dst
  {

#line 1544 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0x23 /*primary()*/));
  
#line 22326 "ad_x86.cpp"
  }
  {

#line 2096 "src/hotspot/cpu/x86/x86_32.ad"

    int reg_encoding = opnd_array(2)->reg(ra_,this,idx2)/* dst */;
    int base  = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int displace = opnd_array(1)->disp(ra_,this,idx1);
    relocInfo::relocType disp_reloc = opnd_array(1)->disp_reloc();
    encode_RegMem(cbuf, reg_encoding, base, index, scale, displace, disp_reloc);
  
#line 22340 "ad_x86.cpp"
  }
}

void andI_mem_eRegNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {

#line 1544 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0x21 /*primary()*/));
  
#line 22356 "ad_x86.cpp"
  }
  {

#line 2096 "src/hotspot/cpu/x86/x86_32.ad"

    int reg_encoding = opnd_array(2)->reg(ra_,this,idx2)/* src */;
    int base  = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int displace = opnd_array(1)->disp(ra_,this,idx1);
    relocInfo::relocType disp_reloc = opnd_array(1)->disp_reloc();
    encode_RegMem(cbuf, reg_encoding, base, index, scale, displace, disp_reloc);
  
#line 22370 "ad_x86.cpp"
  }
}

void andI_mem_eReg_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {

#line 1544 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0x21 /*primary()*/));
  
#line 22386 "ad_x86.cpp"
  }
  {

#line 2096 "src/hotspot/cpu/x86/x86_32.ad"

    int reg_encoding = opnd_array(2)->reg(ra_,this,idx2)/* src */;
    int base  = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int displace = opnd_array(1)->disp(ra_,this,idx1);
    relocInfo::relocType disp_reloc = opnd_array(1)->disp_reloc();
    encode_RegMem(cbuf, reg_encoding, base, index, scale, displace, disp_reloc);
  
#line 22400 "ad_x86.cpp"
  }
}

void andI_mem_immNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {

#line 1624 "src/hotspot/cpu/x86/x86_32.ad"

    // Check for 8-bit immediate, and set sign extend bit in opcode
    if ((opnd_array(2)->constant() >= -128) && (opnd_array(2)->constant() <= 127)) {
      emit_opcode(cbuf, (0x81 /*primary()*/) | 0x02);
    }
    else {                          // If 32-bit immediate
      emit_opcode(cbuf, (0x81 /*primary()*/));
    }
  
#line 22422 "ad_x86.cpp"
  }
  {

#line 2186 "src/hotspot/cpu/x86/x86_32.ad"

    int rm_byte_opcode = (0x4 /*secondary()*/);
    int base     = opnd_array(1)->base(ra_,this,idx1);
    int index    = opnd_array(1)->index(ra_,this,idx1);
    int scale    = opnd_array(1)->scale();
    int displace = opnd_array(1)->disp(ra_,this,idx1);
    relocInfo::relocType disp_reloc = opnd_array(1)->disp_reloc(); // disp-as-oop when working with static globals
    encode_RegMem(cbuf, rm_byte_opcode, base, index, scale, displace, disp_reloc);
  
#line 22436 "ad_x86.cpp"
  }
  {

#line 1646 "src/hotspot/cpu/x86/x86_32.ad"

    // Check for 8-bit immediate, and set sign extend bit in opcode
    if ((opnd_array(2)->constant() >= -128) && (opnd_array(2)->constant() <= 127)) {
      emit_d8(cbuf, opnd_array(2)->constant());
    }
    else {                          // If 32-bit immediate
      // Output immediate
      emit_d32(cbuf, opnd_array(2)->constant());
    }
  
#line 22451 "ad_x86.cpp"
  }
}

void andnI_rReg_rReg_rRegNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// minus_1
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// src2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 8242 "src/hotspot/cpu/x86/x86_32.ad"

    __ andnl(opnd_array(0)->as_Register(ra_,this)/* dst */, opnd_array(1)->as_Register(ra_,this,idx1)/* src1 */, opnd_array(3)->as_Register(ra_,this,idx3)/* src2 */);
  
#line 22469 "ad_x86.cpp"
  }
}

void andnI_rReg_rReg_rReg_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src2
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src1
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// minus_1
  {
    C2_MacroAssembler _masm(&cbuf);

#line 8242 "src/hotspot/cpu/x86/x86_32.ad"

    __ andnl(opnd_array(0)->as_Register(ra_,this)/* dst */, opnd_array(2)->as_Register(ra_,this,idx2)/* src1 */, opnd_array(1)->as_Register(ra_,this,idx1)/* src2 */);
  
#line 22487 "ad_x86.cpp"
  }
}

void andnI_rReg_rReg_memNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// minus_1
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// src2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 8256 "src/hotspot/cpu/x86/x86_32.ad"

    __ andnl(opnd_array(0)->as_Register(ra_,this)/* dst */, opnd_array(1)->as_Register(ra_,this,idx1)/* src1 */, Address::make_raw(opnd_array(3)->base(ra_,this,idx3), opnd_array(3)->index(ra_,this,idx3), opnd_array(3)->scale(), opnd_array(3)->disp(ra_,this,idx3), opnd_array(3)->disp_reloc()));
  
#line 22505 "ad_x86.cpp"
  }
}

void andnI_rReg_rReg_mem_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// src2
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src1
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// minus_1
  {
    C2_MacroAssembler _masm(&cbuf);

#line 8256 "src/hotspot/cpu/x86/x86_32.ad"

    __ andnl(opnd_array(0)->as_Register(ra_,this)/* dst */, opnd_array(2)->as_Register(ra_,this,idx2)/* src1 */, Address::make_raw(opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1), opnd_array(1)->disp_reloc()));
  
#line 22523 "ad_x86.cpp"
  }
}

void blsiI_rReg_rRegNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// imm_zero
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 8269 "src/hotspot/cpu/x86/x86_32.ad"

    __ blsil(opnd_array(0)->as_Register(ra_,this)/* dst */, opnd_array(2)->as_Register(ra_,this,idx2)/* src */);
  
#line 22540 "ad_x86.cpp"
  }
}

void blsiI_rReg_rReg_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// imm_zero
  {
    C2_MacroAssembler _masm(&cbuf);

#line 8269 "src/hotspot/cpu/x86/x86_32.ad"

    __ blsil(opnd_array(0)->as_Register(ra_,this)/* dst */, opnd_array(1)->as_Register(ra_,this,idx1)/* src */);
  
#line 22557 "ad_x86.cpp"
  }
}

void blsiI_rReg_memNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// imm_zero
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 8283 "src/hotspot/cpu/x86/x86_32.ad"

    __ blsil(opnd_array(0)->as_Register(ra_,this)/* dst */, Address::make_raw(opnd_array(2)->base(ra_,this,idx2), opnd_array(2)->index(ra_,this,idx2), opnd_array(2)->scale(), opnd_array(2)->disp(ra_,this,idx2), opnd_array(2)->disp_reloc()));
  
#line 22574 "ad_x86.cpp"
  }
}

void blsiI_rReg_mem_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// imm_zero
  {
    C2_MacroAssembler _masm(&cbuf);

#line 8283 "src/hotspot/cpu/x86/x86_32.ad"

    __ blsil(opnd_array(0)->as_Register(ra_,this)/* dst */, Address::make_raw(opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1), opnd_array(1)->disp_reloc()));
  
#line 22591 "ad_x86.cpp"
  }
}

void blsmskI_rReg_rRegNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// minus_1
  {
    C2_MacroAssembler _masm(&cbuf);

#line 8297 "src/hotspot/cpu/x86/x86_32.ad"

    __ blsmskl(opnd_array(0)->as_Register(ra_,this)/* dst */, opnd_array(1)->as_Register(ra_,this,idx1)/* src */);
  
#line 22608 "ad_x86.cpp"
  }
}

void blsmskI_rReg_rReg_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// minus_1
  {
    C2_MacroAssembler _masm(&cbuf);

#line 8297 "src/hotspot/cpu/x86/x86_32.ad"

    __ blsmskl(opnd_array(0)->as_Register(ra_,this)/* dst */, opnd_array(1)->as_Register(ra_,this,idx1)/* src */);
  
#line 22625 "ad_x86.cpp"
  }
}

void blsmskI_rReg_memNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// minus_1
  {
    C2_MacroAssembler _masm(&cbuf);

#line 8313 "src/hotspot/cpu/x86/x86_32.ad"

    __ blsmskl(opnd_array(0)->as_Register(ra_,this)/* dst */, Address::make_raw(opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1), opnd_array(1)->disp_reloc()));
  
#line 22642 "ad_x86.cpp"
  }
}

void blsmskI_rReg_mem_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// minus_1
  {
    C2_MacroAssembler _masm(&cbuf);

#line 8313 "src/hotspot/cpu/x86/x86_32.ad"

    __ blsmskl(opnd_array(0)->as_Register(ra_,this)/* dst */, Address::make_raw(opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1), opnd_array(1)->disp_reloc()));
  
#line 22659 "ad_x86.cpp"
  }
}

void blsrI_rReg_rRegNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// minus_1
  {
    C2_MacroAssembler _masm(&cbuf);

#line 8328 "src/hotspot/cpu/x86/x86_32.ad"

    __ blsrl(opnd_array(0)->as_Register(ra_,this)/* dst */, opnd_array(1)->as_Register(ra_,this,idx1)/* src */);
  
#line 22676 "ad_x86.cpp"
  }
}

void blsrI_rReg_rReg_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// minus_1
  {
    C2_MacroAssembler _masm(&cbuf);

#line 8328 "src/hotspot/cpu/x86/x86_32.ad"

    __ blsrl(opnd_array(0)->as_Register(ra_,this)/* dst */, opnd_array(1)->as_Register(ra_,this,idx1)/* src */);
  
#line 22693 "ad_x86.cpp"
  }
}

void blsrI_rReg_memNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// minus_1
  {
    C2_MacroAssembler _masm(&cbuf);

#line 8344 "src/hotspot/cpu/x86/x86_32.ad"

    __ blsrl(opnd_array(0)->as_Register(ra_,this)/* dst */, Address::make_raw(opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1), opnd_array(1)->disp_reloc()));
  
#line 22710 "ad_x86.cpp"
  }
}

void blsrI_rReg_mem_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// minus_1
  {
    C2_MacroAssembler _masm(&cbuf);

#line 8344 "src/hotspot/cpu/x86/x86_32.ad"

    __ blsrl(opnd_array(0)->as_Register(ra_,this)/* dst */, Address::make_raw(opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1), opnd_array(1)->disp_reloc()));
  
#line 22727 "ad_x86.cpp"
  }
}

void orI_eRegNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {

#line 1544 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0x0B /*primary()*/));
  
#line 22743 "ad_x86.cpp"
  }
  {

#line 1959 "src/hotspot/cpu/x86/x86_32.ad"

    emit_rm(cbuf, 0x3, opnd_array(1)->reg(ra_,this,idx1)/* dst */, opnd_array(2)->reg(ra_,this,idx2)/* src */);
  
#line 22751 "ad_x86.cpp"
  }
}

uint orI_eRegNode::size(PhaseRegAlloc *ra_) const {
  assert(VerifyOops || MachNode::size(ra_) <= 2, "bad fixed size");
  return (VerifyOops ? MachNode::size(ra_) : 2);
}

void orI_eReg_castP2XNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {

#line 1544 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0x0B /*primary()*/));
  
#line 22772 "ad_x86.cpp"
  }
  {

#line 1959 "src/hotspot/cpu/x86/x86_32.ad"

    emit_rm(cbuf, 0x3, opnd_array(1)->reg(ra_,this,idx1)/* dst */, opnd_array(2)->reg(ra_,this,idx2)/* src */);
  
#line 22780 "ad_x86.cpp"
  }
}

uint orI_eReg_castP2XNode::size(PhaseRegAlloc *ra_) const {
  assert(VerifyOops || MachNode::size(ra_) <= 2, "bad fixed size");
  return (VerifyOops ? MachNode::size(ra_) : 2);
}

void orI_eReg_castP2X_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// dst
  {

#line 1544 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0x0B /*primary()*/));
  
#line 22801 "ad_x86.cpp"
  }
  {

#line 1959 "src/hotspot/cpu/x86/x86_32.ad"

    emit_rm(cbuf, 0x3, opnd_array(2)->reg(ra_,this,idx2)/* dst */, opnd_array(1)->reg(ra_,this,idx1)/* src */);
  
#line 22809 "ad_x86.cpp"
  }
}

uint orI_eReg_castP2X_0Node::size(PhaseRegAlloc *ra_) const {
  assert(VerifyOops || MachNode::size(ra_) <= 2, "bad fixed size");
  return (VerifyOops ? MachNode::size(ra_) : 2);
}

void orI_eReg_immNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {

#line 1634 "src/hotspot/cpu/x86/x86_32.ad"

    // Emit primary opcode and set sign-extend bit
    // Check for 8-bit immediate, and set sign extend bit in opcode
    if ((opnd_array(2)->constant() >= -128) && (opnd_array(2)->constant() <= 127)) {
      emit_opcode(cbuf, (0x81 /*primary()*/) | 0x02);    }
    else {                          // If 32-bit immediate
      emit_opcode(cbuf, (0x81 /*primary()*/));
    }
    // Emit r/m byte with secondary opcode, after primary opcode.
    emit_rm(cbuf, 0x3, (0x01 /*secondary()*/), opnd_array(1)->reg(ra_,this,idx1)/* dst */);
  
#line 22838 "ad_x86.cpp"
  }
  {

#line 1646 "src/hotspot/cpu/x86/x86_32.ad"

    // Check for 8-bit immediate, and set sign extend bit in opcode
    if ((opnd_array(2)->constant() >= -128) && (opnd_array(2)->constant() <= 127)) {
      emit_d8(cbuf, opnd_array(2)->constant());
    }
    else {                          // If 32-bit immediate
      // Output immediate
      emit_d32(cbuf, opnd_array(2)->constant());
    }
  
#line 22853 "ad_x86.cpp"
  }
}

void orI_eReg_memNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {

#line 1544 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0x0B /*primary()*/));
  
#line 22869 "ad_x86.cpp"
  }
  {

#line 2096 "src/hotspot/cpu/x86/x86_32.ad"

    int reg_encoding = opnd_array(1)->reg(ra_,this,idx1)/* dst */;
    int base  = opnd_array(2)->base(ra_,this,idx2);
    int index = opnd_array(2)->index(ra_,this,idx2);
    int scale = opnd_array(2)->scale();
    int displace = opnd_array(2)->disp(ra_,this,idx2);
    relocInfo::relocType disp_reloc = opnd_array(2)->disp_reloc();
    encode_RegMem(cbuf, reg_encoding, base, index, scale, displace, disp_reloc);
  
#line 22883 "ad_x86.cpp"
  }
}

void orI_eReg_mem_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// dst
  {

#line 1544 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0x0B /*primary()*/));
  
#line 22899 "ad_x86.cpp"
  }
  {

#line 2096 "src/hotspot/cpu/x86/x86_32.ad"

    int reg_encoding = opnd_array(2)->reg(ra_,this,idx2)/* dst */;
    int base  = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int displace = opnd_array(1)->disp(ra_,this,idx1);
    relocInfo::relocType disp_reloc = opnd_array(1)->disp_reloc();
    encode_RegMem(cbuf, reg_encoding, base, index, scale, displace, disp_reloc);
  
#line 22913 "ad_x86.cpp"
  }
}

void orI_mem_eRegNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {

#line 1544 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0x09 /*primary()*/));
  
#line 22929 "ad_x86.cpp"
  }
  {

#line 2096 "src/hotspot/cpu/x86/x86_32.ad"

    int reg_encoding = opnd_array(2)->reg(ra_,this,idx2)/* src */;
    int base  = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int displace = opnd_array(1)->disp(ra_,this,idx1);
    relocInfo::relocType disp_reloc = opnd_array(1)->disp_reloc();
    encode_RegMem(cbuf, reg_encoding, base, index, scale, displace, disp_reloc);
  
#line 22943 "ad_x86.cpp"
  }
}

void orI_mem_eReg_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {

#line 1544 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0x09 /*primary()*/));
  
#line 22959 "ad_x86.cpp"
  }
  {

#line 2096 "src/hotspot/cpu/x86/x86_32.ad"

    int reg_encoding = opnd_array(2)->reg(ra_,this,idx2)/* src */;
    int base  = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int displace = opnd_array(1)->disp(ra_,this,idx1);
    relocInfo::relocType disp_reloc = opnd_array(1)->disp_reloc();
    encode_RegMem(cbuf, reg_encoding, base, index, scale, displace, disp_reloc);
  
#line 22973 "ad_x86.cpp"
  }
}

void orI_mem_immNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {

#line 1624 "src/hotspot/cpu/x86/x86_32.ad"

    // Check for 8-bit immediate, and set sign extend bit in opcode
    if ((opnd_array(2)->constant() >= -128) && (opnd_array(2)->constant() <= 127)) {
      emit_opcode(cbuf, (0x81 /*primary()*/) | 0x02);
    }
    else {                          // If 32-bit immediate
      emit_opcode(cbuf, (0x81 /*primary()*/));
    }
  
#line 22995 "ad_x86.cpp"
  }
  {

#line 2186 "src/hotspot/cpu/x86/x86_32.ad"

    int rm_byte_opcode = (0x1 /*secondary()*/);
    int base     = opnd_array(1)->base(ra_,this,idx1);
    int index    = opnd_array(1)->index(ra_,this,idx1);
    int scale    = opnd_array(1)->scale();
    int displace = opnd_array(1)->disp(ra_,this,idx1);
    relocInfo::relocType disp_reloc = opnd_array(1)->disp_reloc(); // disp-as-oop when working with static globals
    encode_RegMem(cbuf, rm_byte_opcode, base, index, scale, displace, disp_reloc);
  
#line 23009 "ad_x86.cpp"
  }
  {

#line 1646 "src/hotspot/cpu/x86/x86_32.ad"

    // Check for 8-bit immediate, and set sign extend bit in opcode
    if ((opnd_array(2)->constant() >= -128) && (opnd_array(2)->constant() <= 127)) {
      emit_d8(cbuf, opnd_array(2)->constant());
    }
    else {                          // If 32-bit immediate
      // Output immediate
      emit_d32(cbuf, opnd_array(2)->constant());
    }
  
#line 23024 "ad_x86.cpp"
  }
}

void rolI_eReg_imm1Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// shift
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// cr
  {

#line 1544 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0xD1 /*primary()*/));
  
#line 23040 "ad_x86.cpp"
  }
  {

#line 1697 "src/hotspot/cpu/x86/x86_32.ad"

    emit_rm(cbuf, 0x3, (0x0 /*secondary()*/), opnd_array(1)->reg(ra_,this,idx1)/* dst */ );
  
#line 23048 "ad_x86.cpp"
  }
}

void rolI_eReg_imm8Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// shift
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// cr
  {

#line 1899 "src/hotspot/cpu/x86/x86_32.ad"

    emit_d8(cbuf, (0xC1 /*primary()*/));
    emit_rm(cbuf, 0x3, (0x0 /*secondary()*/), opnd_array(1)->reg(ra_,this,idx1)/* dst */);
    emit_d8(cbuf, opnd_array(2)->constant());
  
#line 23066 "ad_x86.cpp"
  }
}

void rolI_eReg_CLNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// shift
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// cr
  {

#line 1544 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0xD3 /*primary()*/));
  
#line 23082 "ad_x86.cpp"
  }
  {

#line 1697 "src/hotspot/cpu/x86/x86_32.ad"

    emit_rm(cbuf, 0x3, (0x0 /*secondary()*/), opnd_array(1)->reg(ra_,this,idx1)/* dst */ );
  
#line 23090 "ad_x86.cpp"
  }
}

void rorI_eReg_imm1Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// shift
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// cr
  {

#line 1544 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0xD1 /*primary()*/));
  
#line 23106 "ad_x86.cpp"
  }
  {

#line 1697 "src/hotspot/cpu/x86/x86_32.ad"

    emit_rm(cbuf, 0x3, (0x1 /*secondary()*/), opnd_array(1)->reg(ra_,this,idx1)/* dst */ );
  
#line 23114 "ad_x86.cpp"
  }
}

void rorI_eReg_imm8Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// shift
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// cr
  {

#line 1899 "src/hotspot/cpu/x86/x86_32.ad"

    emit_d8(cbuf, (0xC1 /*primary()*/));
    emit_rm(cbuf, 0x3, (0x1 /*secondary()*/), opnd_array(1)->reg(ra_,this,idx1)/* dst */);
    emit_d8(cbuf, opnd_array(2)->constant());
  
#line 23132 "ad_x86.cpp"
  }
}

void rorI_eReg_CLNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// shift
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// cr
  {

#line 1544 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0xD3 /*primary()*/));
  
#line 23148 "ad_x86.cpp"
  }
  {

#line 1697 "src/hotspot/cpu/x86/x86_32.ad"

    emit_rm(cbuf, 0x3, (0x1 /*secondary()*/), opnd_array(1)->reg(ra_,this,idx1)/* dst */ );
  
#line 23156 "ad_x86.cpp"
  }
}

void xorI_eRegNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {

#line 1544 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0x33 /*primary()*/));
  
#line 23172 "ad_x86.cpp"
  }
  {

#line 1959 "src/hotspot/cpu/x86/x86_32.ad"

    emit_rm(cbuf, 0x3, opnd_array(1)->reg(ra_,this,idx1)/* dst */, opnd_array(2)->reg(ra_,this,idx2)/* src */);
  
#line 23180 "ad_x86.cpp"
  }
}

uint xorI_eRegNode::size(PhaseRegAlloc *ra_) const {
  assert(VerifyOops || MachNode::size(ra_) <= 2, "bad fixed size");
  return (VerifyOops ? MachNode::size(ra_) : 2);
}

void xorI_eReg_im1Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// imm
  {
    C2_MacroAssembler _masm(&cbuf);

#line 8577 "src/hotspot/cpu/x86/x86_32.ad"

     __ notl(opnd_array(1)->as_Register(ra_,this,idx1)/* dst */);
  
#line 23202 "ad_x86.cpp"
  }
}

uint xorI_eReg_im1Node::size(PhaseRegAlloc *ra_) const {
  assert(VerifyOops || MachNode::size(ra_) <= 2, "bad fixed size");
  return (VerifyOops ? MachNode::size(ra_) : 2);
}

void xorI_eReg_immNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {

#line 1634 "src/hotspot/cpu/x86/x86_32.ad"

    // Emit primary opcode and set sign-extend bit
    // Check for 8-bit immediate, and set sign extend bit in opcode
    if ((opnd_array(2)->constant() >= -128) && (opnd_array(2)->constant() <= 127)) {
      emit_opcode(cbuf, (0x81 /*primary()*/) | 0x02);    }
    else {                          // If 32-bit immediate
      emit_opcode(cbuf, (0x81 /*primary()*/));
    }
    // Emit r/m byte with secondary opcode, after primary opcode.
    emit_rm(cbuf, 0x3, (0x06 /*secondary()*/), opnd_array(1)->reg(ra_,this,idx1)/* dst */);
  
#line 23231 "ad_x86.cpp"
  }
  {

#line 1646 "src/hotspot/cpu/x86/x86_32.ad"

    // Check for 8-bit immediate, and set sign extend bit in opcode
    if ((opnd_array(2)->constant() >= -128) && (opnd_array(2)->constant() <= 127)) {
      emit_d8(cbuf, opnd_array(2)->constant());
    }
    else {                          // If 32-bit immediate
      // Output immediate
      emit_d32(cbuf, opnd_array(2)->constant());
    }
  
#line 23246 "ad_x86.cpp"
  }
}

void xorI_eReg_memNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {

#line 1544 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0x33 /*primary()*/));
  
#line 23262 "ad_x86.cpp"
  }
  {

#line 2096 "src/hotspot/cpu/x86/x86_32.ad"

    int reg_encoding = opnd_array(1)->reg(ra_,this,idx1)/* dst */;
    int base  = opnd_array(2)->base(ra_,this,idx2);
    int index = opnd_array(2)->index(ra_,this,idx2);
    int scale = opnd_array(2)->scale();
    int displace = opnd_array(2)->disp(ra_,this,idx2);
    relocInfo::relocType disp_reloc = opnd_array(2)->disp_reloc();
    encode_RegMem(cbuf, reg_encoding, base, index, scale, displace, disp_reloc);
  
#line 23276 "ad_x86.cpp"
  }
}

void xorI_eReg_mem_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// dst
  {

#line 1544 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0x33 /*primary()*/));
  
#line 23292 "ad_x86.cpp"
  }
  {

#line 2096 "src/hotspot/cpu/x86/x86_32.ad"

    int reg_encoding = opnd_array(2)->reg(ra_,this,idx2)/* dst */;
    int base  = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int displace = opnd_array(1)->disp(ra_,this,idx1);
    relocInfo::relocType disp_reloc = opnd_array(1)->disp_reloc();
    encode_RegMem(cbuf, reg_encoding, base, index, scale, displace, disp_reloc);
  
#line 23306 "ad_x86.cpp"
  }
}

void xorI_mem_eRegNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {

#line 1544 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0x31 /*primary()*/));
  
#line 23322 "ad_x86.cpp"
  }
  {

#line 2096 "src/hotspot/cpu/x86/x86_32.ad"

    int reg_encoding = opnd_array(2)->reg(ra_,this,idx2)/* src */;
    int base  = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int displace = opnd_array(1)->disp(ra_,this,idx1);
    relocInfo::relocType disp_reloc = opnd_array(1)->disp_reloc();
    encode_RegMem(cbuf, reg_encoding, base, index, scale, displace, disp_reloc);
  
#line 23336 "ad_x86.cpp"
  }
}

void xorI_mem_eReg_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {

#line 1544 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0x31 /*primary()*/));
  
#line 23352 "ad_x86.cpp"
  }
  {

#line 2096 "src/hotspot/cpu/x86/x86_32.ad"

    int reg_encoding = opnd_array(2)->reg(ra_,this,idx2)/* src */;
    int base  = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int displace = opnd_array(1)->disp(ra_,this,idx1);
    relocInfo::relocType disp_reloc = opnd_array(1)->disp_reloc();
    encode_RegMem(cbuf, reg_encoding, base, index, scale, displace, disp_reloc);
  
#line 23366 "ad_x86.cpp"
  }
}

void xorI_mem_immNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {

#line 1624 "src/hotspot/cpu/x86/x86_32.ad"

    // Check for 8-bit immediate, and set sign extend bit in opcode
    if ((opnd_array(2)->constant() >= -128) && (opnd_array(2)->constant() <= 127)) {
      emit_opcode(cbuf, (0x81 /*primary()*/) | 0x02);
    }
    else {                          // If 32-bit immediate
      emit_opcode(cbuf, (0x81 /*primary()*/));
    }
  
#line 23388 "ad_x86.cpp"
  }
  {

#line 2186 "src/hotspot/cpu/x86/x86_32.ad"

    int rm_byte_opcode = (0x6 /*secondary()*/);
    int base     = opnd_array(1)->base(ra_,this,idx1);
    int index    = opnd_array(1)->index(ra_,this,idx1);
    int scale    = opnd_array(1)->scale();
    int displace = opnd_array(1)->disp(ra_,this,idx1);
    relocInfo::relocType disp_reloc = opnd_array(1)->disp_reloc(); // disp-as-oop when working with static globals
    encode_RegMem(cbuf, rm_byte_opcode, base, index, scale, displace, disp_reloc);
  
#line 23402 "ad_x86.cpp"
  }
  {

#line 1646 "src/hotspot/cpu/x86/x86_32.ad"

    // Check for 8-bit immediate, and set sign extend bit in opcode
    if ((opnd_array(2)->constant() >= -128) && (opnd_array(2)->constant() <= 127)) {
      emit_d8(cbuf, opnd_array(2)->constant());
    }
    else {                          // If 32-bit immediate
      // Output immediate
      emit_d32(cbuf, opnd_array(2)->constant());
    }
  
#line 23417 "ad_x86.cpp"
  }
}

void movI_nocopyNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {

#line 1951 "src/hotspot/cpu/x86/x86_32.ad"

    encode_Copy( cbuf, opnd_array(0)->reg(ra_,this)/* dst */, opnd_array(1)->reg(ra_,this,idx1)/* src */ );
  
#line 23432 "ad_x86.cpp"
  }
}

void ci2bNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// cr
  {

#line 2249 "src/hotspot/cpu/x86/x86_32.ad"

    // NEG $dst
    emit_opcode(cbuf,0xF7);
    emit_rm(cbuf, 0x3, 0x03, opnd_array(1)->reg(ra_,this,idx1)/* dst */ );
  
#line 23450 "ad_x86.cpp"
  }
  {

#line 1566 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf,(0x13));
    emit_rm(cbuf, 0x3, opnd_array(1)->reg(ra_,this,idx1)/* dst */, opnd_array(2)->reg(ra_,this,idx2)/* src */);
  
#line 23459 "ad_x86.cpp"
  }
}

uint ci2bNode::size(PhaseRegAlloc *ra_) const {
  assert(VerifyOops || MachNode::size(ra_) <= 4, "bad fixed size");
  return (VerifyOops ? MachNode::size(ra_) : 4);
}

void movP_nocopyNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {

#line 1951 "src/hotspot/cpu/x86/x86_32.ad"

    encode_Copy( cbuf, opnd_array(0)->reg(ra_,this)/* dst */, opnd_array(1)->reg(ra_,this,idx1)/* src */ );
  
#line 23479 "ad_x86.cpp"
  }
}

void cp2bNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// cr
  {

#line 2249 "src/hotspot/cpu/x86/x86_32.ad"

    // NEG $dst
    emit_opcode(cbuf,0xF7);
    emit_rm(cbuf, 0x3, 0x03, opnd_array(1)->reg(ra_,this,idx1)/* dst */ );
  
#line 23497 "ad_x86.cpp"
  }
  {

#line 1566 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf,(0x13));
    emit_rm(cbuf, 0x3, opnd_array(1)->reg(ra_,this,idx1)/* dst */, opnd_array(2)->reg(ra_,this,idx2)/* src */);
  
#line 23506 "ad_x86.cpp"
  }
}

void cmpLTMaskNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// p
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// q
  {
    C2_MacroAssembler _masm(&cbuf);

#line 8695 "src/hotspot/cpu/x86/x86_32.ad"

    Register Rp = opnd_array(1)->as_Register(ra_,this,idx1)/* p */;
    Register Rq = opnd_array(2)->as_Register(ra_,this,idx2)/* q */;
    Register Rd = opnd_array(0)->as_Register(ra_,this)/* dst */;
    Label done;
    __ xorl(Rd, Rd);
    __ cmpl(Rp, Rq);
    __ setb(Assembler::less, Rd);
    __ negl(Rd);
  
#line 23530 "ad_x86.cpp"
  }
}

void cmpLTMask0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// zero
  {
    C2_MacroAssembler _masm(&cbuf);

#line 8715 "src/hotspot/cpu/x86/x86_32.ad"

  __ sarl(opnd_array(1)->as_Register(ra_,this,idx1)/* dst */, 31);
  
#line 23547 "ad_x86.cpp"
  }
}

void cadd_cmpLTMaskNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// p
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// q
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// y
  {
    C2_MacroAssembler _masm(&cbuf);

#line 8730 "src/hotspot/cpu/x86/x86_32.ad"

    Register Rp = opnd_array(1)->as_Register(ra_,this,idx1)/* p */;
    Register Rq = opnd_array(2)->as_Register(ra_,this,idx2)/* q */;
    Register Ry = opnd_array(3)->as_Register(ra_,this,idx3)/* y */;
    Label done;
    __ subl(Rp, Rq);
    __ jccb(Assembler::greaterEqual, done);
    __ addl(Rp, Ry);
    __ bind(done);
  
#line 23572 "ad_x86.cpp"
  }
}

void cadd_cmpLTMask_1Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// p
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// q
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// y
  {
    C2_MacroAssembler _masm(&cbuf);

#line 8730 "src/hotspot/cpu/x86/x86_32.ad"

    Register Rp = opnd_array(1)->as_Register(ra_,this,idx1)/* p */;
    Register Rq = opnd_array(2)->as_Register(ra_,this,idx2)/* q */;
    Register Ry = opnd_array(3)->as_Register(ra_,this,idx3)/* y */;
    Label done;
    __ subl(Rp, Rq);
    __ jccb(Assembler::greaterEqual, done);
    __ addl(Rp, Ry);
    __ bind(done);
  
#line 23597 "ad_x86.cpp"
  }
}

void cadd_cmpLTMask_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// y
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// p
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// q
  {
    C2_MacroAssembler _masm(&cbuf);

#line 8730 "src/hotspot/cpu/x86/x86_32.ad"

    Register Rp = opnd_array(2)->as_Register(ra_,this,idx2)/* p */;
    Register Rq = opnd_array(3)->as_Register(ra_,this,idx3)/* q */;
    Register Ry = opnd_array(1)->as_Register(ra_,this,idx1)/* y */;
    Label done;
    __ subl(Rp, Rq);
    __ jccb(Assembler::greaterEqual, done);
    __ addl(Rp, Ry);
    __ bind(done);
  
#line 23622 "ad_x86.cpp"
  }
}

void cadd_cmpLTMask_2Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// p
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// q
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// y
  {
    C2_MacroAssembler _masm(&cbuf);

#line 8730 "src/hotspot/cpu/x86/x86_32.ad"

    Register Rp = opnd_array(1)->as_Register(ra_,this,idx1)/* p */;
    Register Rq = opnd_array(2)->as_Register(ra_,this,idx2)/* q */;
    Register Ry = opnd_array(3)->as_Register(ra_,this,idx3)/* y */;
    Label done;
    __ subl(Rp, Rq);
    __ jccb(Assembler::greaterEqual, done);
    __ addl(Rp, Ry);
    __ bind(done);
  
#line 23647 "ad_x86.cpp"
  }
}

void and_cmpLTMaskNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// p
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// q
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// y
  {
    C2_MacroAssembler _masm(&cbuf);

#line 8755 "src/hotspot/cpu/x86/x86_32.ad"

    Register Rp = opnd_array(1)->as_Register(ra_,this,idx1)/* p */;
    Register Rq = opnd_array(2)->as_Register(ra_,this,idx2)/* q */;
    Register Ry = opnd_array(3)->as_Register(ra_,this,idx3)/* y */;
    Label done;
    __ cmpl(Rp, Rq);
    __ jccb(Assembler::less, done);
    __ xorl(Ry, Ry);
    __ bind(done);
  
#line 23672 "ad_x86.cpp"
  }
}

void and_cmpLTMask_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// y
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// p
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// q
  {
    C2_MacroAssembler _masm(&cbuf);

#line 8755 "src/hotspot/cpu/x86/x86_32.ad"

    Register Rp = opnd_array(2)->as_Register(ra_,this,idx2)/* p */;
    Register Rq = opnd_array(3)->as_Register(ra_,this,idx3)/* q */;
    Register Ry = opnd_array(1)->as_Register(ra_,this,idx1)/* y */;
    Label done;
    __ cmpl(Rp, Rq);
    __ jccb(Assembler::less, done);
    __ xorl(Ry, Ry);
    __ bind(done);
  
#line 23697 "ad_x86.cpp"
  }
}

void overflowAddI_eRegNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// op1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// op2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 8782 "src/hotspot/cpu/x86/x86_32.ad"

    __ addl(opnd_array(1)->as_Register(ra_,this,idx1)/* op1 */, opnd_array(2)->as_Register(ra_,this,idx2)/* op2 */);
  
#line 23714 "ad_x86.cpp"
  }
}

void overflowAddI_rReg_immNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// op1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// op2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 8795 "src/hotspot/cpu/x86/x86_32.ad"

    __ addl(opnd_array(1)->as_Register(ra_,this,idx1)/* op1 */, opnd_array(2)->constant());
  
#line 23731 "ad_x86.cpp"
  }
}

void overflowSubI_rRegNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// op1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// op2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 8806 "src/hotspot/cpu/x86/x86_32.ad"

    __ cmpl(opnd_array(1)->as_Register(ra_,this,idx1)/* op1 */, opnd_array(2)->as_Register(ra_,this,idx2)/* op2 */);
  
#line 23748 "ad_x86.cpp"
  }
}

void overflowSubI_rReg_immNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// op1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// op2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 8817 "src/hotspot/cpu/x86/x86_32.ad"

    __ cmpl(opnd_array(1)->as_Register(ra_,this,idx1)/* op1 */, opnd_array(2)->constant());
  
#line 23765 "ad_x86.cpp"
  }
}

void overflowNegI_rRegNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// zero
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// op2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 8829 "src/hotspot/cpu/x86/x86_32.ad"

    __ negl(opnd_array(2)->as_Register(ra_,this,idx2)/* op2 */);
  
#line 23782 "ad_x86.cpp"
  }
}

void overflowMulI_rRegNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// op1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// op2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 8841 "src/hotspot/cpu/x86/x86_32.ad"

    __ imull(opnd_array(1)->as_Register(ra_,this,idx1)/* op1 */, opnd_array(2)->as_Register(ra_,this,idx2)/* op2 */);
  
#line 23799 "ad_x86.cpp"
  }
}

void overflowMulI_rReg_immNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// op1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// op2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// tmp
  {
    C2_MacroAssembler _masm(&cbuf);

#line 8853 "src/hotspot/cpu/x86/x86_32.ad"

    __ imull(opnd_array(3)->as_Register(ra_,this,idx3)/* tmp */, opnd_array(1)->as_Register(ra_,this,idx1)/* op1 */, opnd_array(2)->constant());
  
#line 23817 "ad_x86.cpp"
  }
}

void absI_rRegNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// dst
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// tmp
  {
    C2_MacroAssembler _masm(&cbuf);

#line 8870 "src/hotspot/cpu/x86/x86_32.ad"

    __ movl(opnd_array(3)->as_Register(ra_,this,idx3)/* tmp */, opnd_array(1)->as_Register(ra_,this,idx1)/* src */);
    __ sarl(opnd_array(3)->as_Register(ra_,this,idx3)/* tmp */, 31);
    __ movl(opnd_array(2)->as_Register(ra_,this,idx2)/* dst */, opnd_array(1)->as_Register(ra_,this,idx1)/* src */);
    __ xorl(opnd_array(2)->as_Register(ra_,this,idx2)/* dst */, opnd_array(3)->as_Register(ra_,this,idx3)/* tmp */);
    __ subl(opnd_array(2)->as_Register(ra_,this,idx2)/* dst */, opnd_array(3)->as_Register(ra_,this,idx3)/* tmp */);
  
#line 23839 "ad_x86.cpp"
  }
}

void addL_eRegNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {

#line 1963 "src/hotspot/cpu/x86/x86_32.ad"

    emit_d8(cbuf, (0x03 /*primary()*/));
    emit_rm(cbuf, 0x3, opnd_array(1)->reg(ra_,this,idx1)/* dst */, opnd_array(2)->reg(ra_,this,idx2)/* src */);
  
#line 23856 "ad_x86.cpp"
  }
  {

#line 1968 "src/hotspot/cpu/x86/x86_32.ad"

    emit_d8(cbuf, (0x13 /*secondary()*/));
    emit_rm(cbuf, 0x3, HIGH_FROM_LOW_ENC(opnd_array(1)->reg(ra_,this,idx1)/* dst */), HIGH_FROM_LOW_ENC(opnd_array(2)->reg(ra_,this,idx2)/* src */));
  
#line 23865 "ad_x86.cpp"
  }
}

void addL_eReg_immNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {

#line 1657 "src/hotspot/cpu/x86/x86_32.ad"

    // Emit primary opcode and set sign-extend bit
    // Check for 8-bit immediate, and set sign extend bit in opcode
    int con = (int)opnd_array(2)->constantL(); // Throw away top bits
    emit_opcode(cbuf, ((con >= -128) && (con <= 127)) ? ((0x81 /*primary()*/) | 0x02) : (0x81 /*primary()*/));
    // Emit r/m byte with secondary opcode, after primary opcode.
    emit_rm(cbuf, 0x3, (0x00 /*secondary()*/), opnd_array(1)->reg(ra_,this,idx1)/* dst */);
    if ((con >= -128) && (con <= 127)) emit_d8 (cbuf,con);
    else                               emit_d32(cbuf,con);
  
#line 23888 "ad_x86.cpp"
  }
  {

#line 1668 "src/hotspot/cpu/x86/x86_32.ad"

    // Emit primary opcode and set sign-extend bit
    // Check for 8-bit immediate, and set sign extend bit in opcode
    int con = (int)(opnd_array(2)->constantL() >> 32); // Throw away bottom bits
    emit_opcode(cbuf, ((con >= -128) && (con <= 127)) ? ((0x81 /*primary()*/) | 0x02) : (0x81 /*primary()*/));
    // Emit r/m byte with tertiary opcode, after primary opcode.
    emit_rm(cbuf, 0x3, (0x02 /*tertiary()*/), HIGH_FROM_LOW_ENC(opnd_array(1)->reg(ra_,this,idx1)/* dst */));
    if ((con >= -128) && (con <= 127)) emit_d8 (cbuf,con);
    else                               emit_d32(cbuf,con);
  
#line 23903 "ad_x86.cpp"
  }
}

void addL_eReg_memNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// mem
  {

#line 1544 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0x03 /*primary()*/));
  
#line 23919 "ad_x86.cpp"
  }
  {

#line 2096 "src/hotspot/cpu/x86/x86_32.ad"

    int reg_encoding = opnd_array(1)->reg(ra_,this,idx1)/* dst */;
    int base  = opnd_array(2)->base(ra_,this,idx2);
    int index = opnd_array(2)->index(ra_,this,idx2);
    int scale = opnd_array(2)->scale();
    int displace = opnd_array(2)->disp(ra_,this,idx2);
    relocInfo::relocType disp_reloc = opnd_array(2)->disp_reloc();
    encode_RegMem(cbuf, reg_encoding, base, index, scale, displace, disp_reloc);
  
#line 23933 "ad_x86.cpp"
  }
  {

#line 1549 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0x13 /*secondary()*/));
  
#line 23941 "ad_x86.cpp"
  }
  {

#line 2106 "src/hotspot/cpu/x86/x86_32.ad"

    int reg_encoding = HIGH_FROM_LOW_ENC(opnd_array(1)->reg(ra_,this,idx1)/* dst */);  // Hi register of pair, computed from lo
    int base  = opnd_array(2)->base(ra_,this,idx2);
    int index = opnd_array(2)->index(ra_,this,idx2);
    int scale = opnd_array(2)->scale();
    int displace = opnd_array(2)->disp(ra_,this,idx2) + 4;      // Offset is 4 further in memory
    assert( opnd_array(2)->disp_reloc() == relocInfo::none, "Cannot add 4 to oop" );
    encode_RegMem(cbuf, reg_encoding, base, index, scale, displace, relocInfo::none);
  
#line 23955 "ad_x86.cpp"
  }
}

void addL_eReg_mem_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// dst
  {

#line 1544 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0x03 /*primary()*/));
  
#line 23971 "ad_x86.cpp"
  }
  {

#line 2096 "src/hotspot/cpu/x86/x86_32.ad"

    int reg_encoding = opnd_array(2)->reg(ra_,this,idx2)/* dst */;
    int base  = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int displace = opnd_array(1)->disp(ra_,this,idx1);
    relocInfo::relocType disp_reloc = opnd_array(1)->disp_reloc();
    encode_RegMem(cbuf, reg_encoding, base, index, scale, displace, disp_reloc);
  
#line 23985 "ad_x86.cpp"
  }
  {

#line 1549 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0x13 /*secondary()*/));
  
#line 23993 "ad_x86.cpp"
  }
  {

#line 2106 "src/hotspot/cpu/x86/x86_32.ad"

    int reg_encoding = HIGH_FROM_LOW_ENC(opnd_array(2)->reg(ra_,this,idx2)/* dst */);  // Hi register of pair, computed from lo
    int base  = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int displace = opnd_array(1)->disp(ra_,this,idx1) + 4;      // Offset is 4 further in memory
    assert( opnd_array(1)->disp_reloc() == relocInfo::none, "Cannot add 4 to oop" );
    encode_RegMem(cbuf, reg_encoding, base, index, scale, displace, relocInfo::none);
  
#line 24007 "ad_x86.cpp"
  }
}

void subL_eRegNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {

#line 1963 "src/hotspot/cpu/x86/x86_32.ad"

    emit_d8(cbuf, (0x2B /*primary()*/));
    emit_rm(cbuf, 0x3, opnd_array(1)->reg(ra_,this,idx1)/* dst */, opnd_array(2)->reg(ra_,this,idx2)/* src */);
  
#line 24024 "ad_x86.cpp"
  }
  {

#line 1968 "src/hotspot/cpu/x86/x86_32.ad"

    emit_d8(cbuf, (0x1B /*secondary()*/));
    emit_rm(cbuf, 0x3, HIGH_FROM_LOW_ENC(opnd_array(1)->reg(ra_,this,idx1)/* dst */), HIGH_FROM_LOW_ENC(opnd_array(2)->reg(ra_,this,idx2)/* src */));
  
#line 24033 "ad_x86.cpp"
  }
}

void subL_eReg_immNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {

#line 1657 "src/hotspot/cpu/x86/x86_32.ad"

    // Emit primary opcode and set sign-extend bit
    // Check for 8-bit immediate, and set sign extend bit in opcode
    int con = (int)opnd_array(2)->constantL(); // Throw away top bits
    emit_opcode(cbuf, ((con >= -128) && (con <= 127)) ? ((0x81 /*primary()*/) | 0x02) : (0x81 /*primary()*/));
    // Emit r/m byte with secondary opcode, after primary opcode.
    emit_rm(cbuf, 0x3, (0x05 /*secondary()*/), opnd_array(1)->reg(ra_,this,idx1)/* dst */);
    if ((con >= -128) && (con <= 127)) emit_d8 (cbuf,con);
    else                               emit_d32(cbuf,con);
  
#line 24056 "ad_x86.cpp"
  }
  {

#line 1668 "src/hotspot/cpu/x86/x86_32.ad"

    // Emit primary opcode and set sign-extend bit
    // Check for 8-bit immediate, and set sign extend bit in opcode
    int con = (int)(opnd_array(2)->constantL() >> 32); // Throw away bottom bits
    emit_opcode(cbuf, ((con >= -128) && (con <= 127)) ? ((0x81 /*primary()*/) | 0x02) : (0x81 /*primary()*/));
    // Emit r/m byte with tertiary opcode, after primary opcode.
    emit_rm(cbuf, 0x3, (0x03 /*tertiary()*/), HIGH_FROM_LOW_ENC(opnd_array(1)->reg(ra_,this,idx1)/* dst */));
    if ((con >= -128) && (con <= 127)) emit_d8 (cbuf,con);
    else                               emit_d32(cbuf,con);
  
#line 24071 "ad_x86.cpp"
  }
}

void subL_eReg_memNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// mem
  {

#line 1544 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0x2B /*primary()*/));
  
#line 24087 "ad_x86.cpp"
  }
  {

#line 2096 "src/hotspot/cpu/x86/x86_32.ad"

    int reg_encoding = opnd_array(1)->reg(ra_,this,idx1)/* dst */;
    int base  = opnd_array(2)->base(ra_,this,idx2);
    int index = opnd_array(2)->index(ra_,this,idx2);
    int scale = opnd_array(2)->scale();
    int displace = opnd_array(2)->disp(ra_,this,idx2);
    relocInfo::relocType disp_reloc = opnd_array(2)->disp_reloc();
    encode_RegMem(cbuf, reg_encoding, base, index, scale, displace, disp_reloc);
  
#line 24101 "ad_x86.cpp"
  }
  {

#line 1549 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0x1B /*secondary()*/));
  
#line 24109 "ad_x86.cpp"
  }
  {

#line 2106 "src/hotspot/cpu/x86/x86_32.ad"

    int reg_encoding = HIGH_FROM_LOW_ENC(opnd_array(1)->reg(ra_,this,idx1)/* dst */);  // Hi register of pair, computed from lo
    int base  = opnd_array(2)->base(ra_,this,idx2);
    int index = opnd_array(2)->index(ra_,this,idx2);
    int scale = opnd_array(2)->scale();
    int displace = opnd_array(2)->disp(ra_,this,idx2) + 4;      // Offset is 4 further in memory
    assert( opnd_array(2)->disp_reloc() == relocInfo::none, "Cannot add 4 to oop" );
    encode_RegMem(cbuf, reg_encoding, base, index, scale, displace, relocInfo::none);
  
#line 24123 "ad_x86.cpp"
  }
}

void negL_eRegNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// zero
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// dst
  {

#line 2890 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf,0xF7);    // NEG hi
    emit_rm    (cbuf,0x3, 0x3, HIGH_FROM_LOW_ENC(opnd_array(2)->reg(ra_,this,idx2)/* dst */));
    emit_opcode(cbuf,0xF7);    // NEG lo
    emit_rm    (cbuf,0x3, 0x3,               opnd_array(2)->reg(ra_,this,idx2)/* dst */ );
    emit_opcode(cbuf,0x83);    // SBB hi,0
    emit_rm    (cbuf,0x3, 0x3, HIGH_FROM_LOW_ENC(opnd_array(2)->reg(ra_,this,idx2)/* dst */));
    emit_d8    (cbuf,0 );
  
#line 24145 "ad_x86.cpp"
  }
}

void andL_eRegNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {

#line 1963 "src/hotspot/cpu/x86/x86_32.ad"

    emit_d8(cbuf, (0x23 /*primary()*/));
    emit_rm(cbuf, 0x3, opnd_array(1)->reg(ra_,this,idx1)/* dst */, opnd_array(2)->reg(ra_,this,idx2)/* src */);
  
#line 24162 "ad_x86.cpp"
  }
  {

#line 1968 "src/hotspot/cpu/x86/x86_32.ad"

    emit_d8(cbuf, (0x23 /*secondary()*/));
    emit_rm(cbuf, 0x3, HIGH_FROM_LOW_ENC(opnd_array(1)->reg(ra_,this,idx1)/* dst */), HIGH_FROM_LOW_ENC(opnd_array(2)->reg(ra_,this,idx2)/* src */));
  
#line 24171 "ad_x86.cpp"
  }
}

void andL_eReg_immNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {

#line 1657 "src/hotspot/cpu/x86/x86_32.ad"

    // Emit primary opcode and set sign-extend bit
    // Check for 8-bit immediate, and set sign extend bit in opcode
    int con = (int)opnd_array(2)->constantL(); // Throw away top bits
    emit_opcode(cbuf, ((con >= -128) && (con <= 127)) ? ((0x81 /*primary()*/) | 0x02) : (0x81 /*primary()*/));
    // Emit r/m byte with secondary opcode, after primary opcode.
    emit_rm(cbuf, 0x3, (0x04 /*secondary()*/), opnd_array(1)->reg(ra_,this,idx1)/* dst */);
    if ((con >= -128) && (con <= 127)) emit_d8 (cbuf,con);
    else                               emit_d32(cbuf,con);
  
#line 24194 "ad_x86.cpp"
  }
  {

#line 1668 "src/hotspot/cpu/x86/x86_32.ad"

    // Emit primary opcode and set sign-extend bit
    // Check for 8-bit immediate, and set sign extend bit in opcode
    int con = (int)(opnd_array(2)->constantL() >> 32); // Throw away bottom bits
    emit_opcode(cbuf, ((con >= -128) && (con <= 127)) ? ((0x81 /*primary()*/) | 0x02) : (0x81 /*primary()*/));
    // Emit r/m byte with tertiary opcode, after primary opcode.
    emit_rm(cbuf, 0x3, (0x04 /*tertiary()*/), HIGH_FROM_LOW_ENC(opnd_array(1)->reg(ra_,this,idx1)/* dst */));
    if ((con >= -128) && (con <= 127)) emit_d8 (cbuf,con);
    else                               emit_d32(cbuf,con);
  
#line 24209 "ad_x86.cpp"
  }
}

void andL_eReg_memNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// mem
  {

#line 1544 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0x23 /*primary()*/));
  
#line 24225 "ad_x86.cpp"
  }
  {

#line 2096 "src/hotspot/cpu/x86/x86_32.ad"

    int reg_encoding = opnd_array(1)->reg(ra_,this,idx1)/* dst */;
    int base  = opnd_array(2)->base(ra_,this,idx2);
    int index = opnd_array(2)->index(ra_,this,idx2);
    int scale = opnd_array(2)->scale();
    int displace = opnd_array(2)->disp(ra_,this,idx2);
    relocInfo::relocType disp_reloc = opnd_array(2)->disp_reloc();
    encode_RegMem(cbuf, reg_encoding, base, index, scale, displace, disp_reloc);
  
#line 24239 "ad_x86.cpp"
  }
  {

#line 1549 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0x23 /*secondary()*/));
  
#line 24247 "ad_x86.cpp"
  }
  {

#line 2106 "src/hotspot/cpu/x86/x86_32.ad"

    int reg_encoding = HIGH_FROM_LOW_ENC(opnd_array(1)->reg(ra_,this,idx1)/* dst */);  // Hi register of pair, computed from lo
    int base  = opnd_array(2)->base(ra_,this,idx2);
    int index = opnd_array(2)->index(ra_,this,idx2);
    int scale = opnd_array(2)->scale();
    int displace = opnd_array(2)->disp(ra_,this,idx2) + 4;      // Offset is 4 further in memory
    assert( opnd_array(2)->disp_reloc() == relocInfo::none, "Cannot add 4 to oop" );
    encode_RegMem(cbuf, reg_encoding, base, index, scale, displace, relocInfo::none);
  
#line 24261 "ad_x86.cpp"
  }
}

void andL_eReg_mem_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// dst
  {

#line 1544 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0x23 /*primary()*/));
  
#line 24277 "ad_x86.cpp"
  }
  {

#line 2096 "src/hotspot/cpu/x86/x86_32.ad"

    int reg_encoding = opnd_array(2)->reg(ra_,this,idx2)/* dst */;
    int base  = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int displace = opnd_array(1)->disp(ra_,this,idx1);
    relocInfo::relocType disp_reloc = opnd_array(1)->disp_reloc();
    encode_RegMem(cbuf, reg_encoding, base, index, scale, displace, disp_reloc);
  
#line 24291 "ad_x86.cpp"
  }
  {

#line 1549 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0x23 /*secondary()*/));
  
#line 24299 "ad_x86.cpp"
  }
  {

#line 2106 "src/hotspot/cpu/x86/x86_32.ad"

    int reg_encoding = HIGH_FROM_LOW_ENC(opnd_array(2)->reg(ra_,this,idx2)/* dst */);  // Hi register of pair, computed from lo
    int base  = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int displace = opnd_array(1)->disp(ra_,this,idx1) + 4;      // Offset is 4 further in memory
    assert( opnd_array(1)->disp_reloc() == relocInfo::none, "Cannot add 4 to oop" );
    encode_RegMem(cbuf, reg_encoding, base, index, scale, displace, relocInfo::none);
  
#line 24313 "ad_x86.cpp"
  }
}

void andnL_eReg_eReg_eRegNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// minus_1
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// src2
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// dst
  {
    C2_MacroAssembler _masm(&cbuf);

#line 9005 "src/hotspot/cpu/x86/x86_32.ad"

    Register Rdst = opnd_array(4)->as_Register(ra_,this,idx4)/* dst */;
    Register Rsrc1 = opnd_array(1)->as_Register(ra_,this,idx1)/* src1 */;
    Register Rsrc2 = opnd_array(3)->as_Register(ra_,this,idx3)/* src2 */;
    __ andnl(Rdst, Rsrc1, Rsrc2);
    __ andnl(HIGH_FROM_LOW(Rdst), HIGH_FROM_LOW(Rsrc1), HIGH_FROM_LOW(Rsrc2));
  
#line 24336 "ad_x86.cpp"
  }
}

void andnL_eReg_eReg_eReg_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src2
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src1
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// minus_1
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// dst
  {
    C2_MacroAssembler _masm(&cbuf);

#line 9005 "src/hotspot/cpu/x86/x86_32.ad"

    Register Rdst = opnd_array(4)->as_Register(ra_,this,idx4)/* dst */;
    Register Rsrc1 = opnd_array(2)->as_Register(ra_,this,idx2)/* src1 */;
    Register Rsrc2 = opnd_array(1)->as_Register(ra_,this,idx1)/* src2 */;
    __ andnl(Rdst, Rsrc1, Rsrc2);
    __ andnl(HIGH_FROM_LOW(Rdst), HIGH_FROM_LOW(Rsrc1), HIGH_FROM_LOW(Rsrc2));
  
#line 24359 "ad_x86.cpp"
  }
}

void andnL_eReg_eReg_memNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// minus_1
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// src2
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// dst
  {
    C2_MacroAssembler _masm(&cbuf);

#line 9025 "src/hotspot/cpu/x86/x86_32.ad"

    Register Rdst = opnd_array(4)->as_Register(ra_,this,idx4)/* dst */;
    Register Rsrc1 = opnd_array(1)->as_Register(ra_,this,idx1)/* src1 */;
    Address src2_hi = Address::make_raw(opnd_array(3)->base(ra_,this,idx3), opnd_array(3)->index(ra_,this,idx3), opnd_array(3)->scale(), opnd_array(3)->disp(ra_,this,idx3)+ 4, relocInfo::none);

    __ andnl(Rdst, Rsrc1, Address::make_raw(opnd_array(3)->base(ra_,this,idx3), opnd_array(3)->index(ra_,this,idx3), opnd_array(3)->scale(), opnd_array(3)->disp(ra_,this,idx3), opnd_array(3)->disp_reloc()));
    __ andnl(HIGH_FROM_LOW(Rdst), HIGH_FROM_LOW(Rsrc1), src2_hi);
  
#line 24383 "ad_x86.cpp"
  }
}

void andnL_eReg_eReg_mem_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// src2
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src1
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// minus_1
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// dst
  {
    C2_MacroAssembler _masm(&cbuf);

#line 9025 "src/hotspot/cpu/x86/x86_32.ad"

    Register Rdst = opnd_array(4)->as_Register(ra_,this,idx4)/* dst */;
    Register Rsrc1 = opnd_array(2)->as_Register(ra_,this,idx2)/* src1 */;
    Address src2_hi = Address::make_raw(opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1)+ 4, relocInfo::none);

    __ andnl(Rdst, Rsrc1, Address::make_raw(opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1), opnd_array(1)->disp_reloc()));
    __ andnl(HIGH_FROM_LOW(Rdst), HIGH_FROM_LOW(Rsrc1), src2_hi);
  
#line 24407 "ad_x86.cpp"
  }
}

void blsiL_eReg_eRegNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// imm_zero
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// dst
  {
    C2_MacroAssembler _masm(&cbuf);

#line 9048 "src/hotspot/cpu/x86/x86_32.ad"

    Label done;
    Register Rdst = opnd_array(3)->as_Register(ra_,this,idx3)/* dst */;
    Register Rsrc = opnd_array(2)->as_Register(ra_,this,idx2)/* src */;
    __ movl(HIGH_FROM_LOW(Rdst), 0);
    __ blsil(Rdst, Rsrc);
    __ jccb(Assembler::notZero, done);
    __ blsil(HIGH_FROM_LOW(Rdst), HIGH_FROM_LOW(Rsrc));
    __ bind(done);
  
#line 24432 "ad_x86.cpp"
  }
}

void blsiL_eReg_eReg_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// imm_zero
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// dst
  {
    C2_MacroAssembler _masm(&cbuf);

#line 9048 "src/hotspot/cpu/x86/x86_32.ad"

    Label done;
    Register Rdst = opnd_array(3)->as_Register(ra_,this,idx3)/* dst */;
    Register Rsrc = opnd_array(1)->as_Register(ra_,this,idx1)/* src */;
    __ movl(HIGH_FROM_LOW(Rdst), 0);
    __ blsil(Rdst, Rsrc);
    __ jccb(Assembler::notZero, done);
    __ blsil(HIGH_FROM_LOW(Rdst), HIGH_FROM_LOW(Rsrc));
    __ bind(done);
  
#line 24457 "ad_x86.cpp"
  }
}

void blsiL_eReg_memNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// imm_zero
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// dst
  {
    C2_MacroAssembler _masm(&cbuf);

#line 9074 "src/hotspot/cpu/x86/x86_32.ad"

    Label done;
    Register Rdst = opnd_array(3)->as_Register(ra_,this,idx3)/* dst */;
    Address src_hi = Address::make_raw(opnd_array(2)->base(ra_,this,idx2), opnd_array(2)->index(ra_,this,idx2), opnd_array(2)->scale(), opnd_array(2)->disp(ra_,this,idx2)+ 4, relocInfo::none);

    __ movl(HIGH_FROM_LOW(Rdst), 0);
    __ blsil(Rdst, Address::make_raw(opnd_array(2)->base(ra_,this,idx2), opnd_array(2)->index(ra_,this,idx2), opnd_array(2)->scale(), opnd_array(2)->disp(ra_,this,idx2), opnd_array(2)->disp_reloc()));
    __ jccb(Assembler::notZero, done);
    __ blsil(HIGH_FROM_LOW(Rdst), src_hi);
    __ bind(done);
  
#line 24483 "ad_x86.cpp"
  }
}

void blsiL_eReg_mem_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// imm_zero
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// dst
  {
    C2_MacroAssembler _masm(&cbuf);

#line 9074 "src/hotspot/cpu/x86/x86_32.ad"

    Label done;
    Register Rdst = opnd_array(3)->as_Register(ra_,this,idx3)/* dst */;
    Address src_hi = Address::make_raw(opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1)+ 4, relocInfo::none);

    __ movl(HIGH_FROM_LOW(Rdst), 0);
    __ blsil(Rdst, Address::make_raw(opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1), opnd_array(1)->disp_reloc()));
    __ jccb(Assembler::notZero, done);
    __ blsil(HIGH_FROM_LOW(Rdst), src_hi);
    __ bind(done);
  
#line 24509 "ad_x86.cpp"
  }
}

void blsmskL_eReg_eRegNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// minus_1
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// dst
  {
    C2_MacroAssembler _masm(&cbuf);

#line 9101 "src/hotspot/cpu/x86/x86_32.ad"

    Label done;
    Register Rdst = opnd_array(3)->as_Register(ra_,this,idx3)/* dst */;
    Register Rsrc = opnd_array(1)->as_Register(ra_,this,idx1)/* src */;
    __ movl(HIGH_FROM_LOW(Rdst), 0);
    __ blsmskl(Rdst, Rsrc);
    __ jccb(Assembler::carryClear, done);
    __ blsmskl(HIGH_FROM_LOW(Rdst), HIGH_FROM_LOW(Rsrc));
    __ bind(done);
  
#line 24534 "ad_x86.cpp"
  }
}

void blsmskL_eReg_eReg_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// minus_1
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// dst
  {
    C2_MacroAssembler _masm(&cbuf);

#line 9101 "src/hotspot/cpu/x86/x86_32.ad"

    Label done;
    Register Rdst = opnd_array(3)->as_Register(ra_,this,idx3)/* dst */;
    Register Rsrc = opnd_array(1)->as_Register(ra_,this,idx1)/* src */;
    __ movl(HIGH_FROM_LOW(Rdst), 0);
    __ blsmskl(Rdst, Rsrc);
    __ jccb(Assembler::carryClear, done);
    __ blsmskl(HIGH_FROM_LOW(Rdst), HIGH_FROM_LOW(Rsrc));
    __ bind(done);
  
#line 24559 "ad_x86.cpp"
  }
}

void blsmskL_eReg_memNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// minus_1
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// dst
  {
    C2_MacroAssembler _masm(&cbuf);

#line 9129 "src/hotspot/cpu/x86/x86_32.ad"

    Label done;
    Register Rdst = opnd_array(3)->as_Register(ra_,this,idx3)/* dst */;
    Address src_hi = Address::make_raw(opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1)+ 4, relocInfo::none);

    __ movl(HIGH_FROM_LOW(Rdst), 0);
    __ blsmskl(Rdst, Address::make_raw(opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1), opnd_array(1)->disp_reloc()));
    __ jccb(Assembler::carryClear, done);
    __ blsmskl(HIGH_FROM_LOW(Rdst), src_hi);
    __ bind(done);
  
#line 24585 "ad_x86.cpp"
  }
}

void blsmskL_eReg_mem_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// minus_1
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// dst
  {
    C2_MacroAssembler _masm(&cbuf);

#line 9129 "src/hotspot/cpu/x86/x86_32.ad"

    Label done;
    Register Rdst = opnd_array(3)->as_Register(ra_,this,idx3)/* dst */;
    Address src_hi = Address::make_raw(opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1)+ 4, relocInfo::none);

    __ movl(HIGH_FROM_LOW(Rdst), 0);
    __ blsmskl(Rdst, Address::make_raw(opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1), opnd_array(1)->disp_reloc()));
    __ jccb(Assembler::carryClear, done);
    __ blsmskl(HIGH_FROM_LOW(Rdst), src_hi);
    __ bind(done);
  
#line 24611 "ad_x86.cpp"
  }
}

void blsrL_eReg_eRegNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// minus_1
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// dst
  {
    C2_MacroAssembler _masm(&cbuf);

#line 9157 "src/hotspot/cpu/x86/x86_32.ad"

    Label done;
    Register Rdst = opnd_array(3)->as_Register(ra_,this,idx3)/* dst */;
    Register Rsrc = opnd_array(1)->as_Register(ra_,this,idx1)/* src */;
    __ movl(HIGH_FROM_LOW(Rdst), HIGH_FROM_LOW(Rsrc));
    __ blsrl(Rdst, Rsrc);
    __ jccb(Assembler::carryClear, done);
    __ blsrl(HIGH_FROM_LOW(Rdst), HIGH_FROM_LOW(Rsrc));
    __ bind(done);
  
#line 24636 "ad_x86.cpp"
  }
}

void blsrL_eReg_eReg_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// minus_1
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// dst
  {
    C2_MacroAssembler _masm(&cbuf);

#line 9157 "src/hotspot/cpu/x86/x86_32.ad"

    Label done;
    Register Rdst = opnd_array(3)->as_Register(ra_,this,idx3)/* dst */;
    Register Rsrc = opnd_array(1)->as_Register(ra_,this,idx1)/* src */;
    __ movl(HIGH_FROM_LOW(Rdst), HIGH_FROM_LOW(Rsrc));
    __ blsrl(Rdst, Rsrc);
    __ jccb(Assembler::carryClear, done);
    __ blsrl(HIGH_FROM_LOW(Rdst), HIGH_FROM_LOW(Rsrc));
    __ bind(done);
  
#line 24661 "ad_x86.cpp"
  }
}

void blsrL_eReg_memNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// minus_1
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// dst
  {
    C2_MacroAssembler _masm(&cbuf);

#line 9185 "src/hotspot/cpu/x86/x86_32.ad"

    Label done;
    Register Rdst = opnd_array(3)->as_Register(ra_,this,idx3)/* dst */;
    Address src_hi = Address::make_raw(opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1)+ 4, relocInfo::none);
    __ movl(HIGH_FROM_LOW(Rdst), src_hi);
    __ blsrl(Rdst, Address::make_raw(opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1), opnd_array(1)->disp_reloc()));
    __ jccb(Assembler::carryClear, done);
    __ blsrl(HIGH_FROM_LOW(Rdst), src_hi);
    __ bind(done);
  
#line 24686 "ad_x86.cpp"
  }
}

void blsrL_eReg_mem_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// minus_1
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// dst
  {
    C2_MacroAssembler _masm(&cbuf);

#line 9185 "src/hotspot/cpu/x86/x86_32.ad"

    Label done;
    Register Rdst = opnd_array(3)->as_Register(ra_,this,idx3)/* dst */;
    Address src_hi = Address::make_raw(opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1)+ 4, relocInfo::none);
    __ movl(HIGH_FROM_LOW(Rdst), src_hi);
    __ blsrl(Rdst, Address::make_raw(opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1), opnd_array(1)->disp_reloc()));
    __ jccb(Assembler::carryClear, done);
    __ blsrl(HIGH_FROM_LOW(Rdst), src_hi);
    __ bind(done);
  
#line 24711 "ad_x86.cpp"
  }
}

void orl_eRegNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {

#line 1963 "src/hotspot/cpu/x86/x86_32.ad"

    emit_d8(cbuf, (0x0B /*primary()*/));
    emit_rm(cbuf, 0x3, opnd_array(1)->reg(ra_,this,idx1)/* dst */, opnd_array(2)->reg(ra_,this,idx2)/* src */);
  
#line 24728 "ad_x86.cpp"
  }
  {

#line 1968 "src/hotspot/cpu/x86/x86_32.ad"

    emit_d8(cbuf, (0x0B /*secondary()*/));
    emit_rm(cbuf, 0x3, HIGH_FROM_LOW_ENC(opnd_array(1)->reg(ra_,this,idx1)/* dst */), HIGH_FROM_LOW_ENC(opnd_array(2)->reg(ra_,this,idx2)/* src */));
  
#line 24737 "ad_x86.cpp"
  }
}

void orl_eReg_immNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {

#line 1657 "src/hotspot/cpu/x86/x86_32.ad"

    // Emit primary opcode and set sign-extend bit
    // Check for 8-bit immediate, and set sign extend bit in opcode
    int con = (int)opnd_array(2)->constantL(); // Throw away top bits
    emit_opcode(cbuf, ((con >= -128) && (con <= 127)) ? ((0x81 /*primary()*/) | 0x02) : (0x81 /*primary()*/));
    // Emit r/m byte with secondary opcode, after primary opcode.
    emit_rm(cbuf, 0x3, (0x01 /*secondary()*/), opnd_array(1)->reg(ra_,this,idx1)/* dst */);
    if ((con >= -128) && (con <= 127)) emit_d8 (cbuf,con);
    else                               emit_d32(cbuf,con);
  
#line 24760 "ad_x86.cpp"
  }
  {

#line 1668 "src/hotspot/cpu/x86/x86_32.ad"

    // Emit primary opcode and set sign-extend bit
    // Check for 8-bit immediate, and set sign extend bit in opcode
    int con = (int)(opnd_array(2)->constantL() >> 32); // Throw away bottom bits
    emit_opcode(cbuf, ((con >= -128) && (con <= 127)) ? ((0x81 /*primary()*/) | 0x02) : (0x81 /*primary()*/));
    // Emit r/m byte with tertiary opcode, after primary opcode.
    emit_rm(cbuf, 0x3, (0x01 /*tertiary()*/), HIGH_FROM_LOW_ENC(opnd_array(1)->reg(ra_,this,idx1)/* dst */));
    if ((con >= -128) && (con <= 127)) emit_d8 (cbuf,con);
    else                               emit_d32(cbuf,con);
  
#line 24775 "ad_x86.cpp"
  }
}

void orl_eReg_memNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// mem
  {

#line 1544 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0x0B /*primary()*/));
  
#line 24791 "ad_x86.cpp"
  }
  {

#line 2096 "src/hotspot/cpu/x86/x86_32.ad"

    int reg_encoding = opnd_array(1)->reg(ra_,this,idx1)/* dst */;
    int base  = opnd_array(2)->base(ra_,this,idx2);
    int index = opnd_array(2)->index(ra_,this,idx2);
    int scale = opnd_array(2)->scale();
    int displace = opnd_array(2)->disp(ra_,this,idx2);
    relocInfo::relocType disp_reloc = opnd_array(2)->disp_reloc();
    encode_RegMem(cbuf, reg_encoding, base, index, scale, displace, disp_reloc);
  
#line 24805 "ad_x86.cpp"
  }
  {

#line 1549 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0x0B /*secondary()*/));
  
#line 24813 "ad_x86.cpp"
  }
  {

#line 2106 "src/hotspot/cpu/x86/x86_32.ad"

    int reg_encoding = HIGH_FROM_LOW_ENC(opnd_array(1)->reg(ra_,this,idx1)/* dst */);  // Hi register of pair, computed from lo
    int base  = opnd_array(2)->base(ra_,this,idx2);
    int index = opnd_array(2)->index(ra_,this,idx2);
    int scale = opnd_array(2)->scale();
    int displace = opnd_array(2)->disp(ra_,this,idx2) + 4;      // Offset is 4 further in memory
    assert( opnd_array(2)->disp_reloc() == relocInfo::none, "Cannot add 4 to oop" );
    encode_RegMem(cbuf, reg_encoding, base, index, scale, displace, relocInfo::none);
  
#line 24827 "ad_x86.cpp"
  }
}

void orl_eReg_mem_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// dst
  {

#line 1544 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0x0B /*primary()*/));
  
#line 24843 "ad_x86.cpp"
  }
  {

#line 2096 "src/hotspot/cpu/x86/x86_32.ad"

    int reg_encoding = opnd_array(2)->reg(ra_,this,idx2)/* dst */;
    int base  = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int displace = opnd_array(1)->disp(ra_,this,idx1);
    relocInfo::relocType disp_reloc = opnd_array(1)->disp_reloc();
    encode_RegMem(cbuf, reg_encoding, base, index, scale, displace, disp_reloc);
  
#line 24857 "ad_x86.cpp"
  }
  {

#line 1549 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0x0B /*secondary()*/));
  
#line 24865 "ad_x86.cpp"
  }
  {

#line 2106 "src/hotspot/cpu/x86/x86_32.ad"

    int reg_encoding = HIGH_FROM_LOW_ENC(opnd_array(2)->reg(ra_,this,idx2)/* dst */);  // Hi register of pair, computed from lo
    int base  = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int displace = opnd_array(1)->disp(ra_,this,idx1) + 4;      // Offset is 4 further in memory
    assert( opnd_array(1)->disp_reloc() == relocInfo::none, "Cannot add 4 to oop" );
    encode_RegMem(cbuf, reg_encoding, base, index, scale, displace, relocInfo::none);
  
#line 24879 "ad_x86.cpp"
  }
}

void xorl_eRegNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {

#line 1963 "src/hotspot/cpu/x86/x86_32.ad"

    emit_d8(cbuf, (0x33 /*primary()*/));
    emit_rm(cbuf, 0x3, opnd_array(1)->reg(ra_,this,idx1)/* dst */, opnd_array(2)->reg(ra_,this,idx2)/* src */);
  
#line 24896 "ad_x86.cpp"
  }
  {

#line 1968 "src/hotspot/cpu/x86/x86_32.ad"

    emit_d8(cbuf, (0x33 /*secondary()*/));
    emit_rm(cbuf, 0x3, HIGH_FROM_LOW_ENC(opnd_array(1)->reg(ra_,this,idx1)/* dst */), HIGH_FROM_LOW_ENC(opnd_array(2)->reg(ra_,this,idx2)/* src */));
  
#line 24905 "ad_x86.cpp"
  }
}

void xorl_eReg_im1Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// imm
  {
    C2_MacroAssembler _masm(&cbuf);

#line 9249 "src/hotspot/cpu/x86/x86_32.ad"

     __ notl(opnd_array(1)->as_Register(ra_,this,idx1)/* dst */);
     __ notl(HIGH_FROM_LOW(opnd_array(1)->as_Register(ra_,this,idx1)/* dst */));
  
#line 24923 "ad_x86.cpp"
  }
}

void xorl_eReg_immNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {

#line 1657 "src/hotspot/cpu/x86/x86_32.ad"

    // Emit primary opcode and set sign-extend bit
    // Check for 8-bit immediate, and set sign extend bit in opcode
    int con = (int)opnd_array(2)->constantL(); // Throw away top bits
    emit_opcode(cbuf, ((con >= -128) && (con <= 127)) ? ((0x81 /*primary()*/) | 0x02) : (0x81 /*primary()*/));
    // Emit r/m byte with secondary opcode, after primary opcode.
    emit_rm(cbuf, 0x3, (0x06 /*secondary()*/), opnd_array(1)->reg(ra_,this,idx1)/* dst */);
    if ((con >= -128) && (con <= 127)) emit_d8 (cbuf,con);
    else                               emit_d32(cbuf,con);
  
#line 24946 "ad_x86.cpp"
  }
  {

#line 1668 "src/hotspot/cpu/x86/x86_32.ad"

    // Emit primary opcode and set sign-extend bit
    // Check for 8-bit immediate, and set sign extend bit in opcode
    int con = (int)(opnd_array(2)->constantL() >> 32); // Throw away bottom bits
    emit_opcode(cbuf, ((con >= -128) && (con <= 127)) ? ((0x81 /*primary()*/) | 0x02) : (0x81 /*primary()*/));
    // Emit r/m byte with tertiary opcode, after primary opcode.
    emit_rm(cbuf, 0x3, (0x06 /*tertiary()*/), HIGH_FROM_LOW_ENC(opnd_array(1)->reg(ra_,this,idx1)/* dst */));
    if ((con >= -128) && (con <= 127)) emit_d8 (cbuf,con);
    else                               emit_d32(cbuf,con);
  
#line 24961 "ad_x86.cpp"
  }
}

void xorl_eReg_memNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// mem
  {

#line 1544 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0x33 /*primary()*/));
  
#line 24977 "ad_x86.cpp"
  }
  {

#line 2096 "src/hotspot/cpu/x86/x86_32.ad"

    int reg_encoding = opnd_array(1)->reg(ra_,this,idx1)/* dst */;
    int base  = opnd_array(2)->base(ra_,this,idx2);
    int index = opnd_array(2)->index(ra_,this,idx2);
    int scale = opnd_array(2)->scale();
    int displace = opnd_array(2)->disp(ra_,this,idx2);
    relocInfo::relocType disp_reloc = opnd_array(2)->disp_reloc();
    encode_RegMem(cbuf, reg_encoding, base, index, scale, displace, disp_reloc);
  
#line 24991 "ad_x86.cpp"
  }
  {

#line 1549 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0x33 /*secondary()*/));
  
#line 24999 "ad_x86.cpp"
  }
  {

#line 2106 "src/hotspot/cpu/x86/x86_32.ad"

    int reg_encoding = HIGH_FROM_LOW_ENC(opnd_array(1)->reg(ra_,this,idx1)/* dst */);  // Hi register of pair, computed from lo
    int base  = opnd_array(2)->base(ra_,this,idx2);
    int index = opnd_array(2)->index(ra_,this,idx2);
    int scale = opnd_array(2)->scale();
    int displace = opnd_array(2)->disp(ra_,this,idx2) + 4;      // Offset is 4 further in memory
    assert( opnd_array(2)->disp_reloc() == relocInfo::none, "Cannot add 4 to oop" );
    encode_RegMem(cbuf, reg_encoding, base, index, scale, displace, relocInfo::none);
  
#line 25013 "ad_x86.cpp"
  }
}

void xorl_eReg_mem_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// dst
  {

#line 1544 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0x33 /*primary()*/));
  
#line 25029 "ad_x86.cpp"
  }
  {

#line 2096 "src/hotspot/cpu/x86/x86_32.ad"

    int reg_encoding = opnd_array(2)->reg(ra_,this,idx2)/* dst */;
    int base  = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int displace = opnd_array(1)->disp(ra_,this,idx1);
    relocInfo::relocType disp_reloc = opnd_array(1)->disp_reloc();
    encode_RegMem(cbuf, reg_encoding, base, index, scale, displace, disp_reloc);
  
#line 25043 "ad_x86.cpp"
  }
  {

#line 1549 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0x33 /*secondary()*/));
  
#line 25051 "ad_x86.cpp"
  }
  {

#line 2106 "src/hotspot/cpu/x86/x86_32.ad"

    int reg_encoding = HIGH_FROM_LOW_ENC(opnd_array(2)->reg(ra_,this,idx2)/* dst */);  // Hi register of pair, computed from lo
    int base  = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int displace = opnd_array(1)->disp(ra_,this,idx1) + 4;      // Offset is 4 further in memory
    assert( opnd_array(1)->disp_reloc() == relocInfo::none, "Cannot add 4 to oop" );
    encode_RegMem(cbuf, reg_encoding, base, index, scale, displace, relocInfo::none);
  
#line 25065 "ad_x86.cpp"
  }
}

void shlL_eReg_1Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// cnt
  {
    C2_MacroAssembler _masm(&cbuf);

#line 9287 "src/hotspot/cpu/x86/x86_32.ad"

    __ addl(opnd_array(1)->as_Register(ra_,this,idx1)/* dst */,opnd_array(1)->as_Register(ra_,this,idx1)/* dst */);
    __ adcl(HIGH_FROM_LOW(opnd_array(1)->as_Register(ra_,this,idx1)/* dst */),HIGH_FROM_LOW(opnd_array(1)->as_Register(ra_,this,idx1)/* dst */));
  
#line 25083 "ad_x86.cpp"
  }
}

void shlL_eReg_2Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// cnt
  {
    C2_MacroAssembler _masm(&cbuf);

#line 9304 "src/hotspot/cpu/x86/x86_32.ad"

    __ addl(opnd_array(1)->as_Register(ra_,this,idx1)/* dst */,opnd_array(1)->as_Register(ra_,this,idx1)/* dst */);
    __ adcl(HIGH_FROM_LOW(opnd_array(1)->as_Register(ra_,this,idx1)/* dst */),HIGH_FROM_LOW(opnd_array(1)->as_Register(ra_,this,idx1)/* dst */));
    __ addl(opnd_array(1)->as_Register(ra_,this,idx1)/* dst */,opnd_array(1)->as_Register(ra_,this,idx1)/* dst */);
    __ adcl(HIGH_FROM_LOW(opnd_array(1)->as_Register(ra_,this,idx1)/* dst */),HIGH_FROM_LOW(opnd_array(1)->as_Register(ra_,this,idx1)/* dst */));
  
#line 25103 "ad_x86.cpp"
  }
}

void shlL_eReg_3Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// cnt
  {
    C2_MacroAssembler _masm(&cbuf);

#line 9325 "src/hotspot/cpu/x86/x86_32.ad"

    __ addl(opnd_array(1)->as_Register(ra_,this,idx1)/* dst */,opnd_array(1)->as_Register(ra_,this,idx1)/* dst */);
    __ adcl(HIGH_FROM_LOW(opnd_array(1)->as_Register(ra_,this,idx1)/* dst */),HIGH_FROM_LOW(opnd_array(1)->as_Register(ra_,this,idx1)/* dst */));
    __ addl(opnd_array(1)->as_Register(ra_,this,idx1)/* dst */,opnd_array(1)->as_Register(ra_,this,idx1)/* dst */);
    __ adcl(HIGH_FROM_LOW(opnd_array(1)->as_Register(ra_,this,idx1)/* dst */),HIGH_FROM_LOW(opnd_array(1)->as_Register(ra_,this,idx1)/* dst */));
    __ addl(opnd_array(1)->as_Register(ra_,this,idx1)/* dst */,opnd_array(1)->as_Register(ra_,this,idx1)/* dst */);
    __ adcl(HIGH_FROM_LOW(opnd_array(1)->as_Register(ra_,this,idx1)/* dst */),HIGH_FROM_LOW(opnd_array(1)->as_Register(ra_,this,idx1)/* dst */));
  
#line 25125 "ad_x86.cpp"
  }
}

void shlL_eReg_1_31Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// cnt
  {

#line 2116 "src/hotspot/cpu/x86/x86_32.ad"

    int r1, r2;
    if( (0xA4 /*tertiary()*/) == 0xA4 ) { r1 = opnd_array(1)->reg(ra_,this,idx1)/* dst */;  r2 = HIGH_FROM_LOW_ENC(opnd_array(1)->reg(ra_,this,idx1)/* dst */); }
    else                    { r2 = opnd_array(1)->reg(ra_,this,idx1)/* dst */;  r1 = HIGH_FROM_LOW_ENC(opnd_array(1)->reg(ra_,this,idx1)/* dst */); }
    emit_opcode(cbuf,0x0F);
    emit_opcode(cbuf,(0xA4 /*tertiary()*/));
    emit_rm(cbuf, 0x3, r1, r2);
    emit_d8(cbuf,opnd_array(2)->constant());
    emit_d8(cbuf,(0xC1 /*primary()*/));
    emit_rm(cbuf, 0x3, (0x4 /*secondary()*/), r1);
    emit_d8(cbuf,opnd_array(2)->constant());
  
#line 25150 "ad_x86.cpp"
  }
}

void shlL_eReg_32_63Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// cnt
  {

#line 2142 "src/hotspot/cpu/x86/x86_32.ad"

    int r1, r2;
    if( (0x4 /*secondary()*/) == 0x5 ) { r1 = opnd_array(1)->reg(ra_,this,idx1)/* dst */;  r2 = HIGH_FROM_LOW_ENC(opnd_array(1)->reg(ra_,this,idx1)/* dst */); }
    else                    { r2 = opnd_array(1)->reg(ra_,this,idx1)/* dst */;  r1 = HIGH_FROM_LOW_ENC(opnd_array(1)->reg(ra_,this,idx1)/* dst */); }

    emit_opcode( cbuf, 0x8B ); // Move r1,r2
    emit_rm(cbuf, 0x3, r1, r2);
    if( opnd_array(2)->constant() > 32 ) { // Shift, if not by zero
      emit_opcode(cbuf,(0xC1 /*primary()*/));
      emit_rm(cbuf, 0x3, (0x4 /*secondary()*/), r1);
      emit_d8(cbuf,opnd_array(2)->constant()-32);
    }
    emit_opcode(cbuf,0x33);  // XOR r2,r2
    emit_rm(cbuf, 0x3, r2, r2);
  
#line 25178 "ad_x86.cpp"
  }
}

void salL_eReg_CLNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// shift
  {

#line 2279 "src/hotspot/cpu/x86/x86_32.ad"

    // TEST shift,32
    emit_opcode(cbuf,0xF7);
    emit_rm(cbuf, 0x3, 0, ECX_enc);
    emit_d32(cbuf,0x20);
    // JEQ,s small
    emit_opcode(cbuf, 0x74);
    emit_d8(cbuf, 0x04);
    // MOV    $dst.hi,$dst.lo
    emit_opcode( cbuf, 0x8B );
    emit_rm(cbuf, 0x3, HIGH_FROM_LOW_ENC(opnd_array(1)->reg(ra_,this,idx1)/* dst */), opnd_array(1)->reg(ra_,this,idx1)/* dst */ );
    // CLR    $dst.lo
    emit_opcode(cbuf, 0x33);
    emit_rm(cbuf, 0x3, opnd_array(1)->reg(ra_,this,idx1)/* dst */, opnd_array(1)->reg(ra_,this,idx1)/* dst */);
// small:
    // SHLD   $dst.hi,$dst.lo,$shift
    emit_opcode(cbuf,0x0F);
    emit_opcode(cbuf,0xA5);
    emit_rm(cbuf, 0x3, opnd_array(1)->reg(ra_,this,idx1)/* dst */, HIGH_FROM_LOW_ENC(opnd_array(1)->reg(ra_,this,idx1)/* dst */));
    // SHL    $dst.lo,$shift"
    emit_opcode(cbuf,0xD3);
    emit_rm(cbuf, 0x3, 0x4, opnd_array(1)->reg(ra_,this,idx1)/* dst */ );
  
#line 25214 "ad_x86.cpp"
  }
}

uint salL_eReg_CLNode::size(PhaseRegAlloc *ra_) const {
  assert(VerifyOops || MachNode::size(ra_) <= 17, "bad fixed size");
  return (VerifyOops ? MachNode::size(ra_) : 17);
}

void shrL_eReg_1_31Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// cnt
  {

#line 2116 "src/hotspot/cpu/x86/x86_32.ad"

    int r1, r2;
    if( (0xAC /*tertiary()*/) == 0xA4 ) { r1 = opnd_array(1)->reg(ra_,this,idx1)/* dst */;  r2 = HIGH_FROM_LOW_ENC(opnd_array(1)->reg(ra_,this,idx1)/* dst */); }
    else                    { r2 = opnd_array(1)->reg(ra_,this,idx1)/* dst */;  r1 = HIGH_FROM_LOW_ENC(opnd_array(1)->reg(ra_,this,idx1)/* dst */); }
    emit_opcode(cbuf,0x0F);
    emit_opcode(cbuf,(0xAC /*tertiary()*/));
    emit_rm(cbuf, 0x3, r1, r2);
    emit_d8(cbuf,opnd_array(2)->constant());
    emit_d8(cbuf,(0xC1 /*primary()*/));
    emit_rm(cbuf, 0x3, (0x5 /*secondary()*/), r1);
    emit_d8(cbuf,opnd_array(2)->constant());
  
#line 25244 "ad_x86.cpp"
  }
}

void shrL_eReg_32_63Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// cnt
  {

#line 2142 "src/hotspot/cpu/x86/x86_32.ad"

    int r1, r2;
    if( (0x5 /*secondary()*/) == 0x5 ) { r1 = opnd_array(1)->reg(ra_,this,idx1)/* dst */;  r2 = HIGH_FROM_LOW_ENC(opnd_array(1)->reg(ra_,this,idx1)/* dst */); }
    else                    { r2 = opnd_array(1)->reg(ra_,this,idx1)/* dst */;  r1 = HIGH_FROM_LOW_ENC(opnd_array(1)->reg(ra_,this,idx1)/* dst */); }

    emit_opcode( cbuf, 0x8B ); // Move r1,r2
    emit_rm(cbuf, 0x3, r1, r2);
    if( opnd_array(2)->constant() > 32 ) { // Shift, if not by zero
      emit_opcode(cbuf,(0xC1 /*primary()*/));
      emit_rm(cbuf, 0x3, (0x5 /*secondary()*/), r1);
      emit_d8(cbuf,opnd_array(2)->constant()-32);
    }
    emit_opcode(cbuf,0x33);  // XOR r2,r2
    emit_rm(cbuf, 0x3, r2, r2);
  
#line 25272 "ad_x86.cpp"
  }
}

void shrL_eReg_CLNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// shift
  {

#line 2303 "src/hotspot/cpu/x86/x86_32.ad"

    // TEST shift,32
    emit_opcode(cbuf,0xF7);
    emit_rm(cbuf, 0x3, 0, ECX_enc);
    emit_d32(cbuf,0x20);
    // JEQ,s small
    emit_opcode(cbuf, 0x74);
    emit_d8(cbuf, 0x04);
    // MOV    $dst.lo,$dst.hi
    emit_opcode( cbuf, 0x8B );
    emit_rm(cbuf, 0x3, opnd_array(1)->reg(ra_,this,idx1)/* dst */, HIGH_FROM_LOW_ENC(opnd_array(1)->reg(ra_,this,idx1)/* dst */) );
    // CLR    $dst.hi
    emit_opcode(cbuf, 0x33);
    emit_rm(cbuf, 0x3, HIGH_FROM_LOW_ENC(opnd_array(1)->reg(ra_,this,idx1)/* dst */), HIGH_FROM_LOW_ENC(opnd_array(1)->reg(ra_,this,idx1)/* dst */));
// small:
    // SHRD   $dst.lo,$dst.hi,$shift
    emit_opcode(cbuf,0x0F);
    emit_opcode(cbuf,0xAD);
    emit_rm(cbuf, 0x3, HIGH_FROM_LOW_ENC(opnd_array(1)->reg(ra_,this,idx1)/* dst */), opnd_array(1)->reg(ra_,this,idx1)/* dst */);
    // SHR    $dst.hi,$shift"
    emit_opcode(cbuf,0xD3);
    emit_rm(cbuf, 0x3, 0x5, HIGH_FROM_LOW_ENC(opnd_array(1)->reg(ra_,this,idx1)/* dst */) );
  
#line 25308 "ad_x86.cpp"
  }
}

uint shrL_eReg_CLNode::size(PhaseRegAlloc *ra_) const {
  assert(VerifyOops || MachNode::size(ra_) <= 17, "bad fixed size");
  return (VerifyOops ? MachNode::size(ra_) : 17);
}

void sarL_eReg_1_31Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// cnt
  {

#line 2116 "src/hotspot/cpu/x86/x86_32.ad"

    int r1, r2;
    if( (0xAC /*tertiary()*/) == 0xA4 ) { r1 = opnd_array(1)->reg(ra_,this,idx1)/* dst */;  r2 = HIGH_FROM_LOW_ENC(opnd_array(1)->reg(ra_,this,idx1)/* dst */); }
    else                    { r2 = opnd_array(1)->reg(ra_,this,idx1)/* dst */;  r1 = HIGH_FROM_LOW_ENC(opnd_array(1)->reg(ra_,this,idx1)/* dst */); }
    emit_opcode(cbuf,0x0F);
    emit_opcode(cbuf,(0xAC /*tertiary()*/));
    emit_rm(cbuf, 0x3, r1, r2);
    emit_d8(cbuf,opnd_array(2)->constant());
    emit_d8(cbuf,(0xC1 /*primary()*/));
    emit_rm(cbuf, 0x3, (0x7 /*secondary()*/), r1);
    emit_d8(cbuf,opnd_array(2)->constant());
  
#line 25338 "ad_x86.cpp"
  }
}

void sarL_eReg_32_63Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// cnt
  {

#line 2129 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode( cbuf, 0x8B ); // Move
    emit_rm(cbuf, 0x3, opnd_array(1)->reg(ra_,this,idx1)/* dst */, HIGH_FROM_LOW_ENC(opnd_array(1)->reg(ra_,this,idx1)/* dst */));
    if( opnd_array(2)->constant() > 32 ) { // Shift, if not by zero
      emit_d8(cbuf,(0xC1 /*primary()*/));
      emit_rm(cbuf, 0x3, (0x7 /*secondary()*/), opnd_array(1)->reg(ra_,this,idx1)/* dst */);
      emit_d8(cbuf,opnd_array(2)->constant()-32);
    }
    emit_d8(cbuf,(0xC1 /*primary()*/));
    emit_rm(cbuf, 0x3, (0x7 /*secondary()*/), HIGH_FROM_LOW_ENC(opnd_array(1)->reg(ra_,this,idx1)/* dst */));
    emit_d8(cbuf,31);
  
#line 25363 "ad_x86.cpp"
  }
}

void sarL_eReg_CLNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// shift
  {

#line 2327 "src/hotspot/cpu/x86/x86_32.ad"

    // TEST shift,32
    emit_opcode(cbuf,0xF7);
    emit_rm(cbuf, 0x3, 0, ECX_enc);
    emit_d32(cbuf,0x20);
    // JEQ,s small
    emit_opcode(cbuf, 0x74);
    emit_d8(cbuf, 0x05);
    // MOV    $dst.lo,$dst.hi
    emit_opcode( cbuf, 0x8B );
    emit_rm(cbuf, 0x3, opnd_array(1)->reg(ra_,this,idx1)/* dst */, HIGH_FROM_LOW_ENC(opnd_array(1)->reg(ra_,this,idx1)/* dst */) );
    // SAR    $dst.hi,31
    emit_opcode(cbuf, 0xC1);
    emit_rm(cbuf, 0x3, 7, HIGH_FROM_LOW_ENC(opnd_array(1)->reg(ra_,this,idx1)/* dst */) );
    emit_d8(cbuf, 0x1F );
// small:
    // SHRD   $dst.lo,$dst.hi,$shift
    emit_opcode(cbuf,0x0F);
    emit_opcode(cbuf,0xAD);
    emit_rm(cbuf, 0x3, HIGH_FROM_LOW_ENC(opnd_array(1)->reg(ra_,this,idx1)/* dst */), opnd_array(1)->reg(ra_,this,idx1)/* dst */);
    // SAR    $dst.hi,$shift"
    emit_opcode(cbuf,0xD3);
    emit_rm(cbuf, 0x3, 0x7, HIGH_FROM_LOW_ENC(opnd_array(1)->reg(ra_,this,idx1)/* dst */) );
  
#line 25400 "ad_x86.cpp"
  }
}

uint sarL_eReg_CLNode::size(PhaseRegAlloc *ra_) const {
  assert(VerifyOops || MachNode::size(ra_) <= 18, "bad fixed size");
  return (VerifyOops ? MachNode::size(ra_) : 18);
}

void cmpDPR_cc_P6Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {

#line 2372 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode( cbuf, 0xD9 );
    emit_d8( cbuf, 0xC0-1+opnd_array(1)->reg(ra_,this,idx1)/* src1 */ );   // FLD ST(i-1)
  
#line 25422 "ad_x86.cpp"
  }
  {

#line 1544 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0xDF /*primary()*/));
  
#line 25430 "ad_x86.cpp"
  }
  {

#line 1697 "src/hotspot/cpu/x86/x86_32.ad"

    emit_rm(cbuf, 0x3, (0x05 /*secondary()*/), opnd_array(2)->reg(ra_,this,idx2)/* src2 */ );
  
#line 25438 "ad_x86.cpp"
  }
  {

#line 2614 "src/hotspot/cpu/x86/x86_32.ad"

    // Fixup the integer flags in case comparison involved a NaN
    //
    // JNP exit (no unordered comparison, P-flag is set by NaN)
    emit_opcode( cbuf, 0x7B );
    emit_d8    ( cbuf, 0x03 );
    // MOV AH,1 - treat as LT case (set carry flag)
    emit_opcode( cbuf, 0xB4 );
    emit_d8    ( cbuf, 0x01 );
    // SAHF
    emit_opcode( cbuf, 0x9E);
    // NOP     // target for branch to avoid branch to branch
    emit_opcode( cbuf, 0x90);
  
#line 25457 "ad_x86.cpp"
  }
}

void cmpDPR_cc_P6CFNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {

#line 2372 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode( cbuf, 0xD9 );
    emit_d8( cbuf, 0xC0-1+opnd_array(1)->reg(ra_,this,idx1)/* src1 */ );   // FLD ST(i-1)
  
#line 25474 "ad_x86.cpp"
  }
  {

#line 1544 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0xDF /*primary()*/));
  
#line 25482 "ad_x86.cpp"
  }
  {

#line 1697 "src/hotspot/cpu/x86/x86_32.ad"

    emit_rm(cbuf, 0x3, (0x05 /*secondary()*/), opnd_array(2)->reg(ra_,this,idx2)/* src2 */ );
  
#line 25490 "ad_x86.cpp"
  }
}

void cmpDPR_ccNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {

#line 2372 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode( cbuf, 0xD9 );
    emit_d8( cbuf, 0xC0-1+opnd_array(1)->reg(ra_,this,idx1)/* src1 */ );   // FLD ST(i-1)
  
#line 25507 "ad_x86.cpp"
  }
  {

#line 1544 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0xD8 /*primary()*/));
  
#line 25515 "ad_x86.cpp"
  }
  {

#line 1697 "src/hotspot/cpu/x86/x86_32.ad"

    emit_rm(cbuf, 0x3, (0x3 /*secondary()*/), opnd_array(2)->reg(ra_,this,idx2)/* src2 */ );
  
#line 25523 "ad_x86.cpp"
  }
  {

#line 2591 "src/hotspot/cpu/x86/x86_32.ad"

    // fnstsw_ax
    emit_opcode( cbuf, 0xDF);
    emit_opcode( cbuf, 0xE0);
    // test ax,0x0400
    emit_opcode( cbuf, 0x66 );   // operand-size prefix for 16-bit immediate
    emit_opcode( cbuf, 0xA9 );
    emit_d16   ( cbuf, 0x0400 );
    // // // This sequence works, but stalls for 12-16 cycles on PPro
    // // test rax,0x0400
    // emit_opcode( cbuf, 0xA9 );
    // emit_d32   ( cbuf, 0x00000400 );
    //
    // jz exit (no unordered comparison)
    emit_opcode( cbuf, 0x74 );
    emit_d8    ( cbuf, 0x02 );
    // mov ah,1 - treat as LT case (set carry flag)
    emit_opcode( cbuf, 0xB4 );
    emit_d8    ( cbuf, 0x01 );
    // sahf
    emit_opcode( cbuf, 0x9E);
  
#line 25550 "ad_x86.cpp"
  }
}

void cmpDPR_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// zero
  {

#line 2372 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode( cbuf, 0xD9 );
    emit_d8( cbuf, 0xC0-1+opnd_array(1)->reg(ra_,this,idx1)/* src1 */ );   // FLD ST(i-1)
  
#line 25567 "ad_x86.cpp"
  }
  {

#line 1549 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0xD9 /*secondary()*/));
  
#line 25575 "ad_x86.cpp"
  }
  {

#line 1544 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0xE4 /*primary()*/));
  
#line 25583 "ad_x86.cpp"
  }
  {

#line 2361 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode( cbuf, 0xDD );
    emit_d8( cbuf, 0xD8 );
  
#line 25592 "ad_x86.cpp"
  }
  {

#line 2644 "src/hotspot/cpu/x86/x86_32.ad"

    // fnstsw_ax();
    emit_opcode( cbuf, 0xDF);
    emit_opcode( cbuf, 0xE0);
    // sahf
    emit_opcode( cbuf, 0x9E);
    // movl(dst, nan_result);
    emit_opcode( cbuf, 0xB8 + opnd_array(0)->reg(ra_,this)/* dst */);
    emit_d32( cbuf, -1 );
    // jcc(Assembler::parity, exit);
    emit_opcode( cbuf, 0x7A );
    emit_d8    ( cbuf, 0x13 );
    // movl(dst, less_result);
    emit_opcode( cbuf, 0xB8 + opnd_array(0)->reg(ra_,this)/* dst */);
    emit_d32( cbuf, -1 );
    // jcc(Assembler::below, exit);
    emit_opcode( cbuf, 0x72 );
    emit_d8    ( cbuf, 0x0C );
    // movl(dst, equal_result);
    emit_opcode( cbuf, 0xB8 + opnd_array(0)->reg(ra_,this)/* dst */);
    emit_d32( cbuf, 0 );
    // jcc(Assembler::equal, exit);
    emit_opcode( cbuf, 0x74 );
    emit_d8    ( cbuf, 0x05 );
    // movl(dst, greater_result);
    emit_opcode( cbuf, 0xB8 + opnd_array(0)->reg(ra_,this)/* dst */);
    emit_d32( cbuf, 1 );
  
#line 25625 "ad_x86.cpp"
  }
}

void cmpDPR_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {

#line 2372 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode( cbuf, 0xD9 );
    emit_d8( cbuf, 0xC0-1+opnd_array(1)->reg(ra_,this,idx1)/* src1 */ );   // FLD ST(i-1)
  
#line 25642 "ad_x86.cpp"
  }
  {

#line 1544 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0xD8 /*primary()*/));
  
#line 25650 "ad_x86.cpp"
  }
  {

#line 1697 "src/hotspot/cpu/x86/x86_32.ad"

    emit_rm(cbuf, 0x3, (0x3 /*secondary()*/), opnd_array(2)->reg(ra_,this,idx2)/* src2 */ );
  
#line 25658 "ad_x86.cpp"
  }
  {

#line 2644 "src/hotspot/cpu/x86/x86_32.ad"

    // fnstsw_ax();
    emit_opcode( cbuf, 0xDF);
    emit_opcode( cbuf, 0xE0);
    // sahf
    emit_opcode( cbuf, 0x9E);
    // movl(dst, nan_result);
    emit_opcode( cbuf, 0xB8 + opnd_array(0)->reg(ra_,this)/* dst */);
    emit_d32( cbuf, -1 );
    // jcc(Assembler::parity, exit);
    emit_opcode( cbuf, 0x7A );
    emit_d8    ( cbuf, 0x13 );
    // movl(dst, less_result);
    emit_opcode( cbuf, 0xB8 + opnd_array(0)->reg(ra_,this)/* dst */);
    emit_d32( cbuf, -1 );
    // jcc(Assembler::below, exit);
    emit_opcode( cbuf, 0x72 );
    emit_d8    ( cbuf, 0x0C );
    // movl(dst, equal_result);
    emit_opcode( cbuf, 0xB8 + opnd_array(0)->reg(ra_,this)/* dst */);
    emit_d32( cbuf, 0 );
    // jcc(Assembler::equal, exit);
    emit_opcode( cbuf, 0x74 );
    emit_d8    ( cbuf, 0x05 );
    // movl(dst, greater_result);
    emit_opcode( cbuf, 0xB8 + opnd_array(0)->reg(ra_,this)/* dst */);
    emit_d32( cbuf, 1 );
  
#line 25691 "ad_x86.cpp"
  }
}

void cmpD_ccNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 9555 "src/hotspot/cpu/x86/x86_32.ad"

    __ ucomisd(opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src1 */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src2 */);
    emit_cmpfp_fixup(_masm);
  
#line 25709 "ad_x86.cpp"
  }
}

void cmpD_ccCFNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 9567 "src/hotspot/cpu/x86/x86_32.ad"

    __ ucomisd(opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src1 */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src2 */);
  
#line 25726 "ad_x86.cpp"
  }
}

void cmpD_ccmemNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 9584 "src/hotspot/cpu/x86/x86_32.ad"

    __ ucomisd(opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src1 */, Address::make_raw(opnd_array(2)->base(ra_,this,idx2), opnd_array(2)->index(ra_,this,idx2), opnd_array(2)->scale(), opnd_array(2)->disp(ra_,this,idx2), opnd_array(2)->disp_reloc()));
    emit_cmpfp_fixup(_masm);
  
#line 25744 "ad_x86.cpp"
  }
}

void cmpD_ccmemCFNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 9596 "src/hotspot/cpu/x86/x86_32.ad"

    __ ucomisd(opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src1 */, Address::make_raw(opnd_array(2)->base(ra_,this,idx2), opnd_array(2)->index(ra_,this,idx2), opnd_array(2)->scale(), opnd_array(2)->disp(ra_,this,idx2), opnd_array(2)->disp_reloc()));
  
#line 25761 "ad_x86.cpp"
  }
}

void cmpD_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 9615 "src/hotspot/cpu/x86/x86_32.ad"

    __ ucomisd(opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src1 */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src2 */);
    emit_cmpfp3(_masm, opnd_array(0)->as_Register(ra_,this)/* dst */);
  
#line 25779 "ad_x86.cpp"
  }
}

void cmpD_regmemNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 9635 "src/hotspot/cpu/x86/x86_32.ad"

    __ ucomisd(opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src1 */, Address::make_raw(opnd_array(2)->base(ra_,this,idx2), opnd_array(2)->index(ra_,this,idx2), opnd_array(2)->scale(), opnd_array(2)->disp(ra_,this,idx2), opnd_array(2)->disp_reloc()));
    emit_cmpfp3(_masm, opnd_array(0)->as_Register(ra_,this)/* dst */);
  
#line 25797 "ad_x86.cpp"
  }
}

void subDPR_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {

#line 2372 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode( cbuf, 0xD9 );
    emit_d8( cbuf, 0xC0-1+opnd_array(2)->reg(ra_,this,idx2)/* src */ );   // FLD ST(i-1)
  
#line 25814 "ad_x86.cpp"
  }
  {

#line 1544 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0xDE /*primary()*/));
  
#line 25822 "ad_x86.cpp"
  }
  {

#line 1697 "src/hotspot/cpu/x86/x86_32.ad"

    emit_rm(cbuf, 0x3, (0x5 /*secondary()*/), opnd_array(1)->reg(ra_,this,idx1)/* dst */ );
  
#line 25830 "ad_x86.cpp"
  }
}

void subDPR_reg_roundNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {

#line 2372 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode( cbuf, 0xD9 );
    emit_d8( cbuf, 0xC0-1+opnd_array(2)->reg(ra_,this,idx2)/* src2 */ );   // FLD ST(i-1)
  
#line 25847 "ad_x86.cpp"
  }
  {

#line 1544 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0xD8 /*primary()*/));
  
#line 25855 "ad_x86.cpp"
  }
  {

#line 1697 "src/hotspot/cpu/x86/x86_32.ad"

    emit_rm(cbuf, 0x3, (0x5 /*secondary()*/), opnd_array(1)->reg(ra_,this,idx1)/* src1 */ );
  
#line 25863 "ad_x86.cpp"
  }
  {

#line 2418 "src/hotspot/cpu/x86/x86_32.ad"

    store_to_stackslot( cbuf, 0xDD, 0x03, opnd_array(0)->disp(ra_,this,0) );
  
#line 25871 "ad_x86.cpp"
  }
}

void subDPR_reg_memNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {

#line 1554 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0xDD /*tertiary()*/));
  
#line 25887 "ad_x86.cpp"
  }
  {

#line 2186 "src/hotspot/cpu/x86/x86_32.ad"

    int rm_byte_opcode = (0x00);
    int base     = opnd_array(2)->base(ra_,this,idx2);
    int index    = opnd_array(2)->index(ra_,this,idx2);
    int scale    = opnd_array(2)->scale();
    int displace = opnd_array(2)->disp(ra_,this,idx2);
    relocInfo::relocType disp_reloc = opnd_array(2)->disp_reloc(); // disp-as-oop when working with static globals
    encode_RegMem(cbuf, rm_byte_opcode, base, index, scale, displace, disp_reloc);
  
#line 25901 "ad_x86.cpp"
  }
  {

#line 1544 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0xDE /*primary()*/));
  
#line 25909 "ad_x86.cpp"
  }
  {

#line 1697 "src/hotspot/cpu/x86/x86_32.ad"

    emit_rm(cbuf, 0x3, (0x5 /*secondary()*/), opnd_array(1)->reg(ra_,this,idx1)/* dst */ );
  
#line 25917 "ad_x86.cpp"
  }
}

void absDPR_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {

#line 1549 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0xD9 /*secondary()*/));
  
#line 25932 "ad_x86.cpp"
  }
  {

#line 1544 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0xE1 /*primary()*/));
  
#line 25940 "ad_x86.cpp"
  }
}

void negDPR_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {

#line 1549 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0xD9 /*secondary()*/));
  
#line 25955 "ad_x86.cpp"
  }
  {

#line 1544 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0xE0 /*primary()*/));
  
#line 25963 "ad_x86.cpp"
  }
}

void addDPR_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {

#line 2372 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode( cbuf, 0xD9 );
    emit_d8( cbuf, 0xC0-1+opnd_array(2)->reg(ra_,this,idx2)/* src */ );   // FLD ST(i-1)
  
#line 25980 "ad_x86.cpp"
  }
  {

#line 1544 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0xDE /*primary()*/));
  
#line 25988 "ad_x86.cpp"
  }
  {

#line 1697 "src/hotspot/cpu/x86/x86_32.ad"

    emit_rm(cbuf, 0x3, (0x0 /*secondary()*/), opnd_array(1)->reg(ra_,this,idx1)/* dst */ );
  
#line 25996 "ad_x86.cpp"
  }
}

uint addDPR_regNode::size(PhaseRegAlloc *ra_) const {
  assert(VerifyOops || MachNode::size(ra_) <= 4, "bad fixed size");
  return (VerifyOops ? MachNode::size(ra_) : 4);
}

void addDPR_reg_roundNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {

#line 2372 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode( cbuf, 0xD9 );
    emit_d8( cbuf, 0xC0-1+opnd_array(2)->reg(ra_,this,idx2)/* src2 */ );   // FLD ST(i-1)
  
#line 26018 "ad_x86.cpp"
  }
  {

#line 1544 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0xD8 /*primary()*/));
  
#line 26026 "ad_x86.cpp"
  }
  {

#line 1697 "src/hotspot/cpu/x86/x86_32.ad"

    emit_rm(cbuf, 0x3, (0x0 /*secondary()*/), opnd_array(1)->reg(ra_,this,idx1)/* src1 */ );
  
#line 26034 "ad_x86.cpp"
  }
  {

#line 2418 "src/hotspot/cpu/x86/x86_32.ad"

    store_to_stackslot( cbuf, 0xDD, 0x03, opnd_array(0)->disp(ra_,this,0) );
  
#line 26042 "ad_x86.cpp"
  }
}

void addDPR_reg_memNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {

#line 1554 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0xDD /*tertiary()*/));
  
#line 26058 "ad_x86.cpp"
  }
  {

#line 2186 "src/hotspot/cpu/x86/x86_32.ad"

    int rm_byte_opcode = (0x00);
    int base     = opnd_array(2)->base(ra_,this,idx2);
    int index    = opnd_array(2)->index(ra_,this,idx2);
    int scale    = opnd_array(2)->scale();
    int displace = opnd_array(2)->disp(ra_,this,idx2);
    relocInfo::relocType disp_reloc = opnd_array(2)->disp_reloc(); // disp-as-oop when working with static globals
    encode_RegMem(cbuf, rm_byte_opcode, base, index, scale, displace, disp_reloc);
  
#line 26072 "ad_x86.cpp"
  }
  {

#line 1544 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0xDE /*primary()*/));
  
#line 26080 "ad_x86.cpp"
  }
  {

#line 1697 "src/hotspot/cpu/x86/x86_32.ad"

    emit_rm(cbuf, 0x3, (0x0 /*secondary()*/), opnd_array(1)->reg(ra_,this,idx1)/* dst */ );
  
#line 26088 "ad_x86.cpp"
  }
}

void addDPR_reg_mem_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// dst
  {

#line 1554 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0xDD /*tertiary()*/));
  
#line 26104 "ad_x86.cpp"
  }
  {

#line 2186 "src/hotspot/cpu/x86/x86_32.ad"

    int rm_byte_opcode = (0x00);
    int base     = opnd_array(1)->base(ra_,this,idx1);
    int index    = opnd_array(1)->index(ra_,this,idx1);
    int scale    = opnd_array(1)->scale();
    int displace = opnd_array(1)->disp(ra_,this,idx1);
    relocInfo::relocType disp_reloc = opnd_array(1)->disp_reloc(); // disp-as-oop when working with static globals
    encode_RegMem(cbuf, rm_byte_opcode, base, index, scale, displace, disp_reloc);
  
#line 26118 "ad_x86.cpp"
  }
  {

#line 1544 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0xDE /*primary()*/));
  
#line 26126 "ad_x86.cpp"
  }
  {

#line 1697 "src/hotspot/cpu/x86/x86_32.ad"

    emit_rm(cbuf, 0x3, (0x0 /*secondary()*/), opnd_array(2)->reg(ra_,this,idx2)/* dst */ );
  
#line 26134 "ad_x86.cpp"
  }
}

void addDPR_mem_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {

#line 1554 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0xDD));
  
#line 26150 "ad_x86.cpp"
  }
  {

#line 2186 "src/hotspot/cpu/x86/x86_32.ad"

    int rm_byte_opcode = (0x00);
    int base     = opnd_array(1)->base(ra_,this,idx1);
    int index    = opnd_array(1)->index(ra_,this,idx1);
    int scale    = opnd_array(1)->scale();
    int displace = opnd_array(1)->disp(ra_,this,idx1);
    relocInfo::relocType disp_reloc = opnd_array(1)->disp_reloc(); // disp-as-oop when working with static globals
    encode_RegMem(cbuf, rm_byte_opcode, base, index, scale, displace, disp_reloc);
  
#line 26164 "ad_x86.cpp"
  }
  {

#line 1554 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0xD8));
  
#line 26172 "ad_x86.cpp"
  }
  {

#line 1697 "src/hotspot/cpu/x86/x86_32.ad"

    emit_rm(cbuf, 0x3, (0x0 /*secondary()*/), opnd_array(2)->reg(ra_,this,idx2)/* src */ );
  
#line 26180 "ad_x86.cpp"
  }
  {

#line 2092 "src/hotspot/cpu/x86/x86_32.ad"

    cbuf.set_insts_mark();            // Mark start of opcode for reloc info in mem operand
  
#line 26188 "ad_x86.cpp"
  }
  {

#line 1554 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0xDD));
  
#line 26196 "ad_x86.cpp"
  }
  {

#line 2186 "src/hotspot/cpu/x86/x86_32.ad"

    int rm_byte_opcode = (0x03);
    int base     = opnd_array(1)->base(ra_,this,idx1);
    int index    = opnd_array(1)->index(ra_,this,idx1);
    int scale    = opnd_array(1)->scale();
    int displace = opnd_array(1)->disp(ra_,this,idx1);
    relocInfo::relocType disp_reloc = opnd_array(1)->disp_reloc(); // disp-as-oop when working with static globals
    encode_RegMem(cbuf, rm_byte_opcode, base, index, scale, displace, disp_reloc);
  
#line 26210 "ad_x86.cpp"
  }
}

void addDPR_mem_reg_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {

#line 1554 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0xDD));
  
#line 26226 "ad_x86.cpp"
  }
  {

#line 2186 "src/hotspot/cpu/x86/x86_32.ad"

    int rm_byte_opcode = (0x00);
    int base     = opnd_array(1)->base(ra_,this,idx1);
    int index    = opnd_array(1)->index(ra_,this,idx1);
    int scale    = opnd_array(1)->scale();
    int displace = opnd_array(1)->disp(ra_,this,idx1);
    relocInfo::relocType disp_reloc = opnd_array(1)->disp_reloc(); // disp-as-oop when working with static globals
    encode_RegMem(cbuf, rm_byte_opcode, base, index, scale, displace, disp_reloc);
  
#line 26240 "ad_x86.cpp"
  }
  {

#line 1554 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0xD8));
  
#line 26248 "ad_x86.cpp"
  }
  {

#line 1697 "src/hotspot/cpu/x86/x86_32.ad"

    emit_rm(cbuf, 0x3, (0x0 /*secondary()*/), opnd_array(2)->reg(ra_,this,idx2)/* src */ );
  
#line 26256 "ad_x86.cpp"
  }
  {

#line 2092 "src/hotspot/cpu/x86/x86_32.ad"

    cbuf.set_insts_mark();            // Mark start of opcode for reloc info in mem operand
  
#line 26264 "ad_x86.cpp"
  }
  {

#line 1554 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0xDD));
  
#line 26272 "ad_x86.cpp"
  }
  {

#line 2186 "src/hotspot/cpu/x86/x86_32.ad"

    int rm_byte_opcode = (0x03);
    int base     = opnd_array(1)->base(ra_,this,idx1);
    int index    = opnd_array(1)->index(ra_,this,idx1);
    int scale    = opnd_array(1)->scale();
    int displace = opnd_array(1)->disp(ra_,this,idx1);
    relocInfo::relocType disp_reloc = opnd_array(1)->disp_reloc(); // disp-as-oop when working with static globals
    encode_RegMem(cbuf, rm_byte_opcode, base, index, scale, displace, disp_reloc);
  
#line 26286 "ad_x86.cpp"
  }
}

void addDPR_reg_imm1Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// con
  {
    C2_MacroAssembler _masm(&cbuf);

#line 9769 "src/hotspot/cpu/x86/x86_32.ad"

    __ fld1();
    __ faddp(opnd_array(1)->reg(ra_,this,idx1)/* dst */);
  
#line 26304 "ad_x86.cpp"
  }
}

void addDPR_reg_immNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// con
  {
    C2_MacroAssembler _masm(&cbuf);

#line 9782 "src/hotspot/cpu/x86/x86_32.ad"

    __ fld_d(InternalAddress(__ code()->consts()->start() + constant_offset()));
    __ faddp(opnd_array(1)->reg(ra_,this,idx1)/* dst */);
  
#line 26322 "ad_x86.cpp"
  }
}

void addDPR_reg_immNode::eval_constant(Compile* C) {
  {

#line 9783 "src/hotspot/cpu/x86/x86_32.ad"
    _constant = C->output()->constant_table().add(this, opnd_array(2));
#line 26331 "ad_x86.cpp"
  }
}
void addDPR_reg_imm_roundNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// con
  {
    C2_MacroAssembler _masm(&cbuf);

#line 9796 "src/hotspot/cpu/x86/x86_32.ad"

    __ fld_d(InternalAddress(__ code()->consts()->start() + constant_offset()));
    __ fadd(opnd_array(1)->reg(ra_,this,idx1)/* src */);
    __ fstp_d(Address(rsp, opnd_array(0)->disp(ra_,this,0)));
  
#line 26349 "ad_x86.cpp"
  }
}

void addDPR_reg_imm_roundNode::eval_constant(Compile* C) {
  {

#line 9797 "src/hotspot/cpu/x86/x86_32.ad"
    _constant = C->output()->constant_table().add(this, opnd_array(2));
#line 26358 "ad_x86.cpp"
  }
}
void mulDPR_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {

#line 2372 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode( cbuf, 0xD9 );
    emit_d8( cbuf, 0xC0-1+opnd_array(2)->reg(ra_,this,idx2)/* src */ );   // FLD ST(i-1)
  
#line 26374 "ad_x86.cpp"
  }
  {

#line 1544 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0xDE /*primary()*/));
  
#line 26382 "ad_x86.cpp"
  }
  {

#line 1697 "src/hotspot/cpu/x86/x86_32.ad"

    emit_rm(cbuf, 0x3, (0x1 /*secondary()*/), opnd_array(1)->reg(ra_,this,idx1)/* dst */ );
  
#line 26390 "ad_x86.cpp"
  }
}

void strictfp_mulDPR_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {

#line 2377 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode( cbuf, 0xDB );           // FLD m80real
    emit_opcode( cbuf, 0x2D );
    emit_d32( cbuf, (int)StubRoutines::x86::addr_fpu_subnormal_bias1() );
    emit_opcode( cbuf, 0xDE );           // FMULP ST(dst), ST0
    emit_opcode( cbuf, 0xC8+opnd_array(1)->reg(ra_,this,idx1)/* dst */ );
  
#line 26410 "ad_x86.cpp"
  }
  {

#line 2372 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode( cbuf, 0xD9 );
    emit_d8( cbuf, 0xC0-1+opnd_array(2)->reg(ra_,this,idx2)/* src */ );   // FLD ST(i-1)
  
#line 26419 "ad_x86.cpp"
  }
  {

#line 1544 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0xDE /*primary()*/));
  
#line 26427 "ad_x86.cpp"
  }
  {

#line 1697 "src/hotspot/cpu/x86/x86_32.ad"

    emit_rm(cbuf, 0x3, (0x1 /*secondary()*/), opnd_array(1)->reg(ra_,this,idx1)/* dst */ );
  
#line 26435 "ad_x86.cpp"
  }
  {

#line 2385 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode( cbuf, 0xDB );           // FLD m80real
    emit_opcode( cbuf, 0x2D );
    emit_d32( cbuf, (int)StubRoutines::x86::addr_fpu_subnormal_bias2() );
    emit_opcode( cbuf, 0xDE );           // FMULP ST(dst), ST0
    emit_opcode( cbuf, 0xC8+opnd_array(1)->reg(ra_,this,idx1)/* dst */ );
  
#line 26447 "ad_x86.cpp"
  }
}

void mulDPR_reg_immNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// con
  {
    C2_MacroAssembler _masm(&cbuf);

#line 9849 "src/hotspot/cpu/x86/x86_32.ad"

    __ fld_d(InternalAddress(__ code()->consts()->start() + constant_offset()));
    __ fmulp(opnd_array(1)->reg(ra_,this,idx1)/* dst */);
  
#line 26465 "ad_x86.cpp"
  }
}

void mulDPR_reg_immNode::eval_constant(Compile* C) {
  {

#line 9850 "src/hotspot/cpu/x86/x86_32.ad"
    _constant = C->output()->constant_table().add(this, opnd_array(2));
#line 26474 "ad_x86.cpp"
  }
}
void mulDPR_reg_memNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {

#line 1554 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0xDD /*tertiary()*/));
  
#line 26489 "ad_x86.cpp"
  }
  {

#line 2186 "src/hotspot/cpu/x86/x86_32.ad"

    int rm_byte_opcode = (0x00);
    int base     = opnd_array(2)->base(ra_,this,idx2);
    int index    = opnd_array(2)->index(ra_,this,idx2);
    int scale    = opnd_array(2)->scale();
    int displace = opnd_array(2)->disp(ra_,this,idx2);
    relocInfo::relocType disp_reloc = opnd_array(2)->disp_reloc(); // disp-as-oop when working with static globals
    encode_RegMem(cbuf, rm_byte_opcode, base, index, scale, displace, disp_reloc);
  
#line 26503 "ad_x86.cpp"
  }
  {

#line 1544 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0xDE /*primary()*/));
  
#line 26511 "ad_x86.cpp"
  }
  {

#line 1697 "src/hotspot/cpu/x86/x86_32.ad"

    emit_rm(cbuf, 0x3, (0x1 /*secondary()*/), opnd_array(1)->reg(ra_,this,idx1)/* dst */ );
  
#line 26519 "ad_x86.cpp"
  }
}

void mulDPR_reg_mem_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// dst
  {

#line 1554 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0xDD /*tertiary()*/));
  
#line 26535 "ad_x86.cpp"
  }
  {

#line 2186 "src/hotspot/cpu/x86/x86_32.ad"

    int rm_byte_opcode = (0x00);
    int base     = opnd_array(1)->base(ra_,this,idx1);
    int index    = opnd_array(1)->index(ra_,this,idx1);
    int scale    = opnd_array(1)->scale();
    int displace = opnd_array(1)->disp(ra_,this,idx1);
    relocInfo::relocType disp_reloc = opnd_array(1)->disp_reloc(); // disp-as-oop when working with static globals
    encode_RegMem(cbuf, rm_byte_opcode, base, index, scale, displace, disp_reloc);
  
#line 26549 "ad_x86.cpp"
  }
  {

#line 1544 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0xDE /*primary()*/));
  
#line 26557 "ad_x86.cpp"
  }
  {

#line 1697 "src/hotspot/cpu/x86/x86_32.ad"

    emit_rm(cbuf, 0x3, (0x1 /*secondary()*/), opnd_array(2)->reg(ra_,this,idx2)/* dst */ );
  
#line 26565 "ad_x86.cpp"
  }
}

void mulDPR_reg_mem_ciscNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// mem
  {

#line 1554 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0xD9 /*tertiary()*/));
  
#line 26581 "ad_x86.cpp"
  }
  {

#line 2186 "src/hotspot/cpu/x86/x86_32.ad"

    int rm_byte_opcode = (0x00);
    int base     = opnd_array(2)->base(ra_,this,idx2);
    int index    = opnd_array(2)->index(ra_,this,idx2);
    int scale    = opnd_array(2)->scale();
    int displace = opnd_array(2)->disp(ra_,this,idx2);
    relocInfo::relocType disp_reloc = opnd_array(2)->disp_reloc(); // disp-as-oop when working with static globals
    encode_RegMem(cbuf, rm_byte_opcode, base, index, scale, displace, disp_reloc);
  
#line 26595 "ad_x86.cpp"
  }
  {

#line 2355 "src/hotspot/cpu/x86/x86_32.ad"

    emit_d8(cbuf, (0xD8 /*primary()*/));
    emit_rm(cbuf, 0x3, (0x1 /*secondary()*/), opnd_array(1)->reg(ra_,this,idx1)/* src */ );
  
#line 26604 "ad_x86.cpp"
  }
  {

#line 2367 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode( cbuf, 0xDD );           // FSTP   ST(i)
    emit_d8( cbuf, 0xD8+opnd_array(0)->reg(ra_,this)/* dst */ );
  
#line 26613 "ad_x86.cpp"
  }
}

void mulDPR_reg_mem_cisc_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {

#line 1554 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0xD9 /*tertiary()*/));
  
#line 26629 "ad_x86.cpp"
  }
  {

#line 2186 "src/hotspot/cpu/x86/x86_32.ad"

    int rm_byte_opcode = (0x00);
    int base     = opnd_array(1)->base(ra_,this,idx1);
    int index    = opnd_array(1)->index(ra_,this,idx1);
    int scale    = opnd_array(1)->scale();
    int displace = opnd_array(1)->disp(ra_,this,idx1);
    relocInfo::relocType disp_reloc = opnd_array(1)->disp_reloc(); // disp-as-oop when working with static globals
    encode_RegMem(cbuf, rm_byte_opcode, base, index, scale, displace, disp_reloc);
  
#line 26643 "ad_x86.cpp"
  }
  {

#line 2355 "src/hotspot/cpu/x86/x86_32.ad"

    emit_d8(cbuf, (0xD8 /*primary()*/));
    emit_rm(cbuf, 0x3, (0x1 /*secondary()*/), opnd_array(2)->reg(ra_,this,idx2)/* src */ );
  
#line 26652 "ad_x86.cpp"
  }
  {

#line 2367 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode( cbuf, 0xDD );           // FSTP   ST(i)
    emit_d8( cbuf, 0xD8+opnd_array(0)->reg(ra_,this)/* dst */ );
  
#line 26661 "ad_x86.cpp"
  }
}

void addDPR_mulDPR_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src0
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src1
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// src2
  {

#line 2427 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode( cbuf, 0xD9 );           // FLD    ST(i-1)
    emit_d8( cbuf, 0xC0-1+opnd_array(1)->reg(ra_,this,idx1)/* src0 */ );
  
#line 26679 "ad_x86.cpp"
  }
  {

#line 3008 "src/hotspot/cpu/x86/x86_32.ad"

    // Operand was loaded from memory into fp ST (stack top)
    // FMUL   ST,$src  /* D8 C8+i */
    emit_opcode(cbuf, 0xD8);
    emit_opcode(cbuf, 0xC8 + opnd_array(2)->reg(ra_,this,idx2)/* src1 */);
  
#line 26690 "ad_x86.cpp"
  }
  {

#line 3022 "src/hotspot/cpu/x86/x86_32.ad"

    // FADDP  src2,ST  /* DE C0+i */
    emit_opcode(cbuf, 0xDE);
    emit_opcode(cbuf, 0xC0 + opnd_array(3)->reg(ra_,this,idx3)/* src2 */);
  
#line 26700 "ad_x86.cpp"
  }
}

void addDPR_mulDPR_reg_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src2
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src0
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// src1
  {

#line 2427 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode( cbuf, 0xD9 );           // FLD    ST(i-1)
    emit_d8( cbuf, 0xC0-1+opnd_array(2)->reg(ra_,this,idx2)/* src0 */ );
  
#line 26718 "ad_x86.cpp"
  }
  {

#line 3008 "src/hotspot/cpu/x86/x86_32.ad"

    // Operand was loaded from memory into fp ST (stack top)
    // FMUL   ST,$src  /* D8 C8+i */
    emit_opcode(cbuf, 0xD8);
    emit_opcode(cbuf, 0xC8 + opnd_array(3)->reg(ra_,this,idx3)/* src1 */);
  
#line 26729 "ad_x86.cpp"
  }
  {

#line 3022 "src/hotspot/cpu/x86/x86_32.ad"

    // FADDP  src2,ST  /* DE C0+i */
    emit_opcode(cbuf, 0xDE);
    emit_opcode(cbuf, 0xC0 + opnd_array(1)->reg(ra_,this,idx1)/* src2 */);
  
#line 26739 "ad_x86.cpp"
  }
}

void subDPR_mulDPR_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src0
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src1
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// src2
  {

#line 2427 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode( cbuf, 0xD9 );           // FLD    ST(i-1)
    emit_d8( cbuf, 0xC0-1+opnd_array(1)->reg(ra_,this,idx1)/* src0 */ );
  
#line 26757 "ad_x86.cpp"
  }
  {

#line 3008 "src/hotspot/cpu/x86/x86_32.ad"

    // Operand was loaded from memory into fp ST (stack top)
    // FMUL   ST,$src  /* D8 C8+i */
    emit_opcode(cbuf, 0xD8);
    emit_opcode(cbuf, 0xC8 + opnd_array(2)->reg(ra_,this,idx2)/* src1 */);
  
#line 26768 "ad_x86.cpp"
  }
  {

#line 1554 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0xDE));
  
#line 26776 "ad_x86.cpp"
  }
  {

#line 1618 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0xE0) + opnd_array(3)->reg(ra_,this,idx3)/* src2 */);
  
#line 26784 "ad_x86.cpp"
  }
}

void divDPR_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {

#line 2372 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode( cbuf, 0xD9 );
    emit_d8( cbuf, 0xC0-1+opnd_array(2)->reg(ra_,this,idx2)/* src */ );   // FLD ST(i-1)
  
#line 26801 "ad_x86.cpp"
  }
  {

#line 1544 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0xDE /*primary()*/));
  
#line 26809 "ad_x86.cpp"
  }
  {

#line 1697 "src/hotspot/cpu/x86/x86_32.ad"

    emit_rm(cbuf, 0x3, (0x7 /*secondary()*/), opnd_array(1)->reg(ra_,this,idx1)/* dst */ );
  
#line 26817 "ad_x86.cpp"
  }
}

void strictfp_divDPR_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {

#line 2377 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode( cbuf, 0xDB );           // FLD m80real
    emit_opcode( cbuf, 0x2D );
    emit_d32( cbuf, (int)StubRoutines::x86::addr_fpu_subnormal_bias1() );
    emit_opcode( cbuf, 0xDE );           // FMULP ST(dst), ST0
    emit_opcode( cbuf, 0xC8+opnd_array(1)->reg(ra_,this,idx1)/* dst */ );
  
#line 26837 "ad_x86.cpp"
  }
  {

#line 2372 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode( cbuf, 0xD9 );
    emit_d8( cbuf, 0xC0-1+opnd_array(2)->reg(ra_,this,idx2)/* src */ );   // FLD ST(i-1)
  
#line 26846 "ad_x86.cpp"
  }
  {

#line 1544 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0xDE /*primary()*/));
  
#line 26854 "ad_x86.cpp"
  }
  {

#line 1697 "src/hotspot/cpu/x86/x86_32.ad"

    emit_rm(cbuf, 0x3, (0x7 /*secondary()*/), opnd_array(1)->reg(ra_,this,idx1)/* dst */ );
  
#line 26862 "ad_x86.cpp"
  }
  {

#line 2385 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode( cbuf, 0xDB );           // FLD m80real
    emit_opcode( cbuf, 0x2D );
    emit_d32( cbuf, (int)StubRoutines::x86::addr_fpu_subnormal_bias2() );
    emit_opcode( cbuf, 0xDE );           // FMULP ST(dst), ST0
    emit_opcode( cbuf, 0xC8+opnd_array(1)->reg(ra_,this,idx1)/* dst */ );
  
#line 26874 "ad_x86.cpp"
  }
}

void modDPR_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {

#line 2467 "src/hotspot/cpu/x86/x86_32.ad"

    // load dst in FPR0
    emit_opcode( cbuf, 0xD9 );
    emit_d8( cbuf, 0xC0-1+opnd_array(1)->reg(ra_,this,idx1)/* dst */ );
    if (opnd_array(2)->reg(ra_,this,idx2)/* src */ != FPR1L_enc) {
      // fincstp
      emit_opcode (cbuf, 0xD9);
      emit_opcode (cbuf, 0xF7);
      // swap src with FPR1:
      // FXCH FPR1 with src
      emit_opcode(cbuf, 0xD9);
      emit_d8(cbuf, 0xC8-1+opnd_array(2)->reg(ra_,this,idx2)/* src */ );
      // fdecstp
      emit_opcode (cbuf, 0xD9);
      emit_opcode (cbuf, 0xF6);
    }
  
#line 26904 "ad_x86.cpp"
  }
  {

#line 2569 "src/hotspot/cpu/x86/x86_32.ad"

    // fprem must be iterative
    // :: loop
    // fprem
    emit_opcode( cbuf, 0xD9 );
    emit_opcode( cbuf, 0xF8 );
    // wait
    emit_opcode( cbuf, 0x9b );
    // fnstsw ax
    emit_opcode( cbuf, 0xDF );
    emit_opcode( cbuf, 0xE0 );
    // sahf
    emit_opcode( cbuf, 0x9E );
    // jp  ::loop
    emit_opcode( cbuf, 0x0F );
    emit_opcode( cbuf, 0x8A );
    emit_opcode( cbuf, 0xF4 );
    emit_opcode( cbuf, 0xFF );
    emit_opcode( cbuf, 0xFF );
    emit_opcode( cbuf, 0xFF );
  
#line 26930 "ad_x86.cpp"
  }
  {

#line 2540 "src/hotspot/cpu/x86/x86_32.ad"

    if (opnd_array(2)->reg(ra_,this,idx2)/* src */ != FPR1L_enc) {
      // fincstp
      emit_opcode (cbuf, 0xD9);
      emit_opcode (cbuf, 0xF7);
      // FXCH FPR1 with src
      emit_opcode(cbuf, 0xD9);
      emit_d8(cbuf, 0xC8-1+opnd_array(2)->reg(ra_,this,idx2)/* src */ );
      // fdecstp
      emit_opcode (cbuf, 0xD9);
      emit_opcode (cbuf, 0xF6);
    }
    // // following asm replaced with Pop_Reg_F or Pop_Mem_F
    // // FSTP   FPR$dst$$reg
    // emit_opcode( cbuf, 0xDD );
    // emit_d8( cbuf, 0xD8+$dst$$reg );
  
#line 26952 "ad_x86.cpp"
  }
  {

#line 2367 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode( cbuf, 0xDD );           // FSTP   ST(i)
    emit_d8( cbuf, 0xD8+opnd_array(1)->reg(ra_,this,idx1)/* dst */ );
  
#line 26961 "ad_x86.cpp"
  }
}

void modD_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src0
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src1
  {

#line 2485 "src/hotspot/cpu/x86/x86_32.ad"

    MacroAssembler _masm(&cbuf);
    __ subptr(rsp, 8);
    __ movdbl(Address(rsp, 0), opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src1 */);
    __ fld_d(Address(rsp, 0));
    __ movdbl(Address(rsp, 0), opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src0 */);
    __ fld_d(Address(rsp, 0));
  
#line 26982 "ad_x86.cpp"
  }
  {

#line 2569 "src/hotspot/cpu/x86/x86_32.ad"

    // fprem must be iterative
    // :: loop
    // fprem
    emit_opcode( cbuf, 0xD9 );
    emit_opcode( cbuf, 0xF8 );
    // wait
    emit_opcode( cbuf, 0x9b );
    // fnstsw ax
    emit_opcode( cbuf, 0xDF );
    emit_opcode( cbuf, 0xE0 );
    // sahf
    emit_opcode( cbuf, 0x9E );
    // jp  ::loop
    emit_opcode( cbuf, 0x0F );
    emit_opcode( cbuf, 0x8A );
    emit_opcode( cbuf, 0xF4 );
    emit_opcode( cbuf, 0xFF );
    emit_opcode( cbuf, 0xFF );
    emit_opcode( cbuf, 0xFF );
  
#line 27008 "ad_x86.cpp"
  }
  {

#line 2503 "src/hotspot/cpu/x86/x86_32.ad"

    MacroAssembler _masm(&cbuf);
    __ fstp_d(Address(rsp, 0));
    __ movdbl(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, Address(rsp, 0));
    __ addptr(rsp, 8);
  
#line 27019 "ad_x86.cpp"
  }
  {

#line 2361 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode( cbuf, 0xDD );
    emit_d8( cbuf, 0xD8 );
  
#line 27028 "ad_x86.cpp"
  }
}

void atanDPR_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {

#line 2372 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode( cbuf, 0xD9 );
    emit_d8( cbuf, 0xC0-1+opnd_array(2)->reg(ra_,this,idx2)/* src */ );   // FLD ST(i-1)
  
#line 27045 "ad_x86.cpp"
  }
  {

#line 1544 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0xD9 /*primary()*/));
  
#line 27053 "ad_x86.cpp"
  }
  {

#line 1549 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0xF3 /*secondary()*/));
  
#line 27061 "ad_x86.cpp"
  }
  {

#line 1697 "src/hotspot/cpu/x86/x86_32.ad"

    emit_rm(cbuf, 0x3, (0xF3 /*secondary()*/), opnd_array(1)->reg(ra_,this,idx1)/* dst */ );
  
#line 27069 "ad_x86.cpp"
  }
}

void atanD_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {

#line 2517 "src/hotspot/cpu/x86/x86_32.ad"

    MacroAssembler _masm(&cbuf);
    __ subptr(rsp, 8);
    __ movdbl(Address(rsp, 0), opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src */);
    __ fld_d(Address(rsp, 0));
  
#line 27088 "ad_x86.cpp"
  }
  {

#line 1544 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0xD9 /*primary()*/));
  
#line 27096 "ad_x86.cpp"
  }
  {

#line 1549 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0xF3 /*secondary()*/));
  
#line 27104 "ad_x86.cpp"
  }
  {

#line 2503 "src/hotspot/cpu/x86/x86_32.ad"

    MacroAssembler _masm(&cbuf);
    __ fstp_d(Address(rsp, 0));
    __ movdbl(opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* dst */, Address(rsp, 0));
    __ addptr(rsp, 8);
  
#line 27115 "ad_x86.cpp"
  }
}

void sqrtDPR_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {

#line 2372 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode( cbuf, 0xD9 );
    emit_d8( cbuf, 0xC0-1+opnd_array(1)->reg(ra_,this,idx1)/* src */ );   // FLD ST(i-1)
  
#line 27131 "ad_x86.cpp"
  }
  {

#line 1549 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0xD9 /*secondary()*/));
  
#line 27139 "ad_x86.cpp"
  }
  {

#line 1544 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0xFA /*primary()*/));
  
#line 27147 "ad_x86.cpp"
  }
  {

#line 2367 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode( cbuf, 0xDD );           // FSTP   ST(i)
    emit_d8( cbuf, 0xD8+opnd_array(0)->reg(ra_,this)/* dst */ );
  
#line 27156 "ad_x86.cpp"
  }
}

void cmpFPR_cc_P6Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {

#line 2372 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode( cbuf, 0xD9 );
    emit_d8( cbuf, 0xC0-1+opnd_array(1)->reg(ra_,this,idx1)/* src1 */ );   // FLD ST(i-1)
  
#line 27173 "ad_x86.cpp"
  }
  {

#line 1544 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0xDF /*primary()*/));
  
#line 27181 "ad_x86.cpp"
  }
  {

#line 1697 "src/hotspot/cpu/x86/x86_32.ad"

    emit_rm(cbuf, 0x3, (0x05 /*secondary()*/), opnd_array(2)->reg(ra_,this,idx2)/* src2 */ );
  
#line 27189 "ad_x86.cpp"
  }
  {

#line 2614 "src/hotspot/cpu/x86/x86_32.ad"

    // Fixup the integer flags in case comparison involved a NaN
    //
    // JNP exit (no unordered comparison, P-flag is set by NaN)
    emit_opcode( cbuf, 0x7B );
    emit_d8    ( cbuf, 0x03 );
    // MOV AH,1 - treat as LT case (set carry flag)
    emit_opcode( cbuf, 0xB4 );
    emit_d8    ( cbuf, 0x01 );
    // SAHF
    emit_opcode( cbuf, 0x9E);
    // NOP     // target for branch to avoid branch to branch
    emit_opcode( cbuf, 0x90);
  
#line 27208 "ad_x86.cpp"
  }
}

void cmpFPR_cc_P6CFNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {

#line 2372 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode( cbuf, 0xD9 );
    emit_d8( cbuf, 0xC0-1+opnd_array(1)->reg(ra_,this,idx1)/* src1 */ );   // FLD ST(i-1)
  
#line 27225 "ad_x86.cpp"
  }
  {

#line 1544 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0xDF /*primary()*/));
  
#line 27233 "ad_x86.cpp"
  }
  {

#line 1697 "src/hotspot/cpu/x86/x86_32.ad"

    emit_rm(cbuf, 0x3, (0x05 /*secondary()*/), opnd_array(2)->reg(ra_,this,idx2)/* src2 */ );
  
#line 27241 "ad_x86.cpp"
  }
}

void cmpFPR_ccNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {

#line 2372 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode( cbuf, 0xD9 );
    emit_d8( cbuf, 0xC0-1+opnd_array(1)->reg(ra_,this,idx1)/* src1 */ );   // FLD ST(i-1)
  
#line 27258 "ad_x86.cpp"
  }
  {

#line 1544 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0xD8 /*primary()*/));
  
#line 27266 "ad_x86.cpp"
  }
  {

#line 1697 "src/hotspot/cpu/x86/x86_32.ad"

    emit_rm(cbuf, 0x3, (0x3 /*secondary()*/), opnd_array(2)->reg(ra_,this,idx2)/* src2 */ );
  
#line 27274 "ad_x86.cpp"
  }
  {

#line 2591 "src/hotspot/cpu/x86/x86_32.ad"

    // fnstsw_ax
    emit_opcode( cbuf, 0xDF);
    emit_opcode( cbuf, 0xE0);
    // test ax,0x0400
    emit_opcode( cbuf, 0x66 );   // operand-size prefix for 16-bit immediate
    emit_opcode( cbuf, 0xA9 );
    emit_d16   ( cbuf, 0x0400 );
    // // // This sequence works, but stalls for 12-16 cycles on PPro
    // // test rax,0x0400
    // emit_opcode( cbuf, 0xA9 );
    // emit_d32   ( cbuf, 0x00000400 );
    //
    // jz exit (no unordered comparison)
    emit_opcode( cbuf, 0x74 );
    emit_d8    ( cbuf, 0x02 );
    // mov ah,1 - treat as LT case (set carry flag)
    emit_opcode( cbuf, 0xB4 );
    emit_d8    ( cbuf, 0x01 );
    // sahf
    emit_opcode( cbuf, 0x9E);
  
#line 27301 "ad_x86.cpp"
  }
}

void cmpFPR_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// zero
  {

#line 2372 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode( cbuf, 0xD9 );
    emit_d8( cbuf, 0xC0-1+opnd_array(1)->reg(ra_,this,idx1)/* src1 */ );   // FLD ST(i-1)
  
#line 27318 "ad_x86.cpp"
  }
  {

#line 1549 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0xD9 /*secondary()*/));
  
#line 27326 "ad_x86.cpp"
  }
  {

#line 1544 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0xE4 /*primary()*/));
  
#line 27334 "ad_x86.cpp"
  }
  {

#line 2361 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode( cbuf, 0xDD );
    emit_d8( cbuf, 0xD8 );
  
#line 27343 "ad_x86.cpp"
  }
  {

#line 2644 "src/hotspot/cpu/x86/x86_32.ad"

    // fnstsw_ax();
    emit_opcode( cbuf, 0xDF);
    emit_opcode( cbuf, 0xE0);
    // sahf
    emit_opcode( cbuf, 0x9E);
    // movl(dst, nan_result);
    emit_opcode( cbuf, 0xB8 + opnd_array(0)->reg(ra_,this)/* dst */);
    emit_d32( cbuf, -1 );
    // jcc(Assembler::parity, exit);
    emit_opcode( cbuf, 0x7A );
    emit_d8    ( cbuf, 0x13 );
    // movl(dst, less_result);
    emit_opcode( cbuf, 0xB8 + opnd_array(0)->reg(ra_,this)/* dst */);
    emit_d32( cbuf, -1 );
    // jcc(Assembler::below, exit);
    emit_opcode( cbuf, 0x72 );
    emit_d8    ( cbuf, 0x0C );
    // movl(dst, equal_result);
    emit_opcode( cbuf, 0xB8 + opnd_array(0)->reg(ra_,this)/* dst */);
    emit_d32( cbuf, 0 );
    // jcc(Assembler::equal, exit);
    emit_opcode( cbuf, 0x74 );
    emit_d8    ( cbuf, 0x05 );
    // movl(dst, greater_result);
    emit_opcode( cbuf, 0xB8 + opnd_array(0)->reg(ra_,this)/* dst */);
    emit_d32( cbuf, 1 );
  
#line 27376 "ad_x86.cpp"
  }
}

void cmpFPR_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {

#line 2372 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode( cbuf, 0xD9 );
    emit_d8( cbuf, 0xC0-1+opnd_array(1)->reg(ra_,this,idx1)/* src1 */ );   // FLD ST(i-1)
  
#line 27393 "ad_x86.cpp"
  }
  {

#line 1544 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0xD8 /*primary()*/));
  
#line 27401 "ad_x86.cpp"
  }
  {

#line 1697 "src/hotspot/cpu/x86/x86_32.ad"

    emit_rm(cbuf, 0x3, (0x3 /*secondary()*/), opnd_array(2)->reg(ra_,this,idx2)/* src2 */ );
  
#line 27409 "ad_x86.cpp"
  }
  {

#line 2644 "src/hotspot/cpu/x86/x86_32.ad"

    // fnstsw_ax();
    emit_opcode( cbuf, 0xDF);
    emit_opcode( cbuf, 0xE0);
    // sahf
    emit_opcode( cbuf, 0x9E);
    // movl(dst, nan_result);
    emit_opcode( cbuf, 0xB8 + opnd_array(0)->reg(ra_,this)/* dst */);
    emit_d32( cbuf, -1 );
    // jcc(Assembler::parity, exit);
    emit_opcode( cbuf, 0x7A );
    emit_d8    ( cbuf, 0x13 );
    // movl(dst, less_result);
    emit_opcode( cbuf, 0xB8 + opnd_array(0)->reg(ra_,this)/* dst */);
    emit_d32( cbuf, -1 );
    // jcc(Assembler::below, exit);
    emit_opcode( cbuf, 0x72 );
    emit_d8    ( cbuf, 0x0C );
    // movl(dst, equal_result);
    emit_opcode( cbuf, 0xB8 + opnd_array(0)->reg(ra_,this)/* dst */);
    emit_d32( cbuf, 0 );
    // jcc(Assembler::equal, exit);
    emit_opcode( cbuf, 0x74 );
    emit_d8    ( cbuf, 0x05 );
    // movl(dst, greater_result);
    emit_opcode( cbuf, 0xB8 + opnd_array(0)->reg(ra_,this)/* dst */);
    emit_d32( cbuf, 1 );
  
#line 27442 "ad_x86.cpp"
  }
}

void cmpF_ccNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 10138 "src/hotspot/cpu/x86/x86_32.ad"

    __ ucomiss(opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src1 */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src2 */);
    emit_cmpfp_fixup(_masm);
  
#line 27460 "ad_x86.cpp"
  }
}

void cmpF_ccCFNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 10150 "src/hotspot/cpu/x86/x86_32.ad"

    __ ucomiss(opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src1 */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src2 */);
  
#line 27477 "ad_x86.cpp"
  }
}

void cmpF_ccmemNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 10167 "src/hotspot/cpu/x86/x86_32.ad"

    __ ucomiss(opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src1 */, Address::make_raw(opnd_array(2)->base(ra_,this,idx2), opnd_array(2)->index(ra_,this,idx2), opnd_array(2)->scale(), opnd_array(2)->disp(ra_,this,idx2), opnd_array(2)->disp_reloc()));
    emit_cmpfp_fixup(_masm);
  
#line 27495 "ad_x86.cpp"
  }
}

void cmpF_ccmemCFNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 10179 "src/hotspot/cpu/x86/x86_32.ad"

    __ ucomiss(opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src1 */, Address::make_raw(opnd_array(2)->base(ra_,this,idx2), opnd_array(2)->index(ra_,this,idx2), opnd_array(2)->scale(), opnd_array(2)->disp(ra_,this,idx2), opnd_array(2)->disp_reloc()));
  
#line 27512 "ad_x86.cpp"
  }
}

void cmpF_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 10198 "src/hotspot/cpu/x86/x86_32.ad"

    __ ucomiss(opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src1 */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src2 */);
    emit_cmpfp3(_masm, opnd_array(0)->as_Register(ra_,this)/* dst */);
  
#line 27530 "ad_x86.cpp"
  }
}

void cmpF_regmemNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 10218 "src/hotspot/cpu/x86/x86_32.ad"

    __ ucomiss(opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src1 */, Address::make_raw(opnd_array(2)->base(ra_,this,idx2), opnd_array(2)->index(ra_,this,idx2), opnd_array(2)->scale(), opnd_array(2)->disp(ra_,this,idx2), opnd_array(2)->disp_reloc()));
    emit_cmpfp3(_masm, opnd_array(0)->as_Register(ra_,this)/* dst */);
  
#line 27548 "ad_x86.cpp"
  }
}

void subFPR24_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {

#line 2427 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode( cbuf, 0xD9 );           // FLD    ST(i-1)
    emit_d8( cbuf, 0xC0-1+opnd_array(1)->reg(ra_,this,idx1)/* src1 */ );
  
#line 27565 "ad_x86.cpp"
  }
  {

#line 2355 "src/hotspot/cpu/x86/x86_32.ad"

    emit_d8(cbuf, (0xD8 /*primary()*/));
    emit_rm(cbuf, 0x3, (0x4 /*secondary()*/), opnd_array(2)->reg(ra_,this,idx2)/* src2 */ );
  
#line 27574 "ad_x86.cpp"
  }
  {

#line 2412 "src/hotspot/cpu/x86/x86_32.ad"

    store_to_stackslot( cbuf, 0xD9, 0x03, opnd_array(0)->disp(ra_,this,0) );
  
#line 27582 "ad_x86.cpp"
  }
}

void subFPR_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {

#line 2427 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode( cbuf, 0xD9 );           // FLD    ST(i-1)
    emit_d8( cbuf, 0xC0-1+opnd_array(2)->reg(ra_,this,idx2)/* src */ );
  
#line 27599 "ad_x86.cpp"
  }
  {

#line 1544 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0xDE /*primary()*/));
  
#line 27607 "ad_x86.cpp"
  }
  {

#line 1697 "src/hotspot/cpu/x86/x86_32.ad"

    emit_rm(cbuf, 0x3, (0x5 /*secondary()*/), opnd_array(1)->reg(ra_,this,idx1)/* dst */ );
  
#line 27615 "ad_x86.cpp"
  }
}

void addFPR24_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {

#line 2427 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode( cbuf, 0xD9 );           // FLD    ST(i-1)
    emit_d8( cbuf, 0xC0-1+opnd_array(2)->reg(ra_,this,idx2)/* src2 */ );
  
#line 27632 "ad_x86.cpp"
  }
  {

#line 2355 "src/hotspot/cpu/x86/x86_32.ad"

    emit_d8(cbuf, (0xD8 /*primary()*/));
    emit_rm(cbuf, 0x3, (0x0 /*secondary()*/), opnd_array(1)->reg(ra_,this,idx1)/* src1 */ );
  
#line 27641 "ad_x86.cpp"
  }
  {

#line 2412 "src/hotspot/cpu/x86/x86_32.ad"

    store_to_stackslot( cbuf, 0xD9, 0x03, opnd_array(0)->disp(ra_,this,0) );
  
#line 27649 "ad_x86.cpp"
  }
}

void addFPR_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {

#line 2427 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode( cbuf, 0xD9 );           // FLD    ST(i-1)
    emit_d8( cbuf, 0xC0-1+opnd_array(2)->reg(ra_,this,idx2)/* src */ );
  
#line 27666 "ad_x86.cpp"
  }
  {

#line 1544 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0xDE /*primary()*/));
  
#line 27674 "ad_x86.cpp"
  }
  {

#line 1697 "src/hotspot/cpu/x86/x86_32.ad"

    emit_rm(cbuf, 0x3, (0x0 /*secondary()*/), opnd_array(1)->reg(ra_,this,idx1)/* dst */ );
  
#line 27682 "ad_x86.cpp"
  }
}

void absFPR_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {

#line 1549 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0xD9 /*secondary()*/));
  
#line 27697 "ad_x86.cpp"
  }
  {

#line 1544 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0xE1 /*primary()*/));
  
#line 27705 "ad_x86.cpp"
  }
}

void negFPR_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {

#line 1549 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0xD9 /*secondary()*/));
  
#line 27720 "ad_x86.cpp"
  }
  {

#line 1544 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0xE0 /*primary()*/));
  
#line 27728 "ad_x86.cpp"
  }
}

void addFPR24_reg_memNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {

#line 1554 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0xD9 /*tertiary()*/));
  
#line 27744 "ad_x86.cpp"
  }
  {

#line 2186 "src/hotspot/cpu/x86/x86_32.ad"

    int rm_byte_opcode = (0x00);
    int base     = opnd_array(2)->base(ra_,this,idx2);
    int index    = opnd_array(2)->index(ra_,this,idx2);
    int scale    = opnd_array(2)->scale();
    int displace = opnd_array(2)->disp(ra_,this,idx2);
    relocInfo::relocType disp_reloc = opnd_array(2)->disp_reloc(); // disp-as-oop when working with static globals
    encode_RegMem(cbuf, rm_byte_opcode, base, index, scale, displace, disp_reloc);
  
#line 27758 "ad_x86.cpp"
  }
  {

#line 2355 "src/hotspot/cpu/x86/x86_32.ad"

    emit_d8(cbuf, (0xD8 /*primary()*/));
    emit_rm(cbuf, 0x3, (0x0 /*secondary()*/), opnd_array(1)->reg(ra_,this,idx1)/* src1 */ );
  
#line 27767 "ad_x86.cpp"
  }
  {

#line 2412 "src/hotspot/cpu/x86/x86_32.ad"

    store_to_stackslot( cbuf, 0xD9, 0x03, opnd_array(0)->disp(ra_,this,0) );
  
#line 27775 "ad_x86.cpp"
  }
}

void addFPR24_reg_mem_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// src2
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src1
  {

#line 1554 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0xD9 /*tertiary()*/));
  
#line 27791 "ad_x86.cpp"
  }
  {

#line 2186 "src/hotspot/cpu/x86/x86_32.ad"

    int rm_byte_opcode = (0x00);
    int base     = opnd_array(1)->base(ra_,this,idx1);
    int index    = opnd_array(1)->index(ra_,this,idx1);
    int scale    = opnd_array(1)->scale();
    int displace = opnd_array(1)->disp(ra_,this,idx1);
    relocInfo::relocType disp_reloc = opnd_array(1)->disp_reloc(); // disp-as-oop when working with static globals
    encode_RegMem(cbuf, rm_byte_opcode, base, index, scale, displace, disp_reloc);
  
#line 27805 "ad_x86.cpp"
  }
  {

#line 2355 "src/hotspot/cpu/x86/x86_32.ad"

    emit_d8(cbuf, (0xD8 /*primary()*/));
    emit_rm(cbuf, 0x3, (0x0 /*secondary()*/), opnd_array(2)->reg(ra_,this,idx2)/* src1 */ );
  
#line 27814 "ad_x86.cpp"
  }
  {

#line 2412 "src/hotspot/cpu/x86/x86_32.ad"

    store_to_stackslot( cbuf, 0xD9, 0x03, opnd_array(0)->disp(ra_,this,0) );
  
#line 27822 "ad_x86.cpp"
  }
}

void addFPR_reg_memNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {

#line 1554 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0xD9 /*tertiary()*/));
  
#line 27838 "ad_x86.cpp"
  }
  {

#line 2186 "src/hotspot/cpu/x86/x86_32.ad"

    int rm_byte_opcode = (0x00);
    int base     = opnd_array(2)->base(ra_,this,idx2);
    int index    = opnd_array(2)->index(ra_,this,idx2);
    int scale    = opnd_array(2)->scale();
    int displace = opnd_array(2)->disp(ra_,this,idx2);
    relocInfo::relocType disp_reloc = opnd_array(2)->disp_reloc(); // disp-as-oop when working with static globals
    encode_RegMem(cbuf, rm_byte_opcode, base, index, scale, displace, disp_reloc);
  
#line 27852 "ad_x86.cpp"
  }
  {

#line 1544 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0xDE /*primary()*/));
  
#line 27860 "ad_x86.cpp"
  }
  {

#line 1697 "src/hotspot/cpu/x86/x86_32.ad"

    emit_rm(cbuf, 0x3, (0x0 /*secondary()*/), opnd_array(1)->reg(ra_,this,idx1)/* dst */ );
  
#line 27868 "ad_x86.cpp"
  }
}

void addFPR_reg_mem_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// dst
  {

#line 1554 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0xD9 /*tertiary()*/));
  
#line 27884 "ad_x86.cpp"
  }
  {

#line 2186 "src/hotspot/cpu/x86/x86_32.ad"

    int rm_byte_opcode = (0x00);
    int base     = opnd_array(1)->base(ra_,this,idx1);
    int index    = opnd_array(1)->index(ra_,this,idx1);
    int scale    = opnd_array(1)->scale();
    int displace = opnd_array(1)->disp(ra_,this,idx1);
    relocInfo::relocType disp_reloc = opnd_array(1)->disp_reloc(); // disp-as-oop when working with static globals
    encode_RegMem(cbuf, rm_byte_opcode, base, index, scale, displace, disp_reloc);
  
#line 27898 "ad_x86.cpp"
  }
  {

#line 1544 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0xDE /*primary()*/));
  
#line 27906 "ad_x86.cpp"
  }
  {

#line 1697 "src/hotspot/cpu/x86/x86_32.ad"

    emit_rm(cbuf, 0x3, (0x0 /*secondary()*/), opnd_array(2)->reg(ra_,this,idx2)/* dst */ );
  
#line 27914 "ad_x86.cpp"
  }
}

void addFPR24_mem_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {

#line 1554 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0xD9 /*tertiary()*/));
  
#line 27930 "ad_x86.cpp"
  }
  {

#line 2186 "src/hotspot/cpu/x86/x86_32.ad"

    int rm_byte_opcode = (0x00);
    int base     = opnd_array(1)->base(ra_,this,idx1);
    int index    = opnd_array(1)->index(ra_,this,idx1);
    int scale    = opnd_array(1)->scale();
    int displace = opnd_array(1)->disp(ra_,this,idx1);
    relocInfo::relocType disp_reloc = opnd_array(1)->disp_reloc(); // disp-as-oop when working with static globals
    encode_RegMem(cbuf, rm_byte_opcode, base, index, scale, displace, disp_reloc);
  
#line 27944 "ad_x86.cpp"
  }
  {

#line 2355 "src/hotspot/cpu/x86/x86_32.ad"

    emit_d8(cbuf, (0xD8 /*primary()*/));
    emit_rm(cbuf, 0x3, (0x0 /*secondary()*/), opnd_array(2)->reg(ra_,this,idx2)/* src2 */ );
  
#line 27953 "ad_x86.cpp"
  }
  {

#line 2412 "src/hotspot/cpu/x86/x86_32.ad"

    store_to_stackslot( cbuf, 0xD9, 0x03, opnd_array(0)->disp(ra_,this,0) );
  
#line 27961 "ad_x86.cpp"
  }
}

void addFPR24_mem_ciscNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {

#line 1554 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0xD9 /*tertiary()*/));
  
#line 27977 "ad_x86.cpp"
  }
  {

#line 2186 "src/hotspot/cpu/x86/x86_32.ad"

    int rm_byte_opcode = (0x00);
    int base     = opnd_array(2)->base(ra_,this,idx2);
    int index    = opnd_array(2)->index(ra_,this,idx2);
    int scale    = opnd_array(2)->scale();
    int displace = opnd_array(2)->disp(ra_,this,idx2);
    relocInfo::relocType disp_reloc = opnd_array(2)->disp_reloc(); // disp-as-oop when working with static globals
    encode_RegMem(cbuf, rm_byte_opcode, base, index, scale, displace, disp_reloc);
  
#line 27991 "ad_x86.cpp"
  }
  {

#line 2092 "src/hotspot/cpu/x86/x86_32.ad"

    cbuf.set_insts_mark();            // Mark start of opcode for reloc info in mem operand
  
#line 27999 "ad_x86.cpp"
  }
  {

#line 1544 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0xD8 /*primary()*/));
  
#line 28007 "ad_x86.cpp"
  }
  {

#line 2186 "src/hotspot/cpu/x86/x86_32.ad"

    int rm_byte_opcode = (0x0 /*secondary()*/);
    int base     = opnd_array(1)->base(ra_,this,idx1);
    int index    = opnd_array(1)->index(ra_,this,idx1);
    int scale    = opnd_array(1)->scale();
    int displace = opnd_array(1)->disp(ra_,this,idx1);
    relocInfo::relocType disp_reloc = opnd_array(1)->disp_reloc(); // disp-as-oop when working with static globals
    encode_RegMem(cbuf, rm_byte_opcode, base, index, scale, displace, disp_reloc);
  
#line 28021 "ad_x86.cpp"
  }
  {

#line 2412 "src/hotspot/cpu/x86/x86_32.ad"

    store_to_stackslot( cbuf, 0xD9, 0x03, opnd_array(0)->disp(ra_,this,0) );
  
#line 28029 "ad_x86.cpp"
  }
}

void addFPR24_mem_cisc_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// src2
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src1
  {

#line 1554 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0xD9 /*tertiary()*/));
  
#line 28045 "ad_x86.cpp"
  }
  {

#line 2186 "src/hotspot/cpu/x86/x86_32.ad"

    int rm_byte_opcode = (0x00);
    int base     = opnd_array(1)->base(ra_,this,idx1);
    int index    = opnd_array(1)->index(ra_,this,idx1);
    int scale    = opnd_array(1)->scale();
    int displace = opnd_array(1)->disp(ra_,this,idx1);
    relocInfo::relocType disp_reloc = opnd_array(1)->disp_reloc(); // disp-as-oop when working with static globals
    encode_RegMem(cbuf, rm_byte_opcode, base, index, scale, displace, disp_reloc);
  
#line 28059 "ad_x86.cpp"
  }
  {

#line 2092 "src/hotspot/cpu/x86/x86_32.ad"

    cbuf.set_insts_mark();            // Mark start of opcode for reloc info in mem operand
  
#line 28067 "ad_x86.cpp"
  }
  {

#line 1544 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0xD8 /*primary()*/));
  
#line 28075 "ad_x86.cpp"
  }
  {

#line 2186 "src/hotspot/cpu/x86/x86_32.ad"

    int rm_byte_opcode = (0x0 /*secondary()*/);
    int base     = opnd_array(2)->base(ra_,this,idx2);
    int index    = opnd_array(2)->index(ra_,this,idx2);
    int scale    = opnd_array(2)->scale();
    int displace = opnd_array(2)->disp(ra_,this,idx2);
    relocInfo::relocType disp_reloc = opnd_array(2)->disp_reloc(); // disp-as-oop when working with static globals
    encode_RegMem(cbuf, rm_byte_opcode, base, index, scale, displace, disp_reloc);
  
#line 28089 "ad_x86.cpp"
  }
  {

#line 2412 "src/hotspot/cpu/x86/x86_32.ad"

    store_to_stackslot( cbuf, 0xD9, 0x03, opnd_array(0)->disp(ra_,this,0) );
  
#line 28097 "ad_x86.cpp"
  }
}

void addFPR24_mem_memNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {

#line 1554 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0xD9 /*tertiary()*/));
  
#line 28113 "ad_x86.cpp"
  }
  {

#line 2186 "src/hotspot/cpu/x86/x86_32.ad"

    int rm_byte_opcode = (0x00);
    int base     = opnd_array(2)->base(ra_,this,idx2);
    int index    = opnd_array(2)->index(ra_,this,idx2);
    int scale    = opnd_array(2)->scale();
    int displace = opnd_array(2)->disp(ra_,this,idx2);
    relocInfo::relocType disp_reloc = opnd_array(2)->disp_reloc(); // disp-as-oop when working with static globals
    encode_RegMem(cbuf, rm_byte_opcode, base, index, scale, displace, disp_reloc);
  
#line 28127 "ad_x86.cpp"
  }
  {

#line 2092 "src/hotspot/cpu/x86/x86_32.ad"

    cbuf.set_insts_mark();            // Mark start of opcode for reloc info in mem operand
  
#line 28135 "ad_x86.cpp"
  }
  {

#line 1544 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0xD8 /*primary()*/));
  
#line 28143 "ad_x86.cpp"
  }
  {

#line 2186 "src/hotspot/cpu/x86/x86_32.ad"

    int rm_byte_opcode = (0x0 /*secondary()*/);
    int base     = opnd_array(1)->base(ra_,this,idx1);
    int index    = opnd_array(1)->index(ra_,this,idx1);
    int scale    = opnd_array(1)->scale();
    int displace = opnd_array(1)->disp(ra_,this,idx1);
    relocInfo::relocType disp_reloc = opnd_array(1)->disp_reloc(); // disp-as-oop when working with static globals
    encode_RegMem(cbuf, rm_byte_opcode, base, index, scale, displace, disp_reloc);
  
#line 28157 "ad_x86.cpp"
  }
  {

#line 2412 "src/hotspot/cpu/x86/x86_32.ad"

    store_to_stackslot( cbuf, 0xD9, 0x03, opnd_array(0)->disp(ra_,this,0) );
  
#line 28165 "ad_x86.cpp"
  }
}

void addFPR24_reg_immNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// con
  {
    C2_MacroAssembler _masm(&cbuf);

#line 10376 "src/hotspot/cpu/x86/x86_32.ad"

    __ fld_s(opnd_array(1)->reg(ra_,this,idx1)/* src */- 1);  // FLD ST(i-1)
    __ fadd_s(InternalAddress(__ code()->consts()->start() + constant_offset()));
    __ fstp_s(Address(rsp, opnd_array(0)->disp(ra_,this,0)));
  
#line 28184 "ad_x86.cpp"
  }
}

void addFPR24_reg_immNode::eval_constant(Compile* C) {
  {

#line 10378 "src/hotspot/cpu/x86/x86_32.ad"
    _constant = C->output()->constant_table().add(this, opnd_array(2));
#line 28193 "ad_x86.cpp"
  }
}
void addFPR_reg_immNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// con
  {
    C2_MacroAssembler _masm(&cbuf);

#line 10391 "src/hotspot/cpu/x86/x86_32.ad"

    __ fld_s(opnd_array(1)->reg(ra_,this,idx1)/* src */- 1);  // FLD ST(i-1)
    __ fadd_s(InternalAddress(__ code()->consts()->start() + constant_offset()));
    __ fstp_d(opnd_array(0)->reg(ra_,this)/* dst */);
  
#line 28211 "ad_x86.cpp"
  }
}

void addFPR_reg_immNode::eval_constant(Compile* C) {
  {

#line 10393 "src/hotspot/cpu/x86/x86_32.ad"
    _constant = C->output()->constant_table().add(this, opnd_array(2));
#line 28220 "ad_x86.cpp"
  }
}
void mulFPR24_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {

#line 2427 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode( cbuf, 0xD9 );           // FLD    ST(i-1)
    emit_d8( cbuf, 0xC0-1+opnd_array(1)->reg(ra_,this,idx1)/* src1 */ );
  
#line 28236 "ad_x86.cpp"
  }
  {

#line 2355 "src/hotspot/cpu/x86/x86_32.ad"

    emit_d8(cbuf, (0xD8 /*primary()*/));
    emit_rm(cbuf, 0x3, (0x1 /*secondary()*/), opnd_array(2)->reg(ra_,this,idx2)/* src2 */ );
  
#line 28245 "ad_x86.cpp"
  }
  {

#line 2412 "src/hotspot/cpu/x86/x86_32.ad"

    store_to_stackslot( cbuf, 0xD9, 0x03, opnd_array(0)->disp(ra_,this,0) );
  
#line 28253 "ad_x86.cpp"
  }
}

void mulFPR_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {

#line 2427 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode( cbuf, 0xD9 );           // FLD    ST(i-1)
    emit_d8( cbuf, 0xC0-1+opnd_array(2)->reg(ra_,this,idx2)/* src2 */ );
  
#line 28270 "ad_x86.cpp"
  }
  {

#line 2355 "src/hotspot/cpu/x86/x86_32.ad"

    emit_d8(cbuf, (0xD8 /*primary()*/));
    emit_rm(cbuf, 0x3, (0x1 /*secondary()*/), opnd_array(1)->reg(ra_,this,idx1)/* src1 */ );
  
#line 28279 "ad_x86.cpp"
  }
  {

#line 2422 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode( cbuf, 0xDD );           // FSTP   ST(i)
    emit_d8( cbuf, 0xD8+opnd_array(0)->reg(ra_,this)/* dst */ );
  
#line 28288 "ad_x86.cpp"
  }
}

void mulFPR24_reg_memNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {

#line 1554 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0xD9 /*tertiary()*/));
  
#line 28304 "ad_x86.cpp"
  }
  {

#line 2186 "src/hotspot/cpu/x86/x86_32.ad"

    int rm_byte_opcode = (0x00);
    int base     = opnd_array(2)->base(ra_,this,idx2);
    int index    = opnd_array(2)->index(ra_,this,idx2);
    int scale    = opnd_array(2)->scale();
    int displace = opnd_array(2)->disp(ra_,this,idx2);
    relocInfo::relocType disp_reloc = opnd_array(2)->disp_reloc(); // disp-as-oop when working with static globals
    encode_RegMem(cbuf, rm_byte_opcode, base, index, scale, displace, disp_reloc);
  
#line 28318 "ad_x86.cpp"
  }
  {

#line 2355 "src/hotspot/cpu/x86/x86_32.ad"

    emit_d8(cbuf, (0xD8 /*primary()*/));
    emit_rm(cbuf, 0x3, (0x1 /*secondary()*/), opnd_array(1)->reg(ra_,this,idx1)/* src1 */ );
  
#line 28327 "ad_x86.cpp"
  }
  {

#line 2412 "src/hotspot/cpu/x86/x86_32.ad"

    store_to_stackslot( cbuf, 0xD9, 0x03, opnd_array(0)->disp(ra_,this,0) );
  
#line 28335 "ad_x86.cpp"
  }
}

void mulFPR24_reg_mem_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// src2
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src1
  {

#line 1554 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0xD9 /*tertiary()*/));
  
#line 28351 "ad_x86.cpp"
  }
  {

#line 2186 "src/hotspot/cpu/x86/x86_32.ad"

    int rm_byte_opcode = (0x00);
    int base     = opnd_array(1)->base(ra_,this,idx1);
    int index    = opnd_array(1)->index(ra_,this,idx1);
    int scale    = opnd_array(1)->scale();
    int displace = opnd_array(1)->disp(ra_,this,idx1);
    relocInfo::relocType disp_reloc = opnd_array(1)->disp_reloc(); // disp-as-oop when working with static globals
    encode_RegMem(cbuf, rm_byte_opcode, base, index, scale, displace, disp_reloc);
  
#line 28365 "ad_x86.cpp"
  }
  {

#line 2355 "src/hotspot/cpu/x86/x86_32.ad"

    emit_d8(cbuf, (0xD8 /*primary()*/));
    emit_rm(cbuf, 0x3, (0x1 /*secondary()*/), opnd_array(2)->reg(ra_,this,idx2)/* src1 */ );
  
#line 28374 "ad_x86.cpp"
  }
  {

#line 2412 "src/hotspot/cpu/x86/x86_32.ad"

    store_to_stackslot( cbuf, 0xD9, 0x03, opnd_array(0)->disp(ra_,this,0) );
  
#line 28382 "ad_x86.cpp"
  }
}

void mulFPR_reg_memNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {

#line 1554 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0xD9 /*tertiary()*/));
  
#line 28398 "ad_x86.cpp"
  }
  {

#line 2186 "src/hotspot/cpu/x86/x86_32.ad"

    int rm_byte_opcode = (0x00);
    int base     = opnd_array(2)->base(ra_,this,idx2);
    int index    = opnd_array(2)->index(ra_,this,idx2);
    int scale    = opnd_array(2)->scale();
    int displace = opnd_array(2)->disp(ra_,this,idx2);
    relocInfo::relocType disp_reloc = opnd_array(2)->disp_reloc(); // disp-as-oop when working with static globals
    encode_RegMem(cbuf, rm_byte_opcode, base, index, scale, displace, disp_reloc);
  
#line 28412 "ad_x86.cpp"
  }
  {

#line 2355 "src/hotspot/cpu/x86/x86_32.ad"

    emit_d8(cbuf, (0xD8 /*primary()*/));
    emit_rm(cbuf, 0x3, (0x1 /*secondary()*/), opnd_array(1)->reg(ra_,this,idx1)/* src1 */ );
  
#line 28421 "ad_x86.cpp"
  }
  {

#line 2422 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode( cbuf, 0xDD );           // FSTP   ST(i)
    emit_d8( cbuf, 0xD8+opnd_array(0)->reg(ra_,this)/* dst */ );
  
#line 28430 "ad_x86.cpp"
  }
}

void mulFPR_reg_mem_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// src2
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src1
  {

#line 1554 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0xD9 /*tertiary()*/));
  
#line 28446 "ad_x86.cpp"
  }
  {

#line 2186 "src/hotspot/cpu/x86/x86_32.ad"

    int rm_byte_opcode = (0x00);
    int base     = opnd_array(1)->base(ra_,this,idx1);
    int index    = opnd_array(1)->index(ra_,this,idx1);
    int scale    = opnd_array(1)->scale();
    int displace = opnd_array(1)->disp(ra_,this,idx1);
    relocInfo::relocType disp_reloc = opnd_array(1)->disp_reloc(); // disp-as-oop when working with static globals
    encode_RegMem(cbuf, rm_byte_opcode, base, index, scale, displace, disp_reloc);
  
#line 28460 "ad_x86.cpp"
  }
  {

#line 2355 "src/hotspot/cpu/x86/x86_32.ad"

    emit_d8(cbuf, (0xD8 /*primary()*/));
    emit_rm(cbuf, 0x3, (0x1 /*secondary()*/), opnd_array(2)->reg(ra_,this,idx2)/* src1 */ );
  
#line 28469 "ad_x86.cpp"
  }
  {

#line 2422 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode( cbuf, 0xDD );           // FSTP   ST(i)
    emit_d8( cbuf, 0xD8+opnd_array(0)->reg(ra_,this)/* dst */ );
  
#line 28478 "ad_x86.cpp"
  }
}

void mulFPR24_mem_memNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {

#line 1554 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0xD9 /*tertiary()*/));
  
#line 28494 "ad_x86.cpp"
  }
  {

#line 2186 "src/hotspot/cpu/x86/x86_32.ad"

    int rm_byte_opcode = (0x00);
    int base     = opnd_array(2)->base(ra_,this,idx2);
    int index    = opnd_array(2)->index(ra_,this,idx2);
    int scale    = opnd_array(2)->scale();
    int displace = opnd_array(2)->disp(ra_,this,idx2);
    relocInfo::relocType disp_reloc = opnd_array(2)->disp_reloc(); // disp-as-oop when working with static globals
    encode_RegMem(cbuf, rm_byte_opcode, base, index, scale, displace, disp_reloc);
  
#line 28508 "ad_x86.cpp"
  }
  {

#line 2092 "src/hotspot/cpu/x86/x86_32.ad"

    cbuf.set_insts_mark();            // Mark start of opcode for reloc info in mem operand
  
#line 28516 "ad_x86.cpp"
  }
  {

#line 1544 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0xD8 /*primary()*/));
  
#line 28524 "ad_x86.cpp"
  }
  {

#line 2186 "src/hotspot/cpu/x86/x86_32.ad"

    int rm_byte_opcode = (0x1 /*secondary()*/);
    int base     = opnd_array(1)->base(ra_,this,idx1);
    int index    = opnd_array(1)->index(ra_,this,idx1);
    int scale    = opnd_array(1)->scale();
    int displace = opnd_array(1)->disp(ra_,this,idx1);
    relocInfo::relocType disp_reloc = opnd_array(1)->disp_reloc(); // disp-as-oop when working with static globals
    encode_RegMem(cbuf, rm_byte_opcode, base, index, scale, displace, disp_reloc);
  
#line 28538 "ad_x86.cpp"
  }
  {

#line 2412 "src/hotspot/cpu/x86/x86_32.ad"

    store_to_stackslot( cbuf, 0xD9, 0x03, opnd_array(0)->disp(ra_,this,0) );
  
#line 28546 "ad_x86.cpp"
  }
}

void mulFPR24_reg_immNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// con
  {
    C2_MacroAssembler _masm(&cbuf);

#line 10482 "src/hotspot/cpu/x86/x86_32.ad"

    __ fld_s(opnd_array(1)->reg(ra_,this,idx1)/* src */- 1);  // FLD ST(i-1)
    __ fmul_s(InternalAddress(__ code()->consts()->start() + constant_offset()));
    __ fstp_s(Address(rsp, opnd_array(0)->disp(ra_,this,0)));
  
#line 28565 "ad_x86.cpp"
  }
}

void mulFPR24_reg_immNode::eval_constant(Compile* C) {
  {

#line 10484 "src/hotspot/cpu/x86/x86_32.ad"
    _constant = C->output()->constant_table().add(this, opnd_array(2));
#line 28574 "ad_x86.cpp"
  }
}
void mulFPR_reg_immNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// con
  {
    C2_MacroAssembler _masm(&cbuf);

#line 10498 "src/hotspot/cpu/x86/x86_32.ad"

    __ fld_s(opnd_array(1)->reg(ra_,this,idx1)/* src */- 1);  // FLD ST(i-1)
    __ fmul_s(InternalAddress(__ code()->consts()->start() + constant_offset()));
    __ fstp_d(opnd_array(0)->reg(ra_,this)/* dst */);
  
#line 28592 "ad_x86.cpp"
  }
}

void mulFPR_reg_immNode::eval_constant(Compile* C) {
  {

#line 10500 "src/hotspot/cpu/x86/x86_32.ad"
    _constant = C->output()->constant_table().add(this, opnd_array(2));
#line 28601 "ad_x86.cpp"
  }
}
void mulFPR_reg_load1Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {

#line 1554 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0xD9 /*tertiary()*/));
  
#line 28616 "ad_x86.cpp"
  }
  {

#line 2186 "src/hotspot/cpu/x86/x86_32.ad"

    int rm_byte_opcode = (0x00);
    int base     = opnd_array(1)->base(ra_,this,idx1);
    int index    = opnd_array(1)->index(ra_,this,idx1);
    int scale    = opnd_array(1)->scale();
    int displace = opnd_array(1)->disp(ra_,this,idx1);
    relocInfo::relocType disp_reloc = opnd_array(1)->disp_reloc(); // disp-as-oop when working with static globals
    encode_RegMem(cbuf, rm_byte_opcode, base, index, scale, displace, disp_reloc);
  
#line 28630 "ad_x86.cpp"
  }
  {

#line 2355 "src/hotspot/cpu/x86/x86_32.ad"

    emit_d8(cbuf, (0xD8 /*primary()*/));
    emit_rm(cbuf, 0x3, (0x1 /*secondary()*/), opnd_array(2)->reg(ra_,this,idx2)/* src */ );
  
#line 28639 "ad_x86.cpp"
  }
  {

#line 2422 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode( cbuf, 0xDD );           // FSTP   ST(i)
    emit_d8( cbuf, 0xD8+opnd_array(0)->reg(ra_,this)/* dst */ );
  
#line 28648 "ad_x86.cpp"
  }
}

void mulFPR_reg_load1_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// mem1
  {

#line 1554 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0xD9 /*tertiary()*/));
  
#line 28664 "ad_x86.cpp"
  }
  {

#line 2186 "src/hotspot/cpu/x86/x86_32.ad"

    int rm_byte_opcode = (0x00);
    int base     = opnd_array(2)->base(ra_,this,idx2);
    int index    = opnd_array(2)->index(ra_,this,idx2);
    int scale    = opnd_array(2)->scale();
    int displace = opnd_array(2)->disp(ra_,this,idx2);
    relocInfo::relocType disp_reloc = opnd_array(2)->disp_reloc(); // disp-as-oop when working with static globals
    encode_RegMem(cbuf, rm_byte_opcode, base, index, scale, displace, disp_reloc);
  
#line 28678 "ad_x86.cpp"
  }
  {

#line 2355 "src/hotspot/cpu/x86/x86_32.ad"

    emit_d8(cbuf, (0xD8 /*primary()*/));
    emit_rm(cbuf, 0x3, (0x1 /*secondary()*/), opnd_array(1)->reg(ra_,this,idx1)/* src */ );
  
#line 28687 "ad_x86.cpp"
  }
  {

#line 2422 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode( cbuf, 0xDD );           // FSTP   ST(i)
    emit_d8( cbuf, 0xD8+opnd_array(0)->reg(ra_,this)/* dst */ );
  
#line 28696 "ad_x86.cpp"
  }
}

void addFPR_mulFPR_reg_load1Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src1
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// src2
  {

#line 1544 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0xD9 /*primary()*/));
  
#line 28713 "ad_x86.cpp"
  }
  {

#line 2186 "src/hotspot/cpu/x86/x86_32.ad"

    int rm_byte_opcode = (0x00);
    int base     = opnd_array(1)->base(ra_,this,idx1);
    int index    = opnd_array(1)->index(ra_,this,idx1);
    int scale    = opnd_array(1)->scale();
    int displace = opnd_array(1)->disp(ra_,this,idx1);
    relocInfo::relocType disp_reloc = opnd_array(1)->disp_reloc(); // disp-as-oop when working with static globals
    encode_RegMem(cbuf, rm_byte_opcode, base, index, scale, displace, disp_reloc);
  
#line 28727 "ad_x86.cpp"
  }
  {

#line 3008 "src/hotspot/cpu/x86/x86_32.ad"

    // Operand was loaded from memory into fp ST (stack top)
    // FMUL   ST,$src  /* D8 C8+i */
    emit_opcode(cbuf, 0xD8);
    emit_opcode(cbuf, 0xC8 + opnd_array(2)->reg(ra_,this,idx2)/* src1 */);
  
#line 28738 "ad_x86.cpp"
  }
  {

#line 3015 "src/hotspot/cpu/x86/x86_32.ad"

    // FADDP  ST,src2  /* D8 C0+i */
    emit_opcode(cbuf, 0xD8);
    emit_opcode(cbuf, 0xC0 + opnd_array(3)->reg(ra_,this,idx3)/* src2 */);
    //could use FADDP  src2,fpST  /* DE C0+i */
  
#line 28749 "ad_x86.cpp"
  }
  {

#line 2422 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode( cbuf, 0xDD );           // FSTP   ST(i)
    emit_d8( cbuf, 0xD8+opnd_array(0)->reg(ra_,this)/* dst */ );
  
#line 28758 "ad_x86.cpp"
  }
}

void addFPR_mulFPR_reg_load1_1Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// src2
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// mem1
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// src1
  {

#line 1544 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0xD9 /*primary()*/));
  
#line 28775 "ad_x86.cpp"
  }
  {

#line 2186 "src/hotspot/cpu/x86/x86_32.ad"

    int rm_byte_opcode = (0x00);
    int base     = opnd_array(2)->base(ra_,this,idx2);
    int index    = opnd_array(2)->index(ra_,this,idx2);
    int scale    = opnd_array(2)->scale();
    int displace = opnd_array(2)->disp(ra_,this,idx2);
    relocInfo::relocType disp_reloc = opnd_array(2)->disp_reloc(); // disp-as-oop when working with static globals
    encode_RegMem(cbuf, rm_byte_opcode, base, index, scale, displace, disp_reloc);
  
#line 28789 "ad_x86.cpp"
  }
  {

#line 3008 "src/hotspot/cpu/x86/x86_32.ad"

    // Operand was loaded from memory into fp ST (stack top)
    // FMUL   ST,$src  /* D8 C8+i */
    emit_opcode(cbuf, 0xD8);
    emit_opcode(cbuf, 0xC8 + opnd_array(3)->reg(ra_,this,idx3)/* src1 */);
  
#line 28800 "ad_x86.cpp"
  }
  {

#line 3015 "src/hotspot/cpu/x86/x86_32.ad"

    // FADDP  ST,src2  /* D8 C0+i */
    emit_opcode(cbuf, 0xD8);
    emit_opcode(cbuf, 0xC0 + opnd_array(1)->reg(ra_,this,idx1)/* src2 */);
    //could use FADDP  src2,fpST  /* DE C0+i */
  
#line 28811 "ad_x86.cpp"
  }
  {

#line 2422 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode( cbuf, 0xDD );           // FSTP   ST(i)
    emit_d8( cbuf, 0xD8+opnd_array(0)->reg(ra_,this)/* dst */ );
  
#line 28820 "ad_x86.cpp"
  }
}

void addFPR_mulFPR_reg_load1_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// mem1
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// src2
  {

#line 1544 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0xD9 /*primary()*/));
  
#line 28837 "ad_x86.cpp"
  }
  {

#line 2186 "src/hotspot/cpu/x86/x86_32.ad"

    int rm_byte_opcode = (0x00);
    int base     = opnd_array(2)->base(ra_,this,idx2);
    int index    = opnd_array(2)->index(ra_,this,idx2);
    int scale    = opnd_array(2)->scale();
    int displace = opnd_array(2)->disp(ra_,this,idx2);
    relocInfo::relocType disp_reloc = opnd_array(2)->disp_reloc(); // disp-as-oop when working with static globals
    encode_RegMem(cbuf, rm_byte_opcode, base, index, scale, displace, disp_reloc);
  
#line 28851 "ad_x86.cpp"
  }
  {

#line 3008 "src/hotspot/cpu/x86/x86_32.ad"

    // Operand was loaded from memory into fp ST (stack top)
    // FMUL   ST,$src  /* D8 C8+i */
    emit_opcode(cbuf, 0xD8);
    emit_opcode(cbuf, 0xC8 + opnd_array(1)->reg(ra_,this,idx1)/* src1 */);
  
#line 28862 "ad_x86.cpp"
  }
  {

#line 3015 "src/hotspot/cpu/x86/x86_32.ad"

    // FADDP  ST,src2  /* D8 C0+i */
    emit_opcode(cbuf, 0xD8);
    emit_opcode(cbuf, 0xC0 + opnd_array(3)->reg(ra_,this,idx3)/* src2 */);
    //could use FADDP  src2,fpST  /* DE C0+i */
  
#line 28873 "ad_x86.cpp"
  }
  {

#line 2422 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode( cbuf, 0xDD );           // FSTP   ST(i)
    emit_d8( cbuf, 0xD8+opnd_array(0)->reg(ra_,this)/* dst */ );
  
#line 28882 "ad_x86.cpp"
  }
}

void addFPR_mulFPR_reg_load1_2Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// src2
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src1
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// mem1
  {

#line 1544 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0xD9 /*primary()*/));
  
#line 28899 "ad_x86.cpp"
  }
  {

#line 2186 "src/hotspot/cpu/x86/x86_32.ad"

    int rm_byte_opcode = (0x00);
    int base     = opnd_array(3)->base(ra_,this,idx3);
    int index    = opnd_array(3)->index(ra_,this,idx3);
    int scale    = opnd_array(3)->scale();
    int displace = opnd_array(3)->disp(ra_,this,idx3);
    relocInfo::relocType disp_reloc = opnd_array(3)->disp_reloc(); // disp-as-oop when working with static globals
    encode_RegMem(cbuf, rm_byte_opcode, base, index, scale, displace, disp_reloc);
  
#line 28913 "ad_x86.cpp"
  }
  {

#line 3008 "src/hotspot/cpu/x86/x86_32.ad"

    // Operand was loaded from memory into fp ST (stack top)
    // FMUL   ST,$src  /* D8 C8+i */
    emit_opcode(cbuf, 0xD8);
    emit_opcode(cbuf, 0xC8 + opnd_array(2)->reg(ra_,this,idx2)/* src1 */);
  
#line 28924 "ad_x86.cpp"
  }
  {

#line 3015 "src/hotspot/cpu/x86/x86_32.ad"

    // FADDP  ST,src2  /* D8 C0+i */
    emit_opcode(cbuf, 0xD8);
    emit_opcode(cbuf, 0xC0 + opnd_array(1)->reg(ra_,this,idx1)/* src2 */);
    //could use FADDP  src2,fpST  /* DE C0+i */
  
#line 28935 "ad_x86.cpp"
  }
  {

#line 2422 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode( cbuf, 0xDD );           // FSTP   ST(i)
    emit_d8( cbuf, 0xD8+opnd_array(0)->reg(ra_,this)/* dst */ );
  
#line 28944 "ad_x86.cpp"
  }
}

void addFPR_mulFPR_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src0
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src1
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// src2
  {

#line 2427 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode( cbuf, 0xD9 );           // FLD    ST(i-1)
    emit_d8( cbuf, 0xC0-1+opnd_array(1)->reg(ra_,this,idx1)/* src0 */ );
  
#line 28962 "ad_x86.cpp"
  }
  {

#line 3008 "src/hotspot/cpu/x86/x86_32.ad"

    // Operand was loaded from memory into fp ST (stack top)
    // FMUL   ST,$src  /* D8 C8+i */
    emit_opcode(cbuf, 0xD8);
    emit_opcode(cbuf, 0xC8 + opnd_array(2)->reg(ra_,this,idx2)/* src1 */);
  
#line 28973 "ad_x86.cpp"
  }
  {

#line 3022 "src/hotspot/cpu/x86/x86_32.ad"

    // FADDP  src2,ST  /* DE C0+i */
    emit_opcode(cbuf, 0xDE);
    emit_opcode(cbuf, 0xC0 + opnd_array(3)->reg(ra_,this,idx3)/* src2 */);
  
#line 28983 "ad_x86.cpp"
  }
}

void addFPR_mulFPR_reg_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src2
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src0
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// src1
  {

#line 2427 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode( cbuf, 0xD9 );           // FLD    ST(i-1)
    emit_d8( cbuf, 0xC0-1+opnd_array(2)->reg(ra_,this,idx2)/* src0 */ );
  
#line 29001 "ad_x86.cpp"
  }
  {

#line 3008 "src/hotspot/cpu/x86/x86_32.ad"

    // Operand was loaded from memory into fp ST (stack top)
    // FMUL   ST,$src  /* D8 C8+i */
    emit_opcode(cbuf, 0xD8);
    emit_opcode(cbuf, 0xC8 + opnd_array(3)->reg(ra_,this,idx3)/* src1 */);
  
#line 29012 "ad_x86.cpp"
  }
  {

#line 3022 "src/hotspot/cpu/x86/x86_32.ad"

    // FADDP  src2,ST  /* DE C0+i */
    emit_opcode(cbuf, 0xDE);
    emit_opcode(cbuf, 0xC0 + opnd_array(1)->reg(ra_,this,idx1)/* src2 */);
  
#line 29022 "ad_x86.cpp"
  }
}

void subFPR_divFPR_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src2
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src1
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// src3
  {

#line 2427 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode( cbuf, 0xD9 );           // FLD    ST(i-1)
    emit_d8( cbuf, 0xC0-1+opnd_array(1)->reg(ra_,this,idx1)/* src2 */ );
  
#line 29040 "ad_x86.cpp"
  }
  {

#line 3028 "src/hotspot/cpu/x86/x86_32.ad"

    // Operand has been loaded into fp ST (stack top)
      // FSUB   ST,$src1
      emit_opcode(cbuf, 0xD8);
      emit_opcode(cbuf, 0xE0 + opnd_array(2)->reg(ra_,this,idx2)/* src1 */);

      // FDIV
      emit_opcode(cbuf, 0xD8);
      emit_opcode(cbuf, 0xF0 + opnd_array(3)->reg(ra_,this,idx3)/* src3 */);
  
#line 29055 "ad_x86.cpp"
  }
  {

#line 2422 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode( cbuf, 0xDD );           // FSTP   ST(i)
    emit_d8( cbuf, 0xD8+opnd_array(0)->reg(ra_,this)/* dst */ );
  
#line 29064 "ad_x86.cpp"
  }
}

void divFPR24_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {

#line 2427 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode( cbuf, 0xD9 );           // FLD    ST(i-1)
    emit_d8( cbuf, 0xC0-1+opnd_array(1)->reg(ra_,this,idx1)/* src1 */ );
  
#line 29081 "ad_x86.cpp"
  }
  {

#line 2355 "src/hotspot/cpu/x86/x86_32.ad"

    emit_d8(cbuf, (0xD8 /*primary()*/));
    emit_rm(cbuf, 0x3, (0x6 /*secondary()*/), opnd_array(2)->reg(ra_,this,idx2)/* src2 */ );
  
#line 29090 "ad_x86.cpp"
  }
  {

#line 2412 "src/hotspot/cpu/x86/x86_32.ad"

    store_to_stackslot( cbuf, 0xD9, 0x03, opnd_array(0)->disp(ra_,this,0) );
  
#line 29098 "ad_x86.cpp"
  }
}

void divFPR_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {

#line 2427 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode( cbuf, 0xD9 );           // FLD    ST(i-1)
    emit_d8( cbuf, 0xC0-1+opnd_array(2)->reg(ra_,this,idx2)/* src */ );
  
#line 29115 "ad_x86.cpp"
  }
  {

#line 1544 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0xDE /*primary()*/));
  
#line 29123 "ad_x86.cpp"
  }
  {

#line 1697 "src/hotspot/cpu/x86/x86_32.ad"

    emit_rm(cbuf, 0x3, (0x7 /*secondary()*/), opnd_array(1)->reg(ra_,this,idx1)/* dst */ );
  
#line 29131 "ad_x86.cpp"
  }
}

void modFPR24_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {

#line 2467 "src/hotspot/cpu/x86/x86_32.ad"

    // load dst in FPR0
    emit_opcode( cbuf, 0xD9 );
    emit_d8( cbuf, 0xC0-1+opnd_array(1)->reg(ra_,this,idx1)/* src1 */ );
    if (opnd_array(2)->reg(ra_,this,idx2)/* src2 */ != FPR1L_enc) {
      // fincstp
      emit_opcode (cbuf, 0xD9);
      emit_opcode (cbuf, 0xF7);
      // swap src with FPR1:
      // FXCH FPR1 with src
      emit_opcode(cbuf, 0xD9);
      emit_d8(cbuf, 0xC8-1+opnd_array(2)->reg(ra_,this,idx2)/* src2 */ );
      // fdecstp
      emit_opcode (cbuf, 0xD9);
      emit_opcode (cbuf, 0xF6);
    }
  
#line 29161 "ad_x86.cpp"
  }
  {

#line 2569 "src/hotspot/cpu/x86/x86_32.ad"

    // fprem must be iterative
    // :: loop
    // fprem
    emit_opcode( cbuf, 0xD9 );
    emit_opcode( cbuf, 0xF8 );
    // wait
    emit_opcode( cbuf, 0x9b );
    // fnstsw ax
    emit_opcode( cbuf, 0xDF );
    emit_opcode( cbuf, 0xE0 );
    // sahf
    emit_opcode( cbuf, 0x9E );
    // jp  ::loop
    emit_opcode( cbuf, 0x0F );
    emit_opcode( cbuf, 0x8A );
    emit_opcode( cbuf, 0xF4 );
    emit_opcode( cbuf, 0xFF );
    emit_opcode( cbuf, 0xFF );
    emit_opcode( cbuf, 0xFF );
  
#line 29187 "ad_x86.cpp"
  }
  {

#line 2540 "src/hotspot/cpu/x86/x86_32.ad"

    if (opnd_array(2)->reg(ra_,this,idx2)/* src2 */ != FPR1L_enc) {
      // fincstp
      emit_opcode (cbuf, 0xD9);
      emit_opcode (cbuf, 0xF7);
      // FXCH FPR1 with src
      emit_opcode(cbuf, 0xD9);
      emit_d8(cbuf, 0xC8-1+opnd_array(2)->reg(ra_,this,idx2)/* src2 */ );
      // fdecstp
      emit_opcode (cbuf, 0xD9);
      emit_opcode (cbuf, 0xF6);
    }
    // // following asm replaced with Pop_Reg_F or Pop_Mem_F
    // // FSTP   FPR$dst$$reg
    // emit_opcode( cbuf, 0xDD );
    // emit_d8( cbuf, 0xD8+$dst$$reg );
  
#line 29209 "ad_x86.cpp"
  }
  {

#line 2412 "src/hotspot/cpu/x86/x86_32.ad"

    store_to_stackslot( cbuf, 0xD9, 0x03, opnd_array(0)->disp(ra_,this,0) );
  
#line 29217 "ad_x86.cpp"
  }
}

void modFPR_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {

#line 2467 "src/hotspot/cpu/x86/x86_32.ad"

    // load dst in FPR0
    emit_opcode( cbuf, 0xD9 );
    emit_d8( cbuf, 0xC0-1+opnd_array(1)->reg(ra_,this,idx1)/* dst */ );
    if (opnd_array(2)->reg(ra_,this,idx2)/* src */ != FPR1L_enc) {
      // fincstp
      emit_opcode (cbuf, 0xD9);
      emit_opcode (cbuf, 0xF7);
      // swap src with FPR1:
      // FXCH FPR1 with src
      emit_opcode(cbuf, 0xD9);
      emit_d8(cbuf, 0xC8-1+opnd_array(2)->reg(ra_,this,idx2)/* src */ );
      // fdecstp
      emit_opcode (cbuf, 0xD9);
      emit_opcode (cbuf, 0xF6);
    }
  
#line 29247 "ad_x86.cpp"
  }
  {

#line 2569 "src/hotspot/cpu/x86/x86_32.ad"

    // fprem must be iterative
    // :: loop
    // fprem
    emit_opcode( cbuf, 0xD9 );
    emit_opcode( cbuf, 0xF8 );
    // wait
    emit_opcode( cbuf, 0x9b );
    // fnstsw ax
    emit_opcode( cbuf, 0xDF );
    emit_opcode( cbuf, 0xE0 );
    // sahf
    emit_opcode( cbuf, 0x9E );
    // jp  ::loop
    emit_opcode( cbuf, 0x0F );
    emit_opcode( cbuf, 0x8A );
    emit_opcode( cbuf, 0xF4 );
    emit_opcode( cbuf, 0xFF );
    emit_opcode( cbuf, 0xFF );
    emit_opcode( cbuf, 0xFF );
  
#line 29273 "ad_x86.cpp"
  }
  {

#line 2540 "src/hotspot/cpu/x86/x86_32.ad"

    if (opnd_array(2)->reg(ra_,this,idx2)/* src */ != FPR1L_enc) {
      // fincstp
      emit_opcode (cbuf, 0xD9);
      emit_opcode (cbuf, 0xF7);
      // FXCH FPR1 with src
      emit_opcode(cbuf, 0xD9);
      emit_d8(cbuf, 0xC8-1+opnd_array(2)->reg(ra_,this,idx2)/* src */ );
      // fdecstp
      emit_opcode (cbuf, 0xD9);
      emit_opcode (cbuf, 0xF6);
    }
    // // following asm replaced with Pop_Reg_F or Pop_Mem_F
    // // FSTP   FPR$dst$$reg
    // emit_opcode( cbuf, 0xDD );
    // emit_d8( cbuf, 0xD8+$dst$$reg );
  
#line 29295 "ad_x86.cpp"
  }
  {

#line 2422 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode( cbuf, 0xDD );           // FSTP   ST(i)
    emit_d8( cbuf, 0xD8+opnd_array(1)->reg(ra_,this,idx1)/* dst */ );
  
#line 29304 "ad_x86.cpp"
  }
}

void modF_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src0
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src1
  {

#line 2494 "src/hotspot/cpu/x86/x86_32.ad"

    MacroAssembler _masm(&cbuf);
    __ subptr(rsp, 4);
    __ movflt(Address(rsp, 0), opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src1 */);
    __ fld_s(Address(rsp, 0));
    __ movflt(Address(rsp, 0), opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src0 */);
    __ fld_s(Address(rsp, 0));
  
#line 29325 "ad_x86.cpp"
  }
  {

#line 2569 "src/hotspot/cpu/x86/x86_32.ad"

    // fprem must be iterative
    // :: loop
    // fprem
    emit_opcode( cbuf, 0xD9 );
    emit_opcode( cbuf, 0xF8 );
    // wait
    emit_opcode( cbuf, 0x9b );
    // fnstsw ax
    emit_opcode( cbuf, 0xDF );
    emit_opcode( cbuf, 0xE0 );
    // sahf
    emit_opcode( cbuf, 0x9E );
    // jp  ::loop
    emit_opcode( cbuf, 0x0F );
    emit_opcode( cbuf, 0x8A );
    emit_opcode( cbuf, 0xF4 );
    emit_opcode( cbuf, 0xFF );
    emit_opcode( cbuf, 0xFF );
    emit_opcode( cbuf, 0xFF );
  
#line 29351 "ad_x86.cpp"
  }
  {

#line 2510 "src/hotspot/cpu/x86/x86_32.ad"

    MacroAssembler _masm(&cbuf);
    __ fstp_s(Address(rsp, 0));
    __ movflt(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, Address(rsp, 0));
    __ addptr(rsp, (0x4));
  
#line 29362 "ad_x86.cpp"
  }
  {

#line 2361 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode( cbuf, 0xDD );
    emit_d8( cbuf, 0xD8 );
  
#line 29371 "ad_x86.cpp"
  }
}

void roundFloat_mem_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {

#line 2433 "src/hotspot/cpu/x86/x86_32.ad"

    int pop = 0x02;
    if (opnd_array(1)->reg(ra_,this,idx1)/* src */ != FPR1L_enc) {
      emit_opcode( cbuf, 0xD9 );         // FLD    ST(i-1)
      emit_d8( cbuf, 0xC0-1+opnd_array(1)->reg(ra_,this,idx1)/* src */ );
      pop = 0x03;
    }
    store_to_stackslot( cbuf, 0xD9, pop, opnd_array(0)->disp(ra_,this,0) ); // FST<P>_S  [ESP+dst]
  
#line 29392 "ad_x86.cpp"
  }
}

void roundDouble_mem_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {

#line 2444 "src/hotspot/cpu/x86/x86_32.ad"

    int pop = 0x02;
    if (opnd_array(1)->reg(ra_,this,idx1)/* src */ != FPR1L_enc) {
      emit_opcode( cbuf, 0xD9 );         // FLD    ST(i-1)
      emit_d8( cbuf, 0xC0-1+opnd_array(1)->reg(ra_,this,idx1)/* src */ );
      pop = 0x03;
    }
    store_to_stackslot( cbuf, 0xDD, pop, opnd_array(0)->disp(ra_,this,0) ); // FST<P>_D  [ESP+dst]
  
#line 29413 "ad_x86.cpp"
  }
}

void convDPR2F_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 10698 "src/hotspot/cpu/x86/x86_32.ad"

    __ subptr(rsp, 4);
    if (opnd_array(1)->reg(ra_,this,idx1)/* src */!= FPR1L_enc) {
      __ fld_s(opnd_array(1)->reg(ra_,this,idx1)/* src */-1);
      __ fstp_s(Address(rsp, 0));
    } else {
      __ fst_s(Address(rsp, 0));
    }
    __ movflt(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, Address(rsp, 0));
    __ addptr(rsp, 4);
  
#line 29437 "ad_x86.cpp"
  }
}

void convD2F_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 10717 "src/hotspot/cpu/x86/x86_32.ad"

    __ cvtsd2ss (opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */);
  
#line 29453 "ad_x86.cpp"
  }
}

void convFPR2DPR_reg_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {

#line 2455 "src/hotspot/cpu/x86/x86_32.ad"

    int pop = 0xD0 - 1; // -1 since we skip FLD
    if (opnd_array(1)->reg(ra_,this,idx1)/* src */ != FPR1L_enc) {
      emit_opcode( cbuf, 0xD9 );         // FLD    ST(src-1)
      emit_d8( cbuf, 0xC0-1+opnd_array(1)->reg(ra_,this,idx1)/* src */ );
      pop = 0xD8;
    }
    emit_opcode( cbuf, 0xDD );
    emit_d8( cbuf, pop+opnd_array(0)->reg(ra_,this)/* dst */ );      // FST<P> ST(i)
  
#line 29475 "ad_x86.cpp"
  }
}

void convF2DPR_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 10749 "src/hotspot/cpu/x86/x86_32.ad"

    __ subptr(rsp, 4);
    __ movflt(Address(rsp, 0), opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */);
    __ fld_s(Address(rsp, 0));
    __ addptr(rsp, 4);
    __ fstp_d(opnd_array(0)->reg(ra_,this)/* dst */);
  
#line 29495 "ad_x86.cpp"
  }
}

void convF2D_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 10763 "src/hotspot/cpu/x86/x86_32.ad"

    __ cvtss2sd (opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */);
  
#line 29511 "ad_x86.cpp"
  }
}

void convDPR2I_reg_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {

#line 2372 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode( cbuf, 0xD9 );
    emit_d8( cbuf, 0xC0-1+opnd_array(1)->reg(ra_,this,idx1)/* src */ );   // FLD ST(i-1)
  
#line 29527 "ad_x86.cpp"
  }
  {

#line 2919 "src/hotspot/cpu/x86/x86_32.ad"

    // Flip to round-to-zero mode.  We attempted to allow invalid-op
    // exceptions here, so that a NAN or other corner-case value will
    // thrown an exception (but normal values get converted at full speed).
    // However, I2C adapters and other float-stack manglers leave pending
    // invalid-op exceptions hanging.  We would have to clear them before
    // enabling them and that is more expensive than just testing for the
    // invalid value Intel stores down in the corner cases.
    emit_opcode(cbuf,0xD9);            // FLDCW  trunc
    emit_opcode(cbuf,0x2D);
    emit_d32(cbuf,(int)StubRoutines::x86::addr_fpu_cntrl_wrd_trunc());
    // Allocate a word
    emit_opcode(cbuf,0x83);            // SUB ESP,4
    emit_opcode(cbuf,0xEC);
    emit_d8(cbuf,0x04);
    // Encoding assumes a double has been pushed into FPR0.
    // Store down the double as an int, popping the FPU stack
    emit_opcode(cbuf,0xDB);            // FISTP [ESP]
    emit_opcode(cbuf,0x1C);
    emit_d8(cbuf,0x24);
    // Restore the rounding mode; mask the exception
    emit_opcode(cbuf,0xD9);            // FLDCW   std/24-bit mode
    emit_opcode(cbuf,0x2D);
    emit_d32( cbuf, Compile::current()->in_24_bit_fp_mode()
        ? (int)StubRoutines::x86::addr_fpu_cntrl_wrd_24()
        : (int)StubRoutines::x86::addr_fpu_cntrl_wrd_std());

    // Load the converted int; adjust CPU stack
    emit_opcode(cbuf,0x58);       // POP EAX
    emit_opcode(cbuf,0x3D);       // CMP EAX,imm
    emit_d32   (cbuf,0x80000000); //         0x80000000
    emit_opcode(cbuf,0x75);       // JNE around_slow_call
    emit_d8    (cbuf,0x07);       // Size of slow_call
    // Push src onto stack slow-path
    emit_opcode(cbuf,0xD9 );      // FLD     ST(i)
    emit_d8    (cbuf,0xC0-1+opnd_array(1)->reg(ra_,this,idx1)/* src */ );
    // CALL directly to the runtime
    MacroAssembler _masm(&cbuf);
    cbuf.set_insts_mark();
    emit_opcode(cbuf,0xE8);       // Call into runtime
    emit_d32_reloc(cbuf, (StubRoutines::x86::d2i_wrapper() - cbuf.insts_end()) - 4, runtime_call_Relocation::spec(), RELOC_IMM32 );
    __ post_call_nop();
    // Carry on here...
  
#line 29576 "ad_x86.cpp"
  }
}

void convD2I_reg_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 10803 "src/hotspot/cpu/x86/x86_32.ad"

    Label fast;
    __ cvttsd2sil(opnd_array(0)->as_Register(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */);
    __ cmpl(opnd_array(0)->as_Register(ra_,this)/* dst */, 0x80000000);
    __ jccb(Assembler::notEqual, fast);
    __ subptr(rsp, 8);
    __ movdbl(Address(rsp, 0), opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */);
    __ fld_d(Address(rsp, 0));
    __ addptr(rsp, 8);
    __ call(RuntimeAddress(CAST_FROM_FN_PTR(address, StubRoutines::x86::d2i_wrapper())));
    __ post_call_nop();
    __ bind(fast);
  
#line 29602 "ad_x86.cpp"
  }
}

void convDPR2L_reg_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {

#line 2372 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode( cbuf, 0xD9 );
    emit_d8( cbuf, 0xC0-1+opnd_array(1)->reg(ra_,this,idx1)/* src */ );   // FLD ST(i-1)
  
#line 29618 "ad_x86.cpp"
  }
  {

#line 2964 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf,0xD9);            // FLDCW  trunc
    emit_opcode(cbuf,0x2D);
    emit_d32(cbuf,(int)StubRoutines::x86::addr_fpu_cntrl_wrd_trunc());
    // Allocate a word
    emit_opcode(cbuf,0x83);            // SUB ESP,8
    emit_opcode(cbuf,0xEC);
    emit_d8(cbuf,0x08);
    // Encoding assumes a double has been pushed into FPR0.
    // Store down the double as a long, popping the FPU stack
    emit_opcode(cbuf,0xDF);            // FISTP [ESP]
    emit_opcode(cbuf,0x3C);
    emit_d8(cbuf,0x24);
    // Restore the rounding mode; mask the exception
    emit_opcode(cbuf,0xD9);            // FLDCW   std/24-bit mode
    emit_opcode(cbuf,0x2D);
    emit_d32( cbuf, Compile::current()->in_24_bit_fp_mode()
        ? (int)StubRoutines::x86::addr_fpu_cntrl_wrd_24()
        : (int)StubRoutines::x86::addr_fpu_cntrl_wrd_std());

    // Load the converted int; adjust CPU stack
    emit_opcode(cbuf,0x58);       // POP EAX
    emit_opcode(cbuf,0x5A);       // POP EDX
    emit_opcode(cbuf,0x81);       // CMP EDX,imm
    emit_d8    (cbuf,0xFA);       // rdx
    emit_d32   (cbuf,0x80000000); //         0x80000000
    emit_opcode(cbuf,0x75);       // JNE around_slow_call
    emit_d8    (cbuf,0x07+4);     // Size of slow_call
    emit_opcode(cbuf,0x85);       // TEST EAX,EAX
    emit_opcode(cbuf,0xC0);       // 2/rax,/rax,
    emit_opcode(cbuf,0x75);       // JNE around_slow_call
    emit_d8    (cbuf,0x07);       // Size of slow_call
    // Push src onto stack slow-path
    emit_opcode(cbuf,0xD9 );      // FLD     ST(i)
    emit_d8    (cbuf,0xC0-1+opnd_array(1)->reg(ra_,this,idx1)/* src */ );
    // CALL directly to the runtime
    MacroAssembler _masm(&cbuf);
    cbuf.set_insts_mark();
    emit_opcode(cbuf,0xE8);       // Call into runtime
    emit_d32_reloc(cbuf, (StubRoutines::x86::d2l_wrapper() - cbuf.insts_end()) - 4, runtime_call_Relocation::spec(), RELOC_IMM32 );
    __ post_call_nop();
    // Carry on here...
  
#line 29666 "ad_x86.cpp"
  }
}

void convD2L_reg_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 10864 "src/hotspot/cpu/x86/x86_32.ad"

    Label fast;
    __ subptr(rsp, 8);
    __ movdbl(Address(rsp, 0), opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */);
    __ fld_d(Address(rsp, 0));
    __ fldcw(ExternalAddress(StubRoutines::x86::addr_fpu_cntrl_wrd_trunc()));
    __ fistp_d(Address(rsp, 0));
    // Restore the rounding mode, mask the exception
    if (Compile::current()->in_24_bit_fp_mode()) {
      __ fldcw(ExternalAddress(StubRoutines::x86::addr_fpu_cntrl_wrd_24()));
    } else {
      __ fldcw(ExternalAddress(StubRoutines::x86::addr_fpu_cntrl_wrd_std()));
    }
    // Load the converted long, adjust CPU stack
    __ pop(rax);
    __ pop(rdx);
    __ cmpl(rdx, 0x80000000);
    __ jccb(Assembler::notEqual, fast);
    __ testl(rax, rax);
    __ jccb(Assembler::notEqual, fast);
    __ subptr(rsp, 8);
    __ movdbl(Address(rsp, 0), opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */);
    __ fld_d(Address(rsp, 0));
    __ addptr(rsp, 8);
    __ call(RuntimeAddress(CAST_FROM_FN_PTR(address, StubRoutines::x86::d2l_wrapper())));
    __ post_call_nop();
    __ bind(fast);
  
#line 29707 "ad_x86.cpp"
  }
}

void convFPR2I_reg_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {

#line 2427 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode( cbuf, 0xD9 );           // FLD    ST(i-1)
    emit_d8( cbuf, 0xC0-1+opnd_array(1)->reg(ra_,this,idx1)/* src */ );
  
#line 29723 "ad_x86.cpp"
  }
  {

#line 2919 "src/hotspot/cpu/x86/x86_32.ad"

    // Flip to round-to-zero mode.  We attempted to allow invalid-op
    // exceptions here, so that a NAN or other corner-case value will
    // thrown an exception (but normal values get converted at full speed).
    // However, I2C adapters and other float-stack manglers leave pending
    // invalid-op exceptions hanging.  We would have to clear them before
    // enabling them and that is more expensive than just testing for the
    // invalid value Intel stores down in the corner cases.
    emit_opcode(cbuf,0xD9);            // FLDCW  trunc
    emit_opcode(cbuf,0x2D);
    emit_d32(cbuf,(int)StubRoutines::x86::addr_fpu_cntrl_wrd_trunc());
    // Allocate a word
    emit_opcode(cbuf,0x83);            // SUB ESP,4
    emit_opcode(cbuf,0xEC);
    emit_d8(cbuf,0x04);
    // Encoding assumes a double has been pushed into FPR0.
    // Store down the double as an int, popping the FPU stack
    emit_opcode(cbuf,0xDB);            // FISTP [ESP]
    emit_opcode(cbuf,0x1C);
    emit_d8(cbuf,0x24);
    // Restore the rounding mode; mask the exception
    emit_opcode(cbuf,0xD9);            // FLDCW   std/24-bit mode
    emit_opcode(cbuf,0x2D);
    emit_d32( cbuf, Compile::current()->in_24_bit_fp_mode()
        ? (int)StubRoutines::x86::addr_fpu_cntrl_wrd_24()
        : (int)StubRoutines::x86::addr_fpu_cntrl_wrd_std());

    // Load the converted int; adjust CPU stack
    emit_opcode(cbuf,0x58);       // POP EAX
    emit_opcode(cbuf,0x3D);       // CMP EAX,imm
    emit_d32   (cbuf,0x80000000); //         0x80000000
    emit_opcode(cbuf,0x75);       // JNE around_slow_call
    emit_d8    (cbuf,0x07);       // Size of slow_call
    // Push src onto stack slow-path
    emit_opcode(cbuf,0xD9 );      // FLD     ST(i)
    emit_d8    (cbuf,0xC0-1+opnd_array(1)->reg(ra_,this,idx1)/* src */ );
    // CALL directly to the runtime
    MacroAssembler _masm(&cbuf);
    cbuf.set_insts_mark();
    emit_opcode(cbuf,0xE8);       // Call into runtime
    emit_d32_reloc(cbuf, (StubRoutines::x86::d2i_wrapper() - cbuf.insts_end()) - 4, runtime_call_Relocation::spec(), RELOC_IMM32 );
    __ post_call_nop();
    // Carry on here...
  
#line 29772 "ad_x86.cpp"
  }
}

void convF2I_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 10935 "src/hotspot/cpu/x86/x86_32.ad"

    Label fast;
    __ cvttss2sil(opnd_array(0)->as_Register(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */);
    __ cmpl(opnd_array(0)->as_Register(ra_,this)/* dst */, 0x80000000);
    __ jccb(Assembler::notEqual, fast);
    __ subptr(rsp, 4);
    __ movflt(Address(rsp, 0), opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */);
    __ fld_s(Address(rsp, 0));
    __ addptr(rsp, 4);
    __ call(RuntimeAddress(CAST_FROM_FN_PTR(address, StubRoutines::x86::d2i_wrapper())));
    __ post_call_nop();
    __ bind(fast);
  
#line 29798 "ad_x86.cpp"
  }
}

void convFPR2L_reg_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {

#line 2427 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode( cbuf, 0xD9 );           // FLD    ST(i-1)
    emit_d8( cbuf, 0xC0-1+opnd_array(1)->reg(ra_,this,idx1)/* src */ );
  
#line 29814 "ad_x86.cpp"
  }
  {

#line 2964 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf,0xD9);            // FLDCW  trunc
    emit_opcode(cbuf,0x2D);
    emit_d32(cbuf,(int)StubRoutines::x86::addr_fpu_cntrl_wrd_trunc());
    // Allocate a word
    emit_opcode(cbuf,0x83);            // SUB ESP,8
    emit_opcode(cbuf,0xEC);
    emit_d8(cbuf,0x08);
    // Encoding assumes a double has been pushed into FPR0.
    // Store down the double as a long, popping the FPU stack
    emit_opcode(cbuf,0xDF);            // FISTP [ESP]
    emit_opcode(cbuf,0x3C);
    emit_d8(cbuf,0x24);
    // Restore the rounding mode; mask the exception
    emit_opcode(cbuf,0xD9);            // FLDCW   std/24-bit mode
    emit_opcode(cbuf,0x2D);
    emit_d32( cbuf, Compile::current()->in_24_bit_fp_mode()
        ? (int)StubRoutines::x86::addr_fpu_cntrl_wrd_24()
        : (int)StubRoutines::x86::addr_fpu_cntrl_wrd_std());

    // Load the converted int; adjust CPU stack
    emit_opcode(cbuf,0x58);       // POP EAX
    emit_opcode(cbuf,0x5A);       // POP EDX
    emit_opcode(cbuf,0x81);       // CMP EDX,imm
    emit_d8    (cbuf,0xFA);       // rdx
    emit_d32   (cbuf,0x80000000); //         0x80000000
    emit_opcode(cbuf,0x75);       // JNE around_slow_call
    emit_d8    (cbuf,0x07+4);     // Size of slow_call
    emit_opcode(cbuf,0x85);       // TEST EAX,EAX
    emit_opcode(cbuf,0xC0);       // 2/rax,/rax,
    emit_opcode(cbuf,0x75);       // JNE around_slow_call
    emit_d8    (cbuf,0x07);       // Size of slow_call
    // Push src onto stack slow-path
    emit_opcode(cbuf,0xD9 );      // FLD     ST(i)
    emit_d8    (cbuf,0xC0-1+opnd_array(1)->reg(ra_,this,idx1)/* src */ );
    // CALL directly to the runtime
    MacroAssembler _masm(&cbuf);
    cbuf.set_insts_mark();
    emit_opcode(cbuf,0xE8);       // Call into runtime
    emit_d32_reloc(cbuf, (StubRoutines::x86::d2l_wrapper() - cbuf.insts_end()) - 4, runtime_call_Relocation::spec(), RELOC_IMM32 );
    __ post_call_nop();
    // Carry on here...
  
#line 29862 "ad_x86.cpp"
  }
}

void convF2L_reg_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 10997 "src/hotspot/cpu/x86/x86_32.ad"

    Label fast;
    __ subptr(rsp, 8);
    __ movflt(Address(rsp, 0), opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */);
    __ fld_s(Address(rsp, 0));
    __ fldcw(ExternalAddress(StubRoutines::x86::addr_fpu_cntrl_wrd_trunc()));
    __ fistp_d(Address(rsp, 0));
    // Restore the rounding mode, mask the exception
    if (Compile::current()->in_24_bit_fp_mode()) {
      __ fldcw(ExternalAddress(StubRoutines::x86::addr_fpu_cntrl_wrd_24()));
    } else {
      __ fldcw(ExternalAddress(StubRoutines::x86::addr_fpu_cntrl_wrd_std()));
    }
    // Load the converted long, adjust CPU stack
    __ pop(rax);
    __ pop(rdx);
    __ cmpl(rdx, 0x80000000);
    __ jccb(Assembler::notEqual, fast);
    __ testl(rax, rax);
    __ jccb(Assembler::notEqual, fast);
    __ subptr(rsp, 4);
    __ movflt(Address(rsp, 0), opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */);
    __ fld_s(Address(rsp, 0));
    __ addptr(rsp, 4);
    __ call(RuntimeAddress(CAST_FROM_FN_PTR(address, StubRoutines::x86::d2l_wrapper())));
    __ post_call_nop();
    __ bind(fast);
  
#line 29903 "ad_x86.cpp"
  }
}

void convI2DPR_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {

#line 2407 "src/hotspot/cpu/x86/x86_32.ad"

    store_to_stackslot( cbuf, (0xDB /*primary()*/), (0x0 /*secondary()*/), opnd_array(1)->disp(ra_,this,idx1) );
  
#line 29918 "ad_x86.cpp"
  }
  {

#line 2367 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode( cbuf, 0xDD );           // FSTP   ST(i)
    emit_d8( cbuf, 0xD8+opnd_array(0)->reg(ra_,this)/* dst */ );
  
#line 29927 "ad_x86.cpp"
  }
}

void convI2D_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 11042 "src/hotspot/cpu/x86/x86_32.ad"

    __ cvtsi2sdl (opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_Register(ra_,this,idx1)/* src */);
  
#line 29943 "ad_x86.cpp"
  }
}

void convI2D_memNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  {
    C2_MacroAssembler _masm(&cbuf);

#line 11052 "src/hotspot/cpu/x86/x86_32.ad"

    __ cvtsi2sdl (opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, Address::make_raw(opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1), opnd_array(1)->disp_reloc()));
  
#line 29959 "ad_x86.cpp"
  }
}

void convXI2D_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 11065 "src/hotspot/cpu/x86/x86_32.ad"

    __ movdl(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_Register(ra_,this,idx1)/* src */);
    __ cvtdq2pd(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(0)->as_XMMRegister(ra_,this)/* dst */);
  
#line 29976 "ad_x86.cpp"
  }
}

void convI2DPR_memNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  {

#line 1544 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0xDB /*primary()*/));
  
#line 29991 "ad_x86.cpp"
  }
  {

#line 2186 "src/hotspot/cpu/x86/x86_32.ad"

    int rm_byte_opcode = (0x00);
    int base     = opnd_array(1)->base(ra_,this,idx1);
    int index    = opnd_array(1)->index(ra_,this,idx1);
    int scale    = opnd_array(1)->scale();
    int displace = opnd_array(1)->disp(ra_,this,idx1);
    relocInfo::relocType disp_reloc = opnd_array(1)->disp_reloc(); // disp-as-oop when working with static globals
    encode_RegMem(cbuf, rm_byte_opcode, base, index, scale, displace, disp_reloc);
  
#line 30005 "ad_x86.cpp"
  }
  {

#line 2367 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode( cbuf, 0xDD );           // FSTP   ST(i)
    emit_d8( cbuf, 0xD8+opnd_array(0)->reg(ra_,this)/* dst */ );
  
#line 30014 "ad_x86.cpp"
  }
}

void conv24I2FPR_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {

#line 2407 "src/hotspot/cpu/x86/x86_32.ad"

    store_to_stackslot( cbuf, (0xDB /*primary()*/), (0x0 /*secondary()*/), opnd_array(1)->disp(ra_,this,idx1) );
  
#line 30029 "ad_x86.cpp"
  }
  {

#line 2422 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode( cbuf, 0xDD );           // FSTP   ST(i)
    emit_d8( cbuf, 0xD8+opnd_array(0)->reg(ra_,this)/* dst */ );
  
#line 30038 "ad_x86.cpp"
  }
}

void convI2FPR_SSFNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {

#line 2407 "src/hotspot/cpu/x86/x86_32.ad"

    store_to_stackslot( cbuf, (0xDB /*primary()*/), (0x0 /*secondary()*/), opnd_array(1)->disp(ra_,this,idx1) );
  
#line 30053 "ad_x86.cpp"
  }
  {

#line 2412 "src/hotspot/cpu/x86/x86_32.ad"

    store_to_stackslot( cbuf, 0xD9, 0x03, opnd_array(0)->disp(ra_,this,0) );
  
#line 30061 "ad_x86.cpp"
  }
}

void convI2FPR_SSF_memNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  {

#line 1544 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0xDB /*primary()*/));
  
#line 30076 "ad_x86.cpp"
  }
  {

#line 2186 "src/hotspot/cpu/x86/x86_32.ad"

    int rm_byte_opcode = (0x00);
    int base     = opnd_array(1)->base(ra_,this,idx1);
    int index    = opnd_array(1)->index(ra_,this,idx1);
    int scale    = opnd_array(1)->scale();
    int displace = opnd_array(1)->disp(ra_,this,idx1);
    relocInfo::relocType disp_reloc = opnd_array(1)->disp_reloc(); // disp-as-oop when working with static globals
    encode_RegMem(cbuf, rm_byte_opcode, base, index, scale, displace, disp_reloc);
  
#line 30090 "ad_x86.cpp"
  }
  {

#line 2412 "src/hotspot/cpu/x86/x86_32.ad"

    store_to_stackslot( cbuf, 0xD9, 0x03, opnd_array(0)->disp(ra_,this,0) );
  
#line 30098 "ad_x86.cpp"
  }
}

void convI2FPR_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {

#line 2407 "src/hotspot/cpu/x86/x86_32.ad"

    store_to_stackslot( cbuf, (0xDB /*primary()*/), (0x0 /*secondary()*/), opnd_array(1)->disp(ra_,this,idx1) );
  
#line 30113 "ad_x86.cpp"
  }
  {

#line 2422 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode( cbuf, 0xDD );           // FSTP   ST(i)
    emit_d8( cbuf, 0xD8+opnd_array(0)->reg(ra_,this)/* dst */ );
  
#line 30122 "ad_x86.cpp"
  }
}

void convI2FPR_memNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  {

#line 1544 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0xDB /*primary()*/));
  
#line 30137 "ad_x86.cpp"
  }
  {

#line 2186 "src/hotspot/cpu/x86/x86_32.ad"

    int rm_byte_opcode = (0x00);
    int base     = opnd_array(1)->base(ra_,this,idx1);
    int index    = opnd_array(1)->index(ra_,this,idx1);
    int scale    = opnd_array(1)->scale();
    int displace = opnd_array(1)->disp(ra_,this,idx1);
    relocInfo::relocType disp_reloc = opnd_array(1)->disp_reloc(); // disp-as-oop when working with static globals
    encode_RegMem(cbuf, rm_byte_opcode, base, index, scale, displace, disp_reloc);
  
#line 30151 "ad_x86.cpp"
  }
  {

#line 2422 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode( cbuf, 0xDD );           // FSTP   ST(i)
    emit_d8( cbuf, 0xD8+opnd_array(0)->reg(ra_,this)/* dst */ );
  
#line 30160 "ad_x86.cpp"
  }
}

void convI2F_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 11150 "src/hotspot/cpu/x86/x86_32.ad"

    __ cvtsi2ssl (opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_Register(ra_,this,idx1)/* src */);
  
#line 30176 "ad_x86.cpp"
  }
}

void convXI2F_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 11163 "src/hotspot/cpu/x86/x86_32.ad"

    __ movdl(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_Register(ra_,this,idx1)/* src */);
    __ cvtdq2ps(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(0)->as_XMMRegister(ra_,this)/* dst */);
  
#line 30193 "ad_x86.cpp"
  }
}

void convI2L_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {

#line 2689 "src/hotspot/cpu/x86/x86_32.ad"

    // mov $dst.lo,$src
    int dst_encoding = opnd_array(0)->reg(ra_,this)/* dst */;
    int src_encoding = opnd_array(1)->reg(ra_,this,idx1)/* src */;
    encode_Copy( cbuf, dst_encoding  , src_encoding );
    // mov $dst.hi,$src
    encode_Copy( cbuf, HIGH_FROM_LOW_ENC(dst_encoding), src_encoding );
    // sar $dst.hi,31
    emit_opcode( cbuf, 0xC1 );
    emit_rm(cbuf, 0x3, 7, HIGH_FROM_LOW_ENC(dst_encoding) );
    emit_d8(cbuf, 0x1F );
  
#line 30217 "ad_x86.cpp"
  }
}

void convI2L_reg_zexNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// mask
  {

#line 1951 "src/hotspot/cpu/x86/x86_32.ad"

    encode_Copy( cbuf, opnd_array(0)->reg(ra_,this)/* dst */, opnd_array(1)->reg(ra_,this,idx1)/* src */ );
  
#line 30233 "ad_x86.cpp"
  }
  {

#line 1544 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0x33 /*primary()*/));
  
#line 30241 "ad_x86.cpp"
  }
  {

#line 1977 "src/hotspot/cpu/x86/x86_32.ad"

    emit_rm(cbuf, 0x3, HIGH_FROM_LOW_ENC(opnd_array(0)->reg(ra_,this)/* dst */), HIGH_FROM_LOW_ENC(opnd_array(0)->reg(ra_,this)/* dst */));
  
#line 30249 "ad_x86.cpp"
  }
}

void zerox_longNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// mask
  {

#line 1951 "src/hotspot/cpu/x86/x86_32.ad"

    encode_Copy( cbuf, opnd_array(0)->reg(ra_,this)/* dst */, opnd_array(1)->reg(ra_,this,idx1)/* src */ );
  
#line 30265 "ad_x86.cpp"
  }
  {

#line 1544 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0x33 /*primary()*/));
  
#line 30273 "ad_x86.cpp"
  }
  {

#line 1977 "src/hotspot/cpu/x86/x86_32.ad"

    emit_rm(cbuf, 0x3, HIGH_FROM_LOW_ENC(opnd_array(0)->reg(ra_,this)/* dst */), HIGH_FROM_LOW_ENC(opnd_array(0)->reg(ra_,this)/* dst */));
  
#line 30281 "ad_x86.cpp"
  }
}

void convL2DPR_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {

#line 2702 "src/hotspot/cpu/x86/x86_32.ad"

    // push $src.hi
    emit_opcode(cbuf, 0x50+HIGH_FROM_LOW_ENC(opnd_array(1)->reg(ra_,this,idx1)/* src */));
    // push $src.lo
    emit_opcode(cbuf, 0x50+opnd_array(1)->reg(ra_,this,idx1)/* src */  );
    // fild 64-bits at [SP]
    emit_opcode(cbuf,0xdf);
    emit_d8(cbuf, 0x6C);
    emit_d8(cbuf, 0x24);
    emit_d8(cbuf, 0x00);
    // pop stack
    emit_opcode(cbuf, 0x83); // add  SP, #8
    emit_rm(cbuf, 0x3, 0x00, ESP_enc);
    emit_d8(cbuf, 0x8);
  
#line 30308 "ad_x86.cpp"
  }
  {

#line 2418 "src/hotspot/cpu/x86/x86_32.ad"

    store_to_stackslot( cbuf, 0xDD, 0x03, opnd_array(0)->disp(ra_,this,0) );
  
#line 30316 "ad_x86.cpp"
  }
}

void convL2D_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {

#line 2732 "src/hotspot/cpu/x86/x86_32.ad"

    // push $src.hi
    emit_opcode(cbuf, 0x50+HIGH_FROM_LOW_ENC(opnd_array(1)->reg(ra_,this,idx1)/* src */));
    // push $src.lo
    emit_opcode(cbuf, 0x50+opnd_array(1)->reg(ra_,this,idx1)/* src */  );
    // fild 64-bits at [SP]
    emit_opcode(cbuf,0xdf);
    emit_d8(cbuf, 0x6C);
    emit_d8(cbuf, 0x24);
    emit_d8(cbuf, 0x00);
  
#line 30339 "ad_x86.cpp"
  }
  {

#line 2503 "src/hotspot/cpu/x86/x86_32.ad"

    MacroAssembler _masm(&cbuf);
    __ fstp_d(Address(rsp, 0));
    __ movdbl(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, Address(rsp, 0));
    __ addptr(rsp, 8);
  
#line 30350 "ad_x86.cpp"
  }
}

void convL2F_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {

#line 2732 "src/hotspot/cpu/x86/x86_32.ad"

    // push $src.hi
    emit_opcode(cbuf, 0x50+HIGH_FROM_LOW_ENC(opnd_array(1)->reg(ra_,this,idx1)/* src */));
    // push $src.lo
    emit_opcode(cbuf, 0x50+opnd_array(1)->reg(ra_,this,idx1)/* src */  );
    // fild 64-bits at [SP]
    emit_opcode(cbuf,0xdf);
    emit_d8(cbuf, 0x6C);
    emit_d8(cbuf, 0x24);
    emit_d8(cbuf, 0x00);
  
#line 30373 "ad_x86.cpp"
  }
  {

#line 2510 "src/hotspot/cpu/x86/x86_32.ad"

    MacroAssembler _masm(&cbuf);
    __ fstp_s(Address(rsp, 0));
    __ movflt(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, Address(rsp, 0));
    __ addptr(rsp, (0x8));
  
#line 30384 "ad_x86.cpp"
  }
}

void convL2FPR_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {

#line 2702 "src/hotspot/cpu/x86/x86_32.ad"

    // push $src.hi
    emit_opcode(cbuf, 0x50+HIGH_FROM_LOW_ENC(opnd_array(1)->reg(ra_,this,idx1)/* src */));
    // push $src.lo
    emit_opcode(cbuf, 0x50+opnd_array(1)->reg(ra_,this,idx1)/* src */  );
    // fild 64-bits at [SP]
    emit_opcode(cbuf,0xdf);
    emit_d8(cbuf, 0x6C);
    emit_d8(cbuf, 0x24);
    emit_d8(cbuf, 0x00);
    // pop stack
    emit_opcode(cbuf, 0x83); // add  SP, #8
    emit_rm(cbuf, 0x3, 0x00, ESP_enc);
    emit_d8(cbuf, 0x8);
  
#line 30411 "ad_x86.cpp"
  }
  {

#line 2412 "src/hotspot/cpu/x86/x86_32.ad"

    store_to_stackslot( cbuf, 0xD9, 0x03, opnd_array(0)->disp(ra_,this,0) );
  
#line 30419 "ad_x86.cpp"
  }
}

void convL2I_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {

#line 1955 "src/hotspot/cpu/x86/x86_32.ad"

    encode_Copy( cbuf, opnd_array(0)->reg(ra_,this)/* dst */, opnd_array(1)->reg(ra_,this,idx1)/* src */ );
  
#line 30434 "ad_x86.cpp"
  }
}

void MoveF2I_stack_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 11275 "src/hotspot/cpu/x86/x86_32.ad"

    __ movl(opnd_array(0)->as_Register(ra_,this)/* dst */, Address(rsp, opnd_array(1)->disp(ra_,this,idx1)));
  
#line 30450 "ad_x86.cpp"
  }
}

void MoveFPR2I_reg_stackNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {

#line 2433 "src/hotspot/cpu/x86/x86_32.ad"

    int pop = 0x02;
    if (opnd_array(1)->reg(ra_,this,idx1)/* src */ != FPR1L_enc) {
      emit_opcode( cbuf, 0xD9 );         // FLD    ST(i-1)
      emit_d8( cbuf, 0xC0-1+opnd_array(1)->reg(ra_,this,idx1)/* src */ );
      pop = 0x03;
    }
    store_to_stackslot( cbuf, 0xD9, pop, opnd_array(0)->disp(ra_,this,0) ); // FST<P>_S  [ESP+dst]
  
#line 30471 "ad_x86.cpp"
  }
}

void MoveF2I_reg_stack_sseNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 11299 "src/hotspot/cpu/x86/x86_32.ad"

    __ movflt(Address(rsp, opnd_array(0)->disp(ra_,this,0)), opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */);
  
#line 30487 "ad_x86.cpp"
  }
}

void MoveF2I_reg_reg_sseNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 11311 "src/hotspot/cpu/x86/x86_32.ad"

    __ movdl(opnd_array(0)->as_Register(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */);
  
#line 30503 "ad_x86.cpp"
  }
}

void MoveI2F_reg_stackNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 11323 "src/hotspot/cpu/x86/x86_32.ad"

    __ movl(Address(rsp, opnd_array(0)->disp(ra_,this,0)), opnd_array(1)->as_Register(ra_,this,idx1)/* src */);
  
#line 30519 "ad_x86.cpp"
  }
}

void MoveI2FPR_stack_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {

#line 1544 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0xD9 /*primary()*/));
  
#line 30534 "ad_x86.cpp"
  }
  {

#line 2176 "src/hotspot/cpu/x86/x86_32.ad"

    int rm_byte_opcode = (0x00);
    int base     = opnd_array(1)->base(ra_,this,idx1);
    int index    = opnd_array(1)->index(ra_,this,idx1);
    int scale    = opnd_array(1)->scale();
    int displace = opnd_array(1)->disp(ra_,this,idx1);
    assert( opnd_array(1)->disp_reloc() == relocInfo::none, "No oops here because no reloc info allowed" );
    encode_RegMem(cbuf, rm_byte_opcode, base, index, scale, displace, relocInfo::none);
  
#line 30548 "ad_x86.cpp"
  }
  {

#line 2422 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode( cbuf, 0xDD );           // FSTP   ST(i)
    emit_d8( cbuf, 0xD8+opnd_array(0)->reg(ra_,this)/* dst */ );
  
#line 30557 "ad_x86.cpp"
  }
}

void MoveI2F_stack_reg_sseNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 11351 "src/hotspot/cpu/x86/x86_32.ad"

    __ movflt(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, Address(rsp, opnd_array(1)->disp(ra_,this,idx1)));
  
#line 30573 "ad_x86.cpp"
  }
}

void MoveI2F_reg_reg_sseNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 11364 "src/hotspot/cpu/x86/x86_32.ad"

    __ movdl(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_Register(ra_,this,idx1)/* src */);
  
#line 30589 "ad_x86.cpp"
  }
}

void MoveD2L_stack_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {

#line 1544 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0x8B /*primary()*/));
  
#line 30604 "ad_x86.cpp"
  }
  {

#line 2096 "src/hotspot/cpu/x86/x86_32.ad"

    int reg_encoding = opnd_array(0)->reg(ra_,this)/* dst */;
    int base  = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int displace = opnd_array(1)->disp(ra_,this,idx1);
    relocInfo::relocType disp_reloc = opnd_array(1)->disp_reloc();
    encode_RegMem(cbuf, reg_encoding, base, index, scale, displace, disp_reloc);
  
#line 30618 "ad_x86.cpp"
  }
  {

#line 1549 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0x8B /*secondary()*/));
  
#line 30626 "ad_x86.cpp"
  }
  {

#line 2106 "src/hotspot/cpu/x86/x86_32.ad"

    int reg_encoding = HIGH_FROM_LOW_ENC(opnd_array(0)->reg(ra_,this)/* dst */);  // Hi register of pair, computed from lo
    int base  = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int displace = opnd_array(1)->disp(ra_,this,idx1) + 4;      // Offset is 4 further in memory
    assert( opnd_array(1)->disp_reloc() == relocInfo::none, "Cannot add 4 to oop" );
    encode_RegMem(cbuf, reg_encoding, base, index, scale, displace, relocInfo::none);
  
#line 30640 "ad_x86.cpp"
  }
}

void MoveDPR2L_reg_stackNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {

#line 2444 "src/hotspot/cpu/x86/x86_32.ad"

    int pop = 0x02;
    if (opnd_array(1)->reg(ra_,this,idx1)/* src */ != FPR1L_enc) {
      emit_opcode( cbuf, 0xD9 );         // FLD    ST(i-1)
      emit_d8( cbuf, 0xC0-1+opnd_array(1)->reg(ra_,this,idx1)/* src */ );
      pop = 0x03;
    }
    store_to_stackslot( cbuf, 0xDD, pop, opnd_array(0)->disp(ra_,this,0) ); // FST<P>_D  [ESP+dst]
  
#line 30661 "ad_x86.cpp"
  }
}

void MoveD2L_reg_stack_sseNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 11399 "src/hotspot/cpu/x86/x86_32.ad"

    __ movdbl(Address(rsp, opnd_array(0)->disp(ra_,this,0)), opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */);
  
#line 30677 "ad_x86.cpp"
  }
}

void MoveD2L_reg_reg_sseNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// tmp
  {
    C2_MacroAssembler _masm(&cbuf);

#line 11413 "src/hotspot/cpu/x86/x86_32.ad"

    __ movdl(opnd_array(0)->as_Register(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */);
    __ pshuflw(opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* tmp */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, 0x4e);
    __ movdl(HIGH_FROM_LOW(opnd_array(0)->as_Register(ra_,this)/* dst */), opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* tmp */);
  
#line 30696 "ad_x86.cpp"
  }
}

void MoveL2D_reg_stackNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {

#line 1544 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0x89 /*primary()*/));
  
#line 30711 "ad_x86.cpp"
  }
  {

#line 2096 "src/hotspot/cpu/x86/x86_32.ad"

    int reg_encoding = opnd_array(1)->reg(ra_,this,idx1)/* src */;
    int base  = opnd_array(0)->base(ra_,this,idx0);
    int index = opnd_array(0)->index(ra_,this,idx0);
    int scale = opnd_array(0)->scale();
    int displace = opnd_array(0)->disp(ra_,this,0);
    relocInfo::relocType disp_reloc = opnd_array(0)->disp_reloc();
    encode_RegMem(cbuf, reg_encoding, base, index, scale, displace, disp_reloc);
  
#line 30725 "ad_x86.cpp"
  }
  {

#line 1549 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0x89 /*secondary()*/));
  
#line 30733 "ad_x86.cpp"
  }
  {

#line 2106 "src/hotspot/cpu/x86/x86_32.ad"

    int reg_encoding = HIGH_FROM_LOW_ENC(opnd_array(1)->reg(ra_,this,idx1)/* src */);  // Hi register of pair, computed from lo
    int base  = opnd_array(0)->base(ra_,this,idx0);
    int index = opnd_array(0)->index(ra_,this,idx0);
    int scale = opnd_array(0)->scale();
    int displace = opnd_array(0)->disp(ra_,this,0) + 4;      // Offset is 4 further in memory
    assert( opnd_array(0)->disp_reloc() == relocInfo::none, "Cannot add 4 to oop" );
    encode_RegMem(cbuf, reg_encoding, base, index, scale, displace, relocInfo::none);
  
#line 30747 "ad_x86.cpp"
  }
}

void MoveL2DPR_stack_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {

#line 1544 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0xDD /*primary()*/));
  
#line 30762 "ad_x86.cpp"
  }
  {

#line 2176 "src/hotspot/cpu/x86/x86_32.ad"

    int rm_byte_opcode = (0x00);
    int base     = opnd_array(1)->base(ra_,this,idx1);
    int index    = opnd_array(1)->index(ra_,this,idx1);
    int scale    = opnd_array(1)->scale();
    int displace = opnd_array(1)->disp(ra_,this,idx1);
    assert( opnd_array(1)->disp_reloc() == relocInfo::none, "No oops here because no reloc info allowed" );
    encode_RegMem(cbuf, rm_byte_opcode, base, index, scale, displace, relocInfo::none);
  
#line 30776 "ad_x86.cpp"
  }
  {

#line 2367 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode( cbuf, 0xDD );           // FSTP   ST(i)
    emit_d8( cbuf, 0xD8+opnd_array(0)->reg(ra_,this)/* dst */ );
  
#line 30785 "ad_x86.cpp"
  }
}

void MoveL2D_stack_reg_sseNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 11456 "src/hotspot/cpu/x86/x86_32.ad"

    __ movdbl(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, Address(rsp, opnd_array(1)->disp(ra_,this,idx1)));
  
#line 30801 "ad_x86.cpp"
  }
}

void MoveL2D_stack_reg_sse_partialNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 11469 "src/hotspot/cpu/x86/x86_32.ad"

    __ movdbl(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, Address(rsp, opnd_array(1)->disp(ra_,this,idx1)));
  
#line 30817 "ad_x86.cpp"
  }
}

void MoveL2D_reg_reg_sseNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// dst
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// tmp
  {
    C2_MacroAssembler _masm(&cbuf);

#line 11483 "src/hotspot/cpu/x86/x86_32.ad"

    __ movdl(opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* dst */, opnd_array(1)->as_Register(ra_,this,idx1)/* src */);
    __ movdl(opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* tmp */, HIGH_FROM_LOW(opnd_array(1)->as_Register(ra_,this,idx1)/* src */));
    __ punpckldq(opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* dst */, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* tmp */);
  
#line 30837 "ad_x86.cpp"
  }
}

void compressBitsL_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// mask
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// rtmp
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// xtmp
  {
    C2_MacroAssembler _masm(&cbuf);

#line 11498 "src/hotspot/cpu/x86/x86_32.ad"

    Label exit, partail_result;
    // Parallely extract both upper and lower 32 bits of source into destination register pair.
    // Merge the results of upper and lower destination registers such that upper destination
    // results are contiguously laid out after the lower destination result.
    __ pextl(opnd_array(0)->as_Register(ra_,this)/* dst */, opnd_array(1)->as_Register(ra_,this,idx1)/* src */, opnd_array(2)->as_Register(ra_,this,idx2)/* mask */);
    __ pextl(HIGH_FROM_LOW(opnd_array(0)->as_Register(ra_,this)/* dst */), HIGH_FROM_LOW(opnd_array(1)->as_Register(ra_,this,idx1)/* src */), HIGH_FROM_LOW(opnd_array(2)->as_Register(ra_,this,idx2)/* mask */));
    __ popcntl(opnd_array(3)->as_Register(ra_,this,idx3)/* rtmp */, opnd_array(2)->as_Register(ra_,this,idx2)/* mask */);
    // Skip merging if bit count of lower mask register is equal to 32 (register size).
    __ cmpl(opnd_array(3)->as_Register(ra_,this,idx3)/* rtmp */, 32);
    __ jccb(Assembler::equal, exit);
    // Due to constraint on number of GPRs on 32 bit target, using XMM register as potential spill slot.
    __ movdl(opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* xtmp */, opnd_array(3)->as_Register(ra_,this,idx3)/* rtmp */);
    // Shift left the contents of upper destination register by true bit count of lower mask register
    // and merge with lower destination register.
    __ shlxl(opnd_array(3)->as_Register(ra_,this,idx3)/* rtmp */, HIGH_FROM_LOW(opnd_array(0)->as_Register(ra_,this)/* dst */), opnd_array(3)->as_Register(ra_,this,idx3)/* rtmp */);
    __ orl(opnd_array(0)->as_Register(ra_,this)/* dst */, opnd_array(3)->as_Register(ra_,this,idx3)/* rtmp */);
    __ movdl(opnd_array(3)->as_Register(ra_,this,idx3)/* rtmp */, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* xtmp */);
    // Zero out upper destination register if true bit count of lower 32 bit mask is zero
    // since contents of upper destination have already been copied to lower destination
    // register.
    __ cmpl(opnd_array(3)->as_Register(ra_,this,idx3)/* rtmp */, 0);
    __ jccb(Assembler::greater, partail_result);
    __ movl(HIGH_FROM_LOW(opnd_array(0)->as_Register(ra_,this)/* dst */), 0);
    __ jmp(exit);
    __ bind(partail_result);
    // Perform right shift over upper destination register to move out bits already copied
    // to lower destination register.
    __ subl(opnd_array(3)->as_Register(ra_,this,idx3)/* rtmp */, 32);
    __ negl(opnd_array(3)->as_Register(ra_,this,idx3)/* rtmp */);
    __ shrxl(HIGH_FROM_LOW(opnd_array(0)->as_Register(ra_,this)/* dst */), HIGH_FROM_LOW(opnd_array(0)->as_Register(ra_,this)/* dst */), opnd_array(3)->as_Register(ra_,this,idx3)/* rtmp */);
    __ bind(exit);
  
#line 30886 "ad_x86.cpp"
  }
}

void expandBitsL_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// mask
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// rtmp
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// xtmp
  {
    C2_MacroAssembler _masm(&cbuf);

#line 11539 "src/hotspot/cpu/x86/x86_32.ad"

    // Extraction operation sequentially reads the bits from source register starting from LSB
    // and lays them out into destination register at bit locations corresponding to true bits
    // in mask register. Thus number of source bits read are equal to combined true bit count
    // of mask register pair.
    Label exit, mask_clipping;
    __ pdepl(opnd_array(0)->as_Register(ra_,this)/* dst */, opnd_array(1)->as_Register(ra_,this,idx1)/* src */, opnd_array(2)->as_Register(ra_,this,idx2)/* mask */);
    __ pdepl(HIGH_FROM_LOW(opnd_array(0)->as_Register(ra_,this)/* dst */), HIGH_FROM_LOW(opnd_array(1)->as_Register(ra_,this,idx1)/* src */), HIGH_FROM_LOW(opnd_array(2)->as_Register(ra_,this,idx2)/* mask */));
    __ popcntl(opnd_array(3)->as_Register(ra_,this,idx3)/* rtmp */, opnd_array(2)->as_Register(ra_,this,idx2)/* mask */);
    // If true bit count of lower mask register is 32 then none of bit of lower source register
    // will feed to upper destination register.
    __ cmpl(opnd_array(3)->as_Register(ra_,this,idx3)/* rtmp */, 32);
    __ jccb(Assembler::equal, exit);
    // Due to constraint on number of GPRs on 32 bit target, using XMM register as potential spill slot.
    __ movdl(opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* xtmp */, opnd_array(3)->as_Register(ra_,this,idx3)/* rtmp */);
    // Shift right the contents of lower source register to remove already consumed bits.
    __ shrxl(opnd_array(3)->as_Register(ra_,this,idx3)/* rtmp */, opnd_array(1)->as_Register(ra_,this,idx1)/* src */, opnd_array(3)->as_Register(ra_,this,idx3)/* rtmp */);
    // Extract the bits from lower source register starting from LSB under the influence
    // of upper mask register.
    __ pdepl(HIGH_FROM_LOW(opnd_array(0)->as_Register(ra_,this)/* dst */), opnd_array(3)->as_Register(ra_,this,idx3)/* rtmp */, HIGH_FROM_LOW(opnd_array(2)->as_Register(ra_,this,idx2)/* mask */));
    __ movdl(opnd_array(3)->as_Register(ra_,this,idx3)/* rtmp */, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* xtmp */);
    __ subl(opnd_array(3)->as_Register(ra_,this,idx3)/* rtmp */, 32);
    __ negl(opnd_array(3)->as_Register(ra_,this,idx3)/* rtmp */);
    __ movdl(opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* xtmp */, opnd_array(2)->as_Register(ra_,this,idx2)/* mask */);
    __ movl(opnd_array(2)->as_Register(ra_,this,idx2)/* mask */, HIGH_FROM_LOW(opnd_array(2)->as_Register(ra_,this,idx2)/* mask */));
    // Clear the set bits in upper mask register which have been used to extract the contents
    // from lower source register.
    __ bind(mask_clipping);
    __ blsrl(opnd_array(2)->as_Register(ra_,this,idx2)/* mask */, opnd_array(2)->as_Register(ra_,this,idx2)/* mask */);
    __ decrementl(opnd_array(3)->as_Register(ra_,this,idx3)/* rtmp */, 1);
    __ jccb(Assembler::greater, mask_clipping);
    // Starting from LSB extract the bits from upper source register under the influence of
    // remaining set bits in upper mask register.
    __ pdepl(opnd_array(3)->as_Register(ra_,this,idx3)/* rtmp */, HIGH_FROM_LOW(opnd_array(1)->as_Register(ra_,this,idx1)/* src */), opnd_array(2)->as_Register(ra_,this,idx2)/* mask */);
    // Merge the partial results extracted from lower and upper source register bits.
    __ orl(HIGH_FROM_LOW(opnd_array(0)->as_Register(ra_,this)/* dst */), opnd_array(3)->as_Register(ra_,this,idx3)/* rtmp */);
    __ movdl(opnd_array(2)->as_Register(ra_,this,idx2)/* mask */, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* xtmp */);
    __ bind(exit);
  
#line 30941 "ad_x86.cpp"
  }
}

void rep_stosNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// cnt
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// base
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// tmp
  {
    C2_MacroAssembler _masm(&cbuf);

#line 11636 "src/hotspot/cpu/x86/x86_32.ad"

    __ clear_mem(opnd_array(2)->as_Register(ra_,this,idx2)/* base */, opnd_array(1)->as_Register(ra_,this,idx1)/* cnt */, as_Register(EAX_enc),
                 opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* tmp */, false, knoreg);
  
#line 30960 "ad_x86.cpp"
  }
}

void rep_stos_evexNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// cnt
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// base
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// tmp
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// ktmp
  {
    C2_MacroAssembler _masm(&cbuf);

#line 11697 "src/hotspot/cpu/x86/x86_32.ad"

    __ clear_mem(opnd_array(2)->as_Register(ra_,this,idx2)/* base */, opnd_array(1)->as_Register(ra_,this,idx1)/* cnt */, as_Register(EAX_enc),
                 opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* tmp */, false, opnd_array(4)->as_KRegister(ra_,this,idx4)/* ktmp */);
  
#line 30980 "ad_x86.cpp"
  }
}

void rep_stos_largeNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// cnt
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// base
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// tmp
  {
    C2_MacroAssembler _masm(&cbuf);

#line 11747 "src/hotspot/cpu/x86/x86_32.ad"

    __ clear_mem(opnd_array(2)->as_Register(ra_,this,idx2)/* base */, opnd_array(1)->as_Register(ra_,this,idx1)/* cnt */, as_Register(EAX_enc),
                 opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* tmp */, true, knoreg);
  
#line 30999 "ad_x86.cpp"
  }
}

void rep_stos_large_evexNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// cnt
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// base
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// tmp
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// ktmp
  {
    C2_MacroAssembler _masm(&cbuf);

#line 11797 "src/hotspot/cpu/x86/x86_32.ad"

    __ clear_mem(opnd_array(2)->as_Register(ra_,this,idx2)/* base */, opnd_array(1)->as_Register(ra_,this,idx1)/* cnt */, as_Register(EAX_enc),
                 opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* tmp */, true, opnd_array(4)->as_KRegister(ra_,this,idx4)/* ktmp */);
  
#line 31019 "ad_x86.cpp"
  }
}

void rep_stos_imNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// cnt
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// base
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// ktmp
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// tmp
  unsigned idx5 = idx4 + opnd_array(4)->num_edges(); 	// zero
  {
    C2_MacroAssembler _masm(&cbuf);

#line 11813 "src/hotspot/cpu/x86/x86_32.ad"

   __ clear_mem(opnd_array(2)->as_Register(ra_,this,idx2)/* base */, opnd_array(1)->constant(), opnd_array(5)->as_Register(ra_,this,idx5)/* zero */, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* tmp */, opnd_array(3)->as_KRegister(ra_,this,idx3)/* ktmp */);
  
#line 31039 "ad_x86.cpp"
  }
}

void string_compareLNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// str1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// cnt1
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// str2
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// cnt2
  unsigned idx5 = idx4 + opnd_array(4)->num_edges(); 	// tmp1
  {
    C2_MacroAssembler _masm(&cbuf);

#line 11826 "src/hotspot/cpu/x86/x86_32.ad"

    __ string_compare(opnd_array(1)->as_Register(ra_,this,idx1)/* str1 */, opnd_array(3)->as_Register(ra_,this,idx3)/* str2 */,
                      opnd_array(2)->as_Register(ra_,this,idx2)/* cnt1 */, opnd_array(4)->as_Register(ra_,this,idx4)/* cnt2 */, opnd_array(0)->as_Register(ra_,this)/* result */,
                      opnd_array(5)->as_XMMRegister(ra_,this,idx5)/* tmp1 */, StrIntrinsicNode::LL, knoreg);
  
#line 31061 "ad_x86.cpp"
  }
}

void string_compareL_evexNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// str1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// cnt1
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// str2
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// cnt2
  unsigned idx5 = idx4 + opnd_array(4)->num_edges(); 	// tmp1
  unsigned idx6 = idx5 + opnd_array(5)->num_edges(); 	// ktmp
  {
    C2_MacroAssembler _masm(&cbuf);

#line 11841 "src/hotspot/cpu/x86/x86_32.ad"

    __ string_compare(opnd_array(1)->as_Register(ra_,this,idx1)/* str1 */, opnd_array(3)->as_Register(ra_,this,idx3)/* str2 */,
                      opnd_array(2)->as_Register(ra_,this,idx2)/* cnt1 */, opnd_array(4)->as_Register(ra_,this,idx4)/* cnt2 */, opnd_array(0)->as_Register(ra_,this)/* result */,
                      opnd_array(5)->as_XMMRegister(ra_,this,idx5)/* tmp1 */, StrIntrinsicNode::LL, opnd_array(6)->as_KRegister(ra_,this,idx6)/* ktmp */);
  
#line 31084 "ad_x86.cpp"
  }
}

void string_compareUNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// str1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// cnt1
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// str2
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// cnt2
  unsigned idx5 = idx4 + opnd_array(4)->num_edges(); 	// tmp1
  {
    C2_MacroAssembler _masm(&cbuf);

#line 11856 "src/hotspot/cpu/x86/x86_32.ad"

    __ string_compare(opnd_array(1)->as_Register(ra_,this,idx1)/* str1 */, opnd_array(3)->as_Register(ra_,this,idx3)/* str2 */,
                      opnd_array(2)->as_Register(ra_,this,idx2)/* cnt1 */, opnd_array(4)->as_Register(ra_,this,idx4)/* cnt2 */, opnd_array(0)->as_Register(ra_,this)/* result */,
                      opnd_array(5)->as_XMMRegister(ra_,this,idx5)/* tmp1 */, StrIntrinsicNode::UU, knoreg);
  
#line 31106 "ad_x86.cpp"
  }
}

void string_compareU_evexNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// str1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// cnt1
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// str2
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// cnt2
  unsigned idx5 = idx4 + opnd_array(4)->num_edges(); 	// tmp1
  unsigned idx6 = idx5 + opnd_array(5)->num_edges(); 	// ktmp
  {
    C2_MacroAssembler _masm(&cbuf);

#line 11871 "src/hotspot/cpu/x86/x86_32.ad"

    __ string_compare(opnd_array(1)->as_Register(ra_,this,idx1)/* str1 */, opnd_array(3)->as_Register(ra_,this,idx3)/* str2 */,
                      opnd_array(2)->as_Register(ra_,this,idx2)/* cnt1 */, opnd_array(4)->as_Register(ra_,this,idx4)/* cnt2 */, opnd_array(0)->as_Register(ra_,this)/* result */,
                      opnd_array(5)->as_XMMRegister(ra_,this,idx5)/* tmp1 */, StrIntrinsicNode::UU, opnd_array(6)->as_KRegister(ra_,this,idx6)/* ktmp */);
  
#line 31129 "ad_x86.cpp"
  }
}

void string_compareLUNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// str1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// cnt1
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// str2
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// cnt2
  unsigned idx5 = idx4 + opnd_array(4)->num_edges(); 	// tmp1
  {
    C2_MacroAssembler _masm(&cbuf);

#line 11886 "src/hotspot/cpu/x86/x86_32.ad"

    __ string_compare(opnd_array(1)->as_Register(ra_,this,idx1)/* str1 */, opnd_array(3)->as_Register(ra_,this,idx3)/* str2 */,
                      opnd_array(2)->as_Register(ra_,this,idx2)/* cnt1 */, opnd_array(4)->as_Register(ra_,this,idx4)/* cnt2 */, opnd_array(0)->as_Register(ra_,this)/* result */,
                      opnd_array(5)->as_XMMRegister(ra_,this,idx5)/* tmp1 */, StrIntrinsicNode::LU, knoreg);
  
#line 31151 "ad_x86.cpp"
  }
}

void string_compareLU_evexNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// str1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// cnt1
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// str2
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// cnt2
  unsigned idx5 = idx4 + opnd_array(4)->num_edges(); 	// tmp1
  unsigned idx6 = idx5 + opnd_array(5)->num_edges(); 	// ktmp
  {
    C2_MacroAssembler _masm(&cbuf);

#line 11901 "src/hotspot/cpu/x86/x86_32.ad"

    __ string_compare(opnd_array(1)->as_Register(ra_,this,idx1)/* str1 */, opnd_array(3)->as_Register(ra_,this,idx3)/* str2 */,
                      opnd_array(2)->as_Register(ra_,this,idx2)/* cnt1 */, opnd_array(4)->as_Register(ra_,this,idx4)/* cnt2 */, opnd_array(0)->as_Register(ra_,this)/* result */,
                      opnd_array(5)->as_XMMRegister(ra_,this,idx5)/* tmp1 */, StrIntrinsicNode::LU, opnd_array(6)->as_KRegister(ra_,this,idx6)/* ktmp */);
  
#line 31174 "ad_x86.cpp"
  }
}

void string_compareULNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// str1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// cnt1
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// str2
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// cnt2
  unsigned idx5 = idx4 + opnd_array(4)->num_edges(); 	// tmp1
  {
    C2_MacroAssembler _masm(&cbuf);

#line 11916 "src/hotspot/cpu/x86/x86_32.ad"

    __ string_compare(opnd_array(3)->as_Register(ra_,this,idx3)/* str2 */, opnd_array(1)->as_Register(ra_,this,idx1)/* str1 */,
                      opnd_array(4)->as_Register(ra_,this,idx4)/* cnt2 */, opnd_array(2)->as_Register(ra_,this,idx2)/* cnt1 */, opnd_array(0)->as_Register(ra_,this)/* result */,
                      opnd_array(5)->as_XMMRegister(ra_,this,idx5)/* tmp1 */, StrIntrinsicNode::UL, knoreg);
  
#line 31196 "ad_x86.cpp"
  }
}

void string_compareUL_evexNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// str1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// cnt1
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// str2
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// cnt2
  unsigned idx5 = idx4 + opnd_array(4)->num_edges(); 	// tmp1
  unsigned idx6 = idx5 + opnd_array(5)->num_edges(); 	// ktmp
  {
    C2_MacroAssembler _masm(&cbuf);

#line 11931 "src/hotspot/cpu/x86/x86_32.ad"

    __ string_compare(opnd_array(3)->as_Register(ra_,this,idx3)/* str2 */, opnd_array(1)->as_Register(ra_,this,idx1)/* str1 */,
                      opnd_array(4)->as_Register(ra_,this,idx4)/* cnt2 */, opnd_array(2)->as_Register(ra_,this,idx2)/* cnt1 */, opnd_array(0)->as_Register(ra_,this)/* result */,
                      opnd_array(5)->as_XMMRegister(ra_,this,idx5)/* tmp1 */, StrIntrinsicNode::UL, opnd_array(6)->as_KRegister(ra_,this,idx6)/* ktmp */);
  
#line 31219 "ad_x86.cpp"
  }
}

void string_equalsNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// str1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// str2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// cnt
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// tmp1
  unsigned idx5 = idx4 + opnd_array(4)->num_edges(); 	// tmp2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 11947 "src/hotspot/cpu/x86/x86_32.ad"

    __ arrays_equals(false, opnd_array(1)->as_Register(ra_,this,idx1)/* str1 */, opnd_array(2)->as_Register(ra_,this,idx2)/* str2 */,
                     opnd_array(3)->as_Register(ra_,this,idx3)/* cnt */, opnd_array(0)->as_Register(ra_,this)/* result */, as_Register(EBX_enc),
                     opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* tmp1 */, opnd_array(5)->as_XMMRegister(ra_,this,idx5)/* tmp2 */, false /* char */, knoreg);
  
#line 31241 "ad_x86.cpp"
  }
}

void string_equals_evexNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// str1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// str2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// cnt
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// tmp1
  unsigned idx5 = idx4 + opnd_array(4)->num_edges(); 	// tmp2
  unsigned idx6 = idx5 + opnd_array(5)->num_edges(); 	// ktmp
  {
    C2_MacroAssembler _masm(&cbuf);

#line 11963 "src/hotspot/cpu/x86/x86_32.ad"

    __ arrays_equals(false, opnd_array(1)->as_Register(ra_,this,idx1)/* str1 */, opnd_array(2)->as_Register(ra_,this,idx2)/* str2 */,
                     opnd_array(3)->as_Register(ra_,this,idx3)/* cnt */, opnd_array(0)->as_Register(ra_,this)/* result */, as_Register(EBX_enc),
                     opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* tmp1 */, opnd_array(5)->as_XMMRegister(ra_,this,idx5)/* tmp2 */, false /* char */, opnd_array(6)->as_KRegister(ra_,this,idx6)/* ktmp */);
  
#line 31264 "ad_x86.cpp"
  }
}

void string_indexof_conLNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// str1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// cnt1
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// str2
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// int_cnt2
  unsigned idx5 = idx4 + opnd_array(4)->num_edges(); 	// vec1
  {
    C2_MacroAssembler _masm(&cbuf);

#line 11981 "src/hotspot/cpu/x86/x86_32.ad"

    int icnt2 = (int)opnd_array(4)->constant();
    if (icnt2 >= 16) {
      // IndexOf for constant substrings with size >= 16 elements
      // which don't need to be loaded through stack.
      __ string_indexofC8(opnd_array(1)->as_Register(ra_,this,idx1)/* str1 */, opnd_array(3)->as_Register(ra_,this,idx3)/* str2 */,
                          opnd_array(2)->as_Register(ra_,this,idx2)/* cnt1 */, as_Register(EAX_enc),
                          icnt2, opnd_array(0)->as_Register(ra_,this)/* result */,
                          opnd_array(5)->as_XMMRegister(ra_,this,idx5)/* vec1 */, as_Register(ECX_enc), StrIntrinsicNode::LL);
    } else {
      // Small strings are loaded through stack if they cross page boundary.
      __ string_indexof(opnd_array(1)->as_Register(ra_,this,idx1)/* str1 */, opnd_array(3)->as_Register(ra_,this,idx3)/* str2 */,
                        opnd_array(2)->as_Register(ra_,this,idx2)/* cnt1 */, as_Register(EAX_enc),
                        icnt2, opnd_array(0)->as_Register(ra_,this)/* result */,
                        opnd_array(5)->as_XMMRegister(ra_,this,idx5)/* vec1 */, as_Register(ECX_enc), StrIntrinsicNode::LL);
    }
  
#line 31298 "ad_x86.cpp"
  }
}

void string_indexof_conUNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// str1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// cnt1
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// str2
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// int_cnt2
  unsigned idx5 = idx4 + opnd_array(4)->num_edges(); 	// vec1
  {
    C2_MacroAssembler _masm(&cbuf);

#line 12009 "src/hotspot/cpu/x86/x86_32.ad"

    int icnt2 = (int)opnd_array(4)->constant();
    if (icnt2 >= 8) {
      // IndexOf for constant substrings with size >= 8 elements
      // which don't need to be loaded through stack.
      __ string_indexofC8(opnd_array(1)->as_Register(ra_,this,idx1)/* str1 */, opnd_array(3)->as_Register(ra_,this,idx3)/* str2 */,
                          opnd_array(2)->as_Register(ra_,this,idx2)/* cnt1 */, as_Register(EAX_enc),
                          icnt2, opnd_array(0)->as_Register(ra_,this)/* result */,
                          opnd_array(5)->as_XMMRegister(ra_,this,idx5)/* vec1 */, as_Register(ECX_enc), StrIntrinsicNode::UU);
    } else {
      // Small strings are loaded through stack if they cross page boundary.
      __ string_indexof(opnd_array(1)->as_Register(ra_,this,idx1)/* str1 */, opnd_array(3)->as_Register(ra_,this,idx3)/* str2 */,
                        opnd_array(2)->as_Register(ra_,this,idx2)/* cnt1 */, as_Register(EAX_enc),
                        icnt2, opnd_array(0)->as_Register(ra_,this)/* result */,
                        opnd_array(5)->as_XMMRegister(ra_,this,idx5)/* vec1 */, as_Register(ECX_enc), StrIntrinsicNode::UU);
    }
  
#line 31332 "ad_x86.cpp"
  }
}

void string_indexof_conULNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// str1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// cnt1
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// str2
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// int_cnt2
  unsigned idx5 = idx4 + opnd_array(4)->num_edges(); 	// vec1
  {
    C2_MacroAssembler _masm(&cbuf);

#line 12037 "src/hotspot/cpu/x86/x86_32.ad"

    int icnt2 = (int)opnd_array(4)->constant();
    if (icnt2 >= 8) {
      // IndexOf for constant substrings with size >= 8 elements
      // which don't need to be loaded through stack.
      __ string_indexofC8(opnd_array(1)->as_Register(ra_,this,idx1)/* str1 */, opnd_array(3)->as_Register(ra_,this,idx3)/* str2 */,
                          opnd_array(2)->as_Register(ra_,this,idx2)/* cnt1 */, as_Register(EAX_enc),
                          icnt2, opnd_array(0)->as_Register(ra_,this)/* result */,
                          opnd_array(5)->as_XMMRegister(ra_,this,idx5)/* vec1 */, as_Register(ECX_enc), StrIntrinsicNode::UL);
    } else {
      // Small strings are loaded through stack if they cross page boundary.
      __ string_indexof(opnd_array(1)->as_Register(ra_,this,idx1)/* str1 */, opnd_array(3)->as_Register(ra_,this,idx3)/* str2 */,
                        opnd_array(2)->as_Register(ra_,this,idx2)/* cnt1 */, as_Register(EAX_enc),
                        icnt2, opnd_array(0)->as_Register(ra_,this)/* result */,
                        opnd_array(5)->as_XMMRegister(ra_,this,idx5)/* vec1 */, as_Register(ECX_enc), StrIntrinsicNode::UL);
    }
  
#line 31366 "ad_x86.cpp"
  }
}

void string_indexofLNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// str1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// cnt1
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// str2
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// cnt2
  unsigned idx5 = idx4 + opnd_array(4)->num_edges(); 	// vec1
  {
    C2_MacroAssembler _masm(&cbuf);

#line 12064 "src/hotspot/cpu/x86/x86_32.ad"

    __ string_indexof(opnd_array(1)->as_Register(ra_,this,idx1)/* str1 */, opnd_array(3)->as_Register(ra_,this,idx3)/* str2 */,
                      opnd_array(2)->as_Register(ra_,this,idx2)/* cnt1 */, opnd_array(4)->as_Register(ra_,this,idx4)/* cnt2 */,
                      (-1), opnd_array(0)->as_Register(ra_,this)/* result */,
                      opnd_array(5)->as_XMMRegister(ra_,this,idx5)/* vec1 */, as_Register(ECX_enc), StrIntrinsicNode::LL);
  
#line 31389 "ad_x86.cpp"
  }
}

void string_indexofUNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// str1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// cnt1
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// str2
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// cnt2
  unsigned idx5 = idx4 + opnd_array(4)->num_edges(); 	// vec1
  {
    C2_MacroAssembler _masm(&cbuf);

#line 12080 "src/hotspot/cpu/x86/x86_32.ad"

    __ string_indexof(opnd_array(1)->as_Register(ra_,this,idx1)/* str1 */, opnd_array(3)->as_Register(ra_,this,idx3)/* str2 */,
                      opnd_array(2)->as_Register(ra_,this,idx2)/* cnt1 */, opnd_array(4)->as_Register(ra_,this,idx4)/* cnt2 */,
                      (-1), opnd_array(0)->as_Register(ra_,this)/* result */,
                      opnd_array(5)->as_XMMRegister(ra_,this,idx5)/* vec1 */, as_Register(ECX_enc), StrIntrinsicNode::UU);
  
#line 31412 "ad_x86.cpp"
  }
}

void string_indexofULNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// str1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// cnt1
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// str2
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// cnt2
  unsigned idx5 = idx4 + opnd_array(4)->num_edges(); 	// vec1
  {
    C2_MacroAssembler _masm(&cbuf);

#line 12096 "src/hotspot/cpu/x86/x86_32.ad"

    __ string_indexof(opnd_array(1)->as_Register(ra_,this,idx1)/* str1 */, opnd_array(3)->as_Register(ra_,this,idx3)/* str2 */,
                      opnd_array(2)->as_Register(ra_,this,idx2)/* cnt1 */, opnd_array(4)->as_Register(ra_,this,idx4)/* cnt2 */,
                      (-1), opnd_array(0)->as_Register(ra_,this)/* result */,
                      opnd_array(5)->as_XMMRegister(ra_,this,idx5)/* vec1 */, as_Register(ECX_enc), StrIntrinsicNode::UL);
  
#line 31435 "ad_x86.cpp"
  }
}

void string_indexof_charNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// str1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// cnt1
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// ch
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// vec1
  unsigned idx5 = idx4 + opnd_array(4)->num_edges(); 	// vec2
  unsigned idx6 = idx5 + opnd_array(5)->num_edges(); 	// vec3
  unsigned idx7 = idx6 + opnd_array(6)->num_edges(); 	// tmp
  {
    C2_MacroAssembler _masm(&cbuf);

#line 12111 "src/hotspot/cpu/x86/x86_32.ad"

    __ string_indexof_char(opnd_array(1)->as_Register(ra_,this,idx1)/* str1 */, opnd_array(2)->as_Register(ra_,this,idx2)/* cnt1 */, opnd_array(3)->as_Register(ra_,this,idx3)/* ch */, opnd_array(0)->as_Register(ra_,this)/* result */,
                           opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* vec1 */, opnd_array(5)->as_XMMRegister(ra_,this,idx5)/* vec2 */, opnd_array(6)->as_XMMRegister(ra_,this,idx6)/* vec3 */, opnd_array(7)->as_Register(ra_,this,idx7)/* tmp */);
  
#line 31458 "ad_x86.cpp"
  }
}

void stringL_indexof_charNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// str1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// cnt1
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// ch
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// vec1
  unsigned idx5 = idx4 + opnd_array(4)->num_edges(); 	// vec2
  unsigned idx6 = idx5 + opnd_array(5)->num_edges(); 	// vec3
  unsigned idx7 = idx6 + opnd_array(6)->num_edges(); 	// tmp
  {
    C2_MacroAssembler _masm(&cbuf);

#line 12124 "src/hotspot/cpu/x86/x86_32.ad"

    __ stringL_indexof_char(opnd_array(1)->as_Register(ra_,this,idx1)/* str1 */, opnd_array(2)->as_Register(ra_,this,idx2)/* cnt1 */, opnd_array(3)->as_Register(ra_,this,idx3)/* ch */, opnd_array(0)->as_Register(ra_,this)/* result */,
                           opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* vec1 */, opnd_array(5)->as_XMMRegister(ra_,this,idx5)/* vec2 */, opnd_array(6)->as_XMMRegister(ra_,this,idx6)/* vec3 */, opnd_array(7)->as_Register(ra_,this,idx7)/* tmp */);
  
#line 31481 "ad_x86.cpp"
  }
}

void array_equalsBNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// ary1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// ary2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// tmp1
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// tmp2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 12142 "src/hotspot/cpu/x86/x86_32.ad"

    __ arrays_equals(true, opnd_array(1)->as_Register(ra_,this,idx1)/* ary1 */, opnd_array(2)->as_Register(ra_,this,idx2)/* ary2 */,
                     as_Register(ECX_enc), opnd_array(0)->as_Register(ra_,this)/* result */, as_Register(EBX_enc),
                     opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* tmp1 */, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* tmp2 */, false /* char */, knoreg);
  
#line 31502 "ad_x86.cpp"
  }
}

void array_equalsB_evexNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// ary1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// ary2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// tmp1
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// tmp2
  unsigned idx5 = idx4 + opnd_array(4)->num_edges(); 	// ktmp
  {
    C2_MacroAssembler _masm(&cbuf);

#line 12159 "src/hotspot/cpu/x86/x86_32.ad"

    __ arrays_equals(true, opnd_array(1)->as_Register(ra_,this,idx1)/* ary1 */, opnd_array(2)->as_Register(ra_,this,idx2)/* ary2 */,
                     as_Register(ECX_enc), opnd_array(0)->as_Register(ra_,this)/* result */, as_Register(EBX_enc),
                     opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* tmp1 */, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* tmp2 */, false /* char */, opnd_array(5)->as_KRegister(ra_,this,idx5)/* ktmp */);
  
#line 31524 "ad_x86.cpp"
  }
}

void array_equalsCNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// ary1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// ary2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// tmp1
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// tmp2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 12176 "src/hotspot/cpu/x86/x86_32.ad"

    __ arrays_equals(true, opnd_array(1)->as_Register(ra_,this,idx1)/* ary1 */, opnd_array(2)->as_Register(ra_,this,idx2)/* ary2 */,
                     as_Register(ECX_enc), opnd_array(0)->as_Register(ra_,this)/* result */, as_Register(EBX_enc),
                     opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* tmp1 */, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* tmp2 */, true /* char */, knoreg);
  
#line 31545 "ad_x86.cpp"
  }
}

void array_equalsC_evexNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// ary1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// ary2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// tmp1
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// tmp2
  unsigned idx5 = idx4 + opnd_array(4)->num_edges(); 	// ktmp
  {
    C2_MacroAssembler _masm(&cbuf);

#line 12193 "src/hotspot/cpu/x86/x86_32.ad"

    __ arrays_equals(true, opnd_array(1)->as_Register(ra_,this,idx1)/* ary1 */, opnd_array(2)->as_Register(ra_,this,idx2)/* ary2 */,
                     as_Register(ECX_enc), opnd_array(0)->as_Register(ra_,this)/* result */, as_Register(EBX_enc),
                     opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* tmp1 */, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* tmp2 */, true /* char */, opnd_array(5)->as_KRegister(ra_,this,idx5)/* ktmp */);
  
#line 31567 "ad_x86.cpp"
  }
}

void count_positivesNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// ary1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// len
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// tmp1
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// tmp2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 12209 "src/hotspot/cpu/x86/x86_32.ad"

    __ count_positives(opnd_array(1)->as_Register(ra_,this,idx1)/* ary1 */, opnd_array(2)->as_Register(ra_,this,idx2)/* len */,
                       opnd_array(0)->as_Register(ra_,this)/* result */, as_Register(EBX_enc),
                       opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* tmp1 */, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* tmp2 */, knoreg, knoreg);
  
#line 31588 "ad_x86.cpp"
  }
}

void count_positives_evexNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// ary1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// len
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// tmp1
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// tmp2
  unsigned idx5 = idx4 + opnd_array(4)->num_edges(); 	// ktmp1
  unsigned idx6 = idx5 + opnd_array(5)->num_edges(); 	// ktmp2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 12225 "src/hotspot/cpu/x86/x86_32.ad"

    __ count_positives(opnd_array(1)->as_Register(ra_,this,idx1)/* ary1 */, opnd_array(2)->as_Register(ra_,this,idx2)/* len */,
                       opnd_array(0)->as_Register(ra_,this)/* result */, as_Register(EBX_enc),
                       opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* tmp1 */, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* tmp2 */, opnd_array(5)->as_KRegister(ra_,this,idx5)/* ktmp1 */, opnd_array(6)->as_KRegister(ra_,this,idx6)/* ktmp2 */);
  
#line 31611 "ad_x86.cpp"
  }
}

void string_compressNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// dst
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// len
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// tmp1
  unsigned idx5 = idx4 + opnd_array(4)->num_edges(); 	// tmp2
  unsigned idx6 = idx5 + opnd_array(5)->num_edges(); 	// tmp3
  unsigned idx7 = idx6 + opnd_array(6)->num_edges(); 	// tmp4
  {
    C2_MacroAssembler _masm(&cbuf);

#line 12242 "src/hotspot/cpu/x86/x86_32.ad"

    __ char_array_compress(opnd_array(1)->as_Register(ra_,this,idx1)/* src */, opnd_array(2)->as_Register(ra_,this,idx2)/* dst */, opnd_array(3)->as_Register(ra_,this,idx3)/* len */,
                           opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* tmp1 */, opnd_array(5)->as_XMMRegister(ra_,this,idx5)/* tmp2 */, opnd_array(6)->as_XMMRegister(ra_,this,idx6)/* tmp3 */,
                           opnd_array(7)->as_XMMRegister(ra_,this,idx7)/* tmp4 */, as_Register(ECX_enc), opnd_array(0)->as_Register(ra_,this)/* result */,
                           knoreg, knoreg);
  
#line 31636 "ad_x86.cpp"
  }
}

void string_compress_evexNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// dst
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// len
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// tmp1
  unsigned idx5 = idx4 + opnd_array(4)->num_edges(); 	// tmp2
  unsigned idx6 = idx5 + opnd_array(5)->num_edges(); 	// tmp3
  unsigned idx7 = idx6 + opnd_array(6)->num_edges(); 	// tmp4
  unsigned idx8 = idx7 + opnd_array(7)->num_edges(); 	// ktmp1
  unsigned idx9 = idx8 + opnd_array(8)->num_edges(); 	// ktmp2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 12258 "src/hotspot/cpu/x86/x86_32.ad"

    __ char_array_compress(opnd_array(1)->as_Register(ra_,this,idx1)/* src */, opnd_array(2)->as_Register(ra_,this,idx2)/* dst */, opnd_array(3)->as_Register(ra_,this,idx3)/* len */,
                           opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* tmp1 */, opnd_array(5)->as_XMMRegister(ra_,this,idx5)/* tmp2 */, opnd_array(6)->as_XMMRegister(ra_,this,idx6)/* tmp3 */,
                           opnd_array(7)->as_XMMRegister(ra_,this,idx7)/* tmp4 */, as_Register(ECX_enc), opnd_array(0)->as_Register(ra_,this)/* result */,
                           opnd_array(8)->as_KRegister(ra_,this,idx8)/* ktmp1 */, opnd_array(9)->as_KRegister(ra_,this,idx9)/* ktmp2 */);
  
#line 31663 "ad_x86.cpp"
  }
}

void string_inflateNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// dst
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// len
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// tmp1
  unsigned idx5 = idx4 + opnd_array(4)->num_edges(); 	// tmp2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 12275 "src/hotspot/cpu/x86/x86_32.ad"

    __ byte_array_inflate(opnd_array(1)->as_Register(ra_,this,idx1)/* src */, opnd_array(2)->as_Register(ra_,this,idx2)/* dst */, opnd_array(3)->as_Register(ra_,this,idx3)/* len */,
                          opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* tmp1 */, opnd_array(5)->as_Register(ra_,this,idx5)/* tmp2 */, knoreg);
  
#line 31684 "ad_x86.cpp"
  }
}

void string_inflate_evexNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// dst
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// len
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// tmp1
  unsigned idx5 = idx4 + opnd_array(4)->num_edges(); 	// ktmp
  unsigned idx6 = idx5 + opnd_array(5)->num_edges(); 	// tmp2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 12289 "src/hotspot/cpu/x86/x86_32.ad"

    __ byte_array_inflate(opnd_array(1)->as_Register(ra_,this,idx1)/* src */, opnd_array(2)->as_Register(ra_,this,idx2)/* dst */, opnd_array(3)->as_Register(ra_,this,idx3)/* len */,
                          opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* tmp1 */, opnd_array(6)->as_Register(ra_,this,idx6)/* tmp2 */, opnd_array(5)->as_KRegister(ra_,this,idx5)/* ktmp */);
  
#line 31706 "ad_x86.cpp"
  }
}

void encode_iso_arrayNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// dst
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// len
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// tmp1
  unsigned idx5 = idx4 + opnd_array(4)->num_edges(); 	// tmp2
  unsigned idx6 = idx5 + opnd_array(5)->num_edges(); 	// tmp3
  unsigned idx7 = idx6 + opnd_array(6)->num_edges(); 	// tmp4
  {
    C2_MacroAssembler _masm(&cbuf);

#line 12305 "src/hotspot/cpu/x86/x86_32.ad"

    __ encode_iso_array(opnd_array(1)->as_Register(ra_,this,idx1)/* src */, opnd_array(2)->as_Register(ra_,this,idx2)/* dst */, opnd_array(3)->as_Register(ra_,this,idx3)/* len */,
                        opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* tmp1 */, opnd_array(5)->as_XMMRegister(ra_,this,idx5)/* tmp2 */, opnd_array(6)->as_XMMRegister(ra_,this,idx6)/* tmp3 */,
                        opnd_array(7)->as_XMMRegister(ra_,this,idx7)/* tmp4 */, as_Register(ECX_enc), opnd_array(0)->as_Register(ra_,this)/* result */, false);
  
#line 31730 "ad_x86.cpp"
  }
}

void encode_ascii_arrayNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// dst
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// len
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// tmp1
  unsigned idx5 = idx4 + opnd_array(4)->num_edges(); 	// tmp2
  unsigned idx6 = idx5 + opnd_array(5)->num_edges(); 	// tmp3
  unsigned idx7 = idx6 + opnd_array(6)->num_edges(); 	// tmp4
  {
    C2_MacroAssembler _masm(&cbuf);

#line 12322 "src/hotspot/cpu/x86/x86_32.ad"

    __ encode_iso_array(opnd_array(1)->as_Register(ra_,this,idx1)/* src */, opnd_array(2)->as_Register(ra_,this,idx2)/* dst */, opnd_array(3)->as_Register(ra_,this,idx3)/* len */,
                        opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* tmp1 */, opnd_array(5)->as_XMMRegister(ra_,this,idx5)/* tmp2 */, opnd_array(6)->as_XMMRegister(ra_,this,idx6)/* tmp3 */,
                        opnd_array(7)->as_XMMRegister(ra_,this,idx7)/* tmp4 */, as_Register(ECX_enc), opnd_array(0)->as_Register(ra_,this)/* result */, true);
  
#line 31754 "ad_x86.cpp"
  }
}

void compI_eRegNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// op1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// op2
  {

#line 1544 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0x3B /*primary()*/));
  
#line 31770 "ad_x86.cpp"
  }
  {

#line 1959 "src/hotspot/cpu/x86/x86_32.ad"

    emit_rm(cbuf, 0x3, opnd_array(1)->reg(ra_,this,idx1)/* op1 */, opnd_array(2)->reg(ra_,this,idx2)/* op2 */);
  
#line 31778 "ad_x86.cpp"
  }
}

void compI_eReg_immNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// op1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// op2
  {

#line 1634 "src/hotspot/cpu/x86/x86_32.ad"

    // Emit primary opcode and set sign-extend bit
    // Check for 8-bit immediate, and set sign extend bit in opcode
    if ((opnd_array(2)->constant() >= -128) && (opnd_array(2)->constant() <= 127)) {
      emit_opcode(cbuf, (0x81 /*primary()*/) | 0x02);    }
    else {                          // If 32-bit immediate
      emit_opcode(cbuf, (0x81 /*primary()*/));
    }
    // Emit r/m byte with secondary opcode, after primary opcode.
    emit_rm(cbuf, 0x3, (0x07 /*secondary()*/), opnd_array(1)->reg(ra_,this,idx1)/* op1 */);
  
#line 31802 "ad_x86.cpp"
  }
  {

#line 1646 "src/hotspot/cpu/x86/x86_32.ad"

    // Check for 8-bit immediate, and set sign extend bit in opcode
    if ((opnd_array(2)->constant() >= -128) && (opnd_array(2)->constant() <= 127)) {
      emit_d8(cbuf, opnd_array(2)->constant());
    }
    else {                          // If 32-bit immediate
      // Output immediate
      emit_d32(cbuf, opnd_array(2)->constant());
    }
  
#line 31817 "ad_x86.cpp"
  }
}

void compI_eReg_memNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// op1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// op2
  {

#line 1544 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0x3B /*primary()*/));
  
#line 31833 "ad_x86.cpp"
  }
  {

#line 2096 "src/hotspot/cpu/x86/x86_32.ad"

    int reg_encoding = opnd_array(1)->reg(ra_,this,idx1)/* op1 */;
    int base  = opnd_array(2)->base(ra_,this,idx2);
    int index = opnd_array(2)->index(ra_,this,idx2);
    int scale = opnd_array(2)->scale();
    int displace = opnd_array(2)->disp(ra_,this,idx2);
    relocInfo::relocType disp_reloc = opnd_array(2)->disp_reloc();
    encode_RegMem(cbuf, reg_encoding, base, index, scale, displace, disp_reloc);
  
#line 31847 "ad_x86.cpp"
  }
}

void testI_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// zero
  {

#line 1544 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0x85 /*primary()*/));
  
#line 31863 "ad_x86.cpp"
  }
  {

#line 1959 "src/hotspot/cpu/x86/x86_32.ad"

    emit_rm(cbuf, 0x3, opnd_array(1)->reg(ra_,this,idx1)/* src */, opnd_array(1)->reg(ra_,this,idx1)/* src */);
  
#line 31871 "ad_x86.cpp"
  }
}

void testI_reg_immNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// con
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// zero
  {

#line 1544 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0xF7 /*primary()*/));
  
#line 31888 "ad_x86.cpp"
  }
  {

#line 1697 "src/hotspot/cpu/x86/x86_32.ad"

    emit_rm(cbuf, 0x3, (0x00 /*secondary()*/), opnd_array(1)->reg(ra_,this,idx1)/* src */ );
  
#line 31896 "ad_x86.cpp"
  }
  {

#line 1985 "src/hotspot/cpu/x86/x86_32.ad"

    // Output immediate
    emit_d32(cbuf, opnd_array(2)->constant());
  
#line 31905 "ad_x86.cpp"
  }
}

void testI_reg_memNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// mem
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// zero
  {

#line 1544 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0x85 /*primary()*/));
  
#line 31922 "ad_x86.cpp"
  }
  {

#line 2096 "src/hotspot/cpu/x86/x86_32.ad"

    int reg_encoding = opnd_array(1)->reg(ra_,this,idx1)/* src */;
    int base  = opnd_array(2)->base(ra_,this,idx2);
    int index = opnd_array(2)->index(ra_,this,idx2);
    int scale = opnd_array(2)->scale();
    int displace = opnd_array(2)->disp(ra_,this,idx2);
    relocInfo::relocType disp_reloc = opnd_array(2)->disp_reloc();
    encode_RegMem(cbuf, reg_encoding, base, index, scale, displace, disp_reloc);
  
#line 31936 "ad_x86.cpp"
  }
}

void compU_eRegNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// op1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// op2
  {

#line 1544 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0x3B /*primary()*/));
  
#line 31952 "ad_x86.cpp"
  }
  {

#line 1959 "src/hotspot/cpu/x86/x86_32.ad"

    emit_rm(cbuf, 0x3, opnd_array(1)->reg(ra_,this,idx1)/* op1 */, opnd_array(2)->reg(ra_,this,idx2)/* op2 */);
  
#line 31960 "ad_x86.cpp"
  }
}

void compU_eReg_immNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// op1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// op2
  {

#line 1634 "src/hotspot/cpu/x86/x86_32.ad"

    // Emit primary opcode and set sign-extend bit
    // Check for 8-bit immediate, and set sign extend bit in opcode
    if ((opnd_array(2)->constant() >= -128) && (opnd_array(2)->constant() <= 127)) {
      emit_opcode(cbuf, (0x81 /*primary()*/) | 0x02);    }
    else {                          // If 32-bit immediate
      emit_opcode(cbuf, (0x81 /*primary()*/));
    }
    // Emit r/m byte with secondary opcode, after primary opcode.
    emit_rm(cbuf, 0x3, (0x07 /*secondary()*/), opnd_array(1)->reg(ra_,this,idx1)/* op1 */);
  
#line 31984 "ad_x86.cpp"
  }
  {

#line 1646 "src/hotspot/cpu/x86/x86_32.ad"

    // Check for 8-bit immediate, and set sign extend bit in opcode
    if ((opnd_array(2)->constant() >= -128) && (opnd_array(2)->constant() <= 127)) {
      emit_d8(cbuf, opnd_array(2)->constant());
    }
    else {                          // If 32-bit immediate
      // Output immediate
      emit_d32(cbuf, opnd_array(2)->constant());
    }
  
#line 31999 "ad_x86.cpp"
  }
}

void compU_eReg_memNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// op1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// op2
  {

#line 1544 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0x3B /*primary()*/));
  
#line 32015 "ad_x86.cpp"
  }
  {

#line 2096 "src/hotspot/cpu/x86/x86_32.ad"

    int reg_encoding = opnd_array(1)->reg(ra_,this,idx1)/* op1 */;
    int base  = opnd_array(2)->base(ra_,this,idx2);
    int index = opnd_array(2)->index(ra_,this,idx2);
    int scale = opnd_array(2)->scale();
    int displace = opnd_array(2)->disp(ra_,this,idx2);
    relocInfo::relocType disp_reloc = opnd_array(2)->disp_reloc();
    encode_RegMem(cbuf, reg_encoding, base, index, scale, displace, disp_reloc);
  
#line 32029 "ad_x86.cpp"
  }
}

void testU_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// zero
  {

#line 1544 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0x85 /*primary()*/));
  
#line 32045 "ad_x86.cpp"
  }
  {

#line 1959 "src/hotspot/cpu/x86/x86_32.ad"

    emit_rm(cbuf, 0x3, opnd_array(1)->reg(ra_,this,idx1)/* src */, opnd_array(1)->reg(ra_,this,idx1)/* src */);
  
#line 32053 "ad_x86.cpp"
  }
}

void compP_eRegNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// op1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// op2
  {

#line 1544 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0x3B /*primary()*/));
  
#line 32069 "ad_x86.cpp"
  }
  {

#line 1959 "src/hotspot/cpu/x86/x86_32.ad"

    emit_rm(cbuf, 0x3, opnd_array(1)->reg(ra_,this,idx1)/* op1 */, opnd_array(2)->reg(ra_,this,idx2)/* op2 */);
  
#line 32077 "ad_x86.cpp"
  }
}

void compP_eReg_immNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// op1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// op2
  {

#line 1634 "src/hotspot/cpu/x86/x86_32.ad"

    // Emit primary opcode and set sign-extend bit
    // Check for 8-bit immediate, and set sign extend bit in opcode
    if ((opnd_array(2)->constant() >= -128) && (opnd_array(2)->constant() <= 127)) {
      emit_opcode(cbuf, (0x81 /*primary()*/) | 0x02);    }
    else {                          // If 32-bit immediate
      emit_opcode(cbuf, (0x81 /*primary()*/));
    }
    // Emit r/m byte with secondary opcode, after primary opcode.
    emit_rm(cbuf, 0x3, (0x07 /*secondary()*/), opnd_array(1)->reg(ra_,this,idx1)/* op1 */);
  
#line 32101 "ad_x86.cpp"
  }
  {

#line 1646 "src/hotspot/cpu/x86/x86_32.ad"

    // Check for 8-bit immediate, and set sign extend bit in opcode
    if ((opnd_array(2)->constant() >= -128) && (opnd_array(2)->constant() <= 127)) {
      emit_d8(cbuf, opnd_array(2)->constant());
    }
    else {                          // If 32-bit immediate
      // Output immediate
      
if ( opnd_array(2)->constant_reloc() != relocInfo::none ) {
  emit_d32_reloc(cbuf, opnd_array(2)->constant(), opnd_array(2)->constant_reloc(), 0);
} else {
  emit_d32(cbuf, opnd_array(2)->constant());
};
    }
  
#line 32121 "ad_x86.cpp"
  }
}

void compP_eReg_memNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// op1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// op2
  {

#line 1544 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0x3B /*primary()*/));
  
#line 32137 "ad_x86.cpp"
  }
  {

#line 2096 "src/hotspot/cpu/x86/x86_32.ad"

    int reg_encoding = opnd_array(1)->reg(ra_,this,idx1)/* op1 */;
    int base  = opnd_array(2)->base(ra_,this,idx2);
    int index = opnd_array(2)->index(ra_,this,idx2);
    int scale = opnd_array(2)->scale();
    int displace = opnd_array(2)->disp(ra_,this,idx2);
    relocInfo::relocType disp_reloc = opnd_array(2)->disp_reloc();
    encode_RegMem(cbuf, reg_encoding, base, index, scale, displace, disp_reloc);
  
#line 32151 "ad_x86.cpp"
  }
}

void compP_mem_eRegNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// op1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// op2
  {

#line 1544 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0x3B /*primary()*/));
  
#line 32167 "ad_x86.cpp"
  }
  {

#line 2096 "src/hotspot/cpu/x86/x86_32.ad"

    int reg_encoding = opnd_array(1)->reg(ra_,this,idx1)/* op1 */;
    int base  = opnd_array(2)->base(ra_,this,idx2);
    int index = opnd_array(2)->index(ra_,this,idx2);
    int scale = opnd_array(2)->scale();
    int displace = opnd_array(2)->disp(ra_,this,idx2);
    relocInfo::relocType disp_reloc = opnd_array(2)->disp_reloc();
    encode_RegMem(cbuf, reg_encoding, base, index, scale, displace, disp_reloc);
  
#line 32181 "ad_x86.cpp"
  }
}

void testP_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// zero
  {

#line 1544 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0x85 /*primary()*/));
  
#line 32197 "ad_x86.cpp"
  }
  {

#line 1959 "src/hotspot/cpu/x86/x86_32.ad"

    emit_rm(cbuf, 0x3, opnd_array(1)->reg(ra_,this,idx1)/* src */, opnd_array(1)->reg(ra_,this,idx1)/* src */);
  
#line 32205 "ad_x86.cpp"
  }
}

void testP_Reg_memNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// op
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// zero
  {

#line 1544 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0xF7 /*primary()*/));
  
#line 32221 "ad_x86.cpp"
  }
  {

#line 2186 "src/hotspot/cpu/x86/x86_32.ad"

    int rm_byte_opcode = (0x00);
    int base     = opnd_array(1)->base(ra_,this,idx1);
    int index    = opnd_array(1)->index(ra_,this,idx1);
    int scale    = opnd_array(1)->scale();
    int displace = opnd_array(1)->disp(ra_,this,idx1);
    relocInfo::relocType disp_reloc = opnd_array(1)->disp_reloc(); // disp-as-oop when working with static globals
    encode_RegMem(cbuf, rm_byte_opcode, base, index, scale, displace, disp_reloc);
  
#line 32235 "ad_x86.cpp"
  }
  {

#line 2009 "src/hotspot/cpu/x86/x86_32.ad"

    emit_d32(cbuf,(0xFFFFFFFF));
  
#line 32243 "ad_x86.cpp"
  }
}

void minI_eRegNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {

#line 2206 "src/hotspot/cpu/x86/x86_32.ad"

    // Compare dst,src
    emit_opcode(cbuf,0x3B);
    emit_rm(cbuf, 0x3, opnd_array(1)->reg(ra_,this,idx1)/* dst */, opnd_array(2)->reg(ra_,this,idx2)/* src */);
    // jmp dst < src around move
    emit_opcode(cbuf,0x7C);
    emit_d8(cbuf,2);
    // move dst,src
    emit_opcode(cbuf,0x8B);
    emit_rm(cbuf, 0x3, opnd_array(1)->reg(ra_,this,idx1)/* dst */, opnd_array(2)->reg(ra_,this,idx2)/* src */);
  
#line 32267 "ad_x86.cpp"
  }
}

void maxI_eRegNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {

#line 2218 "src/hotspot/cpu/x86/x86_32.ad"

    // Compare dst,src
    emit_opcode(cbuf,0x3B);
    emit_rm(cbuf, 0x3, opnd_array(1)->reg(ra_,this,idx1)/* dst */, opnd_array(2)->reg(ra_,this,idx2)/* src */);
    // jmp dst > src around move
    emit_opcode(cbuf,0x7F);
    emit_d8(cbuf,2);
    // move dst,src
    emit_opcode(cbuf,0x8B);
    emit_rm(cbuf, 0x3, opnd_array(1)->reg(ra_,this,idx1)/* dst */, opnd_array(2)->reg(ra_,this,idx2)/* src */);
  
#line 32291 "ad_x86.cpp"
  }
}

void loopLimit_eRegNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// init
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// limit
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// stride
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// limit_hi
  unsigned idx5 = idx4 + opnd_array(4)->num_edges(); 	// tmp
  {
    C2_MacroAssembler _masm(&cbuf);

#line 12605 "src/hotspot/cpu/x86/x86_32.ad"

    int strd = (int)opnd_array(3)->constant();
    assert(strd != 1 && strd != -1, "sanity");
    int m1 = (strd > 0) ? 1 : -1;
    // Convert limit to long (EAX:EDX)
    __ cdql();
    // Convert init to long (init:tmp)
    __ movl(opnd_array(5)->as_Register(ra_,this,idx5)/* tmp */, opnd_array(1)->as_Register(ra_,this,idx1)/* init */);
    __ sarl(opnd_array(5)->as_Register(ra_,this,idx5)/* tmp */, 31);
    // $limit - $init
    __ subl(opnd_array(2)->as_Register(ra_,this,idx2)/* limit */, opnd_array(1)->as_Register(ra_,this,idx1)/* init */);
    __ sbbl(opnd_array(4)->as_Register(ra_,this,idx4)/* limit_hi */, opnd_array(5)->as_Register(ra_,this,idx5)/* tmp */);
    // + ($stride - 1)
    if (strd > 0) {
      __ addl(opnd_array(2)->as_Register(ra_,this,idx2)/* limit */, (strd - 1));
      __ adcl(opnd_array(4)->as_Register(ra_,this,idx4)/* limit_hi */, 0);
      __ movl(opnd_array(5)->as_Register(ra_,this,idx5)/* tmp */, strd);
    } else {
      __ addl(opnd_array(2)->as_Register(ra_,this,idx2)/* limit */, (strd + 1));
      __ adcl(opnd_array(4)->as_Register(ra_,this,idx4)/* limit_hi */, -1);
      __ lneg(opnd_array(4)->as_Register(ra_,this,idx4)/* limit_hi */, opnd_array(2)->as_Register(ra_,this,idx2)/* limit */);
      __ movl(opnd_array(5)->as_Register(ra_,this,idx5)/* tmp */, -strd);
    }
    // signed division: (EAX:EDX) / pos_stride
    __ idivl(opnd_array(5)->as_Register(ra_,this,idx5)/* tmp */);
    if (strd < 0) {
      // restore sign
      __ negl(opnd_array(5)->as_Register(ra_,this,idx5)/* tmp */);
    }
    // (EAX) * stride
    __ mull(opnd_array(5)->as_Register(ra_,this,idx5)/* tmp */);
    // + init (ignore upper bits)
    __ addl(opnd_array(2)->as_Register(ra_,this,idx2)/* limit */, opnd_array(1)->as_Register(ra_,this,idx1)/* init */);
  
#line 32342 "ad_x86.cpp"
  }
}

void jumpXtndNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  ra_->C->output()->constant_table().fill_jump_table(cbuf, (MachConstantNode*) this, _index2label);
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// 
  {
    C2_MacroAssembler _masm(&cbuf);

#line 12649 "src/hotspot/cpu/x86/x86_32.ad"

    // Jump to Address(table_base + switch_reg)
    Address index(noreg, opnd_array(1)->as_Register(ra_,this,idx1)/* switch_val */, Address::times_1);
    __ jump(ArrayAddress(InternalAddress(__ code()->consts()->start() + constant_offset()), index), noreg);
  
#line 32361 "ad_x86.cpp"
  }
}

void jumpXtndNode::eval_constant(Compile* C) {
  _constant = C->output()->constant_table().add_jump_table(this);
  // User did not define an encode section.
}
void jmpDirNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// 
  {
    C2_MacroAssembler _masm(&cbuf);

#line 12665 "src/hotspot/cpu/x86/x86_32.ad"

    Label* L = opnd_array(1)->label();
    __ jmp(*L, false); // Always long jump
  
#line 32382 "ad_x86.cpp"
  }
}

uint jmpDirNode::size(PhaseRegAlloc *ra_) const {
  assert(VerifyOops || MachNode::size(ra_) <= 5, "bad fixed size");
  return (VerifyOops ? MachNode::size(ra_) : 5);
}

void jmpConNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// cr
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// labl
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// 
  {
    C2_MacroAssembler _masm(&cbuf);

#line 12680 "src/hotspot/cpu/x86/x86_32.ad"

    Label* L = opnd_array(3)->label();
    __ jcc((Assembler::Condition)(opnd_array(1)->ccode()), *L, false); // Always long jump
  
#line 32406 "ad_x86.cpp"
  }
}

uint jmpConNode::size(PhaseRegAlloc *ra_) const {
  assert(VerifyOops || MachNode::size(ra_) <= 6, "bad fixed size");
  return (VerifyOops ? MachNode::size(ra_) : 6);
}

void jmpLoopEndNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// cr
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// labl
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// 
  {
    C2_MacroAssembler _masm(&cbuf);

#line 12695 "src/hotspot/cpu/x86/x86_32.ad"

    Label* L = opnd_array(3)->label();
    __ jcc((Assembler::Condition)(opnd_array(1)->ccode()), *L, false); // Always long jump
  
#line 32430 "ad_x86.cpp"
  }
}

uint jmpLoopEndNode::size(PhaseRegAlloc *ra_) const {
  assert(VerifyOops || MachNode::size(ra_) <= 6, "bad fixed size");
  return (VerifyOops ? MachNode::size(ra_) : 6);
}

void jmpConUNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// cmp
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// labl
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// 
  {
    C2_MacroAssembler _masm(&cbuf);

#line 12710 "src/hotspot/cpu/x86/x86_32.ad"

    Label* L = opnd_array(3)->label();
    __ jcc((Assembler::Condition)(opnd_array(1)->ccode()), *L, false); // Always long jump
  
#line 32454 "ad_x86.cpp"
  }
}

uint jmpConUNode::size(PhaseRegAlloc *ra_) const {
  assert(VerifyOops || MachNode::size(ra_) <= 6, "bad fixed size");
  return (VerifyOops ? MachNode::size(ra_) : 6);
}

void jmpConUCFNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// cmp
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// labl
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// 
  {
    C2_MacroAssembler _masm(&cbuf);

#line 12724 "src/hotspot/cpu/x86/x86_32.ad"

    Label* L = opnd_array(3)->label();
    __ jcc((Assembler::Condition)(opnd_array(1)->ccode()), *L, false); // Always long jump
  
#line 32478 "ad_x86.cpp"
  }
}

uint jmpConUCFNode::size(PhaseRegAlloc *ra_) const {
  assert(VerifyOops || MachNode::size(ra_) <= 6, "bad fixed size");
  return (VerifyOops ? MachNode::size(ra_) : 6);
}

void jmpConUCF2Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// cmp
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// labl
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// 
  {
    C2_MacroAssembler _masm(&cbuf);

#line 12746 "src/hotspot/cpu/x86/x86_32.ad"

    Label* l = opnd_array(3)->label();
    if (opnd_array(1)->ccode()== Assembler::notEqual) {
      __ jcc(Assembler::parity, *l, false);
      __ jcc(Assembler::notEqual, *l, false);
    } else if (opnd_array(1)->ccode()== Assembler::equal) {
      Label done;
      __ jccb(Assembler::parity, done);
      __ jcc(Assembler::equal, *l, false);
      __ bind(done);
    } else {
       ShouldNotReachHere();
    }
  
#line 32512 "ad_x86.cpp"
  }
}

void partialSubtypeCheckNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// sub
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// super
  {

#line 1719 "src/hotspot/cpu/x86/x86_32.ad"

    Register Redi = as_Register(EDI_enc); // result register
    Register Reax = as_Register(EAX_enc); // super class
    Register Recx = as_Register(ECX_enc); // killed
    Register Resi = as_Register(ESI_enc); // sub class
    Label miss;

    MacroAssembler _masm(&cbuf);
    __ check_klass_subtype_slow_path(Resi, Reax, Recx, Redi,
                                     NULL, &miss,
                                     /*set_cond_codes:*/ true);
    if ((0x1 /*primary()*/)) {
      __ xorptr(Redi, Redi);
    }
    __ bind(miss);
  
#line 32541 "ad_x86.cpp"
  }
}

void partialSubtypeCheck_vs_ZeroNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// sub
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// super
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// zero
  {

#line 1719 "src/hotspot/cpu/x86/x86_32.ad"

    Register Redi = as_Register(EDI_enc); // result register
    Register Reax = as_Register(EAX_enc); // super class
    Register Recx = as_Register(ECX_enc); // killed
    Register Resi = as_Register(ESI_enc); // sub class
    Label miss;

    MacroAssembler _masm(&cbuf);
    __ check_klass_subtype_slow_path(Resi, Reax, Recx, Redi,
                                     NULL, &miss,
                                     /*set_cond_codes:*/ true);
    if ((0x0 /*primary()*/)) {
      __ xorptr(Redi, Redi);
    }
    __ bind(miss);
  
#line 32571 "ad_x86.cpp"
  }
}

void jmpDir_shortNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// 
  {
    C2_MacroAssembler _masm(&cbuf);

#line 12825 "src/hotspot/cpu/x86/x86_32.ad"

    Label* L = opnd_array(1)->label();
    __ jmpb(*L);
  
#line 32588 "ad_x86.cpp"
  }
}

uint jmpDir_shortNode::size(PhaseRegAlloc *ra_) const {
  assert(VerifyOops || MachNode::size(ra_) <= 2, "bad fixed size");
  return (VerifyOops ? MachNode::size(ra_) : 2);
}

void jmpCon_shortNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// cr
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// labl
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// 
  {
    C2_MacroAssembler _masm(&cbuf);

#line 12841 "src/hotspot/cpu/x86/x86_32.ad"

    Label* L = opnd_array(3)->label();
    __ jccb((Assembler::Condition)(opnd_array(1)->ccode()), *L);
  
#line 32612 "ad_x86.cpp"
  }
}

uint jmpCon_shortNode::size(PhaseRegAlloc *ra_) const {
  assert(VerifyOops || MachNode::size(ra_) <= 2, "bad fixed size");
  return (VerifyOops ? MachNode::size(ra_) : 2);
}

void jmpLoopEnd_shortNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// cr
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// labl
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// 
  {
    C2_MacroAssembler _masm(&cbuf);

#line 12857 "src/hotspot/cpu/x86/x86_32.ad"

    Label* L = opnd_array(3)->label();
    __ jccb((Assembler::Condition)(opnd_array(1)->ccode()), *L);
  
#line 32636 "ad_x86.cpp"
  }
}

uint jmpLoopEnd_shortNode::size(PhaseRegAlloc *ra_) const {
  assert(VerifyOops || MachNode::size(ra_) <= 2, "bad fixed size");
  return (VerifyOops ? MachNode::size(ra_) : 2);
}

void jmpConU_shortNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// cmp
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// labl
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// 
  {
    C2_MacroAssembler _masm(&cbuf);

#line 12873 "src/hotspot/cpu/x86/x86_32.ad"

    Label* L = opnd_array(3)->label();
    __ jccb((Assembler::Condition)(opnd_array(1)->ccode()), *L);
  
#line 32660 "ad_x86.cpp"
  }
}

uint jmpConU_shortNode::size(PhaseRegAlloc *ra_) const {
  assert(VerifyOops || MachNode::size(ra_) <= 2, "bad fixed size");
  return (VerifyOops ? MachNode::size(ra_) : 2);
}

void jmpConUCF_shortNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// cmp
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// labl
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// 
  {
    C2_MacroAssembler _masm(&cbuf);

#line 12888 "src/hotspot/cpu/x86/x86_32.ad"

    Label* L = opnd_array(3)->label();
    __ jccb((Assembler::Condition)(opnd_array(1)->ccode()), *L);
  
#line 32684 "ad_x86.cpp"
  }
}

uint jmpConUCF_shortNode::size(PhaseRegAlloc *ra_) const {
  assert(VerifyOops || MachNode::size(ra_) <= 2, "bad fixed size");
  return (VerifyOops ? MachNode::size(ra_) : 2);
}

void jmpConUCF2_shortNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// cmp
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// labl
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// 
  {
    C2_MacroAssembler _masm(&cbuf);

#line 12912 "src/hotspot/cpu/x86/x86_32.ad"

    Label* l = opnd_array(3)->label();
    if (opnd_array(1)->ccode()== Assembler::notEqual) {
      __ jccb(Assembler::parity, *l);
      __ jccb(Assembler::notEqual, *l);
    } else if (opnd_array(1)->ccode()== Assembler::equal) {
      Label done;
      __ jccb(Assembler::parity, done);
      __ jccb(Assembler::equal, *l);
      __ bind(done);
    } else {
       ShouldNotReachHere();
    }
  
#line 32718 "ad_x86.cpp"
  }
}

uint jmpConUCF2_shortNode::size(PhaseRegAlloc *ra_) const {
  assert(VerifyOops || MachNode::size(ra_) <= 4, "bad fixed size");
  return (VerifyOops ? MachNode::size(ra_) : 4);
}

void cmpL3_reg_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 12968 "src/hotspot/cpu/x86/x86_32.ad"

    Label p_one, m_one, done;
    __ xorptr(opnd_array(0)->as_Register(ra_,this)/* dst */, opnd_array(0)->as_Register(ra_,this)/* dst */);
    __ cmpl(HIGH_FROM_LOW(opnd_array(1)->as_Register(ra_,this,idx1)/* src1 */), HIGH_FROM_LOW(opnd_array(2)->as_Register(ra_,this,idx2)/* src2 */));
    __ jccb(Assembler::less,    m_one);
    __ jccb(Assembler::greater, p_one);
    __ cmpl(opnd_array(1)->as_Register(ra_,this,idx1)/* src1 */, opnd_array(2)->as_Register(ra_,this,idx2)/* src2 */);
    __ jccb(Assembler::below,   m_one);
    __ jccb(Assembler::equal,   done);
    __ bind(p_one);
    __ incrementl(opnd_array(0)->as_Register(ra_,this)/* dst */);
    __ jmpb(done);
    __ bind(m_one);
    __ decrementl(opnd_array(0)->as_Register(ra_,this)/* dst */);
    __ bind(done);
  
#line 32753 "ad_x86.cpp"
  }
}

void cmpL_zero_flags_LTGENode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// zero
  {

#line 1544 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0x85 /*primary()*/));
  
#line 32769 "ad_x86.cpp"
  }
  {

#line 1977 "src/hotspot/cpu/x86/x86_32.ad"

    emit_rm(cbuf, 0x3, HIGH_FROM_LOW_ENC(opnd_array(1)->reg(ra_,this,idx1)/* src */), HIGH_FROM_LOW_ENC(opnd_array(1)->reg(ra_,this,idx1)/* src */));
  
#line 32777 "ad_x86.cpp"
  }
}

void cmpL_reg_flags_LTGENode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// tmp
  {

#line 2865 "src/hotspot/cpu/x86/x86_32.ad"

    // CMP    $src1.lo,$src2.lo\t! Long compare; set flags for low bits
    emit_opcode( cbuf, 0x3B );
    emit_rm(cbuf, 0x3, opnd_array(1)->reg(ra_,this,idx1)/* src1 */, opnd_array(2)->reg(ra_,this,idx2)/* src2 */ );
    // MOV    $tmp,$src1.hi
    emit_opcode( cbuf, 0x8B );
    emit_rm(cbuf, 0x3, opnd_array(3)->reg(ra_,this,idx3)/* tmp */, HIGH_FROM_LOW_ENC(opnd_array(1)->reg(ra_,this,idx1)/* src1 */) );
    // SBB   $tmp,$src2.hi\t! Compute flags for long compare
    emit_opcode( cbuf, 0x1B );
    emit_rm(cbuf, 0x3, opnd_array(3)->reg(ra_,this,idx3)/* tmp */, HIGH_FROM_LOW_ENC(opnd_array(2)->reg(ra_,this,idx2)/* src2 */) );
  
#line 32802 "ad_x86.cpp"
  }
}

void cmpUL_zero_flags_LTGENode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// zero
  {

#line 1544 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0x85 /*primary()*/));
  
#line 32818 "ad_x86.cpp"
  }
  {

#line 1977 "src/hotspot/cpu/x86/x86_32.ad"

    emit_rm(cbuf, 0x3, HIGH_FROM_LOW_ENC(opnd_array(1)->reg(ra_,this,idx1)/* src */), HIGH_FROM_LOW_ENC(opnd_array(1)->reg(ra_,this,idx1)/* src */));
  
#line 32826 "ad_x86.cpp"
  }
}

void cmpUL_reg_flags_LTGENode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// tmp
  {

#line 2865 "src/hotspot/cpu/x86/x86_32.ad"

    // CMP    $src1.lo,$src2.lo\t! Long compare; set flags for low bits
    emit_opcode( cbuf, 0x3B );
    emit_rm(cbuf, 0x3, opnd_array(1)->reg(ra_,this,idx1)/* src1 */, opnd_array(2)->reg(ra_,this,idx2)/* src2 */ );
    // MOV    $tmp,$src1.hi
    emit_opcode( cbuf, 0x8B );
    emit_rm(cbuf, 0x3, opnd_array(3)->reg(ra_,this,idx3)/* tmp */, HIGH_FROM_LOW_ENC(opnd_array(1)->reg(ra_,this,idx1)/* src1 */) );
    // SBB   $tmp,$src2.hi\t! Compute flags for long compare
    emit_opcode( cbuf, 0x1B );
    emit_rm(cbuf, 0x3, opnd_array(3)->reg(ra_,this,idx3)/* tmp */, HIGH_FROM_LOW_ENC(opnd_array(2)->reg(ra_,this,idx2)/* src2 */) );
  
#line 32851 "ad_x86.cpp"
  }
}

void cmovLL_reg_LTGENode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// cmp
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// flags
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// dst
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// src
  {

#line 1701 "src/hotspot/cpu/x86/x86_32.ad"

    emit_d8(cbuf, (0x0F /*primary()*/));
    emit_cc(cbuf, (0x40 /*secondary()*/), opnd_array(1)->ccode());
  
#line 32870 "ad_x86.cpp"
  }
  {

#line 1973 "src/hotspot/cpu/x86/x86_32.ad"

    emit_rm(cbuf, 0x3, opnd_array(3)->reg(ra_,this,idx3)/* dst */, opnd_array(4)->reg(ra_,this,idx4)/* src */);
  
#line 32878 "ad_x86.cpp"
  }
  {

#line 1701 "src/hotspot/cpu/x86/x86_32.ad"

    emit_d8(cbuf, (0x0F /*primary()*/));
    emit_cc(cbuf, (0x40 /*secondary()*/), opnd_array(1)->ccode());
  
#line 32887 "ad_x86.cpp"
  }
  {

#line 1977 "src/hotspot/cpu/x86/x86_32.ad"

    emit_rm(cbuf, 0x3, HIGH_FROM_LOW_ENC(opnd_array(3)->reg(ra_,this,idx3)/* dst */), HIGH_FROM_LOW_ENC(opnd_array(4)->reg(ra_,this,idx4)/* src */));
  
#line 32895 "ad_x86.cpp"
  }
}

void cmovLL_mem_LTGENode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// cmp
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// flags
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// dst
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// src
  {

#line 1701 "src/hotspot/cpu/x86/x86_32.ad"

    emit_d8(cbuf, (0x0F /*primary()*/));
    emit_cc(cbuf, (0x40 /*secondary()*/), opnd_array(1)->ccode());
  
#line 32914 "ad_x86.cpp"
  }
  {

#line 2096 "src/hotspot/cpu/x86/x86_32.ad"

    int reg_encoding = opnd_array(3)->reg(ra_,this,idx3)/* dst */;
    int base  = opnd_array(4)->base(ra_,this,idx4);
    int index = opnd_array(4)->index(ra_,this,idx4);
    int scale = opnd_array(4)->scale();
    int displace = opnd_array(4)->disp(ra_,this,idx4);
    relocInfo::relocType disp_reloc = opnd_array(4)->disp_reloc();
    encode_RegMem(cbuf, reg_encoding, base, index, scale, displace, disp_reloc);
  
#line 32928 "ad_x86.cpp"
  }
  {

#line 1701 "src/hotspot/cpu/x86/x86_32.ad"

    emit_d8(cbuf, (0x0F /*primary()*/));
    emit_cc(cbuf, (0x40 /*secondary()*/), opnd_array(1)->ccode());
  
#line 32937 "ad_x86.cpp"
  }
  {

#line 2106 "src/hotspot/cpu/x86/x86_32.ad"

    int reg_encoding = HIGH_FROM_LOW_ENC(opnd_array(3)->reg(ra_,this,idx3)/* dst */);  // Hi register of pair, computed from lo
    int base  = opnd_array(4)->base(ra_,this,idx4);
    int index = opnd_array(4)->index(ra_,this,idx4);
    int scale = opnd_array(4)->scale();
    int displace = opnd_array(4)->disp(ra_,this,idx4) + 4;      // Offset is 4 further in memory
    assert( opnd_array(4)->disp_reloc() == relocInfo::none, "Cannot add 4 to oop" );
    encode_RegMem(cbuf, reg_encoding, base, index, scale, displace, relocInfo::none);
  
#line 32951 "ad_x86.cpp"
  }
}

void cmovII_reg_LTGENode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// cmp
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// flags
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// dst
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// src
  {

#line 1701 "src/hotspot/cpu/x86/x86_32.ad"

    emit_d8(cbuf, (0x0F /*primary()*/));
    emit_cc(cbuf, (0x40 /*secondary()*/), opnd_array(1)->ccode());
  
#line 32970 "ad_x86.cpp"
  }
  {

#line 1959 "src/hotspot/cpu/x86/x86_32.ad"

    emit_rm(cbuf, 0x3, opnd_array(3)->reg(ra_,this,idx3)/* dst */, opnd_array(4)->reg(ra_,this,idx4)/* src */);
  
#line 32978 "ad_x86.cpp"
  }
}

void cmovII_mem_LTGENode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// cmp
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// flags
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// dst
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// src
  {

#line 1701 "src/hotspot/cpu/x86/x86_32.ad"

    emit_d8(cbuf, (0x0F /*primary()*/));
    emit_cc(cbuf, (0x40 /*secondary()*/), opnd_array(1)->ccode());
  
#line 32997 "ad_x86.cpp"
  }
  {

#line 2096 "src/hotspot/cpu/x86/x86_32.ad"

    int reg_encoding = opnd_array(3)->reg(ra_,this,idx3)/* dst */;
    int base  = opnd_array(4)->base(ra_,this,idx4);
    int index = opnd_array(4)->index(ra_,this,idx4);
    int scale = opnd_array(4)->scale();
    int displace = opnd_array(4)->disp(ra_,this,idx4);
    relocInfo::relocType disp_reloc = opnd_array(4)->disp_reloc();
    encode_RegMem(cbuf, reg_encoding, base, index, scale, displace, disp_reloc);
  
#line 33011 "ad_x86.cpp"
  }
}

void cmovPP_reg_LTGENode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// cmp
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// flags
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// dst
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// src
  {

#line 1701 "src/hotspot/cpu/x86/x86_32.ad"

    emit_d8(cbuf, (0x0F /*primary()*/));
    emit_cc(cbuf, (0x40 /*secondary()*/), opnd_array(1)->ccode());
  
#line 33030 "ad_x86.cpp"
  }
  {

#line 1959 "src/hotspot/cpu/x86/x86_32.ad"

    emit_rm(cbuf, 0x3, opnd_array(3)->reg(ra_,this,idx3)/* dst */, opnd_array(4)->reg(ra_,this,idx4)/* src */);
  
#line 33038 "ad_x86.cpp"
  }
}

void cmpL_zero_flags_EQNENode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// zero
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// tmp
  {

#line 2844 "src/hotspot/cpu/x86/x86_32.ad"

    // MOV   $tmp,$src.lo
    emit_opcode(cbuf, 0x8B);
    emit_rm(cbuf, 0x3, opnd_array(3)->reg(ra_,this,idx3)/* tmp */, opnd_array(1)->reg(ra_,this,idx1)/* src */);
    // OR    $tmp,$src.hi
    emit_opcode(cbuf, 0x0B);
    emit_rm(cbuf, 0x3, opnd_array(3)->reg(ra_,this,idx3)/* tmp */, HIGH_FROM_LOW_ENC(opnd_array(1)->reg(ra_,this,idx1)/* src */));
  
#line 33060 "ad_x86.cpp"
  }
}

void cmpL_reg_flags_EQNENode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {

#line 2853 "src/hotspot/cpu/x86/x86_32.ad"

    // CMP    $src1.lo,$src2.lo
    emit_opcode( cbuf, 0x3B );
    emit_rm(cbuf, 0x3, opnd_array(1)->reg(ra_,this,idx1)/* src1 */, opnd_array(2)->reg(ra_,this,idx2)/* src2 */ );
    // JNE,s  skip
    emit_cc(cbuf, 0x70, 0x5);
    emit_d8(cbuf,2);
    // CMP    $src1.hi,$src2.hi
    emit_opcode( cbuf, 0x3B );
    emit_rm(cbuf, 0x3, HIGH_FROM_LOW_ENC(opnd_array(1)->reg(ra_,this,idx1)/* src1 */), HIGH_FROM_LOW_ENC(opnd_array(2)->reg(ra_,this,idx2)/* src2 */) );
  
#line 33084 "ad_x86.cpp"
  }
}

void cmpUL_zero_flags_EQNENode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// zero
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// tmp
  {

#line 2844 "src/hotspot/cpu/x86/x86_32.ad"

    // MOV   $tmp,$src.lo
    emit_opcode(cbuf, 0x8B);
    emit_rm(cbuf, 0x3, opnd_array(3)->reg(ra_,this,idx3)/* tmp */, opnd_array(1)->reg(ra_,this,idx1)/* src */);
    // OR    $tmp,$src.hi
    emit_opcode(cbuf, 0x0B);
    emit_rm(cbuf, 0x3, opnd_array(3)->reg(ra_,this,idx3)/* tmp */, HIGH_FROM_LOW_ENC(opnd_array(1)->reg(ra_,this,idx1)/* src */));
  
#line 33106 "ad_x86.cpp"
  }
}

void cmpUL_reg_flags_EQNENode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {

#line 2853 "src/hotspot/cpu/x86/x86_32.ad"

    // CMP    $src1.lo,$src2.lo
    emit_opcode( cbuf, 0x3B );
    emit_rm(cbuf, 0x3, opnd_array(1)->reg(ra_,this,idx1)/* src1 */, opnd_array(2)->reg(ra_,this,idx2)/* src2 */ );
    // JNE,s  skip
    emit_cc(cbuf, 0x70, 0x5);
    emit_d8(cbuf,2);
    // CMP    $src1.hi,$src2.hi
    emit_opcode( cbuf, 0x3B );
    emit_rm(cbuf, 0x3, HIGH_FROM_LOW_ENC(opnd_array(1)->reg(ra_,this,idx1)/* src1 */), HIGH_FROM_LOW_ENC(opnd_array(2)->reg(ra_,this,idx2)/* src2 */) );
  
#line 33130 "ad_x86.cpp"
  }
}

void cmovLL_reg_EQNENode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// cmp
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// flags
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// dst
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// src
  {

#line 1701 "src/hotspot/cpu/x86/x86_32.ad"

    emit_d8(cbuf, (0x0F /*primary()*/));
    emit_cc(cbuf, (0x40 /*secondary()*/), opnd_array(1)->ccode());
  
#line 33149 "ad_x86.cpp"
  }
  {

#line 1973 "src/hotspot/cpu/x86/x86_32.ad"

    emit_rm(cbuf, 0x3, opnd_array(3)->reg(ra_,this,idx3)/* dst */, opnd_array(4)->reg(ra_,this,idx4)/* src */);
  
#line 33157 "ad_x86.cpp"
  }
  {

#line 1701 "src/hotspot/cpu/x86/x86_32.ad"

    emit_d8(cbuf, (0x0F /*primary()*/));
    emit_cc(cbuf, (0x40 /*secondary()*/), opnd_array(1)->ccode());
  
#line 33166 "ad_x86.cpp"
  }
  {

#line 1977 "src/hotspot/cpu/x86/x86_32.ad"

    emit_rm(cbuf, 0x3, HIGH_FROM_LOW_ENC(opnd_array(3)->reg(ra_,this,idx3)/* dst */), HIGH_FROM_LOW_ENC(opnd_array(4)->reg(ra_,this,idx4)/* src */));
  
#line 33174 "ad_x86.cpp"
  }
}

void cmovLL_mem_EQNENode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// cmp
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// flags
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// dst
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// src
  {

#line 1701 "src/hotspot/cpu/x86/x86_32.ad"

    emit_d8(cbuf, (0x0F /*primary()*/));
    emit_cc(cbuf, (0x40 /*secondary()*/), opnd_array(1)->ccode());
  
#line 33193 "ad_x86.cpp"
  }
  {

#line 2096 "src/hotspot/cpu/x86/x86_32.ad"

    int reg_encoding = opnd_array(3)->reg(ra_,this,idx3)/* dst */;
    int base  = opnd_array(4)->base(ra_,this,idx4);
    int index = opnd_array(4)->index(ra_,this,idx4);
    int scale = opnd_array(4)->scale();
    int displace = opnd_array(4)->disp(ra_,this,idx4);
    relocInfo::relocType disp_reloc = opnd_array(4)->disp_reloc();
    encode_RegMem(cbuf, reg_encoding, base, index, scale, displace, disp_reloc);
  
#line 33207 "ad_x86.cpp"
  }
  {

#line 1701 "src/hotspot/cpu/x86/x86_32.ad"

    emit_d8(cbuf, (0x0F /*primary()*/));
    emit_cc(cbuf, (0x40 /*secondary()*/), opnd_array(1)->ccode());
  
#line 33216 "ad_x86.cpp"
  }
  {

#line 2106 "src/hotspot/cpu/x86/x86_32.ad"

    int reg_encoding = HIGH_FROM_LOW_ENC(opnd_array(3)->reg(ra_,this,idx3)/* dst */);  // Hi register of pair, computed from lo
    int base  = opnd_array(4)->base(ra_,this,idx4);
    int index = opnd_array(4)->index(ra_,this,idx4);
    int scale = opnd_array(4)->scale();
    int displace = opnd_array(4)->disp(ra_,this,idx4) + 4;      // Offset is 4 further in memory
    assert( opnd_array(4)->disp_reloc() == relocInfo::none, "Cannot add 4 to oop" );
    encode_RegMem(cbuf, reg_encoding, base, index, scale, displace, relocInfo::none);
  
#line 33230 "ad_x86.cpp"
  }
}

void cmovII_reg_EQNENode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// cmp
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// flags
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// dst
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// src
  {

#line 1701 "src/hotspot/cpu/x86/x86_32.ad"

    emit_d8(cbuf, (0x0F /*primary()*/));
    emit_cc(cbuf, (0x40 /*secondary()*/), opnd_array(1)->ccode());
  
#line 33249 "ad_x86.cpp"
  }
  {

#line 1959 "src/hotspot/cpu/x86/x86_32.ad"

    emit_rm(cbuf, 0x3, opnd_array(3)->reg(ra_,this,idx3)/* dst */, opnd_array(4)->reg(ra_,this,idx4)/* src */);
  
#line 33257 "ad_x86.cpp"
  }
}

void cmovII_mem_EQNENode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// cmp
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// flags
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// dst
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// src
  {

#line 1701 "src/hotspot/cpu/x86/x86_32.ad"

    emit_d8(cbuf, (0x0F /*primary()*/));
    emit_cc(cbuf, (0x40 /*secondary()*/), opnd_array(1)->ccode());
  
#line 33276 "ad_x86.cpp"
  }
  {

#line 2096 "src/hotspot/cpu/x86/x86_32.ad"

    int reg_encoding = opnd_array(3)->reg(ra_,this,idx3)/* dst */;
    int base  = opnd_array(4)->base(ra_,this,idx4);
    int index = opnd_array(4)->index(ra_,this,idx4);
    int scale = opnd_array(4)->scale();
    int displace = opnd_array(4)->disp(ra_,this,idx4);
    relocInfo::relocType disp_reloc = opnd_array(4)->disp_reloc();
    encode_RegMem(cbuf, reg_encoding, base, index, scale, displace, disp_reloc);
  
#line 33290 "ad_x86.cpp"
  }
}

void cmovPP_reg_EQNENode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// cmp
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// flags
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// dst
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// src
  {

#line 1701 "src/hotspot/cpu/x86/x86_32.ad"

    emit_d8(cbuf, (0x0F /*primary()*/));
    emit_cc(cbuf, (0x40 /*secondary()*/), opnd_array(1)->ccode());
  
#line 33309 "ad_x86.cpp"
  }
  {

#line 1959 "src/hotspot/cpu/x86/x86_32.ad"

    emit_rm(cbuf, 0x3, opnd_array(3)->reg(ra_,this,idx3)/* dst */, opnd_array(4)->reg(ra_,this,idx4)/* src */);
  
#line 33317 "ad_x86.cpp"
  }
}

void cmpL_zero_flags_LEGTNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// zero
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// tmp
  {

#line 2877 "src/hotspot/cpu/x86/x86_32.ad"

    // XOR    $tmp,$tmp
    emit_opcode(cbuf,0x33);  // XOR
    emit_rm(cbuf,0x3, opnd_array(3)->reg(ra_,this,idx3)/* tmp */, opnd_array(3)->reg(ra_,this,idx3)/* tmp */);
    // CMP    $tmp,$src.lo
    emit_opcode( cbuf, 0x3B );
    emit_rm(cbuf, 0x3, opnd_array(3)->reg(ra_,this,idx3)/* tmp */, opnd_array(1)->reg(ra_,this,idx1)/* src */ );
    // SBB    $tmp,$src.hi
    emit_opcode( cbuf, 0x1B );
    emit_rm(cbuf, 0x3, opnd_array(3)->reg(ra_,this,idx3)/* tmp */, HIGH_FROM_LOW_ENC(opnd_array(1)->reg(ra_,this,idx1)/* src */) );
  
#line 33342 "ad_x86.cpp"
  }
}

void cmpL_reg_flags_LEGTNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// tmp
  {

#line 2865 "src/hotspot/cpu/x86/x86_32.ad"

    // CMP    $src1.lo,$src2.lo\t! Long compare; set flags for low bits
    emit_opcode( cbuf, 0x3B );
    emit_rm(cbuf, 0x3, opnd_array(2)->reg(ra_,this,idx2)/* src2 */, opnd_array(1)->reg(ra_,this,idx1)/* src1 */ );
    // MOV    $tmp,$src1.hi
    emit_opcode( cbuf, 0x8B );
    emit_rm(cbuf, 0x3, opnd_array(3)->reg(ra_,this,idx3)/* tmp */, HIGH_FROM_LOW_ENC(opnd_array(2)->reg(ra_,this,idx2)/* src2 */) );
    // SBB   $tmp,$src2.hi\t! Compute flags for long compare
    emit_opcode( cbuf, 0x1B );
    emit_rm(cbuf, 0x3, opnd_array(3)->reg(ra_,this,idx3)/* tmp */, HIGH_FROM_LOW_ENC(opnd_array(1)->reg(ra_,this,idx1)/* src1 */) );
  
#line 33367 "ad_x86.cpp"
  }
}

void cmpUL_zero_flags_LEGTNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// zero
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// tmp
  {

#line 2877 "src/hotspot/cpu/x86/x86_32.ad"

    // XOR    $tmp,$tmp
    emit_opcode(cbuf,0x33);  // XOR
    emit_rm(cbuf,0x3, opnd_array(3)->reg(ra_,this,idx3)/* tmp */, opnd_array(3)->reg(ra_,this,idx3)/* tmp */);
    // CMP    $tmp,$src.lo
    emit_opcode( cbuf, 0x3B );
    emit_rm(cbuf, 0x3, opnd_array(3)->reg(ra_,this,idx3)/* tmp */, opnd_array(1)->reg(ra_,this,idx1)/* src */ );
    // SBB    $tmp,$src.hi
    emit_opcode( cbuf, 0x1B );
    emit_rm(cbuf, 0x3, opnd_array(3)->reg(ra_,this,idx3)/* tmp */, HIGH_FROM_LOW_ENC(opnd_array(1)->reg(ra_,this,idx1)/* src */) );
  
#line 33392 "ad_x86.cpp"
  }
}

void cmpUL_reg_flags_LEGTNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// tmp
  {

#line 2865 "src/hotspot/cpu/x86/x86_32.ad"

    // CMP    $src1.lo,$src2.lo\t! Long compare; set flags for low bits
    emit_opcode( cbuf, 0x3B );
    emit_rm(cbuf, 0x3, opnd_array(2)->reg(ra_,this,idx2)/* src2 */, opnd_array(1)->reg(ra_,this,idx1)/* src1 */ );
    // MOV    $tmp,$src1.hi
    emit_opcode( cbuf, 0x8B );
    emit_rm(cbuf, 0x3, opnd_array(3)->reg(ra_,this,idx3)/* tmp */, HIGH_FROM_LOW_ENC(opnd_array(2)->reg(ra_,this,idx2)/* src2 */) );
    // SBB   $tmp,$src2.hi\t! Compute flags for long compare
    emit_opcode( cbuf, 0x1B );
    emit_rm(cbuf, 0x3, opnd_array(3)->reg(ra_,this,idx3)/* tmp */, HIGH_FROM_LOW_ENC(opnd_array(1)->reg(ra_,this,idx1)/* src1 */) );
  
#line 33417 "ad_x86.cpp"
  }
}

void cmovLL_reg_LEGTNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// cmp
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// flags
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// dst
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// src
  {

#line 1701 "src/hotspot/cpu/x86/x86_32.ad"

    emit_d8(cbuf, (0x0F /*primary()*/));
    emit_cc(cbuf, (0x40 /*secondary()*/), opnd_array(1)->ccode());
  
#line 33436 "ad_x86.cpp"
  }
  {

#line 1973 "src/hotspot/cpu/x86/x86_32.ad"

    emit_rm(cbuf, 0x3, opnd_array(3)->reg(ra_,this,idx3)/* dst */, opnd_array(4)->reg(ra_,this,idx4)/* src */);
  
#line 33444 "ad_x86.cpp"
  }
  {

#line 1701 "src/hotspot/cpu/x86/x86_32.ad"

    emit_d8(cbuf, (0x0F /*primary()*/));
    emit_cc(cbuf, (0x40 /*secondary()*/), opnd_array(1)->ccode());
  
#line 33453 "ad_x86.cpp"
  }
  {

#line 1977 "src/hotspot/cpu/x86/x86_32.ad"

    emit_rm(cbuf, 0x3, HIGH_FROM_LOW_ENC(opnd_array(3)->reg(ra_,this,idx3)/* dst */), HIGH_FROM_LOW_ENC(opnd_array(4)->reg(ra_,this,idx4)/* src */));
  
#line 33461 "ad_x86.cpp"
  }
}

void cmovLL_mem_LEGTNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// cmp
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// flags
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// dst
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// src
  {

#line 1701 "src/hotspot/cpu/x86/x86_32.ad"

    emit_d8(cbuf, (0x0F /*primary()*/));
    emit_cc(cbuf, (0x40 /*secondary()*/), opnd_array(1)->ccode());
  
#line 33480 "ad_x86.cpp"
  }
  {

#line 2096 "src/hotspot/cpu/x86/x86_32.ad"

    int reg_encoding = opnd_array(3)->reg(ra_,this,idx3)/* dst */;
    int base  = opnd_array(4)->base(ra_,this,idx4);
    int index = opnd_array(4)->index(ra_,this,idx4);
    int scale = opnd_array(4)->scale();
    int displace = opnd_array(4)->disp(ra_,this,idx4);
    relocInfo::relocType disp_reloc = opnd_array(4)->disp_reloc();
    encode_RegMem(cbuf, reg_encoding, base, index, scale, displace, disp_reloc);
  
#line 33494 "ad_x86.cpp"
  }
  {

#line 1701 "src/hotspot/cpu/x86/x86_32.ad"

    emit_d8(cbuf, (0x0F /*primary()*/));
    emit_cc(cbuf, (0x40 /*secondary()*/), opnd_array(1)->ccode());
  
#line 33503 "ad_x86.cpp"
  }
  {

#line 2106 "src/hotspot/cpu/x86/x86_32.ad"

    int reg_encoding = HIGH_FROM_LOW_ENC(opnd_array(3)->reg(ra_,this,idx3)/* dst */);  // Hi register of pair, computed from lo
    int base  = opnd_array(4)->base(ra_,this,idx4);
    int index = opnd_array(4)->index(ra_,this,idx4);
    int scale = opnd_array(4)->scale();
    int displace = opnd_array(4)->disp(ra_,this,idx4) + 4;      // Offset is 4 further in memory
    assert( opnd_array(4)->disp_reloc() == relocInfo::none, "Cannot add 4 to oop" );
    encode_RegMem(cbuf, reg_encoding, base, index, scale, displace, relocInfo::none);
  
#line 33517 "ad_x86.cpp"
  }
}

void cmovII_reg_LEGTNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// cmp
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// flags
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// dst
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// src
  {

#line 1701 "src/hotspot/cpu/x86/x86_32.ad"

    emit_d8(cbuf, (0x0F /*primary()*/));
    emit_cc(cbuf, (0x40 /*secondary()*/), opnd_array(1)->ccode());
  
#line 33536 "ad_x86.cpp"
  }
  {

#line 1959 "src/hotspot/cpu/x86/x86_32.ad"

    emit_rm(cbuf, 0x3, opnd_array(3)->reg(ra_,this,idx3)/* dst */, opnd_array(4)->reg(ra_,this,idx4)/* src */);
  
#line 33544 "ad_x86.cpp"
  }
}

void cmovII_mem_LEGTNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// cmp
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// flags
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// dst
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// src
  {

#line 1701 "src/hotspot/cpu/x86/x86_32.ad"

    emit_d8(cbuf, (0x0F /*primary()*/));
    emit_cc(cbuf, (0x40 /*secondary()*/), opnd_array(1)->ccode());
  
#line 33563 "ad_x86.cpp"
  }
  {

#line 2096 "src/hotspot/cpu/x86/x86_32.ad"

    int reg_encoding = opnd_array(3)->reg(ra_,this,idx3)/* dst */;
    int base  = opnd_array(4)->base(ra_,this,idx4);
    int index = opnd_array(4)->index(ra_,this,idx4);
    int scale = opnd_array(4)->scale();
    int displace = opnd_array(4)->disp(ra_,this,idx4);
    relocInfo::relocType disp_reloc = opnd_array(4)->disp_reloc();
    encode_RegMem(cbuf, reg_encoding, base, index, scale, displace, disp_reloc);
  
#line 33577 "ad_x86.cpp"
  }
}

void cmovPP_reg_LEGTNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// cmp
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// flags
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// dst
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// src
  {

#line 1701 "src/hotspot/cpu/x86/x86_32.ad"

    emit_d8(cbuf, (0x0F /*primary()*/));
    emit_cc(cbuf, (0x40 /*secondary()*/), opnd_array(1)->ccode());
  
#line 33596 "ad_x86.cpp"
  }
  {

#line 1959 "src/hotspot/cpu/x86/x86_32.ad"

    emit_rm(cbuf, 0x3, opnd_array(3)->reg(ra_,this,idx3)/* dst */, opnd_array(4)->reg(ra_,this,idx4)/* src */);
  
#line 33604 "ad_x86.cpp"
  }
}

void CallStaticJavaDirectNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// 
  {

#line 1796 "src/hotspot/cpu/x86/x86_32.ad"

    // If method sets FPU control word restore it here
    debug_only(int off0 = cbuf.insts_size());
    if (ra_->C->in_24_bit_fp_mode()) {
      MacroAssembler _masm(&cbuf);
      __ fldcw(ExternalAddress(StubRoutines::x86::addr_fpu_cntrl_wrd_std()));
    }
    // Clear upper bits of YMM registers when current compiled code uses
    // wide vectors to avoid AVX <-> SSE transition penalty during call.
    MacroAssembler _masm(&cbuf);
    __ vzeroupper();
    debug_only(int off1 = cbuf.insts_size());
    assert(off1 - off0 == pre_call_resets_size(), "correct size prediction");
  
#line 33630 "ad_x86.cpp"
  }
  {

#line 1819 "src/hotspot/cpu/x86/x86_32.ad"

    // CALL to fixup routine.  Fixup routine uses ScopeDesc info to determine
    // who we intended to call.
    MacroAssembler _masm(&cbuf);
    cbuf.set_insts_mark();
    emit_d8(cbuf, (0xE8 /*primary()*/));

    if (!_method) {
      emit_d32_reloc(cbuf, (opnd_array(1)->method() - (int)(cbuf.insts_end()) - 4),
                     runtime_call_Relocation::spec(),
                     RELOC_IMM32);
      __ post_call_nop();
    } else {
      int method_index = resolved_method_index(cbuf);
      RelocationHolder rspec = _optimized_virtual ? opt_virtual_call_Relocation::spec(method_index)
                                                  : static_call_Relocation::spec(method_index);
      emit_d32_reloc(cbuf, (opnd_array(1)->method() - (int)(cbuf.insts_end()) - 4),
                     rspec, RELOC_DISP32);
      __ post_call_nop();
      address mark = cbuf.insts_mark();
      if (CodeBuffer::supports_shared_stubs() && _method->can_be_statically_bound()) {
        // Calls of the same statically bound method can share
        // a stub to the interpreter.
        cbuf.shared_stub_to_interp_for(_method, cbuf.insts()->mark_off());
      } else {
        // Emit stubs for static call.
        address stub = CompiledStaticCall::emit_to_interp_stub(cbuf, mark);
        if (stub == NULL) {
          ciEnv::current()->record_failure("CodeCache is full");
          return;
        }
      }
    }
  
#line 33669 "ad_x86.cpp"
  }
  {

#line 2770 "src/hotspot/cpu/x86/x86.ad"

    C2_MacroAssembler _masm(&cbuf);
    if (VerifyStackAtCalls) {
      // Check that stack depth is unchanged: find majik cookie on stack
      int framesize = ra_->reg2offset_unchecked(OptoReg::add(ra_->_matcher._old_SP, -3*VMRegImpl::slots_per_word));
      Label L;
      __ cmpptr(Address(rsp, framesize), (int32_t)0xbadb100d);
      __ jccb(Assembler::equal, L);
      // Die if stack mismatch
      __ int3();
      __ bind(L);
    }
  
#line 33687 "ad_x86.cpp"
  }
  {

#line 1811 "src/hotspot/cpu/x86/x86_32.ad"

    // If method sets FPU control word do it here also
    if (Compile::current()->in_24_bit_fp_mode()) {
      MacroAssembler masm(&cbuf);
      masm.fldcw(ExternalAddress(StubRoutines::x86::addr_fpu_cntrl_wrd_24()));
    }
  
#line 33699 "ad_x86.cpp"
  }
}

void CallDynamicJavaDirectNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// 
  {

#line 1796 "src/hotspot/cpu/x86/x86_32.ad"

    // If method sets FPU control word restore it here
    debug_only(int off0 = cbuf.insts_size());
    if (ra_->C->in_24_bit_fp_mode()) {
      MacroAssembler _masm(&cbuf);
      __ fldcw(ExternalAddress(StubRoutines::x86::addr_fpu_cntrl_wrd_std()));
    }
    // Clear upper bits of YMM registers when current compiled code uses
    // wide vectors to avoid AVX <-> SSE transition penalty during call.
    MacroAssembler _masm(&cbuf);
    __ vzeroupper();
    debug_only(int off1 = cbuf.insts_size());
    assert(off1 - off0 == pre_call_resets_size(), "correct size prediction");
  
#line 33725 "ad_x86.cpp"
  }
  {

#line 1854 "src/hotspot/cpu/x86/x86_32.ad"

    MacroAssembler _masm(&cbuf);
    __ ic_call((address)opnd_array(1)->method(), resolved_method_index(cbuf));
    __ post_call_nop();
  
#line 33735 "ad_x86.cpp"
  }
  {

#line 2770 "src/hotspot/cpu/x86/x86.ad"

    C2_MacroAssembler _masm(&cbuf);
    if (VerifyStackAtCalls) {
      // Check that stack depth is unchanged: find majik cookie on stack
      int framesize = ra_->reg2offset_unchecked(OptoReg::add(ra_->_matcher._old_SP, -3*VMRegImpl::slots_per_word));
      Label L;
      __ cmpptr(Address(rsp, framesize), (int32_t)0xbadb100d);
      __ jccb(Assembler::equal, L);
      // Die if stack mismatch
      __ int3();
      __ bind(L);
    }
  
#line 33753 "ad_x86.cpp"
  }
  {

#line 1811 "src/hotspot/cpu/x86/x86_32.ad"

    // If method sets FPU control word do it here also
    if (Compile::current()->in_24_bit_fp_mode()) {
      MacroAssembler masm(&cbuf);
      masm.fldcw(ExternalAddress(StubRoutines::x86::addr_fpu_cntrl_wrd_24()));
    }
  
#line 33765 "ad_x86.cpp"
  }
}

void CallRuntimeDirectNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// 
  {

#line 1796 "src/hotspot/cpu/x86/x86_32.ad"

    // If method sets FPU control word restore it here
    debug_only(int off0 = cbuf.insts_size());
    if (ra_->C->in_24_bit_fp_mode()) {
      MacroAssembler _masm(&cbuf);
      __ fldcw(ExternalAddress(StubRoutines::x86::addr_fpu_cntrl_wrd_std()));
    }
    // Clear upper bits of YMM registers when current compiled code uses
    // wide vectors to avoid AVX <-> SSE transition penalty during call.
    MacroAssembler _masm(&cbuf);
    __ vzeroupper();
    debug_only(int off1 = cbuf.insts_size());
    assert(off1 - off0 == pre_call_resets_size(), "correct size prediction");
  
#line 33791 "ad_x86.cpp"
  }
  {

#line 1736 "src/hotspot/cpu/x86/x86_32.ad"

    MacroAssembler masm(&cbuf);
    int start = masm.offset();
    if (UseSSE >= 2) {
      if (VerifyFPU) {
        masm.verify_FPU(0, "must be empty in SSE2+ mode");
      }
    } else {
      // External c_calling_convention expects the FPU stack to be 'clean'.
      // Compiled code leaves it dirty.  Do cleanup now.
      masm.empty_FPU_stack();
    }
    if (sizeof_FFree_Float_Stack_All == -1) {
      sizeof_FFree_Float_Stack_All = masm.offset() - start;
    } else {
      assert(masm.offset() - start == sizeof_FFree_Float_Stack_All, "wrong size");
    }
  
#line 33814 "ad_x86.cpp"
  }
  {

#line 1762 "src/hotspot/cpu/x86/x86_32.ad"

    // This is the instruction starting address for relocation info.
    MacroAssembler _masm(&cbuf);
    cbuf.set_insts_mark();
    emit_d8(cbuf, (0xE8 /*primary()*/));
    // CALL directly to the runtime
    emit_d32_reloc(cbuf, (opnd_array(1)->method() - (int)(cbuf.insts_end()) - 4),
                runtime_call_Relocation::spec(), RELOC_IMM32 );
    __ post_call_nop();

    if (UseSSE >= 2) {
      MacroAssembler _masm(&cbuf);
      BasicType rt = tf()->return_type();

      if ((rt == T_FLOAT || rt == T_DOUBLE) && !return_value_is_used()) {
        // A C runtime call where the return value is unused.  In SSE2+
        // mode the result needs to be removed from the FPU stack.  It's
        // likely that this function call could be removed by the
        // optimizer if the C function is a pure function.
        __ ffree(0);
      } else if (rt == T_FLOAT) {
        __ lea(rsp, Address(rsp, -4));
        __ fstp_s(Address(rsp, 0));
        __ movflt(xmm0, Address(rsp, 0));
        __ lea(rsp, Address(rsp,  4));
      } else if (rt == T_DOUBLE) {
        __ lea(rsp, Address(rsp, -8));
        __ fstp_d(Address(rsp, 0));
        __ movdbl(xmm0, Address(rsp, 0));
        __ lea(rsp, Address(rsp,  8));
      }
    }
  
#line 33852 "ad_x86.cpp"
  }
  {

#line 1811 "src/hotspot/cpu/x86/x86_32.ad"

    // If method sets FPU control word do it here also
    if (Compile::current()->in_24_bit_fp_mode()) {
      MacroAssembler masm(&cbuf);
      masm.fldcw(ExternalAddress(StubRoutines::x86::addr_fpu_cntrl_wrd_24()));
    }
  
#line 33864 "ad_x86.cpp"
  }
}

void CallLeafDirectNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// 
  {

#line 1796 "src/hotspot/cpu/x86/x86_32.ad"

    // If method sets FPU control word restore it here
    debug_only(int off0 = cbuf.insts_size());
    if (ra_->C->in_24_bit_fp_mode()) {
      MacroAssembler _masm(&cbuf);
      __ fldcw(ExternalAddress(StubRoutines::x86::addr_fpu_cntrl_wrd_std()));
    }
    // Clear upper bits of YMM registers when current compiled code uses
    // wide vectors to avoid AVX <-> SSE transition penalty during call.
    MacroAssembler _masm(&cbuf);
    __ vzeroupper();
    debug_only(int off1 = cbuf.insts_size());
    assert(off1 - off0 == pre_call_resets_size(), "correct size prediction");
  
#line 33890 "ad_x86.cpp"
  }
  {

#line 1736 "src/hotspot/cpu/x86/x86_32.ad"

    MacroAssembler masm(&cbuf);
    int start = masm.offset();
    if (UseSSE >= 2) {
      if (VerifyFPU) {
        masm.verify_FPU(0, "must be empty in SSE2+ mode");
      }
    } else {
      // External c_calling_convention expects the FPU stack to be 'clean'.
      // Compiled code leaves it dirty.  Do cleanup now.
      masm.empty_FPU_stack();
    }
    if (sizeof_FFree_Float_Stack_All == -1) {
      sizeof_FFree_Float_Stack_All = masm.offset() - start;
    } else {
      assert(masm.offset() - start == sizeof_FFree_Float_Stack_All, "wrong size");
    }
  
#line 33913 "ad_x86.cpp"
  }
  {

#line 1762 "src/hotspot/cpu/x86/x86_32.ad"

    // This is the instruction starting address for relocation info.
    MacroAssembler _masm(&cbuf);
    cbuf.set_insts_mark();
    emit_d8(cbuf, (0xE8 /*primary()*/));
    // CALL directly to the runtime
    emit_d32_reloc(cbuf, (opnd_array(1)->method() - (int)(cbuf.insts_end()) - 4),
                runtime_call_Relocation::spec(), RELOC_IMM32 );
    __ post_call_nop();

    if (UseSSE >= 2) {
      MacroAssembler _masm(&cbuf);
      BasicType rt = tf()->return_type();

      if ((rt == T_FLOAT || rt == T_DOUBLE) && !return_value_is_used()) {
        // A C runtime call where the return value is unused.  In SSE2+
        // mode the result needs to be removed from the FPU stack.  It's
        // likely that this function call could be removed by the
        // optimizer if the C function is a pure function.
        __ ffree(0);
      } else if (rt == T_FLOAT) {
        __ lea(rsp, Address(rsp, -4));
        __ fstp_s(Address(rsp, 0));
        __ movflt(xmm0, Address(rsp, 0));
        __ lea(rsp, Address(rsp,  4));
      } else if (rt == T_DOUBLE) {
        __ lea(rsp, Address(rsp, -8));
        __ fstp_d(Address(rsp, 0));
        __ movdbl(xmm0, Address(rsp, 0));
        __ lea(rsp, Address(rsp,  8));
      }
    }
  
#line 33951 "ad_x86.cpp"
  }
  {

#line 1755 "src/hotspot/cpu/x86/x86_32.ad"

    if( VerifyFPU ) {
      MacroAssembler masm(&cbuf);
      masm.verify_FPU( -3, "Returning from Runtime Leaf call");
    }
  
#line 33962 "ad_x86.cpp"
  }
  {

#line 1811 "src/hotspot/cpu/x86/x86_32.ad"

    // If method sets FPU control word do it here also
    if (Compile::current()->in_24_bit_fp_mode()) {
      MacroAssembler masm(&cbuf);
      masm.fldcw(ExternalAddress(StubRoutines::x86::addr_fpu_cntrl_wrd_24()));
    }
  
#line 33974 "ad_x86.cpp"
  }
}

void CallLeafNoFPDirectNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// 
  {

#line 1796 "src/hotspot/cpu/x86/x86_32.ad"

    // If method sets FPU control word restore it here
    debug_only(int off0 = cbuf.insts_size());
    if (ra_->C->in_24_bit_fp_mode()) {
      MacroAssembler _masm(&cbuf);
      __ fldcw(ExternalAddress(StubRoutines::x86::addr_fpu_cntrl_wrd_std()));
    }
    // Clear upper bits of YMM registers when current compiled code uses
    // wide vectors to avoid AVX <-> SSE transition penalty during call.
    MacroAssembler _masm(&cbuf);
    __ vzeroupper();
    debug_only(int off1 = cbuf.insts_size());
    assert(off1 - off0 == pre_call_resets_size(), "correct size prediction");
  
#line 34000 "ad_x86.cpp"
  }
  {

#line 1762 "src/hotspot/cpu/x86/x86_32.ad"

    // This is the instruction starting address for relocation info.
    MacroAssembler _masm(&cbuf);
    cbuf.set_insts_mark();
    emit_d8(cbuf, (0xE8 /*primary()*/));
    // CALL directly to the runtime
    emit_d32_reloc(cbuf, (opnd_array(1)->method() - (int)(cbuf.insts_end()) - 4),
                runtime_call_Relocation::spec(), RELOC_IMM32 );
    __ post_call_nop();

    if (UseSSE >= 2) {
      MacroAssembler _masm(&cbuf);
      BasicType rt = tf()->return_type();

      if ((rt == T_FLOAT || rt == T_DOUBLE) && !return_value_is_used()) {
        // A C runtime call where the return value is unused.  In SSE2+
        // mode the result needs to be removed from the FPU stack.  It's
        // likely that this function call could be removed by the
        // optimizer if the C function is a pure function.
        __ ffree(0);
      } else if (rt == T_FLOAT) {
        __ lea(rsp, Address(rsp, -4));
        __ fstp_s(Address(rsp, 0));
        __ movflt(xmm0, Address(rsp, 0));
        __ lea(rsp, Address(rsp,  4));
      } else if (rt == T_DOUBLE) {
        __ lea(rsp, Address(rsp, -8));
        __ fstp_d(Address(rsp, 0));
        __ movdbl(xmm0, Address(rsp, 0));
        __ lea(rsp, Address(rsp,  8));
      }
    }
  
#line 34038 "ad_x86.cpp"
  }
}

void RetNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 5;
  unsigned idx1 = 5; 	// 
  {

#line 1544 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0xC3 /*primary()*/));
  
#line 34053 "ad_x86.cpp"
  }
}

void TailCalljmpIndNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 5;
  unsigned idx1 = 5; 	// method_ptr
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// 
  {

#line 1544 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0xFF /*primary()*/));
  
#line 34069 "ad_x86.cpp"
  }
  {

#line 1697 "src/hotspot/cpu/x86/x86_32.ad"

    emit_rm(cbuf, 0x3, (0x4 /*secondary()*/), opnd_array(1)->reg(ra_,this,idx1)/* jump_target */ );
  
#line 34077 "ad_x86.cpp"
  }
}

void tailjmpIndNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 5;
  unsigned idx1 = 5; 	// ex_oop
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// 
  {

#line 2900 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf,0x5A);
  
#line 34093 "ad_x86.cpp"
  }
  {

#line 1544 "src/hotspot/cpu/x86/x86_32.ad"

    emit_opcode(cbuf, (0xFF /*primary()*/));
  
#line 34101 "ad_x86.cpp"
  }
  {

#line 1697 "src/hotspot/cpu/x86/x86_32.ad"

    emit_rm(cbuf, 0x3, (0x4 /*secondary()*/), opnd_array(1)->reg(ra_,this,idx1)/* jump_target */ );
  
#line 34109 "ad_x86.cpp"
  }
}

void CreateExceptionNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// 
  // User did not define which encode class to use.
}

uint CreateExceptionNode::size(PhaseRegAlloc *ra_) const {
  assert(VerifyOops || MachNode::size(ra_) <= 0, "bad fixed size");
  return (VerifyOops ? MachNode::size(ra_) : 0);
}

void RethrowExceptionNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 5;
  unsigned idx1 = 5; 	// 
  {

#line 2904 "src/hotspot/cpu/x86/x86_32.ad"

    MacroAssembler _masm(&cbuf);
    cbuf.set_insts_mark();
    emit_opcode(cbuf, 0xE9);        // jmp    entry
    emit_d32_reloc(cbuf, (int)OptoRuntime::rethrow_stub() - ((int)cbuf.insts_end())-4,
                   runtime_call_Relocation::spec(), RELOC_IMM32 );
    __ post_call_nop();
  
#line 34142 "ad_x86.cpp"
  }
}

void cmpFastLockRTMNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// object
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// box
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// tmp
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// scr
  unsigned idx5 = idx4 + opnd_array(4)->num_edges(); 	// cx1
  unsigned idx6 = idx5 + opnd_array(5)->num_edges(); 	// cx2
  unsigned idx7 = idx6 + opnd_array(6)->num_edges(); 	// thread
  {
    C2_MacroAssembler _masm(&cbuf);

#line 13769 "src/hotspot/cpu/x86/x86_32.ad"

    __ get_thread(opnd_array(7)->as_Register(ra_,this,idx7)/* thread */);
    __ fast_lock(opnd_array(1)->as_Register(ra_,this,idx1)/* object */, opnd_array(2)->as_Register(ra_,this,idx2)/* box */, opnd_array(3)->as_Register(ra_,this,idx3)/* tmp */,
                 opnd_array(4)->as_Register(ra_,this,idx4)/* scr */, opnd_array(5)->as_Register(ra_,this,idx5)/* cx1 */, opnd_array(6)->as_Register(ra_,this,idx6)/* cx2 */, opnd_array(7)->as_Register(ra_,this,idx7)/* thread */,
                 _rtm_counters, _stack_rtm_counters,
                 ((Method*)(ra_->C->method()->constant_encoding()))->method_data(),
                 true, ra_->C->profile_rtm());
  
#line 34169 "ad_x86.cpp"
  }
}

void cmpFastLockNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// object
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// box
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// tmp
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// scr
  unsigned idx5 = idx4 + opnd_array(4)->num_edges(); 	// thread
  {
    C2_MacroAssembler _masm(&cbuf);

#line 13786 "src/hotspot/cpu/x86/x86_32.ad"

    __ get_thread(opnd_array(5)->as_Register(ra_,this,idx5)/* thread */);
    __ fast_lock(opnd_array(1)->as_Register(ra_,this,idx1)/* object */, opnd_array(2)->as_Register(ra_,this,idx2)/* box */, opnd_array(3)->as_Register(ra_,this,idx3)/* tmp */,
                 opnd_array(4)->as_Register(ra_,this,idx4)/* scr */, noreg, noreg, opnd_array(5)->as_Register(ra_,this,idx5)/* thread */, nullptr, nullptr, nullptr, false, false);
  
#line 34191 "ad_x86.cpp"
  }
}

void cmpFastUnlockNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// object
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// box
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// tmp
  {
    C2_MacroAssembler _masm(&cbuf);

#line 13799 "src/hotspot/cpu/x86/x86_32.ad"

    __ fast_unlock(opnd_array(1)->as_Register(ra_,this,idx1)/* object */, opnd_array(2)->as_Register(ra_,this,idx2)/* box */, opnd_array(3)->as_Register(ra_,this,idx3)/* tmp */, ra_->C->use_rtm());
  
#line 34209 "ad_x86.cpp"
  }
}

void mask_all_evexL_LT32Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 13809 "src/hotspot/cpu/x86/x86_32.ad"

    int mask_len = Matcher::vector_length(this);
    __ vector_maskall_operation(opnd_array(0)->as_KRegister(ra_,this)/* dst */, opnd_array(1)->as_Register(ra_,this,idx1)/* src */, mask_len);
  
#line 34226 "ad_x86.cpp"
  }
}

void mask_all_evexL_GT32Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// ktmp
  {
    C2_MacroAssembler _masm(&cbuf);

#line 13821 "src/hotspot/cpu/x86/x86_32.ad"

    int mask_len = Matcher::vector_length(this);
    __ vector_maskall_operation32(opnd_array(0)->as_KRegister(ra_,this)/* dst */, opnd_array(1)->as_Register(ra_,this,idx1)/* src */, opnd_array(2)->as_KRegister(ra_,this,idx2)/* ktmp */, mask_len);
  
#line 34244 "ad_x86.cpp"
  }
}

void mask_all_evexI_GT32Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// ktmp
  {
    C2_MacroAssembler _masm(&cbuf);

#line 13833 "src/hotspot/cpu/x86/x86_32.ad"

    int mask_len = Matcher::vector_length(this);
    __ vector_maskall_operation32(opnd_array(0)->as_KRegister(ra_,this)/* dst */, opnd_array(1)->as_Register(ra_,this,idx1)/* src */, opnd_array(2)->as_KRegister(ra_,this,idx2)/* ktmp */, mask_len);
  
#line 34262 "ad_x86.cpp"
  }
}

void safePoint_poll_tlsNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 5;
  unsigned idx1 = 5; 	// cr
  {
    C2_MacroAssembler _masm(&cbuf);

#line 13850 "src/hotspot/cpu/x86/x86_32.ad"

    __ relocate(relocInfo::poll_type);
    address pre_pc = __ pc();
    __ testl(rax, Address(opnd_array(1)->as_Register(ra_,this,idx1)/* poll */, 0));
    address post_pc = __ pc();
    guarantee(pre_pc[0] == 0x85, "must emit test-ax [reg]");
  
#line 34282 "ad_x86.cpp"
  }
}

uint safePoint_poll_tlsNode::size(PhaseRegAlloc *ra_) const {
  assert(VerifyOops || MachNode::size(ra_) <= 2, "bad fixed size");
  return (VerifyOops ? MachNode::size(ra_) : 2);
}

void tlsLoadPNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// cr
  {
    C2_MacroAssembler _masm(&cbuf);

#line 13870 "src/hotspot/cpu/x86/x86_32.ad"

    Register dstReg = as_Register(opnd_array(0)->reg(ra_,this)/* dst */);
    __ get_thread(dstReg);
  
#line 34304 "ad_x86.cpp"
  }
}

void ShouldNotReachHereNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 5;
  unsigned idx1 = 5; 	// 
  {
    C2_MacroAssembler _masm(&cbuf);

#line 2928 "src/hotspot/cpu/x86/x86.ad"

    if (is_reachable()) {
      __ stop(_halt_reason);
    }
  
#line 34322 "ad_x86.cpp"
  }
}

void addF_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 2944 "src/hotspot/cpu/x86/x86.ad"

    __ addss(opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* dst */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src */);
  
#line 34339 "ad_x86.cpp"
  }
}

void addF_memNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 2956 "src/hotspot/cpu/x86/x86.ad"

    __ addss(opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* dst */, Address::make_raw(opnd_array(2)->base(ra_,this,idx2), opnd_array(2)->index(ra_,this,idx2), opnd_array(2)->scale(), opnd_array(2)->disp(ra_,this,idx2), opnd_array(2)->disp_reloc()));
  
#line 34356 "ad_x86.cpp"
  }
}

void addF_mem_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// dst
  {
    C2_MacroAssembler _masm(&cbuf);

#line 2956 "src/hotspot/cpu/x86/x86.ad"

    __ addss(opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* dst */, Address::make_raw(opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1), opnd_array(1)->disp_reloc()));
  
#line 34373 "ad_x86.cpp"
  }
}

void addF_immNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// con
  {
    C2_MacroAssembler _masm(&cbuf);

#line 2967 "src/hotspot/cpu/x86/x86.ad"

    __ addss(opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* dst */, InternalAddress(__ code()->consts()->start() + constant_offset()));
  
#line 34390 "ad_x86.cpp"
  }
}

void addF_immNode::eval_constant(Compile* C) {
  {

#line 2968 "src/hotspot/cpu/x86/x86.ad"
    _constant = C->output()->constant_table().add(this, opnd_array(2));
#line 34399 "ad_x86.cpp"
  }
}
void addF_reg_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 2979 "src/hotspot/cpu/x86/x86.ad"

    __ vaddss(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src1 */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src2 */);
  
#line 34415 "ad_x86.cpp"
  }
}

void addF_reg_memNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 2991 "src/hotspot/cpu/x86/x86.ad"

    __ vaddss(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src1 */, Address::make_raw(opnd_array(2)->base(ra_,this,idx2), opnd_array(2)->index(ra_,this,idx2), opnd_array(2)->scale(), opnd_array(2)->disp(ra_,this,idx2), opnd_array(2)->disp_reloc()));
  
#line 34432 "ad_x86.cpp"
  }
}

void addF_reg_mem_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// src2
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src1
  {
    C2_MacroAssembler _masm(&cbuf);

#line 2991 "src/hotspot/cpu/x86/x86.ad"

    __ vaddss(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src1 */, Address::make_raw(opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1), opnd_array(1)->disp_reloc()));
  
#line 34449 "ad_x86.cpp"
  }
}

void addF_reg_immNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// con
  {
    C2_MacroAssembler _masm(&cbuf);

#line 3003 "src/hotspot/cpu/x86/x86.ad"

    __ vaddss(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, InternalAddress(__ code()->consts()->start() + constant_offset()));
  
#line 34466 "ad_x86.cpp"
  }
}

void addF_reg_immNode::eval_constant(Compile* C) {
  {

#line 3004 "src/hotspot/cpu/x86/x86.ad"
    _constant = C->output()->constant_table().add(this, opnd_array(2));
#line 34475 "ad_x86.cpp"
  }
}
void addD_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 3015 "src/hotspot/cpu/x86/x86.ad"

    __ addsd(opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* dst */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src */);
  
#line 34491 "ad_x86.cpp"
  }
}

void addD_memNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 3027 "src/hotspot/cpu/x86/x86.ad"

    __ addsd(opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* dst */, Address::make_raw(opnd_array(2)->base(ra_,this,idx2), opnd_array(2)->index(ra_,this,idx2), opnd_array(2)->scale(), opnd_array(2)->disp(ra_,this,idx2), opnd_array(2)->disp_reloc()));
  
#line 34508 "ad_x86.cpp"
  }
}

void addD_mem_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// dst
  {
    C2_MacroAssembler _masm(&cbuf);

#line 3027 "src/hotspot/cpu/x86/x86.ad"

    __ addsd(opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* dst */, Address::make_raw(opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1), opnd_array(1)->disp_reloc()));
  
#line 34525 "ad_x86.cpp"
  }
}

void addD_immNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// con
  {
    C2_MacroAssembler _masm(&cbuf);

#line 3038 "src/hotspot/cpu/x86/x86.ad"

    __ addsd(opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* dst */, InternalAddress(__ code()->consts()->start() + constant_offset()));
  
#line 34542 "ad_x86.cpp"
  }
}

void addD_immNode::eval_constant(Compile* C) {
  {

#line 3039 "src/hotspot/cpu/x86/x86.ad"
    _constant = C->output()->constant_table().add(this, opnd_array(2));
#line 34551 "ad_x86.cpp"
  }
}
void addD_reg_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 3050 "src/hotspot/cpu/x86/x86.ad"

    __ vaddsd(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src1 */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src2 */);
  
#line 34567 "ad_x86.cpp"
  }
}

void addD_reg_memNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 3062 "src/hotspot/cpu/x86/x86.ad"

    __ vaddsd(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src1 */, Address::make_raw(opnd_array(2)->base(ra_,this,idx2), opnd_array(2)->index(ra_,this,idx2), opnd_array(2)->scale(), opnd_array(2)->disp(ra_,this,idx2), opnd_array(2)->disp_reloc()));
  
#line 34584 "ad_x86.cpp"
  }
}

void addD_reg_mem_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// src2
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src1
  {
    C2_MacroAssembler _masm(&cbuf);

#line 3062 "src/hotspot/cpu/x86/x86.ad"

    __ vaddsd(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src1 */, Address::make_raw(opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1), opnd_array(1)->disp_reloc()));
  
#line 34601 "ad_x86.cpp"
  }
}

void addD_reg_immNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// con
  {
    C2_MacroAssembler _masm(&cbuf);

#line 3074 "src/hotspot/cpu/x86/x86.ad"

    __ vaddsd(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, InternalAddress(__ code()->consts()->start() + constant_offset()));
  
#line 34618 "ad_x86.cpp"
  }
}

void addD_reg_immNode::eval_constant(Compile* C) {
  {

#line 3075 "src/hotspot/cpu/x86/x86.ad"
    _constant = C->output()->constant_table().add(this, opnd_array(2));
#line 34627 "ad_x86.cpp"
  }
}
void subF_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 3086 "src/hotspot/cpu/x86/x86.ad"

    __ subss(opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* dst */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src */);
  
#line 34643 "ad_x86.cpp"
  }
}

void subF_memNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 3098 "src/hotspot/cpu/x86/x86.ad"

    __ subss(opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* dst */, Address::make_raw(opnd_array(2)->base(ra_,this,idx2), opnd_array(2)->index(ra_,this,idx2), opnd_array(2)->scale(), opnd_array(2)->disp(ra_,this,idx2), opnd_array(2)->disp_reloc()));
  
#line 34660 "ad_x86.cpp"
  }
}

void subF_immNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// con
  {
    C2_MacroAssembler _masm(&cbuf);

#line 3109 "src/hotspot/cpu/x86/x86.ad"

    __ subss(opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* dst */, InternalAddress(__ code()->consts()->start() + constant_offset()));
  
#line 34677 "ad_x86.cpp"
  }
}

void subF_immNode::eval_constant(Compile* C) {
  {

#line 3110 "src/hotspot/cpu/x86/x86.ad"
    _constant = C->output()->constant_table().add(this, opnd_array(2));
#line 34686 "ad_x86.cpp"
  }
}
void subF_reg_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 3121 "src/hotspot/cpu/x86/x86.ad"

    __ vsubss(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src1 */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src2 */);
  
#line 34702 "ad_x86.cpp"
  }
}

void subF_reg_memNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 3133 "src/hotspot/cpu/x86/x86.ad"

    __ vsubss(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src1 */, Address::make_raw(opnd_array(2)->base(ra_,this,idx2), opnd_array(2)->index(ra_,this,idx2), opnd_array(2)->scale(), opnd_array(2)->disp(ra_,this,idx2), opnd_array(2)->disp_reloc()));
  
#line 34719 "ad_x86.cpp"
  }
}

void subF_reg_immNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// con
  {
    C2_MacroAssembler _masm(&cbuf);

#line 3145 "src/hotspot/cpu/x86/x86.ad"

    __ vsubss(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, InternalAddress(__ code()->consts()->start() + constant_offset()));
  
#line 34736 "ad_x86.cpp"
  }
}

void subF_reg_immNode::eval_constant(Compile* C) {
  {

#line 3146 "src/hotspot/cpu/x86/x86.ad"
    _constant = C->output()->constant_table().add(this, opnd_array(2));
#line 34745 "ad_x86.cpp"
  }
}
void subD_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 3157 "src/hotspot/cpu/x86/x86.ad"

    __ subsd(opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* dst */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src */);
  
#line 34761 "ad_x86.cpp"
  }
}

void subD_memNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 3169 "src/hotspot/cpu/x86/x86.ad"

    __ subsd(opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* dst */, Address::make_raw(opnd_array(2)->base(ra_,this,idx2), opnd_array(2)->index(ra_,this,idx2), opnd_array(2)->scale(), opnd_array(2)->disp(ra_,this,idx2), opnd_array(2)->disp_reloc()));
  
#line 34778 "ad_x86.cpp"
  }
}

void subD_immNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// con
  {
    C2_MacroAssembler _masm(&cbuf);

#line 3180 "src/hotspot/cpu/x86/x86.ad"

    __ subsd(opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* dst */, InternalAddress(__ code()->consts()->start() + constant_offset()));
  
#line 34795 "ad_x86.cpp"
  }
}

void subD_immNode::eval_constant(Compile* C) {
  {

#line 3181 "src/hotspot/cpu/x86/x86.ad"
    _constant = C->output()->constant_table().add(this, opnd_array(2));
#line 34804 "ad_x86.cpp"
  }
}
void subD_reg_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 3192 "src/hotspot/cpu/x86/x86.ad"

    __ vsubsd(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src1 */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src2 */);
  
#line 34820 "ad_x86.cpp"
  }
}

void subD_reg_memNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 3204 "src/hotspot/cpu/x86/x86.ad"

    __ vsubsd(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src1 */, Address::make_raw(opnd_array(2)->base(ra_,this,idx2), opnd_array(2)->index(ra_,this,idx2), opnd_array(2)->scale(), opnd_array(2)->disp(ra_,this,idx2), opnd_array(2)->disp_reloc()));
  
#line 34837 "ad_x86.cpp"
  }
}

void subD_reg_immNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// con
  {
    C2_MacroAssembler _masm(&cbuf);

#line 3216 "src/hotspot/cpu/x86/x86.ad"

    __ vsubsd(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, InternalAddress(__ code()->consts()->start() + constant_offset()));
  
#line 34854 "ad_x86.cpp"
  }
}

void subD_reg_immNode::eval_constant(Compile* C) {
  {

#line 3217 "src/hotspot/cpu/x86/x86.ad"
    _constant = C->output()->constant_table().add(this, opnd_array(2));
#line 34863 "ad_x86.cpp"
  }
}
void mulF_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 3228 "src/hotspot/cpu/x86/x86.ad"

    __ mulss(opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* dst */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src */);
  
#line 34879 "ad_x86.cpp"
  }
}

void mulF_memNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 3240 "src/hotspot/cpu/x86/x86.ad"

    __ mulss(opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* dst */, Address::make_raw(opnd_array(2)->base(ra_,this,idx2), opnd_array(2)->index(ra_,this,idx2), opnd_array(2)->scale(), opnd_array(2)->disp(ra_,this,idx2), opnd_array(2)->disp_reloc()));
  
#line 34896 "ad_x86.cpp"
  }
}

void mulF_mem_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// dst
  {
    C2_MacroAssembler _masm(&cbuf);

#line 3240 "src/hotspot/cpu/x86/x86.ad"

    __ mulss(opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* dst */, Address::make_raw(opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1), opnd_array(1)->disp_reloc()));
  
#line 34913 "ad_x86.cpp"
  }
}

void mulF_immNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// con
  {
    C2_MacroAssembler _masm(&cbuf);

#line 3251 "src/hotspot/cpu/x86/x86.ad"

    __ mulss(opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* dst */, InternalAddress(__ code()->consts()->start() + constant_offset()));
  
#line 34930 "ad_x86.cpp"
  }
}

void mulF_immNode::eval_constant(Compile* C) {
  {

#line 3252 "src/hotspot/cpu/x86/x86.ad"
    _constant = C->output()->constant_table().add(this, opnd_array(2));
#line 34939 "ad_x86.cpp"
  }
}
void mulF_reg_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 3263 "src/hotspot/cpu/x86/x86.ad"

    __ vmulss(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src1 */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src2 */);
  
#line 34955 "ad_x86.cpp"
  }
}

void mulF_reg_memNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 3275 "src/hotspot/cpu/x86/x86.ad"

    __ vmulss(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src1 */, Address::make_raw(opnd_array(2)->base(ra_,this,idx2), opnd_array(2)->index(ra_,this,idx2), opnd_array(2)->scale(), opnd_array(2)->disp(ra_,this,idx2), opnd_array(2)->disp_reloc()));
  
#line 34972 "ad_x86.cpp"
  }
}

void mulF_reg_mem_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// src2
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src1
  {
    C2_MacroAssembler _masm(&cbuf);

#line 3275 "src/hotspot/cpu/x86/x86.ad"

    __ vmulss(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src1 */, Address::make_raw(opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1), opnd_array(1)->disp_reloc()));
  
#line 34989 "ad_x86.cpp"
  }
}

void mulF_reg_immNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// con
  {
    C2_MacroAssembler _masm(&cbuf);

#line 3287 "src/hotspot/cpu/x86/x86.ad"

    __ vmulss(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, InternalAddress(__ code()->consts()->start() + constant_offset()));
  
#line 35006 "ad_x86.cpp"
  }
}

void mulF_reg_immNode::eval_constant(Compile* C) {
  {

#line 3288 "src/hotspot/cpu/x86/x86.ad"
    _constant = C->output()->constant_table().add(this, opnd_array(2));
#line 35015 "ad_x86.cpp"
  }
}
void mulD_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 3299 "src/hotspot/cpu/x86/x86.ad"

    __ mulsd(opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* dst */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src */);
  
#line 35031 "ad_x86.cpp"
  }
}

void mulD_memNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 3311 "src/hotspot/cpu/x86/x86.ad"

    __ mulsd(opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* dst */, Address::make_raw(opnd_array(2)->base(ra_,this,idx2), opnd_array(2)->index(ra_,this,idx2), opnd_array(2)->scale(), opnd_array(2)->disp(ra_,this,idx2), opnd_array(2)->disp_reloc()));
  
#line 35048 "ad_x86.cpp"
  }
}

void mulD_mem_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// dst
  {
    C2_MacroAssembler _masm(&cbuf);

#line 3311 "src/hotspot/cpu/x86/x86.ad"

    __ mulsd(opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* dst */, Address::make_raw(opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1), opnd_array(1)->disp_reloc()));
  
#line 35065 "ad_x86.cpp"
  }
}

void mulD_immNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// con
  {
    C2_MacroAssembler _masm(&cbuf);

#line 3322 "src/hotspot/cpu/x86/x86.ad"

    __ mulsd(opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* dst */, InternalAddress(__ code()->consts()->start() + constant_offset()));
  
#line 35082 "ad_x86.cpp"
  }
}

void mulD_immNode::eval_constant(Compile* C) {
  {

#line 3323 "src/hotspot/cpu/x86/x86.ad"
    _constant = C->output()->constant_table().add(this, opnd_array(2));
#line 35091 "ad_x86.cpp"
  }
}
void mulD_reg_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 3334 "src/hotspot/cpu/x86/x86.ad"

    __ vmulsd(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src1 */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src2 */);
  
#line 35107 "ad_x86.cpp"
  }
}

void mulD_reg_memNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 3346 "src/hotspot/cpu/x86/x86.ad"

    __ vmulsd(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src1 */, Address::make_raw(opnd_array(2)->base(ra_,this,idx2), opnd_array(2)->index(ra_,this,idx2), opnd_array(2)->scale(), opnd_array(2)->disp(ra_,this,idx2), opnd_array(2)->disp_reloc()));
  
#line 35124 "ad_x86.cpp"
  }
}

void mulD_reg_mem_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// src2
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src1
  {
    C2_MacroAssembler _masm(&cbuf);

#line 3346 "src/hotspot/cpu/x86/x86.ad"

    __ vmulsd(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src1 */, Address::make_raw(opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1), opnd_array(1)->disp_reloc()));
  
#line 35141 "ad_x86.cpp"
  }
}

void mulD_reg_immNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// con
  {
    C2_MacroAssembler _masm(&cbuf);

#line 3358 "src/hotspot/cpu/x86/x86.ad"

    __ vmulsd(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, InternalAddress(__ code()->consts()->start() + constant_offset()));
  
#line 35158 "ad_x86.cpp"
  }
}

void mulD_reg_immNode::eval_constant(Compile* C) {
  {

#line 3359 "src/hotspot/cpu/x86/x86.ad"
    _constant = C->output()->constant_table().add(this, opnd_array(2));
#line 35167 "ad_x86.cpp"
  }
}
void divF_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 3370 "src/hotspot/cpu/x86/x86.ad"

    __ divss(opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* dst */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src */);
  
#line 35183 "ad_x86.cpp"
  }
}

void divF_memNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 3382 "src/hotspot/cpu/x86/x86.ad"

    __ divss(opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* dst */, Address::make_raw(opnd_array(2)->base(ra_,this,idx2), opnd_array(2)->index(ra_,this,idx2), opnd_array(2)->scale(), opnd_array(2)->disp(ra_,this,idx2), opnd_array(2)->disp_reloc()));
  
#line 35200 "ad_x86.cpp"
  }
}

void divF_immNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// con
  {
    C2_MacroAssembler _masm(&cbuf);

#line 3393 "src/hotspot/cpu/x86/x86.ad"

    __ divss(opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* dst */, InternalAddress(__ code()->consts()->start() + constant_offset()));
  
#line 35217 "ad_x86.cpp"
  }
}

void divF_immNode::eval_constant(Compile* C) {
  {

#line 3394 "src/hotspot/cpu/x86/x86.ad"
    _constant = C->output()->constant_table().add(this, opnd_array(2));
#line 35226 "ad_x86.cpp"
  }
}
void divF_reg_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 3405 "src/hotspot/cpu/x86/x86.ad"

    __ vdivss(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src1 */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src2 */);
  
#line 35242 "ad_x86.cpp"
  }
}

void divF_reg_memNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 3417 "src/hotspot/cpu/x86/x86.ad"

    __ vdivss(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src1 */, Address::make_raw(opnd_array(2)->base(ra_,this,idx2), opnd_array(2)->index(ra_,this,idx2), opnd_array(2)->scale(), opnd_array(2)->disp(ra_,this,idx2), opnd_array(2)->disp_reloc()));
  
#line 35259 "ad_x86.cpp"
  }
}

void divF_reg_immNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// con
  {
    C2_MacroAssembler _masm(&cbuf);

#line 3429 "src/hotspot/cpu/x86/x86.ad"

    __ vdivss(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, InternalAddress(__ code()->consts()->start() + constant_offset()));
  
#line 35276 "ad_x86.cpp"
  }
}

void divF_reg_immNode::eval_constant(Compile* C) {
  {

#line 3430 "src/hotspot/cpu/x86/x86.ad"
    _constant = C->output()->constant_table().add(this, opnd_array(2));
#line 35285 "ad_x86.cpp"
  }
}
void divD_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 3441 "src/hotspot/cpu/x86/x86.ad"

    __ divsd(opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* dst */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src */);
  
#line 35301 "ad_x86.cpp"
  }
}

void divD_memNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 3453 "src/hotspot/cpu/x86/x86.ad"

    __ divsd(opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* dst */, Address::make_raw(opnd_array(2)->base(ra_,this,idx2), opnd_array(2)->index(ra_,this,idx2), opnd_array(2)->scale(), opnd_array(2)->disp(ra_,this,idx2), opnd_array(2)->disp_reloc()));
  
#line 35318 "ad_x86.cpp"
  }
}

void divD_immNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// con
  {
    C2_MacroAssembler _masm(&cbuf);

#line 3464 "src/hotspot/cpu/x86/x86.ad"

    __ divsd(opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* dst */, InternalAddress(__ code()->consts()->start() + constant_offset()));
  
#line 35335 "ad_x86.cpp"
  }
}

void divD_immNode::eval_constant(Compile* C) {
  {

#line 3465 "src/hotspot/cpu/x86/x86.ad"
    _constant = C->output()->constant_table().add(this, opnd_array(2));
#line 35344 "ad_x86.cpp"
  }
}
void divD_reg_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 3476 "src/hotspot/cpu/x86/x86.ad"

    __ vdivsd(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src1 */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src2 */);
  
#line 35360 "ad_x86.cpp"
  }
}

void divD_reg_memNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 3488 "src/hotspot/cpu/x86/x86.ad"

    __ vdivsd(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src1 */, Address::make_raw(opnd_array(2)->base(ra_,this,idx2), opnd_array(2)->index(ra_,this,idx2), opnd_array(2)->scale(), opnd_array(2)->disp(ra_,this,idx2), opnd_array(2)->disp_reloc()));
  
#line 35377 "ad_x86.cpp"
  }
}

void divD_reg_immNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// con
  {
    C2_MacroAssembler _masm(&cbuf);

#line 3500 "src/hotspot/cpu/x86/x86.ad"

    __ vdivsd(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, InternalAddress(__ code()->consts()->start() + constant_offset()));
  
#line 35394 "ad_x86.cpp"
  }
}

void divD_reg_immNode::eval_constant(Compile* C) {
  {

#line 3501 "src/hotspot/cpu/x86/x86.ad"
    _constant = C->output()->constant_table().add(this, opnd_array(2));
#line 35403 "ad_x86.cpp"
  }
}
void absF_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  {
    C2_MacroAssembler _masm(&cbuf);

#line 3511 "src/hotspot/cpu/x86/x86.ad"

    __ andps(opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* dst */, ExternalAddress(float_signmask()));
  
#line 35418 "ad_x86.cpp"
  }
}

void absF_reg_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 3522 "src/hotspot/cpu/x86/x86.ad"

    int vlen_enc = Assembler::AVX_128bit;
    __ vandps(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */,
              ExternalAddress(float_signmask()), vlen_enc);
  
#line 35436 "ad_x86.cpp"
  }
}

void absD_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  {
    C2_MacroAssembler _masm(&cbuf);

#line 3536 "src/hotspot/cpu/x86/x86.ad"

    __ andpd(opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* dst */, ExternalAddress(double_signmask()));
  
#line 35452 "ad_x86.cpp"
  }
}

void absD_reg_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 3548 "src/hotspot/cpu/x86/x86.ad"

    int vlen_enc = Assembler::AVX_128bit;
    __ vandpd(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */,
              ExternalAddress(double_signmask()), vlen_enc);
  
#line 35470 "ad_x86.cpp"
  }
}

void negF_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  {
    C2_MacroAssembler _masm(&cbuf);

#line 3561 "src/hotspot/cpu/x86/x86.ad"

    __ xorps(opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* dst */, ExternalAddress(float_signflip()));
  
#line 35486 "ad_x86.cpp"
  }
}

void negF_reg_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 3572 "src/hotspot/cpu/x86/x86.ad"

    __ vnegatess(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */,
                 ExternalAddress(float_signflip()));
  
#line 35503 "ad_x86.cpp"
  }
}

void negD_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  {
    C2_MacroAssembler _masm(&cbuf);

#line 3585 "src/hotspot/cpu/x86/x86.ad"

    __ xorpd(opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* dst */, ExternalAddress(double_signflip()));
  
#line 35519 "ad_x86.cpp"
  }
}

void negD_reg_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 3597 "src/hotspot/cpu/x86/x86.ad"

    __ vnegatesd(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */,
                 ExternalAddress(double_signflip()));
  
#line 35536 "ad_x86.cpp"
  }
}

void sqrtF_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  {
    C2_MacroAssembler _masm(&cbuf);

#line 3610 "src/hotspot/cpu/x86/x86.ad"

    __ sqrtss(opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* dst */);
  
#line 35552 "ad_x86.cpp"
  }
}

void sqrtD_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  {
    C2_MacroAssembler _masm(&cbuf);

#line 3622 "src/hotspot/cpu/x86/x86.ad"

    __ sqrtsd(opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* dst */);
  
#line 35568 "ad_x86.cpp"
  }
}

void convF2HF_reg_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// tmp
  {
    C2_MacroAssembler _masm(&cbuf);

#line 3633 "src/hotspot/cpu/x86/x86.ad"

    __ flt_to_flt16(opnd_array(0)->as_Register(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* tmp */);
  
#line 35585 "ad_x86.cpp"
  }
}

void convF2HF_mem_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// ktmp
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// rtmp
  {
    C2_MacroAssembler _masm(&cbuf);

#line 3644 "src/hotspot/cpu/x86/x86.ad"

    __ movl(opnd_array(4)->as_Register(ra_,this,idx4)/* rtmp */, 0x1);
    __ kmovwl(opnd_array(3)->as_KRegister(ra_,this,idx3)/* ktmp */, opnd_array(4)->as_Register(ra_,this,idx4)/* rtmp */);
    __ evcvtps2ph(Address::make_raw(opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1), opnd_array(1)->disp_reloc()), opnd_array(3)->as_KRegister(ra_,this,idx3)/* ktmp */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src */, 0x04, Assembler::AVX_128bit);
  
#line 35606 "ad_x86.cpp"
  }
}

void vconvF2HFNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 3655 "src/hotspot/cpu/x86/x86.ad"

    int vlen_enc = vector_length_encoding(this, opnd_array(1));
    __ vcvtps2ph(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, 0x04, vlen_enc);
  
#line 35623 "ad_x86.cpp"
  }
}

void vconvF2HF_mem_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 3665 "src/hotspot/cpu/x86/x86.ad"

    int vlen_enc = vector_length_encoding(this, opnd_array(2));
    __ vcvtps2ph(Address::make_raw(opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1), opnd_array(1)->disp_reloc()), opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src */, 0x04, vlen_enc);
  
#line 35641 "ad_x86.cpp"
  }
}

void convHF2F_reg_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 3675 "src/hotspot/cpu/x86/x86.ad"

    __ flt16_to_flt(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_Register(ra_,this,idx1)/* src */);
  
#line 35657 "ad_x86.cpp"
  }
}

void vconvHF2F_reg_memNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  {
    C2_MacroAssembler _masm(&cbuf);

#line 3684 "src/hotspot/cpu/x86/x86.ad"

    int vlen_enc = vector_length_encoding(this);
    __ vcvtph2ps(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, Address::make_raw(opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1), opnd_array(1)->disp_reloc()), vlen_enc);
  
#line 35674 "ad_x86.cpp"
  }
}

void vconvHF2FNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 3695 "src/hotspot/cpu/x86/x86.ad"

    int vlen_enc = vector_length_encoding(this);
    __ vcvtph2ps(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, vlen_enc);
  
#line 35691 "ad_x86.cpp"
  }
}

void reinterpret_maskNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  {
    C2_MacroAssembler _masm(&cbuf);

#line 3709 "src/hotspot/cpu/x86/x86.ad"

    // empty
  
#line 35707 "ad_x86.cpp"
  }
}

void reinterpret_mask_W2BNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// xtmp
  {
    C2_MacroAssembler _masm(&cbuf);

#line 3724 "src/hotspot/cpu/x86/x86.ad"

     int src_sz = Matcher::vector_length(this, opnd_array(1))*type2aelembytes(T_SHORT);
     int dst_sz = Matcher::vector_length(this)*type2aelembytes(T_BYTE);
     assert(src_sz == dst_sz , "src and dst size mismatch");
     int vlen_enc = vector_length_encoding(src_sz);
     __  evpmovm2w(opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* xtmp */, opnd_array(1)->as_KRegister(ra_,this,idx1)/* src */, vlen_enc);
     __  evpmovb2m(opnd_array(0)->as_KRegister(ra_,this)/* dst */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* xtmp */, vlen_enc);
  
#line 35729 "ad_x86.cpp"
  }
}

void reinterpret_mask_D2BNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// xtmp
  {
    C2_MacroAssembler _masm(&cbuf);

#line 3745 "src/hotspot/cpu/x86/x86.ad"

     int src_sz = Matcher::vector_length(this, opnd_array(1))*type2aelembytes(T_INT);
     int dst_sz = Matcher::vector_length(this)*type2aelembytes(T_BYTE);
     assert(src_sz == dst_sz , "src and dst size mismatch");
     int vlen_enc = vector_length_encoding(src_sz);
     __  evpmovm2d(opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* xtmp */, opnd_array(1)->as_KRegister(ra_,this,idx1)/* src */, vlen_enc);
     __  evpmovb2m(opnd_array(0)->as_KRegister(ra_,this)/* dst */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* xtmp */, vlen_enc);
  
#line 35751 "ad_x86.cpp"
  }
}

void reinterpret_mask_Q2BNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// xtmp
  {
    C2_MacroAssembler _masm(&cbuf);

#line 3766 "src/hotspot/cpu/x86/x86.ad"

     int src_sz = Matcher::vector_length(this, opnd_array(1))*type2aelembytes(T_LONG);
     int dst_sz = Matcher::vector_length(this)*type2aelembytes(T_BYTE);
     assert(src_sz == dst_sz , "src and dst size mismatch");
     int vlen_enc = vector_length_encoding(src_sz);
     __  evpmovm2q(opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* xtmp */, opnd_array(1)->as_KRegister(ra_,this,idx1)/* src */, vlen_enc);
     __  evpmovb2m(opnd_array(0)->as_KRegister(ra_,this)/* dst */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* xtmp */, vlen_enc);
  
#line 35773 "ad_x86.cpp"
  }
}

void reinterpretNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  {
    C2_MacroAssembler _masm(&cbuf);

#line 3783 "src/hotspot/cpu/x86/x86.ad"

    // empty
  
#line 35789 "ad_x86.cpp"
  }
}

void reinterpret_expandNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// dst
  {
    C2_MacroAssembler _masm(&cbuf);

#line 3796 "src/hotspot/cpu/x86/x86.ad"

    assert(Matcher::vector_length_in_bytes(this)       <= 16, "required");
    assert(Matcher::vector_length_in_bytes(this, opnd_array(1)) <=  8, "required");

    int src_vlen_in_bytes = Matcher::vector_length_in_bytes(this, opnd_array(1));
    if (src_vlen_in_bytes == 4) {
      __ movdqu(opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* dst */, ExternalAddress(vector_32_bit_mask()), noreg);
    } else {
      assert(src_vlen_in_bytes == 8, "");
      __ movdqu(opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* dst */, ExternalAddress(vector_64_bit_mask()), noreg);
    }
    __ pand(opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */);
  
#line 35816 "ad_x86.cpp"
  }
}

void vreinterpret_expand4Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 3820 "src/hotspot/cpu/x86/x86.ad"

    __ vpand(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, ExternalAddress(vector_32_bit_mask()), 0, noreg);
  
#line 35832 "ad_x86.cpp"
  }
}

void vreinterpret_expandNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 3835 "src/hotspot/cpu/x86/x86.ad"

    switch (Matcher::vector_length_in_bytes(this, opnd_array(1))) {
      case  8: __ movq   (opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */); break;
      case 16: __ movdqu (opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */); break;
      case 32: __ vmovdqu(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */); break;
      default: ShouldNotReachHere();
    }
  
#line 35853 "ad_x86.cpp"
  }
}

void reinterpret_shrinkNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 3852 "src/hotspot/cpu/x86/x86.ad"

    switch (Matcher::vector_length_in_bytes(this)) {
      case  4: __ movfltz(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */); break;
      case  8: __ movq   (opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */); break;
      case 16: __ movdqu (opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */); break;
      case 32: __ vmovdqu(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */); break;
      default: ShouldNotReachHere();
    }
  
#line 35875 "ad_x86.cpp"
  }
}

void onspinwaitNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// 
  {
    C2_MacroAssembler _masm(&cbuf);

#line 3955 "src/hotspot/cpu/x86/x86.ad"

    __ pause();
  
#line 35891 "ad_x86.cpp"
  }
}

void fmaD_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// c
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// a
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// b
  {
    C2_MacroAssembler _masm(&cbuf);

#line 3967 "src/hotspot/cpu/x86/x86.ad"

    __ fmad(opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* c */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* a */, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* b */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* c */);
  
#line 35909 "ad_x86.cpp"
  }
}

void fmaF_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// c
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// a
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// b
  {
    C2_MacroAssembler _masm(&cbuf);

#line 3979 "src/hotspot/cpu/x86/x86.ad"

    __ fmaf(opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* c */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* a */, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* b */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* c */);
  
#line 35927 "ad_x86.cpp"
  }
}

void MoveVec2LegNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 3991 "src/hotspot/cpu/x86/x86.ad"

    ShouldNotReachHere();
  
#line 35943 "ad_x86.cpp"
  }
}

void MoveLeg2VecNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 4000 "src/hotspot/cpu/x86/x86.ad"

    ShouldNotReachHere();
  
#line 35959 "ad_x86.cpp"
  }
}

void loadVNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  {
    C2_MacroAssembler _masm(&cbuf);

#line 4013 "src/hotspot/cpu/x86/x86.ad"

    __ load_vector(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, Address::make_raw(opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1), opnd_array(1)->disp_reloc()), Matcher::vector_length_in_bytes(this));
  
#line 35975 "ad_x86.cpp"
  }
}

void storeVNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 4024 "src/hotspot/cpu/x86/x86.ad"

    switch (Matcher::vector_length_in_bytes(this, opnd_array(2))) {
      case  4: __ movdl    (Address::make_raw(opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1), opnd_array(1)->disp_reloc()), opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src */); break;
      case  8: __ movq     (Address::make_raw(opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1), opnd_array(1)->disp_reloc()), opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src */); break;
      case 16: __ movdqu   (Address::make_raw(opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1), opnd_array(1)->disp_reloc()), opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src */); break;
      case 32: __ vmovdqu  (Address::make_raw(opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1), opnd_array(1)->disp_reloc()), opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src */); break;
      case 64: __ evmovdqul(Address::make_raw(opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1), opnd_array(1)->disp_reloc()), opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src */, Assembler::AVX_512bit); break;
      default: ShouldNotReachHere();
    }
  
#line 35999 "ad_x86.cpp"
  }
}

void gatherNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// idx
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// dst
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// tmp
  unsigned idx5 = idx4 + opnd_array(4)->num_edges(); 	// mask
  {
    C2_MacroAssembler _masm(&cbuf);

#line 4046 "src/hotspot/cpu/x86/x86.ad"

    assert(UseAVX >= 2, "sanity");

    int vlen_enc = vector_length_encoding(this);
    BasicType elem_bt = Matcher::vector_element_basic_type(this);

    assert(Matcher::vector_length_in_bytes(this) >= 16, "sanity");
    assert(!is_subword_type(elem_bt), "sanity"); // T_INT, T_LONG, T_FLOAT, T_DOUBLE

    if (vlen_enc == Assembler::AVX_128bit) {
      __ movdqu(opnd_array(5)->as_XMMRegister(ra_,this,idx5)/* mask */, ExternalAddress(vector_all_bits_set()), noreg);
    } else {
      __ vmovdqu(opnd_array(5)->as_XMMRegister(ra_,this,idx5)/* mask */, ExternalAddress(vector_all_bits_set()), noreg);
    }
    __ lea(opnd_array(4)->as_Register(ra_,this,idx4)/* tmp */, Address::make_raw(opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1), opnd_array(1)->disp_reloc()));
    __ vgather(elem_bt, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(4)->as_Register(ra_,this,idx4)/* tmp */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* idx */, opnd_array(5)->as_XMMRegister(ra_,this,idx5)/* mask */, vlen_enc);
  
#line 36033 "ad_x86.cpp"
  }
}

void evgatherNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// idx
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// dst
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// tmp
  unsigned idx5 = idx4 + opnd_array(4)->num_edges(); 	// ktmp
  {
    C2_MacroAssembler _masm(&cbuf);

#line 4071 "src/hotspot/cpu/x86/x86.ad"

    assert(UseAVX > 2, "sanity");

    int vlen_enc = vector_length_encoding(this);
    BasicType elem_bt = Matcher::vector_element_basic_type(this);

    assert(!is_subword_type(elem_bt), "sanity"); // T_INT, T_LONG, T_FLOAT, T_DOUBLE

    __ kmovwl(opnd_array(5)->as_KRegister(ra_,this,idx5)/* ktmp */, ExternalAddress(vector_all_bits_set()), noreg);
    __ lea(opnd_array(4)->as_Register(ra_,this,idx4)/* tmp */, Address::make_raw(opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1), opnd_array(1)->disp_reloc()));
    __ evgather(elem_bt, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(5)->as_KRegister(ra_,this,idx5)/* ktmp */, opnd_array(4)->as_Register(ra_,this,idx4)/* tmp */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* idx */, vlen_enc);
  
#line 36062 "ad_x86.cpp"
  }
}

void evgather_maskedNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// idx
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// mask
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// dst
  unsigned idx5 = idx4 + opnd_array(4)->num_edges(); 	// ktmp
  unsigned idx6 = idx5 + opnd_array(5)->num_edges(); 	// tmp
  {
    C2_MacroAssembler _masm(&cbuf);

#line 4090 "src/hotspot/cpu/x86/x86.ad"

    assert(UseAVX > 2, "sanity");
    int vlen_enc = vector_length_encoding(this);
    BasicType elem_bt = Matcher::vector_element_basic_type(this);
    assert(!is_subword_type(elem_bt), "sanity"); // T_INT, T_LONG, T_FLOAT, T_DOUBLE
    // Note: Since gather instruction partially updates the opmask register used
    // for predication hense moving mask operand to a temporary.
    __ kmovwl(opnd_array(5)->as_KRegister(ra_,this,idx5)/* ktmp */, opnd_array(3)->as_KRegister(ra_,this,idx3)/* mask */);
    __ vpxor(opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* dst */, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* dst */, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* dst */, vlen_enc);
    __ lea(opnd_array(6)->as_Register(ra_,this,idx6)/* tmp */, Address::make_raw(opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1), opnd_array(1)->disp_reloc()));
    __ evgather(elem_bt, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* dst */, opnd_array(5)->as_KRegister(ra_,this,idx5)/* ktmp */, opnd_array(6)->as_Register(ra_,this,idx6)/* tmp */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* idx */, vlen_enc);
  
#line 36092 "ad_x86.cpp"
  }
}

void scatterNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// idx
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// tmp
  unsigned idx5 = idx4 + opnd_array(4)->num_edges(); 	// ktmp
  {
    C2_MacroAssembler _masm(&cbuf);

#line 4113 "src/hotspot/cpu/x86/x86.ad"

    int vlen_enc = vector_length_encoding(this, opnd_array(2));
    BasicType elem_bt = Matcher::vector_element_basic_type(this, opnd_array(2));

    assert(Matcher::vector_length_in_bytes(this, opnd_array(2)) >= 16, "sanity");
    assert(!is_subword_type(elem_bt), "sanity"); // T_INT, T_LONG, T_FLOAT, T_DOUBLE

    __ kmovwl(opnd_array(5)->as_KRegister(ra_,this,idx5)/* ktmp */, ExternalAddress(vector_all_bits_set()), noreg);
    __ lea(opnd_array(4)->as_Register(ra_,this,idx4)/* tmp */, Address::make_raw(opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1), opnd_array(1)->disp_reloc()));
    __ evscatter(elem_bt, opnd_array(4)->as_Register(ra_,this,idx4)/* tmp */, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* idx */, opnd_array(5)->as_KRegister(ra_,this,idx5)/* ktmp */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src */, vlen_enc);
  
#line 36120 "ad_x86.cpp"
  }
}

void scatter_maskedNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// idx
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// mask
  unsigned idx5 = idx4 + opnd_array(4)->num_edges(); 	// ktmp
  unsigned idx6 = idx5 + opnd_array(5)->num_edges(); 	// tmp
  {
    C2_MacroAssembler _masm(&cbuf);

#line 4131 "src/hotspot/cpu/x86/x86.ad"

    int vlen_enc = vector_length_encoding(this, opnd_array(2));
    BasicType elem_bt = Matcher::vector_element_basic_type(this, opnd_array(2));
    assert(Matcher::vector_length_in_bytes(this, opnd_array(2)) >= 16, "sanity");
    assert(!is_subword_type(elem_bt), "sanity"); // T_INT, T_LONG, T_FLOAT, T_DOUBLE
    // Note: Since scatter instruction partially updates the opmask register used
    // for predication hense moving mask operand to a temporary.
    __ kmovwl(opnd_array(5)->as_KRegister(ra_,this,idx5)/* ktmp */, opnd_array(4)->as_KRegister(ra_,this,idx4)/* mask */);
    __ lea(opnd_array(6)->as_Register(ra_,this,idx6)/* tmp */, Address::make_raw(opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1), opnd_array(1)->disp_reloc()));
    __ evscatter(elem_bt, opnd_array(6)->as_Register(ra_,this,idx6)/* tmp */, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* idx */, opnd_array(5)->as_KRegister(ra_,this,idx5)/* ktmp */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src */, vlen_enc);
  
#line 36149 "ad_x86.cpp"
  }
}

void vReplB_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 4152 "src/hotspot/cpu/x86/x86.ad"

    uint vlen = Matcher::vector_length(this);
    int vlen_enc = vector_length_encoding(this);
    if (vlen == 64 || VM_Version::supports_avx512vlbw()) { // AVX512VL for <512bit operands
      assert(VM_Version::supports_avx512bw(), "required"); // 512-bit byte vectors assume AVX512BW
      __ evpbroadcastb(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_Register(ra_,this,idx1)/* src */, vlen_enc);
    } else {
      __ movdl(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_Register(ra_,this,idx1)/* src */);
      __ vpbroadcastb(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, vlen_enc);
    }
  
#line 36173 "ad_x86.cpp"
  }
}

void ReplB_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 4170 "src/hotspot/cpu/x86/x86.ad"

    uint vlen = Matcher::vector_length(this);
    __ movdl(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_Register(ra_,this,idx1)/* src */);
    __ punpcklbw(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(0)->as_XMMRegister(ra_,this)/* dst */);
    __ pshuflw(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, 0x00);
    if (vlen >= 16) {
      assert(vlen == 16, "");
      __ punpcklqdq(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(0)->as_XMMRegister(ra_,this)/* dst */);
    }
  
#line 36196 "ad_x86.cpp"
  }
}

void ReplB_memNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  {
    C2_MacroAssembler _masm(&cbuf);

#line 4187 "src/hotspot/cpu/x86/x86.ad"

    int vlen_enc = vector_length_encoding(this);
    __ vpbroadcastb(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, Address::make_raw(opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1), opnd_array(1)->disp_reloc()), vlen_enc);
  
#line 36213 "ad_x86.cpp"
  }
}

void vReplS_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 4200 "src/hotspot/cpu/x86/x86.ad"

    uint vlen = Matcher::vector_length(this);
    int vlen_enc = vector_length_encoding(this);
    if (vlen == 32 || VM_Version::supports_avx512vlbw()) { // AVX512VL for <512bit operands
      assert(VM_Version::supports_avx512bw(), "required"); // 512-bit short vectors assume AVX512BW
      __ evpbroadcastw(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_Register(ra_,this,idx1)/* src */, vlen_enc);
    } else {
      __ movdl(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_Register(ra_,this,idx1)/* src */);
      __ vpbroadcastw(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, vlen_enc);
    }
  
#line 36237 "ad_x86.cpp"
  }
}

void ReplS_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 4218 "src/hotspot/cpu/x86/x86.ad"

    uint vlen = Matcher::vector_length(this);
    int vlen_enc = vector_length_encoding(this);
    __ movdl(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_Register(ra_,this,idx1)/* src */);
    __ pshuflw(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, 0x00);
    if (vlen >= 8) {
      assert(vlen == 8, "");
      __ punpcklqdq(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(0)->as_XMMRegister(ra_,this)/* dst */);
    }
  
#line 36260 "ad_x86.cpp"
  }
}

void ReplS_memNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  {
    C2_MacroAssembler _masm(&cbuf);

#line 4235 "src/hotspot/cpu/x86/x86.ad"

    int vlen_enc = vector_length_encoding(this);
    __ vpbroadcastw(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, Address::make_raw(opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1), opnd_array(1)->disp_reloc()), vlen_enc);
  
#line 36277 "ad_x86.cpp"
  }
}

void ReplI_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 4247 "src/hotspot/cpu/x86/x86.ad"

    uint vlen = Matcher::vector_length(this);
    int vlen_enc = vector_length_encoding(this);
    if (vlen == 16 || VM_Version::supports_avx512vl()) { // AVX512VL for <512bit operands
      __ evpbroadcastd(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_Register(ra_,this,idx1)/* src */, vlen_enc);
    } else if (VM_Version::supports_avx2()) {
      __ movdl(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_Register(ra_,this,idx1)/* src */);
      __ vpbroadcastd(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, vlen_enc);
    } else {
      __ movdl(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_Register(ra_,this,idx1)/* src */);
      __ pshufd(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, 0x00);
    }
  
#line 36303 "ad_x86.cpp"
  }
}

void ReplI_memNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  {
    C2_MacroAssembler _masm(&cbuf);

#line 4266 "src/hotspot/cpu/x86/x86.ad"

    int vlen_enc = vector_length_encoding(this);
    if (VM_Version::supports_avx2()) {
      __ vpbroadcastd(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, Address::make_raw(opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1), opnd_array(1)->disp_reloc()), vlen_enc);
    } else if (VM_Version::supports_avx()) {
      __ vbroadcastss(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, Address::make_raw(opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1), opnd_array(1)->disp_reloc()), vlen_enc);
    } else {
      __ movdl(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, Address::make_raw(opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1), opnd_array(1)->disp_reloc()));
      __ pshufd(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, 0x00);
    }
  
#line 36327 "ad_x86.cpp"
  }
}

void ReplI_immNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// con
  {
    C2_MacroAssembler _masm(&cbuf);

#line 4285 "src/hotspot/cpu/x86/x86.ad"

    InternalAddress addr = InternalAddress(__ code()->consts()->start() + constant_offset());
    BasicType bt = Matcher::vector_element_basic_type(this);
    int vlen = Matcher::vector_length_in_bytes(this);
    __ load_constant_vector(bt, opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, addr, vlen);
  
#line 36346 "ad_x86.cpp"
  }
}

void ReplI_immNode::eval_constant(Compile* C) {
  {

#line 4286 "src/hotspot/cpu/x86/x86.ad"
    _constant = C->output()->constant_table().add(this, Matcher::vector_element_basic_type(this),
        vreplicate_imm(Matcher::vector_element_basic_type(this), opnd_array(1)->constant(),
            (VM_Version::supports_sse3() ? (VM_Version::supports_avx() ? 4 : 8) : 8) /
                type2aelembytes(Matcher::vector_element_basic_type(this))));
#line 36358 "ad_x86.cpp"
  }
}
void ReplI_imm_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// con
  {
    C2_MacroAssembler _masm(&cbuf);

#line 4285 "src/hotspot/cpu/x86/x86.ad"

    InternalAddress addr = InternalAddress(__ code()->consts()->start() + constant_offset());
    BasicType bt = Matcher::vector_element_basic_type(this);
    int vlen = Matcher::vector_length_in_bytes(this);
    __ load_constant_vector(bt, opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, addr, vlen);
  
#line 36376 "ad_x86.cpp"
  }
}

void ReplI_imm_0Node::eval_constant(Compile* C) {
  {

#line 4286 "src/hotspot/cpu/x86/x86.ad"
    _constant = C->output()->constant_table().add(this, Matcher::vector_element_basic_type(this),
        vreplicate_imm(Matcher::vector_element_basic_type(this), opnd_array(1)->constant(),
            (VM_Version::supports_sse3() ? (VM_Version::supports_avx() ? 4 : 8) : 8) /
                type2aelembytes(Matcher::vector_element_basic_type(this))));
#line 36388 "ad_x86.cpp"
  }
}
void ReplI_imm_1Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// con
  {
    C2_MacroAssembler _masm(&cbuf);

#line 4285 "src/hotspot/cpu/x86/x86.ad"

    InternalAddress addr = InternalAddress(__ code()->consts()->start() + constant_offset());
    BasicType bt = Matcher::vector_element_basic_type(this);
    int vlen = Matcher::vector_length_in_bytes(this);
    __ load_constant_vector(bt, opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, addr, vlen);
  
#line 36406 "ad_x86.cpp"
  }
}

void ReplI_imm_1Node::eval_constant(Compile* C) {
  {

#line 4286 "src/hotspot/cpu/x86/x86.ad"
    _constant = C->output()->constant_table().add(this, Matcher::vector_element_basic_type(this),
        vreplicate_imm(Matcher::vector_element_basic_type(this), opnd_array(1)->constant(),
            (VM_Version::supports_sse3() ? (VM_Version::supports_avx() ? 4 : 8) : 8) /
                type2aelembytes(Matcher::vector_element_basic_type(this))));
#line 36418 "ad_x86.cpp"
  }
}
void ReplI_zeroNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// zero
  {
    C2_MacroAssembler _masm(&cbuf);

#line 4303 "src/hotspot/cpu/x86/x86.ad"

    int vlen_enc = vector_length_encoding(this);
    if (VM_Version::supports_evex() && !VM_Version::supports_avx512vl()) {
      __ vpxor(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, vlen_enc);
    } else {
      __ pxor(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(0)->as_XMMRegister(ra_,this)/* dst */);
    }
  
#line 36438 "ad_x86.cpp"
  }
}

void ReplI_zero_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// zero
  {
    C2_MacroAssembler _masm(&cbuf);

#line 4303 "src/hotspot/cpu/x86/x86.ad"

    int vlen_enc = vector_length_encoding(this);
    if (VM_Version::supports_evex() && !VM_Version::supports_avx512vl()) {
      __ vpxor(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, vlen_enc);
    } else {
      __ pxor(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(0)->as_XMMRegister(ra_,this)/* dst */);
    }
  
#line 36459 "ad_x86.cpp"
  }
}

void ReplI_zero_1Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// zero
  {
    C2_MacroAssembler _masm(&cbuf);

#line 4303 "src/hotspot/cpu/x86/x86.ad"

    int vlen_enc = vector_length_encoding(this);
    if (VM_Version::supports_evex() && !VM_Version::supports_avx512vl()) {
      __ vpxor(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, vlen_enc);
    } else {
      __ pxor(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(0)->as_XMMRegister(ra_,this)/* dst */);
    }
  
#line 36480 "ad_x86.cpp"
  }
}

void ReplI_M1Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// con
  {
    C2_MacroAssembler _masm(&cbuf);

#line 4320 "src/hotspot/cpu/x86/x86.ad"

    int vector_len = vector_length_encoding(this);
    __ vallones(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, vector_len);
  
#line 36497 "ad_x86.cpp"
  }
}

void ReplI_M1_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// con
  {
    C2_MacroAssembler _masm(&cbuf);

#line 4320 "src/hotspot/cpu/x86/x86.ad"

    int vector_len = vector_length_encoding(this);
    __ vallones(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, vector_len);
  
#line 36514 "ad_x86.cpp"
  }
}

void ReplI_M1_1Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// con
  {
    C2_MacroAssembler _masm(&cbuf);

#line 4320 "src/hotspot/cpu/x86/x86.ad"

    int vector_len = vector_length_encoding(this);
    __ vallones(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, vector_len);
  
#line 36531 "ad_x86.cpp"
  }
}

void ReplL_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// dst
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// tmp
  {
    C2_MacroAssembler _masm(&cbuf);

#line 4356 "src/hotspot/cpu/x86/x86.ad"

    uint vlen = Matcher::vector_length(this);
    if (vlen == 2) {
      __ movdl(opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* dst */, opnd_array(1)->as_Register(ra_,this,idx1)/* src */);
      __ movdl(opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* tmp */, HIGH_FROM_LOW(opnd_array(1)->as_Register(ra_,this,idx1)/* src */));
      __ punpckldq(opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* dst */, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* tmp */);
      __ punpcklqdq(opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* dst */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* dst */);
    } else if (VM_Version::supports_avx512vl()) { // AVX512VL for <512bit operands
      int vlen_enc = Assembler::AVX_256bit;
      __ movdl(opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* dst */, opnd_array(1)->as_Register(ra_,this,idx1)/* src */);
      __ movdl(opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* tmp */, HIGH_FROM_LOW(opnd_array(1)->as_Register(ra_,this,idx1)/* src */));
      __ punpckldq(opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* dst */, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* tmp */);
      __ vpbroadcastq(opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* dst */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* dst */, vlen_enc);
    } else {
      __ movdl(opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* dst */, opnd_array(1)->as_Register(ra_,this,idx1)/* src */);
      __ movdl(opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* tmp */, HIGH_FROM_LOW(opnd_array(1)->as_Register(ra_,this,idx1)/* src */));
      __ punpckldq(opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* dst */, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* tmp */);
      __ punpcklqdq(opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* dst */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* dst */);
      __ vinserti128_high(opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* dst */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* dst */);
    }
  
#line 36567 "ad_x86.cpp"
  }
}

void ReplL_reg_legNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// dst
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// tmp
  {
    C2_MacroAssembler _masm(&cbuf);

#line 4385 "src/hotspot/cpu/x86/x86.ad"

    if (VM_Version::supports_avx512vl()) {
      __ movdl(opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* dst */, opnd_array(1)->as_Register(ra_,this,idx1)/* src */);
      __ movdl(opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* tmp */, HIGH_FROM_LOW(opnd_array(1)->as_Register(ra_,this,idx1)/* src */));
      __ punpckldq(opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* dst */, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* tmp */);
      __ punpcklqdq(opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* dst */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* dst */);
      __ vinserti128_high(opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* dst */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* dst */);
      __ vinserti64x4(opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* dst */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* dst */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* dst */, 0x1);
    } else {
      int vlen_enc = Assembler::AVX_512bit;
      __ movdl(opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* dst */, opnd_array(1)->as_Register(ra_,this,idx1)/* src */);
      __ movdl(opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* tmp */, HIGH_FROM_LOW(opnd_array(1)->as_Register(ra_,this,idx1)/* src */));
      __ punpckldq(opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* dst */, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* tmp */);
      __ vpbroadcastq(opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* dst */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* dst */, vlen_enc);
    }
  
#line 36598 "ad_x86.cpp"
  }
}

void ReplL_memNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  {
    C2_MacroAssembler _masm(&cbuf);

#line 4408 "src/hotspot/cpu/x86/x86.ad"

    int vlen_enc = vector_length_encoding(this);
    if (VM_Version::supports_avx2()) {
      __ vpbroadcastq(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, Address::make_raw(opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1), opnd_array(1)->disp_reloc()), vlen_enc);
    } else if (VM_Version::supports_sse3()) {
      __ movddup(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, Address::make_raw(opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1), opnd_array(1)->disp_reloc()));
    } else {
      __ movq(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, Address::make_raw(opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1), opnd_array(1)->disp_reloc()));
      __ punpcklqdq(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(0)->as_XMMRegister(ra_,this)/* dst */);
    }
  
#line 36622 "ad_x86.cpp"
  }
}

void ReplL_immNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// con
  {
    C2_MacroAssembler _masm(&cbuf);

#line 4426 "src/hotspot/cpu/x86/x86.ad"

    InternalAddress addr = InternalAddress(__ code()->consts()->start() + constant_offset());
    int vlen = Matcher::vector_length_in_bytes(this);
    __ load_constant_vector(T_LONG, opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, addr, vlen);
  
#line 36640 "ad_x86.cpp"
  }
}

void ReplL_immNode::eval_constant(Compile* C) {
  {

#line 4427 "src/hotspot/cpu/x86/x86.ad"
    _constant = C->output()->constant_table().add(this, T_LONG, vreplicate_imm(T_LONG, opnd_array(1)->constantL(), 1));
#line 36649 "ad_x86.cpp"
  }
}
void ReplL_zeroNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// zero
  {
    C2_MacroAssembler _masm(&cbuf);

#line 4437 "src/hotspot/cpu/x86/x86.ad"

    int vlen_enc = vector_length_encoding(this);
    if (VM_Version::supports_evex() && !VM_Version::supports_avx512vl()) {
      __ vpxor(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, vlen_enc);
    } else {
      __ pxor(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(0)->as_XMMRegister(ra_,this)/* dst */);
    }
  
#line 36669 "ad_x86.cpp"
  }
}

void ReplL_M1Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// con
  {
    C2_MacroAssembler _masm(&cbuf);

#line 4452 "src/hotspot/cpu/x86/x86.ad"

    int vector_len = vector_length_encoding(this);
    __ vallones(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, vector_len);
  
#line 36686 "ad_x86.cpp"
  }
}

void vReplF_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 4465 "src/hotspot/cpu/x86/x86.ad"

    uint vlen = Matcher::vector_length(this);
    int vlen_enc = vector_length_encoding(this);
    if (vlen <= 4) {
      __ vpermilps(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, 0x00, Assembler::AVX_128bit);
    } else if (VM_Version::supports_avx2()) {
      __ vbroadcastss(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, vlen_enc); // reg-to-reg variant requires AVX2
    } else {
      assert(vlen == 8, "sanity");
      __ vpermilps(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, 0x00, Assembler::AVX_128bit);
      __ vinsertf128_high(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(0)->as_XMMRegister(ra_,this)/* dst */);
    }
  
#line 36712 "ad_x86.cpp"
  }
}

void ReplF_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 4485 "src/hotspot/cpu/x86/x86.ad"

    __ pshufd(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, 0x00);
  
#line 36728 "ad_x86.cpp"
  }
}

void ReplF_memNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  {
    C2_MacroAssembler _masm(&cbuf);

#line 4495 "src/hotspot/cpu/x86/x86.ad"

    int vlen_enc = vector_length_encoding(this);
    __ vbroadcastss(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, Address::make_raw(opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1), opnd_array(1)->disp_reloc()), vlen_enc);
  
#line 36745 "ad_x86.cpp"
  }
}

void ReplF_immNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// con
  {
    C2_MacroAssembler _masm(&cbuf);

#line 4506 "src/hotspot/cpu/x86/x86.ad"

    InternalAddress addr = InternalAddress(__ code()->consts()->start() + constant_offset());
    int vlen = Matcher::vector_length_in_bytes(this);
    __ load_constant_vector(T_FLOAT, opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, addr, vlen);
  
#line 36763 "ad_x86.cpp"
  }
}

void ReplF_immNode::eval_constant(Compile* C) {
  {

#line 4507 "src/hotspot/cpu/x86/x86.ad"
    _constant = C->output()->constant_table().add(this, T_FLOAT, vreplicate_imm(T_FLOAT, opnd_array(1)->constantF(),
        VM_Version::supports_sse3() ? (VM_Version::supports_avx() ? 1 : 2) : 2));
#line 36773 "ad_x86.cpp"
  }
}
void ReplF_zeroNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// zero
  {
    C2_MacroAssembler _masm(&cbuf);

#line 4518 "src/hotspot/cpu/x86/x86.ad"

    int vlen_enc = vector_length_encoding(this);
    if (VM_Version::supports_evex() && !VM_Version::supports_avx512vldq()) {
      __ vpxor(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, vlen_enc);
    } else {
      __ xorps(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(0)->as_XMMRegister(ra_,this)/* dst */);
    }
  
#line 36793 "ad_x86.cpp"
  }
}

void vReplD_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 4536 "src/hotspot/cpu/x86/x86.ad"

    uint vlen = Matcher::vector_length(this);
    int vlen_enc = vector_length_encoding(this);
    if (vlen <= 2) {
      __ movddup(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */);
    } else if (VM_Version::supports_avx2()) {
      __ vbroadcastsd(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, vlen_enc); // reg-to-reg variant requires AVX2
    } else {
      assert(vlen == 4, "sanity");
      __ movddup(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */);
      __ vinsertf128_high(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(0)->as_XMMRegister(ra_,this)/* dst */);
    }
  
#line 36819 "ad_x86.cpp"
  }
}

void ReplD_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 4556 "src/hotspot/cpu/x86/x86.ad"

    __ pshufd(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, 0x44);
  
#line 36835 "ad_x86.cpp"
  }
}

void ReplD_memNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  {
    C2_MacroAssembler _masm(&cbuf);

#line 4566 "src/hotspot/cpu/x86/x86.ad"

    if (Matcher::vector_length(this) >= 4) {
      int vlen_enc = vector_length_encoding(this);
      __ vbroadcastsd(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, Address::make_raw(opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1), opnd_array(1)->disp_reloc()), vlen_enc);
    } else {
      __ movddup(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, Address::make_raw(opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1), opnd_array(1)->disp_reloc()));
    }
  
#line 36856 "ad_x86.cpp"
  }
}

void ReplD_immNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// con
  {
    C2_MacroAssembler _masm(&cbuf);

#line 4581 "src/hotspot/cpu/x86/x86.ad"

    InternalAddress addr = InternalAddress(__ code()->consts()->start() + constant_offset());
    int vlen = Matcher::vector_length_in_bytes(this);
    __ load_constant_vector(T_DOUBLE, opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, addr, vlen);
  
#line 36874 "ad_x86.cpp"
  }
}

void ReplD_immNode::eval_constant(Compile* C) {
  {

#line 4582 "src/hotspot/cpu/x86/x86.ad"
    _constant = C->output()->constant_table().add(this, T_DOUBLE, vreplicate_imm(T_DOUBLE, opnd_array(1)->constantD(), 1));
#line 36883 "ad_x86.cpp"
  }
}
void ReplD_zeroNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// zero
  {
    C2_MacroAssembler _masm(&cbuf);

#line 4592 "src/hotspot/cpu/x86/x86.ad"

    int vlen_enc = vector_length_encoding(this);
    if (VM_Version::supports_evex() && !VM_Version::supports_avx512vldq()) {
      __ vpxor(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, vlen_enc);
    } else {
      __ xorps(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(0)->as_XMMRegister(ra_,this)/* dst */);
    }
  
#line 36903 "ad_x86.cpp"
  }
}

void insertNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// val
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// idx
  {
    C2_MacroAssembler _masm(&cbuf);

#line 4609 "src/hotspot/cpu/x86/x86.ad"

    assert(UseSSE >= 4, "required");
    assert(Matcher::vector_length_in_bytes(this) >= 8, "required");

    BasicType elem_bt = Matcher::vector_element_basic_type(this);

    assert(is_integral_type(elem_bt), "");
    assert(opnd_array(3)->constant()< (int)Matcher::vector_length(this), "out of bounds");

    __ insert(elem_bt, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* dst */, opnd_array(2)->as_Register(ra_,this,idx2)/* val */, opnd_array(3)->constant());
  
#line 36929 "ad_x86.cpp"
  }
}

void insert32Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// val
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// idx
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// vtmp
  {
    C2_MacroAssembler _masm(&cbuf);

#line 4628 "src/hotspot/cpu/x86/x86.ad"

    int vlen_enc = Assembler::AVX_256bit;
    BasicType elem_bt = Matcher::vector_element_basic_type(this);
    int elem_per_lane = 16/type2aelembytes(elem_bt);
    int log2epr = log2(elem_per_lane);

    assert(is_integral_type(elem_bt), "sanity");
    assert(opnd_array(3)->constant()< (int)Matcher::vector_length(this), "out of bounds");

    uint x_idx = opnd_array(3)->constant()& right_n_bits(log2epr);
    uint y_idx = (opnd_array(3)->constant()>> log2epr) & 1;
    __ vextracti128(opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* vtmp */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, y_idx);
    __ vinsert(elem_bt, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* vtmp */, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* vtmp */, opnd_array(2)->as_Register(ra_,this,idx2)/* val */, x_idx);
    __ vinserti128(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* vtmp */, y_idx);
  
#line 36960 "ad_x86.cpp"
  }
}

void insert64Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// val
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// idx
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// vtmp
  {
    C2_MacroAssembler _masm(&cbuf);

#line 4651 "src/hotspot/cpu/x86/x86.ad"

    assert(UseAVX > 2, "sanity");

    BasicType elem_bt = Matcher::vector_element_basic_type(this);
    int elem_per_lane = 16/type2aelembytes(elem_bt);
    int log2epr = log2(elem_per_lane);

    assert(is_integral_type(elem_bt), "");
    assert(opnd_array(3)->constant()< (int)Matcher::vector_length(this), "out of bounds");

    uint x_idx = opnd_array(3)->constant()& right_n_bits(log2epr);
    uint y_idx = (opnd_array(3)->constant()>> log2epr) & 3;
    __ vextracti32x4(opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* vtmp */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, y_idx);
    __ vinsert(elem_bt, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* vtmp */, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* vtmp */, opnd_array(2)->as_Register(ra_,this,idx2)/* val */, x_idx);
    __ vinserti32x4(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* vtmp */, y_idx);
  
#line 36992 "ad_x86.cpp"
  }
}

void insertFNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// val
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// idx
  {
    C2_MacroAssembler _masm(&cbuf);

#line 4727 "src/hotspot/cpu/x86/x86.ad"

    assert(UseSSE >= 4, "sanity");

    assert(Matcher::vector_element_basic_type(this) == T_FLOAT, "sanity");
    assert(opnd_array(3)->constant()< (int)Matcher::vector_length(this), "out of bounds");

    uint x_idx = opnd_array(3)->constant()& right_n_bits(2);
    __ insertps(opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* dst */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* val */, x_idx << 4);
  
#line 37016 "ad_x86.cpp"
  }
}

void vinsertFNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// val
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// idx
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// vtmp
  {
    C2_MacroAssembler _masm(&cbuf);

#line 4744 "src/hotspot/cpu/x86/x86.ad"

    assert(Matcher::vector_element_basic_type(this) == T_FLOAT, "sanity");
    assert(opnd_array(3)->constant()< (int)Matcher::vector_length(this), "out of bounds");

    int vlen = Matcher::vector_length(this);
    uint x_idx = opnd_array(3)->constant()& right_n_bits(2);
    if (vlen == 8) {
      uint y_idx = (opnd_array(3)->constant()>> 2) & 1;
      int vlen_enc = Assembler::AVX_256bit;
      __ vextracti128(opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* vtmp */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, y_idx);
      __ vinsertps(opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* vtmp */, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* vtmp */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* val */, x_idx << 4);
      __ vinserti128(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* vtmp */, y_idx);
    } else {
      assert(vlen == 16, "sanity");
      uint y_idx = (opnd_array(3)->constant()>> 2) & 3;
      __ vextracti32x4(opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* vtmp */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, y_idx);
      __ vinsertps(opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* vtmp */, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* vtmp */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* val */, x_idx << 4);
      __ vinserti32x4(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* vtmp */, y_idx);
    }
  
#line 37052 "ad_x86.cpp"
  }
}

void reductionINode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// vtmp1
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// vtmp2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 4839 "src/hotspot/cpu/x86/x86.ad"

    int opcode = this->ideal_Opcode();
    int vlen = Matcher::vector_length(this, opnd_array(2));
    __ reduceI(opcode, vlen, opnd_array(0)->as_Register(ra_,this)/* dst */, opnd_array(1)->as_Register(ra_,this,idx1)/* src1 */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src2 */, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* vtmp1 */, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* vtmp2 */);
  
#line 37073 "ad_x86.cpp"
  }
}

void reductionI_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// vtmp1
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// vtmp2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 4839 "src/hotspot/cpu/x86/x86.ad"

    int opcode = this->ideal_Opcode();
    int vlen = Matcher::vector_length(this, opnd_array(2));
    __ reduceI(opcode, vlen, opnd_array(0)->as_Register(ra_,this)/* dst */, opnd_array(1)->as_Register(ra_,this,idx1)/* src1 */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src2 */, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* vtmp1 */, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* vtmp2 */);
  
#line 37094 "ad_x86.cpp"
  }
}

void reductionI_1Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// vtmp1
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// vtmp2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 4839 "src/hotspot/cpu/x86/x86.ad"

    int opcode = this->ideal_Opcode();
    int vlen = Matcher::vector_length(this, opnd_array(2));
    __ reduceI(opcode, vlen, opnd_array(0)->as_Register(ra_,this)/* dst */, opnd_array(1)->as_Register(ra_,this,idx1)/* src1 */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src2 */, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* vtmp1 */, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* vtmp2 */);
  
#line 37115 "ad_x86.cpp"
  }
}

void reductionI_2Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// vtmp1
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// vtmp2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 4839 "src/hotspot/cpu/x86/x86.ad"

    int opcode = this->ideal_Opcode();
    int vlen = Matcher::vector_length(this, opnd_array(2));
    __ reduceI(opcode, vlen, opnd_array(0)->as_Register(ra_,this)/* dst */, opnd_array(1)->as_Register(ra_,this,idx1)/* src1 */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src2 */, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* vtmp1 */, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* vtmp2 */);
  
#line 37136 "ad_x86.cpp"
  }
}

void reductionI_3Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// vtmp1
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// vtmp2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 4839 "src/hotspot/cpu/x86/x86.ad"

    int opcode = this->ideal_Opcode();
    int vlen = Matcher::vector_length(this, opnd_array(2));
    __ reduceI(opcode, vlen, opnd_array(0)->as_Register(ra_,this)/* dst */, opnd_array(1)->as_Register(ra_,this,idx1)/* src1 */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src2 */, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* vtmp1 */, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* vtmp2 */);
  
#line 37157 "ad_x86.cpp"
  }
}

void reductionI_4Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// vtmp1
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// vtmp2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 4839 "src/hotspot/cpu/x86/x86.ad"

    int opcode = this->ideal_Opcode();
    int vlen = Matcher::vector_length(this, opnd_array(2));
    __ reduceI(opcode, vlen, opnd_array(0)->as_Register(ra_,this)/* dst */, opnd_array(1)->as_Register(ra_,this,idx1)/* src1 */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src2 */, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* vtmp1 */, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* vtmp2 */);
  
#line 37178 "ad_x86.cpp"
  }
}

void reductionI_5Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// vtmp1
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// vtmp2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 4839 "src/hotspot/cpu/x86/x86.ad"

    int opcode = this->ideal_Opcode();
    int vlen = Matcher::vector_length(this, opnd_array(2));
    __ reduceI(opcode, vlen, opnd_array(0)->as_Register(ra_,this)/* dst */, opnd_array(1)->as_Register(ra_,this,idx1)/* src1 */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src2 */, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* vtmp1 */, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* vtmp2 */);
  
#line 37199 "ad_x86.cpp"
  }
}

void reductionF128Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// vtmp
  {
    C2_MacroAssembler _masm(&cbuf);

#line 4897 "src/hotspot/cpu/x86/x86.ad"

    int opcode = this->ideal_Opcode();
    int vlen = Matcher::vector_length(this, opnd_array(2));
    __ reduce_fp(opcode, vlen, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* dst */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src */, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* vtmp */);
  
#line 37219 "ad_x86.cpp"
  }
}

void reductionF128_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// vtmp
  {
    C2_MacroAssembler _masm(&cbuf);

#line 4897 "src/hotspot/cpu/x86/x86.ad"

    int opcode = this->ideal_Opcode();
    int vlen = Matcher::vector_length(this, opnd_array(2));
    __ reduce_fp(opcode, vlen, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* dst */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src */, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* vtmp */);
  
#line 37239 "ad_x86.cpp"
  }
}

void reduction8FNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// vtmp1
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// vtmp2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 4911 "src/hotspot/cpu/x86/x86.ad"

    int opcode = this->ideal_Opcode();
    int vlen = Matcher::vector_length(this, opnd_array(2));
    __ reduce_fp(opcode, vlen, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* dst */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src */, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* vtmp1 */, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* vtmp2 */);
  
#line 37260 "ad_x86.cpp"
  }
}

void reduction8F_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// vtmp1
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// vtmp2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 4911 "src/hotspot/cpu/x86/x86.ad"

    int opcode = this->ideal_Opcode();
    int vlen = Matcher::vector_length(this, opnd_array(2));
    __ reduce_fp(opcode, vlen, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* dst */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src */, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* vtmp1 */, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* vtmp2 */);
  
#line 37281 "ad_x86.cpp"
  }
}

void reduction16FNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// vtmp1
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// vtmp2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 4925 "src/hotspot/cpu/x86/x86.ad"

    int opcode = this->ideal_Opcode();
    int vlen = Matcher::vector_length(this, opnd_array(2));
    __ reduce_fp(opcode, vlen, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* dst */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src */, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* vtmp1 */, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* vtmp2 */);
  
#line 37302 "ad_x86.cpp"
  }
}

void reduction16F_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// vtmp1
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// vtmp2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 4925 "src/hotspot/cpu/x86/x86.ad"

    int opcode = this->ideal_Opcode();
    int vlen = Matcher::vector_length(this, opnd_array(2));
    __ reduce_fp(opcode, vlen, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* dst */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src */, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* vtmp1 */, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* vtmp2 */);
  
#line 37323 "ad_x86.cpp"
  }
}

void reduction2DNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// vtmp
  {
    C2_MacroAssembler _masm(&cbuf);

#line 4941 "src/hotspot/cpu/x86/x86.ad"

    int opcode = this->ideal_Opcode();
    int vlen = Matcher::vector_length(this, opnd_array(2));
    __ reduce_fp(opcode, vlen, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* dst */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src */, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* vtmp */);

#line 37343 "ad_x86.cpp"
  }
}

void reduction2D_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// vtmp
  {
    C2_MacroAssembler _masm(&cbuf);

#line 4941 "src/hotspot/cpu/x86/x86.ad"

    int opcode = this->ideal_Opcode();
    int vlen = Matcher::vector_length(this, opnd_array(2));
    __ reduce_fp(opcode, vlen, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* dst */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src */, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* vtmp */);

#line 37363 "ad_x86.cpp"
  }
}

void reduction4DNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// vtmp1
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// vtmp2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 4955 "src/hotspot/cpu/x86/x86.ad"

    int opcode = this->ideal_Opcode();
    int vlen = Matcher::vector_length(this, opnd_array(2));
    __ reduce_fp(opcode, vlen, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* dst */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src */, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* vtmp1 */, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* vtmp2 */);
  
#line 37384 "ad_x86.cpp"
  }
}

void reduction4D_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// vtmp1
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// vtmp2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 4955 "src/hotspot/cpu/x86/x86.ad"

    int opcode = this->ideal_Opcode();
    int vlen = Matcher::vector_length(this, opnd_array(2));
    __ reduce_fp(opcode, vlen, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* dst */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src */, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* vtmp1 */, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* vtmp2 */);
  
#line 37405 "ad_x86.cpp"
  }
}

void reduction8DNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// vtmp1
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// vtmp2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 4969 "src/hotspot/cpu/x86/x86.ad"

    int opcode = this->ideal_Opcode();
    int vlen = Matcher::vector_length(this, opnd_array(2));
    __ reduce_fp(opcode, vlen, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* dst */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src */, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* vtmp1 */, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* vtmp2 */);
  
#line 37426 "ad_x86.cpp"
  }
}

void reduction8D_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// vtmp1
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// vtmp2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 4969 "src/hotspot/cpu/x86/x86.ad"

    int opcode = this->ideal_Opcode();
    int vlen = Matcher::vector_length(this, opnd_array(2));
    __ reduce_fp(opcode, vlen, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* dst */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src */, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* vtmp1 */, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* vtmp2 */);
  
#line 37447 "ad_x86.cpp"
  }
}

void reductionSNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// vtmp1
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// vtmp2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 5030 "src/hotspot/cpu/x86/x86.ad"

    int opcode = this->ideal_Opcode();
    int vlen = Matcher::vector_length(this, opnd_array(2));
    __ reduceS(opcode, vlen, opnd_array(0)->as_Register(ra_,this)/* dst */, opnd_array(1)->as_Register(ra_,this,idx1)/* src1 */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src2 */, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* vtmp1 */, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* vtmp2 */);
  
#line 37468 "ad_x86.cpp"
  }
}

void reductionS_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// vtmp1
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// vtmp2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 5030 "src/hotspot/cpu/x86/x86.ad"

    int opcode = this->ideal_Opcode();
    int vlen = Matcher::vector_length(this, opnd_array(2));
    __ reduceS(opcode, vlen, opnd_array(0)->as_Register(ra_,this)/* dst */, opnd_array(1)->as_Register(ra_,this,idx1)/* src1 */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src2 */, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* vtmp1 */, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* vtmp2 */);
  
#line 37489 "ad_x86.cpp"
  }
}

void reductionS_1Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// vtmp1
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// vtmp2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 5030 "src/hotspot/cpu/x86/x86.ad"

    int opcode = this->ideal_Opcode();
    int vlen = Matcher::vector_length(this, opnd_array(2));
    __ reduceS(opcode, vlen, opnd_array(0)->as_Register(ra_,this)/* dst */, opnd_array(1)->as_Register(ra_,this,idx1)/* src1 */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src2 */, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* vtmp1 */, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* vtmp2 */);
  
#line 37510 "ad_x86.cpp"
  }
}

void reductionS_2Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// vtmp1
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// vtmp2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 5030 "src/hotspot/cpu/x86/x86.ad"

    int opcode = this->ideal_Opcode();
    int vlen = Matcher::vector_length(this, opnd_array(2));
    __ reduceS(opcode, vlen, opnd_array(0)->as_Register(ra_,this)/* dst */, opnd_array(1)->as_Register(ra_,this,idx1)/* src1 */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src2 */, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* vtmp1 */, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* vtmp2 */);
  
#line 37531 "ad_x86.cpp"
  }
}

void reductionS_3Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// vtmp1
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// vtmp2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 5030 "src/hotspot/cpu/x86/x86.ad"

    int opcode = this->ideal_Opcode();
    int vlen = Matcher::vector_length(this, opnd_array(2));
    __ reduceS(opcode, vlen, opnd_array(0)->as_Register(ra_,this)/* dst */, opnd_array(1)->as_Register(ra_,this,idx1)/* src1 */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src2 */, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* vtmp1 */, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* vtmp2 */);
  
#line 37552 "ad_x86.cpp"
  }
}

void reductionS_4Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// vtmp1
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// vtmp2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 5030 "src/hotspot/cpu/x86/x86.ad"

    int opcode = this->ideal_Opcode();
    int vlen = Matcher::vector_length(this, opnd_array(2));
    __ reduceS(opcode, vlen, opnd_array(0)->as_Register(ra_,this)/* dst */, opnd_array(1)->as_Register(ra_,this,idx1)/* src1 */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src2 */, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* vtmp1 */, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* vtmp2 */);
  
#line 37573 "ad_x86.cpp"
  }
}

void reductionS_5Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// vtmp1
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// vtmp2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 5030 "src/hotspot/cpu/x86/x86.ad"

    int opcode = this->ideal_Opcode();
    int vlen = Matcher::vector_length(this, opnd_array(2));
    __ reduceS(opcode, vlen, opnd_array(0)->as_Register(ra_,this)/* dst */, opnd_array(1)->as_Register(ra_,this,idx1)/* src1 */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src2 */, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* vtmp1 */, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* vtmp2 */);
  
#line 37594 "ad_x86.cpp"
  }
}

void mul_reductionBNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// dst
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// vtmp1
  unsigned idx5 = idx4 + opnd_array(4)->num_edges(); 	// vtmp2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 5046 "src/hotspot/cpu/x86/x86.ad"

    int opcode = this->ideal_Opcode();
    int vlen = Matcher::vector_length(this, opnd_array(2));
    __ mulreduceB(opcode, vlen, opnd_array(3)->as_Register(ra_,this,idx3)/* dst */, opnd_array(1)->as_Register(ra_,this,idx1)/* src1 */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src2 */, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* vtmp1 */, opnd_array(5)->as_XMMRegister(ra_,this,idx5)/* vtmp2 */);
  
#line 37616 "ad_x86.cpp"
  }
}

void mul_reduction64BNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// dst
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// vtmp1
  unsigned idx5 = idx4 + opnd_array(4)->num_edges(); 	// vtmp2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 5060 "src/hotspot/cpu/x86/x86.ad"

    int opcode = this->ideal_Opcode();
    int vlen = Matcher::vector_length(this, opnd_array(2));
    __ mulreduceB(opcode, vlen, opnd_array(3)->as_Register(ra_,this,idx3)/* dst */, opnd_array(1)->as_Register(ra_,this,idx1)/* src1 */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src2 */, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* vtmp1 */, opnd_array(5)->as_XMMRegister(ra_,this,idx5)/* vtmp2 */);
  
#line 37638 "ad_x86.cpp"
  }
}

void minmax_reduction2FNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// dst
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// tmp
  unsigned idx5 = idx4 + opnd_array(4)->num_edges(); 	// atmp
  unsigned idx6 = idx5 + opnd_array(5)->num_edges(); 	// btmp
  unsigned idx7 = idx6 + opnd_array(6)->num_edges(); 	// xmm_1
  {
    C2_MacroAssembler _masm(&cbuf);

#line 5080 "src/hotspot/cpu/x86/x86.ad"

    assert(UseAVX > 0, "sanity");

    int opcode = this->ideal_Opcode();
    int vlen = Matcher::vector_length(this, opnd_array(2));
    __ reduceFloatMinMax(opcode, vlen, false, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src2 */, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* tmp */,
                         opnd_array(5)->as_XMMRegister(ra_,this,idx5)/* atmp */, opnd_array(6)->as_XMMRegister(ra_,this,idx6)/* btmp */, opnd_array(7)->as_XMMRegister(ra_,this,idx7)/* xmm_1 */);
  
#line 37665 "ad_x86.cpp"
  }
}

void minmax_reduction2F_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// dst
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// tmp
  unsigned idx5 = idx4 + opnd_array(4)->num_edges(); 	// atmp
  unsigned idx6 = idx5 + opnd_array(5)->num_edges(); 	// btmp
  unsigned idx7 = idx6 + opnd_array(6)->num_edges(); 	// xmm_1
  {
    C2_MacroAssembler _masm(&cbuf);

#line 5080 "src/hotspot/cpu/x86/x86.ad"

    assert(UseAVX > 0, "sanity");

    int opcode = this->ideal_Opcode();
    int vlen = Matcher::vector_length(this, opnd_array(2));
    __ reduceFloatMinMax(opcode, vlen, false, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src2 */, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* tmp */,
                         opnd_array(5)->as_XMMRegister(ra_,this,idx5)/* atmp */, opnd_array(6)->as_XMMRegister(ra_,this,idx6)/* btmp */, opnd_array(7)->as_XMMRegister(ra_,this,idx7)/* xmm_1 */);
  
#line 37692 "ad_x86.cpp"
  }
}

void minmax_reductionFNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// dst
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// tmp
  unsigned idx5 = idx4 + opnd_array(4)->num_edges(); 	// atmp
  unsigned idx6 = idx5 + opnd_array(5)->num_edges(); 	// btmp
  unsigned idx7 = idx6 + opnd_array(6)->num_edges(); 	// xmm_0
  unsigned idx8 = idx7 + opnd_array(7)->num_edges(); 	// xmm_1
  {
    C2_MacroAssembler _masm(&cbuf);

#line 5101 "src/hotspot/cpu/x86/x86.ad"

    assert(UseAVX > 0, "sanity");

    int opcode = this->ideal_Opcode();
    int vlen = Matcher::vector_length(this, opnd_array(2));
    __ reduceFloatMinMax(opcode, vlen, false, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src2 */, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* tmp */,
                         opnd_array(5)->as_XMMRegister(ra_,this,idx5)/* atmp */, opnd_array(6)->as_XMMRegister(ra_,this,idx6)/* btmp */, opnd_array(7)->as_XMMRegister(ra_,this,idx7)/* xmm_0 */, opnd_array(8)->as_XMMRegister(ra_,this,idx8)/* xmm_1 */);
  
#line 37720 "ad_x86.cpp"
  }
}

void minmax_reductionF_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// dst
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// tmp
  unsigned idx5 = idx4 + opnd_array(4)->num_edges(); 	// atmp
  unsigned idx6 = idx5 + opnd_array(5)->num_edges(); 	// btmp
  unsigned idx7 = idx6 + opnd_array(6)->num_edges(); 	// xmm_0
  unsigned idx8 = idx7 + opnd_array(7)->num_edges(); 	// xmm_1
  {
    C2_MacroAssembler _masm(&cbuf);

#line 5101 "src/hotspot/cpu/x86/x86.ad"

    assert(UseAVX > 0, "sanity");

    int opcode = this->ideal_Opcode();
    int vlen = Matcher::vector_length(this, opnd_array(2));
    __ reduceFloatMinMax(opcode, vlen, false, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src2 */, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* tmp */,
                         opnd_array(5)->as_XMMRegister(ra_,this,idx5)/* atmp */, opnd_array(6)->as_XMMRegister(ra_,this,idx6)/* btmp */, opnd_array(7)->as_XMMRegister(ra_,this,idx7)/* xmm_0 */, opnd_array(8)->as_XMMRegister(ra_,this,idx8)/* xmm_1 */);
  
#line 37748 "ad_x86.cpp"
  }
}

void minmax_reduction2F_avNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// tmp
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// atmp
  unsigned idx5 = idx4 + opnd_array(4)->num_edges(); 	// btmp
  unsigned idx6 = idx5 + opnd_array(5)->num_edges(); 	// xmm_1
  {
    C2_MacroAssembler _masm(&cbuf);

#line 5120 "src/hotspot/cpu/x86/x86.ad"

    assert(UseAVX > 0, "sanity");

    int opcode = this->ideal_Opcode();
    int vlen = Matcher::vector_length(this, opnd_array(2));
    __ reduceFloatMinMax(opcode, vlen, true, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* dst */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src */, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* tmp */,
                         opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* atmp */, opnd_array(5)->as_XMMRegister(ra_,this,idx5)/* btmp */, opnd_array(6)->as_XMMRegister(ra_,this,idx6)/* xmm_1 */);
  
#line 37774 "ad_x86.cpp"
  }
}

void minmax_reduction2F_av_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// tmp
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// atmp
  unsigned idx5 = idx4 + opnd_array(4)->num_edges(); 	// btmp
  unsigned idx6 = idx5 + opnd_array(5)->num_edges(); 	// xmm_1
  {
    C2_MacroAssembler _masm(&cbuf);

#line 5120 "src/hotspot/cpu/x86/x86.ad"

    assert(UseAVX > 0, "sanity");

    int opcode = this->ideal_Opcode();
    int vlen = Matcher::vector_length(this, opnd_array(2));
    __ reduceFloatMinMax(opcode, vlen, true, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* dst */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src */, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* tmp */,
                         opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* atmp */, opnd_array(5)->as_XMMRegister(ra_,this,idx5)/* btmp */, opnd_array(6)->as_XMMRegister(ra_,this,idx6)/* xmm_1 */);
  
#line 37800 "ad_x86.cpp"
  }
}

void minmax_reductionF_avNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// tmp
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// atmp
  unsigned idx5 = idx4 + opnd_array(4)->num_edges(); 	// btmp
  unsigned idx6 = idx5 + opnd_array(5)->num_edges(); 	// xmm_0
  unsigned idx7 = idx6 + opnd_array(6)->num_edges(); 	// xmm_1
  {
    C2_MacroAssembler _masm(&cbuf);

#line 5140 "src/hotspot/cpu/x86/x86.ad"

    assert(UseAVX > 0, "sanity");

    int opcode = this->ideal_Opcode();
    int vlen = Matcher::vector_length(this, opnd_array(2));
    __ reduceFloatMinMax(opcode, vlen, true, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* dst */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src */, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* tmp */,
                         opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* atmp */, opnd_array(5)->as_XMMRegister(ra_,this,idx5)/* btmp */, opnd_array(6)->as_XMMRegister(ra_,this,idx6)/* xmm_0 */, opnd_array(7)->as_XMMRegister(ra_,this,idx7)/* xmm_1 */);
  
#line 37827 "ad_x86.cpp"
  }
}

void minmax_reductionF_av_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// tmp
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// atmp
  unsigned idx5 = idx4 + opnd_array(4)->num_edges(); 	// btmp
  unsigned idx6 = idx5 + opnd_array(5)->num_edges(); 	// xmm_0
  unsigned idx7 = idx6 + opnd_array(6)->num_edges(); 	// xmm_1
  {
    C2_MacroAssembler _masm(&cbuf);

#line 5140 "src/hotspot/cpu/x86/x86.ad"

    assert(UseAVX > 0, "sanity");

    int opcode = this->ideal_Opcode();
    int vlen = Matcher::vector_length(this, opnd_array(2));
    __ reduceFloatMinMax(opcode, vlen, true, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* dst */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src */, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* tmp */,
                         opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* atmp */, opnd_array(5)->as_XMMRegister(ra_,this,idx5)/* btmp */, opnd_array(6)->as_XMMRegister(ra_,this,idx6)/* xmm_0 */, opnd_array(7)->as_XMMRegister(ra_,this,idx7)/* xmm_1 */);
  
#line 37854 "ad_x86.cpp"
  }
}

void minmax_reduction2DNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// dst
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// tmp1
  unsigned idx5 = idx4 + opnd_array(4)->num_edges(); 	// tmp2
  unsigned idx6 = idx5 + opnd_array(5)->num_edges(); 	// tmp3
  unsigned idx7 = idx6 + opnd_array(6)->num_edges(); 	// tmp4
  {
    C2_MacroAssembler _masm(&cbuf);

#line 5164 "src/hotspot/cpu/x86/x86.ad"

    assert(UseAVX > 0, "sanity");

    int opcode = this->ideal_Opcode();
    int vlen = Matcher::vector_length(this, opnd_array(2));
    __ reduceDoubleMinMax(opcode, vlen, false, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src2 */,
                          opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* tmp1 */, opnd_array(5)->as_XMMRegister(ra_,this,idx5)/* tmp2 */, opnd_array(6)->as_XMMRegister(ra_,this,idx6)/* tmp3 */, opnd_array(7)->as_XMMRegister(ra_,this,idx7)/* tmp4 */);
  
#line 37881 "ad_x86.cpp"
  }
}

void minmax_reduction2D_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// dst
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// tmp1
  unsigned idx5 = idx4 + opnd_array(4)->num_edges(); 	// tmp2
  unsigned idx6 = idx5 + opnd_array(5)->num_edges(); 	// tmp3
  unsigned idx7 = idx6 + opnd_array(6)->num_edges(); 	// tmp4
  {
    C2_MacroAssembler _masm(&cbuf);

#line 5164 "src/hotspot/cpu/x86/x86.ad"

    assert(UseAVX > 0, "sanity");

    int opcode = this->ideal_Opcode();
    int vlen = Matcher::vector_length(this, opnd_array(2));
    __ reduceDoubleMinMax(opcode, vlen, false, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src2 */,
                          opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* tmp1 */, opnd_array(5)->as_XMMRegister(ra_,this,idx5)/* tmp2 */, opnd_array(6)->as_XMMRegister(ra_,this,idx6)/* tmp3 */, opnd_array(7)->as_XMMRegister(ra_,this,idx7)/* tmp4 */);
  
#line 37908 "ad_x86.cpp"
  }
}

void minmax_reductionDNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// dst
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// tmp1
  unsigned idx5 = idx4 + opnd_array(4)->num_edges(); 	// tmp2
  unsigned idx6 = idx5 + opnd_array(5)->num_edges(); 	// tmp3
  unsigned idx7 = idx6 + opnd_array(6)->num_edges(); 	// tmp4
  unsigned idx8 = idx7 + opnd_array(7)->num_edges(); 	// tmp5
  {
    C2_MacroAssembler _masm(&cbuf);

#line 5186 "src/hotspot/cpu/x86/x86.ad"

    assert(UseAVX > 0, "sanity");

    int opcode = this->ideal_Opcode();
    int vlen = Matcher::vector_length(this, opnd_array(2));
    __ reduceDoubleMinMax(opcode, vlen, false, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src2 */,
                          opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* tmp1 */, opnd_array(5)->as_XMMRegister(ra_,this,idx5)/* tmp2 */, opnd_array(6)->as_XMMRegister(ra_,this,idx6)/* tmp3 */, opnd_array(7)->as_XMMRegister(ra_,this,idx7)/* tmp4 */, opnd_array(8)->as_XMMRegister(ra_,this,idx8)/* tmp5 */);
  
#line 37936 "ad_x86.cpp"
  }
}

void minmax_reductionD_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// dst
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// tmp1
  unsigned idx5 = idx4 + opnd_array(4)->num_edges(); 	// tmp2
  unsigned idx6 = idx5 + opnd_array(5)->num_edges(); 	// tmp3
  unsigned idx7 = idx6 + opnd_array(6)->num_edges(); 	// tmp4
  unsigned idx8 = idx7 + opnd_array(7)->num_edges(); 	// tmp5
  {
    C2_MacroAssembler _masm(&cbuf);

#line 5186 "src/hotspot/cpu/x86/x86.ad"

    assert(UseAVX > 0, "sanity");

    int opcode = this->ideal_Opcode();
    int vlen = Matcher::vector_length(this, opnd_array(2));
    __ reduceDoubleMinMax(opcode, vlen, false, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src2 */,
                          opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* tmp1 */, opnd_array(5)->as_XMMRegister(ra_,this,idx5)/* tmp2 */, opnd_array(6)->as_XMMRegister(ra_,this,idx6)/* tmp3 */, opnd_array(7)->as_XMMRegister(ra_,this,idx7)/* tmp4 */, opnd_array(8)->as_XMMRegister(ra_,this,idx8)/* tmp5 */);
  
#line 37964 "ad_x86.cpp"
  }
}

void minmax_reduction2D_avNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// tmp1
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// tmp2
  unsigned idx5 = idx4 + opnd_array(4)->num_edges(); 	// tmp3
  unsigned idx6 = idx5 + opnd_array(5)->num_edges(); 	// tmp4
  {
    C2_MacroAssembler _masm(&cbuf);

#line 5207 "src/hotspot/cpu/x86/x86.ad"

    assert(UseAVX > 0, "sanity");

    int opcode = this->ideal_Opcode();
    int vlen = Matcher::vector_length(this, opnd_array(2));
    __ reduceDoubleMinMax(opcode, vlen, true, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* dst */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src */,
                          opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* tmp1 */, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* tmp2 */, opnd_array(5)->as_XMMRegister(ra_,this,idx5)/* tmp3 */, opnd_array(6)->as_XMMRegister(ra_,this,idx6)/* tmp4 */);
  
#line 37990 "ad_x86.cpp"
  }
}

void minmax_reduction2D_av_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// tmp1
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// tmp2
  unsigned idx5 = idx4 + opnd_array(4)->num_edges(); 	// tmp3
  unsigned idx6 = idx5 + opnd_array(5)->num_edges(); 	// tmp4
  {
    C2_MacroAssembler _masm(&cbuf);

#line 5207 "src/hotspot/cpu/x86/x86.ad"

    assert(UseAVX > 0, "sanity");

    int opcode = this->ideal_Opcode();
    int vlen = Matcher::vector_length(this, opnd_array(2));
    __ reduceDoubleMinMax(opcode, vlen, true, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* dst */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src */,
                          opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* tmp1 */, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* tmp2 */, opnd_array(5)->as_XMMRegister(ra_,this,idx5)/* tmp3 */, opnd_array(6)->as_XMMRegister(ra_,this,idx6)/* tmp4 */);
  
#line 38016 "ad_x86.cpp"
  }
}

void minmax_reductionD_avNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// tmp1
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// tmp2
  unsigned idx5 = idx4 + opnd_array(4)->num_edges(); 	// tmp3
  unsigned idx6 = idx5 + opnd_array(5)->num_edges(); 	// tmp4
  unsigned idx7 = idx6 + opnd_array(6)->num_edges(); 	// tmp5
  {
    C2_MacroAssembler _masm(&cbuf);

#line 5227 "src/hotspot/cpu/x86/x86.ad"

    assert(UseAVX > 0, "sanity");

    int opcode = this->ideal_Opcode();
    int vlen = Matcher::vector_length(this, opnd_array(2));
    __ reduceDoubleMinMax(opcode, vlen, true, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* dst */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src */,
                          opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* tmp1 */, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* tmp2 */, opnd_array(5)->as_XMMRegister(ra_,this,idx5)/* tmp3 */, opnd_array(6)->as_XMMRegister(ra_,this,idx6)/* tmp4 */, opnd_array(7)->as_XMMRegister(ra_,this,idx7)/* tmp5 */);
  
#line 38043 "ad_x86.cpp"
  }
}

void minmax_reductionD_av_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// tmp1
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// tmp2
  unsigned idx5 = idx4 + opnd_array(4)->num_edges(); 	// tmp3
  unsigned idx6 = idx5 + opnd_array(5)->num_edges(); 	// tmp4
  unsigned idx7 = idx6 + opnd_array(6)->num_edges(); 	// tmp5
  {
    C2_MacroAssembler _masm(&cbuf);

#line 5227 "src/hotspot/cpu/x86/x86.ad"

    assert(UseAVX > 0, "sanity");

    int opcode = this->ideal_Opcode();
    int vlen = Matcher::vector_length(this, opnd_array(2));
    __ reduceDoubleMinMax(opcode, vlen, true, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* dst */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src */,
                          opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* tmp1 */, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* tmp2 */, opnd_array(5)->as_XMMRegister(ra_,this,idx5)/* tmp3 */, opnd_array(6)->as_XMMRegister(ra_,this,idx6)/* tmp4 */, opnd_array(7)->as_XMMRegister(ra_,this,idx7)/* tmp5 */);
  
#line 38070 "ad_x86.cpp"
  }
}

void vaddBNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 5247 "src/hotspot/cpu/x86/x86.ad"

    __ paddb(opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* dst */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src */);
  
#line 38087 "ad_x86.cpp"
  }
}

void vaddB_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 5257 "src/hotspot/cpu/x86/x86.ad"

    int vlen_enc = vector_length_encoding(this);
    __ vpaddb(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src1 */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src2 */, vlen_enc);
  
#line 38105 "ad_x86.cpp"
  }
}

void vaddB_memNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// mem
  {
    C2_MacroAssembler _masm(&cbuf);

#line 5269 "src/hotspot/cpu/x86/x86.ad"

    int vlen_enc = vector_length_encoding(this);
    __ vpaddb(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, Address::make_raw(opnd_array(2)->base(ra_,this,idx2), opnd_array(2)->index(ra_,this,idx2), opnd_array(2)->scale(), opnd_array(2)->disp(ra_,this,idx2), opnd_array(2)->disp_reloc()), vlen_enc);
  
#line 38123 "ad_x86.cpp"
  }
}

void vaddB_mem_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 5269 "src/hotspot/cpu/x86/x86.ad"

    int vlen_enc = vector_length_encoding(this);
    __ vpaddb(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src */, Address::make_raw(opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1), opnd_array(1)->disp_reloc()), vlen_enc);
  
#line 38141 "ad_x86.cpp"
  }
}

void vaddSNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 5281 "src/hotspot/cpu/x86/x86.ad"

    __ paddw(opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* dst */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src */);
  
#line 38158 "ad_x86.cpp"
  }
}

void vaddS_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 5291 "src/hotspot/cpu/x86/x86.ad"

    int vlen_enc = vector_length_encoding(this);
    __ vpaddw(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src1 */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src2 */, vlen_enc);
  
#line 38176 "ad_x86.cpp"
  }
}

void vaddS_memNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// mem
  {
    C2_MacroAssembler _masm(&cbuf);

#line 5303 "src/hotspot/cpu/x86/x86.ad"

    int vlen_enc = vector_length_encoding(this);
    __ vpaddw(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, Address::make_raw(opnd_array(2)->base(ra_,this,idx2), opnd_array(2)->index(ra_,this,idx2), opnd_array(2)->scale(), opnd_array(2)->disp(ra_,this,idx2), opnd_array(2)->disp_reloc()), vlen_enc);
  
#line 38194 "ad_x86.cpp"
  }
}

void vaddS_mem_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 5303 "src/hotspot/cpu/x86/x86.ad"

    int vlen_enc = vector_length_encoding(this);
    __ vpaddw(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src */, Address::make_raw(opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1), opnd_array(1)->disp_reloc()), vlen_enc);
  
#line 38212 "ad_x86.cpp"
  }
}

void vaddINode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 5315 "src/hotspot/cpu/x86/x86.ad"

    __ paddd(opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* dst */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src */);
  
#line 38229 "ad_x86.cpp"
  }
}

void vaddI_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 5325 "src/hotspot/cpu/x86/x86.ad"

    int vlen_enc = vector_length_encoding(this);
    __ vpaddd(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src1 */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src2 */, vlen_enc);
  
#line 38247 "ad_x86.cpp"
  }
}

void vaddI_memNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// mem
  {
    C2_MacroAssembler _masm(&cbuf);

#line 5338 "src/hotspot/cpu/x86/x86.ad"

    int vlen_enc = vector_length_encoding(this);
    __ vpaddd(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, Address::make_raw(opnd_array(2)->base(ra_,this,idx2), opnd_array(2)->index(ra_,this,idx2), opnd_array(2)->scale(), opnd_array(2)->disp(ra_,this,idx2), opnd_array(2)->disp_reloc()), vlen_enc);
  
#line 38265 "ad_x86.cpp"
  }
}

void vaddI_mem_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 5338 "src/hotspot/cpu/x86/x86.ad"

    int vlen_enc = vector_length_encoding(this);
    __ vpaddd(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src */, Address::make_raw(opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1), opnd_array(1)->disp_reloc()), vlen_enc);
  
#line 38283 "ad_x86.cpp"
  }
}

void vaddLNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 5350 "src/hotspot/cpu/x86/x86.ad"

    __ paddq(opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* dst */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src */);
  
#line 38300 "ad_x86.cpp"
  }
}

void vaddL_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 5360 "src/hotspot/cpu/x86/x86.ad"

    int vlen_enc = vector_length_encoding(this);
    __ vpaddq(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src1 */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src2 */, vlen_enc);
  
#line 38318 "ad_x86.cpp"
  }
}

void vaddL_memNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// mem
  {
    C2_MacroAssembler _masm(&cbuf);

#line 5372 "src/hotspot/cpu/x86/x86.ad"

    int vlen_enc = vector_length_encoding(this);
    __ vpaddq(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, Address::make_raw(opnd_array(2)->base(ra_,this,idx2), opnd_array(2)->index(ra_,this,idx2), opnd_array(2)->scale(), opnd_array(2)->disp(ra_,this,idx2), opnd_array(2)->disp_reloc()), vlen_enc);
  
#line 38336 "ad_x86.cpp"
  }
}

void vaddL_mem_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 5372 "src/hotspot/cpu/x86/x86.ad"

    int vlen_enc = vector_length_encoding(this);
    __ vpaddq(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src */, Address::make_raw(opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1), opnd_array(1)->disp_reloc()), vlen_enc);
  
#line 38354 "ad_x86.cpp"
  }
}

void vaddFNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 5384 "src/hotspot/cpu/x86/x86.ad"

    __ addps(opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* dst */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src */);
  
#line 38371 "ad_x86.cpp"
  }
}

void vaddF_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 5394 "src/hotspot/cpu/x86/x86.ad"

    int vlen_enc = vector_length_encoding(this);
    __ vaddps(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src1 */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src2 */, vlen_enc);
  
#line 38389 "ad_x86.cpp"
  }
}

void vaddF_memNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// mem
  {
    C2_MacroAssembler _masm(&cbuf);

#line 5406 "src/hotspot/cpu/x86/x86.ad"

    int vlen_enc = vector_length_encoding(this);
    __ vaddps(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, Address::make_raw(opnd_array(2)->base(ra_,this,idx2), opnd_array(2)->index(ra_,this,idx2), opnd_array(2)->scale(), opnd_array(2)->disp(ra_,this,idx2), opnd_array(2)->disp_reloc()), vlen_enc);
  
#line 38407 "ad_x86.cpp"
  }
}

void vaddF_mem_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 5406 "src/hotspot/cpu/x86/x86.ad"

    int vlen_enc = vector_length_encoding(this);
    __ vaddps(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src */, Address::make_raw(opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1), opnd_array(1)->disp_reloc()), vlen_enc);
  
#line 38425 "ad_x86.cpp"
  }
}

void vaddDNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 5418 "src/hotspot/cpu/x86/x86.ad"

    __ addpd(opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* dst */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src */);
  
#line 38442 "ad_x86.cpp"
  }
}

void vaddD_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 5428 "src/hotspot/cpu/x86/x86.ad"

    int vlen_enc = vector_length_encoding(this);
    __ vaddpd(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src1 */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src2 */, vlen_enc);
  
#line 38460 "ad_x86.cpp"
  }
}

void vaddD_memNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// mem
  {
    C2_MacroAssembler _masm(&cbuf);

#line 5440 "src/hotspot/cpu/x86/x86.ad"

    int vlen_enc = vector_length_encoding(this);
    __ vaddpd(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, Address::make_raw(opnd_array(2)->base(ra_,this,idx2), opnd_array(2)->index(ra_,this,idx2), opnd_array(2)->scale(), opnd_array(2)->disp(ra_,this,idx2), opnd_array(2)->disp_reloc()), vlen_enc);
  
#line 38478 "ad_x86.cpp"
  }
}

void vaddD_mem_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 5440 "src/hotspot/cpu/x86/x86.ad"

    int vlen_enc = vector_length_encoding(this);
    __ vaddpd(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src */, Address::make_raw(opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1), opnd_array(1)->disp_reloc()), vlen_enc);
  
#line 38496 "ad_x86.cpp"
  }
}

void vsubBNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 5454 "src/hotspot/cpu/x86/x86.ad"

    __ psubb(opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* dst */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src */);
  
#line 38513 "ad_x86.cpp"
  }
}

void vsubB_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 5464 "src/hotspot/cpu/x86/x86.ad"

    int vlen_enc = vector_length_encoding(this);
    __ vpsubb(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src1 */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src2 */, vlen_enc);
  
#line 38531 "ad_x86.cpp"
  }
}

void vsubB_memNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// mem
  {
    C2_MacroAssembler _masm(&cbuf);

#line 5476 "src/hotspot/cpu/x86/x86.ad"

    int vlen_enc = vector_length_encoding(this);
    __ vpsubb(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, Address::make_raw(opnd_array(2)->base(ra_,this,idx2), opnd_array(2)->index(ra_,this,idx2), opnd_array(2)->scale(), opnd_array(2)->disp(ra_,this,idx2), opnd_array(2)->disp_reloc()), vlen_enc);
  
#line 38549 "ad_x86.cpp"
  }
}

void vsubSNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 5488 "src/hotspot/cpu/x86/x86.ad"

    __ psubw(opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* dst */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src */);
  
#line 38566 "ad_x86.cpp"
  }
}

void vsubS_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 5499 "src/hotspot/cpu/x86/x86.ad"

    int vlen_enc = vector_length_encoding(this);
    __ vpsubw(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src1 */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src2 */, vlen_enc);
  
#line 38584 "ad_x86.cpp"
  }
}

void vsubS_memNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// mem
  {
    C2_MacroAssembler _masm(&cbuf);

#line 5511 "src/hotspot/cpu/x86/x86.ad"

    int vlen_enc = vector_length_encoding(this);
    __ vpsubw(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, Address::make_raw(opnd_array(2)->base(ra_,this,idx2), opnd_array(2)->index(ra_,this,idx2), opnd_array(2)->scale(), opnd_array(2)->disp(ra_,this,idx2), opnd_array(2)->disp_reloc()), vlen_enc);
  
#line 38602 "ad_x86.cpp"
  }
}

void vsubINode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 5523 "src/hotspot/cpu/x86/x86.ad"

    __ psubd(opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* dst */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src */);
  
#line 38619 "ad_x86.cpp"
  }
}

void vsubI_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 5533 "src/hotspot/cpu/x86/x86.ad"

    int vlen_enc = vector_length_encoding(this);
    __ vpsubd(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src1 */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src2 */, vlen_enc);
  
#line 38637 "ad_x86.cpp"
  }
}

void vsubI_memNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// mem
  {
    C2_MacroAssembler _masm(&cbuf);

#line 5545 "src/hotspot/cpu/x86/x86.ad"

    int vlen_enc = vector_length_encoding(this);
    __ vpsubd(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, Address::make_raw(opnd_array(2)->base(ra_,this,idx2), opnd_array(2)->index(ra_,this,idx2), opnd_array(2)->scale(), opnd_array(2)->disp(ra_,this,idx2), opnd_array(2)->disp_reloc()), vlen_enc);
  
#line 38655 "ad_x86.cpp"
  }
}

void vsubLNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 5557 "src/hotspot/cpu/x86/x86.ad"

    __ psubq(opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* dst */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src */);
  
#line 38672 "ad_x86.cpp"
  }
}

void vsubL_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 5567 "src/hotspot/cpu/x86/x86.ad"

    int vlen_enc = vector_length_encoding(this);
    __ vpsubq(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src1 */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src2 */, vlen_enc);
  
#line 38690 "ad_x86.cpp"
  }
}

void vsubL_memNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// mem
  {
    C2_MacroAssembler _masm(&cbuf);

#line 5580 "src/hotspot/cpu/x86/x86.ad"

    int vlen_enc = vector_length_encoding(this);
    __ vpsubq(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, Address::make_raw(opnd_array(2)->base(ra_,this,idx2), opnd_array(2)->index(ra_,this,idx2), opnd_array(2)->scale(), opnd_array(2)->disp(ra_,this,idx2), opnd_array(2)->disp_reloc()), vlen_enc);
  
#line 38708 "ad_x86.cpp"
  }
}

void vsubFNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 5592 "src/hotspot/cpu/x86/x86.ad"

    __ subps(opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* dst */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src */);
  
#line 38725 "ad_x86.cpp"
  }
}

void vsubF_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 5602 "src/hotspot/cpu/x86/x86.ad"

    int vlen_enc = vector_length_encoding(this);
    __ vsubps(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src1 */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src2 */, vlen_enc);
  
#line 38743 "ad_x86.cpp"
  }
}

void vsubF_memNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// mem
  {
    C2_MacroAssembler _masm(&cbuf);

#line 5614 "src/hotspot/cpu/x86/x86.ad"

    int vlen_enc = vector_length_encoding(this);
    __ vsubps(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, Address::make_raw(opnd_array(2)->base(ra_,this,idx2), opnd_array(2)->index(ra_,this,idx2), opnd_array(2)->scale(), opnd_array(2)->disp(ra_,this,idx2), opnd_array(2)->disp_reloc()), vlen_enc);
  
#line 38761 "ad_x86.cpp"
  }
}

void vsubDNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 5626 "src/hotspot/cpu/x86/x86.ad"

    __ subpd(opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* dst */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src */);
  
#line 38778 "ad_x86.cpp"
  }
}

void vsubD_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 5636 "src/hotspot/cpu/x86/x86.ad"

    int vlen_enc = vector_length_encoding(this);
    __ vsubpd(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src1 */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src2 */, vlen_enc);
  
#line 38796 "ad_x86.cpp"
  }
}

void vsubD_memNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// mem
  {
    C2_MacroAssembler _masm(&cbuf);

#line 5648 "src/hotspot/cpu/x86/x86.ad"

    int vlen_enc = vector_length_encoding(this);
    __ vsubpd(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, Address::make_raw(opnd_array(2)->base(ra_,this,idx2), opnd_array(2)->index(ra_,this,idx2), opnd_array(2)->scale(), opnd_array(2)->disp(ra_,this,idx2), opnd_array(2)->disp_reloc()), vlen_enc);
  
#line 38814 "ad_x86.cpp"
  }
}

void vmul8BNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// dst
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// xtmp
  {
    C2_MacroAssembler _masm(&cbuf);

#line 5663 "src/hotspot/cpu/x86/x86.ad"

    assert(UseSSE > 3, "required");
    __ pmovsxbw(opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src1 */);
    __ pmovsxbw(opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* xtmp */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src2 */);
    __ pmullw(opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* xtmp */);
    __ psllw(opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, 8);
    __ psrlw(opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, 8);
    __ packuswb(opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */);
  
#line 38839 "ad_x86.cpp"
  }
}

void vmulBNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// dst
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// xtmp
  {
    C2_MacroAssembler _masm(&cbuf);

#line 5680 "src/hotspot/cpu/x86/x86.ad"

    assert(UseSSE > 3, "required");
    // Odd-index elements
    __ movdqu(opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src1 */);
    __ psrlw(opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, 8);
    __ movdqu(opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* xtmp */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src2 */);
    __ psrlw(opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* xtmp */, 8);
    __ pmullw(opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* xtmp */);
    __ psllw(opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, 8);
    // Even-index elements
    __ movdqu(opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* xtmp */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src1 */);
    __ pmullw(opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* xtmp */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src2 */);
    __ psllw(opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* xtmp */, 8);
    __ psrlw(opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* xtmp */, 8);
    // Combine
    __ por(opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* xtmp */);
  
#line 38872 "ad_x86.cpp"
  }
}

void vmulB_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// xtmp1
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// xtmp2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 5705 "src/hotspot/cpu/x86/x86.ad"

    int vlen_enc = vector_length_encoding(this);
    // Odd-index elements
    __ vpsrlw(opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* xtmp2 */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src1 */, 8, vlen_enc);
    __ vpsrlw(opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* xtmp1 */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src2 */, 8, vlen_enc);
    __ vpmullw(opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* xtmp2 */, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* xtmp1 */, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* xtmp2 */, vlen_enc);
    __ vpsllw(opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* xtmp2 */, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* xtmp2 */, 8, vlen_enc);
    // Even-index elements
    __ vpmullw(opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* xtmp1 */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src1 */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src2 */, vlen_enc);
    __ vpsllw(opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* xtmp1 */, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* xtmp1 */, 8, vlen_enc);
    __ vpsrlw(opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* xtmp1 */, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* xtmp1 */, 8, vlen_enc);
    // Combine
    __ vpor(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* xtmp1 */, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* xtmp2 */, vlen_enc);
  
#line 38902 "ad_x86.cpp"
  }
}

void vmulSNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 5727 "src/hotspot/cpu/x86/x86.ad"

    __ pmullw(opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* dst */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src */);
  
#line 38919 "ad_x86.cpp"
  }
}

void vmulS_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 5737 "src/hotspot/cpu/x86/x86.ad"

    int vlen_enc = vector_length_encoding(this);
    __ vpmullw(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src1 */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src2 */, vlen_enc);
  
#line 38937 "ad_x86.cpp"
  }
}

void vmulS_memNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// mem
  {
    C2_MacroAssembler _masm(&cbuf);

#line 5749 "src/hotspot/cpu/x86/x86.ad"

    int vlen_enc = vector_length_encoding(this);
    __ vpmullw(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, Address::make_raw(opnd_array(2)->base(ra_,this,idx2), opnd_array(2)->index(ra_,this,idx2), opnd_array(2)->scale(), opnd_array(2)->disp(ra_,this,idx2), opnd_array(2)->disp_reloc()), vlen_enc);
  
#line 38955 "ad_x86.cpp"
  }
}

void vmulS_mem_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 5749 "src/hotspot/cpu/x86/x86.ad"

    int vlen_enc = vector_length_encoding(this);
    __ vpmullw(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src */, Address::make_raw(opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1), opnd_array(1)->disp_reloc()), vlen_enc);
  
#line 38973 "ad_x86.cpp"
  }
}

void vmulINode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 5761 "src/hotspot/cpu/x86/x86.ad"

    assert(UseSSE > 3, "required");
    __ pmulld(opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* dst */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src */);
  
#line 38991 "ad_x86.cpp"
  }
}

void vmulI_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 5772 "src/hotspot/cpu/x86/x86.ad"

    int vlen_enc = vector_length_encoding(this);
    __ vpmulld(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src1 */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src2 */, vlen_enc);
  
#line 39009 "ad_x86.cpp"
  }
}

void vmulI_memNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// mem
  {
    C2_MacroAssembler _masm(&cbuf);

#line 5784 "src/hotspot/cpu/x86/x86.ad"

    int vlen_enc = vector_length_encoding(this);
    __ vpmulld(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, Address::make_raw(opnd_array(2)->base(ra_,this,idx2), opnd_array(2)->index(ra_,this,idx2), opnd_array(2)->scale(), opnd_array(2)->disp(ra_,this,idx2), opnd_array(2)->disp_reloc()), vlen_enc);
  
#line 39027 "ad_x86.cpp"
  }
}

void vmulI_mem_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 5784 "src/hotspot/cpu/x86/x86.ad"

    int vlen_enc = vector_length_encoding(this);
    __ vpmulld(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src */, Address::make_raw(opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1), opnd_array(1)->disp_reloc()), vlen_enc);
  
#line 39045 "ad_x86.cpp"
  }
}

void evmulL_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 5798 "src/hotspot/cpu/x86/x86.ad"

    assert(UseAVX > 2, "required");
    int vlen_enc = vector_length_encoding(this);
    __ evpmullq(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src1 */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src2 */, vlen_enc);
  
#line 39064 "ad_x86.cpp"
  }
}

void evmulL_memNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// mem
  {
    C2_MacroAssembler _masm(&cbuf);

#line 5813 "src/hotspot/cpu/x86/x86.ad"

    assert(UseAVX > 2, "required");
    int vlen_enc = vector_length_encoding(this);
    __ evpmullq(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, Address::make_raw(opnd_array(2)->base(ra_,this,idx2), opnd_array(2)->index(ra_,this,idx2), opnd_array(2)->scale(), opnd_array(2)->disp(ra_,this,idx2), opnd_array(2)->disp_reloc()), vlen_enc);
  
#line 39083 "ad_x86.cpp"
  }
}

void evmulL_mem_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 5813 "src/hotspot/cpu/x86/x86.ad"

    assert(UseAVX > 2, "required");
    int vlen_enc = vector_length_encoding(this);
    __ evpmullq(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src */, Address::make_raw(opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1), opnd_array(1)->disp_reloc()), vlen_enc);
  
#line 39102 "ad_x86.cpp"
  }
}

void vmulLNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// dst
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// xtmp
  {
    C2_MacroAssembler _masm(&cbuf);

#line 5826 "src/hotspot/cpu/x86/x86.ad"

    assert(VM_Version::supports_sse4_1(), "required");
    // Get the lo-hi products, only the lower 32 bits is in concerns
    __ pshufd(opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* xtmp */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src2 */, 0xB1);
    __ pmulld(opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* xtmp */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src1 */);
    __ pshufd(opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* xtmp */, 0xB1);
    __ paddd(opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* xtmp */);
    __ psllq(opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, 32);
    // Get the lo-lo products
    __ movdqu(opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* xtmp */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src1 */);
    __ pmuludq(opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* xtmp */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src2 */);
    __ paddq(opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* xtmp */);
  
#line 39131 "ad_x86.cpp"
  }
}

void vmulL_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// xtmp1
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// xtmp2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 5851 "src/hotspot/cpu/x86/x86.ad"

    int vlen_enc = vector_length_encoding(this);
    // Get the lo-hi products, only the lower 32 bits is in concerns
    __ vpshufd(opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* xtmp1 */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src2 */, 0xB1, vlen_enc);
    __ vpmulld(opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* xtmp1 */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src1 */, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* xtmp1 */, vlen_enc);
    __ vpshufd(opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* xtmp2 */, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* xtmp1 */, 0xB1, vlen_enc);
    __ vpaddd(opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* xtmp2 */, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* xtmp2 */, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* xtmp1 */, vlen_enc);
    __ vpsllq(opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* xtmp2 */, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* xtmp2 */, 32, vlen_enc);
    // Get the lo-lo products
    __ vpmuludq(opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* xtmp1 */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src1 */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src2 */, vlen_enc);
    __ vpaddq(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* xtmp1 */, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* xtmp2 */, vlen_enc);
  
#line 39159 "ad_x86.cpp"
  }
}

void vmulFNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 5871 "src/hotspot/cpu/x86/x86.ad"

    __ mulps(opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* dst */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src */);
  
#line 39176 "ad_x86.cpp"
  }
}

void vmulF_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 5881 "src/hotspot/cpu/x86/x86.ad"

    int vlen_enc = vector_length_encoding(this);
    __ vmulps(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src1 */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src2 */, vlen_enc);
  
#line 39194 "ad_x86.cpp"
  }
}

void vmulF_memNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// mem
  {
    C2_MacroAssembler _masm(&cbuf);

#line 5893 "src/hotspot/cpu/x86/x86.ad"

    int vlen_enc = vector_length_encoding(this);
    __ vmulps(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, Address::make_raw(opnd_array(2)->base(ra_,this,idx2), opnd_array(2)->index(ra_,this,idx2), opnd_array(2)->scale(), opnd_array(2)->disp(ra_,this,idx2), opnd_array(2)->disp_reloc()), vlen_enc);
  
#line 39212 "ad_x86.cpp"
  }
}

void vmulF_mem_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 5893 "src/hotspot/cpu/x86/x86.ad"

    int vlen_enc = vector_length_encoding(this);
    __ vmulps(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src */, Address::make_raw(opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1), opnd_array(1)->disp_reloc()), vlen_enc);
  
#line 39230 "ad_x86.cpp"
  }
}

void vmulDNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 5905 "src/hotspot/cpu/x86/x86.ad"

    __ mulpd(opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* dst */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src */);
  
#line 39247 "ad_x86.cpp"
  }
}

void vmulD_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 5915 "src/hotspot/cpu/x86/x86.ad"

    int vlen_enc = vector_length_encoding(this);
    __ vmulpd(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src1 */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src2 */, vlen_enc);
  
#line 39265 "ad_x86.cpp"
  }
}

void vmulD_memNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// mem
  {
    C2_MacroAssembler _masm(&cbuf);

#line 5927 "src/hotspot/cpu/x86/x86.ad"

    int vlen_enc = vector_length_encoding(this);
    __ vmulpd(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, Address::make_raw(opnd_array(2)->base(ra_,this,idx2), opnd_array(2)->index(ra_,this,idx2), opnd_array(2)->scale(), opnd_array(2)->disp(ra_,this,idx2), opnd_array(2)->disp_reloc()), vlen_enc);
  
#line 39283 "ad_x86.cpp"
  }
}

void vmulD_mem_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 5927 "src/hotspot/cpu/x86/x86.ad"

    int vlen_enc = vector_length_encoding(this);
    __ vmulpd(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src */, Address::make_raw(opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1), opnd_array(1)->disp_reloc()), vlen_enc);
  
#line 39301 "ad_x86.cpp"
  }
}

void vdivFNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 5941 "src/hotspot/cpu/x86/x86.ad"

    __ divps(opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* dst */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src */);
  
#line 39318 "ad_x86.cpp"
  }
}

void vdivF_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 5951 "src/hotspot/cpu/x86/x86.ad"

    int vlen_enc = vector_length_encoding(this);
    __ vdivps(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src1 */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src2 */, vlen_enc);
  
#line 39336 "ad_x86.cpp"
  }
}

void vdivF_memNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// mem
  {
    C2_MacroAssembler _masm(&cbuf);

#line 5963 "src/hotspot/cpu/x86/x86.ad"

    int vlen_enc = vector_length_encoding(this);
    __ vdivps(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, Address::make_raw(opnd_array(2)->base(ra_,this,idx2), opnd_array(2)->index(ra_,this,idx2), opnd_array(2)->scale(), opnd_array(2)->disp(ra_,this,idx2), opnd_array(2)->disp_reloc()), vlen_enc);
  
#line 39354 "ad_x86.cpp"
  }
}

void vdivDNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 5975 "src/hotspot/cpu/x86/x86.ad"

    __ divpd(opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* dst */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src */);
  
#line 39371 "ad_x86.cpp"
  }
}

void vdivD_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 5985 "src/hotspot/cpu/x86/x86.ad"

    int vlen_enc = vector_length_encoding(this);
    __ vdivpd(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src1 */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src2 */, vlen_enc);
  
#line 39389 "ad_x86.cpp"
  }
}

void vdivD_memNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// mem
  {
    C2_MacroAssembler _masm(&cbuf);

#line 5997 "src/hotspot/cpu/x86/x86.ad"

    int vlen_enc = vector_length_encoding(this);
    __ vdivpd(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, Address::make_raw(opnd_array(2)->base(ra_,this,idx2), opnd_array(2)->index(ra_,this,idx2), opnd_array(2)->scale(), opnd_array(2)->disp(ra_,this,idx2), opnd_array(2)->disp_reloc()), vlen_enc);
  
#line 39407 "ad_x86.cpp"
  }
}

void minmax_reg_sseNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 6013 "src/hotspot/cpu/x86/x86.ad"

    assert(UseSSE >= 4, "required");

    int opcode = this->ideal_Opcode();
    BasicType elem_bt = Matcher::vector_element_basic_type(this);
    __ pminmax(opcode, elem_bt, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* dst */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src */);
  
#line 39428 "ad_x86.cpp"
  }
}

void minmax_reg_sse_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 6013 "src/hotspot/cpu/x86/x86.ad"

    assert(UseSSE >= 4, "required");

    int opcode = this->ideal_Opcode();
    BasicType elem_bt = Matcher::vector_element_basic_type(this);
    __ pminmax(opcode, elem_bt, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* dst */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src */);
  
#line 39449 "ad_x86.cpp"
  }
}

void vminmax_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 6029 "src/hotspot/cpu/x86/x86.ad"

    int opcode = this->ideal_Opcode();
    int vlen_enc = vector_length_encoding(this);
    BasicType elem_bt = Matcher::vector_element_basic_type(this);

    __ vpminmax(opcode, elem_bt, opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src1 */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src2 */, vlen_enc);
  
#line 39470 "ad_x86.cpp"
  }
}

void vminmax_reg_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 6029 "src/hotspot/cpu/x86/x86.ad"

    int opcode = this->ideal_Opcode();
    int vlen_enc = vector_length_encoding(this);
    BasicType elem_bt = Matcher::vector_element_basic_type(this);

    __ vpminmax(opcode, elem_bt, opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src1 */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src2 */, vlen_enc);
  
#line 39491 "ad_x86.cpp"
  }
}

void minmaxL_reg_sseNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// tmp
  {
    C2_MacroAssembler _masm(&cbuf);

#line 6047 "src/hotspot/cpu/x86/x86.ad"

    assert(UseSSE >= 4, "required");

    int opcode = this->ideal_Opcode();
    BasicType elem_bt = Matcher::vector_element_basic_type(this);
    assert(elem_bt == T_LONG, "sanity");

    __ pminmax(opcode, elem_bt, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* dst */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src */, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* tmp */);
  
#line 39515 "ad_x86.cpp"
  }
}

void minmaxL_reg_sse_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// dst
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// tmp
  {
    C2_MacroAssembler _masm(&cbuf);

#line 6047 "src/hotspot/cpu/x86/x86.ad"

    assert(UseSSE >= 4, "required");

    int opcode = this->ideal_Opcode();
    BasicType elem_bt = Matcher::vector_element_basic_type(this);
    assert(elem_bt == T_LONG, "sanity");

    __ pminmax(opcode, elem_bt, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* tmp */);
  
#line 39539 "ad_x86.cpp"
  }
}

void vminmaxL_reg_avxNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// dst
  {
    C2_MacroAssembler _masm(&cbuf);

#line 6066 "src/hotspot/cpu/x86/x86.ad"

    int vlen_enc = vector_length_encoding(this);
    int opcode = this->ideal_Opcode();
    BasicType elem_bt = Matcher::vector_element_basic_type(this);
    assert(elem_bt == T_LONG, "sanity");

    __ vpminmax(opcode, elem_bt, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src1 */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src2 */, vlen_enc);
  
#line 39562 "ad_x86.cpp"
  }
}

void vminmaxL_reg_avx_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// dst
  {
    C2_MacroAssembler _masm(&cbuf);

#line 6066 "src/hotspot/cpu/x86/x86.ad"

    int vlen_enc = vector_length_encoding(this);
    int opcode = this->ideal_Opcode();
    BasicType elem_bt = Matcher::vector_element_basic_type(this);
    assert(elem_bt == T_LONG, "sanity");

    __ vpminmax(opcode, elem_bt, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src1 */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src2 */, vlen_enc);
  
#line 39585 "ad_x86.cpp"
  }
}

void vminmaxL_reg_evexNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 6083 "src/hotspot/cpu/x86/x86.ad"

    assert(UseAVX > 2, "required");

    int vlen_enc = vector_length_encoding(this);
    int opcode = this->ideal_Opcode();
    BasicType elem_bt = Matcher::vector_element_basic_type(this);
    assert(elem_bt == T_LONG, "sanity");

    __ vpminmax(opcode, elem_bt, opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src1 */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src2 */, vlen_enc);
  
#line 39609 "ad_x86.cpp"
  }
}

void vminmaxL_reg_evex_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 6083 "src/hotspot/cpu/x86/x86.ad"

    assert(UseAVX > 2, "required");

    int vlen_enc = vector_length_encoding(this);
    int opcode = this->ideal_Opcode();
    BasicType elem_bt = Matcher::vector_element_basic_type(this);
    assert(elem_bt == T_LONG, "sanity");

    __ vpminmax(opcode, elem_bt, opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src1 */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src2 */, vlen_enc);
  
#line 39633 "ad_x86.cpp"
  }
}

void minmaxFP_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// a
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// b
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// tmp
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// atmp
  unsigned idx5 = idx4 + opnd_array(4)->num_edges(); 	// btmp
  {
    C2_MacroAssembler _masm(&cbuf);

#line 6105 "src/hotspot/cpu/x86/x86.ad"

    assert(UseAVX > 0, "required");

    int opcode = this->ideal_Opcode();
    int vlen_enc = vector_length_encoding(this);
    BasicType elem_bt = Matcher::vector_element_basic_type(this);

    __ vminmax_fp(opcode, elem_bt,
                  opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* a */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* b */,
                  opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* tmp */, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* atmp */, opnd_array(5)->as_XMMRegister(ra_,this,idx5)/* btmp */, vlen_enc);
  
#line 39661 "ad_x86.cpp"
  }
}

void minmaxFP_reg_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// a
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// b
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// tmp
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// atmp
  unsigned idx5 = idx4 + opnd_array(4)->num_edges(); 	// btmp
  {
    C2_MacroAssembler _masm(&cbuf);

#line 6105 "src/hotspot/cpu/x86/x86.ad"

    assert(UseAVX > 0, "required");

    int opcode = this->ideal_Opcode();
    int vlen_enc = vector_length_encoding(this);
    BasicType elem_bt = Matcher::vector_element_basic_type(this);

    __ vminmax_fp(opcode, elem_bt,
                  opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* a */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* b */,
                  opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* tmp */, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* atmp */, opnd_array(5)->as_XMMRegister(ra_,this,idx5)/* btmp */, vlen_enc);
  
#line 39689 "ad_x86.cpp"
  }
}

void evminmaxFP_reg_eavxNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// a
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// b
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// dst
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// atmp
  unsigned idx5 = idx4 + opnd_array(4)->num_edges(); 	// btmp
  unsigned idx6 = idx5 + opnd_array(5)->num_edges(); 	// ktmp
  {
    C2_MacroAssembler _masm(&cbuf);

#line 6126 "src/hotspot/cpu/x86/x86.ad"

    assert(UseAVX > 2, "required");

    int opcode = this->ideal_Opcode();
    int vlen_enc = vector_length_encoding(this);
    BasicType elem_bt = Matcher::vector_element_basic_type(this);

    __ evminmax_fp(opcode, elem_bt,
                   opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* a */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* b */,
                   opnd_array(6)->as_KRegister(ra_,this,idx6)/* ktmp */, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* atmp */, opnd_array(5)->as_XMMRegister(ra_,this,idx5)/* btmp */, vlen_enc);
  
#line 39718 "ad_x86.cpp"
  }
}

void evminmaxFP_reg_eavx_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// a
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// b
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// dst
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// atmp
  unsigned idx5 = idx4 + opnd_array(4)->num_edges(); 	// btmp
  unsigned idx6 = idx5 + opnd_array(5)->num_edges(); 	// ktmp
  {
    C2_MacroAssembler _masm(&cbuf);

#line 6126 "src/hotspot/cpu/x86/x86.ad"

    assert(UseAVX > 2, "required");

    int opcode = this->ideal_Opcode();
    int vlen_enc = vector_length_encoding(this);
    BasicType elem_bt = Matcher::vector_element_basic_type(this);

    __ evminmax_fp(opcode, elem_bt,
                   opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* a */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* b */,
                   opnd_array(6)->as_KRegister(ra_,this,idx6)/* ktmp */, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* atmp */, opnd_array(5)->as_XMMRegister(ra_,this,idx5)/* btmp */, vlen_enc);
  
#line 39747 "ad_x86.cpp"
  }
}

void signumF_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// zero
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// one
  {
    C2_MacroAssembler _masm(&cbuf);

#line 6146 "src/hotspot/cpu/x86/x86.ad"

    int opcode = this->ideal_Opcode();
    __ signum_fp(opcode, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* dst */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* zero */, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* one */);
  
#line 39766 "ad_x86.cpp"
  }
}

void signumD_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// zero
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// one
  {
    C2_MacroAssembler _masm(&cbuf);

#line 6157 "src/hotspot/cpu/x86/x86.ad"

    int opcode = this->ideal_Opcode();
    __ signum_fp(opcode, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* dst */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* zero */, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* one */);
  
#line 39785 "ad_x86.cpp"
  }
}

void signumV_reg_avxNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// zero
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// one
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// dst
  unsigned idx5 = idx4 + opnd_array(4)->num_edges(); 	// xtmp1
  {
    C2_MacroAssembler _masm(&cbuf);

#line 6170 "src/hotspot/cpu/x86/x86.ad"

    int opcode = this->ideal_Opcode();
    int vec_enc = vector_length_encoding(this);
    __ vector_signum_avx(opcode, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* zero */, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* one */,
                         opnd_array(5)->as_XMMRegister(ra_,this,idx5)/* xtmp1 */, vec_enc);
  
#line 39808 "ad_x86.cpp"
  }
}

void signumV_reg_avx_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// zero
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// one
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// dst
  unsigned idx5 = idx4 + opnd_array(4)->num_edges(); 	// xtmp1
  {
    C2_MacroAssembler _masm(&cbuf);

#line 6170 "src/hotspot/cpu/x86/x86.ad"

    int opcode = this->ideal_Opcode();
    int vec_enc = vector_length_encoding(this);
    __ vector_signum_avx(opcode, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* zero */, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* one */,
                         opnd_array(5)->as_XMMRegister(ra_,this,idx5)/* xtmp1 */, vec_enc);
  
#line 39831 "ad_x86.cpp"
  }
}

void signumV_reg_evexNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// zero
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// one
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// dst
  unsigned idx5 = idx4 + opnd_array(4)->num_edges(); 	// ktmp1
  {
    C2_MacroAssembler _masm(&cbuf);

#line 6185 "src/hotspot/cpu/x86/x86.ad"

    int opcode = this->ideal_Opcode();
    int vec_enc = vector_length_encoding(this);
    __ vector_signum_evex(opcode, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* zero */, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* one */,
                          opnd_array(5)->as_KRegister(ra_,this,idx5)/* ktmp1 */, vec_enc);
  
#line 39854 "ad_x86.cpp"
  }
}

void signumV_reg_evex_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// zero
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// one
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// dst
  unsigned idx5 = idx4 + opnd_array(4)->num_edges(); 	// ktmp1
  {
    C2_MacroAssembler _masm(&cbuf);

#line 6185 "src/hotspot/cpu/x86/x86.ad"

    int opcode = this->ideal_Opcode();
    int vec_enc = vector_length_encoding(this);
    __ vector_signum_evex(opcode, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* zero */, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* one */,
                          opnd_array(5)->as_KRegister(ra_,this,idx5)/* ktmp1 */, vec_enc);
  
#line 39877 "ad_x86.cpp"
  }
}

void compressBitsI_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// mask
  {
    C2_MacroAssembler _masm(&cbuf);

#line 6248 "src/hotspot/cpu/x86/x86.ad"

    __ pextl(opnd_array(0)->as_Register(ra_,this)/* dst */, opnd_array(1)->as_Register(ra_,this,idx1)/* src */, opnd_array(2)->as_Register(ra_,this,idx2)/* mask */);
  
#line 39894 "ad_x86.cpp"
  }
}

void expandBitsI_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// mask
  {
    C2_MacroAssembler _masm(&cbuf);

#line 6258 "src/hotspot/cpu/x86/x86.ad"

    __ pdepl(opnd_array(0)->as_Register(ra_,this)/* dst */, opnd_array(1)->as_Register(ra_,this,idx1)/* src */, opnd_array(2)->as_Register(ra_,this,idx2)/* mask */);
  
#line 39911 "ad_x86.cpp"
  }
}

void compressBitsI_memNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// mask
  {
    C2_MacroAssembler _masm(&cbuf);

#line 6268 "src/hotspot/cpu/x86/x86.ad"

    __ pextl(opnd_array(0)->as_Register(ra_,this)/* dst */, opnd_array(1)->as_Register(ra_,this,idx1)/* src */, Address::make_raw(opnd_array(2)->base(ra_,this,idx2), opnd_array(2)->index(ra_,this,idx2), opnd_array(2)->scale(), opnd_array(2)->disp(ra_,this,idx2), opnd_array(2)->disp_reloc()));
  
#line 39928 "ad_x86.cpp"
  }
}

void expandBitsI_memNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// mask
  {
    C2_MacroAssembler _masm(&cbuf);

#line 6278 "src/hotspot/cpu/x86/x86.ad"

    __ pdepl(opnd_array(0)->as_Register(ra_,this)/* dst */, opnd_array(1)->as_Register(ra_,this,idx1)/* src */, Address::make_raw(opnd_array(2)->base(ra_,this,idx2), opnd_array(2)->index(ra_,this,idx2), opnd_array(2)->scale(), opnd_array(2)->disp(ra_,this,idx2), opnd_array(2)->disp_reloc()));
  
#line 39945 "ad_x86.cpp"
  }
}

void vsqrtF_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 6289 "src/hotspot/cpu/x86/x86.ad"

    assert(UseAVX > 0, "required");
    int vlen_enc = vector_length_encoding(this);
    __ vsqrtps(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, vlen_enc);
  
#line 39963 "ad_x86.cpp"
  }
}

void vsqrtF_memNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  {
    C2_MacroAssembler _masm(&cbuf);

#line 6301 "src/hotspot/cpu/x86/x86.ad"

    assert(UseAVX > 0, "required");
    int vlen_enc = vector_length_encoding(this);
    __ vsqrtps(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, Address::make_raw(opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1), opnd_array(1)->disp_reloc()), vlen_enc);
  
#line 39981 "ad_x86.cpp"
  }
}

void vsqrtD_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 6313 "src/hotspot/cpu/x86/x86.ad"

    assert(UseAVX > 0, "required");
    int vlen_enc = vector_length_encoding(this);
    __ vsqrtpd(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, vlen_enc);
  
#line 39999 "ad_x86.cpp"
  }
}

void vsqrtD_memNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  {
    C2_MacroAssembler _masm(&cbuf);

#line 6325 "src/hotspot/cpu/x86/x86.ad"

    assert(UseAVX > 0, "required");
    int vlen_enc = vector_length_encoding(this);
    __ vsqrtpd(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, Address::make_raw(opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1), opnd_array(1)->disp_reloc()), vlen_enc);
  
#line 40017 "ad_x86.cpp"
  }
}

void vshiftcntNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// cnt
  {
    C2_MacroAssembler _masm(&cbuf);

#line 6341 "src/hotspot/cpu/x86/x86.ad"

    __ movdl(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_Register(ra_,this,idx1)/* cnt */);
  
#line 40033 "ad_x86.cpp"
  }
}

void vshiftcnt_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// cnt
  {
    C2_MacroAssembler _masm(&cbuf);

#line 6341 "src/hotspot/cpu/x86/x86.ad"

    __ movdl(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_Register(ra_,this,idx1)/* cnt */);
  
#line 40049 "ad_x86.cpp"
  }
}

void vshiftBNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// shift
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// dst
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// tmp
  {
    C2_MacroAssembler _masm(&cbuf);

#line 6355 "src/hotspot/cpu/x86/x86.ad"

    assert(UseSSE > 3, "required");
    int opcode = this->ideal_Opcode();
    bool sign = (opcode != Op_URShiftVB);
    __ vextendbw(sign, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* tmp */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */);
    __ vshiftw(opcode, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* tmp */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* shift */);
    __ movdqu(opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, ExternalAddress(vector_short_to_byte_mask()), noreg);
    __ pand(opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* tmp */);
    __ packuswb(opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */);
  
#line 40075 "ad_x86.cpp"
  }
}

void vshiftB_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// shift
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// dst
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// tmp
  {
    C2_MacroAssembler _masm(&cbuf);

#line 6355 "src/hotspot/cpu/x86/x86.ad"

    assert(UseSSE > 3, "required");
    int opcode = this->ideal_Opcode();
    bool sign = (opcode != Op_URShiftVB);
    __ vextendbw(sign, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* tmp */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */);
    __ vshiftw(opcode, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* tmp */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* shift */);
    __ movdqu(opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, ExternalAddress(vector_short_to_byte_mask()), noreg);
    __ pand(opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* tmp */);
    __ packuswb(opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */);
  
#line 40101 "ad_x86.cpp"
  }
}

void vshiftB_1Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// shift
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// dst
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// tmp
  {
    C2_MacroAssembler _masm(&cbuf);

#line 6355 "src/hotspot/cpu/x86/x86.ad"

    assert(UseSSE > 3, "required");
    int opcode = this->ideal_Opcode();
    bool sign = (opcode != Op_URShiftVB);
    __ vextendbw(sign, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* tmp */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */);
    __ vshiftw(opcode, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* tmp */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* shift */);
    __ movdqu(opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, ExternalAddress(vector_short_to_byte_mask()), noreg);
    __ pand(opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* tmp */);
    __ packuswb(opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */);
  
#line 40127 "ad_x86.cpp"
  }
}

void vshift16BNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// shift
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// dst
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// tmp1
  unsigned idx5 = idx4 + opnd_array(4)->num_edges(); 	// tmp2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 6376 "src/hotspot/cpu/x86/x86.ad"

    assert(UseSSE > 3, "required");
    int opcode = this->ideal_Opcode();
    bool sign = (opcode != Op_URShiftVB);
    __ vextendbw(sign, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* tmp1 */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */);
    __ vshiftw(opcode, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* tmp1 */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* shift */);
    __ pshufd(opnd_array(5)->as_XMMRegister(ra_,this,idx5)/* tmp2 */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, 0xE);
    __ vextendbw(sign, opnd_array(5)->as_XMMRegister(ra_,this,idx5)/* tmp2 */, opnd_array(5)->as_XMMRegister(ra_,this,idx5)/* tmp2 */);
    __ vshiftw(opcode, opnd_array(5)->as_XMMRegister(ra_,this,idx5)/* tmp2 */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* shift */);
    __ movdqu(opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, ExternalAddress(vector_short_to_byte_mask()), noreg);
    __ pand(opnd_array(5)->as_XMMRegister(ra_,this,idx5)/* tmp2 */, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */);
    __ pand(opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* tmp1 */);
    __ packuswb(opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(5)->as_XMMRegister(ra_,this,idx5)/* tmp2 */);
  
#line 40158 "ad_x86.cpp"
  }
}

void vshift16B_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// shift
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// dst
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// tmp1
  unsigned idx5 = idx4 + opnd_array(4)->num_edges(); 	// tmp2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 6376 "src/hotspot/cpu/x86/x86.ad"

    assert(UseSSE > 3, "required");
    int opcode = this->ideal_Opcode();
    bool sign = (opcode != Op_URShiftVB);
    __ vextendbw(sign, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* tmp1 */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */);
    __ vshiftw(opcode, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* tmp1 */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* shift */);
    __ pshufd(opnd_array(5)->as_XMMRegister(ra_,this,idx5)/* tmp2 */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, 0xE);
    __ vextendbw(sign, opnd_array(5)->as_XMMRegister(ra_,this,idx5)/* tmp2 */, opnd_array(5)->as_XMMRegister(ra_,this,idx5)/* tmp2 */);
    __ vshiftw(opcode, opnd_array(5)->as_XMMRegister(ra_,this,idx5)/* tmp2 */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* shift */);
    __ movdqu(opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, ExternalAddress(vector_short_to_byte_mask()), noreg);
    __ pand(opnd_array(5)->as_XMMRegister(ra_,this,idx5)/* tmp2 */, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */);
    __ pand(opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* tmp1 */);
    __ packuswb(opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(5)->as_XMMRegister(ra_,this,idx5)/* tmp2 */);
  
#line 40189 "ad_x86.cpp"
  }
}

void vshift16B_1Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// shift
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// dst
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// tmp1
  unsigned idx5 = idx4 + opnd_array(4)->num_edges(); 	// tmp2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 6376 "src/hotspot/cpu/x86/x86.ad"

    assert(UseSSE > 3, "required");
    int opcode = this->ideal_Opcode();
    bool sign = (opcode != Op_URShiftVB);
    __ vextendbw(sign, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* tmp1 */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */);
    __ vshiftw(opcode, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* tmp1 */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* shift */);
    __ pshufd(opnd_array(5)->as_XMMRegister(ra_,this,idx5)/* tmp2 */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, 0xE);
    __ vextendbw(sign, opnd_array(5)->as_XMMRegister(ra_,this,idx5)/* tmp2 */, opnd_array(5)->as_XMMRegister(ra_,this,idx5)/* tmp2 */);
    __ vshiftw(opcode, opnd_array(5)->as_XMMRegister(ra_,this,idx5)/* tmp2 */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* shift */);
    __ movdqu(opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, ExternalAddress(vector_short_to_byte_mask()), noreg);
    __ pand(opnd_array(5)->as_XMMRegister(ra_,this,idx5)/* tmp2 */, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */);
    __ pand(opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* tmp1 */);
    __ packuswb(opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(5)->as_XMMRegister(ra_,this,idx5)/* tmp2 */);
  
#line 40220 "ad_x86.cpp"
  }
}

void vshift16B_avxNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// shift
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// dst
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// tmp
  {
    C2_MacroAssembler _masm(&cbuf);

#line 6401 "src/hotspot/cpu/x86/x86.ad"

    int opcode = this->ideal_Opcode();
    bool sign = (opcode != Op_URShiftVB);
    int vlen_enc = Assembler::AVX_256bit;
    __ vextendbw(sign, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* tmp */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, vlen_enc);
    __ vshiftw(opcode, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* tmp */, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* tmp */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* shift */, vlen_enc);
    __ vpand(opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* tmp */, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* tmp */, ExternalAddress(vector_short_to_byte_mask()), vlen_enc, noreg);
    __ vextracti128_high(opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* tmp */);
    __ vpackuswb(opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* tmp */, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, 0);
  
#line 40246 "ad_x86.cpp"
  }
}

void vshift16B_avx_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// shift
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// dst
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// tmp
  {
    C2_MacroAssembler _masm(&cbuf);

#line 6401 "src/hotspot/cpu/x86/x86.ad"

    int opcode = this->ideal_Opcode();
    bool sign = (opcode != Op_URShiftVB);
    int vlen_enc = Assembler::AVX_256bit;
    __ vextendbw(sign, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* tmp */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, vlen_enc);
    __ vshiftw(opcode, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* tmp */, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* tmp */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* shift */, vlen_enc);
    __ vpand(opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* tmp */, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* tmp */, ExternalAddress(vector_short_to_byte_mask()), vlen_enc, noreg);
    __ vextracti128_high(opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* tmp */);
    __ vpackuswb(opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* tmp */, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, 0);
  
#line 40272 "ad_x86.cpp"
  }
}

void vshift16B_avx_1Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// shift
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// dst
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// tmp
  {
    C2_MacroAssembler _masm(&cbuf);

#line 6401 "src/hotspot/cpu/x86/x86.ad"

    int opcode = this->ideal_Opcode();
    bool sign = (opcode != Op_URShiftVB);
    int vlen_enc = Assembler::AVX_256bit;
    __ vextendbw(sign, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* tmp */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, vlen_enc);
    __ vshiftw(opcode, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* tmp */, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* tmp */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* shift */, vlen_enc);
    __ vpand(opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* tmp */, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* tmp */, ExternalAddress(vector_short_to_byte_mask()), vlen_enc, noreg);
    __ vextracti128_high(opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* tmp */);
    __ vpackuswb(opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* tmp */, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, 0);
  
#line 40298 "ad_x86.cpp"
  }
}

void vshift32B_avxNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// shift
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// dst
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// tmp
  {
    C2_MacroAssembler _masm(&cbuf);

#line 6421 "src/hotspot/cpu/x86/x86.ad"

    assert(UseAVX > 1, "required");
    int opcode = this->ideal_Opcode();
    bool sign = (opcode != Op_URShiftVB);
    int vlen_enc = Assembler::AVX_256bit;
    __ vextracti128_high(opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* tmp */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */);
    __ vextendbw(sign, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* tmp */, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* tmp */, vlen_enc);
    __ vextendbw(sign, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, vlen_enc);
    __ vshiftw(opcode, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* tmp */, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* tmp */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* shift */, vlen_enc);
    __ vshiftw(opcode, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* shift */, vlen_enc);
    __ vpand(opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* tmp */, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* tmp */, ExternalAddress(vector_short_to_byte_mask()), vlen_enc, noreg);
    __ vpand(opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, ExternalAddress(vector_short_to_byte_mask()), vlen_enc, noreg);
    __ vpackuswb(opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* tmp */, vlen_enc);
    __ vpermq(opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, 0xD8, vlen_enc);
  
#line 40329 "ad_x86.cpp"
  }
}

void vshift32B_avx_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// shift
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// dst
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// tmp
  {
    C2_MacroAssembler _masm(&cbuf);

#line 6421 "src/hotspot/cpu/x86/x86.ad"

    assert(UseAVX > 1, "required");
    int opcode = this->ideal_Opcode();
    bool sign = (opcode != Op_URShiftVB);
    int vlen_enc = Assembler::AVX_256bit;
    __ vextracti128_high(opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* tmp */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */);
    __ vextendbw(sign, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* tmp */, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* tmp */, vlen_enc);
    __ vextendbw(sign, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, vlen_enc);
    __ vshiftw(opcode, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* tmp */, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* tmp */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* shift */, vlen_enc);
    __ vshiftw(opcode, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* shift */, vlen_enc);
    __ vpand(opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* tmp */, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* tmp */, ExternalAddress(vector_short_to_byte_mask()), vlen_enc, noreg);
    __ vpand(opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, ExternalAddress(vector_short_to_byte_mask()), vlen_enc, noreg);
    __ vpackuswb(opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* tmp */, vlen_enc);
    __ vpermq(opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, 0xD8, vlen_enc);
  
#line 40360 "ad_x86.cpp"
  }
}

void vshift32B_avx_1Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// shift
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// dst
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// tmp
  {
    C2_MacroAssembler _masm(&cbuf);

#line 6421 "src/hotspot/cpu/x86/x86.ad"

    assert(UseAVX > 1, "required");
    int opcode = this->ideal_Opcode();
    bool sign = (opcode != Op_URShiftVB);
    int vlen_enc = Assembler::AVX_256bit;
    __ vextracti128_high(opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* tmp */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */);
    __ vextendbw(sign, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* tmp */, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* tmp */, vlen_enc);
    __ vextendbw(sign, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, vlen_enc);
    __ vshiftw(opcode, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* tmp */, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* tmp */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* shift */, vlen_enc);
    __ vshiftw(opcode, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* shift */, vlen_enc);
    __ vpand(opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* tmp */, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* tmp */, ExternalAddress(vector_short_to_byte_mask()), vlen_enc, noreg);
    __ vpand(opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, ExternalAddress(vector_short_to_byte_mask()), vlen_enc, noreg);
    __ vpackuswb(opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* tmp */, vlen_enc);
    __ vpermq(opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, 0xD8, vlen_enc);
  
#line 40391 "ad_x86.cpp"
  }
}

void vshift64B_avxNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// shift
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// dst
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// tmp1
  unsigned idx5 = idx4 + opnd_array(4)->num_edges(); 	// tmp2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 6446 "src/hotspot/cpu/x86/x86.ad"

    assert(UseAVX > 2, "required");
    int opcode = this->ideal_Opcode();
    bool sign = (opcode != Op_URShiftVB);
    int vlen_enc = Assembler::AVX_512bit;
    __ vextracti64x4(opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* tmp1 */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, 1);
    __ vextendbw(sign, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* tmp1 */, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* tmp1 */, vlen_enc);
    __ vextendbw(sign, opnd_array(5)->as_XMMRegister(ra_,this,idx5)/* tmp2 */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, vlen_enc);
    __ vshiftw(opcode, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* tmp1 */, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* tmp1 */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* shift */, vlen_enc);
    __ vshiftw(opcode, opnd_array(5)->as_XMMRegister(ra_,this,idx5)/* tmp2 */, opnd_array(5)->as_XMMRegister(ra_,this,idx5)/* tmp2 */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* shift */, vlen_enc);
    __ vmovdqu(opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, ExternalAddress(vector_short_to_byte_mask()), noreg);
    __ vpbroadcastd(opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, vlen_enc);
    __ vpand(opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* tmp1 */, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* tmp1 */, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, vlen_enc);
    __ vpand(opnd_array(5)->as_XMMRegister(ra_,this,idx5)/* tmp2 */, opnd_array(5)->as_XMMRegister(ra_,this,idx5)/* tmp2 */, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, vlen_enc);
    __ vpackuswb(opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* tmp1 */, opnd_array(5)->as_XMMRegister(ra_,this,idx5)/* tmp2 */, vlen_enc);
    __ evmovdquq(opnd_array(5)->as_XMMRegister(ra_,this,idx5)/* tmp2 */, ExternalAddress(vector_byte_perm_mask()), vlen_enc, noreg);
    __ vpermq(opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(5)->as_XMMRegister(ra_,this,idx5)/* tmp2 */, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, vlen_enc);
  
#line 40426 "ad_x86.cpp"
  }
}

void vshift64B_avx_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// shift
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// dst
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// tmp1
  unsigned idx5 = idx4 + opnd_array(4)->num_edges(); 	// tmp2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 6446 "src/hotspot/cpu/x86/x86.ad"

    assert(UseAVX > 2, "required");
    int opcode = this->ideal_Opcode();
    bool sign = (opcode != Op_URShiftVB);
    int vlen_enc = Assembler::AVX_512bit;
    __ vextracti64x4(opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* tmp1 */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, 1);
    __ vextendbw(sign, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* tmp1 */, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* tmp1 */, vlen_enc);
    __ vextendbw(sign, opnd_array(5)->as_XMMRegister(ra_,this,idx5)/* tmp2 */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, vlen_enc);
    __ vshiftw(opcode, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* tmp1 */, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* tmp1 */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* shift */, vlen_enc);
    __ vshiftw(opcode, opnd_array(5)->as_XMMRegister(ra_,this,idx5)/* tmp2 */, opnd_array(5)->as_XMMRegister(ra_,this,idx5)/* tmp2 */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* shift */, vlen_enc);
    __ vmovdqu(opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, ExternalAddress(vector_short_to_byte_mask()), noreg);
    __ vpbroadcastd(opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, vlen_enc);
    __ vpand(opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* tmp1 */, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* tmp1 */, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, vlen_enc);
    __ vpand(opnd_array(5)->as_XMMRegister(ra_,this,idx5)/* tmp2 */, opnd_array(5)->as_XMMRegister(ra_,this,idx5)/* tmp2 */, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, vlen_enc);
    __ vpackuswb(opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* tmp1 */, opnd_array(5)->as_XMMRegister(ra_,this,idx5)/* tmp2 */, vlen_enc);
    __ evmovdquq(opnd_array(5)->as_XMMRegister(ra_,this,idx5)/* tmp2 */, ExternalAddress(vector_byte_perm_mask()), vlen_enc, noreg);
    __ vpermq(opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(5)->as_XMMRegister(ra_,this,idx5)/* tmp2 */, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, vlen_enc);
  
#line 40461 "ad_x86.cpp"
  }
}

void vshift64B_avx_1Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// shift
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// dst
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// tmp1
  unsigned idx5 = idx4 + opnd_array(4)->num_edges(); 	// tmp2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 6446 "src/hotspot/cpu/x86/x86.ad"

    assert(UseAVX > 2, "required");
    int opcode = this->ideal_Opcode();
    bool sign = (opcode != Op_URShiftVB);
    int vlen_enc = Assembler::AVX_512bit;
    __ vextracti64x4(opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* tmp1 */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, 1);
    __ vextendbw(sign, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* tmp1 */, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* tmp1 */, vlen_enc);
    __ vextendbw(sign, opnd_array(5)->as_XMMRegister(ra_,this,idx5)/* tmp2 */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, vlen_enc);
    __ vshiftw(opcode, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* tmp1 */, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* tmp1 */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* shift */, vlen_enc);
    __ vshiftw(opcode, opnd_array(5)->as_XMMRegister(ra_,this,idx5)/* tmp2 */, opnd_array(5)->as_XMMRegister(ra_,this,idx5)/* tmp2 */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* shift */, vlen_enc);
    __ vmovdqu(opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, ExternalAddress(vector_short_to_byte_mask()), noreg);
    __ vpbroadcastd(opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, vlen_enc);
    __ vpand(opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* tmp1 */, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* tmp1 */, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, vlen_enc);
    __ vpand(opnd_array(5)->as_XMMRegister(ra_,this,idx5)/* tmp2 */, opnd_array(5)->as_XMMRegister(ra_,this,idx5)/* tmp2 */, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, vlen_enc);
    __ vpackuswb(opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* tmp1 */, opnd_array(5)->as_XMMRegister(ra_,this,idx5)/* tmp2 */, vlen_enc);
    __ evmovdquq(opnd_array(5)->as_XMMRegister(ra_,this,idx5)/* tmp2 */, ExternalAddress(vector_byte_perm_mask()), vlen_enc, noreg);
    __ vpermq(opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(5)->as_XMMRegister(ra_,this,idx5)/* tmp2 */, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, vlen_enc);
  
#line 40496 "ad_x86.cpp"
  }
}

void vshiftSNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// shift
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// dst
  {
    C2_MacroAssembler _masm(&cbuf);

#line 6479 "src/hotspot/cpu/x86/x86.ad"

    int opcode = this->ideal_Opcode();
    if (UseAVX > 0) {
      int vlen_enc = vector_length_encoding(this);
      __ vshiftw(opcode, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* shift */, vlen_enc);
    } else {
      int vlen = Matcher::vector_length(this);
      if (vlen == 2) {
        __ movflt(opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */);
        __ vshiftw(opcode, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* shift */);
      } else if (vlen == 4) {
        __ movdbl(opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */);
        __ vshiftw(opcode, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* shift */);
      } else {
        assert (vlen == 8, "sanity");
        __ movdqu(opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */);
        __ vshiftw(opcode, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* shift */);
      }
    }
  
#line 40531 "ad_x86.cpp"
  }
}

void vshiftS_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// shift
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// dst
  {
    C2_MacroAssembler _masm(&cbuf);

#line 6479 "src/hotspot/cpu/x86/x86.ad"

    int opcode = this->ideal_Opcode();
    if (UseAVX > 0) {
      int vlen_enc = vector_length_encoding(this);
      __ vshiftw(opcode, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* shift */, vlen_enc);
    } else {
      int vlen = Matcher::vector_length(this);
      if (vlen == 2) {
        __ movflt(opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */);
        __ vshiftw(opcode, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* shift */);
      } else if (vlen == 4) {
        __ movdbl(opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */);
        __ vshiftw(opcode, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* shift */);
      } else {
        assert (vlen == 8, "sanity");
        __ movdqu(opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */);
        __ vshiftw(opcode, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* shift */);
      }
    }
  
#line 40566 "ad_x86.cpp"
  }
}

void vshiftS_1Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// shift
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// dst
  {
    C2_MacroAssembler _masm(&cbuf);

#line 6479 "src/hotspot/cpu/x86/x86.ad"

    int opcode = this->ideal_Opcode();
    if (UseAVX > 0) {
      int vlen_enc = vector_length_encoding(this);
      __ vshiftw(opcode, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* shift */, vlen_enc);
    } else {
      int vlen = Matcher::vector_length(this);
      if (vlen == 2) {
        __ movflt(opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */);
        __ vshiftw(opcode, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* shift */);
      } else if (vlen == 4) {
        __ movdbl(opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */);
        __ vshiftw(opcode, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* shift */);
      } else {
        assert (vlen == 8, "sanity");
        __ movdqu(opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */);
        __ vshiftw(opcode, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* shift */);
      }
    }
  
#line 40601 "ad_x86.cpp"
  }
}

void vshiftINode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// shift
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// dst
  {
    C2_MacroAssembler _masm(&cbuf);

#line 6510 "src/hotspot/cpu/x86/x86.ad"

    int opcode = this->ideal_Opcode();
    if (UseAVX > 0) {
      int vlen_enc = vector_length_encoding(this);
      __ vshiftd(opcode, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* shift */, vlen_enc);
    } else {
      int vlen = Matcher::vector_length(this);
      if (vlen == 2) {
        __ movdbl(opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */);
        __ vshiftd(opcode, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* shift */);
      } else {
        assert(vlen == 4, "sanity");
        __ movdqu(opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */);
        __ vshiftd(opcode, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* shift */);
      }
    }
  
#line 40633 "ad_x86.cpp"
  }
}

void vshiftI_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// shift
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// dst
  {
    C2_MacroAssembler _masm(&cbuf);

#line 6510 "src/hotspot/cpu/x86/x86.ad"

    int opcode = this->ideal_Opcode();
    if (UseAVX > 0) {
      int vlen_enc = vector_length_encoding(this);
      __ vshiftd(opcode, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* shift */, vlen_enc);
    } else {
      int vlen = Matcher::vector_length(this);
      if (vlen == 2) {
        __ movdbl(opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */);
        __ vshiftd(opcode, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* shift */);
      } else {
        assert(vlen == 4, "sanity");
        __ movdqu(opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */);
        __ vshiftd(opcode, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* shift */);
      }
    }
  
#line 40665 "ad_x86.cpp"
  }
}

void vshiftI_1Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// shift
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// dst
  {
    C2_MacroAssembler _masm(&cbuf);

#line 6510 "src/hotspot/cpu/x86/x86.ad"

    int opcode = this->ideal_Opcode();
    if (UseAVX > 0) {
      int vlen_enc = vector_length_encoding(this);
      __ vshiftd(opcode, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* shift */, vlen_enc);
    } else {
      int vlen = Matcher::vector_length(this);
      if (vlen == 2) {
        __ movdbl(opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */);
        __ vshiftd(opcode, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* shift */);
      } else {
        assert(vlen == 4, "sanity");
        __ movdqu(opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */);
        __ vshiftd(opcode, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* shift */);
      }
    }
  
#line 40697 "ad_x86.cpp"
  }
}

void vshiftI_immNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// shift
  {
    C2_MacroAssembler _masm(&cbuf);

#line 6536 "src/hotspot/cpu/x86/x86.ad"

    int opcode = this->ideal_Opcode();
    if (UseAVX > 0) {
      int vector_len = vector_length_encoding(this);
      __ vshiftd_imm(opcode, opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, opnd_array(2)->constant(), vector_len);
    } else {
      int vlen = Matcher::vector_length(this);
      if (vlen == 2) {
        __ movdbl(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */);
        __ vshiftd_imm(opcode, opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(2)->constant());
      } else {
        assert(vlen == 4, "sanity");
        __ movdqu(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */);
        __ vshiftd_imm(opcode, opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(2)->constant());
      }
    }
  
#line 40728 "ad_x86.cpp"
  }
}

void vshiftI_imm_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// shift
  {
    C2_MacroAssembler _masm(&cbuf);

#line 6536 "src/hotspot/cpu/x86/x86.ad"

    int opcode = this->ideal_Opcode();
    if (UseAVX > 0) {
      int vector_len = vector_length_encoding(this);
      __ vshiftd_imm(opcode, opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, opnd_array(2)->constant(), vector_len);
    } else {
      int vlen = Matcher::vector_length(this);
      if (vlen == 2) {
        __ movdbl(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */);
        __ vshiftd_imm(opcode, opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(2)->constant());
      } else {
        assert(vlen == 4, "sanity");
        __ movdqu(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */);
        __ vshiftd_imm(opcode, opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(2)->constant());
      }
    }
  
#line 40759 "ad_x86.cpp"
  }
}

void vshiftI_imm_1Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// shift
  {
    C2_MacroAssembler _masm(&cbuf);

#line 6536 "src/hotspot/cpu/x86/x86.ad"

    int opcode = this->ideal_Opcode();
    if (UseAVX > 0) {
      int vector_len = vector_length_encoding(this);
      __ vshiftd_imm(opcode, opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, opnd_array(2)->constant(), vector_len);
    } else {
      int vlen = Matcher::vector_length(this);
      if (vlen == 2) {
        __ movdbl(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */);
        __ vshiftd_imm(opcode, opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(2)->constant());
      } else {
        assert(vlen == 4, "sanity");
        __ movdqu(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */);
        __ vshiftd_imm(opcode, opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(2)->constant());
      }
    }
  
#line 40790 "ad_x86.cpp"
  }
}

void vshiftLNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// shift
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// dst
  {
    C2_MacroAssembler _masm(&cbuf);

#line 6563 "src/hotspot/cpu/x86/x86.ad"

    int opcode = this->ideal_Opcode();
    if (UseAVX > 0) {
      int vlen_enc = vector_length_encoding(this);
      __ vshiftq(opcode, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* shift */, vlen_enc);
    } else {
      assert(Matcher::vector_length(this) == 2, "");
      __ movdqu(opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */);
      __ vshiftq(opcode, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* shift */);
    }
  
#line 40816 "ad_x86.cpp"
  }
}

void vshiftL_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// shift
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// dst
  {
    C2_MacroAssembler _masm(&cbuf);

#line 6563 "src/hotspot/cpu/x86/x86.ad"

    int opcode = this->ideal_Opcode();
    if (UseAVX > 0) {
      int vlen_enc = vector_length_encoding(this);
      __ vshiftq(opcode, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* shift */, vlen_enc);
    } else {
      assert(Matcher::vector_length(this) == 2, "");
      __ movdqu(opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */);
      __ vshiftq(opcode, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* shift */);
    }
  
#line 40842 "ad_x86.cpp"
  }
}

void vshiftL_immNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// shift
  {
    C2_MacroAssembler _masm(&cbuf);

#line 6582 "src/hotspot/cpu/x86/x86.ad"

    int opcode = this->ideal_Opcode();
    if (UseAVX > 0) {
      int vector_len = vector_length_encoding(this);
      __ vshiftq_imm(opcode, opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, opnd_array(2)->constant(), vector_len);
    } else {
      assert(Matcher::vector_length(this) == 2, "");
      __ movdqu(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */);
      __ vshiftq_imm(opcode, opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(2)->constant());
    }
  
#line 40867 "ad_x86.cpp"
  }
}

void vshiftL_imm_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// shift
  {
    C2_MacroAssembler _masm(&cbuf);

#line 6582 "src/hotspot/cpu/x86/x86.ad"

    int opcode = this->ideal_Opcode();
    if (UseAVX > 0) {
      int vector_len = vector_length_encoding(this);
      __ vshiftq_imm(opcode, opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, opnd_array(2)->constant(), vector_len);
    } else {
      assert(Matcher::vector_length(this) == 2, "");
      __ movdqu(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */);
      __ vshiftq_imm(opcode, opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(2)->constant());
    }
  
#line 40892 "ad_x86.cpp"
  }
}

void vshiftL_arith_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// shift
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// dst
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// tmp
  {
    C2_MacroAssembler _masm(&cbuf);

#line 6603 "src/hotspot/cpu/x86/x86.ad"

    uint vlen = Matcher::vector_length(this);
    if (vlen == 2) {
      assert(UseSSE >= 2, "required");
      __ movdqu(opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */);
      __ psrlq(opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* shift */);
      __ movdqu(opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* tmp */, ExternalAddress(vector_long_sign_mask()), noreg);
      __ psrlq(opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* tmp */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* shift */);
      __ pxor(opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* tmp */);
      __ psubq(opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* tmp */);
    } else {
      assert(vlen == 4, "sanity");
      assert(UseAVX > 1, "required");
      int vlen_enc = Assembler::AVX_256bit;
      __ vpsrlq(opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* shift */, vlen_enc);
      __ vmovdqu(opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* tmp */, ExternalAddress(vector_long_sign_mask()), noreg);
      __ vpsrlq(opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* tmp */, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* tmp */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* shift */, vlen_enc);
      __ vpxor(opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* tmp */, vlen_enc);
      __ vpsubq(opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* tmp */, vlen_enc);
    }
  
#line 40929 "ad_x86.cpp"
  }
}

void vshiftL_arith_reg_evexNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// shift
  {
    C2_MacroAssembler _masm(&cbuf);

#line 6631 "src/hotspot/cpu/x86/x86.ad"

    int vlen_enc = vector_length_encoding(this);
    __ evpsraq(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* shift */, vlen_enc);
  
#line 40947 "ad_x86.cpp"
  }
}

void vshift8B_var_nobwNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// shift
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// dst
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// vtmp
  {
    C2_MacroAssembler _masm(&cbuf);

#line 6649 "src/hotspot/cpu/x86/x86.ad"

    assert(UseAVX >= 2, "required");

    int opcode = this->ideal_Opcode();
    int vlen_enc = Assembler::AVX_128bit;
    __ varshiftbw(opcode, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* shift */, vlen_enc, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* vtmp */);
    __ vpackuswb(opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, 0);
  
#line 40971 "ad_x86.cpp"
  }
}

void vshift8B_var_nobw_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// shift
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// dst
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// vtmp
  {
    C2_MacroAssembler _masm(&cbuf);

#line 6649 "src/hotspot/cpu/x86/x86.ad"

    assert(UseAVX >= 2, "required");

    int opcode = this->ideal_Opcode();
    int vlen_enc = Assembler::AVX_128bit;
    __ varshiftbw(opcode, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* shift */, vlen_enc, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* vtmp */);
    __ vpackuswb(opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, 0);
  
#line 40995 "ad_x86.cpp"
  }
}

void vshift8B_var_nobw_1Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// shift
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// dst
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// vtmp
  {
    C2_MacroAssembler _masm(&cbuf);

#line 6649 "src/hotspot/cpu/x86/x86.ad"

    assert(UseAVX >= 2, "required");

    int opcode = this->ideal_Opcode();
    int vlen_enc = Assembler::AVX_128bit;
    __ varshiftbw(opcode, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* shift */, vlen_enc, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* vtmp */);
    __ vpackuswb(opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, 0);
  
#line 41019 "ad_x86.cpp"
  }
}

void vshift16B_var_nobwNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// shift
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// dst
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// vtmp1
  unsigned idx5 = idx4 + opnd_array(4)->num_edges(); 	// vtmp2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 6669 "src/hotspot/cpu/x86/x86.ad"

    assert(UseAVX >= 2, "required");

    int opcode = this->ideal_Opcode();
    int vlen_enc = Assembler::AVX_128bit;
    // Shift lower half and get word result in dst
    __ varshiftbw(opcode, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* shift */, vlen_enc, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* vtmp1 */);

    // Shift upper half and get word result in vtmp1
    __ vpshufd(opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* vtmp1 */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, 0xE, 0);
    __ vpshufd(opnd_array(5)->as_XMMRegister(ra_,this,idx5)/* vtmp2 */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* shift */, 0xE, 0);
    __ varshiftbw(opcode, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* vtmp1 */, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* vtmp1 */, opnd_array(5)->as_XMMRegister(ra_,this,idx5)/* vtmp2 */, vlen_enc, opnd_array(5)->as_XMMRegister(ra_,this,idx5)/* vtmp2 */);

    // Merge and down convert the two word results to byte in dst
    __ vpackuswb(opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* vtmp1 */, 0);
  
#line 41052 "ad_x86.cpp"
  }
}

void vshift16B_var_nobw_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// shift
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// dst
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// vtmp1
  unsigned idx5 = idx4 + opnd_array(4)->num_edges(); 	// vtmp2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 6669 "src/hotspot/cpu/x86/x86.ad"

    assert(UseAVX >= 2, "required");

    int opcode = this->ideal_Opcode();
    int vlen_enc = Assembler::AVX_128bit;
    // Shift lower half and get word result in dst
    __ varshiftbw(opcode, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* shift */, vlen_enc, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* vtmp1 */);

    // Shift upper half and get word result in vtmp1
    __ vpshufd(opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* vtmp1 */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, 0xE, 0);
    __ vpshufd(opnd_array(5)->as_XMMRegister(ra_,this,idx5)/* vtmp2 */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* shift */, 0xE, 0);
    __ varshiftbw(opcode, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* vtmp1 */, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* vtmp1 */, opnd_array(5)->as_XMMRegister(ra_,this,idx5)/* vtmp2 */, vlen_enc, opnd_array(5)->as_XMMRegister(ra_,this,idx5)/* vtmp2 */);

    // Merge and down convert the two word results to byte in dst
    __ vpackuswb(opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* vtmp1 */, 0);
  
#line 41085 "ad_x86.cpp"
  }
}

void vshift16B_var_nobw_1Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// shift
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// dst
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// vtmp1
  unsigned idx5 = idx4 + opnd_array(4)->num_edges(); 	// vtmp2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 6669 "src/hotspot/cpu/x86/x86.ad"

    assert(UseAVX >= 2, "required");

    int opcode = this->ideal_Opcode();
    int vlen_enc = Assembler::AVX_128bit;
    // Shift lower half and get word result in dst
    __ varshiftbw(opcode, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* shift */, vlen_enc, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* vtmp1 */);

    // Shift upper half and get word result in vtmp1
    __ vpshufd(opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* vtmp1 */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, 0xE, 0);
    __ vpshufd(opnd_array(5)->as_XMMRegister(ra_,this,idx5)/* vtmp2 */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* shift */, 0xE, 0);
    __ varshiftbw(opcode, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* vtmp1 */, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* vtmp1 */, opnd_array(5)->as_XMMRegister(ra_,this,idx5)/* vtmp2 */, vlen_enc, opnd_array(5)->as_XMMRegister(ra_,this,idx5)/* vtmp2 */);

    // Merge and down convert the two word results to byte in dst
    __ vpackuswb(opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* vtmp1 */, 0);
  
#line 41118 "ad_x86.cpp"
  }
}

void vshift32B_var_nobwNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// shift
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// dst
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// vtmp1
  unsigned idx5 = idx4 + opnd_array(4)->num_edges(); 	// vtmp2
  unsigned idx6 = idx5 + opnd_array(5)->num_edges(); 	// vtmp3
  unsigned idx7 = idx6 + opnd_array(6)->num_edges(); 	// vtmp4
  {
    C2_MacroAssembler _masm(&cbuf);

#line 6697 "src/hotspot/cpu/x86/x86.ad"

    assert(UseAVX >= 2, "required");

    int opcode = this->ideal_Opcode();
    int vlen_enc = Assembler::AVX_128bit;
    // Process lower 128 bits and get result in dst
    __ varshiftbw(opcode, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* shift */, vlen_enc, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* vtmp1 */);
    __ vpshufd(opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* vtmp1 */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, 0xE, 0);
    __ vpshufd(opnd_array(5)->as_XMMRegister(ra_,this,idx5)/* vtmp2 */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* shift */, 0xE, 0);
    __ varshiftbw(opcode, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* vtmp1 */, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* vtmp1 */, opnd_array(5)->as_XMMRegister(ra_,this,idx5)/* vtmp2 */, vlen_enc, opnd_array(5)->as_XMMRegister(ra_,this,idx5)/* vtmp2 */);
    __ vpackuswb(opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* vtmp1 */, 0);

    // Process higher 128 bits and get result in vtmp3
    __ vextracti128_high(opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* vtmp1 */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */);
    __ vextracti128_high(opnd_array(5)->as_XMMRegister(ra_,this,idx5)/* vtmp2 */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* shift */);
    __ varshiftbw(opcode, opnd_array(6)->as_XMMRegister(ra_,this,idx6)/* vtmp3 */, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* vtmp1 */, opnd_array(5)->as_XMMRegister(ra_,this,idx5)/* vtmp2 */, vlen_enc, opnd_array(7)->as_XMMRegister(ra_,this,idx7)/* vtmp4 */);
    __ vpshufd(opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* vtmp1 */, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* vtmp1 */, 0xE, 0);
    __ vpshufd(opnd_array(5)->as_XMMRegister(ra_,this,idx5)/* vtmp2 */, opnd_array(5)->as_XMMRegister(ra_,this,idx5)/* vtmp2 */, 0xE, 0);
    __ varshiftbw(opcode, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* vtmp1 */, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* vtmp1 */, opnd_array(5)->as_XMMRegister(ra_,this,idx5)/* vtmp2 */, vlen_enc, opnd_array(5)->as_XMMRegister(ra_,this,idx5)/* vtmp2 */);
    __ vpackuswb(opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* vtmp1 */, opnd_array(6)->as_XMMRegister(ra_,this,idx6)/* vtmp3 */, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* vtmp1 */, 0);

    // Merge the two results in dst
    __ vinserti128(opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* vtmp1 */, 0x1);
  
#line 41161 "ad_x86.cpp"
  }
}

void vshift32B_var_nobw_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// shift
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// dst
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// vtmp1
  unsigned idx5 = idx4 + opnd_array(4)->num_edges(); 	// vtmp2
  unsigned idx6 = idx5 + opnd_array(5)->num_edges(); 	// vtmp3
  unsigned idx7 = idx6 + opnd_array(6)->num_edges(); 	// vtmp4
  {
    C2_MacroAssembler _masm(&cbuf);

#line 6697 "src/hotspot/cpu/x86/x86.ad"

    assert(UseAVX >= 2, "required");

    int opcode = this->ideal_Opcode();
    int vlen_enc = Assembler::AVX_128bit;
    // Process lower 128 bits and get result in dst
    __ varshiftbw(opcode, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* shift */, vlen_enc, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* vtmp1 */);
    __ vpshufd(opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* vtmp1 */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, 0xE, 0);
    __ vpshufd(opnd_array(5)->as_XMMRegister(ra_,this,idx5)/* vtmp2 */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* shift */, 0xE, 0);
    __ varshiftbw(opcode, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* vtmp1 */, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* vtmp1 */, opnd_array(5)->as_XMMRegister(ra_,this,idx5)/* vtmp2 */, vlen_enc, opnd_array(5)->as_XMMRegister(ra_,this,idx5)/* vtmp2 */);
    __ vpackuswb(opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* vtmp1 */, 0);

    // Process higher 128 bits and get result in vtmp3
    __ vextracti128_high(opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* vtmp1 */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */);
    __ vextracti128_high(opnd_array(5)->as_XMMRegister(ra_,this,idx5)/* vtmp2 */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* shift */);
    __ varshiftbw(opcode, opnd_array(6)->as_XMMRegister(ra_,this,idx6)/* vtmp3 */, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* vtmp1 */, opnd_array(5)->as_XMMRegister(ra_,this,idx5)/* vtmp2 */, vlen_enc, opnd_array(7)->as_XMMRegister(ra_,this,idx7)/* vtmp4 */);
    __ vpshufd(opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* vtmp1 */, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* vtmp1 */, 0xE, 0);
    __ vpshufd(opnd_array(5)->as_XMMRegister(ra_,this,idx5)/* vtmp2 */, opnd_array(5)->as_XMMRegister(ra_,this,idx5)/* vtmp2 */, 0xE, 0);
    __ varshiftbw(opcode, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* vtmp1 */, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* vtmp1 */, opnd_array(5)->as_XMMRegister(ra_,this,idx5)/* vtmp2 */, vlen_enc, opnd_array(5)->as_XMMRegister(ra_,this,idx5)/* vtmp2 */);
    __ vpackuswb(opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* vtmp1 */, opnd_array(6)->as_XMMRegister(ra_,this,idx6)/* vtmp3 */, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* vtmp1 */, 0);

    // Merge the two results in dst
    __ vinserti128(opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* vtmp1 */, 0x1);
  
#line 41204 "ad_x86.cpp"
  }
}

void vshift32B_var_nobw_1Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// shift
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// dst
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// vtmp1
  unsigned idx5 = idx4 + opnd_array(4)->num_edges(); 	// vtmp2
  unsigned idx6 = idx5 + opnd_array(5)->num_edges(); 	// vtmp3
  unsigned idx7 = idx6 + opnd_array(6)->num_edges(); 	// vtmp4
  {
    C2_MacroAssembler _masm(&cbuf);

#line 6697 "src/hotspot/cpu/x86/x86.ad"

    assert(UseAVX >= 2, "required");

    int opcode = this->ideal_Opcode();
    int vlen_enc = Assembler::AVX_128bit;
    // Process lower 128 bits and get result in dst
    __ varshiftbw(opcode, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* shift */, vlen_enc, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* vtmp1 */);
    __ vpshufd(opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* vtmp1 */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, 0xE, 0);
    __ vpshufd(opnd_array(5)->as_XMMRegister(ra_,this,idx5)/* vtmp2 */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* shift */, 0xE, 0);
    __ varshiftbw(opcode, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* vtmp1 */, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* vtmp1 */, opnd_array(5)->as_XMMRegister(ra_,this,idx5)/* vtmp2 */, vlen_enc, opnd_array(5)->as_XMMRegister(ra_,this,idx5)/* vtmp2 */);
    __ vpackuswb(opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* vtmp1 */, 0);

    // Process higher 128 bits and get result in vtmp3
    __ vextracti128_high(opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* vtmp1 */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */);
    __ vextracti128_high(opnd_array(5)->as_XMMRegister(ra_,this,idx5)/* vtmp2 */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* shift */);
    __ varshiftbw(opcode, opnd_array(6)->as_XMMRegister(ra_,this,idx6)/* vtmp3 */, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* vtmp1 */, opnd_array(5)->as_XMMRegister(ra_,this,idx5)/* vtmp2 */, vlen_enc, opnd_array(7)->as_XMMRegister(ra_,this,idx7)/* vtmp4 */);
    __ vpshufd(opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* vtmp1 */, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* vtmp1 */, 0xE, 0);
    __ vpshufd(opnd_array(5)->as_XMMRegister(ra_,this,idx5)/* vtmp2 */, opnd_array(5)->as_XMMRegister(ra_,this,idx5)/* vtmp2 */, 0xE, 0);
    __ varshiftbw(opcode, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* vtmp1 */, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* vtmp1 */, opnd_array(5)->as_XMMRegister(ra_,this,idx5)/* vtmp2 */, vlen_enc, opnd_array(5)->as_XMMRegister(ra_,this,idx5)/* vtmp2 */);
    __ vpackuswb(opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* vtmp1 */, opnd_array(6)->as_XMMRegister(ra_,this,idx6)/* vtmp3 */, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* vtmp1 */, 0);

    // Merge the two results in dst
    __ vinserti128(opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* vtmp1 */, 0x1);
  
#line 41247 "ad_x86.cpp"
  }
}

void vshiftB_var_evex_bwNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// shift
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// dst
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// vtmp
  {
    C2_MacroAssembler _masm(&cbuf);

#line 6733 "src/hotspot/cpu/x86/x86.ad"

    assert(UseAVX > 2, "required");

    int opcode = this->ideal_Opcode();
    int vlen_enc = vector_length_encoding(this);
    __ evarshiftb(opcode, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* shift */, vlen_enc, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* vtmp */);
  
#line 41270 "ad_x86.cpp"
  }
}

void vshiftB_var_evex_bw_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// shift
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// dst
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// vtmp
  {
    C2_MacroAssembler _masm(&cbuf);

#line 6733 "src/hotspot/cpu/x86/x86.ad"

    assert(UseAVX > 2, "required");

    int opcode = this->ideal_Opcode();
    int vlen_enc = vector_length_encoding(this);
    __ evarshiftb(opcode, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* shift */, vlen_enc, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* vtmp */);
  
#line 41293 "ad_x86.cpp"
  }
}

void vshiftB_var_evex_bw_1Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// shift
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// dst
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// vtmp
  {
    C2_MacroAssembler _masm(&cbuf);

#line 6733 "src/hotspot/cpu/x86/x86.ad"

    assert(UseAVX > 2, "required");

    int opcode = this->ideal_Opcode();
    int vlen_enc = vector_length_encoding(this);
    __ evarshiftb(opcode, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* shift */, vlen_enc, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* vtmp */);
  
#line 41316 "ad_x86.cpp"
  }
}

void vshift64B_var_evex_bwNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// shift
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// dst
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// vtmp1
  unsigned idx5 = idx4 + opnd_array(4)->num_edges(); 	// vtmp2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 6752 "src/hotspot/cpu/x86/x86.ad"

    assert(UseAVX > 2, "required");

    int opcode = this->ideal_Opcode();
    int vlen_enc = Assembler::AVX_256bit;
    __ evarshiftb(opcode, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* shift */, vlen_enc, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* vtmp1 */);
    __ vextracti64x4_high(opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* vtmp1 */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */);
    __ vextracti64x4_high(opnd_array(5)->as_XMMRegister(ra_,this,idx5)/* vtmp2 */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* shift */);
    __ evarshiftb(opcode, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* vtmp1 */, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* vtmp1 */, opnd_array(5)->as_XMMRegister(ra_,this,idx5)/* vtmp2 */, vlen_enc, opnd_array(5)->as_XMMRegister(ra_,this,idx5)/* vtmp2 */);
    __ vinserti64x4(opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* vtmp1 */, 0x1);
  
#line 41344 "ad_x86.cpp"
  }
}

void vshift64B_var_evex_bw_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// shift
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// dst
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// vtmp1
  unsigned idx5 = idx4 + opnd_array(4)->num_edges(); 	// vtmp2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 6752 "src/hotspot/cpu/x86/x86.ad"

    assert(UseAVX > 2, "required");

    int opcode = this->ideal_Opcode();
    int vlen_enc = Assembler::AVX_256bit;
    __ evarshiftb(opcode, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* shift */, vlen_enc, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* vtmp1 */);
    __ vextracti64x4_high(opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* vtmp1 */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */);
    __ vextracti64x4_high(opnd_array(5)->as_XMMRegister(ra_,this,idx5)/* vtmp2 */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* shift */);
    __ evarshiftb(opcode, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* vtmp1 */, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* vtmp1 */, opnd_array(5)->as_XMMRegister(ra_,this,idx5)/* vtmp2 */, vlen_enc, opnd_array(5)->as_XMMRegister(ra_,this,idx5)/* vtmp2 */);
    __ vinserti64x4(opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* vtmp1 */, 0x1);
  
#line 41372 "ad_x86.cpp"
  }
}

void vshift64B_var_evex_bw_1Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// shift
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// dst
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// vtmp1
  unsigned idx5 = idx4 + opnd_array(4)->num_edges(); 	// vtmp2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 6752 "src/hotspot/cpu/x86/x86.ad"

    assert(UseAVX > 2, "required");

    int opcode = this->ideal_Opcode();
    int vlen_enc = Assembler::AVX_256bit;
    __ evarshiftb(opcode, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* shift */, vlen_enc, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* vtmp1 */);
    __ vextracti64x4_high(opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* vtmp1 */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */);
    __ vextracti64x4_high(opnd_array(5)->as_XMMRegister(ra_,this,idx5)/* vtmp2 */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* shift */);
    __ evarshiftb(opcode, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* vtmp1 */, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* vtmp1 */, opnd_array(5)->as_XMMRegister(ra_,this,idx5)/* vtmp2 */, vlen_enc, opnd_array(5)->as_XMMRegister(ra_,this,idx5)/* vtmp2 */);
    __ vinserti64x4(opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* vtmp1 */, 0x1);
  
#line 41400 "ad_x86.cpp"
  }
}

void vshift8S_var_nobwNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// shift
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// dst
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// vtmp
  {
    C2_MacroAssembler _masm(&cbuf);

#line 6776 "src/hotspot/cpu/x86/x86.ad"

    assert(UseAVX >= 2, "required");

    int opcode = this->ideal_Opcode();
    bool sign = (opcode != Op_URShiftVS);
    int vlen_enc = Assembler::AVX_256bit;
    __ vextendwd(sign, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, 1);
    __ vpmovzxwd(opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* vtmp */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* shift */, 1);
    __ varshiftd(opcode, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* vtmp */, vlen_enc);
    __ vpand(opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, ExternalAddress(vector_int_to_short_mask()), vlen_enc, noreg);
    __ vextracti128_high(opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* vtmp */, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */);
    __ vpackusdw(opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* vtmp */, 0);
  
#line 41429 "ad_x86.cpp"
  }
}

void vshift8S_var_nobw_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// shift
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// dst
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// vtmp
  {
    C2_MacroAssembler _masm(&cbuf);

#line 6776 "src/hotspot/cpu/x86/x86.ad"

    assert(UseAVX >= 2, "required");

    int opcode = this->ideal_Opcode();
    bool sign = (opcode != Op_URShiftVS);
    int vlen_enc = Assembler::AVX_256bit;
    __ vextendwd(sign, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, 1);
    __ vpmovzxwd(opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* vtmp */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* shift */, 1);
    __ varshiftd(opcode, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* vtmp */, vlen_enc);
    __ vpand(opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, ExternalAddress(vector_int_to_short_mask()), vlen_enc, noreg);
    __ vextracti128_high(opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* vtmp */, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */);
    __ vpackusdw(opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* vtmp */, 0);
  
#line 41458 "ad_x86.cpp"
  }
}

void vshift8S_var_nobw_1Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// shift
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// dst
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// vtmp
  {
    C2_MacroAssembler _masm(&cbuf);

#line 6776 "src/hotspot/cpu/x86/x86.ad"

    assert(UseAVX >= 2, "required");

    int opcode = this->ideal_Opcode();
    bool sign = (opcode != Op_URShiftVS);
    int vlen_enc = Assembler::AVX_256bit;
    __ vextendwd(sign, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, 1);
    __ vpmovzxwd(opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* vtmp */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* shift */, 1);
    __ varshiftd(opcode, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* vtmp */, vlen_enc);
    __ vpand(opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, ExternalAddress(vector_int_to_short_mask()), vlen_enc, noreg);
    __ vextracti128_high(opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* vtmp */, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */);
    __ vpackusdw(opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* vtmp */, 0);
  
#line 41487 "ad_x86.cpp"
  }
}

void vshift16S_var_nobwNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// shift
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// dst
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// vtmp1
  unsigned idx5 = idx4 + opnd_array(4)->num_edges(); 	// vtmp2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 6801 "src/hotspot/cpu/x86/x86.ad"

    assert(UseAVX >= 2, "required");

    int opcode = this->ideal_Opcode();
    bool sign = (opcode != Op_URShiftVS);
    int vlen_enc = Assembler::AVX_256bit;
    // Shift lower half, with result in vtmp2 using vtmp1 as TEMP
    __ vextendwd(sign, opnd_array(5)->as_XMMRegister(ra_,this,idx5)/* vtmp2 */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, vlen_enc);
    __ vpmovzxwd(opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* vtmp1 */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* shift */, vlen_enc);
    __ varshiftd(opcode, opnd_array(5)->as_XMMRegister(ra_,this,idx5)/* vtmp2 */, opnd_array(5)->as_XMMRegister(ra_,this,idx5)/* vtmp2 */, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* vtmp1 */, vlen_enc);
    __ vpand(opnd_array(5)->as_XMMRegister(ra_,this,idx5)/* vtmp2 */, opnd_array(5)->as_XMMRegister(ra_,this,idx5)/* vtmp2 */, ExternalAddress(vector_int_to_short_mask()), vlen_enc, noreg);

    // Shift upper half, with result in dst using vtmp1 as TEMP
    __ vextracti128_high(opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */);
    __ vextracti128_high(opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* vtmp1 */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* shift */);
    __ vextendwd(sign, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, vlen_enc);
    __ vpmovzxwd(opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* vtmp1 */, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* vtmp1 */, vlen_enc);
    __ varshiftd(opcode, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* vtmp1 */, vlen_enc);
    __ vpand(opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, ExternalAddress(vector_int_to_short_mask()), vlen_enc, noreg);

    // Merge lower and upper half result into dst
    __ vpackusdw(opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(5)->as_XMMRegister(ra_,this,idx5)/* vtmp2 */, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, vlen_enc);
    __ vpermq(opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, 0xD8, vlen_enc);
  
#line 41528 "ad_x86.cpp"
  }
}

void vshift16S_var_nobw_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// shift
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// dst
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// vtmp1
  unsigned idx5 = idx4 + opnd_array(4)->num_edges(); 	// vtmp2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 6801 "src/hotspot/cpu/x86/x86.ad"

    assert(UseAVX >= 2, "required");

    int opcode = this->ideal_Opcode();
    bool sign = (opcode != Op_URShiftVS);
    int vlen_enc = Assembler::AVX_256bit;
    // Shift lower half, with result in vtmp2 using vtmp1 as TEMP
    __ vextendwd(sign, opnd_array(5)->as_XMMRegister(ra_,this,idx5)/* vtmp2 */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, vlen_enc);
    __ vpmovzxwd(opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* vtmp1 */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* shift */, vlen_enc);
    __ varshiftd(opcode, opnd_array(5)->as_XMMRegister(ra_,this,idx5)/* vtmp2 */, opnd_array(5)->as_XMMRegister(ra_,this,idx5)/* vtmp2 */, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* vtmp1 */, vlen_enc);
    __ vpand(opnd_array(5)->as_XMMRegister(ra_,this,idx5)/* vtmp2 */, opnd_array(5)->as_XMMRegister(ra_,this,idx5)/* vtmp2 */, ExternalAddress(vector_int_to_short_mask()), vlen_enc, noreg);

    // Shift upper half, with result in dst using vtmp1 as TEMP
    __ vextracti128_high(opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */);
    __ vextracti128_high(opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* vtmp1 */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* shift */);
    __ vextendwd(sign, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, vlen_enc);
    __ vpmovzxwd(opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* vtmp1 */, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* vtmp1 */, vlen_enc);
    __ varshiftd(opcode, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* vtmp1 */, vlen_enc);
    __ vpand(opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, ExternalAddress(vector_int_to_short_mask()), vlen_enc, noreg);

    // Merge lower and upper half result into dst
    __ vpackusdw(opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(5)->as_XMMRegister(ra_,this,idx5)/* vtmp2 */, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, vlen_enc);
    __ vpermq(opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, 0xD8, vlen_enc);
  
#line 41569 "ad_x86.cpp"
  }
}

void vshift16S_var_nobw_1Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// shift
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// dst
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// vtmp1
  unsigned idx5 = idx4 + opnd_array(4)->num_edges(); 	// vtmp2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 6801 "src/hotspot/cpu/x86/x86.ad"

    assert(UseAVX >= 2, "required");

    int opcode = this->ideal_Opcode();
    bool sign = (opcode != Op_URShiftVS);
    int vlen_enc = Assembler::AVX_256bit;
    // Shift lower half, with result in vtmp2 using vtmp1 as TEMP
    __ vextendwd(sign, opnd_array(5)->as_XMMRegister(ra_,this,idx5)/* vtmp2 */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, vlen_enc);
    __ vpmovzxwd(opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* vtmp1 */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* shift */, vlen_enc);
    __ varshiftd(opcode, opnd_array(5)->as_XMMRegister(ra_,this,idx5)/* vtmp2 */, opnd_array(5)->as_XMMRegister(ra_,this,idx5)/* vtmp2 */, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* vtmp1 */, vlen_enc);
    __ vpand(opnd_array(5)->as_XMMRegister(ra_,this,idx5)/* vtmp2 */, opnd_array(5)->as_XMMRegister(ra_,this,idx5)/* vtmp2 */, ExternalAddress(vector_int_to_short_mask()), vlen_enc, noreg);

    // Shift upper half, with result in dst using vtmp1 as TEMP
    __ vextracti128_high(opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */);
    __ vextracti128_high(opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* vtmp1 */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* shift */);
    __ vextendwd(sign, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, vlen_enc);
    __ vpmovzxwd(opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* vtmp1 */, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* vtmp1 */, vlen_enc);
    __ varshiftd(opcode, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* vtmp1 */, vlen_enc);
    __ vpand(opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, ExternalAddress(vector_int_to_short_mask()), vlen_enc, noreg);

    // Merge lower and upper half result into dst
    __ vpackusdw(opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(5)->as_XMMRegister(ra_,this,idx5)/* vtmp2 */, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, vlen_enc);
    __ vpermq(opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, 0xD8, vlen_enc);
  
#line 41610 "ad_x86.cpp"
  }
}

void vshift16S_var_evex_bwNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// shift
  {
    C2_MacroAssembler _masm(&cbuf);

#line 6835 "src/hotspot/cpu/x86/x86.ad"

    assert(UseAVX > 2, "required");

    int opcode = this->ideal_Opcode();
    int vlen_enc = vector_length_encoding(this);
    if (!VM_Version::supports_avx512vl()) {
      vlen_enc = Assembler::AVX_512bit;
    }
    __ varshiftw(opcode, opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* shift */, vlen_enc);
  
#line 41634 "ad_x86.cpp"
  }
}

void vshift16S_var_evex_bw_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// shift
  {
    C2_MacroAssembler _masm(&cbuf);

#line 6835 "src/hotspot/cpu/x86/x86.ad"

    assert(UseAVX > 2, "required");

    int opcode = this->ideal_Opcode();
    int vlen_enc = vector_length_encoding(this);
    if (!VM_Version::supports_avx512vl()) {
      vlen_enc = Assembler::AVX_512bit;
    }
    __ varshiftw(opcode, opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* shift */, vlen_enc);
  
#line 41658 "ad_x86.cpp"
  }
}

void vshift16S_var_evex_bw_1Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// shift
  {
    C2_MacroAssembler _masm(&cbuf);

#line 6835 "src/hotspot/cpu/x86/x86.ad"

    assert(UseAVX > 2, "required");

    int opcode = this->ideal_Opcode();
    int vlen_enc = vector_length_encoding(this);
    if (!VM_Version::supports_avx512vl()) {
      vlen_enc = Assembler::AVX_512bit;
    }
    __ varshiftw(opcode, opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* shift */, vlen_enc);
  
#line 41682 "ad_x86.cpp"
  }
}

void vshiftI_varNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// shift
  {
    C2_MacroAssembler _masm(&cbuf);

#line 6855 "src/hotspot/cpu/x86/x86.ad"

    assert(UseAVX >= 2, "required");

    int opcode = this->ideal_Opcode();
    int vlen_enc = vector_length_encoding(this);
    __ varshiftd(opcode, opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* shift */, vlen_enc);
  
#line 41703 "ad_x86.cpp"
  }
}

void vshiftI_var_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// shift
  {
    C2_MacroAssembler _masm(&cbuf);

#line 6855 "src/hotspot/cpu/x86/x86.ad"

    assert(UseAVX >= 2, "required");

    int opcode = this->ideal_Opcode();
    int vlen_enc = vector_length_encoding(this);
    __ varshiftd(opcode, opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* shift */, vlen_enc);
  
#line 41724 "ad_x86.cpp"
  }
}

void vshiftI_var_1Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// shift
  {
    C2_MacroAssembler _masm(&cbuf);

#line 6855 "src/hotspot/cpu/x86/x86.ad"

    assert(UseAVX >= 2, "required");

    int opcode = this->ideal_Opcode();
    int vlen_enc = vector_length_encoding(this);
    __ varshiftd(opcode, opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* shift */, vlen_enc);
  
#line 41745 "ad_x86.cpp"
  }
}

void vshiftL_varNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// shift
  {
    C2_MacroAssembler _masm(&cbuf);

#line 6871 "src/hotspot/cpu/x86/x86.ad"

    assert(UseAVX >= 2, "required");

    int opcode = this->ideal_Opcode();
    int vlen_enc = vector_length_encoding(this);
    __ varshiftq(opcode, opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* shift */, vlen_enc);
  
#line 41766 "ad_x86.cpp"
  }
}

void vshiftL_var_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// shift
  {
    C2_MacroAssembler _masm(&cbuf);

#line 6871 "src/hotspot/cpu/x86/x86.ad"

    assert(UseAVX >= 2, "required");

    int opcode = this->ideal_Opcode();
    int vlen_enc = vector_length_encoding(this);
    __ varshiftq(opcode, opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* shift */, vlen_enc);
  
#line 41787 "ad_x86.cpp"
  }
}

void vshiftL_arith_varNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// shift
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// dst
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// vtmp
  {
    C2_MacroAssembler _masm(&cbuf);

#line 6889 "src/hotspot/cpu/x86/x86.ad"

    int opcode = this->ideal_Opcode();
    int vlen_enc = vector_length_encoding(this);
    __ varshiftq(opcode, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* shift */, vlen_enc,
                 opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* vtmp */);
  
#line 41809 "ad_x86.cpp"
  }
}

void vshiftL_arith_var_evexNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// shift
  {
    C2_MacroAssembler _masm(&cbuf);

#line 6903 "src/hotspot/cpu/x86/x86.ad"

    int opcode = this->ideal_Opcode();
    int vlen_enc = vector_length_encoding(this);
    __ varshiftq(opcode, opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* shift */, vlen_enc);
  
#line 41828 "ad_x86.cpp"
  }
}

void vandNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 6917 "src/hotspot/cpu/x86/x86.ad"

    __ pand(opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* dst */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src */);
  
#line 41845 "ad_x86.cpp"
  }
}

void vand_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 6927 "src/hotspot/cpu/x86/x86.ad"

    int vlen_enc = vector_length_encoding(this);
    __ vpand(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src1 */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src2 */, vlen_enc);
  
#line 41863 "ad_x86.cpp"
  }
}

void vand_memNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// mem
  {
    C2_MacroAssembler _masm(&cbuf);

#line 6939 "src/hotspot/cpu/x86/x86.ad"

    int vlen_enc = vector_length_encoding(this);
    __ vpand(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, Address::make_raw(opnd_array(2)->base(ra_,this,idx2), opnd_array(2)->index(ra_,this,idx2), opnd_array(2)->scale(), opnd_array(2)->disp(ra_,this,idx2), opnd_array(2)->disp_reloc()), vlen_enc);
  
#line 41881 "ad_x86.cpp"
  }
}

void vand_mem_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 6939 "src/hotspot/cpu/x86/x86.ad"

    int vlen_enc = vector_length_encoding(this);
    __ vpand(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src */, Address::make_raw(opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1), opnd_array(1)->disp_reloc()), vlen_enc);
  
#line 41899 "ad_x86.cpp"
  }
}

void vorNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 6952 "src/hotspot/cpu/x86/x86.ad"

    __ por(opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* dst */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src */);
  
#line 41916 "ad_x86.cpp"
  }
}

void vor_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 6962 "src/hotspot/cpu/x86/x86.ad"

    int vlen_enc = vector_length_encoding(this);
    __ vpor(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src1 */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src2 */, vlen_enc);
  
#line 41934 "ad_x86.cpp"
  }
}

void vor_memNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// mem
  {
    C2_MacroAssembler _masm(&cbuf);

#line 6974 "src/hotspot/cpu/x86/x86.ad"

    int vlen_enc = vector_length_encoding(this);
    __ vpor(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, Address::make_raw(opnd_array(2)->base(ra_,this,idx2), opnd_array(2)->index(ra_,this,idx2), opnd_array(2)->scale(), opnd_array(2)->disp(ra_,this,idx2), opnd_array(2)->disp_reloc()), vlen_enc);
  
#line 41952 "ad_x86.cpp"
  }
}

void vor_mem_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 6974 "src/hotspot/cpu/x86/x86.ad"

    int vlen_enc = vector_length_encoding(this);
    __ vpor(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src */, Address::make_raw(opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1), opnd_array(1)->disp_reloc()), vlen_enc);
  
#line 41970 "ad_x86.cpp"
  }
}

void vxorNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 6987 "src/hotspot/cpu/x86/x86.ad"

    __ pxor(opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* dst */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src */);
  
#line 41987 "ad_x86.cpp"
  }
}

void vxor_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 6997 "src/hotspot/cpu/x86/x86.ad"

    int vlen_enc = vector_length_encoding(this);
    __ vpxor(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src1 */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src2 */, vlen_enc);
  
#line 42005 "ad_x86.cpp"
  }
}

void vxor_memNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// mem
  {
    C2_MacroAssembler _masm(&cbuf);

#line 7009 "src/hotspot/cpu/x86/x86.ad"

    int vlen_enc = vector_length_encoding(this);
    __ vpxor(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, Address::make_raw(opnd_array(2)->base(ra_,this,idx2), opnd_array(2)->index(ra_,this,idx2), opnd_array(2)->scale(), opnd_array(2)->disp(ra_,this,idx2), opnd_array(2)->disp_reloc()), vlen_enc);
  
#line 42023 "ad_x86.cpp"
  }
}

void vxor_mem_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 7009 "src/hotspot/cpu/x86/x86.ad"

    int vlen_enc = vector_length_encoding(this);
    __ vpxor(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src */, Address::make_raw(opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1), opnd_array(1)->disp_reloc()), vlen_enc);
  
#line 42041 "ad_x86.cpp"
  }
}

void vcastBtoXNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 7021 "src/hotspot/cpu/x86/x86.ad"

    assert(UseAVX > 0, "required");

    BasicType to_elem_bt = Matcher::vector_element_basic_type(this);
    int vlen_enc = vector_length_encoding(this);
    __ vconvert_b2x(to_elem_bt, opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, vlen_enc);
  
#line 42061 "ad_x86.cpp"
  }
}

void castStoXNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 7037 "src/hotspot/cpu/x86/x86.ad"

    assert(UseAVX > 0, "required");

    __ vpand(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, ExternalAddress(vector_short_to_byte_mask()), 0, noreg);
    __ vpackuswb(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, 0);
  
#line 42080 "ad_x86.cpp"
  }
}

void vcastStoXNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// dst
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// vtmp
  {
    C2_MacroAssembler _masm(&cbuf);

#line 7053 "src/hotspot/cpu/x86/x86.ad"

    assert(UseAVX > 0, "required");

    int vlen_enc = vector_length_encoding(Matcher::vector_length_in_bytes(this, opnd_array(1)));
    __ vpand(opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, ExternalAddress(vector_short_to_byte_mask()), vlen_enc, noreg);
    __ vextracti128(opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* vtmp */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* dst */, 0x1);
    __ vpackuswb(opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* dst */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* dst */, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* vtmp */, 0);
  
#line 42103 "ad_x86.cpp"
  }
}

void vcastStoX_evexNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 7069 "src/hotspot/cpu/x86/x86.ad"

    BasicType to_elem_bt = Matcher::vector_element_basic_type(this);
    int src_vlen_enc = vector_length_encoding(this, opnd_array(1));
    int vlen_enc = vector_length_encoding(this);
    switch (to_elem_bt) {
      case T_BYTE:
        if (!VM_Version::supports_avx512vl()) {
          vlen_enc = Assembler::AVX_512bit;
        }
        __ evpmovwb(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, src_vlen_enc);
        break;
      case T_INT:
        __ vpmovsxwd(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, vlen_enc);
        break;
      case T_FLOAT:
        __ vpmovsxwd(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, vlen_enc);
        __ vcvtdq2ps(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, vlen_enc);
        break;
      case T_LONG:
        __ vpmovsxwq(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, vlen_enc);
        break;
      case T_DOUBLE: {
        int mid_vlen_enc = (vlen_enc == Assembler::AVX_512bit) ? Assembler::AVX_256bit : Assembler::AVX_128bit;
        __ vpmovsxwd(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, mid_vlen_enc);
        __ vcvtdq2pd(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, vlen_enc);
        break;
      }
      default:
        ShouldNotReachHere();
    }
  
#line 42147 "ad_x86.cpp"
  }
}

void castItoXNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 7109 "src/hotspot/cpu/x86/x86.ad"

    assert(UseAVX > 0, "required");

    BasicType to_elem_bt = Matcher::vector_element_basic_type(this);
    int vlen_enc = vector_length_encoding(this, opnd_array(1));

    if (to_elem_bt == T_BYTE) {
      __ vpand(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, ExternalAddress(vector_int_to_byte_mask()), vlen_enc, noreg);
      __ vpackusdw(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, vlen_enc);
      __ vpackuswb(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, vlen_enc);
    } else {
      assert(to_elem_bt == T_SHORT, "%s", type2name(to_elem_bt));
      __ vpand(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, ExternalAddress(vector_int_to_short_mask()), vlen_enc, noreg);
      __ vpackusdw(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, vlen_enc);
    }
  
#line 42176 "ad_x86.cpp"
  }
}

void vcastItoXNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// dst
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// vtmp
  {
    C2_MacroAssembler _masm(&cbuf);

#line 7135 "src/hotspot/cpu/x86/x86.ad"

    assert(UseAVX > 0, "required");

    BasicType to_elem_bt = Matcher::vector_element_basic_type(this);
    int vlen_enc = vector_length_encoding(this, opnd_array(1));

    if (to_elem_bt == T_BYTE) {
      __ vpand(opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* vtmp */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, ExternalAddress(vector_int_to_byte_mask()), vlen_enc, noreg);
      __ vextracti128(opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* dst */, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* vtmp */, 0x1);
      __ vpackusdw(opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* dst */, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* vtmp */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* dst */, vlen_enc);
      __ vpackuswb(opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* dst */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* dst */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* dst */, Assembler::AVX_128bit);
    } else {
      assert(to_elem_bt == T_SHORT, "%s", type2name(to_elem_bt));
      __ vpand(opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* vtmp */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, ExternalAddress(vector_int_to_short_mask()), vlen_enc, noreg);
      __ vextracti128(opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* dst */, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* vtmp */, 0x1);
      __ vpackusdw(opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* dst */, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* vtmp */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* dst */, vlen_enc);
    }
  
#line 42209 "ad_x86.cpp"
  }
}

void vcastItoX_evexNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 7161 "src/hotspot/cpu/x86/x86.ad"

    assert(UseAVX > 0, "required");

    BasicType dst_elem_bt = Matcher::vector_element_basic_type(this);
    int src_vlen_enc = vector_length_encoding(this, opnd_array(1));
    int dst_vlen_enc = vector_length_encoding(this);
    switch (dst_elem_bt) {
      case T_BYTE:
        if (!VM_Version::supports_avx512vl()) {
          src_vlen_enc = Assembler::AVX_512bit;
        }
        __ evpmovdb(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, src_vlen_enc);
        break;
      case T_SHORT:
        if (!VM_Version::supports_avx512vl()) {
          src_vlen_enc = Assembler::AVX_512bit;
        }
        __ evpmovdw(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, src_vlen_enc);
        break;
      case T_FLOAT:
        __ vcvtdq2ps(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, dst_vlen_enc);
        break;
      case T_LONG:
        __ vpmovsxdq(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, dst_vlen_enc);
        break;
      case T_DOUBLE:
        __ vcvtdq2pd(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, dst_vlen_enc);
        break;
      default:
        ShouldNotReachHere();
    }
  
#line 42254 "ad_x86.cpp"
  }
}

void vcastLtoBSNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 7201 "src/hotspot/cpu/x86/x86.ad"

    assert(UseAVX > 0, "required");

    int vlen = Matcher::vector_length_in_bytes(this, opnd_array(1));
    BasicType to_elem_bt  = Matcher::vector_element_basic_type(this);
    AddressLiteral mask_addr = (to_elem_bt == T_BYTE) ? ExternalAddress(vector_int_to_byte_mask())
                                                      : ExternalAddress(vector_int_to_short_mask());
    if (vlen <= 16) {
      __ vpshufd(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, 8, Assembler::AVX_128bit);
      __ vpand(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, mask_addr, Assembler::AVX_128bit, noreg);
      __ vpackusdw(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, Assembler::AVX_128bit);
    } else {
      assert(vlen <= 32, "required");
      __ vpermilps(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, 8, Assembler::AVX_256bit);
      __ vpermpd(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, 8, Assembler::AVX_256bit);
      __ vpand(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, mask_addr, Assembler::AVX_128bit, noreg);
      __ vpackusdw(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, Assembler::AVX_128bit);
    }
    if (to_elem_bt == T_BYTE) {
      __ vpackuswb(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, Assembler::AVX_128bit);
    }
  
#line 42289 "ad_x86.cpp"
  }
}

void vcastLtoX_evexNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 7233 "src/hotspot/cpu/x86/x86.ad"

    BasicType to_elem_bt = Matcher::vector_element_basic_type(this);
    int vlen = Matcher::vector_length_in_bytes(this, opnd_array(1));
    int vlen_enc = vector_length_encoding(this, opnd_array(1));
    switch (to_elem_bt) {
      case T_BYTE:
        if (UseAVX > 2 && !VM_Version::supports_avx512vl()) {
          vlen_enc = Assembler::AVX_512bit;
        }
        __ evpmovqb(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, vlen_enc);
        break;
      case T_SHORT:
        if (UseAVX > 2 && !VM_Version::supports_avx512vl()) {
          vlen_enc = Assembler::AVX_512bit;
        }
        __ evpmovqw(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, vlen_enc);
        break;
      case T_INT:
        if (vlen == 8) {
          if (opnd_array(0)->as_XMMRegister(ra_,this)/* dst */!= opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */) {
            __ movflt(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */);
          }
        } else if (vlen == 16) {
          __ pshufd(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, 8);
        } else if (vlen == 32) {
          if (UseAVX > 2) {
            if (!VM_Version::supports_avx512vl()) {
              vlen_enc = Assembler::AVX_512bit;
            }
            __ evpmovqd(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, vlen_enc);
          } else {
            __ vpermilps(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, 8, vlen_enc);
            __ vpermpd(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, 8, vlen_enc);
          }
        } else { // vlen == 64
          __ evpmovqd(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, vlen_enc);
        }
        break;
      case T_FLOAT:
        assert(UseAVX > 2 && VM_Version::supports_avx512dq(), "required");
        __ evcvtqq2ps(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, vlen_enc);
        break;
      case T_DOUBLE:
        assert(UseAVX > 2 && VM_Version::supports_avx512dq(), "required");
        __ evcvtqq2pd(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, vlen_enc);
        break;

      default: assert(false, "%s", type2name(to_elem_bt));
    }
  
#line 42352 "ad_x86.cpp"
  }
}

void vcastFtoD_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 7290 "src/hotspot/cpu/x86/x86.ad"

    int vlen_enc = vector_length_encoding(this);
    __ vcvtps2pd(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, vlen_enc);
  
#line 42369 "ad_x86.cpp"
  }
}

void castFtoX_reg_avxNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// dst
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// xtmp1
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// xtmp2
  unsigned idx5 = idx4 + opnd_array(4)->num_edges(); 	// xtmp3
  unsigned idx6 = idx5 + opnd_array(5)->num_edges(); 	// xtmp4
  {
    C2_MacroAssembler _masm(&cbuf);

#line 7304 "src/hotspot/cpu/x86/x86.ad"

    int vlen_enc = vector_length_encoding(this, opnd_array(1));
    BasicType to_elem_bt = Matcher::vector_element_basic_type(this);
    // JDK-8292878 removed the need for an explicit scratch register needed to load greater than
    // 32 bit addresses for register indirect addressing mode since stub constants
    // are part of code cache and there is a cap of 2G on ReservedCodeCacheSize currently.
    // However, targets are free to increase this limit, but having a large code cache size
    // greater than 2G looks unreasonable in practical scenario, on the hind side with given
    // cap we save a temporary register allocation which in limiting case can prevent
    // spilling in high register pressure blocks.
    __ vector_castF2X_avx(to_elem_bt, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* xtmp1 */,
                          opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* xtmp2 */, opnd_array(5)->as_XMMRegister(ra_,this,idx5)/* xtmp3 */, opnd_array(6)->as_XMMRegister(ra_,this,idx6)/* xtmp4 */,
                          ExternalAddress(vector_float_signflip()), noreg, vlen_enc);
  
#line 42401 "ad_x86.cpp"
  }
}

void castFtoX_reg_evexNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// dst
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// xtmp1
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// xtmp2
  unsigned idx5 = idx4 + opnd_array(4)->num_edges(); 	// ktmp1
  unsigned idx6 = idx5 + opnd_array(5)->num_edges(); 	// ktmp2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 7327 "src/hotspot/cpu/x86/x86.ad"

    BasicType to_elem_bt = Matcher::vector_element_basic_type(this);
    if (to_elem_bt == T_LONG) {
      int vlen_enc = vector_length_encoding(this);
      __ vector_castF2L_evex(opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* xtmp1 */,
                             opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* xtmp2 */, opnd_array(5)->as_KRegister(ra_,this,idx5)/* ktmp1 */, opnd_array(6)->as_KRegister(ra_,this,idx6)/* ktmp2 */,
                             ExternalAddress(vector_double_signflip()), noreg, vlen_enc);
    } else {
      int vlen_enc = vector_length_encoding(this, opnd_array(1));
      __ vector_castF2X_evex(to_elem_bt, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* xtmp1 */,
                             opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* xtmp2 */, opnd_array(5)->as_KRegister(ra_,this,idx5)/* ktmp1 */, opnd_array(6)->as_KRegister(ra_,this,idx6)/* ktmp2 */,
                             ExternalAddress(vector_float_signflip()), noreg, vlen_enc);
    }
  
#line 42433 "ad_x86.cpp"
  }
}

void vcastDtoF_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 7348 "src/hotspot/cpu/x86/x86.ad"

    int vlen_enc = vector_length_encoding(this, opnd_array(1));
    __ vcvtpd2ps(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, vlen_enc);
  
#line 42450 "ad_x86.cpp"
  }
}

void castDtoX_reg_avxNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// dst
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// xtmp1
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// xtmp2
  unsigned idx5 = idx4 + opnd_array(4)->num_edges(); 	// xtmp3
  unsigned idx6 = idx5 + opnd_array(5)->num_edges(); 	// xtmp4
  unsigned idx7 = idx6 + opnd_array(6)->num_edges(); 	// xtmp5
  {
    C2_MacroAssembler _masm(&cbuf);

#line 7361 "src/hotspot/cpu/x86/x86.ad"

    int vlen_enc = vector_length_encoding(this, opnd_array(1));
    BasicType to_elem_bt = Matcher::vector_element_basic_type(this);
    __ vector_castD2X_avx(to_elem_bt, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* xtmp1 */,
                          opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* xtmp2 */, opnd_array(5)->as_XMMRegister(ra_,this,idx5)/* xtmp3 */, opnd_array(6)->as_XMMRegister(ra_,this,idx6)/* xtmp4 */, opnd_array(7)->as_XMMRegister(ra_,this,idx7)/* xtmp5 */,
                          ExternalAddress(vector_float_signflip()), noreg, vlen_enc);
  
#line 42476 "ad_x86.cpp"
  }
}

void castDtoX_reg_evexNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// dst
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// xtmp1
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// xtmp2
  unsigned idx5 = idx4 + opnd_array(4)->num_edges(); 	// ktmp1
  unsigned idx6 = idx5 + opnd_array(5)->num_edges(); 	// ktmp2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 7377 "src/hotspot/cpu/x86/x86.ad"

    int vlen_enc = vector_length_encoding(this, opnd_array(1));
    BasicType to_elem_bt = Matcher::vector_element_basic_type(this);
    AddressLiteral signflip = VM_Version::supports_avx512dq() ? ExternalAddress(vector_double_signflip()) :
                              ExternalAddress(vector_float_signflip());
    __ vector_castD2X_evex(to_elem_bt, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* xtmp1 */,
                           opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* xtmp2 */, opnd_array(5)->as_KRegister(ra_,this,idx5)/* ktmp1 */, opnd_array(6)->as_KRegister(ra_,this,idx6)/* ktmp2 */, signflip, noreg, vlen_enc);
  
#line 42502 "ad_x86.cpp"
  }
}

void vucastNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 7393 "src/hotspot/cpu/x86/x86.ad"

    assert(UseAVX > 0, "required");

    BasicType from_elem_bt = Matcher::vector_element_basic_type(this, opnd_array(1));
    BasicType to_elem_bt = Matcher::vector_element_basic_type(this);
    int vlen_enc = vector_length_encoding(this);
    __ vector_unsigned_cast(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, vlen_enc, from_elem_bt, to_elem_bt);
  
#line 42523 "ad_x86.cpp"
  }
}

void vucast_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 7393 "src/hotspot/cpu/x86/x86.ad"

    assert(UseAVX > 0, "required");

    BasicType from_elem_bt = Matcher::vector_element_basic_type(this, opnd_array(1));
    BasicType to_elem_bt = Matcher::vector_element_basic_type(this);
    int vlen_enc = vector_length_encoding(this);
    __ vector_unsigned_cast(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, vlen_enc, from_elem_bt, to_elem_bt);
  
#line 42544 "ad_x86.cpp"
  }
}

void vucast_1Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 7393 "src/hotspot/cpu/x86/x86.ad"

    assert(UseAVX > 0, "required");

    BasicType from_elem_bt = Matcher::vector_element_basic_type(this, opnd_array(1));
    BasicType to_elem_bt = Matcher::vector_element_basic_type(this);
    int vlen_enc = vector_length_encoding(this);
    __ vector_unsigned_cast(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, vlen_enc, from_elem_bt, to_elem_bt);
  
#line 42565 "ad_x86.cpp"
  }
}

void vcmpFDNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// cond
  {
    C2_MacroAssembler _masm(&cbuf);

#line 7465 "src/hotspot/cpu/x86/x86.ad"

    int vlen_enc = vector_length_encoding(this, opnd_array(1));
    Assembler::ComparisonPredicateFP cmp = booltest_pred_to_comparison_pred_fp(opnd_array(3)->constant());
    if (Matcher::vector_element_basic_type(this, opnd_array(1)) == T_FLOAT) {
      __ vcmpps(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src1 */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src2 */, cmp, vlen_enc);
    } else {
      __ vcmppd(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src1 */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src2 */, cmp, vlen_enc);
    }
  
#line 42589 "ad_x86.cpp"
  }
}

void evcmpFD64Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// cond
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// ktmp
  {
    C2_MacroAssembler _masm(&cbuf);

#line 7484 "src/hotspot/cpu/x86/x86.ad"

    int vlen_enc = Assembler::AVX_512bit;
    Assembler::ComparisonPredicateFP cmp = booltest_pred_to_comparison_pred_fp(opnd_array(3)->constant());
    KRegister mask = k0; // The comparison itself is not being masked.
    if (Matcher::vector_element_basic_type(this, opnd_array(1)) == T_FLOAT) {
      __ evcmpps(opnd_array(4)->as_KRegister(ra_,this,idx4)/* ktmp */, mask, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src1 */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src2 */, cmp, vlen_enc);
      __ evmovdqul(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(4)->as_KRegister(ra_,this,idx4)/* ktmp */, ExternalAddress(vector_all_bits_set()), false, vlen_enc, noreg);
    } else {
      __ evcmppd(opnd_array(4)->as_KRegister(ra_,this,idx4)/* ktmp */, mask, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src1 */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src2 */, cmp, vlen_enc);
      __ evmovdquq(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(4)->as_KRegister(ra_,this,idx4)/* ktmp */, ExternalAddress(vector_all_bits_set()), false, vlen_enc, noreg);
    }
  
#line 42617 "ad_x86.cpp"
  }
}

void evcmpFDNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// cond
  {
    C2_MacroAssembler _masm(&cbuf);

#line 7504 "src/hotspot/cpu/x86/x86.ad"

    assert(bottom_type()->isa_vectmask(), "TypeVectMask expected");
    int vlen_enc = vector_length_encoding(this, opnd_array(1));
    Assembler::ComparisonPredicateFP cmp = booltest_pred_to_comparison_pred_fp(opnd_array(3)->constant());
    KRegister mask = k0; // The comparison itself is not being masked.
    if (Matcher::vector_element_basic_type(this, opnd_array(1)) == T_FLOAT) {
      __ evcmpps(opnd_array(0)->as_KRegister(ra_,this)/* dst */, mask, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src1 */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src2 */, cmp, vlen_enc);
    } else {
      __ evcmppd(opnd_array(0)->as_KRegister(ra_,this)/* dst */, mask, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src1 */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src2 */, cmp, vlen_enc);
    }
  
#line 42643 "ad_x86.cpp"
  }
}

void vcmp_directNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// cond
  {
    C2_MacroAssembler _masm(&cbuf);

#line 7529 "src/hotspot/cpu/x86/x86.ad"

    int vlen_enc = vector_length_encoding(this, opnd_array(1));
    Assembler::ComparisonPredicate cmp = booltest_pred_to_comparison_pred(opnd_array(3)->constant());
    Assembler::Width ww = widthForType(Matcher::vector_element_basic_type(this, opnd_array(1)));
    __ vpcmpCCW(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src1 */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src2 */, xnoreg, cmp, ww, vlen_enc);
  
#line 42664 "ad_x86.cpp"
  }
}

void vcmp_negateNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// cond
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// dst
  unsigned idx5 = idx4 + opnd_array(4)->num_edges(); 	// xtmp
  {
    C2_MacroAssembler _masm(&cbuf);

#line 7550 "src/hotspot/cpu/x86/x86.ad"

    int vlen_enc = vector_length_encoding(this, opnd_array(1));
    Assembler::ComparisonPredicate cmp = booltest_pred_to_comparison_pred(opnd_array(3)->constant());
    Assembler::Width ww = widthForType(Matcher::vector_element_basic_type(this, opnd_array(1)));
    __ vpcmpCCW(opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src1 */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src2 */, opnd_array(5)->as_XMMRegister(ra_,this,idx5)/* xtmp */, cmp, ww, vlen_enc);
  
#line 42687 "ad_x86.cpp"
  }
}

void vcmpuNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// cond
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// dst
  unsigned idx5 = idx4 + opnd_array(4)->num_edges(); 	// xtmp
  {
    C2_MacroAssembler _masm(&cbuf);

#line 7568 "src/hotspot/cpu/x86/x86.ad"

    InternalAddress flip_bit = InternalAddress(__ code()->consts()->start() + constant_offset());
    int vlen_enc = vector_length_encoding(this, opnd_array(1));
    Assembler::ComparisonPredicate cmp = booltest_pred_to_comparison_pred(opnd_array(3)->constant());
    Assembler::Width ww = widthForType(Matcher::vector_element_basic_type(this, opnd_array(1)));

    if (vlen_enc == Assembler::AVX_128bit) {
      __ vmovddup(opnd_array(5)->as_XMMRegister(ra_,this,idx5)/* xtmp */, flip_bit, vlen_enc, noreg);
    } else {
      __ vbroadcastsd(opnd_array(5)->as_XMMRegister(ra_,this,idx5)/* xtmp */, flip_bit, vlen_enc, noreg);
    }
    __ vpxor(opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* dst */, opnd_array(5)->as_XMMRegister(ra_,this,idx5)/* xtmp */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src1 */, vlen_enc);
    __ vpxor(opnd_array(5)->as_XMMRegister(ra_,this,idx5)/* xtmp */, opnd_array(5)->as_XMMRegister(ra_,this,idx5)/* xtmp */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src2 */, vlen_enc);
    __ vpcmpCCW(opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* dst */, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* dst */, opnd_array(5)->as_XMMRegister(ra_,this,idx5)/* xtmp */, opnd_array(5)->as_XMMRegister(ra_,this,idx5)/* xtmp */, cmp, ww, vlen_enc);
  
#line 42719 "ad_x86.cpp"
  }
}

void vcmpuNode::eval_constant(Compile* C) {
  {

#line 7569 "src/hotspot/cpu/x86/x86.ad"
    _constant = C->output()->constant_table().add(this, high_bit_set(Matcher::vector_element_basic_type(this, opnd_array(1))));
#line 42728 "ad_x86.cpp"
  }
}
void vcmp64Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// cond
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// ktmp
  {
    C2_MacroAssembler _masm(&cbuf);

#line 7593 "src/hotspot/cpu/x86/x86.ad"

    assert(UseAVX > 2, "required");

    int vlen_enc = vector_length_encoding(this, opnd_array(1));
    Assembler::ComparisonPredicate cmp = booltest_pred_to_comparison_pred(opnd_array(3)->constant());
    bool is_unsigned = Matcher::is_unsigned_booltest_pred(opnd_array(3)->constant());
    KRegister mask = k0; // The comparison itself is not being masked.
    bool merge = false;
    BasicType src1_elem_bt = Matcher::vector_element_basic_type(this, opnd_array(1));

    switch (src1_elem_bt) {
      case T_INT: {
        __ evpcmpd(opnd_array(4)->as_KRegister(ra_,this,idx4)/* ktmp */, mask, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src1 */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src2 */, cmp, !is_unsigned, vlen_enc);
        __ evmovdqul(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(4)->as_KRegister(ra_,this,idx4)/* ktmp */, ExternalAddress(vector_all_bits_set()), merge, vlen_enc, noreg);
        break;
      }
      case T_LONG: {
        __ evpcmpq(opnd_array(4)->as_KRegister(ra_,this,idx4)/* ktmp */, mask, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src1 */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src2 */, cmp, !is_unsigned, vlen_enc);
        __ evmovdquq(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(4)->as_KRegister(ra_,this,idx4)/* ktmp */, ExternalAddress(vector_all_bits_set()), merge, vlen_enc, noreg);
        break;
      }
      default: assert(false, "%s", type2name(src1_elem_bt));
    }
  
#line 42767 "ad_x86.cpp"
  }
}

void evcmpNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// cond
  {
    C2_MacroAssembler _masm(&cbuf);

#line 7626 "src/hotspot/cpu/x86/x86.ad"

    assert(UseAVX > 2, "required");
    assert(bottom_type()->isa_vectmask(), "TypeVectMask expected");

    int vlen_enc = vector_length_encoding(this, opnd_array(1));
    Assembler::ComparisonPredicate cmp = booltest_pred_to_comparison_pred(opnd_array(3)->constant());
    bool is_unsigned = Matcher::is_unsigned_booltest_pred(opnd_array(3)->constant());
    BasicType src1_elem_bt = Matcher::vector_element_basic_type(this, opnd_array(1));

    // Comparison i
    switch (src1_elem_bt) {
      case T_BYTE: {
        __ evpcmpb(opnd_array(0)->as_KRegister(ra_,this)/* dst */, k0, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src1 */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src2 */, cmp, !is_unsigned, vlen_enc);
        break;
      }
      case T_SHORT: {
        __ evpcmpw(opnd_array(0)->as_KRegister(ra_,this)/* dst */, k0, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src1 */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src2 */, cmp, !is_unsigned, vlen_enc);
        break;
      }
      case T_INT: {
        __ evpcmpd(opnd_array(0)->as_KRegister(ra_,this)/* dst */, k0, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src1 */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src2 */, cmp, !is_unsigned, vlen_enc);
        break;
      }
      case T_LONG: {
        __ evpcmpq(opnd_array(0)->as_KRegister(ra_,this)/* dst */, k0, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src1 */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src2 */, cmp, !is_unsigned, vlen_enc);
        break;
      }
      default: assert(false, "%s", type2name(src1_elem_bt));
    }
  
#line 42812 "ad_x86.cpp"
  }
}

void extractINode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// idx
  {
    C2_MacroAssembler _masm(&cbuf);

#line 7669 "src/hotspot/cpu/x86/x86.ad"

    assert(opnd_array(2)->constant()< (int)Matcher::vector_length(this, opnd_array(1)), "out of bounds");

    BasicType elem_bt = Matcher::vector_element_basic_type(this, opnd_array(1));
    __ get_elem(elem_bt, opnd_array(0)->as_Register(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, opnd_array(2)->constant());
  
#line 42832 "ad_x86.cpp"
  }
}

void extractI_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// idx
  {
    C2_MacroAssembler _masm(&cbuf);

#line 7669 "src/hotspot/cpu/x86/x86.ad"

    assert(opnd_array(2)->constant()< (int)Matcher::vector_length(this, opnd_array(1)), "out of bounds");

    BasicType elem_bt = Matcher::vector_element_basic_type(this, opnd_array(1));
    __ get_elem(elem_bt, opnd_array(0)->as_Register(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, opnd_array(2)->constant());
  
#line 42852 "ad_x86.cpp"
  }
}

void vextractINode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// idx
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// vtmp
  {
    C2_MacroAssembler _masm(&cbuf);

#line 7688 "src/hotspot/cpu/x86/x86.ad"

    assert(opnd_array(2)->constant()< (int)Matcher::vector_length(this, opnd_array(1)), "out of bounds");

    BasicType elem_bt = Matcher::vector_element_basic_type(this, opnd_array(1));
    XMMRegister lane_xmm = __ get_lane(elem_bt, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* vtmp */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, opnd_array(2)->constant());
    __ get_elem(elem_bt, opnd_array(0)->as_Register(ra_,this)/* dst */, lane_xmm, opnd_array(2)->constant());
  
#line 42874 "ad_x86.cpp"
  }
}

void vextractI_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// idx
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// vtmp
  {
    C2_MacroAssembler _masm(&cbuf);

#line 7688 "src/hotspot/cpu/x86/x86.ad"

    assert(opnd_array(2)->constant()< (int)Matcher::vector_length(this, opnd_array(1)), "out of bounds");

    BasicType elem_bt = Matcher::vector_element_basic_type(this, opnd_array(1));
    XMMRegister lane_xmm = __ get_lane(elem_bt, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* vtmp */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, opnd_array(2)->constant());
    __ get_elem(elem_bt, opnd_array(0)->as_Register(ra_,this)/* dst */, lane_xmm, opnd_array(2)->constant());
  
#line 42896 "ad_x86.cpp"
  }
}

void extractFNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// idx
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// dst
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// vtmp
  {
    C2_MacroAssembler _masm(&cbuf);

#line 7733 "src/hotspot/cpu/x86/x86.ad"

    assert(opnd_array(2)->constant()< (int)Matcher::vector_length(this, opnd_array(1)), "out of bounds");

    __ get_elem(T_FLOAT, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, opnd_array(2)->constant(), opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* vtmp */);
  
#line 42917 "ad_x86.cpp"
  }
}

void vextractFNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// idx
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// vtmp
  {
    C2_MacroAssembler _masm(&cbuf);

#line 7747 "src/hotspot/cpu/x86/x86.ad"

    assert(opnd_array(2)->constant()< (int)Matcher::vector_length(this, opnd_array(1)), "out of bounds");

    XMMRegister lane_reg = __ get_lane(T_FLOAT, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* vtmp */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, opnd_array(2)->constant());
    __ get_elem(T_FLOAT, opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, lane_reg, opnd_array(2)->constant());
  
#line 42938 "ad_x86.cpp"
  }
}

void extractDNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// idx
  {
    C2_MacroAssembler _masm(&cbuf);

#line 7760 "src/hotspot/cpu/x86/x86.ad"

    assert(opnd_array(2)->constant()< (int)Matcher::vector_length(this, opnd_array(1)), "out of bounds");

    __ get_elem(T_DOUBLE, opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, opnd_array(2)->constant());
  
#line 42957 "ad_x86.cpp"
  }
}

void vextractDNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// idx
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// vtmp
  {
    C2_MacroAssembler _masm(&cbuf);

#line 7774 "src/hotspot/cpu/x86/x86.ad"

    assert(opnd_array(2)->constant()< (int)Matcher::vector_length(this, opnd_array(1)), "out of bounds");

    XMMRegister lane_reg = __ get_lane(T_DOUBLE, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* vtmp */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, opnd_array(2)->constant());
    __ get_elem(T_DOUBLE, opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, lane_reg, opnd_array(2)->constant());
  
#line 42978 "ad_x86.cpp"
  }
}

void blendvpNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// mask
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// tmp
  {
    C2_MacroAssembler _masm(&cbuf);

#line 7790 "src/hotspot/cpu/x86/x86.ad"

    assert(UseSSE >= 4, "required");

    if (opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* mask */!= opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* tmp */) {
      __ movdqu(opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* tmp */, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* mask */);
    }
    __ pblendvb(opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* dst */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src */); // uses xmm0 as mask
  
#line 43002 "ad_x86.cpp"
  }
}

void vblendvpINode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// mask
  {
    C2_MacroAssembler _masm(&cbuf);

#line 7808 "src/hotspot/cpu/x86/x86.ad"

    int vlen_enc = vector_length_encoding(this);
    __ vpblendvb(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src1 */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src2 */, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* mask */, vlen_enc);
  
#line 43021 "ad_x86.cpp"
  }
}

void vblendvpFDNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// mask
  {
    C2_MacroAssembler _masm(&cbuf);

#line 7822 "src/hotspot/cpu/x86/x86.ad"

    int vlen_enc = vector_length_encoding(this);
    __ vblendvps(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src1 */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src2 */, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* mask */, vlen_enc);
  
#line 43040 "ad_x86.cpp"
  }
}

void evblendvp64Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// mask
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// ktmp
  {
    C2_MacroAssembler _masm(&cbuf);

#line 7835 "src/hotspot/cpu/x86/x86.ad"

     int vlen_enc = Assembler::AVX_512bit;
     BasicType elem_bt = Matcher::vector_element_basic_type(this);
    __ evpcmp(elem_bt, opnd_array(4)->as_KRegister(ra_,this,idx4)/* ktmp */, k0, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* mask */, ExternalAddress(vector_all_bits_set()), Assembler::eq, vlen_enc, noreg);
    __ evpblend(elem_bt, opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(4)->as_KRegister(ra_,this,idx4)/* ktmp */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src1 */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src2 */, true, vlen_enc);
  
#line 43062 "ad_x86.cpp"
  }
}

void evblendvp64_maskedNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// mask
  {
    C2_MacroAssembler _masm(&cbuf);

#line 7851 "src/hotspot/cpu/x86/x86.ad"

    int vlen_enc = vector_length_encoding(this);
    BasicType elem_bt = Matcher::vector_element_basic_type(this);
    __ evpblend(elem_bt, opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(3)->as_KRegister(ra_,this,idx3)/* mask */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src1 */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src2 */, true, vlen_enc);
  
#line 43082 "ad_x86.cpp"
  }
}

void vabsB_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 7864 "src/hotspot/cpu/x86/x86.ad"

    uint vlen = Matcher::vector_length(this);
    if (vlen <= 16) {
      __ pabsb(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */);
    } else {
      int vlen_enc = vector_length_encoding(this);
      __ vpabsb(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, vlen_enc);
    }
  
#line 43104 "ad_x86.cpp"
  }
}

void vabsS_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 7879 "src/hotspot/cpu/x86/x86.ad"

    uint vlen = Matcher::vector_length(this);
    if (vlen <= 8) {
      __ pabsw(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */);
    } else {
      int vlen_enc = vector_length_encoding(this);
      __ vpabsw(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, vlen_enc);
    }
  
#line 43126 "ad_x86.cpp"
  }
}

void vabsI_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 7894 "src/hotspot/cpu/x86/x86.ad"

    uint vlen = Matcher::vector_length(this);
    if (vlen <= 4) {
      __ pabsd(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */);
    } else {
      int vlen_enc = vector_length_encoding(this);
      __ vpabsd(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, vlen_enc);
    }
  
#line 43148 "ad_x86.cpp"
  }
}

void vabsL_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 7909 "src/hotspot/cpu/x86/x86.ad"

    assert(UseAVX > 2, "required");
    int vlen_enc = vector_length_encoding(this);
    if (!VM_Version::supports_avx512vl()) {
      vlen_enc = Assembler::AVX_512bit;
    }
    __ evpabsq(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, vlen_enc);
  
#line 43169 "ad_x86.cpp"
  }
}

void vabsnegFNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 7928 "src/hotspot/cpu/x86/x86.ad"

    int opcode = this->ideal_Opcode();
    int vlen = Matcher::vector_length(this);
    if (vlen == 2) {
      __ vabsnegf(opcode, opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */);
    } else {
      assert(vlen == 8 || vlen == 16, "required");
      int vlen_enc = vector_length_encoding(this);
      __ vabsnegf(opcode, opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, vlen_enc);
    }
  
#line 43193 "ad_x86.cpp"
  }
}

void vabsnegF_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 7928 "src/hotspot/cpu/x86/x86.ad"

    int opcode = this->ideal_Opcode();
    int vlen = Matcher::vector_length(this);
    if (vlen == 2) {
      __ vabsnegf(opcode, opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */);
    } else {
      assert(vlen == 8 || vlen == 16, "required");
      int vlen_enc = vector_length_encoding(this);
      __ vabsnegf(opcode, opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, vlen_enc);
    }
  
#line 43217 "ad_x86.cpp"
  }
}

void vabsneg4FNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  {
    C2_MacroAssembler _masm(&cbuf);

#line 7948 "src/hotspot/cpu/x86/x86.ad"

    int opcode = this->ideal_Opcode();
    __ vabsnegf(opcode, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* dst */);
  
#line 43234 "ad_x86.cpp"
  }
}

void vabsneg4F_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  {
    C2_MacroAssembler _masm(&cbuf);

#line 7948 "src/hotspot/cpu/x86/x86.ad"

    int opcode = this->ideal_Opcode();
    __ vabsnegf(opcode, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* dst */);
  
#line 43251 "ad_x86.cpp"
  }
}

void vabsnegDNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 7959 "src/hotspot/cpu/x86/x86.ad"

    int opcode = this->ideal_Opcode();
    uint vlen = Matcher::vector_length(this);
    if (vlen == 2) {
      assert(UseSSE >= 2, "required");
      __ vabsnegd(opcode, opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */);
    } else {
      int vlen_enc = vector_length_encoding(this);
      __ vabsnegd(opcode, opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, vlen_enc);
    }
  
#line 43275 "ad_x86.cpp"
  }
}

void vabsnegD_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 7959 "src/hotspot/cpu/x86/x86.ad"

    int opcode = this->ideal_Opcode();
    uint vlen = Matcher::vector_length(this);
    if (vlen == 2) {
      assert(UseSSE >= 2, "required");
      __ vabsnegd(opcode, opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */);
    } else {
      int vlen_enc = vector_length_encoding(this);
      __ vabsnegd(opcode, opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, vlen_enc);
    }
  
#line 43299 "ad_x86.cpp"
  }
}

void loadMaskNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// dst
  {
    C2_MacroAssembler _masm(&cbuf);

#line 8052 "src/hotspot/cpu/x86/x86.ad"

    int vlen_in_bytes = Matcher::vector_length_in_bytes(this);
    BasicType elem_bt = Matcher::vector_element_basic_type(this);
    __ load_vector_mask(opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, vlen_in_bytes, elem_bt, true);
  
#line 43318 "ad_x86.cpp"
  }
}

void loadMask64Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// xtmp
  {
    C2_MacroAssembler _masm(&cbuf);

#line 8065 "src/hotspot/cpu/x86/x86.ad"

    __ load_vector_mask(opnd_array(0)->as_KRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* xtmp */,
                        true, Assembler::AVX_512bit);
  
#line 43336 "ad_x86.cpp"
  }
}

void loadMask_evexNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// xtmp
  {
    C2_MacroAssembler _masm(&cbuf);

#line 8077 "src/hotspot/cpu/x86/x86.ad"

    int vlen_enc = vector_length_encoding(in(1));
    __ load_vector_mask(opnd_array(0)->as_KRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* xtmp */,
                        false, vlen_enc);
  
#line 43355 "ad_x86.cpp"
  }
}

void vstoreMask1BNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// size
  {
    C2_MacroAssembler _masm(&cbuf);

#line 8091 "src/hotspot/cpu/x86/x86.ad"

    int vlen = Matcher::vector_length(this);
    if (vlen <= 16 && UseAVX <= 2) {
      assert(UseSSE >= 3, "required");
      __ pabsb(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */);
    } else {
      assert(UseAVX > 0, "required");
      int src_vlen_enc = vector_length_encoding(this, opnd_array(1));
      __ vpabsb(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, src_vlen_enc);
    }
  
#line 43380 "ad_x86.cpp"
  }
}

void vstoreMask2BNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// size
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// dst
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// xtmp
  {
    C2_MacroAssembler _masm(&cbuf);

#line 8110 "src/hotspot/cpu/x86/x86.ad"

    int vlen_enc = Assembler::AVX_128bit;
    int vlen = Matcher::vector_length(this);
    if (vlen <= 8) {
      assert(UseSSE >= 3, "required");
      __ pxor(opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* xtmp */, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* xtmp */);
      __ pabsw(opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */);
      __ packuswb(opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* xtmp */);
    } else {
      assert(UseAVX > 0, "required");
      __ vextracti128(opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, 0x1);
      __ vpacksswb(opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, vlen_enc);
      __ vpabsb(opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, vlen_enc);
    }
  
#line 43411 "ad_x86.cpp"
  }
}

void vstoreMask4BNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// size
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// dst
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// xtmp
  {
    C2_MacroAssembler _masm(&cbuf);

#line 8133 "src/hotspot/cpu/x86/x86.ad"

    int vlen_enc = Assembler::AVX_128bit;
    int vlen = Matcher::vector_length(this);
    if (vlen <= 4) {
      assert(UseSSE >= 3, "required");
      __ pxor(opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* xtmp */, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* xtmp */);
      __ pabsd(opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */);
      __ packusdw(opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* xtmp */);
      __ packuswb(opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* xtmp */);
    } else {
      assert(UseAVX > 0, "required");
      __ vpxor(opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* xtmp */, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* xtmp */, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* xtmp */, vlen_enc);
      __ vextracti128(opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, 0x1);
      __ vpackssdw(opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, vlen_enc);
      __ vpacksswb(opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* xtmp */, vlen_enc);
      __ vpabsb(opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, vlen_enc);
    }
  
#line 43445 "ad_x86.cpp"
  }
}

void storeMask8BNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// size
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// dst
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// xtmp
  {
    C2_MacroAssembler _masm(&cbuf);

#line 8159 "src/hotspot/cpu/x86/x86.ad"

    assert(UseSSE >= 3, "required");
    __ pxor(opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* xtmp */, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* xtmp */);
    __ pshufd(opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, 0x8);
    __ pabsd(opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */);
    __ packusdw(opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* xtmp */);
    __ packuswb(opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* xtmp */);
  
#line 43469 "ad_x86.cpp"
  }
}

void storeMask8B_avxNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// size
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// dst
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// vtmp
  {
    C2_MacroAssembler _masm(&cbuf);

#line 8175 "src/hotspot/cpu/x86/x86.ad"

    int vlen_enc = Assembler::AVX_128bit;
    __ vshufps(opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, 0x88, Assembler::AVX_256bit);
    __ vextracti128(opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* vtmp */, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, 0x1);
    __ vblendps(opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* vtmp */, 0xC, vlen_enc);
    __ vpxor(opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* vtmp */, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* vtmp */, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* vtmp */, vlen_enc);
    __ vpackssdw(opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* vtmp */, vlen_enc);
    __ vpacksswb(opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* vtmp */, vlen_enc);
    __ vpabsb(opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, vlen_enc);
  
#line 43495 "ad_x86.cpp"
  }
}

void vstoreMask4B_evex_novectmaskNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// size
  {
    C2_MacroAssembler _masm(&cbuf);

#line 8192 "src/hotspot/cpu/x86/x86.ad"

    int src_vlen_enc = vector_length_encoding(this, opnd_array(1));
    int dst_vlen_enc = vector_length_encoding(this);
    if (!VM_Version::supports_avx512vl()) {
      src_vlen_enc = Assembler::AVX_512bit;
    }
    __ evpmovdb(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, src_vlen_enc);
    __ vpabsb(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, dst_vlen_enc);
  
#line 43518 "ad_x86.cpp"
  }
}

void vstoreMask8B_evex_novectmaskNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// size
  {
    C2_MacroAssembler _masm(&cbuf);

#line 8208 "src/hotspot/cpu/x86/x86.ad"

    int src_vlen_enc = vector_length_encoding(this, opnd_array(1));
    int dst_vlen_enc = vector_length_encoding(this);
    if (!VM_Version::supports_avx512vl()) {
      src_vlen_enc = Assembler::AVX_512bit;
    }
    __ evpmovqb(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, src_vlen_enc);
    __ vpabsb(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, dst_vlen_enc);
  
#line 43541 "ad_x86.cpp"
  }
}

void vstoreMask_evex_vectmaskNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// mask
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// size
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// dst
  {
    C2_MacroAssembler _masm(&cbuf);

#line 8225 "src/hotspot/cpu/x86/x86.ad"

    assert(Matcher::vector_length_in_bytes(this, opnd_array(1)) == 64, "");
    __ evmovdqul(opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(1)->as_KRegister(ra_,this,idx1)/* mask */, ExternalAddress(vector_int_mask_cmp_bits()),
                 false, Assembler::AVX_512bit, noreg);
    __ evpmovdb(opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, Assembler::AVX_512bit);
  
#line 43562 "ad_x86.cpp"
  }
}

void vstoreMask_evexNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// mask
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// size
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// dst
  {
    C2_MacroAssembler _masm(&cbuf);

#line 8239 "src/hotspot/cpu/x86/x86.ad"

    int dst_vlen_enc = vector_length_encoding(this);
    __ evpmovm2b(opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(1)->as_KRegister(ra_,this,idx1)/* mask */, dst_vlen_enc);
    __ vpabsb(opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, dst_vlen_enc);
  
#line 43582 "ad_x86.cpp"
  }
}

void vmaskcast_evexNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  {
    C2_MacroAssembler _masm(&cbuf);

#line 8251 "src/hotspot/cpu/x86/x86.ad"

    // empty
  
#line 43598 "ad_x86.cpp"
  }
}

void vmaskcastNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  {
    C2_MacroAssembler _masm(&cbuf);

#line 8262 "src/hotspot/cpu/x86/x86.ad"

    // empty
  
#line 43614 "ad_x86.cpp"
  }
}

void vmaskcast_avxNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 8272 "src/hotspot/cpu/x86/x86.ad"

    int vlen = Matcher::vector_length(this);
    BasicType src_bt = Matcher::vector_element_basic_type(this, opnd_array(1));
    BasicType dst_bt = Matcher::vector_element_basic_type(this);
    __ vector_mask_cast(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, dst_bt, src_bt, vlen);
  
#line 43633 "ad_x86.cpp"
  }
}

void loadIotaIndicesNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 8286 "src/hotspot/cpu/x86/x86.ad"

     int vlen_in_bytes = Matcher::vector_length_in_bytes(this);
     BasicType bt = Matcher::vector_element_basic_type(this);
     __ load_iota_indices(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, vlen_in_bytes, bt);
  
#line 43651 "ad_x86.cpp"
  }
}

void loadShuffleBNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  {
    C2_MacroAssembler _masm(&cbuf);

#line 8335 "src/hotspot/cpu/x86/x86.ad"

    // empty
  
#line 43667 "ad_x86.cpp"
  }
}

void rearrangeBNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// shuffle
  {
    C2_MacroAssembler _masm(&cbuf);

#line 8346 "src/hotspot/cpu/x86/x86.ad"

    assert(UseSSE >= 4, "required");
    __ pshufb(opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* dst */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* shuffle */);
  
#line 43685 "ad_x86.cpp"
  }
}

void rearrangeB_avxNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// shuffle
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// dst
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// vtmp1
  unsigned idx5 = idx4 + opnd_array(4)->num_edges(); 	// vtmp2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 8359 "src/hotspot/cpu/x86/x86.ad"

    assert(UseAVX >= 2, "required");
    // Swap src into vtmp1
    __ vperm2i128(opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* vtmp1 */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, 1);
    // Shuffle swapped src to get entries from other 128 bit lane
    __ vpshufb(opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* vtmp1 */, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* vtmp1 */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* shuffle */, Assembler::AVX_256bit);
    // Shuffle original src to get entries from self 128 bit lane
    __ vpshufb(opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* shuffle */, Assembler::AVX_256bit);
    // Create a blend mask by setting high bits for entries coming from other lane in shuffle
    __ vpaddb(opnd_array(5)->as_XMMRegister(ra_,this,idx5)/* vtmp2 */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* shuffle */, ExternalAddress(vector_byte_shufflemask()), Assembler::AVX_256bit, noreg);
    // Perform the blend
    __ vpblendvb(opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* vtmp1 */, opnd_array(5)->as_XMMRegister(ra_,this,idx5)/* vtmp2 */, Assembler::AVX_256bit);
  
#line 43715 "ad_x86.cpp"
  }
}

void rearrangeB_evexNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// shuffle
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// dst
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// xtmp1
  unsigned idx5 = idx4 + opnd_array(4)->num_edges(); 	// xtmp2
  unsigned idx6 = idx5 + opnd_array(5)->num_edges(); 	// xtmp3
  unsigned idx7 = idx6 + opnd_array(6)->num_edges(); 	// ktmp
  unsigned idx8 = idx7 + opnd_array(7)->num_edges(); 	// rtmp
  {
    C2_MacroAssembler _masm(&cbuf);

#line 8382 "src/hotspot/cpu/x86/x86.ad"

    int vlen_enc = vector_length_encoding(this);
    __ rearrange_bytes(opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* shuffle */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */,
                       opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* xtmp1 */, opnd_array(5)->as_XMMRegister(ra_,this,idx5)/* xtmp2 */, opnd_array(6)->as_XMMRegister(ra_,this,idx6)/* xtmp3 */,
                       opnd_array(8)->as_Register(ra_,this,idx8)/* rtmp */, opnd_array(7)->as_KRegister(ra_,this,idx7)/* ktmp */, vlen_enc);
  
#line 43741 "ad_x86.cpp"
  }
}

void rearrangeB_evex_vbmiNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// shuffle
  {
    C2_MacroAssembler _masm(&cbuf);

#line 8396 "src/hotspot/cpu/x86/x86.ad"

    int vlen_enc = vector_length_encoding(this);
    __ vpermb(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* shuffle */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, vlen_enc);
  
#line 43759 "ad_x86.cpp"
  }
}

void loadShuffleSNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// dst
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// vtmp
  {
    C2_MacroAssembler _masm(&cbuf);

#line 8411 "src/hotspot/cpu/x86/x86.ad"

    // Create a byte shuffle mask from short shuffle mask
    // only byte shuffle instruction available on these platforms
    int vlen_in_bytes = Matcher::vector_length_in_bytes(this);
    if (UseAVX == 0) {
      assert(vlen_in_bytes <= 16, "required");
      // Multiply each shuffle by two to get byte index
      __ pmovzxbw(opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* vtmp */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */);
      __ psllw(opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* vtmp */, 1);

      // Duplicate to create 2 copies of byte index
      __ movdqu(opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* dst */, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* vtmp */);
      __ psllw(opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* dst */, 8);
      __ por(opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* dst */, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* vtmp */);

      // Add one to get alternate byte index
      __ movdqu(opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* vtmp */, ExternalAddress(vector_short_shufflemask()), noreg);
      __ paddb(opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* dst */, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* vtmp */);
    } else {
      assert(UseAVX > 1 || vlen_in_bytes <= 16, "required");
      int vlen_enc = vector_length_encoding(this);
      // Multiply each shuffle by two to get byte index
      __ vpmovzxbw(opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* vtmp */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, vlen_enc);
      __ vpsllw(opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* vtmp */, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* vtmp */, 1, vlen_enc);

      // Duplicate to create 2 copies of byte index
      __ vpsllw(opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* dst */, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* vtmp */,  8, vlen_enc);
      __ vpor(opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* dst */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* dst */, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* vtmp */, vlen_enc);

      // Add one to get alternate byte index
      __ vpaddb(opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* dst */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* dst */, ExternalAddress(vector_short_shufflemask()), vlen_enc, noreg);
    }
  
#line 43807 "ad_x86.cpp"
  }
}

void rearrangeSNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// shuffle
  {
    C2_MacroAssembler _masm(&cbuf);

#line 8452 "src/hotspot/cpu/x86/x86.ad"

    assert(UseSSE >= 4, "required");
    __ pshufb(opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* dst */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* shuffle */);
  
#line 43825 "ad_x86.cpp"
  }
}

void rearrangeS_avxNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// shuffle
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// dst
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// vtmp1
  unsigned idx5 = idx4 + opnd_array(4)->num_edges(); 	// vtmp2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 8465 "src/hotspot/cpu/x86/x86.ad"

    assert(UseAVX >= 2, "required");
    // Swap src into vtmp1
    __ vperm2i128(opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* vtmp1 */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, 1);
    // Shuffle swapped src to get entries from other 128 bit lane
    __ vpshufb(opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* vtmp1 */, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* vtmp1 */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* shuffle */, Assembler::AVX_256bit);
    // Shuffle original src to get entries from self 128 bit lane
    __ vpshufb(opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* shuffle */, Assembler::AVX_256bit);
    // Create a blend mask by setting high bits for entries coming from other lane in shuffle
    __ vpaddb(opnd_array(5)->as_XMMRegister(ra_,this,idx5)/* vtmp2 */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* shuffle */, ExternalAddress(vector_byte_shufflemask()), Assembler::AVX_256bit, noreg);
    // Perform the blend
    __ vpblendvb(opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* vtmp1 */, opnd_array(5)->as_XMMRegister(ra_,this,idx5)/* vtmp2 */, Assembler::AVX_256bit);
  
#line 43855 "ad_x86.cpp"
  }
}

void loadShuffleS_evexNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 8486 "src/hotspot/cpu/x86/x86.ad"

    int vlen_enc = vector_length_encoding(this);
    if (!VM_Version::supports_avx512vl()) {
      vlen_enc = Assembler::AVX_512bit;
    }
    __ vpmovzxbw(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, vlen_enc);
  
#line 43875 "ad_x86.cpp"
  }
}

void rearrangeS_evexNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// shuffle
  {
    C2_MacroAssembler _masm(&cbuf);

#line 8501 "src/hotspot/cpu/x86/x86.ad"

    int vlen_enc = vector_length_encoding(this);
    if (!VM_Version::supports_avx512vl()) {
      vlen_enc = Assembler::AVX_512bit;
    }
    __ vpermw(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* shuffle */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, vlen_enc);
  
#line 43896 "ad_x86.cpp"
  }
}

void loadShuffleINode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// dst
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// vtmp
  {
    C2_MacroAssembler _masm(&cbuf);

#line 8519 "src/hotspot/cpu/x86/x86.ad"

    assert(UseSSE >= 4, "required");

    // Create a byte shuffle mask from int shuffle mask
    // only byte shuffle instruction available on these platforms

    // Duplicate and multiply each shuffle by 4
    __ pmovzxbd(opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* vtmp */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */);
    __ pshuflw(opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* vtmp */, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* vtmp */, 0xA0);
    __ pshufhw(opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* vtmp */, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* vtmp */, 0xA0);
    __ psllw(opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* vtmp */, 2);

    // Duplicate again to create 4 copies of byte index
    __ movdqu(opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* dst */, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* vtmp */);
    __ psllw(opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* dst */, 8);
    __ por(opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* vtmp */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* dst */);

    // Add 3,2,1,0 to get alternate byte index
    __ movdqu(opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* dst */, ExternalAddress(vector_int_shufflemask()), noreg);
    __ paddb(opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* dst */, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* vtmp */);
  
#line 43932 "ad_x86.cpp"
  }
}

void rearrangeINode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// shuffle
  {
    C2_MacroAssembler _masm(&cbuf);

#line 8548 "src/hotspot/cpu/x86/x86.ad"

    assert(UseSSE >= 4, "required");
    __ pshufb(opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* dst */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* shuffle */);
  
#line 43950 "ad_x86.cpp"
  }
}

void loadShuffleI_avxNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 8560 "src/hotspot/cpu/x86/x86.ad"

    int vlen_enc = vector_length_encoding(this);
    __ vpmovzxbd(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, vlen_enc);
  
#line 43967 "ad_x86.cpp"
  }
}

void rearrangeI_avxNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// shuffle
  {
    C2_MacroAssembler _masm(&cbuf);

#line 8572 "src/hotspot/cpu/x86/x86.ad"

    int vlen_enc = vector_length_encoding(this);
    BasicType bt = Matcher::vector_element_basic_type(this);
    __ vector_rearrange_int_float(bt, opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* shuffle */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, vlen_enc);
  
#line 43986 "ad_x86.cpp"
  }
}

void loadShuffleLNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// dst
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// vtmp
  {
    C2_MacroAssembler _masm(&cbuf);

#line 8588 "src/hotspot/cpu/x86/x86.ad"

    assert(UseAVX >= 2, "required");

    int vlen_enc = vector_length_encoding(this);
    // Create a double word shuffle mask from long shuffle mask
    // only double word shuffle instruction available on these platforms

    // Multiply each shuffle by two to get double word index
    __ vpmovzxbq(opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* vtmp */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, vlen_enc);
    __ vpsllq(opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* vtmp */, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* vtmp */, 1, vlen_enc);

    // Duplicate each double word shuffle
    __ vpsllq(opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* dst */, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* vtmp */, 32, vlen_enc);
    __ vpor(opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* dst */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* dst */, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* vtmp */, vlen_enc);

    // Add one to get alternate double word index
    __ vpaddd(opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* dst */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* dst */, ExternalAddress(vector_long_shufflemask()), vlen_enc, noreg);
  
#line 44019 "ad_x86.cpp"
  }
}

void rearrangeLNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// shuffle
  {
    C2_MacroAssembler _masm(&cbuf);

#line 8614 "src/hotspot/cpu/x86/x86.ad"

    assert(UseAVX >= 2, "required");

    int vlen_enc = vector_length_encoding(this);
    __ vpermd(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* shuffle */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, vlen_enc);
  
#line 44039 "ad_x86.cpp"
  }
}

void loadShuffleL_evexNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 8628 "src/hotspot/cpu/x86/x86.ad"

    assert(UseAVX > 2, "required");

    int vlen_enc = vector_length_encoding(this);
    __ vpmovzxbq(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, vlen_enc);
  
#line 44058 "ad_x86.cpp"
  }
}

void rearrangeL_evexNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// shuffle
  {
    C2_MacroAssembler _masm(&cbuf);

#line 8642 "src/hotspot/cpu/x86/x86.ad"

    assert(UseAVX > 2, "required");

    int vlen_enc = vector_length_encoding(this);
    if (vlen_enc == Assembler::AVX_128bit) {
      vlen_enc = Assembler::AVX_256bit;
    }
    __ vpermq(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* shuffle */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, vlen_enc);
  
#line 44081 "ad_x86.cpp"
  }
}

void vfmaF_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// c
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// a
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// b
  {
    C2_MacroAssembler _masm(&cbuf);

#line 8661 "src/hotspot/cpu/x86/x86.ad"

    assert(UseFMA, "not enabled");
    int vlen_enc = vector_length_encoding(this);
    __ vfmaf(opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* c */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* a */, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* b */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* c */, vlen_enc);
  
#line 44101 "ad_x86.cpp"
  }
}

void vfmaF_memNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// c
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// a
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// b
  {
    C2_MacroAssembler _masm(&cbuf);

#line 8674 "src/hotspot/cpu/x86/x86.ad"

    assert(UseFMA, "not enabled");
    int vlen_enc = vector_length_encoding(this);
    __ vfmaf(opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* c */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* a */, Address::make_raw(opnd_array(3)->base(ra_,this,idx3), opnd_array(3)->index(ra_,this,idx3), opnd_array(3)->scale(), opnd_array(3)->disp(ra_,this,idx3), opnd_array(3)->disp_reloc()), opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* c */, vlen_enc);
  
#line 44121 "ad_x86.cpp"
  }
}

void vfmaD_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// c
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// a
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// b
  {
    C2_MacroAssembler _masm(&cbuf);

#line 8686 "src/hotspot/cpu/x86/x86.ad"

    assert(UseFMA, "not enabled");
    int vlen_enc = vector_length_encoding(this);
    __ vfmad(opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* c */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* a */, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* b */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* c */, vlen_enc);
  
#line 44141 "ad_x86.cpp"
  }
}

void vfmaD_memNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// c
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// a
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// b
  {
    C2_MacroAssembler _masm(&cbuf);

#line 8699 "src/hotspot/cpu/x86/x86.ad"

    assert(UseFMA, "not enabled");
    int vlen_enc = vector_length_encoding(this);
    __ vfmad(opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* c */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* a */, Address::make_raw(opnd_array(3)->base(ra_,this,idx3), opnd_array(3)->index(ra_,this,idx3), opnd_array(3)->scale(), opnd_array(3)->disp(ra_,this,idx3), opnd_array(3)->disp_reloc()), opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* c */, vlen_enc);
  
#line 44161 "ad_x86.cpp"
  }
}

void vmuladdS2I_reg_sseNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src1
  {
    C2_MacroAssembler _masm(&cbuf);

#line 8713 "src/hotspot/cpu/x86/x86.ad"

    __ pmaddwd(opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* dst */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src1 */);
  
#line 44178 "ad_x86.cpp"
  }
}

void vmuladdS2I_reg_avxNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 8723 "src/hotspot/cpu/x86/x86.ad"

    int vlen_enc = vector_length_encoding(this);
    __ vpmaddwd(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src1 */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src2 */, vlen_enc);
  
#line 44196 "ad_x86.cpp"
  }
}

void vmuladdaddS2I_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// dst
  {
    C2_MacroAssembler _masm(&cbuf);

#line 8736 "src/hotspot/cpu/x86/x86.ad"

    assert(UseAVX > 2, "required");
    int vlen_enc = vector_length_encoding(this);
    __ evpdpwssd(opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src1 */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src2 */, vlen_enc);
  
#line 44216 "ad_x86.cpp"
  }
}

void vmuladdaddS2I_reg_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src1
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// src2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 8736 "src/hotspot/cpu/x86/x86.ad"

    assert(UseAVX > 2, "required");
    int vlen_enc = vector_length_encoding(this);
    __ evpdpwssd(opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* dst */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src1 */, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* src2 */, vlen_enc);
  
#line 44236 "ad_x86.cpp"
  }
}

void vpopcount_integral_reg_evexNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 8752 "src/hotspot/cpu/x86/x86.ad"

    int opcode = this->ideal_Opcode();
    int vlen_enc = vector_length_encoding(this, opnd_array(1));
    BasicType bt = Matcher::vector_element_basic_type(this, opnd_array(1));
    __ vector_popcount_integral_evex(bt, opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, k0, true, vlen_enc);
  
#line 44255 "ad_x86.cpp"
  }
}

void vpopcount_integral_reg_evex_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 8752 "src/hotspot/cpu/x86/x86.ad"

    int opcode = this->ideal_Opcode();
    int vlen_enc = vector_length_encoding(this, opnd_array(1));
    BasicType bt = Matcher::vector_element_basic_type(this, opnd_array(1));
    __ vector_popcount_integral_evex(bt, opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, k0, true, vlen_enc);
  
#line 44274 "ad_x86.cpp"
  }
}

void vpopcount_integral_reg_evex_maskedNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// mask
  {
    C2_MacroAssembler _masm(&cbuf);

#line 8766 "src/hotspot/cpu/x86/x86.ad"

    int vlen_enc = vector_length_encoding(this, opnd_array(1));
    BasicType bt = Matcher::vector_element_basic_type(this, opnd_array(1));
    __ evmovdquq(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, vlen_enc);
    __ vector_popcount_integral_evex(bt, opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, opnd_array(2)->as_KRegister(ra_,this,idx2)/* mask */, true, vlen_enc);
  
#line 44294 "ad_x86.cpp"
  }
}

void vpopcount_integral_reg_evex_masked_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// mask
  {
    C2_MacroAssembler _masm(&cbuf);

#line 8766 "src/hotspot/cpu/x86/x86.ad"

    int vlen_enc = vector_length_encoding(this, opnd_array(1));
    BasicType bt = Matcher::vector_element_basic_type(this, opnd_array(1));
    __ evmovdquq(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, vlen_enc);
    __ vector_popcount_integral_evex(bt, opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, opnd_array(2)->as_KRegister(ra_,this,idx2)/* mask */, true, vlen_enc);
  
#line 44314 "ad_x86.cpp"
  }
}

void vpopcount_avx_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// dst
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// xtmp1
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// xtmp2
  unsigned idx5 = idx4 + opnd_array(4)->num_edges(); 	// rtmp
  {
    C2_MacroAssembler _masm(&cbuf);

#line 8781 "src/hotspot/cpu/x86/x86.ad"

    int opcode = this->ideal_Opcode();
    int vlen_enc = vector_length_encoding(this, opnd_array(1));
    BasicType bt = Matcher::vector_element_basic_type(this, opnd_array(1));
    __ vector_popcount_integral(bt, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* xtmp1 */,
                                opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* xtmp2 */, opnd_array(5)->as_Register(ra_,this,idx5)/* rtmp */, vlen_enc);
  
#line 44338 "ad_x86.cpp"
  }
}

void vpopcount_avx_reg_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// dst
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// xtmp1
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// xtmp2
  unsigned idx5 = idx4 + opnd_array(4)->num_edges(); 	// rtmp
  {
    C2_MacroAssembler _masm(&cbuf);

#line 8781 "src/hotspot/cpu/x86/x86.ad"

    int opcode = this->ideal_Opcode();
    int vlen_enc = vector_length_encoding(this, opnd_array(1));
    BasicType bt = Matcher::vector_element_basic_type(this, opnd_array(1));
    __ vector_popcount_integral(bt, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* xtmp1 */,
                                opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* xtmp2 */, opnd_array(5)->as_Register(ra_,this,idx5)/* rtmp */, vlen_enc);
  
#line 44362 "ad_x86.cpp"
  }
}

void vcount_trailing_zeros_reg_evexNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// dst
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// xtmp
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// rtmp
  {
    C2_MacroAssembler _masm(&cbuf);

#line 8800 "src/hotspot/cpu/x86/x86.ad"

    int vlen_enc = vector_length_encoding(this, opnd_array(1));
    BasicType bt = Matcher::vector_element_basic_type(this, opnd_array(1));
    __ vector_count_trailing_zeros_evex(bt, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, xnoreg,
                                        xnoreg, xnoreg, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* xtmp */, k0, opnd_array(4)->as_Register(ra_,this,idx4)/* rtmp */, vlen_enc);
  
#line 44384 "ad_x86.cpp"
  }
}

void vcount_trailing_zeros_short_reg_evexNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// dst
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// xtmp1
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// xtmp2
  unsigned idx5 = idx4 + opnd_array(4)->num_edges(); 	// xtmp3
  unsigned idx6 = idx5 + opnd_array(5)->num_edges(); 	// rtmp
  {
    C2_MacroAssembler _masm(&cbuf);

#line 8817 "src/hotspot/cpu/x86/x86.ad"

    int vlen_enc = vector_length_encoding(this, opnd_array(1));
    BasicType bt = Matcher::vector_element_basic_type(this, opnd_array(1));
    __ vector_count_trailing_zeros_evex(bt, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* xtmp1 */,
                                        opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* xtmp2 */, xnoreg, opnd_array(5)->as_XMMRegister(ra_,this,idx5)/* xtmp3 */, k0, opnd_array(6)->as_Register(ra_,this,idx6)/* rtmp */, vlen_enc);
  
#line 44408 "ad_x86.cpp"
  }
}

void vcount_trailing_zeros_byte_reg_evexNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// dst
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// xtmp1
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// xtmp2
  unsigned idx5 = idx4 + opnd_array(4)->num_edges(); 	// xtmp3
  unsigned idx6 = idx5 + opnd_array(5)->num_edges(); 	// xtmp4
  unsigned idx7 = idx6 + opnd_array(6)->num_edges(); 	// ktmp
  unsigned idx8 = idx7 + opnd_array(7)->num_edges(); 	// rtmp
  {
    C2_MacroAssembler _masm(&cbuf);

#line 8832 "src/hotspot/cpu/x86/x86.ad"

    int vlen_enc = vector_length_encoding(this, opnd_array(1));
    BasicType bt = Matcher::vector_element_basic_type(this, opnd_array(1));
    __ vector_count_trailing_zeros_evex(bt, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* xtmp1 */,
                                        opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* xtmp2 */, opnd_array(5)->as_XMMRegister(ra_,this,idx5)/* xtmp3 */, opnd_array(6)->as_XMMRegister(ra_,this,idx6)/* xtmp4 */,
                                        opnd_array(7)->as_KRegister(ra_,this,idx7)/* ktmp */, opnd_array(8)->as_Register(ra_,this,idx8)/* rtmp */, vlen_enc);
  
#line 44435 "ad_x86.cpp"
  }
}

void vcount_trailing_zeros_reg_avxNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// dst
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// xtmp1
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// xtmp2
  unsigned idx5 = idx4 + opnd_array(4)->num_edges(); 	// xtmp3
  unsigned idx6 = idx5 + opnd_array(5)->num_edges(); 	// rtmp
  {
    C2_MacroAssembler _masm(&cbuf);

#line 8847 "src/hotspot/cpu/x86/x86.ad"

    int vlen_enc = vector_length_encoding(this, opnd_array(1));
    BasicType bt = Matcher::vector_element_basic_type(this, opnd_array(1));
    __ vector_count_trailing_zeros_avx(bt, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* xtmp1 */,
                                       opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* xtmp2 */, opnd_array(5)->as_XMMRegister(ra_,this,idx5)/* xtmp3 */, opnd_array(6)->as_Register(ra_,this,idx6)/* rtmp */, vlen_enc);
  
#line 44459 "ad_x86.cpp"
  }
}

void vpternlogNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// src3
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// func
  {
    C2_MacroAssembler _masm(&cbuf);

#line 8863 "src/hotspot/cpu/x86/x86.ad"

    int vector_len = vector_length_encoding(this);
    __ vpternlogd(opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* dst */, opnd_array(4)->constant(), opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src2 */, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* src3 */, vector_len);
  
#line 44479 "ad_x86.cpp"
  }
}

void vpternlog_memNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// src3
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// func
  {
    C2_MacroAssembler _masm(&cbuf);

#line 8875 "src/hotspot/cpu/x86/x86.ad"

    int vector_len = vector_length_encoding(this);
    __ vpternlogd(opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* dst */, opnd_array(4)->constant(), opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src2 */, Address::make_raw(opnd_array(3)->base(ra_,this,idx3), opnd_array(3)->index(ra_,this,idx3), opnd_array(3)->scale(), opnd_array(3)->disp(ra_,this,idx3), opnd_array(3)->disp_reloc()), vector_len);
  
#line 44499 "ad_x86.cpp"
  }
}

void vprotate_immI8Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// shift
  {
    C2_MacroAssembler _masm(&cbuf);

#line 8887 "src/hotspot/cpu/x86/x86.ad"

    int opcode      = this->ideal_Opcode();
    int vector_len  = vector_length_encoding(this);
    BasicType etype = this->bottom_type()->is_vect()->element_basic_type();
    __ vprotate_imm(opcode, etype, opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, opnd_array(2)->constant(), vector_len);
  
#line 44519 "ad_x86.cpp"
  }
}

void vprotate_immI8_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// shift
  {
    C2_MacroAssembler _masm(&cbuf);

#line 8887 "src/hotspot/cpu/x86/x86.ad"

    int opcode      = this->ideal_Opcode();
    int vector_len  = vector_length_encoding(this);
    BasicType etype = this->bottom_type()->is_vect()->element_basic_type();
    __ vprotate_imm(opcode, etype, opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, opnd_array(2)->constant(), vector_len);
  
#line 44539 "ad_x86.cpp"
  }
}

void vprorateNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// shift
  {
    C2_MacroAssembler _masm(&cbuf);

#line 8900 "src/hotspot/cpu/x86/x86.ad"

    int opcode      = this->ideal_Opcode();
    int vector_len  = vector_length_encoding(this);
    BasicType etype = this->bottom_type()->is_vect()->element_basic_type();
    __ vprotate_var(opcode, etype, opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* shift */, vector_len);
  
#line 44559 "ad_x86.cpp"
  }
}

void vprorate_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// shift
  {
    C2_MacroAssembler _masm(&cbuf);

#line 8900 "src/hotspot/cpu/x86/x86.ad"

    int opcode      = this->ideal_Opcode();
    int vector_len  = vector_length_encoding(this);
    BasicType etype = this->bottom_type()->is_vect()->element_basic_type();
    __ vprotate_var(opcode, etype, opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* shift */, vector_len);
  
#line 44579 "ad_x86.cpp"
  }
}

void vmasked_load_avx_non_subwordNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// mask
  {
    C2_MacroAssembler _masm(&cbuf);

#line 8914 "src/hotspot/cpu/x86/x86.ad"

    BasicType elmType = this->bottom_type()->is_vect()->element_basic_type();
    int vlen_enc = vector_length_encoding(this);
    __ vmovmask(elmType, opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, Address::make_raw(opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1), opnd_array(1)->disp_reloc()), opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* mask */, vlen_enc);
  
#line 44598 "ad_x86.cpp"
  }
}

void vmasked_load_evexNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// mask
  {
    C2_MacroAssembler _masm(&cbuf);

#line 8927 "src/hotspot/cpu/x86/x86.ad"

    BasicType elmType =  this->bottom_type()->is_vect()->element_basic_type();
    int vector_len = vector_length_encoding(this);
    __ evmovdqu(elmType, opnd_array(2)->as_KRegister(ra_,this,idx2)/* mask */, opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, Address::make_raw(opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1), opnd_array(1)->disp_reloc()), false, vector_len);
  
#line 44617 "ad_x86.cpp"
  }
}

void vmasked_store_avx_non_subwordNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// mask
  {
    C2_MacroAssembler _masm(&cbuf);

#line 8939 "src/hotspot/cpu/x86/x86.ad"

    const MachNode* src_node = static_cast<const MachNode*>(this->in(this->operand_index(opnd_array(2))));
    int vlen_enc = vector_length_encoding(src_node);
    BasicType elmType =  src_node->bottom_type()->is_vect()->element_basic_type();
    __ vmovmask(elmType, Address::make_raw(opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1), opnd_array(1)->disp_reloc()), opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src */, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* mask */, vlen_enc);
  
#line 44638 "ad_x86.cpp"
  }
}

void vmasked_store_evexNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// mask
  {
    C2_MacroAssembler _masm(&cbuf);

#line 8952 "src/hotspot/cpu/x86/x86.ad"

    const MachNode* src_node = static_cast<const MachNode*>(this->in(this->operand_index(opnd_array(2))));
    BasicType elmType =  src_node->bottom_type()->is_vect()->element_basic_type();
    int vlen_enc = vector_length_encoding(src_node);
    __ evmovdqu(elmType, opnd_array(3)->as_KRegister(ra_,this,idx3)/* mask */, Address::make_raw(opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1), opnd_array(1)->disp_reloc()), opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src */, true, vlen_enc);
  
#line 44659 "ad_x86.cpp"
  }
}

void vreverse_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// dst
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// xtmp1
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// xtmp2
  unsigned idx5 = idx4 + opnd_array(4)->num_edges(); 	// rtmp
  {
    C2_MacroAssembler _masm(&cbuf);

#line 9195 "src/hotspot/cpu/x86/x86.ad"

    int vec_enc = vector_length_encoding(this);
    BasicType bt = Matcher::vector_element_basic_type(this);
    __ vector_reverse_bit(bt, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* xtmp1 */,
                          opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* xtmp2 */, opnd_array(5)->as_Register(ra_,this,idx5)/* rtmp */, vec_enc);
  
#line 44682 "ad_x86.cpp"
  }
}

void vreverse_reg_gfniNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// dst
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// xtmp
  {
    C2_MacroAssembler _masm(&cbuf);

#line 9209 "src/hotspot/cpu/x86/x86.ad"

    int vec_enc = vector_length_encoding(this);
    BasicType bt  = Matcher::vector_element_basic_type(this);
    InternalAddress addr = InternalAddress(__ code()->consts()->start() + constant_offset());
    __ vector_reverse_bit_gfni(bt, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, addr, vec_enc,
                               opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* xtmp */);
  
#line 44704 "ad_x86.cpp"
  }
}

void vreverse_reg_gfniNode::eval_constant(Compile* C) {
  {

#line 9212 "src/hotspot/cpu/x86/x86.ad"
    _constant = C->output()->constant_table().add(this, T_LONG, vreplicate_imm(T_LONG, 0x8040201008040201L, 1));
#line 44713 "ad_x86.cpp"
  }
}
void vreverse_byte_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// dst
  {
    C2_MacroAssembler _masm(&cbuf);

#line 9224 "src/hotspot/cpu/x86/x86.ad"

    int vec_enc = vector_length_encoding(this);
    BasicType bt = Matcher::vector_element_basic_type(this);
    __ vector_reverse_byte(bt, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, vec_enc);
  
#line 44731 "ad_x86.cpp"
  }
}

void vreverse_byte64_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// dst
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// xtmp1
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// xtmp2
  unsigned idx5 = idx4 + opnd_array(4)->num_edges(); 	// rtmp
  {
    C2_MacroAssembler _masm(&cbuf);

#line 9237 "src/hotspot/cpu/x86/x86.ad"

    int vec_enc = vector_length_encoding(this);
    BasicType bt = Matcher::vector_element_basic_type(this);
    __ vector_reverse_byte64(bt, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* xtmp1 */,
                             opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* xtmp2 */, opnd_array(5)->as_Register(ra_,this,idx5)/* rtmp */, vec_enc);
  
#line 44754 "ad_x86.cpp"
  }
}

void vcount_leading_zeros_IL_reg_evexNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 9253 "src/hotspot/cpu/x86/x86.ad"

     int vlen_enc = vector_length_encoding(this, opnd_array(1));
     BasicType bt = Matcher::vector_element_basic_type(this, opnd_array(1));
     __ vector_count_leading_zeros_evex(bt, opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, xnoreg,
                                        xnoreg, xnoreg, k0, noreg, true, vlen_enc);
  
#line 44773 "ad_x86.cpp"
  }
}

void vcount_leading_zeros_IL_reg_evex_maskedNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// mask
  {
    C2_MacroAssembler _masm(&cbuf);

#line 9267 "src/hotspot/cpu/x86/x86.ad"

    int vlen_enc = vector_length_encoding(this, opnd_array(1));
    BasicType bt = Matcher::vector_element_basic_type(this, opnd_array(1));
    __ evmovdquq(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, vlen_enc);
    __ vector_count_leading_zeros_evex(bt, opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, xnoreg, xnoreg,
                                       xnoreg, opnd_array(2)->as_KRegister(ra_,this,idx2)/* mask */, noreg, true, vlen_enc);
  
#line 44794 "ad_x86.cpp"
  }
}

void vcount_leading_zeros_short_reg_evexNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// dst
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// xtmp1
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// xtmp2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 9284 "src/hotspot/cpu/x86/x86.ad"

    int vlen_enc = vector_length_encoding(this, opnd_array(1));
    BasicType bt = Matcher::vector_element_basic_type(this, opnd_array(1));
    __ vector_count_leading_zeros_evex(bt, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* xtmp1 */,
                                       opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* xtmp2 */, xnoreg, k0, noreg, true, vlen_enc);
  
#line 44816 "ad_x86.cpp"
  }
}

void vcount_leading_zeros_byte_reg_evexNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// dst
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// xtmp1
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// xtmp2
  unsigned idx5 = idx4 + opnd_array(4)->num_edges(); 	// xtmp3
  unsigned idx6 = idx5 + opnd_array(5)->num_edges(); 	// ktmp
  unsigned idx7 = idx6 + opnd_array(6)->num_edges(); 	// rtmp
  {
    C2_MacroAssembler _masm(&cbuf);

#line 9298 "src/hotspot/cpu/x86/x86.ad"

    int vlen_enc = vector_length_encoding(this, opnd_array(1));
    BasicType bt = Matcher::vector_element_basic_type(this, opnd_array(1));
    __ vector_count_leading_zeros_evex(bt, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* xtmp1 */,
                                       opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* xtmp2 */, opnd_array(5)->as_XMMRegister(ra_,this,idx5)/* xtmp3 */, opnd_array(6)->as_KRegister(ra_,this,idx6)/* ktmp */,
                                       opnd_array(7)->as_Register(ra_,this,idx7)/* rtmp */, true, vlen_enc);
  
#line 44842 "ad_x86.cpp"
  }
}

void vcount_leading_zeros_int_reg_avxNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// dst
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// xtmp1
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// xtmp2
  unsigned idx5 = idx4 + opnd_array(4)->num_edges(); 	// xtmp3
  {
    C2_MacroAssembler _masm(&cbuf);

#line 9314 "src/hotspot/cpu/x86/x86.ad"

    int vlen_enc = vector_length_encoding(this, opnd_array(1));
    BasicType bt = Matcher::vector_element_basic_type(this, opnd_array(1));
    __ vector_count_leading_zeros_avx(bt, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* xtmp1 */,
                                      opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* xtmp2 */, opnd_array(5)->as_XMMRegister(ra_,this,idx5)/* xtmp3 */, noreg, vlen_enc);
  
#line 44865 "ad_x86.cpp"
  }
}

void vcount_leading_zeros_reg_avxNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// dst
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// xtmp1
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// xtmp2
  unsigned idx5 = idx4 + opnd_array(4)->num_edges(); 	// xtmp3
  unsigned idx6 = idx5 + opnd_array(5)->num_edges(); 	// rtmp
  {
    C2_MacroAssembler _masm(&cbuf);

#line 9329 "src/hotspot/cpu/x86/x86.ad"

    int vlen_enc = vector_length_encoding(this, opnd_array(1));
    BasicType bt = Matcher::vector_element_basic_type(this, opnd_array(1));
    __ vector_count_leading_zeros_avx(bt, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* xtmp1 */,
                                      opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* xtmp2 */, opnd_array(5)->as_XMMRegister(ra_,this,idx5)/* xtmp3 */, opnd_array(6)->as_Register(ra_,this,idx6)/* rtmp */, vlen_enc);
  
#line 44889 "ad_x86.cpp"
  }
}

void vadd_reg_maskedNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// mask
  {
    C2_MacroAssembler _masm(&cbuf);

#line 9348 "src/hotspot/cpu/x86/x86.ad"

    int vlen_enc = vector_length_encoding(this);
    BasicType bt = Matcher::vector_element_basic_type(this);
    int opc = this->ideal_Opcode();
    __ evmasked_op(opc, bt, opnd_array(3)->as_KRegister(ra_,this,idx3)/* mask */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* dst */,
                   opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* dst */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src2 */, true, vlen_enc);
  
#line 44911 "ad_x86.cpp"
  }
}

void vadd_reg_masked_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// mask
  {
    C2_MacroAssembler _masm(&cbuf);

#line 9348 "src/hotspot/cpu/x86/x86.ad"

    int vlen_enc = vector_length_encoding(this);
    BasicType bt = Matcher::vector_element_basic_type(this);
    int opc = this->ideal_Opcode();
    __ evmasked_op(opc, bt, opnd_array(3)->as_KRegister(ra_,this,idx3)/* mask */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* dst */,
                   opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* dst */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src2 */, true, vlen_enc);
  
#line 44933 "ad_x86.cpp"
  }
}

void vadd_reg_masked_1Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// mask
  {
    C2_MacroAssembler _masm(&cbuf);

#line 9348 "src/hotspot/cpu/x86/x86.ad"

    int vlen_enc = vector_length_encoding(this);
    BasicType bt = Matcher::vector_element_basic_type(this);
    int opc = this->ideal_Opcode();
    __ evmasked_op(opc, bt, opnd_array(3)->as_KRegister(ra_,this,idx3)/* mask */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* dst */,
                   opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* dst */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src2 */, true, vlen_enc);
  
#line 44955 "ad_x86.cpp"
  }
}

void vadd_reg_masked_2Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// mask
  {
    C2_MacroAssembler _masm(&cbuf);

#line 9348 "src/hotspot/cpu/x86/x86.ad"

    int vlen_enc = vector_length_encoding(this);
    BasicType bt = Matcher::vector_element_basic_type(this);
    int opc = this->ideal_Opcode();
    __ evmasked_op(opc, bt, opnd_array(3)->as_KRegister(ra_,this,idx3)/* mask */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* dst */,
                   opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* dst */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src2 */, true, vlen_enc);
  
#line 44977 "ad_x86.cpp"
  }
}

void vadd_reg_masked_3Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// mask
  {
    C2_MacroAssembler _masm(&cbuf);

#line 9348 "src/hotspot/cpu/x86/x86.ad"

    int vlen_enc = vector_length_encoding(this);
    BasicType bt = Matcher::vector_element_basic_type(this);
    int opc = this->ideal_Opcode();
    __ evmasked_op(opc, bt, opnd_array(3)->as_KRegister(ra_,this,idx3)/* mask */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* dst */,
                   opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* dst */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src2 */, true, vlen_enc);
  
#line 44999 "ad_x86.cpp"
  }
}

void vadd_reg_masked_4Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// mask
  {
    C2_MacroAssembler _masm(&cbuf);

#line 9348 "src/hotspot/cpu/x86/x86.ad"

    int vlen_enc = vector_length_encoding(this);
    BasicType bt = Matcher::vector_element_basic_type(this);
    int opc = this->ideal_Opcode();
    __ evmasked_op(opc, bt, opnd_array(3)->as_KRegister(ra_,this,idx3)/* mask */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* dst */,
                   opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* dst */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src2 */, true, vlen_enc);
  
#line 45021 "ad_x86.cpp"
  }
}

void vadd_mem_maskedNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// mask
  {
    C2_MacroAssembler _masm(&cbuf);

#line 9366 "src/hotspot/cpu/x86/x86.ad"

    int vlen_enc = vector_length_encoding(this);
    BasicType bt = Matcher::vector_element_basic_type(this);
    int opc = this->ideal_Opcode();
    __ evmasked_op(opc, bt, opnd_array(3)->as_KRegister(ra_,this,idx3)/* mask */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* dst */,
                   opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* dst */, Address::make_raw(opnd_array(2)->base(ra_,this,idx2), opnd_array(2)->index(ra_,this,idx2), opnd_array(2)->scale(), opnd_array(2)->disp(ra_,this,idx2), opnd_array(2)->disp_reloc()), true, vlen_enc);
  
#line 45043 "ad_x86.cpp"
  }
}

void vadd_mem_masked_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// mask
  {
    C2_MacroAssembler _masm(&cbuf);

#line 9366 "src/hotspot/cpu/x86/x86.ad"

    int vlen_enc = vector_length_encoding(this);
    BasicType bt = Matcher::vector_element_basic_type(this);
    int opc = this->ideal_Opcode();
    __ evmasked_op(opc, bt, opnd_array(3)->as_KRegister(ra_,this,idx3)/* mask */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* dst */,
                   opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* dst */, Address::make_raw(opnd_array(2)->base(ra_,this,idx2), opnd_array(2)->index(ra_,this,idx2), opnd_array(2)->scale(), opnd_array(2)->disp(ra_,this,idx2), opnd_array(2)->disp_reloc()), true, vlen_enc);
  
#line 45065 "ad_x86.cpp"
  }
}

void vadd_mem_masked_1Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// mask
  {
    C2_MacroAssembler _masm(&cbuf);

#line 9366 "src/hotspot/cpu/x86/x86.ad"

    int vlen_enc = vector_length_encoding(this);
    BasicType bt = Matcher::vector_element_basic_type(this);
    int opc = this->ideal_Opcode();
    __ evmasked_op(opc, bt, opnd_array(3)->as_KRegister(ra_,this,idx3)/* mask */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* dst */,
                   opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* dst */, Address::make_raw(opnd_array(2)->base(ra_,this,idx2), opnd_array(2)->index(ra_,this,idx2), opnd_array(2)->scale(), opnd_array(2)->disp(ra_,this,idx2), opnd_array(2)->disp_reloc()), true, vlen_enc);
  
#line 45087 "ad_x86.cpp"
  }
}

void vadd_mem_masked_2Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// mask
  {
    C2_MacroAssembler _masm(&cbuf);

#line 9366 "src/hotspot/cpu/x86/x86.ad"

    int vlen_enc = vector_length_encoding(this);
    BasicType bt = Matcher::vector_element_basic_type(this);
    int opc = this->ideal_Opcode();
    __ evmasked_op(opc, bt, opnd_array(3)->as_KRegister(ra_,this,idx3)/* mask */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* dst */,
                   opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* dst */, Address::make_raw(opnd_array(2)->base(ra_,this,idx2), opnd_array(2)->index(ra_,this,idx2), opnd_array(2)->scale(), opnd_array(2)->disp(ra_,this,idx2), opnd_array(2)->disp_reloc()), true, vlen_enc);
  
#line 45109 "ad_x86.cpp"
  }
}

void vadd_mem_masked_3Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// mask
  {
    C2_MacroAssembler _masm(&cbuf);

#line 9366 "src/hotspot/cpu/x86/x86.ad"

    int vlen_enc = vector_length_encoding(this);
    BasicType bt = Matcher::vector_element_basic_type(this);
    int opc = this->ideal_Opcode();
    __ evmasked_op(opc, bt, opnd_array(3)->as_KRegister(ra_,this,idx3)/* mask */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* dst */,
                   opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* dst */, Address::make_raw(opnd_array(2)->base(ra_,this,idx2), opnd_array(2)->index(ra_,this,idx2), opnd_array(2)->scale(), opnd_array(2)->disp(ra_,this,idx2), opnd_array(2)->disp_reloc()), true, vlen_enc);
  
#line 45131 "ad_x86.cpp"
  }
}

void vadd_mem_masked_4Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// mask
  {
    C2_MacroAssembler _masm(&cbuf);

#line 9366 "src/hotspot/cpu/x86/x86.ad"

    int vlen_enc = vector_length_encoding(this);
    BasicType bt = Matcher::vector_element_basic_type(this);
    int opc = this->ideal_Opcode();
    __ evmasked_op(opc, bt, opnd_array(3)->as_KRegister(ra_,this,idx3)/* mask */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* dst */,
                   opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* dst */, Address::make_raw(opnd_array(2)->base(ra_,this,idx2), opnd_array(2)->index(ra_,this,idx2), opnd_array(2)->scale(), opnd_array(2)->disp(ra_,this,idx2), opnd_array(2)->disp_reloc()), true, vlen_enc);
  
#line 45153 "ad_x86.cpp"
  }
}

void vxor_reg_maskedNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// mask
  {
    C2_MacroAssembler _masm(&cbuf);

#line 9379 "src/hotspot/cpu/x86/x86.ad"

    int vlen_enc = vector_length_encoding(this);
    BasicType bt = Matcher::vector_element_basic_type(this);
    int opc = this->ideal_Opcode();
    __ evmasked_op(opc, bt, opnd_array(3)->as_KRegister(ra_,this,idx3)/* mask */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* dst */,
                   opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* dst */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src2 */, true, vlen_enc);
  
#line 45175 "ad_x86.cpp"
  }
}

void vxor_mem_maskedNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// mask
  {
    C2_MacroAssembler _masm(&cbuf);

#line 9392 "src/hotspot/cpu/x86/x86.ad"

    int vlen_enc = vector_length_encoding(this);
    BasicType bt = Matcher::vector_element_basic_type(this);
    int opc = this->ideal_Opcode();
    __ evmasked_op(opc, bt, opnd_array(3)->as_KRegister(ra_,this,idx3)/* mask */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* dst */,
                   opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* dst */, Address::make_raw(opnd_array(2)->base(ra_,this,idx2), opnd_array(2)->index(ra_,this,idx2), opnd_array(2)->scale(), opnd_array(2)->disp(ra_,this,idx2), opnd_array(2)->disp_reloc()), true, vlen_enc);
  
#line 45197 "ad_x86.cpp"
  }
}

void vor_reg_maskedNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// mask
  {
    C2_MacroAssembler _masm(&cbuf);

#line 9405 "src/hotspot/cpu/x86/x86.ad"

    int vlen_enc = vector_length_encoding(this);
    BasicType bt = Matcher::vector_element_basic_type(this);
    int opc = this->ideal_Opcode();
    __ evmasked_op(opc, bt, opnd_array(3)->as_KRegister(ra_,this,idx3)/* mask */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* dst */,
                   opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* dst */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src2 */, true, vlen_enc);
  
#line 45219 "ad_x86.cpp"
  }
}

void vor_mem_maskedNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// mask
  {
    C2_MacroAssembler _masm(&cbuf);

#line 9418 "src/hotspot/cpu/x86/x86.ad"

    int vlen_enc = vector_length_encoding(this);
    BasicType bt = Matcher::vector_element_basic_type(this);
    int opc = this->ideal_Opcode();
    __ evmasked_op(opc, bt, opnd_array(3)->as_KRegister(ra_,this,idx3)/* mask */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* dst */,
                   opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* dst */, Address::make_raw(opnd_array(2)->base(ra_,this,idx2), opnd_array(2)->index(ra_,this,idx2), opnd_array(2)->scale(), opnd_array(2)->disp(ra_,this,idx2), opnd_array(2)->disp_reloc()), true, vlen_enc);
  
#line 45241 "ad_x86.cpp"
  }
}

void vand_reg_maskedNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// mask
  {
    C2_MacroAssembler _masm(&cbuf);

#line 9431 "src/hotspot/cpu/x86/x86.ad"

    int vlen_enc = vector_length_encoding(this);
    BasicType bt = Matcher::vector_element_basic_type(this);
    int opc = this->ideal_Opcode();
    __ evmasked_op(opc, bt, opnd_array(3)->as_KRegister(ra_,this,idx3)/* mask */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* dst */,
                   opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* dst */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src2 */, true, vlen_enc);
  
#line 45263 "ad_x86.cpp"
  }
}

void vand_mem_maskedNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// mask
  {
    C2_MacroAssembler _masm(&cbuf);

#line 9444 "src/hotspot/cpu/x86/x86.ad"

    int vlen_enc = vector_length_encoding(this);
    BasicType bt = Matcher::vector_element_basic_type(this);
    int opc = this->ideal_Opcode();
    __ evmasked_op(opc, bt, opnd_array(3)->as_KRegister(ra_,this,idx3)/* mask */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* dst */,
                   opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* dst */, Address::make_raw(opnd_array(2)->base(ra_,this,idx2), opnd_array(2)->index(ra_,this,idx2), opnd_array(2)->scale(), opnd_array(2)->disp(ra_,this,idx2), opnd_array(2)->disp_reloc()), true, vlen_enc);
  
#line 45285 "ad_x86.cpp"
  }
}

void vsub_reg_maskedNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// mask
  {
    C2_MacroAssembler _masm(&cbuf);

#line 9462 "src/hotspot/cpu/x86/x86.ad"

    int vlen_enc = vector_length_encoding(this);
    BasicType bt = Matcher::vector_element_basic_type(this);
    int opc = this->ideal_Opcode();
    __ evmasked_op(opc, bt, opnd_array(3)->as_KRegister(ra_,this,idx3)/* mask */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* dst */,
                   opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* dst */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src2 */, true, vlen_enc);
  
#line 45307 "ad_x86.cpp"
  }
}

void vsub_reg_masked_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// mask
  {
    C2_MacroAssembler _masm(&cbuf);

#line 9462 "src/hotspot/cpu/x86/x86.ad"

    int vlen_enc = vector_length_encoding(this);
    BasicType bt = Matcher::vector_element_basic_type(this);
    int opc = this->ideal_Opcode();
    __ evmasked_op(opc, bt, opnd_array(3)->as_KRegister(ra_,this,idx3)/* mask */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* dst */,
                   opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* dst */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src2 */, true, vlen_enc);
  
#line 45329 "ad_x86.cpp"
  }
}

void vsub_reg_masked_1Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// mask
  {
    C2_MacroAssembler _masm(&cbuf);

#line 9462 "src/hotspot/cpu/x86/x86.ad"

    int vlen_enc = vector_length_encoding(this);
    BasicType bt = Matcher::vector_element_basic_type(this);
    int opc = this->ideal_Opcode();
    __ evmasked_op(opc, bt, opnd_array(3)->as_KRegister(ra_,this,idx3)/* mask */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* dst */,
                   opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* dst */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src2 */, true, vlen_enc);
  
#line 45351 "ad_x86.cpp"
  }
}

void vsub_reg_masked_2Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// mask
  {
    C2_MacroAssembler _masm(&cbuf);

#line 9462 "src/hotspot/cpu/x86/x86.ad"

    int vlen_enc = vector_length_encoding(this);
    BasicType bt = Matcher::vector_element_basic_type(this);
    int opc = this->ideal_Opcode();
    __ evmasked_op(opc, bt, opnd_array(3)->as_KRegister(ra_,this,idx3)/* mask */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* dst */,
                   opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* dst */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src2 */, true, vlen_enc);
  
#line 45373 "ad_x86.cpp"
  }
}

void vsub_reg_masked_3Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// mask
  {
    C2_MacroAssembler _masm(&cbuf);

#line 9462 "src/hotspot/cpu/x86/x86.ad"

    int vlen_enc = vector_length_encoding(this);
    BasicType bt = Matcher::vector_element_basic_type(this);
    int opc = this->ideal_Opcode();
    __ evmasked_op(opc, bt, opnd_array(3)->as_KRegister(ra_,this,idx3)/* mask */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* dst */,
                   opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* dst */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src2 */, true, vlen_enc);
  
#line 45395 "ad_x86.cpp"
  }
}

void vsub_reg_masked_4Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// mask
  {
    C2_MacroAssembler _masm(&cbuf);

#line 9462 "src/hotspot/cpu/x86/x86.ad"

    int vlen_enc = vector_length_encoding(this);
    BasicType bt = Matcher::vector_element_basic_type(this);
    int opc = this->ideal_Opcode();
    __ evmasked_op(opc, bt, opnd_array(3)->as_KRegister(ra_,this,idx3)/* mask */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* dst */,
                   opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* dst */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src2 */, true, vlen_enc);
  
#line 45417 "ad_x86.cpp"
  }
}

void vsub_mem_maskedNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// mask
  {
    C2_MacroAssembler _masm(&cbuf);

#line 9480 "src/hotspot/cpu/x86/x86.ad"

    int vlen_enc = vector_length_encoding(this);
    BasicType bt = Matcher::vector_element_basic_type(this);
    int opc = this->ideal_Opcode();
    __ evmasked_op(opc, bt, opnd_array(3)->as_KRegister(ra_,this,idx3)/* mask */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* dst */,
                   opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* dst */, Address::make_raw(opnd_array(2)->base(ra_,this,idx2), opnd_array(2)->index(ra_,this,idx2), opnd_array(2)->scale(), opnd_array(2)->disp(ra_,this,idx2), opnd_array(2)->disp_reloc()), true, vlen_enc);
  
#line 45439 "ad_x86.cpp"
  }
}

void vsub_mem_masked_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// mask
  {
    C2_MacroAssembler _masm(&cbuf);

#line 9480 "src/hotspot/cpu/x86/x86.ad"

    int vlen_enc = vector_length_encoding(this);
    BasicType bt = Matcher::vector_element_basic_type(this);
    int opc = this->ideal_Opcode();
    __ evmasked_op(opc, bt, opnd_array(3)->as_KRegister(ra_,this,idx3)/* mask */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* dst */,
                   opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* dst */, Address::make_raw(opnd_array(2)->base(ra_,this,idx2), opnd_array(2)->index(ra_,this,idx2), opnd_array(2)->scale(), opnd_array(2)->disp(ra_,this,idx2), opnd_array(2)->disp_reloc()), true, vlen_enc);
  
#line 45461 "ad_x86.cpp"
  }
}

void vsub_mem_masked_1Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// mask
  {
    C2_MacroAssembler _masm(&cbuf);

#line 9480 "src/hotspot/cpu/x86/x86.ad"

    int vlen_enc = vector_length_encoding(this);
    BasicType bt = Matcher::vector_element_basic_type(this);
    int opc = this->ideal_Opcode();
    __ evmasked_op(opc, bt, opnd_array(3)->as_KRegister(ra_,this,idx3)/* mask */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* dst */,
                   opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* dst */, Address::make_raw(opnd_array(2)->base(ra_,this,idx2), opnd_array(2)->index(ra_,this,idx2), opnd_array(2)->scale(), opnd_array(2)->disp(ra_,this,idx2), opnd_array(2)->disp_reloc()), true, vlen_enc);
  
#line 45483 "ad_x86.cpp"
  }
}

void vsub_mem_masked_2Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// mask
  {
    C2_MacroAssembler _masm(&cbuf);

#line 9480 "src/hotspot/cpu/x86/x86.ad"

    int vlen_enc = vector_length_encoding(this);
    BasicType bt = Matcher::vector_element_basic_type(this);
    int opc = this->ideal_Opcode();
    __ evmasked_op(opc, bt, opnd_array(3)->as_KRegister(ra_,this,idx3)/* mask */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* dst */,
                   opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* dst */, Address::make_raw(opnd_array(2)->base(ra_,this,idx2), opnd_array(2)->index(ra_,this,idx2), opnd_array(2)->scale(), opnd_array(2)->disp(ra_,this,idx2), opnd_array(2)->disp_reloc()), true, vlen_enc);
  
#line 45505 "ad_x86.cpp"
  }
}

void vsub_mem_masked_3Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// mask
  {
    C2_MacroAssembler _masm(&cbuf);

#line 9480 "src/hotspot/cpu/x86/x86.ad"

    int vlen_enc = vector_length_encoding(this);
    BasicType bt = Matcher::vector_element_basic_type(this);
    int opc = this->ideal_Opcode();
    __ evmasked_op(opc, bt, opnd_array(3)->as_KRegister(ra_,this,idx3)/* mask */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* dst */,
                   opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* dst */, Address::make_raw(opnd_array(2)->base(ra_,this,idx2), opnd_array(2)->index(ra_,this,idx2), opnd_array(2)->scale(), opnd_array(2)->disp(ra_,this,idx2), opnd_array(2)->disp_reloc()), true, vlen_enc);
  
#line 45527 "ad_x86.cpp"
  }
}

void vsub_mem_masked_4Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// mask
  {
    C2_MacroAssembler _masm(&cbuf);

#line 9480 "src/hotspot/cpu/x86/x86.ad"

    int vlen_enc = vector_length_encoding(this);
    BasicType bt = Matcher::vector_element_basic_type(this);
    int opc = this->ideal_Opcode();
    __ evmasked_op(opc, bt, opnd_array(3)->as_KRegister(ra_,this,idx3)/* mask */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* dst */,
                   opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* dst */, Address::make_raw(opnd_array(2)->base(ra_,this,idx2), opnd_array(2)->index(ra_,this,idx2), opnd_array(2)->scale(), opnd_array(2)->disp(ra_,this,idx2), opnd_array(2)->disp_reloc()), true, vlen_enc);
  
#line 45549 "ad_x86.cpp"
  }
}

void vmul_reg_maskedNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// mask
  {
    C2_MacroAssembler _masm(&cbuf);

#line 9497 "src/hotspot/cpu/x86/x86.ad"

    int vlen_enc = vector_length_encoding(this);
    BasicType bt = Matcher::vector_element_basic_type(this);
    int opc = this->ideal_Opcode();
    __ evmasked_op(opc, bt, opnd_array(3)->as_KRegister(ra_,this,idx3)/* mask */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* dst */,
                   opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* dst */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src2 */, true, vlen_enc);
  
#line 45571 "ad_x86.cpp"
  }
}

void vmul_reg_masked_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// mask
  {
    C2_MacroAssembler _masm(&cbuf);

#line 9497 "src/hotspot/cpu/x86/x86.ad"

    int vlen_enc = vector_length_encoding(this);
    BasicType bt = Matcher::vector_element_basic_type(this);
    int opc = this->ideal_Opcode();
    __ evmasked_op(opc, bt, opnd_array(3)->as_KRegister(ra_,this,idx3)/* mask */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* dst */,
                   opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* dst */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src2 */, true, vlen_enc);
  
#line 45593 "ad_x86.cpp"
  }
}

void vmul_reg_masked_1Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// mask
  {
    C2_MacroAssembler _masm(&cbuf);

#line 9497 "src/hotspot/cpu/x86/x86.ad"

    int vlen_enc = vector_length_encoding(this);
    BasicType bt = Matcher::vector_element_basic_type(this);
    int opc = this->ideal_Opcode();
    __ evmasked_op(opc, bt, opnd_array(3)->as_KRegister(ra_,this,idx3)/* mask */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* dst */,
                   opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* dst */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src2 */, true, vlen_enc);
  
#line 45615 "ad_x86.cpp"
  }
}

void vmul_reg_masked_2Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// mask
  {
    C2_MacroAssembler _masm(&cbuf);

#line 9497 "src/hotspot/cpu/x86/x86.ad"

    int vlen_enc = vector_length_encoding(this);
    BasicType bt = Matcher::vector_element_basic_type(this);
    int opc = this->ideal_Opcode();
    __ evmasked_op(opc, bt, opnd_array(3)->as_KRegister(ra_,this,idx3)/* mask */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* dst */,
                   opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* dst */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src2 */, true, vlen_enc);
  
#line 45637 "ad_x86.cpp"
  }
}

void vmul_reg_masked_3Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// mask
  {
    C2_MacroAssembler _masm(&cbuf);

#line 9497 "src/hotspot/cpu/x86/x86.ad"

    int vlen_enc = vector_length_encoding(this);
    BasicType bt = Matcher::vector_element_basic_type(this);
    int opc = this->ideal_Opcode();
    __ evmasked_op(opc, bt, opnd_array(3)->as_KRegister(ra_,this,idx3)/* mask */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* dst */,
                   opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* dst */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src2 */, true, vlen_enc);
  
#line 45659 "ad_x86.cpp"
  }
}

void vmul_mem_maskedNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// mask
  {
    C2_MacroAssembler _masm(&cbuf);

#line 9514 "src/hotspot/cpu/x86/x86.ad"

    int vlen_enc = vector_length_encoding(this);
    BasicType bt = Matcher::vector_element_basic_type(this);
    int opc = this->ideal_Opcode();
    __ evmasked_op(opc, bt, opnd_array(3)->as_KRegister(ra_,this,idx3)/* mask */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* dst */,
                   opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* dst */, Address::make_raw(opnd_array(2)->base(ra_,this,idx2), opnd_array(2)->index(ra_,this,idx2), opnd_array(2)->scale(), opnd_array(2)->disp(ra_,this,idx2), opnd_array(2)->disp_reloc()), true, vlen_enc);
  
#line 45681 "ad_x86.cpp"
  }
}

void vmul_mem_masked_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// mask
  {
    C2_MacroAssembler _masm(&cbuf);

#line 9514 "src/hotspot/cpu/x86/x86.ad"

    int vlen_enc = vector_length_encoding(this);
    BasicType bt = Matcher::vector_element_basic_type(this);
    int opc = this->ideal_Opcode();
    __ evmasked_op(opc, bt, opnd_array(3)->as_KRegister(ra_,this,idx3)/* mask */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* dst */,
                   opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* dst */, Address::make_raw(opnd_array(2)->base(ra_,this,idx2), opnd_array(2)->index(ra_,this,idx2), opnd_array(2)->scale(), opnd_array(2)->disp(ra_,this,idx2), opnd_array(2)->disp_reloc()), true, vlen_enc);
  
#line 45703 "ad_x86.cpp"
  }
}

void vmul_mem_masked_1Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// mask
  {
    C2_MacroAssembler _masm(&cbuf);

#line 9514 "src/hotspot/cpu/x86/x86.ad"

    int vlen_enc = vector_length_encoding(this);
    BasicType bt = Matcher::vector_element_basic_type(this);
    int opc = this->ideal_Opcode();
    __ evmasked_op(opc, bt, opnd_array(3)->as_KRegister(ra_,this,idx3)/* mask */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* dst */,
                   opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* dst */, Address::make_raw(opnd_array(2)->base(ra_,this,idx2), opnd_array(2)->index(ra_,this,idx2), opnd_array(2)->scale(), opnd_array(2)->disp(ra_,this,idx2), opnd_array(2)->disp_reloc()), true, vlen_enc);
  
#line 45725 "ad_x86.cpp"
  }
}

void vmul_mem_masked_2Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// mask
  {
    C2_MacroAssembler _masm(&cbuf);

#line 9514 "src/hotspot/cpu/x86/x86.ad"

    int vlen_enc = vector_length_encoding(this);
    BasicType bt = Matcher::vector_element_basic_type(this);
    int opc = this->ideal_Opcode();
    __ evmasked_op(opc, bt, opnd_array(3)->as_KRegister(ra_,this,idx3)/* mask */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* dst */,
                   opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* dst */, Address::make_raw(opnd_array(2)->base(ra_,this,idx2), opnd_array(2)->index(ra_,this,idx2), opnd_array(2)->scale(), opnd_array(2)->disp(ra_,this,idx2), opnd_array(2)->disp_reloc()), true, vlen_enc);
  
#line 45747 "ad_x86.cpp"
  }
}

void vmul_mem_masked_3Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// mask
  {
    C2_MacroAssembler _masm(&cbuf);

#line 9514 "src/hotspot/cpu/x86/x86.ad"

    int vlen_enc = vector_length_encoding(this);
    BasicType bt = Matcher::vector_element_basic_type(this);
    int opc = this->ideal_Opcode();
    __ evmasked_op(opc, bt, opnd_array(3)->as_KRegister(ra_,this,idx3)/* mask */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* dst */,
                   opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* dst */, Address::make_raw(opnd_array(2)->base(ra_,this,idx2), opnd_array(2)->index(ra_,this,idx2), opnd_array(2)->scale(), opnd_array(2)->disp(ra_,this,idx2), opnd_array(2)->disp_reloc()), true, vlen_enc);
  
#line 45769 "ad_x86.cpp"
  }
}

void vsqrt_reg_maskedNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// mask
  {
    C2_MacroAssembler _masm(&cbuf);

#line 9528 "src/hotspot/cpu/x86/x86.ad"

    int vlen_enc = vector_length_encoding(this);
    BasicType bt = Matcher::vector_element_basic_type(this);
    int opc = this->ideal_Opcode();
    __ evmasked_op(opc, bt, opnd_array(2)->as_KRegister(ra_,this,idx2)/* mask */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* dst */,
                   opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* dst */, true, vlen_enc);
  
#line 45790 "ad_x86.cpp"
  }
}

void vsqrt_reg_masked_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// mask
  {
    C2_MacroAssembler _masm(&cbuf);

#line 9528 "src/hotspot/cpu/x86/x86.ad"

    int vlen_enc = vector_length_encoding(this);
    BasicType bt = Matcher::vector_element_basic_type(this);
    int opc = this->ideal_Opcode();
    __ evmasked_op(opc, bt, opnd_array(2)->as_KRegister(ra_,this,idx2)/* mask */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* dst */,
                   opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* dst */, true, vlen_enc);
  
#line 45811 "ad_x86.cpp"
  }
}

void vdiv_reg_maskedNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// mask
  {
    C2_MacroAssembler _masm(&cbuf);

#line 9542 "src/hotspot/cpu/x86/x86.ad"

    int vlen_enc = vector_length_encoding(this);
    BasicType bt = Matcher::vector_element_basic_type(this);
    int opc = this->ideal_Opcode();
    __ evmasked_op(opc, bt, opnd_array(3)->as_KRegister(ra_,this,idx3)/* mask */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* dst */,
                   opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* dst */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src2 */, true, vlen_enc);
  
#line 45833 "ad_x86.cpp"
  }
}

void vdiv_reg_masked_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// mask
  {
    C2_MacroAssembler _masm(&cbuf);

#line 9542 "src/hotspot/cpu/x86/x86.ad"

    int vlen_enc = vector_length_encoding(this);
    BasicType bt = Matcher::vector_element_basic_type(this);
    int opc = this->ideal_Opcode();
    __ evmasked_op(opc, bt, opnd_array(3)->as_KRegister(ra_,this,idx3)/* mask */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* dst */,
                   opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* dst */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src2 */, true, vlen_enc);
  
#line 45855 "ad_x86.cpp"
  }
}

void vdiv_mem_maskedNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// mask
  {
    C2_MacroAssembler _masm(&cbuf);

#line 9556 "src/hotspot/cpu/x86/x86.ad"

    int vlen_enc = vector_length_encoding(this);
    BasicType bt = Matcher::vector_element_basic_type(this);
    int opc = this->ideal_Opcode();
    __ evmasked_op(opc, bt, opnd_array(3)->as_KRegister(ra_,this,idx3)/* mask */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* dst */,
                   opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* dst */, Address::make_raw(opnd_array(2)->base(ra_,this,idx2), opnd_array(2)->index(ra_,this,idx2), opnd_array(2)->scale(), opnd_array(2)->disp(ra_,this,idx2), opnd_array(2)->disp_reloc()), true, vlen_enc);
  
#line 45877 "ad_x86.cpp"
  }
}

void vdiv_mem_masked_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// mask
  {
    C2_MacroAssembler _masm(&cbuf);

#line 9556 "src/hotspot/cpu/x86/x86.ad"

    int vlen_enc = vector_length_encoding(this);
    BasicType bt = Matcher::vector_element_basic_type(this);
    int opc = this->ideal_Opcode();
    __ evmasked_op(opc, bt, opnd_array(3)->as_KRegister(ra_,this,idx3)/* mask */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* dst */,
                   opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* dst */, Address::make_raw(opnd_array(2)->base(ra_,this,idx2), opnd_array(2)->index(ra_,this,idx2), opnd_array(2)->scale(), opnd_array(2)->disp(ra_,this,idx2), opnd_array(2)->disp_reloc()), true, vlen_enc);
  
#line 45899 "ad_x86.cpp"
  }
}

void vrol_imm_maskedNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// shift
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// mask
  {
    C2_MacroAssembler _masm(&cbuf);

#line 9571 "src/hotspot/cpu/x86/x86.ad"

    int vlen_enc = vector_length_encoding(this);
    BasicType bt = Matcher::vector_element_basic_type(this);
    int opc = this->ideal_Opcode();
    __ evmasked_op(opc, bt, opnd_array(3)->as_KRegister(ra_,this,idx3)/* mask */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* dst */,
                   opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* dst */, opnd_array(2)->constant(), true, vlen_enc);
  
#line 45921 "ad_x86.cpp"
  }
}

void vrol_imm_masked_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// shift
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// mask
  {
    C2_MacroAssembler _masm(&cbuf);

#line 9571 "src/hotspot/cpu/x86/x86.ad"

    int vlen_enc = vector_length_encoding(this);
    BasicType bt = Matcher::vector_element_basic_type(this);
    int opc = this->ideal_Opcode();
    __ evmasked_op(opc, bt, opnd_array(3)->as_KRegister(ra_,this,idx3)/* mask */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* dst */,
                   opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* dst */, opnd_array(2)->constant(), true, vlen_enc);
  
#line 45943 "ad_x86.cpp"
  }
}

void vrol_reg_maskedNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// mask
  {
    C2_MacroAssembler _masm(&cbuf);

#line 9585 "src/hotspot/cpu/x86/x86.ad"

    int vlen_enc = vector_length_encoding(this);
    BasicType bt = Matcher::vector_element_basic_type(this);
    int opc = this->ideal_Opcode();
    __ evmasked_op(opc, bt, opnd_array(3)->as_KRegister(ra_,this,idx3)/* mask */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* dst */,
                   opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* dst */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src2 */, true, vlen_enc);
  
#line 45965 "ad_x86.cpp"
  }
}

void vrol_reg_masked_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// mask
  {
    C2_MacroAssembler _masm(&cbuf);

#line 9585 "src/hotspot/cpu/x86/x86.ad"

    int vlen_enc = vector_length_encoding(this);
    BasicType bt = Matcher::vector_element_basic_type(this);
    int opc = this->ideal_Opcode();
    __ evmasked_op(opc, bt, opnd_array(3)->as_KRegister(ra_,this,idx3)/* mask */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* dst */,
                   opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* dst */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src2 */, true, vlen_enc);
  
#line 45987 "ad_x86.cpp"
  }
}

void vlshift_imm_maskedNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// shift
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// mask
  {
    C2_MacroAssembler _masm(&cbuf);

#line 9600 "src/hotspot/cpu/x86/x86.ad"

    int vlen_enc = vector_length_encoding(this);
    BasicType bt = Matcher::vector_element_basic_type(this);
    int opc = this->ideal_Opcode();
    __ evmasked_op(opc, bt, opnd_array(3)->as_KRegister(ra_,this,idx3)/* mask */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* dst */,
                   opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* dst */, opnd_array(2)->constant(), true, vlen_enc);
  
#line 46009 "ad_x86.cpp"
  }
}

void vlshift_imm_masked_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// shift
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// mask
  {
    C2_MacroAssembler _masm(&cbuf);

#line 9600 "src/hotspot/cpu/x86/x86.ad"

    int vlen_enc = vector_length_encoding(this);
    BasicType bt = Matcher::vector_element_basic_type(this);
    int opc = this->ideal_Opcode();
    __ evmasked_op(opc, bt, opnd_array(3)->as_KRegister(ra_,this,idx3)/* mask */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* dst */,
                   opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* dst */, opnd_array(2)->constant(), true, vlen_enc);
  
#line 46031 "ad_x86.cpp"
  }
}

void vlshift_imm_masked_1Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// shift
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// mask
  {
    C2_MacroAssembler _masm(&cbuf);

#line 9600 "src/hotspot/cpu/x86/x86.ad"

    int vlen_enc = vector_length_encoding(this);
    BasicType bt = Matcher::vector_element_basic_type(this);
    int opc = this->ideal_Opcode();
    __ evmasked_op(opc, bt, opnd_array(3)->as_KRegister(ra_,this,idx3)/* mask */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* dst */,
                   opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* dst */, opnd_array(2)->constant(), true, vlen_enc);
  
#line 46053 "ad_x86.cpp"
  }
}

void vlshift_reg_maskedNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// mask
  {
    C2_MacroAssembler _masm(&cbuf);

#line 9616 "src/hotspot/cpu/x86/x86.ad"

    int vlen_enc = vector_length_encoding(this);
    BasicType bt = Matcher::vector_element_basic_type(this);
    int opc = this->ideal_Opcode();
    __ evmasked_op(opc, bt, opnd_array(3)->as_KRegister(ra_,this,idx3)/* mask */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* dst */,
                   opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* dst */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src2 */, true, vlen_enc, false);
  
#line 46075 "ad_x86.cpp"
  }
}

void vlshift_reg_masked_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// mask
  {
    C2_MacroAssembler _masm(&cbuf);

#line 9616 "src/hotspot/cpu/x86/x86.ad"

    int vlen_enc = vector_length_encoding(this);
    BasicType bt = Matcher::vector_element_basic_type(this);
    int opc = this->ideal_Opcode();
    __ evmasked_op(opc, bt, opnd_array(3)->as_KRegister(ra_,this,idx3)/* mask */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* dst */,
                   opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* dst */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src2 */, true, vlen_enc, false);
  
#line 46097 "ad_x86.cpp"
  }
}

void vlshift_reg_masked_1Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// mask
  {
    C2_MacroAssembler _masm(&cbuf);

#line 9616 "src/hotspot/cpu/x86/x86.ad"

    int vlen_enc = vector_length_encoding(this);
    BasicType bt = Matcher::vector_element_basic_type(this);
    int opc = this->ideal_Opcode();
    __ evmasked_op(opc, bt, opnd_array(3)->as_KRegister(ra_,this,idx3)/* mask */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* dst */,
                   opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* dst */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src2 */, true, vlen_enc, false);
  
#line 46119 "ad_x86.cpp"
  }
}

void vlshiftv_reg_maskedNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// mask
  {
    C2_MacroAssembler _masm(&cbuf);

#line 9632 "src/hotspot/cpu/x86/x86.ad"

    int vlen_enc = vector_length_encoding(this);
    BasicType bt = Matcher::vector_element_basic_type(this);
    int opc = this->ideal_Opcode();
    __ evmasked_op(opc, bt, opnd_array(3)->as_KRegister(ra_,this,idx3)/* mask */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* dst */,
                   opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* dst */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src2 */, true, vlen_enc, true);
  
#line 46141 "ad_x86.cpp"
  }
}

void vlshiftv_reg_masked_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// mask
  {
    C2_MacroAssembler _masm(&cbuf);

#line 9632 "src/hotspot/cpu/x86/x86.ad"

    int vlen_enc = vector_length_encoding(this);
    BasicType bt = Matcher::vector_element_basic_type(this);
    int opc = this->ideal_Opcode();
    __ evmasked_op(opc, bt, opnd_array(3)->as_KRegister(ra_,this,idx3)/* mask */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* dst */,
                   opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* dst */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src2 */, true, vlen_enc, true);
  
#line 46163 "ad_x86.cpp"
  }
}

void vlshiftv_reg_masked_1Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// mask
  {
    C2_MacroAssembler _masm(&cbuf);

#line 9632 "src/hotspot/cpu/x86/x86.ad"

    int vlen_enc = vector_length_encoding(this);
    BasicType bt = Matcher::vector_element_basic_type(this);
    int opc = this->ideal_Opcode();
    __ evmasked_op(opc, bt, opnd_array(3)->as_KRegister(ra_,this,idx3)/* mask */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* dst */,
                   opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* dst */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src2 */, true, vlen_enc, true);
  
#line 46185 "ad_x86.cpp"
  }
}

void vlshift_mem_maskedNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// mask
  {
    C2_MacroAssembler _masm(&cbuf);

#line 9647 "src/hotspot/cpu/x86/x86.ad"

    int vlen_enc = vector_length_encoding(this);
    BasicType bt = Matcher::vector_element_basic_type(this);
    int opc = this->ideal_Opcode();
    __ evmasked_op(opc, bt, opnd_array(3)->as_KRegister(ra_,this,idx3)/* mask */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* dst */,
                   opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* dst */, Address::make_raw(opnd_array(2)->base(ra_,this,idx2), opnd_array(2)->index(ra_,this,idx2), opnd_array(2)->scale(), opnd_array(2)->disp(ra_,this,idx2), opnd_array(2)->disp_reloc()), true, vlen_enc);
  
#line 46207 "ad_x86.cpp"
  }
}

void vlshift_mem_masked_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// mask
  {
    C2_MacroAssembler _masm(&cbuf);

#line 9647 "src/hotspot/cpu/x86/x86.ad"

    int vlen_enc = vector_length_encoding(this);
    BasicType bt = Matcher::vector_element_basic_type(this);
    int opc = this->ideal_Opcode();
    __ evmasked_op(opc, bt, opnd_array(3)->as_KRegister(ra_,this,idx3)/* mask */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* dst */,
                   opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* dst */, Address::make_raw(opnd_array(2)->base(ra_,this,idx2), opnd_array(2)->index(ra_,this,idx2), opnd_array(2)->scale(), opnd_array(2)->disp(ra_,this,idx2), opnd_array(2)->disp_reloc()), true, vlen_enc);
  
#line 46229 "ad_x86.cpp"
  }
}

void vlshift_mem_masked_1Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// mask
  {
    C2_MacroAssembler _masm(&cbuf);

#line 9647 "src/hotspot/cpu/x86/x86.ad"

    int vlen_enc = vector_length_encoding(this);
    BasicType bt = Matcher::vector_element_basic_type(this);
    int opc = this->ideal_Opcode();
    __ evmasked_op(opc, bt, opnd_array(3)->as_KRegister(ra_,this,idx3)/* mask */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* dst */,
                   opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* dst */, Address::make_raw(opnd_array(2)->base(ra_,this,idx2), opnd_array(2)->index(ra_,this,idx2), opnd_array(2)->scale(), opnd_array(2)->disp(ra_,this,idx2), opnd_array(2)->disp_reloc()), true, vlen_enc);
  
#line 46251 "ad_x86.cpp"
  }
}

void vrshift_imm_maskedNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// shift
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// mask
  {
    C2_MacroAssembler _masm(&cbuf);

#line 9662 "src/hotspot/cpu/x86/x86.ad"

    int vlen_enc = vector_length_encoding(this);
    BasicType bt = Matcher::vector_element_basic_type(this);
    int opc = this->ideal_Opcode();
    __ evmasked_op(opc, bt, opnd_array(3)->as_KRegister(ra_,this,idx3)/* mask */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* dst */,
                   opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* dst */, opnd_array(2)->constant(), true, vlen_enc);
  
#line 46273 "ad_x86.cpp"
  }
}

void vrshift_imm_masked_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// shift
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// mask
  {
    C2_MacroAssembler _masm(&cbuf);

#line 9662 "src/hotspot/cpu/x86/x86.ad"

    int vlen_enc = vector_length_encoding(this);
    BasicType bt = Matcher::vector_element_basic_type(this);
    int opc = this->ideal_Opcode();
    __ evmasked_op(opc, bt, opnd_array(3)->as_KRegister(ra_,this,idx3)/* mask */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* dst */,
                   opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* dst */, opnd_array(2)->constant(), true, vlen_enc);
  
#line 46295 "ad_x86.cpp"
  }
}

void vrshift_imm_masked_1Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// shift
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// mask
  {
    C2_MacroAssembler _masm(&cbuf);

#line 9662 "src/hotspot/cpu/x86/x86.ad"

    int vlen_enc = vector_length_encoding(this);
    BasicType bt = Matcher::vector_element_basic_type(this);
    int opc = this->ideal_Opcode();
    __ evmasked_op(opc, bt, opnd_array(3)->as_KRegister(ra_,this,idx3)/* mask */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* dst */,
                   opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* dst */, opnd_array(2)->constant(), true, vlen_enc);
  
#line 46317 "ad_x86.cpp"
  }
}

void vrshift_reg_maskedNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// mask
  {
    C2_MacroAssembler _masm(&cbuf);

#line 9678 "src/hotspot/cpu/x86/x86.ad"

    int vlen_enc = vector_length_encoding(this);
    BasicType bt = Matcher::vector_element_basic_type(this);
    int opc = this->ideal_Opcode();
    __ evmasked_op(opc, bt, opnd_array(3)->as_KRegister(ra_,this,idx3)/* mask */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* dst */,
                   opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* dst */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src2 */, true, vlen_enc, false);
  
#line 46339 "ad_x86.cpp"
  }
}

void vrshift_reg_masked_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// mask
  {
    C2_MacroAssembler _masm(&cbuf);

#line 9678 "src/hotspot/cpu/x86/x86.ad"

    int vlen_enc = vector_length_encoding(this);
    BasicType bt = Matcher::vector_element_basic_type(this);
    int opc = this->ideal_Opcode();
    __ evmasked_op(opc, bt, opnd_array(3)->as_KRegister(ra_,this,idx3)/* mask */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* dst */,
                   opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* dst */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src2 */, true, vlen_enc, false);
  
#line 46361 "ad_x86.cpp"
  }
}

void vrshift_reg_masked_1Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// mask
  {
    C2_MacroAssembler _masm(&cbuf);

#line 9678 "src/hotspot/cpu/x86/x86.ad"

    int vlen_enc = vector_length_encoding(this);
    BasicType bt = Matcher::vector_element_basic_type(this);
    int opc = this->ideal_Opcode();
    __ evmasked_op(opc, bt, opnd_array(3)->as_KRegister(ra_,this,idx3)/* mask */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* dst */,
                   opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* dst */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src2 */, true, vlen_enc, false);
  
#line 46383 "ad_x86.cpp"
  }
}

void vrshiftv_reg_maskedNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// mask
  {
    C2_MacroAssembler _masm(&cbuf);

#line 9694 "src/hotspot/cpu/x86/x86.ad"

    int vlen_enc = vector_length_encoding(this);
    BasicType bt = Matcher::vector_element_basic_type(this);
    int opc = this->ideal_Opcode();
    __ evmasked_op(opc, bt, opnd_array(3)->as_KRegister(ra_,this,idx3)/* mask */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* dst */,
                   opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* dst */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src2 */, true, vlen_enc, true);
  
#line 46405 "ad_x86.cpp"
  }
}

void vrshiftv_reg_masked_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// mask
  {
    C2_MacroAssembler _masm(&cbuf);

#line 9694 "src/hotspot/cpu/x86/x86.ad"

    int vlen_enc = vector_length_encoding(this);
    BasicType bt = Matcher::vector_element_basic_type(this);
    int opc = this->ideal_Opcode();
    __ evmasked_op(opc, bt, opnd_array(3)->as_KRegister(ra_,this,idx3)/* mask */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* dst */,
                   opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* dst */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src2 */, true, vlen_enc, true);
  
#line 46427 "ad_x86.cpp"
  }
}

void vrshiftv_reg_masked_1Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// mask
  {
    C2_MacroAssembler _masm(&cbuf);

#line 9694 "src/hotspot/cpu/x86/x86.ad"

    int vlen_enc = vector_length_encoding(this);
    BasicType bt = Matcher::vector_element_basic_type(this);
    int opc = this->ideal_Opcode();
    __ evmasked_op(opc, bt, opnd_array(3)->as_KRegister(ra_,this,idx3)/* mask */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* dst */,
                   opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* dst */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src2 */, true, vlen_enc, true);
  
#line 46449 "ad_x86.cpp"
  }
}

void vrshift_mem_maskedNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// mask
  {
    C2_MacroAssembler _masm(&cbuf);

#line 9709 "src/hotspot/cpu/x86/x86.ad"

    int vlen_enc = vector_length_encoding(this);
    BasicType bt = Matcher::vector_element_basic_type(this);
    int opc = this->ideal_Opcode();
    __ evmasked_op(opc, bt, opnd_array(3)->as_KRegister(ra_,this,idx3)/* mask */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* dst */,
                   opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* dst */, Address::make_raw(opnd_array(2)->base(ra_,this,idx2), opnd_array(2)->index(ra_,this,idx2), opnd_array(2)->scale(), opnd_array(2)->disp(ra_,this,idx2), opnd_array(2)->disp_reloc()), true, vlen_enc);
  
#line 46471 "ad_x86.cpp"
  }
}

void vrshift_mem_masked_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// mask
  {
    C2_MacroAssembler _masm(&cbuf);

#line 9709 "src/hotspot/cpu/x86/x86.ad"

    int vlen_enc = vector_length_encoding(this);
    BasicType bt = Matcher::vector_element_basic_type(this);
    int opc = this->ideal_Opcode();
    __ evmasked_op(opc, bt, opnd_array(3)->as_KRegister(ra_,this,idx3)/* mask */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* dst */,
                   opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* dst */, Address::make_raw(opnd_array(2)->base(ra_,this,idx2), opnd_array(2)->index(ra_,this,idx2), opnd_array(2)->scale(), opnd_array(2)->disp(ra_,this,idx2), opnd_array(2)->disp_reloc()), true, vlen_enc);
  
#line 46493 "ad_x86.cpp"
  }
}

void vrshift_mem_masked_1Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// mask
  {
    C2_MacroAssembler _masm(&cbuf);

#line 9709 "src/hotspot/cpu/x86/x86.ad"

    int vlen_enc = vector_length_encoding(this);
    BasicType bt = Matcher::vector_element_basic_type(this);
    int opc = this->ideal_Opcode();
    __ evmasked_op(opc, bt, opnd_array(3)->as_KRegister(ra_,this,idx3)/* mask */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* dst */,
                   opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* dst */, Address::make_raw(opnd_array(2)->base(ra_,this,idx2), opnd_array(2)->index(ra_,this,idx2), opnd_array(2)->scale(), opnd_array(2)->disp(ra_,this,idx2), opnd_array(2)->disp_reloc()), true, vlen_enc);
  
#line 46515 "ad_x86.cpp"
  }
}

void vurshift_imm_maskedNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// shift
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// mask
  {
    C2_MacroAssembler _masm(&cbuf);

#line 9724 "src/hotspot/cpu/x86/x86.ad"

    int vlen_enc = vector_length_encoding(this);
    BasicType bt = Matcher::vector_element_basic_type(this);
    int opc = this->ideal_Opcode();
    __ evmasked_op(opc, bt, opnd_array(3)->as_KRegister(ra_,this,idx3)/* mask */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* dst */,
                   opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* dst */, opnd_array(2)->constant(), true, vlen_enc);
  
#line 46537 "ad_x86.cpp"
  }
}

void vurshift_imm_masked_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// shift
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// mask
  {
    C2_MacroAssembler _masm(&cbuf);

#line 9724 "src/hotspot/cpu/x86/x86.ad"

    int vlen_enc = vector_length_encoding(this);
    BasicType bt = Matcher::vector_element_basic_type(this);
    int opc = this->ideal_Opcode();
    __ evmasked_op(opc, bt, opnd_array(3)->as_KRegister(ra_,this,idx3)/* mask */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* dst */,
                   opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* dst */, opnd_array(2)->constant(), true, vlen_enc);
  
#line 46559 "ad_x86.cpp"
  }
}

void vurshift_imm_masked_1Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// shift
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// mask
  {
    C2_MacroAssembler _masm(&cbuf);

#line 9724 "src/hotspot/cpu/x86/x86.ad"

    int vlen_enc = vector_length_encoding(this);
    BasicType bt = Matcher::vector_element_basic_type(this);
    int opc = this->ideal_Opcode();
    __ evmasked_op(opc, bt, opnd_array(3)->as_KRegister(ra_,this,idx3)/* mask */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* dst */,
                   opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* dst */, opnd_array(2)->constant(), true, vlen_enc);
  
#line 46581 "ad_x86.cpp"
  }
}

void vurshift_reg_maskedNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// mask
  {
    C2_MacroAssembler _masm(&cbuf);

#line 9740 "src/hotspot/cpu/x86/x86.ad"

    int vlen_enc = vector_length_encoding(this);
    BasicType bt = Matcher::vector_element_basic_type(this);
    int opc = this->ideal_Opcode();
    __ evmasked_op(opc, bt, opnd_array(3)->as_KRegister(ra_,this,idx3)/* mask */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* dst */,
                   opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* dst */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src2 */, true, vlen_enc, false);
  
#line 46603 "ad_x86.cpp"
  }
}

void vurshift_reg_masked_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// mask
  {
    C2_MacroAssembler _masm(&cbuf);

#line 9740 "src/hotspot/cpu/x86/x86.ad"

    int vlen_enc = vector_length_encoding(this);
    BasicType bt = Matcher::vector_element_basic_type(this);
    int opc = this->ideal_Opcode();
    __ evmasked_op(opc, bt, opnd_array(3)->as_KRegister(ra_,this,idx3)/* mask */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* dst */,
                   opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* dst */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src2 */, true, vlen_enc, false);
  
#line 46625 "ad_x86.cpp"
  }
}

void vurshift_reg_masked_1Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// mask
  {
    C2_MacroAssembler _masm(&cbuf);

#line 9740 "src/hotspot/cpu/x86/x86.ad"

    int vlen_enc = vector_length_encoding(this);
    BasicType bt = Matcher::vector_element_basic_type(this);
    int opc = this->ideal_Opcode();
    __ evmasked_op(opc, bt, opnd_array(3)->as_KRegister(ra_,this,idx3)/* mask */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* dst */,
                   opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* dst */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src2 */, true, vlen_enc, false);
  
#line 46647 "ad_x86.cpp"
  }
}

void vurshiftv_reg_maskedNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// mask
  {
    C2_MacroAssembler _masm(&cbuf);

#line 9756 "src/hotspot/cpu/x86/x86.ad"

    int vlen_enc = vector_length_encoding(this);
    BasicType bt = Matcher::vector_element_basic_type(this);
    int opc = this->ideal_Opcode();
    __ evmasked_op(opc, bt, opnd_array(3)->as_KRegister(ra_,this,idx3)/* mask */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* dst */,
                   opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* dst */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src2 */, true, vlen_enc, true);
  
#line 46669 "ad_x86.cpp"
  }
}

void vurshiftv_reg_masked_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// mask
  {
    C2_MacroAssembler _masm(&cbuf);

#line 9756 "src/hotspot/cpu/x86/x86.ad"

    int vlen_enc = vector_length_encoding(this);
    BasicType bt = Matcher::vector_element_basic_type(this);
    int opc = this->ideal_Opcode();
    __ evmasked_op(opc, bt, opnd_array(3)->as_KRegister(ra_,this,idx3)/* mask */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* dst */,
                   opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* dst */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src2 */, true, vlen_enc, true);
  
#line 46691 "ad_x86.cpp"
  }
}

void vurshiftv_reg_masked_1Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// mask
  {
    C2_MacroAssembler _masm(&cbuf);

#line 9756 "src/hotspot/cpu/x86/x86.ad"

    int vlen_enc = vector_length_encoding(this);
    BasicType bt = Matcher::vector_element_basic_type(this);
    int opc = this->ideal_Opcode();
    __ evmasked_op(opc, bt, opnd_array(3)->as_KRegister(ra_,this,idx3)/* mask */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* dst */,
                   opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* dst */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src2 */, true, vlen_enc, true);
  
#line 46713 "ad_x86.cpp"
  }
}

void vurshift_mem_maskedNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// mask
  {
    C2_MacroAssembler _masm(&cbuf);

#line 9771 "src/hotspot/cpu/x86/x86.ad"

    int vlen_enc = vector_length_encoding(this);
    BasicType bt = Matcher::vector_element_basic_type(this);
    int opc = this->ideal_Opcode();
    __ evmasked_op(opc, bt, opnd_array(3)->as_KRegister(ra_,this,idx3)/* mask */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* dst */,
                   opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* dst */, Address::make_raw(opnd_array(2)->base(ra_,this,idx2), opnd_array(2)->index(ra_,this,idx2), opnd_array(2)->scale(), opnd_array(2)->disp(ra_,this,idx2), opnd_array(2)->disp_reloc()), true, vlen_enc);
  
#line 46735 "ad_x86.cpp"
  }
}

void vurshift_mem_masked_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// mask
  {
    C2_MacroAssembler _masm(&cbuf);

#line 9771 "src/hotspot/cpu/x86/x86.ad"

    int vlen_enc = vector_length_encoding(this);
    BasicType bt = Matcher::vector_element_basic_type(this);
    int opc = this->ideal_Opcode();
    __ evmasked_op(opc, bt, opnd_array(3)->as_KRegister(ra_,this,idx3)/* mask */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* dst */,
                   opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* dst */, Address::make_raw(opnd_array(2)->base(ra_,this,idx2), opnd_array(2)->index(ra_,this,idx2), opnd_array(2)->scale(), opnd_array(2)->disp(ra_,this,idx2), opnd_array(2)->disp_reloc()), true, vlen_enc);
  
#line 46757 "ad_x86.cpp"
  }
}

void vurshift_mem_masked_1Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// mask
  {
    C2_MacroAssembler _masm(&cbuf);

#line 9771 "src/hotspot/cpu/x86/x86.ad"

    int vlen_enc = vector_length_encoding(this);
    BasicType bt = Matcher::vector_element_basic_type(this);
    int opc = this->ideal_Opcode();
    __ evmasked_op(opc, bt, opnd_array(3)->as_KRegister(ra_,this,idx3)/* mask */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* dst */,
                   opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* dst */, Address::make_raw(opnd_array(2)->base(ra_,this,idx2), opnd_array(2)->index(ra_,this,idx2), opnd_array(2)->scale(), opnd_array(2)->disp(ra_,this,idx2), opnd_array(2)->disp_reloc()), true, vlen_enc);
  
#line 46779 "ad_x86.cpp"
  }
}

void vmaxv_reg_maskedNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// mask
  {
    C2_MacroAssembler _masm(&cbuf);

#line 9784 "src/hotspot/cpu/x86/x86.ad"

    int vlen_enc = vector_length_encoding(this);
    BasicType bt = Matcher::vector_element_basic_type(this);
    int opc = this->ideal_Opcode();
    __ evmasked_op(opc, bt, opnd_array(3)->as_KRegister(ra_,this,idx3)/* mask */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* dst */,
                   opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* dst */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src2 */, true, vlen_enc);
  
#line 46801 "ad_x86.cpp"
  }
}

void vmaxv_mem_maskedNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// mask
  {
    C2_MacroAssembler _masm(&cbuf);

#line 9797 "src/hotspot/cpu/x86/x86.ad"

    int vlen_enc = vector_length_encoding(this);
    BasicType bt = Matcher::vector_element_basic_type(this);
    int opc = this->ideal_Opcode();
    __ evmasked_op(opc, bt, opnd_array(3)->as_KRegister(ra_,this,idx3)/* mask */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* dst */,
                   opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* dst */, Address::make_raw(opnd_array(2)->base(ra_,this,idx2), opnd_array(2)->index(ra_,this,idx2), opnd_array(2)->scale(), opnd_array(2)->disp(ra_,this,idx2), opnd_array(2)->disp_reloc()), true, vlen_enc);
  
#line 46823 "ad_x86.cpp"
  }
}

void vminv_reg_maskedNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// mask
  {
    C2_MacroAssembler _masm(&cbuf);

#line 9810 "src/hotspot/cpu/x86/x86.ad"

    int vlen_enc = vector_length_encoding(this);
    BasicType bt = Matcher::vector_element_basic_type(this);
    int opc = this->ideal_Opcode();
    __ evmasked_op(opc, bt, opnd_array(3)->as_KRegister(ra_,this,idx3)/* mask */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* dst */,
                   opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* dst */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src2 */, true, vlen_enc);
  
#line 46845 "ad_x86.cpp"
  }
}

void vminv_mem_maskedNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// mask
  {
    C2_MacroAssembler _masm(&cbuf);

#line 9823 "src/hotspot/cpu/x86/x86.ad"

    int vlen_enc = vector_length_encoding(this);
    BasicType bt = Matcher::vector_element_basic_type(this);
    int opc = this->ideal_Opcode();
    __ evmasked_op(opc, bt, opnd_array(3)->as_KRegister(ra_,this,idx3)/* mask */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* dst */,
                   opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* dst */, Address::make_raw(opnd_array(2)->base(ra_,this,idx2), opnd_array(2)->index(ra_,this,idx2), opnd_array(2)->scale(), opnd_array(2)->disp(ra_,this,idx2), opnd_array(2)->disp_reloc()), true, vlen_enc);
  
#line 46867 "ad_x86.cpp"
  }
}

void vrearrangev_reg_maskedNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// mask
  {
    C2_MacroAssembler _masm(&cbuf);

#line 9836 "src/hotspot/cpu/x86/x86.ad"

    int vlen_enc = vector_length_encoding(this);
    BasicType bt = Matcher::vector_element_basic_type(this);
    int opc = this->ideal_Opcode();
    __ evmasked_op(opc, bt, opnd_array(3)->as_KRegister(ra_,this,idx3)/* mask */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* dst */,
                   opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* dst */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src2 */, false, vlen_enc);
  
#line 46889 "ad_x86.cpp"
  }
}

void vabs_maskedNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// mask
  {
    C2_MacroAssembler _masm(&cbuf);

#line 9852 "src/hotspot/cpu/x86/x86.ad"

    int vlen_enc = vector_length_encoding(this);
    BasicType bt = Matcher::vector_element_basic_type(this);
    int opc = this->ideal_Opcode();
    __ evmasked_op(opc, bt, opnd_array(2)->as_KRegister(ra_,this,idx2)/* mask */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* dst */,
                   opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* dst */, true, vlen_enc);
  
#line 46910 "ad_x86.cpp"
  }
}

void vabs_masked_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// mask
  {
    C2_MacroAssembler _masm(&cbuf);

#line 9852 "src/hotspot/cpu/x86/x86.ad"

    int vlen_enc = vector_length_encoding(this);
    BasicType bt = Matcher::vector_element_basic_type(this);
    int opc = this->ideal_Opcode();
    __ evmasked_op(opc, bt, opnd_array(2)->as_KRegister(ra_,this,idx2)/* mask */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* dst */,
                   opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* dst */, true, vlen_enc);
  
#line 46931 "ad_x86.cpp"
  }
}

void vabs_masked_1Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// mask
  {
    C2_MacroAssembler _masm(&cbuf);

#line 9852 "src/hotspot/cpu/x86/x86.ad"

    int vlen_enc = vector_length_encoding(this);
    BasicType bt = Matcher::vector_element_basic_type(this);
    int opc = this->ideal_Opcode();
    __ evmasked_op(opc, bt, opnd_array(2)->as_KRegister(ra_,this,idx2)/* mask */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* dst */,
                   opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* dst */, true, vlen_enc);
  
#line 46952 "ad_x86.cpp"
  }
}

void vabs_masked_2Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// mask
  {
    C2_MacroAssembler _masm(&cbuf);

#line 9852 "src/hotspot/cpu/x86/x86.ad"

    int vlen_enc = vector_length_encoding(this);
    BasicType bt = Matcher::vector_element_basic_type(this);
    int opc = this->ideal_Opcode();
    __ evmasked_op(opc, bt, opnd_array(2)->as_KRegister(ra_,this,idx2)/* mask */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* dst */,
                   opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* dst */, true, vlen_enc);
  
#line 46973 "ad_x86.cpp"
  }
}

void vfma_reg_maskedNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// src3
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// mask
  {
    C2_MacroAssembler _masm(&cbuf);

#line 9866 "src/hotspot/cpu/x86/x86.ad"

    int vlen_enc = vector_length_encoding(this);
    BasicType bt = Matcher::vector_element_basic_type(this);
    int opc = this->ideal_Opcode();
    __ evmasked_op(opc, bt, opnd_array(4)->as_KRegister(ra_,this,idx4)/* mask */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* dst */,
                   opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src2 */, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* src3 */, true, vlen_enc);
  
#line 46996 "ad_x86.cpp"
  }
}

void vfma_reg_masked_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// src3
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// mask
  {
    C2_MacroAssembler _masm(&cbuf);

#line 9866 "src/hotspot/cpu/x86/x86.ad"

    int vlen_enc = vector_length_encoding(this);
    BasicType bt = Matcher::vector_element_basic_type(this);
    int opc = this->ideal_Opcode();
    __ evmasked_op(opc, bt, opnd_array(4)->as_KRegister(ra_,this,idx4)/* mask */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* dst */,
                   opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src2 */, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* src3 */, true, vlen_enc);
  
#line 47019 "ad_x86.cpp"
  }
}

void vfma_mem_maskedNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// src3
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// mask
  {
    C2_MacroAssembler _masm(&cbuf);

#line 9880 "src/hotspot/cpu/x86/x86.ad"

    int vlen_enc = vector_length_encoding(this);
    BasicType bt = Matcher::vector_element_basic_type(this);
    int opc = this->ideal_Opcode();
    __ evmasked_op(opc, bt, opnd_array(4)->as_KRegister(ra_,this,idx4)/* mask */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* dst */,
                   opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src2 */, Address::make_raw(opnd_array(3)->base(ra_,this,idx3), opnd_array(3)->index(ra_,this,idx3), opnd_array(3)->scale(), opnd_array(3)->disp(ra_,this,idx3), opnd_array(3)->disp_reloc()), true, vlen_enc);
  
#line 47042 "ad_x86.cpp"
  }
}

void vfma_mem_masked_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// src3
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// mask
  {
    C2_MacroAssembler _masm(&cbuf);

#line 9880 "src/hotspot/cpu/x86/x86.ad"

    int vlen_enc = vector_length_encoding(this);
    BasicType bt = Matcher::vector_element_basic_type(this);
    int opc = this->ideal_Opcode();
    __ evmasked_op(opc, bt, opnd_array(4)->as_KRegister(ra_,this,idx4)/* mask */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* dst */,
                   opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src2 */, Address::make_raw(opnd_array(3)->base(ra_,this,idx3), opnd_array(3)->index(ra_,this,idx3), opnd_array(3)->scale(), opnd_array(3)->disp(ra_,this,idx3), opnd_array(3)->disp_reloc()), true, vlen_enc);
  
#line 47065 "ad_x86.cpp"
  }
}

void evcmp_maskedNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// cond
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// mask
  {
    C2_MacroAssembler _masm(&cbuf);

#line 9893 "src/hotspot/cpu/x86/x86.ad"

    assert(bottom_type()->isa_vectmask(), "TypeVectMask expected");
    int vlen_enc = vector_length_encoding(this, opnd_array(1));
    BasicType src1_elem_bt = Matcher::vector_element_basic_type(this, opnd_array(1));

    // Comparison i
    switch (src1_elem_bt) {
      case T_BYTE: {
        bool is_unsigned = Matcher::is_unsigned_booltest_pred(opnd_array(3)->constant());
        Assembler::ComparisonPredicate cmp = booltest_pred_to_comparison_pred(opnd_array(3)->constant());
        __ evpcmpb(opnd_array(0)->as_KRegister(ra_,this)/* dst */, opnd_array(4)->as_KRegister(ra_,this,idx4)/* mask */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src1 */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src2 */, cmp, !is_unsigned, vlen_enc);
        break;
      }
      case T_SHORT: {
        bool is_unsigned = Matcher::is_unsigned_booltest_pred(opnd_array(3)->constant());
        Assembler::ComparisonPredicate cmp = booltest_pred_to_comparison_pred(opnd_array(3)->constant());
        __ evpcmpw(opnd_array(0)->as_KRegister(ra_,this)/* dst */, opnd_array(4)->as_KRegister(ra_,this,idx4)/* mask */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src1 */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src2 */, cmp, !is_unsigned, vlen_enc);
        break;
      }
      case T_INT: {
        bool is_unsigned = Matcher::is_unsigned_booltest_pred(opnd_array(3)->constant());
        Assembler::ComparisonPredicate cmp = booltest_pred_to_comparison_pred(opnd_array(3)->constant());
        __ evpcmpd(opnd_array(0)->as_KRegister(ra_,this)/* dst */, opnd_array(4)->as_KRegister(ra_,this,idx4)/* mask */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src1 */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src2 */, cmp, !is_unsigned, vlen_enc);
        break;
      }
      case T_LONG: {
        bool is_unsigned = Matcher::is_unsigned_booltest_pred(opnd_array(3)->constant());
        Assembler::ComparisonPredicate cmp = booltest_pred_to_comparison_pred(opnd_array(3)->constant());
        __ evpcmpq(opnd_array(0)->as_KRegister(ra_,this)/* dst */, opnd_array(4)->as_KRegister(ra_,this,idx4)/* mask */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src1 */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src2 */, cmp, !is_unsigned, vlen_enc);
        break;
      }
      case T_FLOAT: {
        Assembler::ComparisonPredicateFP cmp = booltest_pred_to_comparison_pred_fp(opnd_array(3)->constant());
        __ evcmpps(opnd_array(0)->as_KRegister(ra_,this)/* dst */, opnd_array(4)->as_KRegister(ra_,this,idx4)/* mask */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src1 */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src2 */, cmp, vlen_enc);
        break;
      }
      case T_DOUBLE: {
        Assembler::ComparisonPredicateFP cmp = booltest_pred_to_comparison_pred_fp(opnd_array(3)->constant());
        __ evcmppd(opnd_array(0)->as_KRegister(ra_,this)/* dst */, opnd_array(4)->as_KRegister(ra_,this,idx4)/* mask */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src1 */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src2 */, cmp, vlen_enc);
        break;
      }
      default: assert(false, "%s", type2name(src1_elem_bt)); break;
    }
  
#line 47125 "ad_x86.cpp"
  }
}

void mask_all_evexI_LE32Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {
    C2_MacroAssembler _masm(&cbuf);

#line 9944 "src/hotspot/cpu/x86/x86.ad"

    int mask_len = Matcher::vector_length(this);
    __ vector_maskall_operation(opnd_array(0)->as_KRegister(ra_,this)/* dst */, opnd_array(1)->as_Register(ra_,this,idx1)/* src */, mask_len);
  
#line 47142 "ad_x86.cpp"
  }
}

void mask_opers_evexNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// kscratch
  {
    C2_MacroAssembler _masm(&cbuf);

#line 10024 "src/hotspot/cpu/x86/x86.ad"

    const MachNode* mask1 = static_cast<const MachNode*>(this->in(this->operand_index(opnd_array(1))));
    const MachNode* mask2 = static_cast<const MachNode*>(this->in(this->operand_index(opnd_array(2))));
    assert(0 == Type::cmp(mask1->bottom_type(), mask2->bottom_type()), "");
    uint masklen = Matcher::vector_length(this);
    masklen = (masklen < 16 && !VM_Version::supports_avx512dq()) ? 16 : masklen;
    __ masked_op(this->ideal_Opcode(), masklen, opnd_array(0)->as_KRegister(ra_,this)/* dst */, opnd_array(1)->as_KRegister(ra_,this,idx1)/* src1 */, opnd_array(2)->as_KRegister(ra_,this,idx2)/* src2 */);
  
#line 47165 "ad_x86.cpp"
  }
}

void mask_opers_evex_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// kscratch
  {
    C2_MacroAssembler _masm(&cbuf);

#line 10024 "src/hotspot/cpu/x86/x86.ad"

    const MachNode* mask1 = static_cast<const MachNode*>(this->in(this->operand_index(opnd_array(1))));
    const MachNode* mask2 = static_cast<const MachNode*>(this->in(this->operand_index(opnd_array(2))));
    assert(0 == Type::cmp(mask1->bottom_type(), mask2->bottom_type()), "");
    uint masklen = Matcher::vector_length(this);
    masklen = (masklen < 16 && !VM_Version::supports_avx512dq()) ? 16 : masklen;
    __ masked_op(this->ideal_Opcode(), masklen, opnd_array(0)->as_KRegister(ra_,this)/* dst */, opnd_array(1)->as_KRegister(ra_,this,idx1)/* src1 */, opnd_array(2)->as_KRegister(ra_,this,idx2)/* src2 */);
  
#line 47188 "ad_x86.cpp"
  }
}

void mask_opers_evex_1Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// kscratch
  {
    C2_MacroAssembler _masm(&cbuf);

#line 10024 "src/hotspot/cpu/x86/x86.ad"

    const MachNode* mask1 = static_cast<const MachNode*>(this->in(this->operand_index(opnd_array(1))));
    const MachNode* mask2 = static_cast<const MachNode*>(this->in(this->operand_index(opnd_array(2))));
    assert(0 == Type::cmp(mask1->bottom_type(), mask2->bottom_type()), "");
    uint masklen = Matcher::vector_length(this);
    masklen = (masklen < 16 && !VM_Version::supports_avx512dq()) ? 16 : masklen;
    __ masked_op(this->ideal_Opcode(), masklen, opnd_array(0)->as_KRegister(ra_,this)/* dst */, opnd_array(1)->as_KRegister(ra_,this,idx1)/* src1 */, opnd_array(2)->as_KRegister(ra_,this,idx2)/* src2 */);
  
#line 47211 "ad_x86.cpp"
  }
}

void vternlog_reg_maskedNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// src3
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// func
  unsigned idx5 = idx4 + opnd_array(4)->num_edges(); 	// mask
  {
    C2_MacroAssembler _masm(&cbuf);

#line 10038 "src/hotspot/cpu/x86/x86.ad"

    int vlen_enc = vector_length_encoding(this);
    BasicType bt = Matcher::vector_element_basic_type(this);
    __ evpternlog(opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* dst */, opnd_array(4)->constant(), opnd_array(5)->as_KRegister(ra_,this,idx5)/* mask */,
                  opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src2 */, opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* src3 */, true, bt, vlen_enc);
  
#line 47234 "ad_x86.cpp"
  }
}

void vternlogd_mem_maskedNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// src3
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// func
  unsigned idx5 = idx4 + opnd_array(4)->num_edges(); 	// mask
  {
    C2_MacroAssembler _masm(&cbuf);

#line 10050 "src/hotspot/cpu/x86/x86.ad"

    int vlen_enc = vector_length_encoding(this);
    BasicType bt = Matcher::vector_element_basic_type(this);
    __ evpternlog(opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* dst */, opnd_array(4)->constant(), opnd_array(5)->as_KRegister(ra_,this,idx5)/* mask */,
                  opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src2 */, Address::make_raw(opnd_array(3)->base(ra_,this,idx3), opnd_array(3)->index(ra_,this,idx3), opnd_array(3)->scale(), opnd_array(3)->disp(ra_,this,idx3), opnd_array(3)->disp_reloc()), true, bt, vlen_enc);
  
#line 47257 "ad_x86.cpp"
  }
}

void castMMNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  // User did not define which encode class to use.
}

uint castMMNode::size(PhaseRegAlloc *ra_) const {
  assert(VerifyOops || MachNode::size(ra_) <= 0, "bad fixed size");
  return (VerifyOops ? MachNode::size(ra_) : 0);
}

void castVVNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  // User did not define which encode class to use.
}

uint castVVNode::size(PhaseRegAlloc *ra_) const {
  assert(VerifyOops || MachNode::size(ra_) <= 0, "bad fixed size");
  return (VerifyOops ? MachNode::size(ra_) : 0);
}

void castVVLegNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  // User did not define which encode class to use.
}

uint castVVLegNode::size(PhaseRegAlloc *ra_) const {
  assert(VerifyOops || MachNode::size(ra_) <= 0, "bad fixed size");
  return (VerifyOops ? MachNode::size(ra_) : 0);
}

void FloatClassCheck_reg_reg_vfpclassNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// ktmp
  {
    C2_MacroAssembler _masm(&cbuf);

#line 10097 "src/hotspot/cpu/x86/x86.ad"

    __ vfpclassss(opnd_array(2)->as_KRegister(ra_,this,idx2)/* ktmp */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, 0x18);
    __ kmovbl(opnd_array(0)->as_Register(ra_,this)/* dst */, opnd_array(2)->as_KRegister(ra_,this,idx2)/* ktmp */);
  
#line 47314 "ad_x86.cpp"
  }
}

void DoubleClassCheck_reg_reg_vfpclassNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// ktmp
  {
    C2_MacroAssembler _masm(&cbuf);

#line 10109 "src/hotspot/cpu/x86/x86.ad"

    __ vfpclasssd(opnd_array(2)->as_KRegister(ra_,this,idx2)/* ktmp */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, 0x18);
    __ kmovbl(opnd_array(0)->as_Register(ra_,this)/* dst */, opnd_array(2)->as_KRegister(ra_,this,idx2)/* ktmp */);
  
#line 47332 "ad_x86.cpp"
  }
}

void compareAndSwapP_shenandoahNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem_ptr
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// oldval
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// newval
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// tmp1
  unsigned idx5 = idx4 + opnd_array(4)->num_edges(); 	// tmp2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 42 "src/hotspot/cpu/x86/gc/shenandoah/shenandoah_x86_32.ad"

    ShenandoahBarrierSet::assembler()->cmpxchg_oop(&_masm,
                                                   opnd_array(0)->as_Register(ra_,this)/* res */, Address::make_raw(opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1), opnd_array(1)->disp_reloc()), opnd_array(2)->as_Register(ra_,this,idx2)/* oldval */, opnd_array(3)->as_Register(ra_,this,idx3)/* newval */,
                                                   false, // swap
                                                   opnd_array(4)->as_Register(ra_,this,idx4)/* tmp1 */, opnd_array(5)->as_Register(ra_,this,idx5)/* tmp2 */);
  
#line 47355 "ad_x86.cpp"
  }
}

void compareAndSwapP_shenandoah_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem_ptr
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// oldval
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// newval
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// tmp1
  unsigned idx5 = idx4 + opnd_array(4)->num_edges(); 	// tmp2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 42 "src/hotspot/cpu/x86/gc/shenandoah/shenandoah_x86_32.ad"

    ShenandoahBarrierSet::assembler()->cmpxchg_oop(&_masm,
                                                   opnd_array(0)->as_Register(ra_,this)/* res */, Address::make_raw(opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1), opnd_array(1)->disp_reloc()), opnd_array(2)->as_Register(ra_,this,idx2)/* oldval */, opnd_array(3)->as_Register(ra_,this,idx3)/* newval */,
                                                   false, // swap
                                                   opnd_array(4)->as_Register(ra_,this,idx4)/* tmp1 */, opnd_array(5)->as_Register(ra_,this,idx5)/* tmp2 */);
  
#line 47378 "ad_x86.cpp"
  }
}

void compareAndExchangeP_shenandoahNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem_ptr
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// oldval
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// newval
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// tmp1
  unsigned idx5 = idx4 + opnd_array(4)->num_edges(); 	// tmp2
  {
    C2_MacroAssembler _masm(&cbuf);

#line 63 "src/hotspot/cpu/x86/gc/shenandoah/shenandoah_x86_32.ad"

    ShenandoahBarrierSet::assembler()->cmpxchg_oop(&_masm,
                                                   noreg, Address::make_raw(opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1), opnd_array(1)->disp_reloc()), opnd_array(2)->as_Register(ra_,this,idx2)/* oldval */, opnd_array(3)->as_Register(ra_,this,idx3)/* newval */,
                                                   true,  // exchange
                                                   opnd_array(4)->as_Register(ra_,this,idx4)/* tmp1 */, opnd_array(5)->as_Register(ra_,this,idx5)/* tmp2 */);
  
#line 47401 "ad_x86.cpp"
  }
}

const MachOper* popCountI_memNode::memory_operand() const { return _opnds[1]; }
const MachOper* popCountL_memNode::memory_operand() const { return _opnds[1]; }
const MachOper* loadBNode::memory_operand() const { return _opnds[1]; }
const MachOper* loadB2LNode::memory_operand() const { return _opnds[1]; }
const MachOper* loadUBNode::memory_operand() const { return _opnds[1]; }
const MachOper* loadUB2LNode::memory_operand() const { return _opnds[1]; }
const MachOper* loadUB2L_immINode::memory_operand() const { return _opnds[1]; }
const MachOper* loadSNode::memory_operand() const { return _opnds[1]; }
const MachOper* loadS2BNode::memory_operand() const { return _opnds[1]; }
const MachOper* loadS2LNode::memory_operand() const { return _opnds[1]; }
const MachOper* loadUSNode::memory_operand() const { return _opnds[1]; }
const MachOper* loadUS2BNode::memory_operand() const { return _opnds[1]; }
const MachOper* loadUS2LNode::memory_operand() const { return _opnds[1]; }
const MachOper* loadUS2L_immI_255Node::memory_operand() const { return _opnds[1]; }
const MachOper* loadUS2L_immINode::memory_operand() const { return _opnds[1]; }
const MachOper* loadINode::memory_operand() const { return _opnds[1]; }
const MachOper* loadI2BNode::memory_operand() const { return _opnds[1]; }
const MachOper* loadI2UBNode::memory_operand() const { return _opnds[1]; }
const MachOper* loadI2SNode::memory_operand() const { return _opnds[1]; }
const MachOper* loadI2USNode::memory_operand() const { return _opnds[1]; }
const MachOper* loadI2LNode::memory_operand() const { return _opnds[1]; }
const MachOper* loadI2L_immI_255Node::memory_operand() const { return _opnds[1]; }
const MachOper* loadI2L_immI_65535Node::memory_operand() const { return _opnds[1]; }
const MachOper* loadI2L_immU31Node::memory_operand() const { return _opnds[1]; }
const MachOper* loadUI2LNode::memory_operand() const { return _opnds[1]; }
const MachOper* loadLNode::memory_operand() const { return _opnds[1]; }
const MachOper* loadL_volatileNode::memory_operand() const { return _opnds[1]; }
const MachOper* loadLX_volatileNode::memory_operand() const { return _opnds[1]; }
const MachOper* loadLX_reg_volatileNode::memory_operand() const { return _opnds[1]; }
const MachOper* loadRangeNode::memory_operand() const { return _opnds[1]; }
const MachOper* loadPNode::memory_operand() const { return _opnds[1]; }
const MachOper* loadKlassNode::memory_operand() const { return _opnds[1]; }
const MachOper* loadDPRNode::memory_operand() const { return _opnds[1]; }
const MachOper* loadDNode::memory_operand() const { return _opnds[1]; }
const MachOper* loadD_partialNode::memory_operand() const { return _opnds[1]; }
const MachOper* loadFNode::memory_operand() const { return _opnds[1]; }
const MachOper* loadFPRNode::memory_operand() const { return _opnds[1]; }
const MachOper* prefetchAlloc0Node::memory_operand() const { return _opnds[1]; }
const MachOper* prefetchAllocNode::memory_operand() const { return _opnds[1]; }
const MachOper* prefetchAllocNTANode::memory_operand() const { return _opnds[1]; }
const MachOper* prefetchAllocT0Node::memory_operand() const { return _opnds[1]; }
const MachOper* prefetchAllocT2Node::memory_operand() const { return _opnds[1]; }
const MachOper* storeBNode::memory_operand() const { return _opnds[1]; }
const MachOper* storeCNode::memory_operand() const { return _opnds[1]; }
const MachOper* storeINode::memory_operand() const { return _opnds[1]; }
const MachOper* storeLNode::memory_operand() const { return _opnds[1]; }
const MachOper* storeL2INode::memory_operand() const { return _opnds[1]; }
const MachOper* storeL_volatileNode::memory_operand() const { return _opnds[1]; }
const MachOper* storeLX_volatileNode::memory_operand() const { return _opnds[1]; }
const MachOper* storeLX_reg_volatileNode::memory_operand() const { return _opnds[1]; }
const MachOper* storePNode::memory_operand() const { return _opnds[1]; }
const MachOper* storeImmINode::memory_operand() const { return _opnds[1]; }
const MachOper* storeImmI16Node::memory_operand() const { return _opnds[1]; }
const MachOper* storeImmPNode::memory_operand() const { return _opnds[1]; }
const MachOper* storeImmBNode::memory_operand() const { return _opnds[1]; }
const MachOper* storeImmCMNode::memory_operand() const { return _opnds[1]; }
const MachOper* storeDPRNode::memory_operand() const { return _opnds[1]; }
const MachOper* storeDPR_roundedNode::memory_operand() const { return _opnds[1]; }
const MachOper* storeDNode::memory_operand() const { return _opnds[1]; }
const MachOper* storeFNode::memory_operand() const { return _opnds[1]; }
const MachOper* storeFPRNode::memory_operand() const { return _opnds[1]; }
const MachOper* storeFPR_roundedNode::memory_operand() const { return _opnds[1]; }
const MachOper* storeFPR_DroundedNode::memory_operand() const { return _opnds[1]; }
const MachOper* storeFPR_immNode::memory_operand() const { return _opnds[1]; }
const MachOper* storeF_immNode::memory_operand() const { return _opnds[1]; }
const MachOper* cmovI_memNode::memory_operand() const { return _opnds[4]; }
const MachOper* cmovI_memUNode::memory_operand() const { return _opnds[4]; }
const MachOper* cmovI_memUCFNode::memory_operand() const { return _opnds[4]; }
const MachOper* addI_eReg_memNode::memory_operand() const { return _opnds[2]; }
const MachOper* addI_eReg_mem_0Node::memory_operand() const { return _opnds[1]; }
const MachOper* addI_mem_eRegNode::memory_operand() const { return _opnds[1]; }
const MachOper* addI_mem_eReg_0Node::memory_operand() const { return _opnds[1]; }
const MachOper* addI_mem_immNode::memory_operand() const { return _opnds[1]; }
const MachOper* incI_memNode::memory_operand() const { return _opnds[1]; }
const MachOper* decI_memNode::memory_operand() const { return _opnds[1]; }
const MachOper* xaddB_no_resNode::memory_operand() const { return _opnds[1]; }
const MachOper* xaddBNode::memory_operand() const { return _opnds[1]; }
const MachOper* xaddS_no_resNode::memory_operand() const { return _opnds[1]; }
const MachOper* xaddSNode::memory_operand() const { return _opnds[1]; }
const MachOper* xaddI_no_resNode::memory_operand() const { return _opnds[1]; }
const MachOper* xaddINode::memory_operand() const { return _opnds[1]; }
const MachOper* xchgBNode::memory_operand() const { return _opnds[1]; }
const MachOper* xchgSNode::memory_operand() const { return _opnds[1]; }
const MachOper* xchgINode::memory_operand() const { return _opnds[1]; }
const MachOper* xchgPNode::memory_operand() const { return _opnds[1]; }
const MachOper* subI_eReg_memNode::memory_operand() const { return _opnds[2]; }
const MachOper* subI_mem_eRegNode::memory_operand() const { return _opnds[1]; }
const MachOper* mulI_mem_immNode::memory_operand() const { return _opnds[1]; }
const MachOper* mulINode::memory_operand() const { return _opnds[2]; }
const MachOper* mulI_0Node::memory_operand() const { return _opnds[1]; }
const MachOper* sarI_mem_1Node::memory_operand() const { return _opnds[1]; }
const MachOper* sarI_mem_immNode::memory_operand() const { return _opnds[1]; }
const MachOper* andI_eReg_memNode::memory_operand() const { return _opnds[2]; }
const MachOper* andI_eReg_mem_0Node::memory_operand() const { return _opnds[1]; }
const MachOper* andI_mem_eRegNode::memory_operand() const { return _opnds[1]; }
const MachOper* andI_mem_eReg_0Node::memory_operand() const { return _opnds[1]; }
const MachOper* andI_mem_immNode::memory_operand() const { return _opnds[1]; }
const MachOper* andnI_rReg_rReg_memNode::memory_operand() const { return _opnds[3]; }
const MachOper* andnI_rReg_rReg_mem_0Node::memory_operand() const { return _opnds[1]; }
const MachOper* blsiI_rReg_memNode::memory_operand() const { return _opnds[2]; }
const MachOper* blsiI_rReg_mem_0Node::memory_operand() const { return _opnds[1]; }
const MachOper* blsmskI_rReg_memNode::memory_operand() const { return _opnds[1]; }
const MachOper* blsmskI_rReg_mem_0Node::memory_operand() const { return _opnds[1]; }
const MachOper* blsrI_rReg_memNode::memory_operand() const { return _opnds[1]; }
const MachOper* blsrI_rReg_mem_0Node::memory_operand() const { return _opnds[1]; }
const MachOper* orI_eReg_memNode::memory_operand() const { return _opnds[2]; }
const MachOper* orI_eReg_mem_0Node::memory_operand() const { return _opnds[1]; }
const MachOper* orI_mem_eRegNode::memory_operand() const { return _opnds[1]; }
const MachOper* orI_mem_eReg_0Node::memory_operand() const { return _opnds[1]; }
const MachOper* orI_mem_immNode::memory_operand() const { return _opnds[1]; }
const MachOper* xorI_eReg_memNode::memory_operand() const { return _opnds[2]; }
const MachOper* xorI_eReg_mem_0Node::memory_operand() const { return _opnds[1]; }
const MachOper* xorI_mem_eRegNode::memory_operand() const { return _opnds[1]; }
const MachOper* xorI_mem_eReg_0Node::memory_operand() const { return _opnds[1]; }
const MachOper* xorI_mem_immNode::memory_operand() const { return _opnds[1]; }
const MachOper* addL_eReg_memNode::memory_operand() const { return _opnds[2]; }
const MachOper* addL_eReg_mem_0Node::memory_operand() const { return _opnds[1]; }
const MachOper* subL_eReg_memNode::memory_operand() const { return _opnds[2]; }
const MachOper* andL_eReg_memNode::memory_operand() const { return _opnds[2]; }
const MachOper* andL_eReg_mem_0Node::memory_operand() const { return _opnds[1]; }
const MachOper* andnL_eReg_eReg_memNode::memory_operand() const { return _opnds[3]; }
const MachOper* andnL_eReg_eReg_mem_0Node::memory_operand() const { return _opnds[1]; }
const MachOper* blsiL_eReg_memNode::memory_operand() const { return _opnds[2]; }
const MachOper* blsiL_eReg_mem_0Node::memory_operand() const { return _opnds[1]; }
const MachOper* blsmskL_eReg_memNode::memory_operand() const { return _opnds[1]; }
const MachOper* blsmskL_eReg_mem_0Node::memory_operand() const { return _opnds[1]; }
const MachOper* blsrL_eReg_memNode::memory_operand() const { return _opnds[1]; }
const MachOper* blsrL_eReg_mem_0Node::memory_operand() const { return _opnds[1]; }
const MachOper* orl_eReg_memNode::memory_operand() const { return _opnds[2]; }
const MachOper* orl_eReg_mem_0Node::memory_operand() const { return _opnds[1]; }
const MachOper* xorl_eReg_memNode::memory_operand() const { return _opnds[2]; }
const MachOper* xorl_eReg_mem_0Node::memory_operand() const { return _opnds[1]; }
const MachOper* cmpD_ccmemNode::memory_operand() const { return _opnds[2]; }
const MachOper* cmpD_ccmemCFNode::memory_operand() const { return _opnds[2]; }
const MachOper* cmpD_regmemNode::memory_operand() const { return _opnds[2]; }
const MachOper* subDPR_reg_memNode::memory_operand() const { return _opnds[2]; }
const MachOper* addDPR_reg_memNode::memory_operand() const { return _opnds[2]; }
const MachOper* addDPR_reg_mem_0Node::memory_operand() const { return _opnds[1]; }
const MachOper* addDPR_mem_regNode::memory_operand() const { return _opnds[1]; }
const MachOper* addDPR_mem_reg_0Node::memory_operand() const { return _opnds[1]; }
const MachOper* mulDPR_reg_memNode::memory_operand() const { return _opnds[2]; }
const MachOper* mulDPR_reg_mem_0Node::memory_operand() const { return _opnds[1]; }
const MachOper* mulDPR_reg_mem_ciscNode::memory_operand() const { return _opnds[2]; }
const MachOper* mulDPR_reg_mem_cisc_0Node::memory_operand() const { return _opnds[1]; }
const MachOper* cmpF_ccmemNode::memory_operand() const { return _opnds[2]; }
const MachOper* cmpF_ccmemCFNode::memory_operand() const { return _opnds[2]; }
const MachOper* cmpF_regmemNode::memory_operand() const { return _opnds[2]; }
const MachOper* addFPR24_reg_memNode::memory_operand() const { return _opnds[2]; }
const MachOper* addFPR24_reg_mem_0Node::memory_operand() const { return _opnds[1]; }
const MachOper* addFPR_reg_memNode::memory_operand() const { return _opnds[2]; }
const MachOper* addFPR_reg_mem_0Node::memory_operand() const { return _opnds[1]; }
const MachOper* addFPR24_mem_regNode::memory_operand() const { return _opnds[1]; }
const TypePtr *addFPR24_mem_ciscNode::adr_type() const { return TypePtr::BOTTOM; }
const MachOper* addFPR24_mem_ciscNode::memory_operand() const { return (MachOper*)-1; }
const TypePtr *addFPR24_mem_cisc_0Node::adr_type() const { return TypePtr::BOTTOM; }
const MachOper* addFPR24_mem_cisc_0Node::memory_operand() const { return (MachOper*)-1; }
const TypePtr *addFPR24_mem_memNode::adr_type() const { return TypePtr::BOTTOM; }
const MachOper* addFPR24_mem_memNode::memory_operand() const { return (MachOper*)-1; }
const MachOper* mulFPR24_reg_memNode::memory_operand() const { return _opnds[2]; }
const MachOper* mulFPR24_reg_mem_0Node::memory_operand() const { return _opnds[1]; }
const MachOper* mulFPR_reg_memNode::memory_operand() const { return _opnds[2]; }
const MachOper* mulFPR_reg_mem_0Node::memory_operand() const { return _opnds[1]; }
const TypePtr *mulFPR24_mem_memNode::adr_type() const { return TypePtr::BOTTOM; }
const MachOper* mulFPR24_mem_memNode::memory_operand() const { return (MachOper*)-1; }
const MachOper* mulFPR_reg_load1Node::memory_operand() const { return _opnds[1]; }
const MachOper* mulFPR_reg_load1_0Node::memory_operand() const { return _opnds[2]; }
const MachOper* addFPR_mulFPR_reg_load1Node::memory_operand() const { return _opnds[1]; }
const MachOper* addFPR_mulFPR_reg_load1_1Node::memory_operand() const { return _opnds[2]; }
const MachOper* addFPR_mulFPR_reg_load1_0Node::memory_operand() const { return _opnds[2]; }
const MachOper* addFPR_mulFPR_reg_load1_2Node::memory_operand() const { return _opnds[3]; }
const MachOper* convI2D_memNode::memory_operand() const { return _opnds[1]; }
const MachOper* convI2DPR_memNode::memory_operand() const { return _opnds[1]; }
const MachOper* convI2FPR_SSF_memNode::memory_operand() const { return _opnds[1]; }
const MachOper* convI2FPR_memNode::memory_operand() const { return _opnds[1]; }
const MachOper* compI_eReg_memNode::memory_operand() const { return _opnds[2]; }
const MachOper* testI_reg_memNode::memory_operand() const { return _opnds[2]; }
const MachOper* compU_eReg_memNode::memory_operand() const { return _opnds[2]; }
const MachOper* compP_eReg_memNode::memory_operand() const { return _opnds[2]; }
const MachOper* compP_mem_eRegNode::memory_operand() const { return _opnds[2]; }
const MachOper* testP_Reg_memNode::memory_operand() const { return _opnds[1]; }
const MachOper* cmovLL_mem_LTGENode::memory_operand() const { return _opnds[4]; }
const MachOper* cmovLL_mem_LTGE_UNode::memory_operand() const { return _opnds[4]; }
const MachOper* cmovII_mem_LTGENode::memory_operand() const { return _opnds[4]; }
const MachOper* cmovII_mem_LTGE_UNode::memory_operand() const { return _opnds[4]; }
const MachOper* cmovLL_mem_EQNENode::memory_operand() const { return _opnds[4]; }
const MachOper* cmovII_mem_EQNENode::memory_operand() const { return _opnds[4]; }
const MachOper* cmovII_mem_EQNE_UNode::memory_operand() const { return _opnds[4]; }
const MachOper* cmovLL_mem_LEGTNode::memory_operand() const { return _opnds[4]; }
const MachOper* cmovLL_mem_LEGT_UNode::memory_operand() const { return _opnds[4]; }
const MachOper* cmovII_mem_LEGTNode::memory_operand() const { return _opnds[4]; }
const MachOper* cmovII_mem_LEGT_UNode::memory_operand() const { return _opnds[4]; }
const MachOper* addF_memNode::memory_operand() const { return _opnds[2]; }
const MachOper* addF_mem_0Node::memory_operand() const { return _opnds[1]; }
const MachOper* addF_reg_memNode::memory_operand() const { return _opnds[2]; }
const MachOper* addF_reg_mem_0Node::memory_operand() const { return _opnds[1]; }
const MachOper* addD_memNode::memory_operand() const { return _opnds[2]; }
const MachOper* addD_mem_0Node::memory_operand() const { return _opnds[1]; }
const MachOper* addD_reg_memNode::memory_operand() const { return _opnds[2]; }
const MachOper* addD_reg_mem_0Node::memory_operand() const { return _opnds[1]; }
const MachOper* subF_memNode::memory_operand() const { return _opnds[2]; }
const MachOper* subF_reg_memNode::memory_operand() const { return _opnds[2]; }
const MachOper* subD_memNode::memory_operand() const { return _opnds[2]; }
const MachOper* subD_reg_memNode::memory_operand() const { return _opnds[2]; }
const MachOper* mulF_memNode::memory_operand() const { return _opnds[2]; }
const MachOper* mulF_mem_0Node::memory_operand() const { return _opnds[1]; }
const MachOper* mulF_reg_memNode::memory_operand() const { return _opnds[2]; }
const MachOper* mulF_reg_mem_0Node::memory_operand() const { return _opnds[1]; }
const MachOper* mulD_memNode::memory_operand() const { return _opnds[2]; }
const MachOper* mulD_mem_0Node::memory_operand() const { return _opnds[1]; }
const MachOper* mulD_reg_memNode::memory_operand() const { return _opnds[2]; }
const MachOper* mulD_reg_mem_0Node::memory_operand() const { return _opnds[1]; }
const MachOper* divF_memNode::memory_operand() const { return _opnds[2]; }
const MachOper* divF_reg_memNode::memory_operand() const { return _opnds[2]; }
const MachOper* divD_memNode::memory_operand() const { return _opnds[2]; }
const MachOper* divD_reg_memNode::memory_operand() const { return _opnds[2]; }
const MachOper* convF2HF_mem_regNode::memory_operand() const { return _opnds[1]; }
const MachOper* vconvF2HF_mem_regNode::memory_operand() const { return _opnds[1]; }
const MachOper* vconvHF2F_reg_memNode::memory_operand() const { return _opnds[1]; }
const MachOper* loadVNode::memory_operand() const { return _opnds[1]; }
const MachOper* storeVNode::memory_operand() const { return _opnds[1]; }
const MachOper* gatherNode::memory_operand() const { return _opnds[1]; }
const MachOper* evgatherNode::memory_operand() const { return _opnds[1]; }
const MachOper* evgather_maskedNode::memory_operand() const { return _opnds[1]; }
const MachOper* scatterNode::memory_operand() const { return _opnds[1]; }
const MachOper* scatter_maskedNode::memory_operand() const { return _opnds[1]; }
const MachOper* ReplB_memNode::memory_operand() const { return _opnds[1]; }
const MachOper* ReplS_memNode::memory_operand() const { return _opnds[1]; }
const MachOper* ReplI_memNode::memory_operand() const { return _opnds[1]; }
const MachOper* ReplL_memNode::memory_operand() const { return _opnds[1]; }
const MachOper* ReplF_memNode::memory_operand() const { return _opnds[1]; }
const MachOper* ReplD_memNode::memory_operand() const { return _opnds[1]; }
const MachOper* vaddB_memNode::memory_operand() const { return _opnds[2]; }
const MachOper* vaddB_mem_0Node::memory_operand() const { return _opnds[1]; }
const MachOper* vaddS_memNode::memory_operand() const { return _opnds[2]; }
const MachOper* vaddS_mem_0Node::memory_operand() const { return _opnds[1]; }
const MachOper* vaddI_memNode::memory_operand() const { return _opnds[2]; }
const MachOper* vaddI_mem_0Node::memory_operand() const { return _opnds[1]; }
const MachOper* vaddL_memNode::memory_operand() const { return _opnds[2]; }
const MachOper* vaddL_mem_0Node::memory_operand() const { return _opnds[1]; }
const MachOper* vaddF_memNode::memory_operand() const { return _opnds[2]; }
const MachOper* vaddF_mem_0Node::memory_operand() const { return _opnds[1]; }
const MachOper* vaddD_memNode::memory_operand() const { return _opnds[2]; }
const MachOper* vaddD_mem_0Node::memory_operand() const { return _opnds[1]; }
const MachOper* vsubB_memNode::memory_operand() const { return _opnds[2]; }
const MachOper* vsubS_memNode::memory_operand() const { return _opnds[2]; }
const MachOper* vsubI_memNode::memory_operand() const { return _opnds[2]; }
const MachOper* vsubL_memNode::memory_operand() const { return _opnds[2]; }
const MachOper* vsubF_memNode::memory_operand() const { return _opnds[2]; }
const MachOper* vsubD_memNode::memory_operand() const { return _opnds[2]; }
const MachOper* vmulS_memNode::memory_operand() const { return _opnds[2]; }
const MachOper* vmulS_mem_0Node::memory_operand() const { return _opnds[1]; }
const MachOper* vmulI_memNode::memory_operand() const { return _opnds[2]; }
const MachOper* vmulI_mem_0Node::memory_operand() const { return _opnds[1]; }
const MachOper* evmulL_memNode::memory_operand() const { return _opnds[2]; }
const MachOper* evmulL_mem_0Node::memory_operand() const { return _opnds[1]; }
const MachOper* vmulF_memNode::memory_operand() const { return _opnds[2]; }
const MachOper* vmulF_mem_0Node::memory_operand() const { return _opnds[1]; }
const MachOper* vmulD_memNode::memory_operand() const { return _opnds[2]; }
const MachOper* vmulD_mem_0Node::memory_operand() const { return _opnds[1]; }
const MachOper* vdivF_memNode::memory_operand() const { return _opnds[2]; }
const MachOper* vdivD_memNode::memory_operand() const { return _opnds[2]; }
const MachOper* compressBitsI_memNode::memory_operand() const { return _opnds[2]; }
const MachOper* expandBitsI_memNode::memory_operand() const { return _opnds[2]; }
const MachOper* vsqrtF_memNode::memory_operand() const { return _opnds[1]; }
const MachOper* vsqrtD_memNode::memory_operand() const { return _opnds[1]; }
const MachOper* vand_memNode::memory_operand() const { return _opnds[2]; }
const MachOper* vand_mem_0Node::memory_operand() const { return _opnds[1]; }
const MachOper* vor_memNode::memory_operand() const { return _opnds[2]; }
const MachOper* vor_mem_0Node::memory_operand() const { return _opnds[1]; }
const MachOper* vxor_memNode::memory_operand() const { return _opnds[2]; }
const MachOper* vxor_mem_0Node::memory_operand() const { return _opnds[1]; }
const MachOper* vfmaF_memNode::memory_operand() const { return _opnds[3]; }
const MachOper* vfmaD_memNode::memory_operand() const { return _opnds[3]; }
const MachOper* vpternlog_memNode::memory_operand() const { return _opnds[3]; }
const MachOper* vmasked_load_avx_non_subwordNode::memory_operand() const { return _opnds[1]; }
const MachOper* vmasked_load_evexNode::memory_operand() const { return _opnds[1]; }
const MachOper* vmasked_store_avx_non_subwordNode::memory_operand() const { return _opnds[1]; }
const MachOper* vmasked_store_evexNode::memory_operand() const { return _opnds[1]; }
const MachOper* vadd_mem_maskedNode::memory_operand() const { return _opnds[2]; }
const MachOper* vadd_mem_masked_0Node::memory_operand() const { return _opnds[2]; }
const MachOper* vadd_mem_masked_1Node::memory_operand() const { return _opnds[2]; }
const MachOper* vadd_mem_masked_2Node::memory_operand() const { return _opnds[2]; }
const MachOper* vadd_mem_masked_3Node::memory_operand() const { return _opnds[2]; }
const MachOper* vadd_mem_masked_4Node::memory_operand() const { return _opnds[2]; }
const MachOper* vxor_mem_maskedNode::memory_operand() const { return _opnds[2]; }
const MachOper* vor_mem_maskedNode::memory_operand() const { return _opnds[2]; }
const MachOper* vand_mem_maskedNode::memory_operand() const { return _opnds[2]; }
const MachOper* vsub_mem_maskedNode::memory_operand() const { return _opnds[2]; }
const MachOper* vsub_mem_masked_0Node::memory_operand() const { return _opnds[2]; }
const MachOper* vsub_mem_masked_1Node::memory_operand() const { return _opnds[2]; }
const MachOper* vsub_mem_masked_2Node::memory_operand() const { return _opnds[2]; }
const MachOper* vsub_mem_masked_3Node::memory_operand() const { return _opnds[2]; }
const MachOper* vsub_mem_masked_4Node::memory_operand() const { return _opnds[2]; }
const MachOper* vmul_mem_maskedNode::memory_operand() const { return _opnds[2]; }
const MachOper* vmul_mem_masked_0Node::memory_operand() const { return _opnds[2]; }
const MachOper* vmul_mem_masked_1Node::memory_operand() const { return _opnds[2]; }
const MachOper* vmul_mem_masked_2Node::memory_operand() const { return _opnds[2]; }
const MachOper* vmul_mem_masked_3Node::memory_operand() const { return _opnds[2]; }
const MachOper* vdiv_mem_maskedNode::memory_operand() const { return _opnds[2]; }
const MachOper* vdiv_mem_masked_0Node::memory_operand() const { return _opnds[2]; }
const MachOper* vlshift_mem_maskedNode::memory_operand() const { return _opnds[2]; }
const MachOper* vlshift_mem_masked_0Node::memory_operand() const { return _opnds[2]; }
const MachOper* vlshift_mem_masked_1Node::memory_operand() const { return _opnds[2]; }
const MachOper* vrshift_mem_maskedNode::memory_operand() const { return _opnds[2]; }
const MachOper* vrshift_mem_masked_0Node::memory_operand() const { return _opnds[2]; }
const MachOper* vrshift_mem_masked_1Node::memory_operand() const { return _opnds[2]; }
const MachOper* vurshift_mem_maskedNode::memory_operand() const { return _opnds[2]; }
const MachOper* vurshift_mem_masked_0Node::memory_operand() const { return _opnds[2]; }
const MachOper* vurshift_mem_masked_1Node::memory_operand() const { return _opnds[2]; }
const MachOper* vmaxv_mem_maskedNode::memory_operand() const { return _opnds[2]; }
const MachOper* vminv_mem_maskedNode::memory_operand() const { return _opnds[2]; }
const MachOper* vfma_mem_maskedNode::memory_operand() const { return _opnds[3]; }
const MachOper* vfma_mem_masked_0Node::memory_operand() const { return _opnds[3]; }
const MachOper* vternlogd_mem_maskedNode::memory_operand() const { return _opnds[3]; }
const MachOper* compareAndSwapP_shenandoahNode::memory_operand() const { return _opnds[1]; }
const MachOper* compareAndSwapP_shenandoah_0Node::memory_operand() const { return _opnds[1]; }
const MachOper* compareAndExchangeP_shenandoahNode::memory_operand() const { return _opnds[1]; }


const bool Matcher::has_match_rule(int opcode) {
  assert(_last_machine_leaf < opcode && opcode < _last_opcode, "opcode in range");
  return _hasMatchRule[opcode];
}

const bool Matcher::_hasMatchRule[_last_opcode] = {
    false,  // Node
    false,  // Set
    false,  // RegN
    false,  // RegI
    false,  // RegP
    false,  // RegF
    false,  // RegD
    false,  // RegL
    false,  // VecA
    false,  // VecS
    false,  // VecD
    false,  // VecX
    false,  // VecY
    false,  // VecZ
    false,  // RegVectMask
    false,  // RegFlags
    false,  // _last_machine_leaf
    true ,  // AbsD
    true ,  // AbsF
    true ,  // AbsI
    false,  // AbsL
    true ,  // AddD
    true ,  // AddF
    true ,  // AddI
    true ,  // AddL
    true ,  // AddP
    false,  // Allocate
    false,  // AllocateArray
    true ,  // AndI
    true ,  // AndL
    false,  // ArrayCopy
    true ,  // AryEq
    true ,  // AtanD
    true ,  // Binary
    false,  // Blackhole
    true ,  // Bool
    false,  // BoxLock
    true ,  // ReverseBytesI
    true ,  // ReverseBytesL
    true ,  // ReverseBytesUS
    true ,  // ReverseBytesS
    true ,  // ReverseBytesV
    false,  // CProj
    false,  // CacheWB
    false,  // CacheWBPreSync
    false,  // CacheWBPostSync
    true ,  // CallDynamicJava
    false,  // CallJava
    true ,  // CallLeaf
    true ,  // CallLeafNoFP
    false,  // CallLeafVector
    true ,  // CallRuntime
    true ,  // CallStaticJava
    true ,  // CastDD
    true ,  // CastFF
    true ,  // CastII
    true ,  // CastLL
    true ,  // CastVV
    true ,  // CastX2P
    true ,  // CastP2X
    true ,  // CastPP
    false,  // Catch
    false,  // CatchProj
    true ,  // CheckCastPP
    true ,  // ClearArray
    true ,  // CompressBits
    true ,  // ExpandBits
    false,  // CompressBitsV
    false,  // ExpandBitsV
    false,  // ConstraintCast
    true ,  // CMoveD
    true ,  // CMoveF
    true ,  // CMoveI
    true ,  // CMoveL
    true ,  // CMoveP
    false,  // CMoveN
    false,  // CmpN
    true ,  // CmpD
    true ,  // CmpD3
    true ,  // CmpF
    true ,  // CmpF3
    true ,  // CmpI
    true ,  // CmpL
    true ,  // CmpL3
    true ,  // CmpLTMask
    true ,  // CmpP
    true ,  // CmpU
    false,  // CmpU3
    true ,  // CmpUL
    false,  // CmpUL3
    true ,  // CompareAndSwapB
    true ,  // CompareAndSwapS
    true ,  // CompareAndSwapI
    true ,  // CompareAndSwapL
    true ,  // CompareAndSwapP
    false,  // CompareAndSwapN
    true ,  // WeakCompareAndSwapB
    true ,  // WeakCompareAndSwapS
    true ,  // WeakCompareAndSwapI
    true ,  // WeakCompareAndSwapL
    true ,  // WeakCompareAndSwapP
    false,  // WeakCompareAndSwapN
    true ,  // CompareAndExchangeB
    true ,  // CompareAndExchangeS
    true ,  // CompareAndExchangeI
    true ,  // CompareAndExchangeL
    true ,  // CompareAndExchangeP
    false,  // CompareAndExchangeN
    true ,  // GetAndAddB
    true ,  // GetAndAddS
    true ,  // GetAndAddI
    false,  // GetAndAddL
    true ,  // GetAndSetB
    true ,  // GetAndSetS
    true ,  // GetAndSetI
    false,  // GetAndSetL
    true ,  // GetAndSetP
    false,  // GetAndSetN
    false,  // Con
    false,  // ConN
    false,  // ConNKlass
    true ,  // ConD
    true ,  // ConF
    true ,  // ConI
    true ,  // ConL
    true ,  // ConP
    true ,  // Conv2B
    true ,  // ConvD2F
    true ,  // ConvD2I
    true ,  // ConvD2L
    true ,  // ConvF2D
    true ,  // ConvF2I
    true ,  // ConvF2L
    true ,  // ConvI2D
    true ,  // ConvI2F
    true ,  // ConvI2L
    true ,  // ConvL2D
    true ,  // ConvL2F
    true ,  // ConvL2I
    true ,  // ConvF2HF
    true ,  // ConvHF2F
    false,  // CountedLoop
    true ,  // CountedLoopEnd
    false,  // OuterStripMinedLoop
    false,  // OuterStripMinedLoopEnd
    false,  // LongCountedLoop
    false,  // LongCountedLoopEnd
    true ,  // CountLeadingZerosI
    true ,  // CountLeadingZerosL
    true ,  // CountLeadingZerosV
    true ,  // CountTrailingZerosI
    true ,  // CountTrailingZerosL
    true ,  // CountTrailingZerosV
    true ,  // CreateEx
    false,  // DecodeN
    false,  // DecodeNKlass
    true ,  // DivD
    true ,  // DivF
    true ,  // DivI
    true ,  // DivL
    false,  // UDivI
    false,  // UDivL
    false,  // DivMod
    true ,  // DivModI
    false,  // DivModL
    false,  // UDivModI
    false,  // UDivModL
    true ,  // EncodeISOArray
    false,  // EncodeP
    false,  // EncodePKlass
    true ,  // FastLock
    true ,  // FastUnlock
    true ,  // FmaD
    true ,  // FmaF
    true ,  // Goto
    true ,  // Halt
    true ,  // CountPositives
    true ,  // If
    false,  // RangeCheck
    false,  // IfFalse
    false,  // IfTrue
    false,  // Initialize
    false,  // JProj
    true ,  // Jump
    false,  // JumpProj
    true ,  // LShiftI
    true ,  // LShiftL
    true ,  // LoadB
    true ,  // LoadUB
    true ,  // LoadUS
    true ,  // LoadD
    false,  // LoadD_unaligned
    true ,  // LoadF
    true ,  // LoadI
    true ,  // LoadKlass
    false,  // LoadNKlass
    true ,  // LoadL
    false,  // LoadL_unaligned
    true ,  // LoadP
    false,  // LoadN
    true ,  // LoadRange
    true ,  // LoadS
    false,  // Lock
    false,  // Loop
    true ,  // LoopLimit
    false,  // Mach
    false,  // MachNullCheck
    false,  // MachProj
    true ,  // MulAddS2I
    true ,  // MaxI
    false,  // MaxL
    false,  // MaxD
    false,  // MaxF
    true ,  // MemBarAcquire
    true ,  // LoadFence
    true ,  // MemBarAcquireLock
    false,  // MemBarCPUOrder
    true ,  // MemBarRelease
    true ,  // StoreFence
    true ,  // StoreStoreFence
    true ,  // MemBarReleaseLock
    true ,  // MemBarVolatile
    true ,  // MemBarStoreStore
    false,  // MergeMem
    true ,  // MinI
    false,  // MinL
    false,  // MinF
    false,  // MinD
    true ,  // ModD
    true ,  // ModF
    true ,  // ModI
    true ,  // ModL
    false,  // UModI
    false,  // UModL
    true ,  // MoveI2F
    true ,  // MoveF2I
    true ,  // MoveL2D
    true ,  // MoveD2L
    true ,  // IsInfiniteF
    false,  // IsFiniteF
    true ,  // IsInfiniteD
    false,  // IsFiniteD
    true ,  // MulD
    true ,  // MulF
    false,  // MulHiL
    false,  // UMulHiL
    true ,  // MulI
    true ,  // MulL
    false,  // Multi
    false,  // NegI
    false,  // NegL
    true ,  // NegD
    true ,  // NegF
    false,  // NeverBranch
    true ,  // OnSpinWait
    false,  // Opaque1
    false,  // OpaqueLoopInit
    false,  // OpaqueLoopStride
    false,  // OpaqueZeroTripGuard
    false,  // Opaque3
    false,  // Opaque4
    false,  // ProfileBoolean
    true ,  // OrI
    true ,  // OrL
    true ,  // OverflowAddI
    true ,  // OverflowSubI
    true ,  // OverflowMulI
    false,  // OverflowAddL
    false,  // OverflowSubL
    false,  // OverflowMulL
    false,  // PCTable
    false,  // Parm
    false,  // ParsePredicate
    true ,  // PartialSubtypeCheck
    false,  // SubTypeCheck
    false,  // Phi
    true ,  // PopCountI
    true ,  // PopCountL
    true ,  // PopCountVI
    true ,  // PopCountVL
    false,  // PopulateIndex
    true ,  // PrefetchAllocation
    false,  // Proj
    true ,  // RShiftI
    true ,  // RShiftL
    false,  // Region
    true ,  // Rethrow
    true ,  // Return
    false,  // ReverseI
    false,  // ReverseL
    true ,  // ReverseV
    false,  // Root
    true ,  // RoundDouble
    false,  // RoundDoubleMode
    false,  // RoundDoubleModeV
    true ,  // RoundFloat
    false,  // RotateLeft
    true ,  // RotateLeftV
    false,  // RotateRight
    true ,  // RotateRightV
    true ,  // SafePoint
    false,  // SafePointScalarObject
    true ,  // ShenandoahCompareAndExchangeP
    false,  // ShenandoahCompareAndExchangeN
    false,  // ShenandoahCompareAndSwapN
    true ,  // ShenandoahCompareAndSwapP
    false,  // ShenandoahWeakCompareAndSwapN
    true ,  // ShenandoahWeakCompareAndSwapP
    false,  // ShenandoahIUBarrier
    false,  // ShenandoahLoadReferenceBarrier
    false,  // SCMemProj
    false,  // CopySignD
    false,  // CopySignF
    true ,  // SignumD
    true ,  // SignumF
    true ,  // SignumVF
    true ,  // SignumVD
    true ,  // SqrtD
    true ,  // SqrtF
    false,  // RoundF
    false,  // RoundD
    false,  // Start
    false,  // StartOSR
    true ,  // StoreB
    true ,  // StoreC
    true ,  // StoreCM
    true ,  // StoreD
    true ,  // StoreF
    true ,  // StoreI
    true ,  // StoreL
    true ,  // StoreP
    false,  // StoreN
    false,  // StoreNKlass
    true ,  // StrComp
    true ,  // StrCompressedCopy
    true ,  // StrEquals
    true ,  // StrIndexOf
    true ,  // StrIndexOfChar
    true ,  // StrInflatedCopy
    true ,  // SubD
    true ,  // SubF
    true ,  // SubI
    true ,  // SubL
    true ,  // TailCall
    true ,  // TailJump
    true ,  // MacroLogicV
    true ,  // ThreadLocal
    false,  // Unlock
    false,  // URShiftB
    false,  // URShiftS
    true ,  // URShiftI
    true ,  // URShiftL
    true ,  // XorI
    true ,  // XorL
    false,  // Vector
    true ,  // AddVB
    true ,  // AddVS
    true ,  // AddVI
    true ,  // AddReductionVI
    true ,  // AddVL
    false,  // AddReductionVL
    true ,  // AddVF
    true ,  // AddReductionVF
    true ,  // AddVD
    true ,  // AddReductionVD
    true ,  // SubVB
    true ,  // SubVS
    true ,  // SubVI
    true ,  // SubVL
    true ,  // SubVF
    true ,  // SubVD
    true ,  // MulVB
    true ,  // MulVS
    true ,  // MulVI
    true ,  // MulReductionVI
    true ,  // MulVL
    false,  // MulReductionVL
    true ,  // MulVF
    true ,  // MulReductionVF
    true ,  // MulVD
    true ,  // MulReductionVD
    true ,  // MulAddVS2VI
    true ,  // FmaVD
    true ,  // FmaVF
    true ,  // DivVF
    true ,  // DivVD
    true ,  // AbsVB
    true ,  // AbsVS
    true ,  // AbsVI
    true ,  // AbsVL
    true ,  // AbsVF
    true ,  // AbsVD
    false,  // NegVI
    false,  // NegVL
    true ,  // NegVF
    true ,  // NegVD
    true ,  // SqrtVD
    true ,  // SqrtVF
    true ,  // LShiftCntV
    true ,  // RShiftCntV
    true ,  // LShiftVB
    true ,  // LShiftVS
    true ,  // LShiftVI
    true ,  // LShiftVL
    true ,  // RShiftVB
    true ,  // RShiftVS
    true ,  // RShiftVI
    true ,  // RShiftVL
    true ,  // URShiftVB
    true ,  // URShiftVS
    true ,  // URShiftVI
    true ,  // URShiftVL
    true ,  // AndV
    true ,  // AndReductionV
    true ,  // OrV
    true ,  // OrReductionV
    true ,  // XorV
    true ,  // XorReductionV
    true ,  // MinV
    true ,  // MaxV
    true ,  // MinReductionV
    true ,  // MaxReductionV
    false,  // CompressV
    false,  // CompressM
    false,  // ExpandV
    true ,  // LoadVector
    true ,  // LoadVectorGather
    true ,  // LoadVectorGatherMasked
    true ,  // StoreVector
    true ,  // StoreVectorScatter
    true ,  // StoreVectorScatterMasked
    true ,  // LoadVectorMasked
    true ,  // StoreVectorMasked
    false,  // VectorCmpMasked
    false,  // VectorMaskGen
    false,  // VectorMaskOp
    false,  // VectorMaskTrueCount
    false,  // VectorMaskFirstTrue
    false,  // VectorMaskLastTrue
    false,  // VectorMaskToLong
    false,  // VectorLongToMask
    false,  // Pack
    false,  // PackB
    false,  // PackS
    false,  // PackI
    false,  // PackL
    false,  // PackF
    false,  // PackD
    false,  // Pack2L
    false,  // Pack2D
    true ,  // ReplicateB
    true ,  // ReplicateS
    true ,  // ReplicateI
    true ,  // ReplicateL
    true ,  // ReplicateF
    true ,  // ReplicateD
    false,  // RoundVF
    false,  // RoundVD
    false,  // Extract
    false,  // ExtractB
    false,  // ExtractUB
    false,  // ExtractC
    true ,  // ExtractS
    true ,  // ExtractI
    false,  // ExtractL
    true ,  // ExtractF
    true ,  // ExtractD
    false,  // Digit
    false,  // LowerCase
    false,  // UpperCase
    false,  // Whitespace
    false,  // VectorBox
    false,  // VectorBoxAllocate
    false,  // VectorUnbox
    false,  // VectorMaskWrapper
    true ,  // VectorMaskCmp
    true ,  // VectorMaskCast
    false,  // VectorTest
    true ,  // VectorBlend
    true ,  // VectorRearrange
    true ,  // VectorLoadMask
    true ,  // VectorLoadShuffle
    true ,  // VectorLoadConst
    true ,  // VectorStoreMask
    true ,  // VectorReinterpret
    false,  // VectorCast
    true ,  // VectorCastB2X
    true ,  // VectorCastS2X
    true ,  // VectorCastI2X
    true ,  // VectorCastL2X
    true ,  // VectorCastF2X
    true ,  // VectorCastD2X
    true ,  // VectorCastF2HF
    true ,  // VectorCastHF2F
    true ,  // VectorUCastB2X
    true ,  // VectorUCastS2X
    true ,  // VectorUCastI2X
    false,  // VectorizedHashCode
    true ,  // VectorInsert
    true ,  // MaskAll
    true ,  // AndVMask
    true ,  // OrVMask
    true    // XorVMask
};


int Compile::sync_stack_slots() const { return 1; }

uint Matcher::stack_alignment_in_bytes() { return StackAlignmentInBytes; }

OptoReg::Name Matcher::return_addr() const { return OptoReg::stack2reg(- 1 +
              align_up((Compile::current()->in_preserve_stack_slots() +
                        Compile::current()->fixed_slots()),
                       stack_alignment_in_slots())); }

uint Compile::varargs_C_out_slots_killed() const { return 0; }

OptoRegPair Matcher::return_value(uint ideal_reg) {

#line 3203 "src/hotspot/cpu/x86/x86_32.ad"

    assert( ideal_reg >= Op_RegI && ideal_reg <= Op_RegL, "only return normal values" );
    static int lo[Op_RegL+1] = { 0, 0, OptoReg::Bad, EAX_num,      EAX_num,      FPR1L_num,    FPR1L_num, EAX_num };
    static int hi[Op_RegL+1] = { 0, 0, OptoReg::Bad, OptoReg::Bad, OptoReg::Bad, OptoReg::Bad, FPR1H_num, EDX_num };
    if( ideal_reg == Op_RegD && UseSSE>=2 )
      return OptoRegPair(XMM0b_num,XMM0_num);
    if( ideal_reg == Op_RegF && UseSSE>=1 )
      return OptoRegPair(OptoReg::Bad,XMM0_num);
    return OptoRegPair(hi[ideal_reg],lo[ideal_reg]);
  
#line 48260 "ad_x86.cpp"

}

OptoRegPair Matcher::c_return_value(uint ideal_reg) {

#line 3187 "src/hotspot/cpu/x86/x86_32.ad"

    assert( ideal_reg >= Op_RegI && ideal_reg <= Op_RegL, "only return normal values" );
    static int lo[Op_RegL+1] = { 0, 0, OptoReg::Bad, EAX_num,      EAX_num,      FPR1L_num,    FPR1L_num, EAX_num };
    static int hi[Op_RegL+1] = { 0, 0, OptoReg::Bad, OptoReg::Bad, OptoReg::Bad, OptoReg::Bad, FPR1H_num, EDX_num };

    // in SSE2+ mode we want to keep the FPU stack clean so pretend
    // that C functions return float and double results in XMM0.
    if( ideal_reg == Op_RegD && UseSSE>=2 )
      return OptoRegPair(XMM0b_num,XMM0_num);
    if( ideal_reg == Op_RegF && UseSSE>=2 )
      return OptoRegPair(OptoReg::Bad,XMM0_num);

    return OptoRegPair(hi[ideal_reg],lo[ideal_reg]);
  
#line 48281 "ad_x86.cpp"

}

OptoReg::Name Matcher::inline_cache_reg() { return OptoReg::Name(EAX_num); }

int Matcher::inline_cache_reg_encode() { return _regEncode[inline_cache_reg()]; }

OptoReg::Name Matcher::interpreter_frame_pointer_reg() { return OptoReg::Name(EBP_num); }

OptoReg::Name Matcher::c_frame_pointer() const { return OptoReg::Name(ESP_num); }

// Number of callee-save + always-save registers
int  Matcher::number_of_saved_registers() {
  return 0;
};

bool Compile::needs_deep_clone_jvms() { return false; }

// Check consistency of C++ compilation with ADLC options:
// Check adlc -DLINUX=1
#ifndef LINUX
#  error "LINUX must be defined"
#endif // LINUX
// Check adlc -D_GNU_SOURCE=1
#ifndef _GNU_SOURCE
#  error "_GNU_SOURCE must be defined"
#endif // _GNU_SOURCE
// Check adlc -DIA32=1
#ifndef IA32
#  error "IA32 must be defined"
#endif // IA32
// Check adlc -U_LP64
#ifdef _LP64
#  error "_LP64 must not be defined"
#endif // _LP64
// Check adlc -DNDEBUG=1
#ifndef NDEBUG
#  error "NDEBUG must be defined"
#endif // NDEBUG
// Check adlc -DPRODUCT=1
#ifndef PRODUCT
#  error "PRODUCT must be defined"
#endif // PRODUCT
